@@ -85,20 +85,6 @@ void AddressLiteral::set_rspec(relocInfo::relocType rtype) {
   }
 }
 
-// Initially added to the Assembler interface as a pure virtual:
-//   RegisterConstant delayed_value(..)
-// for:
-//   6812678 macro assembler needs delayed binding of a few constants (for 6655638)
-// this was subsequently modified to its present name and return type
-RegisterOrConstant MacroAssembler::delayed_value_impl(intptr_t* delayed_value_addr,
-                                                      Register tmp,
-                                                      int offset) {
-  ShouldNotReachHere();
-  return RegisterOrConstant(-1);
-}
-
-
-
 
 // virtual method calling
 void MacroAssembler::lookup_virtual_method(Register recv_klass,
@@ -994,7 +980,7 @@ void MacroAssembler::arm_stack_overflow_check(int frame_size_in_bytes, Register
   if (UseStackBanging) {
     const int page_size = os::vm_page_size();
 
-    sub_slow(tmp, SP, JavaThread::stack_shadow_zone_size());
+    sub_slow(tmp, SP, StackOverflow::stack_shadow_zone_size());
     strb(R0, Address(tmp));
     for (; frame_size_in_bytes >= page_size; frame_size_in_bytes -= 0xff0) {
       strb(R0, Address(tmp, -0xff0, pre_indexed));
@@ -1007,7 +993,7 @@ void MacroAssembler::arm_stack_overflow_check(Register Rsize, Register tmp) {
     Label loop;
 
     mov(tmp, SP);
-    add_slow(Rsize, Rsize, JavaThread::stack_shadow_zone_size() - os::vm_page_size());
+    add_slow(Rsize, Rsize, StackOverflow::stack_shadow_zone_size() - os::vm_page_size());
     bind(loop);
     subs(Rsize, Rsize, 0xff0);
     strb(R0, Address(tmp, -0xff0, pre_indexed));
@@ -1322,11 +1308,11 @@ void MacroAssembler::biased_locking_enter_with_cas(Register obj_reg, Register ol
 #endif // !PRODUCT
 }
 
-int MacroAssembler::biased_locking_enter(Register obj_reg, Register swap_reg, Register tmp_reg,
-                                         bool swap_reg_contains_mark,
-                                         Register tmp2,
-                                         Label& done, Label& slow_case,
-                                         BiasedLockingCounters* counters) {
+void MacroAssembler::biased_locking_enter(Register obj_reg, Register swap_reg, Register tmp_reg,
+                                          bool swap_reg_contains_mark,
+                                          Register tmp2,
+                                          Label& done, Label& slow_case,
+                                          BiasedLockingCounters* counters) {
   // obj_reg must be preserved (at least) if the bias locking fails
   // tmp_reg is a temporary register
   // swap_reg was used as a temporary but contained a value
@@ -1357,10 +1343,6 @@ int MacroAssembler::biased_locking_enter(Register obj_reg, Register swap_reg, Re
   // First check to see whether biasing is even enabled for this object
   Label cas_label;
 
-  // The null check applies to the mark loading, if we need to load it.
-  // If the mark has already been loaded in swap_reg then it has already
-  // been performed and the offset is irrelevant.
-  int null_check_offset = offset();
   if (!swap_reg_contains_mark) {
     ldr(swap_reg, mark_addr);
   }
@@ -1504,8 +1486,6 @@ int MacroAssembler::biased_locking_enter(Register obj_reg, Register swap_reg, Re
   // removing the bias bit from the object's header.
 
   bind(cas_label);
-
-  return null_check_offset;
 }
 
 
@@ -1920,7 +1900,7 @@ void MacroAssembler::resolve(DecoratorSet decorators, Register obj) {
 }
 
 void MacroAssembler::safepoint_poll(Register tmp1, Label& slow_path) {
-  ldr_u32(tmp1, Address(Rthread, Thread::polling_page_offset()));
+  ldr_u32(tmp1, Address(Rthread, Thread::polling_word_offset()));
   tst(tmp1, exact_log2(SafepointMechanism::poll_bit()));
   b(slow_path, eq);
 }
