@@ -236,6 +236,31 @@ inline bool MacroAssembler::is_bc_far_variant3_at(address instruction_addr) {
          is_endgroup(instruction_2);
 }
 
+// set dst to -1, 0, +1 as follows: if CCR0bi is "greater than", dst is set to 1,
+// if CCR0bi is "equal", dst is set to 0, otherwise it's set to -1.
+inline void MacroAssembler::set_cmp3(Register dst) {
+  assert_different_registers(dst, R0);
+  // P10, prefer using setbc intructions
+  if (VM_Version::has_brw()) {
+    setbc(R0, CCR0, Assembler::greater); // Set 1 to R0 if CCR0bi is "greater than", otherwise 0
+    setnbc(dst, CCR0, Assembler::less); // Set -1 to dst if CCR0bi is "less than", otherwise 0
+  } else {
+    mfcr(R0); // copy CR register to R0
+    srwi(dst, R0, 30); // copy the first two bits to dst
+    srawi(R0, R0, 31); // move the first bit to last position - sign extended
+  }
+  orr(dst, dst, R0); // dst | R0 will be -1, 0, or +1
+}
+
+// set dst to (treat_unordered_like_less ? -1 : +1)
+inline void MacroAssembler::set_cmpu3(Register dst, bool treat_unordered_like_less) {
+  if (treat_unordered_like_less) {
+    cror(CCR0, Assembler::less, CCR0, Assembler::summary_overflow); // treat unordered like less
+  } else {
+    cror(CCR0, Assembler::greater, CCR0, Assembler::summary_overflow); // treat unordered like greater
+  }
+  set_cmp3(dst);
+}
 
 // Convenience bc_far versions
 inline void MacroAssembler::blt_far(ConditionRegister crx, Label& L, int optimize) { MacroAssembler::bc_far(bcondCRbiIs1, bi0(crx, less), L, optimize); }
