@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2012, 2019 SAP SE. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2020 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -51,6 +51,7 @@ protected:
     vshasig,
     rtm,
     darn,
+    brw,
     num_features // last entry to count features
   };
   enum Feature_Flag_Set {
@@ -74,6 +75,7 @@ protected:
     vshasig_m             = (1 << vshasig),
     rtm_m                 = (1 << rtm    ),
     darn_m                = (1 << darn   ),
+    brw_m                 = (1 << brw    ),
     all_features_m        = (unsigned long)-1
   };
 
@@ -82,8 +84,7 @@ protected:
   static void print_features();
   static void determine_features(); // also measures cache line size
   static void config_dscr(); // Power 8: Configure Data Stream Control Register.
-  static void determine_section_size();
-  static void power6_micro_bench();
+
 public:
   // Initialization
   static void initialize();
@@ -119,6 +120,7 @@ public:
   static bool has_vshasig() { return (_features & vshasig_m) != 0; }
   static bool has_tm()      { return (_features & rtm_m) != 0; }
   static bool has_darn()    { return (_features & darn_m) != 0; }
+  static bool has_brw()     { return (_features & brw_m) != 0; }
 
   static bool has_mtfprd()  { return has_vpmsumb(); } // alias for P8
 
