@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2016, 2019 SAP SE. All rights reserved.
+ * Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2020 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -29,6 +29,7 @@
 #include "compiler/disassembler.hpp"
 #include "code/compiledIC.hpp"
 #include "memory/resourceArea.hpp"
+#include "runtime/globals_extension.hpp"
 #include "runtime/java.hpp"
 #include "runtime/stubCodeGenerator.hpp"
 #include "runtime/vm_version.hpp"
@@ -61,7 +62,9 @@ void VM_Version::initialize() {
 
   intx cache_line_size = Dcache_lineSize(0);
 
+#ifdef COMPILER2
   MaxVectorSize = 8;
+#endif
 
   if (has_PrefetchRaw()) {
     if (FLAG_IS_DEFAULT(AllocatePrefetchStyle)) {  // not preset
@@ -177,6 +180,11 @@ void VM_Version::initialize() {
     FLAG_SET_DEFAULT(UseFMA, true);
   }
 
+  if (UseMD5Intrinsics) {
+    warning("MD5 intrinsics are not available on this CPU");
+    FLAG_SET_DEFAULT(UseMD5Intrinsics, false);
+  }
+
   // On z/Architecture, we take UseSHA as the general switch to enable/disable the SHA intrinsics.
   // The specific switches UseSHAxxxIntrinsics will then be set depending on the actual
   // machine capabilities.
@@ -213,10 +221,16 @@ void VM_Version::initialize() {
     FLAG_SET_DEFAULT(UseSHA512Intrinsics, false);
   }
 
+  if (UseSHA3Intrinsics) {
+    warning("Intrinsics for SHA3-224, SHA3-256, SHA3-384 and SHA3-512 crypto hash functions not available on this CPU.");
+    FLAG_SET_DEFAULT(UseSHA3Intrinsics, false);
+  }
+
   if (!(UseSHA1Intrinsics || UseSHA256Intrinsics || UseSHA512Intrinsics)) {
     FLAG_SET_DEFAULT(UseSHA, false);
   }
 
+#ifdef COMPILER2
   if (FLAG_IS_DEFAULT(UseMultiplyToLenIntrinsic)) {
     FLAG_SET_DEFAULT(UseMultiplyToLenIntrinsic, true);
   }
@@ -226,6 +240,7 @@ void VM_Version::initialize() {
   if (FLAG_IS_DEFAULT(UseMontgomerySquareIntrinsic)) {
     FLAG_SET_DEFAULT(UseMontgomerySquareIntrinsic, true);
   }
+#endif
   if (FLAG_IS_DEFAULT(UsePopCountInstruction)) {
     FLAG_SET_DEFAULT(UsePopCountInstruction, true);
   }
@@ -249,41 +264,48 @@ void VM_Version::initialize() {
 
 
 void VM_Version::set_features_string() {
-
+  // A note on the _features_string format:
+  //   There are jtreg tests checking the _features_string for various properties.
+  //   For some strange reason, these tests require the string to contain
+  //   only _lowercase_ characters. Keep that in mind when being surprised
+  //   about the unusual notation of features - and when adding new ones.
+  //   Features may have one comma at the end.
+  //   Furthermore, use one, and only one, separator space between features.
+  //   Multiple spaces are considered separate tokens, messing up everything.
   unsigned int ambiguity = 0;
   _model_string = z_name[0];
   if (is_z13()) {
-    _features_string = "System z G7-z13  (LDISP_fast, ExtImm, PCrel Load/Store, CmpB, Cond Load/Store, Interlocked Update, TxM, VectorInstr)";
+    _features_string = "system-z, g7-z13, ldisp_fast, extimm, pcrel_load/store, cmpb, cond_load/store, interlocked_update, txm, vectorinstr";
     _model_string = z_name[7];
     ambiguity++;
   }
   if (is_ec12()) {
-    _features_string = "System z G6-EC12 (LDISP_fast, ExtImm, PCrel Load/Store, CmpB, Cond Load/Store, Interlocked Update, TxM)";
+    _features_string = "system-z, g6-ec12, ldisp_fast, extimm, pcrel_load/store, cmpb, cond_load/store, interlocked_update, txm";
     _model_string = z_name[6];
     ambiguity++;
   }
   if (is_z196()) {
-    _features_string = "System z G5-z196 (LDISP_fast, ExtImm, PCrel Load/Store, CmpB, Cond Load/Store, Interlocked Update)";
+    _features_string = "system-z, g5-z196, ldisp_fast, extimm, pcrel_load/store, cmpb, cond_load/store, interlocked_update";
     _model_string = z_name[5];
     ambiguity++;
   }
   if (is_z10()) {
-    _features_string = "System z G4-z10  (LDISP_fast, ExtImm, PCrel Load/Store, CmpB)";
+    _features_string = "system-z, g4-z10, ldisp_fast, extimm, pcrel_load/store, cmpb";
     _model_string = z_name[4];
     ambiguity++;
   }
   if (is_z9()) {
-    _features_string = "System z G3-z9   (LDISP_fast, ExtImm), out-of-support as of 2016-04-01";
+    _features_string = "system-z, g3-z9, ldisp_fast, extimm, out-of-support as of 2016-04-01";
     _model_string = z_name[3];
     ambiguity++;
   }
   if (is_z990()) {
-    _features_string = "System z G2-z990 (LDISP_fast), out-of-support as of 2014-07-01";
+    _features_string = "system-z, g2-z990, ldisp_fast, out-of-support as of 2014-07-01";
     _model_string = z_name[2];
     ambiguity++;
   }
   if (is_z900()) {
-    _features_string = "System z G1-z900 (LDISP), out-of-support as of 2014-07-01";
+    _features_string = "system-z, g1-z900, ldisp, out-of-support as of 2014-07-01";
     _model_string = z_name[1];
     ambiguity++;
   }
@@ -298,25 +320,24 @@ void VM_Version::set_features_string() {
 
   if (has_Crypto_AES()) {
     char buf[256];
-    assert(strlen(_features_string) + 4 + 3*4 + 1 < sizeof(buf), "increase buffer size");
-    jio_snprintf(buf, sizeof(buf), "%s aes%s%s%s", // String 'aes' must be surrounded by spaces so that jtreg tests recognize it.
+    assert(strlen(_features_string) + 3*8 < sizeof(buf), "increase buffer size");
+    jio_snprintf(buf, sizeof(buf), "%s%s%s%s",
                  _features_string,
-                 has_Crypto_AES128() ? " 128" : "",
-                 has_Crypto_AES192() ? " 192" : "",
-                 has_Crypto_AES256() ? " 256" : "");
+                 has_Crypto_AES128() ? ", aes128" : "",
+                 has_Crypto_AES192() ? ", aes192" : "",
+                 has_Crypto_AES256() ? ", aes256" : "");
     _features_string = os::strdup(buf);
   }
 
   if (has_Crypto_SHA()) {
     char buf[256];
-    assert(strlen(_features_string) + 4 + 2 + 2*4 + 6 + 1 < sizeof(buf), "increase buffer size");
-    // String 'sha1' etc must be surrounded by spaces so that jtreg tests recognize it.
-    jio_snprintf(buf, sizeof(buf), "%s %s%s%s%s",
+    assert(strlen(_features_string) + 6 + 2*8 + 7 < sizeof(buf), "increase buffer size");
+    jio_snprintf(buf, sizeof(buf), "%s%s%s%s%s",
                  _features_string,
-                 has_Crypto_SHA1()   ? " sha1"   : "",
-                 has_Crypto_SHA256() ? " sha256" : "",
-                 has_Crypto_SHA512() ? " sha512" : "",
-                 has_Crypto_GHASH()  ? " ghash"  : "");
+                 has_Crypto_SHA1()   ? ", sha1"   : "",
+                 has_Crypto_SHA256() ? ", sha256" : "",
+                 has_Crypto_SHA512() ? ", sha512" : "",
+                 has_Crypto_GHASH()  ? ", ghash"  : "");
     if (has_Crypto_AES()) { os::free((void *)_features_string); }
     _features_string = os::strdup(buf);
   }
@@ -815,7 +836,7 @@ void VM_Version::determine_features() {
                   code_end-code, cbuf_size, cbuf_size-(code_end-code));
 
     // Use existing decode function. This enables the [MachCode] format which is needed to DecodeErrorFile.
-    Disassembler::decode(&cbuf, code, code_end, tty);
+    Disassembler::decode(code, code_end, tty);
   }
 
   // Prepare for detection code execution and clear work buffer.
