@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -33,11 +33,14 @@ const bool CCallingConventionRequiresIntsAsLongs = false;
 
 #define SUPPORTS_NATIVE_CX8
 
+#define CPU_MULTI_COPY_ATOMIC
+
 // The expected size in bytes of a cache line, used to pad data structures.
 #if defined(TIERED)
   #ifdef _LP64
     // tiered, 64-bit, large machine
     #define DEFAULT_CACHE_LINE_SIZE 128
+    #define OM_CACHE_LINE_SIZE 64
   #else
     // tiered, 32-bit, medium machine
     #define DEFAULT_CACHE_LINE_SIZE 64
@@ -50,6 +53,7 @@ const bool CCallingConventionRequiresIntsAsLongs = false;
   #ifdef _LP64
     // pure C2, 64-bit, large machine
     #define DEFAULT_CACHE_LINE_SIZE 128
+    #define OM_CACHE_LINE_SIZE 64
   #else
     // pure C2, 32-bit, medium machine
     #define DEFAULT_CACHE_LINE_SIZE 64
@@ -61,10 +65,14 @@ const bool CCallingConventionRequiresIntsAsLongs = false;
 #define INCLUDE_RTM_OPT 1
 #endif
 
-#if defined(LINUX) || defined(SOLARIS) || defined(__APPLE__)
+#if defined(LINUX) || defined(__APPLE__)
 #define SUPPORT_RESERVED_STACK_AREA
 #endif
 
-#define THREAD_LOCAL_POLL
+#if INCLUDE_JVMCI
+#define COMPRESSED_CLASS_POINTERS_DEPENDS_ON_COMPRESSED_OOPS (EnableJVMCI || UseAOT)
+#else
+#define COMPRESSED_CLASS_POINTERS_DEPENDS_ON_COMPRESSED_OOPS false
+#endif
 
 #endif // CPU_X86_GLOBALDEFINITIONS_X86_HPP
