@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2020, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2014, Red Hat Inc. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -27,9 +27,6 @@
 #define OS_CPU_LINUX_AARCH64_OS_LINUX_AARCH64_HPP
 
   static void setup_fpu();
-  static bool supports_sse();
-
-  static jlong rdtsc();
 
   static bool is_allocatable(size_t bytes);
 
