@@ -98,7 +98,7 @@ void ArchDesc::buildMachRegisterNumbers(FILE *fp_hpp) {
   }
 
   fprintf(fp_hpp, "\n// Size of register-mask in ints\n");
-  fprintf(fp_hpp, "#define RM_SIZE %d\n",RegisterForm::RegMask_Size());
+  fprintf(fp_hpp, "#define RM_SIZE %d\n", RegisterForm::RegMask_Size());
   fprintf(fp_hpp, "// Unroll factor for loops over the data in a RegMask\n");
   fprintf(fp_hpp, "#define FORALL_BODY ");
   int len = RegisterForm::RegMask_Size();
@@ -2012,26 +2012,17 @@ void ArchDesc::declareClasses(FILE *fp) {
 }
 
 void ArchDesc::defineStateClass(FILE *fp) {
-  static const char *state__valid    = "_valid[((uint)index) >> 5] &  (0x1 << (((uint)index) & 0x0001F))";
-  static const char *state__set_valid= "_valid[((uint)index) >> 5] |= (0x1 << (((uint)index) & 0x0001F))";
+  static const char *state__valid    = "_rule[index] & 0x1";
 
   fprintf(fp,"\n");
   fprintf(fp,"// MACROS to inline and constant fold State::valid(index)...\n");
   fprintf(fp,"// when given a constant 'index' in dfa_<arch>.cpp\n");
-  fprintf(fp,"//   uint word   = index >> 5;       // Shift out bit position\n");
-  fprintf(fp,"//   uint bitpos = index & 0x0001F;  // Mask off word bits\n");
-  fprintf(fp,"#define STATE__VALID(index) ");
-  fprintf(fp,"    (%s)\n", state__valid);
-  fprintf(fp,"\n");
   fprintf(fp,"#define STATE__NOT_YET_VALID(index) ");
   fprintf(fp,"  ( (%s) == 0 )\n", state__valid);
   fprintf(fp,"\n");
   fprintf(fp,"#define STATE__VALID_CHILD(state,index) ");
   fprintf(fp,"  ( state && (state->%s) )\n", state__valid);
   fprintf(fp,"\n");
-  fprintf(fp,"#define STATE__SET_VALID(index) ");
-  fprintf(fp,"  (%s)\n", state__set_valid);
-  fprintf(fp,"\n");
   fprintf(fp,
           "//---------------------------State-------------------------------------------\n");
   fprintf(fp,"// State contains an integral cost vector, indexed by machine operand opcodes,\n");
@@ -2040,16 +2031,18 @@ void ArchDesc::defineStateClass(FILE *fp) {
   fprintf(fp,"// tree generated by the Label routines in ideal nodes (currently limited to\n");
   fprintf(fp,"// two for convenience, but this could change).\n");
   fprintf(fp,"class State : public ResourceObj {\n");
+  fprintf(fp,"private:\n");
+  fprintf(fp,"  unsigned int _cost[_LAST_MACH_OPER];  // Costs, indexed by operand opcodes\n");
+  fprintf(fp,"  uint16_t     _rule[_LAST_MACH_OPER];  // Rule and validity, indexed by operand opcodes\n");
+  fprintf(fp,"                                        // Lowest bit encodes validity\n");
+
   fprintf(fp,"public:\n");
-  fprintf(fp,"  int    _id;         // State identifier\n");
-  fprintf(fp,"  Node  *_leaf;       // Ideal (non-machine-node) leaf of match tree\n");
-  fprintf(fp,"  State *_kids[2];       // Children of state node in label tree\n");
-  fprintf(fp,"  unsigned int _cost[_LAST_MACH_OPER];  // Cost vector, indexed by operand opcodes\n");
-  fprintf(fp,"  unsigned int _rule[_LAST_MACH_OPER];  // Rule vector, indexed by operand opcodes\n");
-  fprintf(fp,"  unsigned int _valid[(_LAST_MACH_OPER/32)+1]; // Bit Map of valid Cost/Rule entries\n");
+  fprintf(fp,"  int    _id;                           // State identifier\n");
+  fprintf(fp,"  Node  *_leaf;                         // Ideal (non-machine-node) leaf of match tree\n");
+  fprintf(fp,"  State *_kids[2];                      // Children of state node in label tree\n");
   fprintf(fp,"\n");
-  fprintf(fp,"  State(void);                      // Constructor\n");
-  fprintf(fp,"  DEBUG_ONLY( ~State(void); )       // Destructor\n");
+  fprintf(fp,"  State(void);\n");
+  fprintf(fp,"  DEBUG_ONLY( ~State(void); )\n");
   fprintf(fp,"\n");
   fprintf(fp,"  // Methods created by ADLC and invoked by Reduce\n");
   fprintf(fp,"  MachOper *MachOperGenerator(int opcode);\n");
@@ -2058,14 +2051,14 @@ void ArchDesc::defineStateClass(FILE *fp) {
   fprintf(fp,"  // Assign a state to a node, definition of method produced by ADLC\n");
   fprintf(fp,"  bool DFA( int opcode, const Node *ideal );\n");
   fprintf(fp,"\n");
-  fprintf(fp,"  // Access function for _valid bit vector\n");
   fprintf(fp,"  bool valid(uint index) {\n");
-  fprintf(fp,"    return( STATE__VALID(index) != 0 );\n");
+  fprintf(fp,"    return %s;\n", state__valid);
   fprintf(fp,"  }\n");
-  fprintf(fp,"\n");
-  fprintf(fp,"  // Set function for _valid bit vector\n");
-  fprintf(fp,"  void set_valid(uint index) {\n");
-  fprintf(fp,"    STATE__SET_VALID(index);\n");
+  fprintf(fp,"  unsigned int rule(uint index) {\n");
+  fprintf(fp,"    return _rule[index] >> 1;\n");
+  fprintf(fp,"  }\n");
+  fprintf(fp,"  unsigned int cost(uint index) {\n");
+  fprintf(fp,"    return _cost[index];\n");
   fprintf(fp,"  }\n");
   fprintf(fp,"\n");
   fprintf(fp,"#ifndef PRODUCT\n");
