@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -90,7 +90,7 @@ CodeBuffer::CodeBuffer(CodeBlob* blob) {
   // Provide code buffer with meaningful name
   initialize_misc(blob->name());
   initialize(blob->content_begin(), blob->content_size());
-  verify_section_allocation();
+  debug_only(verify_section_allocation();)
 }
 
 void CodeBuffer::initialize(csize_t code_size, csize_t locs_size) {
@@ -117,7 +117,7 @@ void CodeBuffer::initialize(csize_t code_size, csize_t locs_size) {
     _insts.initialize_locs(locs_size / sizeof(relocInfo));
   }
 
-  verify_section_allocation();
+  debug_only(verify_section_allocation();)
 }
 
 
@@ -140,7 +140,7 @@ CodeBuffer::~CodeBuffer() {
 
   // Claim is that stack allocation ensures resources are cleaned up.
   // This is resource clean up, let's hope that all were properly copied out.
-  free_strings();
+  NOT_PRODUCT(free_strings();)
 
 #ifdef ASSERT
   // Save allocation type to execute assert in ~ResourceObj()
@@ -176,28 +176,6 @@ void CodeBuffer::initialize_section_size(CodeSection* cs, csize_t size) {
   if (_insts.has_locs())  cs->initialize_locs(1);
 }
 
-void CodeBuffer::freeze_section(CodeSection* cs) {
-  CodeSection* next_cs = (cs == consts())? NULL: code_section(cs->index()+1);
-  csize_t frozen_size = cs->size();
-  if (next_cs != NULL) {
-    frozen_size = next_cs->align_at_start(frozen_size);
-  }
-  address old_limit = cs->limit();
-  address new_limit = cs->start() + frozen_size;
-  relocInfo* old_locs_limit = cs->locs_limit();
-  relocInfo* new_locs_limit = cs->locs_end();
-  // Patch the limits.
-  cs->_limit = new_limit;
-  cs->_locs_limit = new_locs_limit;
-  cs->_frozen = true;
-  if (next_cs != NULL && !next_cs->is_allocated() && !next_cs->is_frozen()) {
-    // Give remaining buffer space to the following section.
-    next_cs->initialize(new_limit, old_limit - new_limit);
-    next_cs->initialize_shared_locs(new_locs_limit,
-                                    old_locs_limit - new_locs_limit);
-  }
-}
-
 void CodeBuffer::set_blob(BufferBlob* blob) {
   _blob = blob;
   if (blob != NULL) {
@@ -257,23 +235,19 @@ int CodeBuffer::locator(address addr) const {
   return -1;
 }
 
-address CodeBuffer::locator_address(int locator) const {
-  if (locator < 0)  return NULL;
-  address start = code_section(locator_sect(locator))->start();
-  return start + locator_pos(locator);
-}
 
 bool CodeBuffer::is_backward_branch(Label& L) {
   return L.is_bound() && insts_end() <= locator_address(L.loc());
 }
 
+#ifndef PRODUCT
 address CodeBuffer::decode_begin() {
   address begin = _insts.start();
   if (_decode_begin != NULL && _decode_begin > begin)
     begin = _decode_begin;
   return begin;
 }
-
+#endif // !PRODUCT
 
 GrowableArray<int>* CodeBuffer::create_patch_overflow() {
   if (_overflow_arena == NULL) {
@@ -505,18 +479,6 @@ void CodeBuffer::compute_final_layout(CodeBuffer* dest) const {
       } else {
         guarantee(padding == 0, "In first iteration no padding should be needed.");
       }
-      #ifdef ASSERT
-      if (prev_cs != NULL && prev_cs->is_frozen() && n < (SECT_LIMIT - 1)) {
-        // Make sure the ends still match up.
-        // This is important because a branch in a frozen section
-        // might target code in a following section, via a Label,
-        // and without a relocation record.  See Label::patch_instructions.
-        address dest_start = buf+buf_offset;
-        csize_t start2start = cs->start() - prev_cs->start();
-        csize_t dest_start2start = dest_start - prev_dest_cs->start();
-        assert(start2start == dest_start2start, "cannot stretch frozen sect");
-      }
-      #endif //ASSERT
       prev_dest_cs = dest_cs;
       prev_cs      = cs;
     }
@@ -532,7 +494,7 @@ void CodeBuffer::compute_final_layout(CodeBuffer* dest) const {
 
   // Done calculating sections; did it come out to the right end?
   assert(buf_offset == total_content_size(), "sanity");
-  dest->verify_section_allocation();
+  debug_only(dest->verify_section_allocation();)
 }
 
 // Append an oop reference that keeps the class alive.
@@ -752,7 +714,7 @@ void CodeBuffer::copy_code_to(CodeBlob* dest_blob) {
   relocate_code_to(&dest);
 
   // transfer strings and comments from buffer to blob
-  dest_blob->set_strings(_code_strings);
+  NOT_PRODUCT(dest_blob->set_strings(_code_strings);)
 
   // Done moving code bytes; were they the right size?
   assert((int)align_up(dest.total_content_size(), oopSize) == dest_blob->content_size(), "sanity");
@@ -895,9 +857,6 @@ void CodeBuffer::expand(CodeSection* which_cs, csize_t amount) {
   // Resizing must be allowed
   {
     if (blob() == NULL)  return;  // caller must check for blob == NULL
-    for (int n = 0; n < (int)SECT_LIMIT; n++) {
-      guarantee(!code_section(n)->is_frozen(), "resizing not allowed when frozen");
-    }
   }
 
   // Figure new capacity for each section.
@@ -955,14 +914,13 @@ void CodeBuffer::expand(CodeSection* which_cs, csize_t amount) {
 
   // Zap the old code buffer contents, to avoid mistakenly using them.
   debug_only(Copy::fill_to_bytes(bxp->_total_start, bxp->_total_size,
-                                 badCodeHeapFreeVal));
-
-  _decode_begin = NULL;  // sanity
+                                 badCodeHeapFreeVal);)
 
   // Make certain that the new sections are all snugly inside the new blob.
-  verify_section_allocation();
+  debug_only(verify_section_allocation();)
 
 #ifndef PRODUCT
+  _decode_begin = NULL;  // sanity
   if (PrintNMethods && (WizardMode || Verbose)) {
     tty->print("expanded CodeBuffer:");
     this->print();
@@ -991,24 +949,23 @@ void CodeBuffer::verify_section_allocation() {
   if (tstart == badAddress)  return;  // smashed by set_blob(NULL)
   address tend   = tstart + _total_size;
   if (_blob != NULL) {
-
     guarantee(tstart >= _blob->content_begin(), "sanity");
     guarantee(tend   <= _blob->content_end(),   "sanity");
   }
   // Verify disjointness.
   for (int n = (int) SECT_FIRST; n < (int) SECT_LIMIT; n++) {
     CodeSection* sect = code_section(n);
-    if (!sect->is_allocated() || sect->is_empty())  continue;
-    guarantee((intptr_t)sect->start() % sect->alignment() == 0
-           || sect->is_empty() || _blob == NULL,
+    if (!sect->is_allocated() || sect->is_empty()) {
+      continue;
+    }
+    guarantee(_blob == nullptr || is_aligned(sect->start(), sect->alignment()),
            "start is aligned");
-    for (int m = (int) SECT_FIRST; m < (int) SECT_LIMIT; m++) {
+    for (int m = n + 1; m < (int) SECT_LIMIT; m++) {
       CodeSection* other = code_section(m);
-      if (!other->is_allocated() || other == sect)  continue;
-      guarantee(!other->contains(sect->start()    ), "sanity");
-      // limit is an exclusive address and can be the start of another
-      // section.
-      guarantee(!other->contains(sect->limit() - 1), "sanity");
+      if (!other->is_allocated() || other == sect) {
+        continue;
+      }
+      guarantee(other->disjoint(sect), "sanity");
     }
     guarantee(sect->end() <= tend, "sanity");
     guarantee(sect->end() <= sect->limit(), "sanity");
@@ -1032,10 +989,6 @@ void CodeBuffer::log_section_sizes(const char* name) {
 
 #ifndef PRODUCT
 
-void CodeSection::decode() {
-  Disassembler::decode(start(), end());
-}
-
 void CodeBuffer::block_comment(intptr_t offset, const char * comment) {
   if (_collect_comments) {
     _code_strings.add_comment(offset, comment);
@@ -1054,8 +1007,12 @@ class CodeString: public CHeapObj<mtCode> {
   CodeString*  _prev;
   intptr_t     _offset;
 
+  static long allocated_code_strings;
+
   ~CodeString() {
     assert(_next == NULL && _prev == NULL, "wrong interface for freeing list");
+    allocated_code_strings--;
+    log_trace(codestrings)("Freeing CodeString [%s] (%p)", _string, (void*)_string);
     os::free((void*)_string);
   }
 
@@ -1064,12 +1021,14 @@ class CodeString: public CHeapObj<mtCode> {
  public:
   CodeString(const char * string, intptr_t offset = -1)
     : _next(NULL), _prev(NULL), _offset(offset) {
+    allocated_code_strings++;
     _string = os::strdup(string, mtCode);
+    log_trace(codestrings)("Created CodeString [%s] (%p)", _string, (void*)_string);
   }
 
   const char * string() const { return _string; }
   intptr_t     offset() const { assert(_offset >= 0, "offset for non comment?"); return _offset;  }
-  CodeString* next()    const { return _next; }
+  CodeString*  next()   const { return _next; }
 
   void set_next(CodeString* next) {
     _next = next;
@@ -1094,6 +1053,10 @@ class CodeString: public CHeapObj<mtCode> {
   }
 };
 
+// For tracing statistics. Will use raw increment/decrement, so it might not be
+// exact
+long CodeString::allocated_code_strings = 0;
+
 CodeString* CodeStrings::find(intptr_t offset) const {
   CodeString* a = _strings->first_comment();
   while (a != NULL && a->offset() != offset) {
@@ -1116,7 +1079,7 @@ void CodeStrings::add_comment(intptr_t offset, const char * comment) {
   CodeString* c      = new CodeString(comment, offset);
   CodeString* inspos = (_strings == NULL) ? NULL : find_last(offset);
 
-  if (inspos) {
+  if (inspos != NULL) {
     // insert after already existing comments with same offset
     c->set_next(inspos->next());
     inspos->set_next(c);
@@ -1130,21 +1093,10 @@ void CodeStrings::add_comment(intptr_t offset, const char * comment) {
   }
 }
 
-void CodeStrings::assign(CodeStrings& other) {
-  other.check_valid();
-  assert(is_null(), "Cannot assign onto non-empty CodeStrings");
-  _strings = other._strings;
-  _strings_last = other._strings_last;
-#ifdef ASSERT
-  _defunct = false;
-#endif
-  other.set_null_and_invalidate();
-}
-
 // Deep copy of CodeStrings for consistent memory management.
-// Only used for actual disassembly so this is cheaper than reference counting
-// for the "normal" fastdebug case.
 void CodeStrings::copy(CodeStrings& other) {
+  log_debug(codestrings)("Copying %d Codestring(s)", other.count());
+
   other.check_valid();
   check_valid();
   assert(is_null(), "Cannot copy onto non-empty CodeStrings");
@@ -1152,7 +1104,11 @@ void CodeStrings::copy(CodeStrings& other) {
   CodeString** ps = &_strings;
   CodeString* prev = NULL;
   while (n != NULL) {
-    *ps = new CodeString(n->string(),n->offset());
+    if (n->is_comment()) {
+      *ps = new CodeString(n->string(), n->offset());
+    } else {
+      *ps = new CodeString(n->string());
+    }
     (*ps)->_prev = prev;
     prev = *ps;
     ps = &((*ps)->_next);
@@ -1162,13 +1118,6 @@ void CodeStrings::copy(CodeStrings& other) {
 
 const char* CodeStrings::_prefix = " ;; ";  // default: can be changed via set_prefix
 
-// Check if any block comments are pending for the given offset.
-bool CodeStrings::has_block_comment(intptr_t offset) const {
-  if (_strings == NULL) return false;
-  CodeString* c = find(offset);
-  return c != NULL;
-}
-
 void CodeStrings::print_block_comment(outputStream* stream, intptr_t offset) const {
   check_valid();
   if (_strings != NULL) {
@@ -1184,8 +1133,19 @@ void CodeStrings::print_block_comment(outputStream* stream, intptr_t offset) con
   }
 }
 
-// Also sets isNull()
+int CodeStrings::count() const {
+  int i = 0;
+  CodeString* s = _strings;
+  while (s != NULL) {
+    i++;
+    s = s->_next;
+  }
+  return i;
+}
+
+// Also sets is_null()
 void CodeStrings::free() {
+  log_debug(codestrings)("Freeing %d out of approx. %ld CodeString(s), ", count(), CodeString::allocated_code_strings);
   CodeString* n = _strings;
   while (n) {
     // unlink the node from the list saving a pointer to the next
@@ -1215,15 +1175,14 @@ const char* CodeStrings::add_string(const char * string) {
 
 void CodeBuffer::decode() {
   ttyLocker ttyl;
-  Disassembler::decode(decode_begin(), insts_end(), tty);
+  Disassembler::decode(decode_begin(), insts_end(), tty NOT_PRODUCT(COMMA &strings()));
   _decode_begin = insts_end();
 }
 
 void CodeSection::print(const char* name) {
   csize_t locs_size = locs_end() - locs_start();
-  tty->print_cr(" %7s.code = " PTR_FORMAT " : " PTR_FORMAT " : " PTR_FORMAT " (%d of %d)%s",
-                name, p2i(start()), p2i(end()), p2i(limit()), size(), capacity(),
-                is_frozen()? " [frozen]": "");
+  tty->print_cr(" %7s.code = " PTR_FORMAT " : " PTR_FORMAT " : " PTR_FORMAT " (%d of %d)",
+                name, p2i(start()), p2i(end()), p2i(limit()), size(), capacity());
   tty->print_cr(" %7s.locs = " PTR_FORMAT " : " PTR_FORMAT " : " PTR_FORMAT " (%d of %d) point=%d",
                 name, p2i(locs_start()), p2i(locs_end()), p2i(locs_limit()), locs_size, locs_capacity(), locs_point_off());
   if (PrintRelocations) {
@@ -1246,10 +1205,4 @@ void CodeBuffer::print() {
   }
 }
 
-// Directly disassemble code buffer.
-void CodeBuffer::decode(address start, address end) {
-  ttyLocker ttyl;
-  Disassembler::decode(this, start, end, tty);
-}
-
 #endif // PRODUCT
