@@ -27,6 +27,7 @@
 #include "ci/ciInstance.hpp"
 #include "ci/ciInstanceKlass.hpp"
 #include "ci/ciMemberName.hpp"
+#include "ci/ciNativeEntryPoint.hpp"
 #include "ci/ciMethod.hpp"
 #include "ci/ciMethodData.hpp"
 #include "ci/ciMethodHandle.hpp"
@@ -67,7 +68,7 @@
 // sort of balanced binary tree.
 
 GrowableArray<ciMetadata*>* ciObjectFactory::_shared_ci_metadata = NULL;
-ciSymbol*                 ciObjectFactory::_shared_ci_symbols[vmSymbols::SID_LIMIT];
+ciSymbol*                 ciObjectFactory::_shared_ci_symbols[vmSymbols::number_of_symbols()];
 int                       ciObjectFactory::_shared_ident_limit = 0;
 volatile bool             ciObjectFactory::_initialized = false;
 
@@ -75,30 +76,24 @@ volatile bool             ciObjectFactory::_initialized = false;
 // ------------------------------------------------------------------
 // ciObjectFactory::ciObjectFactory
 ciObjectFactory::ciObjectFactory(Arena* arena,
-                                 int expected_size) {
-
+                                 int expected_size)
+                                 : _arena(arena),
+                                   _ci_metadata(arena, expected_size, 0, NULL),
+                                   _unloaded_methods(arena, 4, 0, NULL),
+                                   _unloaded_klasses(arena, 8, 0, NULL),
+                                   _unloaded_instances(arena, 4, 0, NULL),
+                                   _return_addresses(arena, 8, 0, NULL),
+                                   _symbols(arena, 100, 0, NULL),
+                                   _next_ident(_shared_ident_limit),
+                                   _non_perm_count(0) {
   for (int i = 0; i < NON_PERM_BUCKETS; i++) {
     _non_perm_bucket[i] = NULL;
   }
-  _non_perm_count = 0;
-
-  _next_ident = _shared_ident_limit;
-  _arena = arena;
-  _ci_metadata = new (arena) GrowableArray<ciMetadata*>(arena, expected_size, 0, NULL);
 
   // If the shared ci objects exist append them to this factory's objects
-
   if (_shared_ci_metadata != NULL) {
-    _ci_metadata->appendAll(_shared_ci_metadata);
+    _ci_metadata.appendAll(_shared_ci_metadata);
   }
-
-  _unloaded_methods = new (arena) GrowableArray<ciMethod*>(arena, 4, 0, NULL);
-  _unloaded_klasses = new (arena) GrowableArray<ciKlass*>(arena, 8, 0, NULL);
-  _unloaded_instances = new (arena) GrowableArray<ciInstance*>(arena, 4, 0, NULL);
-  _return_addresses =
-    new (arena) GrowableArray<ciReturnAddress*>(arena, 8, 0, NULL);
-
-  _symbols = new (arena) GrowableArray<ciSymbol*>(arena, 100, 0, NULL);
 }
 
 // ------------------------------------------------------------------
@@ -126,26 +121,25 @@ void ciObjectFactory::init_shared_objects() {
 
   {
     // Create the shared symbols, but not in _shared_ci_metadata.
-    int i;
-    for (i = vmSymbols::FIRST_SID; i < vmSymbols::SID_LIMIT; i++) {
-      Symbol* vmsym = vmSymbols::symbol_at((vmSymbols::SID) i);
-      assert(vmSymbols::find_sid(vmsym) == i, "1-1 mapping");
-      ciSymbol* sym = new (_arena) ciSymbol(vmsym, (vmSymbols::SID) i);
+    for (vmSymbolsIterator it = vmSymbolsRange.begin(); it != vmSymbolsRange.end(); ++it) {
+      vmSymbolID index = *it;
+      Symbol* vmsym = vmSymbols::symbol_at(index);
+      assert(vmSymbols::find_sid(vmsym) == index, "1-1 mapping");
+      ciSymbol* sym = new (_arena) ciSymbol(vmsym, index);
       init_ident_of(sym);
-      _shared_ci_symbols[i] = sym;
+      _shared_ci_symbols[vmSymbols::as_int(index)] = sym;
     }
 #ifdef ASSERT
-    for (i = vmSymbols::FIRST_SID; i < vmSymbols::SID_LIMIT; i++) {
-      Symbol* vmsym = vmSymbols::symbol_at((vmSymbols::SID) i);
-      ciSymbol* sym = vm_symbol_at((vmSymbols::SID) i);
+    for (vmSymbolsIterator it = vmSymbolsRange.begin(); it != vmSymbolsRange.end(); ++it) {
+      vmSymbolID index = *it;
+      Symbol* vmsym = vmSymbols::symbol_at(index);
+      ciSymbol* sym = vm_symbol_at(index);
       assert(sym->get_symbol() == vmsym, "oop must match");
     }
     assert(ciSymbol::void_class_signature()->get_symbol() == vmSymbols::void_class_signature(), "spot check");
 #endif
   }
 
-  _ci_metadata = new (_arena) GrowableArray<ciMetadata*>(_arena, 64, 0, NULL);
-
   for (int i = T_BOOLEAN; i <= T_CONFLICT; i++) {
     BasicType t = (BasicType)i;
     if (type2name(t) != NULL && !is_reference_type(t) &&
@@ -165,10 +159,10 @@ void ciObjectFactory::init_shared_objects() {
   WK_KLASSES_DO(WK_KLASS_DEFN)
 #undef WK_KLASS_DEFN
 
-  for (int len = -1; len != _ci_metadata->length(); ) {
-    len = _ci_metadata->length();
+  for (int len = -1; len != _ci_metadata.length(); ) {
+    len = _ci_metadata.length();
     for (int i2 = 0; i2 < len; i2++) {
-      ciMetadata* obj = _ci_metadata->at(i2);
+      ciMetadata* obj = _ci_metadata.at(i2);
       assert (obj->is_metadata(), "what else would it be?");
       if (obj->is_loaded() && obj->is_instance_klass()) {
         obj->as_instance_klass()->compute_nonstatic_fields();
@@ -193,8 +187,6 @@ void ciObjectFactory::init_shared_objects() {
   get_metadata(Universe::intArrayKlassObj());
   get_metadata(Universe::longArrayKlassObj());
 
-
-
   assert(_non_perm_count == 0, "no shared non-perm objects");
 
   // The shared_ident_limit is the first ident number that will
@@ -203,27 +195,27 @@ void ciObjectFactory::init_shared_objects() {
   // while the higher numbers are recycled afresh by each new ciEnv.
 
   _shared_ident_limit = _next_ident;
-  _shared_ci_metadata = _ci_metadata;
+  _shared_ci_metadata = &_ci_metadata;
 }
 
 
 ciSymbol* ciObjectFactory::get_symbol(Symbol* key) {
-  vmSymbols::SID sid = vmSymbols::find_sid(key);
-  if (sid != vmSymbols::NO_SID) {
+  vmSymbolID sid = vmSymbols::find_sid(key);
+  if (sid != vmSymbolID::NO_SID) {
     // do not pollute the main cache with it
     return vm_symbol_at(sid);
   }
 
-  assert(vmSymbols::find_sid(key) == vmSymbols::NO_SID, "");
-  ciSymbol* s = new (arena()) ciSymbol(key, vmSymbols::NO_SID);
-  _symbols->push(s);
+  assert(vmSymbols::find_sid(key) == vmSymbolID::NO_SID, "");
+  ciSymbol* s = new (arena()) ciSymbol(key, vmSymbolID::NO_SID);
+  _symbols.push(s);
   return s;
 }
 
 // Decrement the refcount when done on symbols referenced by this compilation.
 void ciObjectFactory::remove_symbols() {
-  for (int i = 0; i < _symbols->length(); i++) {
-    ciSymbol* s = _symbols->at(i);
+  for (int i = 0; i < _symbols.length(); i++) {
+    ciSymbol* s = _symbols.at(i);
     s->get_symbol()->decrement_refcount();
   }
   // Since _symbols is resource allocated we're not allowed to delete it
@@ -275,12 +267,12 @@ ciMetadata* ciObjectFactory::cached_metadata(Metadata* key) {
   ASSERT_IN_VM;
 
   bool found = false;
-  int index = _ci_metadata->find_sorted<Metadata*, ciObjectFactory::metadata_compare>(key, found);
+  int index = _ci_metadata.find_sorted<Metadata*, ciObjectFactory::metadata_compare>(key, found);
 
   if (!found) {
     return NULL;
   }
-  return _ci_metadata->at(index)->as_metadata();
+  return _ci_metadata.at(index)->as_metadata();
 }
 
 
@@ -296,20 +288,20 @@ ciMetadata* ciObjectFactory::get_metadata(Metadata* key) {
 #ifdef ASSERT
   if (CIObjectFactoryVerify) {
     Metadata* last = NULL;
-    for (int j = 0; j< _ci_metadata->length(); j++) {
-      Metadata* o = _ci_metadata->at(j)->constant_encoding();
+    for (int j = 0; j < _ci_metadata.length(); j++) {
+      Metadata* o = _ci_metadata.at(j)->constant_encoding();
       assert(last < o, "out of order");
       last = o;
     }
   }
 #endif // ASSERT
-  int len = _ci_metadata->length();
+  int len = _ci_metadata.length();
   bool found = false;
-  int index = _ci_metadata->find_sorted<Metadata*, ciObjectFactory::metadata_compare>(key, found);
+  int index = _ci_metadata.find_sorted<Metadata*, ciObjectFactory::metadata_compare>(key, found);
 #ifdef ASSERT
   if (CIObjectFactoryVerify) {
-    for (int i=0; i<_ci_metadata->length(); i++) {
-      if (_ci_metadata->at(i)->constant_encoding() == key) {
+    for (int i = 0; i < _ci_metadata.length(); i++) {
+      if (_ci_metadata.at(i)->constant_encoding() == key) {
         assert(index == i, " bad lookup");
       }
     }
@@ -323,16 +315,16 @@ ciMetadata* ciObjectFactory::get_metadata(Metadata* key) {
     init_ident_of(new_object);
     assert(new_object->is_metadata(), "must be");
 
-    if (len != _ci_metadata->length()) {
+    if (len != _ci_metadata.length()) {
       // creating the new object has recursively entered new objects
       // into the table.  We need to recompute our index.
-      index = _ci_metadata->find_sorted<Metadata*, ciObjectFactory::metadata_compare>(key, found);
+      index = _ci_metadata.find_sorted<Metadata*, ciObjectFactory::metadata_compare>(key, found);
     }
     assert(!found, "no double insert");
-    _ci_metadata->insert_before(index, new_object);
+    _ci_metadata.insert_before(index, new_object);
     return new_object;
   }
-  return _ci_metadata->at(index)->as_metadata();
+  return _ci_metadata.at(index)->as_metadata();
 }
 
 // ------------------------------------------------------------------
@@ -351,6 +343,8 @@ ciObject* ciObjectFactory::create_new_object(oop o) {
       return new (arena()) ciCallSite(h_i);
     else if (java_lang_invoke_MemberName::is_instance(o))
       return new (arena()) ciMemberName(h_i);
+    else if (jdk_internal_invoke_NativeEntryPoint::is_instance(o))
+      return new (arena()) ciNativeEntryPoint(h_i);
     else if (java_lang_invoke_MethodHandle::is_instance(o))
       return new (arena()) ciMethodHandle(h_i);
     else if (java_lang_invoke_MethodType::is_instance(o))
@@ -419,8 +413,8 @@ ciMethod* ciObjectFactory::get_unloaded_method(ciInstanceKlass* holder,
                                                ciInstanceKlass* accessor) {
   assert(accessor != NULL, "need origin of access");
   ciSignature* that = NULL;
-  for (int i = 0; i < _unloaded_methods->length(); i++) {
-    ciMethod* entry = _unloaded_methods->at(i);
+  for (int i = 0; i < _unloaded_methods.length(); i++) {
+    ciMethod* entry = _unloaded_methods.at(i);
     if (entry->holder()->equals(holder) &&
         entry->name()->equals(name) &&
         entry->signature()->as_symbol()->equals(signature)) {
@@ -444,7 +438,7 @@ ciMethod* ciObjectFactory::get_unloaded_method(ciInstanceKlass* holder,
   ciMethod* new_method = new (arena()) ciMethod(holder, name, signature, accessor);
 
   init_ident_of(new_method);
-  _unloaded_methods->append(new_method);
+  _unloaded_methods.append(new_method);
 
   return new_method;
 }
@@ -467,8 +461,8 @@ ciKlass* ciObjectFactory::get_unloaded_klass(ciKlass* accessing_klass,
     loader = accessing_klass->loader();
     domain = accessing_klass->protection_domain();
   }
-  for (int i=0; i<_unloaded_klasses->length(); i++) {
-    ciKlass* entry = _unloaded_klasses->at(i);
+  for (int i = 0; i < _unloaded_klasses.length(); i++) {
+    ciKlass* entry = _unloaded_klasses.at(i);
     if (entry->name()->equals(name) &&
         entry->loader() == loader &&
         entry->protection_domain() == domain) {
@@ -518,7 +512,7 @@ ciKlass* ciObjectFactory::get_unloaded_klass(ciKlass* accessing_klass,
     new_klass = new (arena()) ciInstanceKlass(name, loader_handle, domain_handle);
   }
   init_ident_of(new_klass);
-  _unloaded_klasses->append(new_klass);
+  _unloaded_klasses.append(new_klass);
 
   return new_klass;
 }
@@ -530,8 +524,8 @@ ciKlass* ciObjectFactory::get_unloaded_klass(ciKlass* accessing_klass,
 // Get a ciInstance representing an as-yet undetermined instance of a given class.
 //
 ciInstance* ciObjectFactory::get_unloaded_instance(ciInstanceKlass* instance_klass) {
-  for (int i=0; i<_unloaded_instances->length(); i++) {
-    ciInstance* entry = _unloaded_instances->at(i);
+  for (int i = 0; i < _unloaded_instances.length(); i++) {
+    ciInstance* entry = _unloaded_instances.at(i);
     if (entry->klass()->equals(instance_klass)) {
       // We've found a match.
       return entry;
@@ -543,7 +537,7 @@ ciInstance* ciObjectFactory::get_unloaded_instance(ciInstanceKlass* instance_kla
   ciInstance* new_instance = new (arena()) ciInstance(instance_klass);
 
   init_ident_of(new_instance);
-  _unloaded_instances->append(new_instance);
+  _unloaded_instances.append(new_instance);
 
   // make sure it looks the way we want:
   assert(!new_instance->is_loaded(), "");
@@ -610,8 +604,8 @@ ciMethodData* ciObjectFactory::get_empty_methodData() {
 //
 // Get a ciReturnAddress for a specified bci.
 ciReturnAddress* ciObjectFactory::get_return_address(int bci) {
-  for (int i=0; i<_return_addresses->length(); i++) {
-    ciReturnAddress* entry = _return_addresses->at(i);
+  for (int i = 0; i < _return_addresses.length(); i++) {
+    ciReturnAddress* entry = _return_addresses.at(i);
     if (entry->bci() == bci) {
       // We've found a match.
       return entry;
@@ -620,7 +614,7 @@ ciReturnAddress* ciObjectFactory::get_return_address(int bci) {
 
   ciReturnAddress* new_ret_addr = new (arena()) ciReturnAddress(bci);
   init_ident_of(new_ret_addr);
-  _return_addresses->append(new_ret_addr);
+  _return_addresses.append(new_ret_addr);
   return new_ret_addr;
 }
 
@@ -678,17 +672,16 @@ void ciObjectFactory::insert_non_perm(ciObjectFactory::NonPermObject* &where, oo
 // ------------------------------------------------------------------
 // ciObjectFactory::vm_symbol_at
 // Get the ciSymbol corresponding to some index in vmSymbols.
-ciSymbol* ciObjectFactory::vm_symbol_at(int index) {
-  assert(index >= vmSymbols::FIRST_SID && index < vmSymbols::SID_LIMIT, "oob");
+ciSymbol* ciObjectFactory::vm_symbol_at(vmSymbolID sid) {
+  int index = vmSymbols::as_int(sid);
   return _shared_ci_symbols[index];
 }
 
 // ------------------------------------------------------------------
 // ciObjectFactory::metadata_do
 void ciObjectFactory::metadata_do(MetadataClosure* f) {
-  if (_ci_metadata == NULL) return;
-  for (int j = 0; j< _ci_metadata->length(); j++) {
-    Metadata* o = _ci_metadata->at(j)->constant_encoding();
+  for (int j = 0; j < _ci_metadata.length(); j++) {
+    Metadata* o = _ci_metadata.at(j)->constant_encoding();
     f->do_metadata(o);
   }
 }
@@ -696,10 +689,10 @@ void ciObjectFactory::metadata_do(MetadataClosure* f) {
 // ------------------------------------------------------------------
 // ciObjectFactory::print_contents_impl
 void ciObjectFactory::print_contents_impl() {
-  int len = _ci_metadata->length();
+  int len = _ci_metadata.length();
   tty->print_cr("ciObjectFactory (%d) meta data contents:", len);
-  for (int i=0; i<len; i++) {
-    _ci_metadata->at(i)->print();
+  for (int i = 0; i < len; i++) {
+    _ci_metadata.at(i)->print();
     tty->cr();
   }
 }
@@ -718,7 +711,7 @@ void ciObjectFactory::print_contents() {
 // Print debugging information about the object factory
 void ciObjectFactory::print() {
   tty->print("<ciObjectFactory oops=%d metadata=%d unloaded_methods=%d unloaded_instances=%d unloaded_klasses=%d>",
-             _non_perm_count, _ci_metadata->length(), _unloaded_methods->length(),
-             _unloaded_instances->length(),
-             _unloaded_klasses->length());
+             _non_perm_count, _ci_metadata.length(), _unloaded_methods.length(),
+             _unloaded_instances.length(),
+             _unloaded_klasses.length());
 }
