@@ -50,21 +50,21 @@ size_t ClassLoaderDataGraph::num_array_classes() {
 }
 
 void ClassLoaderDataGraph::inc_instance_classes(size_t count) {
-  Atomic::add(count, &_num_instance_classes);
+  Atomic::add(&_num_instance_classes, count);
 }
 
 void ClassLoaderDataGraph::dec_instance_classes(size_t count) {
   assert(count <= _num_instance_classes, "Sanity");
-  Atomic::sub(count, &_num_instance_classes);
+  Atomic::sub(&_num_instance_classes, count);
 }
 
 void ClassLoaderDataGraph::inc_array_classes(size_t count) {
-  Atomic::add(count, &_num_array_classes);
+  Atomic::add(&_num_array_classes, count);
 }
 
 void ClassLoaderDataGraph::dec_array_classes(size_t count) {
   assert(count <= _num_array_classes, "Sanity");
-  Atomic::sub(count, &_num_array_classes);
+  Atomic::sub(&_num_array_classes, count);
 }
 
 bool ClassLoaderDataGraph::should_clean_metaspaces_and_reset() {
