@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,15 +25,14 @@
 #ifndef SHARE_CLASSFILE_DICTIONARY_HPP
 #define SHARE_CLASSFILE_DICTIONARY_HPP
 
-#include "classfile/protectionDomainCache.hpp"
-#include "classfile/systemDictionary.hpp"
 #include "oops/instanceKlass.hpp"
 #include "oops/oop.hpp"
+#include "oops/oopHandle.hpp"
 #include "utilities/hashtable.hpp"
 #include "utilities/ostream.hpp"
 
 class DictionaryEntry;
-class BoolObjectClosure;
+class ProtectionDomainEntry;
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // The data structure for the class loader data dictionaries.
@@ -180,7 +179,7 @@ class SymbolPropertyEntry : public HashtableEntry<Symbol*, mtSymbol> {
  private:
   intptr_t _symbol_mode;  // secondary key
   Method*   _method;
-  oop       _method_type;
+  OopHandle _method_type;
 
  public:
   Symbol* symbol() const            { return literal(); }
@@ -191,9 +190,13 @@ class SymbolPropertyEntry : public HashtableEntry<Symbol*, mtSymbol> {
   Method*        method() const     { return _method; }
   void set_method(Method* p)        { _method = p; }
 
-  oop      method_type() const      { return _method_type; }
-  oop*     method_type_addr()       { return &_method_type; }
-  void set_method_type(oop p)       { _method_type = p; }
+  oop      method_type() const;
+  void set_method_type(oop p);
+
+  // We need to clear the OopHandle because these hashtable entries are not constructed properly.
+  void clear_method_type() { _method_type = OopHandle(); }
+
+  void free_entry();
 
   SymbolPropertyEntry* next() const {
     return (SymbolPropertyEntry*)HashtableEntry<Symbol*, mtSymbol>::next();
@@ -203,22 +206,7 @@ class SymbolPropertyEntry : public HashtableEntry<Symbol*, mtSymbol> {
     return (SymbolPropertyEntry**)HashtableEntry<Symbol*, mtSymbol>::next_addr();
   }
 
-  void print_entry(outputStream* st) const {
-    symbol()->print_value_on(st);
-    st->print("/mode=" INTX_FORMAT, symbol_mode());
-    st->print(" -> ");
-    bool printed = false;
-    if (method() != NULL) {
-      method()->print_value_on(st);
-      printed = true;
-    }
-    if (method_type() != NULL) {
-      if (printed)  st->print(" and ");
-      st->print(INTPTR_FORMAT, p2i((void *)method_type()));
-      printed = true;
-    }
-    st->print_cr(printed ? "" : "(empty)");
-  }
+  void print_entry(outputStream* st) const;
 };
 
 // A system-internal mapping of symbols to pointers, both managed
@@ -245,7 +233,7 @@ private:
     symbol->increment_refcount();
     entry->set_symbol_mode(symbol_mode);
     entry->set_method(NULL);
-    entry->set_method_type(NULL);
+    entry->clear_method_type();
     return entry;
   }
 
@@ -253,11 +241,7 @@ public:
   SymbolPropertyTable(int table_size);
   SymbolPropertyTable(int table_size, HashtableBucket<mtSymbol>* t, int number_of_entries);
 
-  void free_entry(SymbolPropertyEntry* entry) {
-    // decrement Symbol refcount here because hashtable doesn't.
-    entry->literal()->decrement_refcount();
-    Hashtable<Symbol*, mtSymbol>::free_entry(entry);
-  }
+  void free_entry(SymbolPropertyEntry* entry);
 
   unsigned int compute_hash(Symbol* sym, intptr_t symbol_mode) {
     // Use the regular identity_hash.
@@ -274,9 +258,6 @@ public:
   // must be done under SystemDictionary_lock
   SymbolPropertyEntry* add_entry(int index, unsigned int hash, Symbol* name, intptr_t name_mode);
 
-  // GC support
-  void oops_do(OopClosure* f);
-
   void methods_do(void f(Method*));
 
   void verify();
