@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,6 +32,7 @@
 #include "utilities/tableStatistics.hpp"
 
 class CompactHashtableWriter;
+class DumpedInternedStrings;
 class JavaThread;
 class SerializeClosure;
 
@@ -46,23 +47,26 @@ class StringTable : public CHeapObj<mtSymbol>{
   friend class StringTableCreateEntry;
 
   static volatile bool _has_work;
-  static volatile size_t _uncleaned_items_count;
 
   // Set if one bucket is out of balance due to hash algorithm deficiency
   static volatile bool _needs_rehashing;
 
+  static OopStorage* _oop_storage;
+
   static void grow(JavaThread* jt);
   static void clean_dead_entries(JavaThread* jt);
 
   static double get_load_factor();
-  static double get_dead_factor();
+  static double get_dead_factor(size_t num_dead);
 
-  static void check_concurrent_work();
+  // GC support
+
+  // Callback for GC to notify of changes that might require cleaning or resize.
+  static void gc_notification(size_t num_dead);
   static void trigger_concurrent_work();
 
   static size_t item_added();
   static void item_removed();
-  static size_t add_items_to_clean(size_t ndead);
 
   static oop intern(Handle string_or_null_h, const jchar* name, int len, TRAPS);
   static oop do_intern(Handle string_or_null, const jchar* name, int len, uintx hash, TRAPS);
@@ -79,20 +83,7 @@ class StringTable : public CHeapObj<mtSymbol>{
   static void create_table();
 
   static void do_concurrent_work(JavaThread* jt);
-  static bool has_work() { return _has_work; }
-
-  // GC support
-
-  // Must be called before a parallel walk where strings might die.
-  static void reset_dead_counter() { _uncleaned_items_count = 0; }
-
-  // After the parallel walk this method must be called to trigger
-  // cleaning. Note it might trigger a resize instead.
-  static void finish_dead_counter() { check_concurrent_work(); }
-
-  // If GC uses ParState directly it should add the number of cleared
-  // strings to this method.
-  static void inc_dead_counter(size_t ndead) { add_items_to_clean(ndead); }
+  static bool has_work();
 
   // Probing
   static oop lookup(Symbol* symbol);
@@ -115,11 +106,10 @@ class StringTable : public CHeapObj<mtSymbol>{
   // Sharing
  private:
   static oop lookup_shared(const jchar* name, int len, unsigned int hash) NOT_CDS_JAVA_HEAP_RETURN_(NULL);
-  static void copy_shared_string_table(CompactHashtableWriter* ch_table) NOT_CDS_JAVA_HEAP_RETURN;
  public:
   static oop create_archived_string(oop s, Thread* THREAD) NOT_CDS_JAVA_HEAP_RETURN_(NULL);
   static void shared_oops_do(OopClosure* f) NOT_CDS_JAVA_HEAP_RETURN;
-  static void write_to_archive() NOT_CDS_JAVA_HEAP_RETURN;
+  static void write_to_archive(const DumpedInternedStrings* dumped_interned_strings) NOT_CDS_JAVA_HEAP_RETURN;
   static void serialize_shared_table_header(SerializeClosure* soc) NOT_CDS_JAVA_HEAP_RETURN;
 
   // Jcmd
