@@ -110,10 +110,12 @@ protected:
 
   ImmutableOopMapSet* _oop_maps;                 // OopMap for this CodeBlob
   bool                _caller_must_gc_arguments;
-  CodeStrings         _strings;
+
   const char*         _name;
   S390_ONLY(int       _ctable_offset;)
 
+  NOT_PRODUCT(CodeStrings _strings;)
+
   CodeBlob(const char* name, CompilerType type, const CodeBlobLayout& layout, int frame_complete_offset, int frame_size, ImmutableOopMapSet* oop_maps, bool caller_must_gc_arguments);
   CodeBlob(const char* name, CompilerType type, const CodeBlobLayout& layout, CodeBuffer* cb, int frame_complete_offset, int frame_size, OopMapSet* oop_maps, bool caller_must_gc_arguments);
 
@@ -147,6 +149,7 @@ public:
   inline bool is_compiled_by_c2() const    { return _type == compiler_c2; };
   inline bool is_compiled_by_jvmci() const { return _type == compiler_jvmci; };
   const char* compiler_name() const;
+  CompilerType compiler_type() const { return _type; }
 
   // Casting
   nmethod* as_nmethod_or_null()                { return is_nmethod() ? (nmethod*) this : NULL; }
@@ -230,29 +233,20 @@ public:
   void dump_for_addr(address addr, outputStream* st, bool verbose) const;
   void print_code();
 
-  bool has_block_comment(address block_begin) const {
-    intptr_t offset = (intptr_t)(block_begin - code_begin());
-    return _strings.has_block_comment(offset);
-  }
   // Print the comment associated with offset on stream, if there is one
   virtual void print_block_comment(outputStream* stream, address block_begin) const {
+  #ifndef PRODUCT
     intptr_t offset = (intptr_t)(block_begin - code_begin());
     _strings.print_block_comment(stream, offset);
+  #endif
   }
 
-  // Transfer ownership of comments to this CodeBlob
+#ifndef PRODUCT
   void set_strings(CodeStrings& strings) {
     assert(!is_aot(), "invalid on aot");
-    _strings.assign(strings);
-  }
-
-  static ByteSize name_field_offset() {
-    return byte_offset_of(CodeBlob, _name);
-  }
-
-  static ByteSize oop_maps_field_offset() {
-    return byte_offset_of(CodeBlob, _oop_maps);
+    _strings.copy(strings);
   }
+#endif
 };
 
 class CodeBlobLayout : public StackObj {
@@ -447,6 +441,8 @@ class VtableBlob: public BufferBlob {
 private:
   VtableBlob(const char*, int);
 
+  void* operator new(size_t s, unsigned size) throw();
+
 public:
   // Creation
   static VtableBlob* create(const char* name, int buffer_size);
@@ -600,10 +596,6 @@ class DeoptimizationBlob: public SingletonBlob {
 
   // Typing
   bool is_deoptimization_stub() const { return true; }
-  bool exception_address_is_unpack_entry(address pc) const {
-    address unpack_pc = unpack();
-    return (pc == unpack_pc || (pc + frame::pc_return_offset) == unpack_pc);
-  }
 
   // GC for args
   void preserve_callee_argument_oops(frame fr, const RegisterMap *reg_map, OopClosure* f) { /* Nothing to do */ }
