@@ -42,6 +42,7 @@
 // - ConstantValue   describes a constant
 
 class ConstantOopReadValue;
+class LocationValue;
 class ObjectValue;
 
 class ScopeValue: public ResourceObj {
@@ -50,6 +51,7 @@ class ScopeValue: public ResourceObj {
   virtual bool is_location() const { return false; }
   virtual bool is_object() const { return false; }
   virtual bool is_auto_box() const { return false; }
+  virtual bool is_marker() const { return false; }
   virtual bool is_constant_int() const { return false; }
   virtual bool is_constant_double() const { return false; }
   virtual bool is_constant_long() const { return false; }
@@ -66,6 +68,11 @@ class ScopeValue: public ResourceObj {
     return (ObjectValue*)this;
   }
 
+  LocationValue* as_LocationValue() {
+    assert(is_location(), "must be");
+    return (LocationValue*)this;
+  }
+
   // Serialization of debugging information
   virtual void write_on(DebugInfoWriteStream* stream) = 0;
   static ScopeValue* read_from(DebugInfoReadStream* stream);
@@ -91,6 +98,19 @@ class LocationValue: public ScopeValue {
   void print_on(outputStream* st) const;
 };
 
+// A placeholder value that has no concrete meaning other than helping constructing
+// other values.
+
+class MarkerValue: public ScopeValue {
+public:
+  bool      is_marker() const                { return true; }
+
+  // Serialization of debugging information
+  void write_on(DebugInfoWriteStream* stream);
+
+  // Printing
+  void print_on(outputStream* st) const;
+};
 
 // An ObjectValue describes an object eliminated by escape analysis.
 
