@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -34,8 +34,10 @@
 #include "oops/methodData.hpp"
 #include "oops/method.inline.hpp"
 #include "oops/oop.inline.hpp"
+#include "prims/jvmtiExport.hpp"
 #include "prims/nativeLookup.hpp"
 #include "runtime/frame.hpp"
+#include "runtime/globals_extension.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/stubRoutines.hpp"
 #include "runtime/thread.hpp"
@@ -105,7 +107,7 @@ void CompilationPolicy::compile_if_required(const methodHandle& selected_method,
     }
     CompileBroker::compile_method(selected_method, InvocationEntryBci,
         CompilationPolicy::policy()->initial_compile_level(selected_method),
-        methodHandle(), 0, CompileTask::Reason_MustBeCompiled, CHECK);
+        methodHandle(), 0, CompileTask::Reason_MustBeCompiled, THREAD);
   }
 }
 
@@ -208,7 +210,7 @@ class CounterDecay : public AllStatic {
 public:
   static void decay();
   static bool is_decay_needed() {
-    return ((os::javaTimeNanos() - _last_timestamp) / (NANOUNITS / MILLIUNITS)) > CounterDecayMinIntervalLength;
+    return nanos_to_millis(os::javaTimeNanos() - _last_timestamp) > CounterDecayMinIntervalLength;
   }
   static void update_last_timestamp() { _last_timestamp = os::javaTimeNanos(); }
 };
@@ -295,8 +297,8 @@ void SimpleCompPolicy::reset_counter_for_invocation_event(const methodHandle& m)
   // Set carry bit and reduce counter's value to min(count, CompileThreshold/2).
   MethodCounters* mcs = m->method_counters();
   assert(mcs != NULL, "MethodCounters cannot be NULL for profiling");
-  mcs->invocation_counter()->set_carry();
-  mcs->backedge_counter()->set_carry();
+  mcs->invocation_counter()->set_carry_and_reduce();
+  mcs->backedge_counter()->set_carry_and_reduce();
 
   assert(!m->was_never_executed(), "don't reset to 0 -- could be mistaken for never-executed");
 }
@@ -312,9 +314,9 @@ void SimpleCompPolicy::reset_counter_for_back_branch_event(const methodHandle& m
   // Don't set invocation_counter's value too low otherwise the method will
   // look like immature (ic < ~5300) which prevents the inlining based on
   // the type profiling.
-  i->set(i->state(), CompileThreshold);
+  i->set(CompileThreshold);
   // Don't reset counter too low - it is used to check if OSR method is ready.
-  b->set(b->state(), CompileThreshold / 2);
+  b->set(CompileThreshold / 2);
 }
 
 // Called at the end of the safepoint
@@ -340,7 +342,7 @@ void SimpleCompPolicy::reprofile(ScopeDesc* trap_scope, bool is_osr) {
     c = mcs->invocation_counter();
     if (is_osr) {
       // It was an OSR method, so bump the count higher.
-      c->set(c->state(), CompileThreshold);
+      c->set(CompileThreshold);
     } else {
       c->reset();
     }
@@ -358,14 +360,6 @@ void SimpleCompPolicy::delay_compilation(Method* method) {
   }
 }
 
-void SimpleCompPolicy::disable_compilation(Method* method) {
-  MethodCounters* mcs = method->method_counters();
-  if (mcs != NULL) {
-    mcs->invocation_counter()->set_state(InvocationCounter::wait_for_nothing);
-    mcs->backedge_counter()->set_state(InvocationCounter::wait_for_nothing);
-  }
-}
-
 CompileTask* SimpleCompPolicy::select_task(CompileQueue* compile_queue) {
   return select_task_helper(compile_queue);
 }
@@ -386,10 +380,10 @@ bool SimpleCompPolicy::is_mature(Method* method) {
 }
 
 nmethod* SimpleCompPolicy::event(const methodHandle& method, const methodHandle& inlinee, int branch_bci,
-                                    int bci, CompLevel comp_level, CompiledMethod* nm, JavaThread* thread) {
+                                    int bci, CompLevel comp_level, CompiledMethod* nm, TRAPS) {
   assert(comp_level == CompLevel_none, "This should be only called from the interpreter");
   NOT_PRODUCT(trace_frequency_counter_overflow(method, branch_bci, bci));
-  if (JvmtiExport::can_post_interpreter_events() && thread->is_interp_only_mode()) {
+  if (JvmtiExport::can_post_interpreter_events() && THREAD->as_Java_thread()->is_interp_only_mode()) {
     // If certain JVMTI events (e.g. frame pop event) are requested then the
     // thread is forced to remain in interpreted code. This is
     // implemented partly by a check in the run_compiled_code
@@ -415,7 +409,7 @@ nmethod* SimpleCompPolicy::event(const methodHandle& method, const methodHandle&
     // when code cache is full, compilation gets switched off, UseCompiler
     // is set to false
     if (!method->has_compiled_code() && UseCompiler) {
-      method_invocation_event(method, thread);
+      method_invocation_event(method, THREAD);
     } else {
       // Force counter overflow on method entry, even if no compilation
       // happened.  (The method_invocation_event call does this also.)
@@ -431,7 +425,7 @@ nmethod* SimpleCompPolicy::event(const methodHandle& method, const methodHandle&
     NOT_PRODUCT(trace_osr_request(method, osr_nm, bci));
     // when code cache is full, we should not compile any more...
     if (osr_nm == NULL && UseCompiler) {
-      method_back_branch_event(method, bci, thread);
+      method_back_branch_event(method, bci, THREAD);
       osr_nm = method->lookup_osr_nmethod_for(bci, CompLevel_highest_tier, true);
     }
     if (osr_nm == NULL) {
@@ -486,7 +480,7 @@ void SimpleCompPolicy::trace_osr_request(const methodHandle& method, nmethod* os
 }
 #endif // !PRODUCT
 
-void SimpleCompPolicy::method_invocation_event(const methodHandle& m, JavaThread* thread) {
+void SimpleCompPolicy::method_invocation_event(const methodHandle& m, TRAPS) {
   const int comp_level = CompLevel_highest_tier;
   const int hot_count = m->invocation_count();
   reset_counter_for_invocation_event(m);
@@ -494,17 +488,17 @@ void SimpleCompPolicy::method_invocation_event(const methodHandle& m, JavaThread
   if (is_compilation_enabled() && can_be_compiled(m, comp_level)) {
     CompiledMethod* nm = m->code();
     if (nm == NULL ) {
-      CompileBroker::compile_method(m, InvocationEntryBci, comp_level, m, hot_count, CompileTask::Reason_InvocationCount, thread);
+      CompileBroker::compile_method(m, InvocationEntryBci, comp_level, m, hot_count, CompileTask::Reason_InvocationCount, THREAD);
     }
   }
 }
 
-void SimpleCompPolicy::method_back_branch_event(const methodHandle& m, int bci, JavaThread* thread) {
+void SimpleCompPolicy::method_back_branch_event(const methodHandle& m, int bci, TRAPS) {
   const int comp_level = CompLevel_highest_tier;
   const int hot_count = m->backedge_count();
 
   if (is_compilation_enabled() && can_be_osr_compiled(m, comp_level)) {
-    CompileBroker::compile_method(m, bci, comp_level, m, hot_count, CompileTask::Reason_BackedgeCount, thread);
+    CompileBroker::compile_method(m, bci, comp_level, m, hot_count, CompileTask::Reason_BackedgeCount, THREAD);
     NOT_PRODUCT(trace_osr_completion(m->lookup_osr_nmethod_for(bci, comp_level, true));)
   }
 }
