@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -24,6 +24,7 @@
 
 #include "precompiled.hpp"
 #include "code/codeCache.hpp"
+#include "runtime/arguments.hpp"
 #include "runtime/globals.hpp"
 #include "runtime/globals_extension.hpp"
 #include "compiler/compilerDefinitions.hpp"
@@ -118,13 +119,17 @@ intx CompilerConfig::scaled_freq_log(intx freq_log, double scale) {
   // max_freq_bits accordingly.
   intx max_freq_bits = InvocationCounter::number_of_count_bits + 1;
   intx scaled_freq = scaled_compile_threshold((intx)1 << freq_log, scale);
+
   if (scaled_freq == 0) {
     // Return 0 right away to avoid calculating log2 of 0.
     return 0;
-  } else if (scaled_freq > nth_bit(max_freq_bits)) {
-    return max_freq_bits;
   } else {
-    return log2_intptr(scaled_freq);
+    intx res = log2_intptr(scaled_freq);
+    if (res > max_freq_bits) {
+      return max_freq_bits;
+    } else {
+      return res;
+    }
   }
 }
 
@@ -278,6 +283,13 @@ void CompilerConfig::set_tiered_flags() {
     }
 #endif // INCLUDE_AOT
   }
+
+  // Reduce stack usage due to inlining of methods which require much stack.
+  // (High tier compiler can inline better based on profiling information.)
+  if (FLAG_IS_DEFAULT(C1InlineStackLimit) &&
+      TieredStopAtLevel == CompLevel_full_optimization && !CompilationModeFlag::quick_only()) {
+    FLAG_SET_DEFAULT(C1InlineStackLimit, 5);
+  }
 }
 
 #endif // TIERED
@@ -395,17 +407,6 @@ bool CompilerConfig::check_args_consistency(bool status) {
     }
     FLAG_SET_CMDLINE(PostLoopMultiversioning, false);
   }
-  if (UseCountedLoopSafepoints && LoopStripMiningIter == 0) {
-    if (!FLAG_IS_DEFAULT(UseCountedLoopSafepoints) || !FLAG_IS_DEFAULT(LoopStripMiningIter)) {
-      warning("When counted loop safepoints are enabled, LoopStripMiningIter must be at least 1 (a safepoint every 1 iteration): setting it to 1");
-    }
-    LoopStripMiningIter = 1;
-  } else if (!UseCountedLoopSafepoints && LoopStripMiningIter > 0) {
-    if (!FLAG_IS_DEFAULT(UseCountedLoopSafepoints) || !FLAG_IS_DEFAULT(LoopStripMiningIter)) {
-      warning("Disabling counted safepoints implies no loop strip mining: setting LoopStripMiningIter to 0");
-    }
-    LoopStripMiningIter = 0;
-  }
 #endif // COMPILER2
 
   if (Arguments::is_interpreter_only()) {
@@ -456,7 +457,7 @@ void CompilerConfig::ergo_initialize() {
 #endif
 
 #if INCLUDE_JVMCI
-  // Check that JVMCI compiler supports selested GC.
+  // Check that JVMCI supports selected GC.
   // Should be done after GCConfig::initialize() was called.
   JVMCIGlobals::check_jvmci_supported_gc();
 
@@ -477,6 +478,13 @@ void CompilerConfig::ergo_initialize() {
     }
   }
 
+  if (FLAG_IS_DEFAULT(SweeperThreshold)) {
+    if ((SweeperThreshold * ReservedCodeCacheSize / 100) > (1.2 * M)) {
+      // Cap default SweeperThreshold value to an equivalent of 1.2 Mb
+      FLAG_SET_ERGO(SweeperThreshold, (1.2 * M * 100) / ReservedCodeCacheSize);
+    }
+  }
+
   if (UseOnStackReplacement && !UseLoopCounter) {
     warning("On-stack-replacement requires loop counters; enabling loop counters");
     FLAG_SET_DEFAULT(UseLoopCounter, true);
@@ -486,15 +494,17 @@ void CompilerConfig::ergo_initialize() {
   if (!EliminateLocks) {
     EliminateNestedLocks = false;
   }
-  if (!Inline) {
+  if (!Inline || !IncrementalInline) {
     IncrementalInline = false;
+    IncrementalInlineMH = false;
+    IncrementalInlineVirtual = false;
   }
 #ifndef PRODUCT
   if (!IncrementalInline) {
     AlwaysIncrementalInline = false;
   }
-  if (PrintIdealGraphLevel > 0) {
-    FLAG_SET_ERGO(PrintIdealGraph, true);
+  if (FLAG_IS_CMDLINE(PrintIdealGraph) && !PrintIdealGraph) {
+    FLAG_SET_ERGO(PrintIdealGraphLevel, -1);
   }
 #endif
   if (!UseTypeSpeculation && FLAG_IS_DEFAULT(TypeProfileLevel)) {
@@ -510,3 +520,42 @@ void CompilerConfig::ergo_initialize() {
   }
 #endif // COMPILER2
 }
+
+static CompLevel highest_compile_level() {
+  return TieredCompilation ? MIN2((CompLevel) TieredStopAtLevel, CompLevel_highest_tier) : CompLevel_highest_tier;
+}
+
+bool is_c1_or_interpreter_only() {
+  if (Arguments::is_interpreter_only()) {
+    return true;
+  }
+
+#if INCLUDE_AOT
+  if (UseAOT) {
+    return false;
+  }
+#endif
+
+  if (highest_compile_level() < CompLevel_full_optimization) {
+#if INCLUDE_JVMCI
+    if (TieredCompilation) {
+       return true;
+    }
+    // This happens on jvm variant with C2 disabled and JVMCI
+    // enabled.
+    return !UseJVMCICompiler;
+#else
+    return true;
+#endif
+  }
+
+#ifdef TIERED
+  // The quick-only compilation mode is c1 only. However,
+  // CompilationModeFlag only takes effect with TieredCompilation
+  // enabled.
+  if (TieredCompilation && CompilationModeFlag::quick_only()) {
+    return true;
+  }
+#endif
+  return false;
+}
