@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,8 +27,10 @@
 #include "compiler/compilerOracle.hpp"
 #include "compiler/tieredThresholdPolicy.hpp"
 #include "memory/resourceArea.hpp"
+#include "prims/jvmtiExport.hpp"
 #include "runtime/arguments.hpp"
 #include "runtime/frame.inline.hpp"
+#include "runtime/globals_extension.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/safepoint.hpp"
 #include "runtime/safepointVerifiers.hpp"
@@ -45,7 +47,7 @@
 
 bool TieredThresholdPolicy::call_predicate_helper(const methodHandle& method, CompLevel cur_level, int i, int b, double scale) {
   double threshold_scaling;
-  if (CompilerOracle::has_option_value(method, "CompileThresholdScaling", threshold_scaling)) {
+  if (CompilerOracle::has_option_value(method, CompileCommand::CompileThresholdScaling, threshold_scaling)) {
     scale *= threshold_scaling;
   }
   switch(cur_level) {
@@ -76,7 +78,7 @@ bool TieredThresholdPolicy::call_predicate_helper(const methodHandle& method, Co
 
 bool TieredThresholdPolicy::loop_predicate_helper(const methodHandle& method, CompLevel cur_level, int i, int b, double scale) {
   double threshold_scaling;
-  if (CompilerOracle::has_option_value(method, "CompileThresholdScaling", threshold_scaling)) {
+  if (CompilerOracle::has_option_value(method, CompileCommand::CompileThresholdScaling, threshold_scaling)) {
     scale *= threshold_scaling;
   }
   switch(cur_level) {
@@ -293,18 +295,18 @@ void TieredThresholdPolicy::initialize() {
   // Some inlining tuning
 #ifdef X86
   if (FLAG_IS_DEFAULT(InlineSmallCode)) {
-    FLAG_SET_DEFAULT(InlineSmallCode, 2000);
+    FLAG_SET_DEFAULT(InlineSmallCode, 2500);
   }
 #endif
 
-#if defined SPARC || defined AARCH64
+#if defined AARCH64
   if (FLAG_IS_DEFAULT(InlineSmallCode)) {
     FLAG_SET_DEFAULT(InlineSmallCode, 2500);
   }
 #endif
 
   set_increase_threshold_at_ratio();
-  set_start_time(os::javaTimeMillis());
+  set_start_time(nanos_to_millis(os::javaTimeNanos()));
 }
 
 
@@ -379,23 +381,17 @@ CompLevel TieredThresholdPolicy::initial_compile_level(const methodHandle& metho
   return limit_level(initial_compile_level_helper(method));
 }
 
-void TieredThresholdPolicy::set_carry_if_necessary(InvocationCounter *counter) {
-  if (!counter->carry() && counter->count() > InvocationCounter::count_limit / 2) {
-    counter->set_carry_flag();
-  }
-}
-
 // Set carry flags on the counters if necessary
 void TieredThresholdPolicy::handle_counter_overflow(Method* method) {
   MethodCounters *mcs = method->method_counters();
   if (mcs != NULL) {
-    set_carry_if_necessary(mcs->invocation_counter());
-    set_carry_if_necessary(mcs->backedge_counter());
+    mcs->invocation_counter()->set_carry_on_overflow();
+    mcs->backedge_counter()->set_carry_on_overflow();
   }
   MethodData* mdo = method->method_data();
   if (mdo != NULL) {
-    set_carry_if_necessary(mdo->invocation_counter());
-    set_carry_if_necessary(mdo->backedge_counter());
+    mdo->invocation_counter()->set_carry_on_overflow();
+    mdo->backedge_counter()->set_carry_on_overflow();
   }
 }
 
@@ -404,7 +400,7 @@ CompileTask* TieredThresholdPolicy::select_task(CompileQueue* compile_queue) {
   CompileTask *max_blocking_task = NULL;
   CompileTask *max_task = NULL;
   Method* max_method = NULL;
-  jlong t = os::javaTimeMillis();
+  jlong t = nanos_to_millis(os::javaTimeNanos());
   // Iterate through the queue and find a method with a maximum rate.
   for (CompileTask* task = compile_queue->first(); task != NULL;) {
     CompileTask* next_task = task->next();
@@ -486,10 +482,10 @@ void TieredThresholdPolicy::reprofile(ScopeDesc* trap_scope, bool is_osr) {
 }
 
 nmethod* TieredThresholdPolicy::event(const methodHandle& method, const methodHandle& inlinee,
-                                      int branch_bci, int bci, CompLevel comp_level, CompiledMethod* nm, JavaThread* thread) {
+                                      int branch_bci, int bci, CompLevel comp_level, CompiledMethod* nm, TRAPS) {
   if (comp_level == CompLevel_none &&
       JvmtiExport::can_post_interpreter_events() &&
-      thread->is_interp_only_mode()) {
+      THREAD->as_Java_thread()->is_interp_only_mode()) {
     return NULL;
   }
   if (ReplayCompiles) {
@@ -507,28 +503,31 @@ nmethod* TieredThresholdPolicy::event(const methodHandle& method, const methodHa
   }
 
   if (bci == InvocationEntryBci) {
-    method_invocation_event(method, inlinee, comp_level, nm, thread);
+    method_invocation_event(method, inlinee, comp_level, nm, THREAD);
   } else {
     // method == inlinee if the event originated in the main method
-    method_back_branch_event(method, inlinee, bci, comp_level, nm, thread);
+    method_back_branch_event(method, inlinee, bci, comp_level, nm, THREAD);
     // Check if event led to a higher level OSR compilation
-    CompLevel expected_comp_level = comp_level;
+    CompLevel expected_comp_level = MIN2(CompLevel_full_optimization, static_cast<CompLevel>(comp_level + 1));
     if (!CompilationModeFlag::disable_intermediate() && inlinee->is_not_osr_compilable(expected_comp_level)) {
       // It's not possble to reach the expected level so fall back to simple.
       expected_comp_level = CompLevel_simple;
     }
-    nmethod* osr_nm = inlinee->lookup_osr_nmethod_for(bci, expected_comp_level, false);
-    assert(osr_nm == NULL || osr_nm->comp_level() >= expected_comp_level, "lookup_osr_nmethod_for is broken");
-    if (osr_nm != NULL) {
-      // Perform OSR with new nmethod
-      return osr_nm;
+    CompLevel max_osr_level = static_cast<CompLevel>(inlinee->highest_osr_comp_level());
+    if (max_osr_level >= expected_comp_level) { // fast check to avoid locking in a typical scenario
+      nmethod* osr_nm = inlinee->lookup_osr_nmethod_for(bci, expected_comp_level, false);
+      assert(osr_nm == NULL || osr_nm->comp_level() >= expected_comp_level, "lookup_osr_nmethod_for is broken");
+      if (osr_nm != NULL && osr_nm->comp_level() != comp_level) {
+        // Perform OSR with new nmethod
+        return osr_nm;
+      }
     }
   }
   return NULL;
 }
 
 // Check if the method can be compiled, change level if necessary
-void TieredThresholdPolicy::compile(const methodHandle& mh, int bci, CompLevel level, JavaThread* thread) {
+void TieredThresholdPolicy::compile(const methodHandle& mh, int bci, CompLevel level, TRAPS) {
   assert(verify_level(level) && level <= TieredStopAtLevel, "Invalid compilation level %d", level);
 
   if (level == CompLevel_none) {
@@ -540,9 +539,10 @@ void TieredThresholdPolicy::compile(const methodHandle& mh, int bci, CompLevel l
         mh->code()->make_not_used();
       }
       // Deoptimize immediately (we don't have to wait for a compile).
-      RegisterMap map(thread, false);
-      frame fr = thread->last_frame().sender(&map);
-      Deoptimization::deoptimize_frame(thread, fr.id());
+      JavaThread* jt = THREAD->as_Java_thread();
+      RegisterMap map(jt, false);
+      frame fr = jt->last_frame().sender(&map);
+      Deoptimization::deoptimize_frame(jt, fr.id());
     }
     return;
   }
@@ -572,7 +572,7 @@ void TieredThresholdPolicy::compile(const methodHandle& mh, int bci, CompLevel l
     // pure C1.
     if ((bci == InvocationEntryBci && !can_be_compiled(mh, level))) {
       if (level == CompLevel_full_optimization && can_be_compiled(mh, CompLevel_simple)) {
-        compile(mh, bci, CompLevel_simple, thread);
+        compile(mh, bci, CompLevel_simple, THREAD);
       }
       return;
     }
@@ -583,7 +583,7 @@ void TieredThresholdPolicy::compile(const methodHandle& mh, int bci, CompLevel l
           // Invalidate the existing OSR nmethod so that a compile at CompLevel_simple is permitted.
           osr_nm->make_not_entrant();
         }
-        compile(mh, bci, CompLevel_simple, thread);
+        compile(mh, bci, CompLevel_simple, THREAD);
       }
       return;
     }
@@ -596,8 +596,8 @@ void TieredThresholdPolicy::compile(const methodHandle& mh, int bci, CompLevel l
       print_event(COMPILE, mh(), mh(), bci, level);
     }
     int hot_count = (bci == InvocationEntryBci) ? mh->invocation_count() : mh->backedge_count();
-    update_rate(os::javaTimeMillis(), mh());
-    CompileBroker::compile_method(mh, bci, level, mh, hot_count, CompileTask::Reason_Tiered, thread);
+    update_rate(nanos_to_millis(os::javaTimeNanos()), mh());
+    CompileBroker::compile_method(mh, bci, level, mh, hot_count, CompileTask::Reason_Tiered, THREAD);
   }
 }
 
@@ -616,7 +616,7 @@ void TieredThresholdPolicy::update_rate(jlong t, Method* m) {
 
   // We don't update the rate if we've just came out of a safepoint.
   // delta_s is the time since last safepoint in milliseconds.
-  jlong delta_s = t - SafepointTracing::end_of_last_safepoint_epoch_ms();
+  jlong delta_s = t - SafepointTracing::end_of_last_safepoint_ms();
   jlong delta_t = t - (m->prev_time() != 0 ? m->prev_time() : start_time()); // milliseconds since the last measurement
   // How many events were there since the last time?
   int event_count = m->invocation_count() + m->backedge_count();
@@ -641,7 +641,7 @@ void TieredThresholdPolicy::update_rate(jlong t, Method* m) {
 // Check if this method has been stale for a given number of milliseconds.
 // See select_task().
 bool TieredThresholdPolicy::is_stale(jlong t, jlong timeout, Method* m) {
-  jlong delta_s = t - SafepointTracing::end_of_last_safepoint_epoch_ms();
+  jlong delta_s = t - SafepointTracing::end_of_last_safepoint_ms();
   jlong delta_t = t - m->prev_time();
   if (delta_t > timeout && delta_s > timeout) {
     int event_count = m->invocation_count() + m->backedge_count();
@@ -790,7 +790,7 @@ bool TieredThresholdPolicy::is_mature(Method* method) {
 // start profiling without waiting for the compiled method to arrive.
 // We also take the load on compilers into the account.
 bool TieredThresholdPolicy::should_create_mdo(const methodHandle& method, CompLevel cur_level) {
-  if (cur_level != CompLevel_none || force_comp_at_level_simple(method)) {
+  if (cur_level != CompLevel_none || force_comp_at_level_simple(method) || !ProfileInterpreter) {
     return false;
   }
   int i = method->invocation_count();
@@ -816,7 +816,7 @@ bool TieredThresholdPolicy::should_not_inline(ciEnv* env, ciMethod* callee) {
 }
 
 // Create MDO if necessary.
-void TieredThresholdPolicy::create_mdo(const methodHandle& mh, JavaThread* THREAD) {
+void TieredThresholdPolicy::create_mdo(const methodHandle& mh, Thread* THREAD) {
   if (mh->is_native() ||
       mh->is_abstract() ||
       mh->is_accessor() ||
@@ -826,6 +826,18 @@ void TieredThresholdPolicy::create_mdo(const methodHandle& mh, JavaThread* THREA
   if (mh->method_data() == NULL) {
     Method::build_interpreter_method_data(mh, CHECK_AND_CLEAR);
   }
+  if (ProfileInterpreter) {
+    MethodData* mdo = mh->method_data();
+    if (mdo != NULL) {
+      JavaThread* jt = THREAD->as_Java_thread();
+      frame last_frame = jt->last_frame();
+      if (last_frame.is_interpreted_frame() && mh == last_frame.interpreter_frame_method()) {
+        int bci = last_frame.interpreter_frame_bci();
+        address dp = mdo->bci_to_dp(bci);
+        last_frame.interpreter_frame_set_mdp(dp);
+      }
+    }
+  }
 }
 
 
@@ -997,7 +1009,7 @@ CompLevel TieredThresholdPolicy::common(Predicate p, const methodHandle& method,
 
 
 // Determine if a method should be compiled with a normal entry point at a different level.
-CompLevel TieredThresholdPolicy::call_event(const methodHandle& method, CompLevel cur_level, JavaThread* thread) {
+CompLevel TieredThresholdPolicy::call_event(const methodHandle& method, CompLevel cur_level, Thread* thread) {
   CompLevel osr_level = MIN2((CompLevel) method->highest_osr_comp_level(),
                              common(&TieredThresholdPolicy::loop_predicate, method, cur_level, true));
   CompLevel next_level = common(&TieredThresholdPolicy::call_predicate, method, cur_level);
@@ -1018,7 +1030,7 @@ CompLevel TieredThresholdPolicy::call_event(const methodHandle& method, CompLeve
 }
 
 // Determine if we should do an OSR compilation of a given method.
-CompLevel TieredThresholdPolicy::loop_event(const methodHandle& method, CompLevel cur_level, JavaThread* thread) {
+CompLevel TieredThresholdPolicy::loop_event(const methodHandle& method, CompLevel cur_level, Thread* thread) {
   CompLevel next_level = common(&TieredThresholdPolicy::loop_predicate, method, cur_level, true);
   if (cur_level == CompLevel_none) {
     // If there is a live OSR method that means that we deopted to the interpreter
@@ -1031,7 +1043,7 @@ CompLevel TieredThresholdPolicy::loop_event(const methodHandle& method, CompLeve
   return next_level;
 }
 
-bool TieredThresholdPolicy::maybe_switch_to_aot(const methodHandle& mh, CompLevel cur_level, CompLevel next_level, JavaThread* thread) {
+bool TieredThresholdPolicy::maybe_switch_to_aot(const methodHandle& mh, CompLevel cur_level, CompLevel next_level, Thread* thread) {
   if (UseAOT) {
     if (cur_level == CompLevel_full_profile || cur_level == CompLevel_none) {
       // If the current level is full profile or interpreter and we're switching to any other level,
@@ -1053,18 +1065,18 @@ bool TieredThresholdPolicy::maybe_switch_to_aot(const methodHandle& mh, CompLeve
 
 // Handle the invocation event.
 void TieredThresholdPolicy::method_invocation_event(const methodHandle& mh, const methodHandle& imh,
-                                                      CompLevel level, CompiledMethod* nm, JavaThread* thread) {
+                                                      CompLevel level, CompiledMethod* nm, TRAPS) {
   if (should_create_mdo(mh, level)) {
-    create_mdo(mh, thread);
+    create_mdo(mh, THREAD);
   }
-  CompLevel next_level = call_event(mh, level, thread);
+  CompLevel next_level = call_event(mh, level, THREAD);
   if (next_level != level) {
-    if (maybe_switch_to_aot(mh, level, next_level, thread)) {
+    if (maybe_switch_to_aot(mh, level, next_level, THREAD)) {
       // No JITting necessary
       return;
     }
     if (is_compilation_enabled() && !CompileBroker::compilation_is_in_queue(mh)) {
-      compile(mh, InvocationEntryBci, next_level, thread);
+      compile(mh, InvocationEntryBci, next_level, THREAD);
     }
   }
 }
@@ -1072,21 +1084,21 @@ void TieredThresholdPolicy::method_invocation_event(const methodHandle& mh, cons
 // Handle the back branch event. Notice that we can compile the method
 // with a regular entry from here.
 void TieredThresholdPolicy::method_back_branch_event(const methodHandle& mh, const methodHandle& imh,
-                                                     int bci, CompLevel level, CompiledMethod* nm, JavaThread* thread) {
+                                                     int bci, CompLevel level, CompiledMethod* nm, TRAPS) {
   if (should_create_mdo(mh, level)) {
-    create_mdo(mh, thread);
+    create_mdo(mh, THREAD);
   }
   // Check if MDO should be created for the inlined method
   if (should_create_mdo(imh, level)) {
-    create_mdo(imh, thread);
+    create_mdo(imh, THREAD);
   }
 
   if (is_compilation_enabled()) {
-    CompLevel next_osr_level = loop_event(imh, level, thread);
+    CompLevel next_osr_level = loop_event(imh, level, THREAD);
     CompLevel max_osr_level = (CompLevel)imh->highest_osr_comp_level();
     // At the very least compile the OSR version
     if (!CompileBroker::compilation_is_in_queue(imh) && (next_osr_level != level)) {
-      compile(imh, bci, next_osr_level, thread);
+      compile(imh, bci, next_osr_level, CHECK);
     }
 
     // Use loop event as an opportunity to also check if there's been
@@ -1097,13 +1109,13 @@ void TieredThresholdPolicy::method_back_branch_event(const methodHandle& mh, con
         // Recompile the enclosing method to prevent infinite OSRs. Stay at AOT level while it's compiling.
         if (max_osr_level != CompLevel_none && !CompileBroker::compilation_is_in_queue(mh)) {
           CompLevel enclosing_level = limit_level(CompLevel_full_profile);
-          compile(mh, InvocationEntryBci, enclosing_level, thread);
+          compile(mh, InvocationEntryBci, enclosing_level, THREAD);
         }
       } else {
         // Current loop event level is not AOT
         guarantee(nm != NULL, "Should have nmethod here");
         cur_level = comp_level(mh());
-        next_level = call_event(mh, cur_level, thread);
+        next_level = call_event(mh, cur_level, THREAD);
 
         if (max_osr_level == CompLevel_full_optimization) {
           // The inlinee OSRed to full opt, we need to modify the enclosing method to avoid deopts
@@ -1132,17 +1144,17 @@ void TieredThresholdPolicy::method_back_branch_event(const methodHandle& mh, con
           next_level = CompLevel_full_profile;
         }
         if (cur_level != next_level) {
-          if (!maybe_switch_to_aot(mh, cur_level, next_level, thread) && !CompileBroker::compilation_is_in_queue(mh)) {
-            compile(mh, InvocationEntryBci, next_level, thread);
+          if (!maybe_switch_to_aot(mh, cur_level, next_level, THREAD) && !CompileBroker::compilation_is_in_queue(mh)) {
+            compile(mh, InvocationEntryBci, next_level, THREAD);
           }
         }
       }
     } else {
       cur_level = comp_level(mh());
-      next_level = call_event(mh, cur_level, thread);
+      next_level = call_event(mh, cur_level, THREAD);
       if (next_level != cur_level) {
-        if (!maybe_switch_to_aot(mh, cur_level, next_level, thread) && !CompileBroker::compilation_is_in_queue(mh)) {
-          compile(mh, InvocationEntryBci, next_level, thread);
+        if (!maybe_switch_to_aot(mh, cur_level, next_level, THREAD) && !CompileBroker::compilation_is_in_queue(mh)) {
+          compile(mh, InvocationEntryBci, next_level, THREAD);
         }
       }
     }
