@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,8 +22,8 @@
  *
  */
 
-#ifndef SHARE_VM_GC_G1_G1ALLOCREGION_INLINE_HPP
-#define SHARE_VM_GC_G1_G1ALLOCREGION_INLINE_HPP
+#ifndef SHARE_GC_G1_G1ALLOCREGION_INLINE_HPP
+#define SHARE_GC_G1_G1ALLOCREGION_INLINE_HPP
 
 #include "gc/g1/g1AllocRegion.hpp"
 #include "gc/g1/heapRegion.inline.hpp"
@@ -143,4 +143,4 @@ inline HeapWord* MutatorAllocRegion::attempt_retained_allocation(size_t min_word
   return NULL;
 }
 
-#endif // SHARE_VM_GC_G1_G1ALLOCREGION_INLINE_HPP
+#endif // SHARE_GC_G1_G1ALLOCREGION_INLINE_HPP
