@@ -281,22 +281,9 @@ HeapWord* G1Allocator::old_attempt_allocation(size_t min_word_size,
   return result;
 }
 
-uint G1PLABAllocator::calc_survivor_alignment_bytes() {
-  assert(SurvivorAlignmentInBytes >= ObjectAlignmentInBytes, "sanity");
-  if (SurvivorAlignmentInBytes == ObjectAlignmentInBytes) {
-    // No need to align objects in the survivors differently, return 0
-    // which means "survivor alignment is not used".
-    return 0;
-  } else {
-    assert(SurvivorAlignmentInBytes > 0, "sanity");
-    return SurvivorAlignmentInBytes;
-  }
-}
-
 G1PLABAllocator::G1PLABAllocator(G1Allocator* allocator) :
   _g1h(G1CollectedHeap::heap()),
-  _allocator(allocator),
-  _survivor_alignment_bytes(calc_survivor_alignment_bytes()) {
+  _allocator(allocator) {
   for (region_type_t state = 0; state < G1HeapRegionAttr::Num; state++) {
     _direct_allocated[state] = 0;
     uint length = alloc_buffers_length(state);
@@ -411,15 +398,8 @@ size_t G1PLABAllocator::undo_waste() const {
   return result;
 }
 
-bool G1ArchiveAllocator::_archive_check_enabled = false;
-G1ArchiveRegionMap G1ArchiveAllocator::_archive_region_map;
-
 G1ArchiveAllocator* G1ArchiveAllocator::create_allocator(G1CollectedHeap* g1h, bool open) {
-  // Create the archive allocator, and also enable archive object checking
-  // in mark-sweep, since we will be creating archive regions.
-  G1ArchiveAllocator* result =  new G1ArchiveAllocator(g1h, open);
-  enable_archive_object_check();
-  return result;
+  return new G1ArchiveAllocator(g1h, open);
 }
 
 bool G1ArchiveAllocator::alloc_new_region() {
@@ -447,9 +427,6 @@ bool G1ArchiveAllocator::alloc_new_region() {
   _bottom = hr->bottom();
   _max = _bottom + HeapRegion::min_region_size_in_words();
 
-  // Tell mark-sweep that objects in this region are not to be marked.
-  set_range_archive(MemRegion(_bottom, HeapRegion::GrainWords), _open);
-
   // Since we've modified the old set, call update_sizes.
   _g1h->g1mm()->update_sizes();
   return true;
