@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,7 +25,7 @@
 #include "precompiled.hpp"
 #include "code/codeCache.hpp"
 #include "gc/g1/g1CollectedHeap.hpp"
-#include "gc/g1/g1FullCollector.hpp"
+#include "gc/g1/g1FullCollector.inline.hpp"
 #include "gc/g1/g1FullGCAdjustTask.hpp"
 #include "gc/g1/g1FullGCCompactTask.hpp"
 #include "gc/g1/g1FullGCMarker.inline.hpp"
@@ -111,21 +111,23 @@ G1FullCollector::G1FullCollector(G1CollectedHeap* heap, bool explicit_gc, bool c
     _array_queue_set(_num_workers),
     _preserved_marks_set(true),
     _serial_compaction_point(),
-    _is_alive(heap->concurrent_mark()->next_mark_bitmap()),
+    _is_alive(this, heap->concurrent_mark()->next_mark_bitmap()),
     _is_alive_mutator(heap->ref_processor_stw(), &_is_alive),
     _always_subject_to_discovery(),
-    _is_subject_mutator(heap->ref_processor_stw(), &_always_subject_to_discovery) {
+    _is_subject_mutator(heap->ref_processor_stw(), &_always_subject_to_discovery),
+    _region_attr_table() {
   assert(SafepointSynchronize::is_at_safepoint(), "must be at a safepoint");
 
   _preserved_marks_set.init(_num_workers);
   _markers = NEW_C_HEAP_ARRAY(G1FullGCMarker*, _num_workers, mtGC);
   _compaction_points = NEW_C_HEAP_ARRAY(G1FullGCCompactionPoint*, _num_workers, mtGC);
   for (uint i = 0; i < _num_workers; i++) {
-    _markers[i] = new G1FullGCMarker(i, _preserved_marks_set.get(i), mark_bitmap());
+    _markers[i] = new G1FullGCMarker(this, i, _preserved_marks_set.get(i));
     _compaction_points[i] = new G1FullGCCompactionPoint();
     _oop_queue_set.register_queue(i, marker(i)->oop_stack());
     _array_queue_set.register_queue(i, marker(i)->objarray_stack());
   }
+  _region_attr_table.initialize(heap->reserved(), HeapRegion::GrainBytes);
 }
 
 G1FullCollector::~G1FullCollector() {
@@ -137,6 +139,19 @@ G1FullCollector::~G1FullCollector() {
   FREE_C_HEAP_ARRAY(G1FullGCCompactionPoint*, _compaction_points);
 }
 
+class PrepareRegionsClosure : public HeapRegionClosure {
+  G1FullCollector* _collector;
+
+public:
+  PrepareRegionsClosure(G1FullCollector* collector) : _collector(collector) { }
+
+  bool do_heap_region(HeapRegion* hr) {
+    G1CollectedHeap::heap()->prepare_region_for_full_compaction(hr);
+    _collector->update_attribute_table(hr);
+    return false;
+  }
+};
+
 void G1FullCollector::prepare_collection() {
   _heap->policy()->record_full_collection_start();
 
@@ -149,6 +164,9 @@ void G1FullCollector::prepare_collection() {
   _heap->gc_prologue(true);
   _heap->prepare_heap_for_full_collection();
 
+  PrepareRegionsClosure cl(this);
+  _heap->heap_region_iterate(&cl);
+
   reference_processor()->enable_discovery();
   reference_processor()->setup_policy(scope()->should_clear_soft_refs());
 
@@ -184,6 +202,10 @@ void G1FullCollector::complete_collection() {
 
   BiasedLocking::restore_marks();
 
+  _heap->concurrent_mark()->swap_mark_bitmaps();
+  // Prepare the bitmap for the next (potentially concurrent) marking.
+  _heap->concurrent_mark()->clear_next_bitmap(_heap->workers());
+
   _heap->prepare_heap_for_mutators();
 
   _heap->policy()->record_full_collection_end();
@@ -194,17 +216,34 @@ void G1FullCollector::complete_collection() {
   _heap->print_heap_after_full_collection(scope()->heap_transition());
 }
 
+void G1FullCollector::update_attribute_table(HeapRegion* hr) {
+  if (hr->is_free()) {
+    return;
+  }
+  if (hr->is_closed_archive()) {
+    _region_attr_table.set_closed_archive(hr->hrm_index());
+  } else if (hr->is_pinned()) {
+    _region_attr_table.set_pinned(hr->hrm_index());
+  } else {
+    _region_attr_table.set_normal(hr->hrm_index());
+  }
+}
+
 void G1FullCollector::phase1_mark_live_objects() {
   // Recursively traverse all live objects and mark them.
   GCTraceTime(Info, gc, phases) info("Phase 1: Mark live objects", scope()->timer());
 
-  // Do the actual marking.
-  G1FullGCMarkTask marking_task(this);
-  run_task(&marking_task);
+  {
+    // Do the actual marking.
+    G1FullGCMarkTask marking_task(this);
+    run_task(&marking_task);
+  }
 
-  // Process references discovered during marking.
-  G1FullGCReferenceProcessingExecutor reference_processing(this);
-  reference_processing.execute(scope()->timer(), scope()->tracer());
+  {
+    // Process references discovered during marking.
+    G1FullGCReferenceProcessingExecutor reference_processing(this);
+    reference_processing.execute(scope()->timer(), scope()->tracer());
+  }
 
   // Weak oops cleanup.
   {
@@ -259,8 +298,7 @@ void G1FullCollector::phase4_do_compaction() {
 }
 
 void G1FullCollector::restore_marks() {
-  SharedRestorePreservedMarksTaskExecutor task_executor(_heap->workers());
-  _preserved_marks_set.restore(&task_executor);
+  _preserved_marks_set.restore(_heap->workers());
   _preserved_marks_set.reclaim();
 }
 
@@ -274,7 +312,6 @@ void G1FullCollector::verify_after_marking() {
     return;
   }
 
-  HandleMark hm;  // handle scope
 #if COMPILER2_OR_JVMCI
   DerivedPointerTableDeactivate dpt_deact;
 #endif
