@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,8 +25,10 @@
 #ifndef SHARE_GC_G1_G1FULLGCMARKER_INLINE_HPP
 #define SHARE_GC_G1_G1FULLGCMARKER_INLINE_HPP
 
+#include "classfile/javaClasses.inline.hpp"
 #include "gc/g1/g1Allocator.inline.hpp"
 #include "gc/g1/g1ConcurrentMarkBitMap.inline.hpp"
+#include "gc/g1/g1FullCollector.inline.hpp"
 #include "gc/g1/g1FullGCMarker.hpp"
 #include "gc/g1/g1FullGCOopClosures.inline.hpp"
 #include "gc/g1/g1StringDedup.hpp"
@@ -38,8 +40,7 @@
 #include "utilities/debug.hpp"
 
 inline bool G1FullGCMarker::mark_object(oop obj) {
-  // Not marking closed archive objects.
-  if (G1ArchiveAllocator::is_closed_archive_object(obj)) {
+  if (_collector->is_in_closed(obj)) {
     return false;
   }
 
@@ -50,14 +51,17 @@ inline bool G1FullGCMarker::mark_object(oop obj) {
   }
 
   // Marked by us, preserve if needed.
-  markOop mark = obj->mark_raw();
-  if (mark->must_be_preserved(obj) &&
-      !G1ArchiveAllocator::is_open_archive_object(obj)) {
+  markWord mark = obj->mark();
+  if (obj->mark_must_be_preserved(mark) &&
+      // It is not necessary to preserve marks for objects in pinned regions because
+      // we do not change their headers (i.e. forward them).
+      !_collector->is_in_pinned(obj)) {
     preserved_stack()->push(obj, mark);
   }
 
   // Check if deduplicatable string.
-  if (G1StringDedup::is_enabled()) {
+  if (G1StringDedup::is_enabled() &&
+      java_lang_String::is_instance_inlined(obj)) {
     G1StringDedup::enqueue_from_mark(obj, _worker_id);
   }
   return true;
@@ -71,7 +75,7 @@ template <class T> inline void G1FullGCMarker::mark_and_push(T* p) {
       _oop_stack.push(obj);
       assert(_bitmap->is_marked(obj), "Must be marked now - map self");
     } else {
-      assert(_bitmap->is_marked(obj) || G1ArchiveAllocator::is_closed_archive_object(obj),
+      assert(_bitmap->is_marked(obj) || _collector->is_in_closed(obj),
              "Must be marked by other or closed archive object");
     }
   }
