@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,6 +23,7 @@
  */
 
 #include "precompiled.hpp"
+#include "classfile/systemDictionary.hpp"
 #include "gc/g1/g1Allocator.inline.hpp"
 #include "gc/g1/g1CollectedHeap.inline.hpp"
 #include "gc/g1/g1CollectionSet.hpp"
@@ -31,19 +32,32 @@
 #include "gc/g1/g1RootClosures.hpp"
 #include "gc/g1/g1StringDedup.hpp"
 #include "gc/g1/g1Trace.hpp"
+#include "gc/shared/partialArrayTaskStepper.inline.hpp"
 #include "gc/shared/taskqueue.inline.hpp"
 #include "memory/allocation.inline.hpp"
 #include "oops/access.inline.hpp"
 #include "oops/oop.inline.hpp"
+#include "runtime/atomic.hpp"
 #include "runtime/prefetch.inline.hpp"
+#include "utilities/globalDefinitions.hpp"
+#include "utilities/macros.hpp"
+
+// In fastdebug builds the code size can get out of hand, potentially
+// tripping over compiler limits (which may be bugs, but nevertheless
+// need to be taken into consideration).  A side benefit of limiting
+// inlining is that we get more call frames that might aid debugging.
+// And the fastdebug compile time for this file is much reduced.
+// Explicit NOINLINE to block ATTRIBUTE_FLATTENing.
+#define MAYBE_INLINE_EVACUATION NOT_DEBUG(inline) DEBUG_ONLY(NOINLINE)
 
 G1ParScanThreadState::G1ParScanThreadState(G1CollectedHeap* g1h,
                                            G1RedirtyCardsQueueSet* rdcqs,
                                            uint worker_id,
+                                           uint n_workers,
                                            size_t young_cset_length,
                                            size_t optional_cset_length)
   : _g1h(g1h),
-    _refs(g1h->task_queue(worker_id)),
+    _task_queue(g1h->task_queue(worker_id)),
     _rdcq(rdcqs),
     _ct(g1h->card_table()),
     _closures(NULL),
@@ -60,17 +74,28 @@ G1ParScanThreadState::G1ParScanThreadState(G1CollectedHeap* g1h,
     _surviving_young_words(NULL),
     _surviving_words_length(young_cset_length + 1),
     _old_gen_is_full(false),
+    _partial_objarray_chunk_size(ParGCArrayScanChunk),
+    _partial_array_stepper(n_workers),
+    _string_klass_or_null(G1StringDedup::is_enabled()
+                          ? SystemDictionary::String_klass()
+                          : nullptr),
     _num_optional_regions(optional_cset_length),
     _numa(g1h->numa()),
     _obj_alloc_stat(NULL)
 {
+  // Verify klass comparison with _string_klass_or_null is sufficient
+  // to determine whether dedup is enabled and the object is a String.
+  assert(SystemDictionary::String_klass()->is_final(), "precondition");
+
   // We allocate number of young gen regions in the collection set plus one
   // entries, since entry 0 keeps track of surviving bytes for non-young regions.
   // We also add a few elements at the beginning and at the end in
   // an attempt to eliminate cache contention
-  size_t array_length = PADDING_ELEM_NUM + _surviving_words_length + PADDING_ELEM_NUM;
+  const size_t padding_elem_num = (DEFAULT_CACHE_LINE_SIZE / sizeof(size_t));
+  size_t array_length = padding_elem_num + _surviving_words_length + padding_elem_num;
+
   _surviving_young_words_base = NEW_C_HEAP_ARRAY(size_t, array_length, mtGC);
-  _surviving_young_words = _surviving_young_words_base + PADDING_ELEM_NUM;
+  _surviving_young_words = _surviving_young_words_base + padding_elem_num;
   memset(_surviving_young_words, 0, _surviving_words_length * sizeof(size_t));
 
   _plab_allocator = new G1PLABAllocator(_g1h->allocator());
@@ -119,46 +144,181 @@ size_t G1ParScanThreadState::lab_undo_waste_words() const {
 }
 
 #ifdef ASSERT
-bool G1ParScanThreadState::verify_ref(narrowOop* ref) const {
-  assert(ref != NULL, "invariant");
+void G1ParScanThreadState::verify_task(narrowOop* task) const {
+  assert(task != NULL, "invariant");
   assert(UseCompressedOops, "sanity");
-  assert(!has_partial_array_mask(ref), "ref=" PTR_FORMAT, p2i(ref));
-  oop p = RawAccess<>::oop_load(ref);
-  assert(_g1h->is_in_g1_reserved(p),
-         "ref=" PTR_FORMAT " p=" PTR_FORMAT, p2i(ref), p2i(p));
-  return true;
+  oop p = RawAccess<>::oop_load(task);
+  assert(_g1h->is_in_reserved(p),
+         "task=" PTR_FORMAT " p=" PTR_FORMAT, p2i(task), p2i(p));
 }
 
-bool G1ParScanThreadState::verify_ref(oop* ref) const {
-  assert(ref != NULL, "invariant");
-  if (has_partial_array_mask(ref)) {
-    // Must be in the collection set--it's already been copied.
-    oop p = clear_partial_array_mask(ref);
-    assert(_g1h->is_in_cset(p),
-           "ref=" PTR_FORMAT " p=" PTR_FORMAT, p2i(ref), p2i(p));
-  } else {
-    oop p = RawAccess<>::oop_load(ref);
-    assert(_g1h->is_in_g1_reserved(p),
-           "ref=" PTR_FORMAT " p=" PTR_FORMAT, p2i(ref), p2i(p));
-  }
-  return true;
+void G1ParScanThreadState::verify_task(oop* task) const {
+  assert(task != NULL, "invariant");
+  oop p = RawAccess<>::oop_load(task);
+  assert(_g1h->is_in_reserved(p),
+         "task=" PTR_FORMAT " p=" PTR_FORMAT, p2i(task), p2i(p));
 }
 
-bool G1ParScanThreadState::verify_task(StarTask ref) const {
-  if (ref.is_narrow()) {
-    return verify_ref((narrowOop*) ref);
+void G1ParScanThreadState::verify_task(PartialArrayScanTask task) const {
+  // Must be in the collection set--it's already been copied.
+  oop p = task.to_source_array();
+  assert(_g1h->is_in_cset(p), "p=" PTR_FORMAT, p2i(p));
+}
+
+void G1ParScanThreadState::verify_task(ScannerTask task) const {
+  if (task.is_narrow_oop_ptr()) {
+    verify_task(task.to_narrow_oop_ptr());
+  } else if (task.is_oop_ptr()) {
+    verify_task(task.to_oop_ptr());
+  } else if (task.is_partial_array_task()) {
+    verify_task(task.to_partial_array_task());
   } else {
-    return verify_ref((oop*) ref);
+    ShouldNotReachHere();
   }
 }
 #endif // ASSERT
 
-void G1ParScanThreadState::trim_queue() {
-  StarTask ref;
+template <class T>
+MAYBE_INLINE_EVACUATION
+void G1ParScanThreadState::do_oop_evac(T* p) {
+  // Reference should not be NULL here as such are never pushed to the task queue.
+  oop obj = RawAccess<IS_NOT_NULL>::oop_load(p);
+
+  // Although we never intentionally push references outside of the collection
+  // set, due to (benign) races in the claim mechanism during RSet scanning more
+  // than one thread might claim the same card. So the same card may be
+  // processed multiple times, and so we might get references into old gen here.
+  // So we need to redo this check.
+  const G1HeapRegionAttr region_attr = _g1h->region_attr(obj);
+  // References pushed onto the work stack should never point to a humongous region
+  // as they are not added to the collection set due to above precondition.
+  assert(!region_attr.is_humongous(),
+         "Obj " PTR_FORMAT " should not refer to humongous region %u from " PTR_FORMAT,
+         p2i(obj), _g1h->addr_to_region(cast_from_oop<HeapWord*>(obj)), p2i(p));
+
+  if (!region_attr.is_in_cset()) {
+    // In this case somebody else already did all the work.
+    return;
+  }
+
+  markWord m = obj->mark();
+  if (m.is_marked()) {
+    obj = (oop) m.decode_pointer();
+  } else {
+    obj = do_copy_to_survivor_space(region_attr, obj, m);
+  }
+  RawAccess<IS_NOT_NULL>::oop_store(p, obj);
+
+  assert(obj != NULL, "Must be");
+  if (HeapRegion::is_in_same_region(p, obj)) {
+    return;
+  }
+  HeapRegion* from = _g1h->heap_region_containing(p);
+  if (!from->is_young()) {
+    enqueue_card_if_tracked(_g1h->region_attr(obj), p, obj);
+  }
+}
+
+MAYBE_INLINE_EVACUATION
+void G1ParScanThreadState::do_partial_array(PartialArrayScanTask task) {
+  oop from_obj = task.to_source_array();
+
+  assert(_g1h->is_in_reserved(from_obj), "must be in heap.");
+  assert(from_obj->is_objArray(), "must be obj array");
+  assert(from_obj->is_forwarded(), "must be forwarded");
+
+  oop to_obj = from_obj->forwardee();
+  assert(from_obj != to_obj, "should not be chunking self-forwarded objects");
+  assert(to_obj->is_objArray(), "must be obj array");
+  objArrayOop to_array = objArrayOop(to_obj);
+
+  PartialArrayTaskStepper::Step step
+    = _partial_array_stepper.next(objArrayOop(from_obj),
+                                  to_array,
+                                  _partial_objarray_chunk_size);
+  for (uint i = 0; i < step._ncreate; ++i) {
+    push_on_queue(ScannerTask(PartialArrayScanTask(from_obj)));
+  }
+
+  HeapRegion* hr = _g1h->heap_region_containing(to_array);
+  G1ScanInYoungSetter x(&_scanner, hr->is_young());
+  // Process claimed task.  The length of to_array is not correct, but
+  // fortunately the iteration ignores the length field and just relies
+  // on start/end.
+  to_array->oop_iterate_range(&_scanner,
+                              step._index,
+                              step._index + _partial_objarray_chunk_size);
+}
+
+MAYBE_INLINE_EVACUATION
+void G1ParScanThreadState::start_partial_objarray(G1HeapRegionAttr dest_attr,
+                                                  oop from_obj,
+                                                  oop to_obj) {
+  assert(from_obj->is_objArray(), "precondition");
+  assert(from_obj->is_forwarded(), "precondition");
+  assert(from_obj->forwardee() == to_obj, "precondition");
+  assert(from_obj != to_obj, "should not be scanning self-forwarded objects");
+  assert(to_obj->is_objArray(), "precondition");
+
+  objArrayOop to_array = objArrayOop(to_obj);
+
+  PartialArrayTaskStepper::Step step
+    = _partial_array_stepper.start(objArrayOop(from_obj),
+                                   to_array,
+                                   _partial_objarray_chunk_size);
+
+  // Push any needed partial scan tasks.  Pushed before processing the
+  // intitial chunk to allow other workers to steal while we're processing.
+  for (uint i = 0; i < step._ncreate; ++i) {
+    push_on_queue(ScannerTask(PartialArrayScanTask(from_obj)));
+  }
+
+  G1ScanInYoungSetter x(&_scanner, dest_attr.is_young());
+  // Process the initial chunk.  No need to process the type in the
+  // klass, as it will already be handled by processing the built-in
+  // module. The length of to_array is not correct, but fortunately
+  // the iteration ignores that length field and relies on start/end.
+  to_array->oop_iterate_range(&_scanner, 0, step._index);
+}
+
+MAYBE_INLINE_EVACUATION
+void G1ParScanThreadState::dispatch_task(ScannerTask task) {
+  verify_task(task);
+  if (task.is_narrow_oop_ptr()) {
+    do_oop_evac(task.to_narrow_oop_ptr());
+  } else if (task.is_oop_ptr()) {
+    do_oop_evac(task.to_oop_ptr());
+  } else {
+    do_partial_array(task.to_partial_array_task());
+  }
+}
+
+// Process tasks until overflow queue is empty and local queue
+// contains no more than threshold entries.  NOINLINE to prevent
+// inlining into steal_and_trim_queue.
+ATTRIBUTE_FLATTEN NOINLINE
+void G1ParScanThreadState::trim_queue_to_threshold(uint threshold) {
+  ScannerTask task;
   do {
-    // Fully drain the queue.
-    trim_queue_to_threshold(0);
-  } while (!_refs->is_empty());
+    while (_task_queue->pop_overflow(task)) {
+      if (!_task_queue->try_push_to_taskqueue(task)) {
+        dispatch_task(task);
+      }
+    }
+    while (_task_queue->pop_local(task, threshold)) {
+      dispatch_task(task);
+    }
+  } while (!_task_queue->overflow_empty());
+}
+
+ATTRIBUTE_FLATTEN
+void G1ParScanThreadState::steal_and_trim_queue(G1ScannerTasksQueueSet* task_queues) {
+  ScannerTask stolen_task;
+  while (task_queues->steal(_worker_id, stolen_task)) {
+    dispatch_task(stolen_task);
+    // Processing stolen task may have added tasks to our queue.
+    trim_queue();
+  }
 }
 
 HeapWord* G1ParScanThreadState::allocate_in_next_plab(G1HeapRegionAttr* dest,
@@ -226,18 +386,61 @@ void G1ParScanThreadState::report_promotion_event(G1HeapRegionAttr const dest_at
   }
 }
 
-oop G1ParScanThreadState::copy_to_survivor_space(G1HeapRegionAttr const region_attr,
-                                                 oop const old,
-                                                 markWord const old_mark) {
-  const size_t word_sz = old->size();
+NOINLINE
+HeapWord* G1ParScanThreadState::allocate_copy_slow(G1HeapRegionAttr* dest_attr,
+                                                   oop old,
+                                                   size_t word_sz,
+                                                   uint age,
+                                                   uint node_index) {
+  HeapWord* obj_ptr = NULL;
+  // Try slow-path allocation unless we're allocating old and old is already full.
+  if (!(dest_attr->is_old() && _old_gen_is_full)) {
+    bool plab_refill_failed = false;
+    obj_ptr = _plab_allocator->allocate_direct_or_new_plab(*dest_attr,
+                                                           word_sz,
+                                                           &plab_refill_failed,
+                                                           node_index);
+    if (obj_ptr == NULL) {
+      obj_ptr = allocate_in_next_plab(dest_attr,
+                                      word_sz,
+                                      plab_refill_failed,
+                                      node_index);
+    }
+  }
+  if (obj_ptr != NULL) {
+    update_numa_stats(node_index);
+    if (_g1h->_gc_tracer_stw->should_report_promotion_events()) {
+      // The events are checked individually as part of the actual commit
+      report_promotion_event(*dest_attr, old, word_sz, age, obj_ptr, node_index);
+    }
+  }
+  return obj_ptr;
+}
+
+NOINLINE
+void G1ParScanThreadState::undo_allocation(G1HeapRegionAttr dest_attr,
+                                           HeapWord* obj_ptr,
+                                           size_t word_sz,
+                                           uint node_index) {
+  _plab_allocator->undo_allocation(dest_attr, obj_ptr, word_sz, node_index);
+}
+
+// Private inline function, for direct internal use and providing the
+// implementation of the public not-inline function.
+MAYBE_INLINE_EVACUATION
+oop G1ParScanThreadState::do_copy_to_survivor_space(G1HeapRegionAttr const region_attr,
+                                                    oop const old,
+                                                    markWord const old_mark) {
+  assert(region_attr.is_in_cset(),
+         "Unexpected region attr type: %s", region_attr.get_type_str());
+
+  // Get the klass once.  We'll need it again later, and this avoids
+  // re-decoding when it's compressed.
+  Klass* klass = old->klass();
+  const size_t word_sz = old->size_given_klass(klass);
 
   uint age = 0;
   G1HeapRegionAttr dest_attr = next_region_attr(region_attr, old_mark, age);
-  // The second clause is to prevent premature evacuation failure in case there
-  // is still space in survivor, but old gen is full.
-  if (_old_gen_is_full && dest_attr.is_old()) {
-    return handle_evacuation_failure_par(old, old_mark);
-  }
   HeapRegion* const from_region = _g1h->heap_region_containing(old);
   uint node_index = from_region->node_index();
 
@@ -246,22 +449,11 @@ oop G1ParScanThreadState::copy_to_survivor_space(G1HeapRegionAttr const region_a
   // PLAB allocations should succeed most of the time, so we'll
   // normally check against NULL once and that's it.
   if (obj_ptr == NULL) {
-    bool plab_refill_failed = false;
-    obj_ptr = _plab_allocator->allocate_direct_or_new_plab(dest_attr, word_sz, &plab_refill_failed, node_index);
+    obj_ptr = allocate_copy_slow(&dest_attr, old, word_sz, age, node_index);
     if (obj_ptr == NULL) {
-      assert(region_attr.is_in_cset(), "Unexpected region attr type: %s", region_attr.get_type_str());
-      obj_ptr = allocate_in_next_plab(&dest_attr, word_sz, plab_refill_failed, node_index);
-      if (obj_ptr == NULL) {
-        // This will either forward-to-self, or detect that someone else has
-        // installed a forwarding pointer.
-        return handle_evacuation_failure_par(old, old_mark);
-      }
-    }
-    update_numa_stats(node_index);
-
-    if (_g1h->_gc_tracer_stw->should_report_promotion_events()) {
-      // The events are checked individually as part of the actual commit
-      report_promotion_event(dest_attr, old, word_sz, age, obj_ptr, node_index);
+      // This will either forward-to-self, or detect that someone else has
+      // installed a forwarding pointer.
+      return handle_evacuation_failure_par(old, old_mark);
     }
   }
 
@@ -273,7 +465,7 @@ oop G1ParScanThreadState::copy_to_survivor_space(G1HeapRegionAttr const region_a
   if (_g1h->evacuation_should_fail()) {
     // Doing this after all the allocation attempts also tests the
     // undo_allocation() method too.
-    _plab_allocator->undo_allocation(dest_attr, obj_ptr, word_sz, node_index);
+    undo_allocation(dest_attr, obj_ptr, word_sz, node_index);
     return handle_evacuation_failure_par(old, old_mark);
   }
 #endif // !PRODUCT
@@ -284,33 +476,51 @@ oop G1ParScanThreadState::copy_to_survivor_space(G1HeapRegionAttr const region_a
   const oop obj = oop(obj_ptr);
   const oop forward_ptr = old->forward_to_atomic(obj, old_mark, memory_order_relaxed);
   if (forward_ptr == NULL) {
-    Copy::aligned_disjoint_words((HeapWord*) old, obj_ptr, word_sz);
+    Copy::aligned_disjoint_words(cast_from_oop<HeapWord*>(old), obj_ptr, word_sz);
 
-    const uint young_index = from_region->young_index_in_cset();
-
-    assert((from_region->is_young() && young_index >  0) ||
-           (!from_region->is_young() && young_index == 0), "invariant" );
+    {
+      const uint young_index = from_region->young_index_in_cset();
+      assert((from_region->is_young() && young_index >  0) ||
+             (!from_region->is_young() && young_index == 0), "invariant" );
+      _surviving_young_words[young_index] += word_sz;
+    }
 
     if (dest_attr.is_young()) {
       if (age < markWord::max_age) {
         age++;
       }
       if (old_mark.has_displaced_mark_helper()) {
-        // In this case, we have to install the mark word first,
-        // otherwise obj looks to be forwarded (the old mark word,
-        // which contains the forward pointer, was copied)
-        obj->set_mark_raw(old_mark);
+        // In this case, we have to install the old mark word containing the
+        // displacement tag, and update the age in the displaced mark word.
         markWord new_mark = old_mark.displaced_mark_helper().set_age(age);
         old_mark.set_displaced_mark_helper(new_mark);
+        obj->set_mark(old_mark);
       } else {
-        obj->set_mark_raw(old_mark.set_age(age));
+        obj->set_mark(old_mark.set_age(age));
       }
       _age_table.add(age, word_sz);
     } else {
-      obj->set_mark_raw(old_mark);
+      obj->set_mark(old_mark);
     }
 
-    if (G1StringDedup::is_enabled()) {
+    // Most objects are not arrays, so do one array check rather than
+    // checking for each array category for each object.
+    if (klass->is_array_klass()) {
+      if (klass->is_objArray_klass()) {
+        start_partial_objarray(dest_attr, old, obj);
+      } else {
+        // Nothing needs to be done for typeArrays.  Body doesn't contain
+        // any oops to scan, and the type in the klass will already be handled
+        // by processing the built-in module.
+        assert(klass->is_typeArray_klass(), "invariant");
+      }
+      return obj;
+    }
+
+    // StringDedup::is_enabled() and java_lang_String::is_instance_inline
+    // test of the obj, combined into a single comparison, using the klass
+    // already in hand and avoiding the null check in is_instance.
+    if (klass == _string_klass_or_null) {
       const bool is_from_young = region_attr.is_young();
       const bool is_to_young = dest_attr.is_young();
       assert(is_from_young == from_region->is_young(),
@@ -323,31 +533,31 @@ oop G1ParScanThreadState::copy_to_survivor_space(G1HeapRegionAttr const region_a
                                              obj);
     }
 
-    _surviving_young_words[young_index] += word_sz;
-
-    if (obj->is_objArray() && arrayOop(obj)->length() >= ParGCArrayScanChunk) {
-      // We keep track of the next start index in the length field of
-      // the to-space object. The actual length can be found in the
-      // length field of the from-space object.
-      arrayOop(obj)->set_length(0);
-      oop* old_p = set_partial_array_mask(old);
-      do_oop_partial_array(old_p);
-    } else {
-      G1ScanInYoungSetter x(&_scanner, dest_attr.is_young());
-      obj->oop_iterate_backwards(&_scanner);
-    }
+    G1ScanInYoungSetter x(&_scanner, dest_attr.is_young());
+    obj->oop_iterate_backwards(&_scanner, klass);
     return obj;
+
   } else {
     _plab_allocator->undo_allocation(dest_attr, obj_ptr, word_sz, node_index);
     return forward_ptr;
   }
 }
 
+// Public not-inline entry point.
+ATTRIBUTE_FLATTEN
+oop G1ParScanThreadState::copy_to_survivor_space(G1HeapRegionAttr region_attr,
+                                                 oop old,
+                                                 markWord old_mark) {
+  return do_copy_to_survivor_space(region_attr, old, old_mark);
+}
+
 G1ParScanThreadState* G1ParScanThreadStateSet::state_for_worker(uint worker_id) {
   assert(worker_id < _n_workers, "out of bounds access");
   if (_states[worker_id] == NULL) {
     _states[worker_id] =
-      new G1ParScanThreadState(_g1h, _rdcqs, worker_id, _young_cset_length, _optional_cset_length);
+      new G1ParScanThreadState(_g1h, _rdcqs,
+                               worker_id, _n_workers,
+                               _young_cset_length, _optional_cset_length);
   }
   return _states[worker_id];
 }
@@ -398,6 +608,7 @@ void G1ParScanThreadStateSet::record_unused_optional_region(HeapRegion* hr) {
   }
 }
 
+NOINLINE
 oop G1ParScanThreadState::handle_evacuation_failure_par(oop old, markWord m) {
   assert(_g1h->is_in_cset(old), "Object " PTR_FORMAT " should be in the CSet", p2i(old));
 
@@ -428,6 +639,33 @@ oop G1ParScanThreadState::handle_evacuation_failure_par(oop old, markWord m) {
     return forward_ptr;
   }
 }
+
+void G1ParScanThreadState::initialize_numa_stats() {
+  if (_numa->is_enabled()) {
+    LogTarget(Info, gc, heap, numa) lt;
+
+    if (lt.is_enabled()) {
+      uint num_nodes = _numa->num_active_nodes();
+      // Record only if there are multiple active nodes.
+      _obj_alloc_stat = NEW_C_HEAP_ARRAY(size_t, num_nodes, mtGC);
+      memset(_obj_alloc_stat, 0, sizeof(size_t) * num_nodes);
+    }
+  }
+}
+
+void G1ParScanThreadState::flush_numa_stats() {
+  if (_obj_alloc_stat != NULL) {
+    uint node_index = _numa->index_of_current_thread();
+    _numa->copy_statistics(G1NUMAStats::LocalObjProcessAtCopyToSurv, node_index, _obj_alloc_stat);
+  }
+}
+
+void G1ParScanThreadState::update_numa_stats(uint node_index) {
+  if (_obj_alloc_stat != NULL) {
+    _obj_alloc_stat[node_index]++;
+  }
+}
+
 G1ParScanThreadStateSet::G1ParScanThreadStateSet(G1CollectedHeap* g1h,
                                                  G1RedirtyCardsQueueSet* rdcqs,
                                                  uint n_workers,
