@@ -27,6 +27,8 @@
 #include "gc/g1/g1ConcurrentMarkThread.inline.hpp"
 #include "gc/g1/g1Policy.hpp"
 #include "gc/g1/g1VMOperations.hpp"
+#include "gc/shared/concurrentGCBreakpoints.hpp"
+#include "gc/shared/gcCause.hpp"
 #include "gc/shared/gcId.hpp"
 #include "gc/shared/gcTimer.hpp"
 #include "gc/shared/gcTraceTime.inline.hpp"
@@ -47,6 +49,7 @@ VM_G1TryInitiateConcMark::VM_G1TryInitiateConcMark(uint gc_count_before,
   _target_pause_time_ms(target_pause_time_ms),
   _transient_failure(false),
   _cycle_already_in_progress(false),
+  _whitebox_attached(false),
   _terminating(false),
   _gc_succeeded(false)
 {}
@@ -54,11 +57,11 @@ VM_G1TryInitiateConcMark::VM_G1TryInitiateConcMark(uint gc_count_before,
 bool VM_G1TryInitiateConcMark::doit_prologue() {
   bool result = VM_GC_Operation::doit_prologue();
   // The prologue can fail for a couple of reasons. The first is that another GC
-  // got scheduled and prevented the scheduling of the initial mark GC. The
+  // got scheduled and prevented the scheduling of the concurrent start GC. The
   // second is that the GC locker may be active and the heap can't be expanded.
-  // In both cases we want to retry the GC so that the initial mark pause is
+  // In both cases we want to retry the GC so that the concurrent start pause is
   // actually scheduled. In the second case, however, we should stall until
-  // until the GC locker is no longer active and then retry the initial mark GC.
+  // until the GC locker is no longer active and then retry the concurrent start GC.
   if (!result) _transient_failure = true;
   return result;
 }
@@ -77,25 +80,26 @@ void VM_G1TryInitiateConcMark::doit() {
     // a young-only or mixed GC (depending on phase).  For a user request
     // there's no point in even doing that much, so done.  For some non-user
     // requests the alternative GC might still be needed.
-  } else if (!g1h->policy()->force_initial_mark_if_outside_cycle(_gc_cause)) {
-    // Failure to force the next GC pause to be an initial mark indicates
+  } else if (!g1h->policy()->force_concurrent_start_if_outside_cycle(_gc_cause)) {
+    // Failure to force the next GC pause to be a concurrent start indicates
     // there is already a concurrent marking cycle in progress.  Set flag
     // to notify the caller and return immediately.
     _cycle_already_in_progress = true;
-  } else if (!g1h->do_collection_pause_at_safepoint(_target_pause_time_ms)) {
+  } else if ((_gc_cause != GCCause::_wb_breakpoint) &&
+             ConcurrentGCBreakpoints::is_controlled()) {
+    // WhiteBox wants to be in control of concurrent cycles, so don't try to
+    // start one.  This check is after the force_concurrent_start_xxx so that a
+    // request will be remembered for a later partial collection, even though
+    // we've rejected this request.
+    _whitebox_attached = true;
+  } else if (g1h->do_collection_pause_at_safepoint(_target_pause_time_ms)) {
+    _gc_succeeded = true;
+  } else {
     // Failure to perform the collection at all occurs because GCLocker is
     // active, and we have the bad luck to be the collection request that
     // makes a later _gc_locker collection needed.  (Else we would have hit
     // the GCLocker check in the prologue.)
     _transient_failure = true;
-  } else if (g1h->should_upgrade_to_full_gc(_gc_cause)) {
-    // GC ran, but we're still in trouble and need a full GC.
-    log_info(gc, ergo)("Attempting maximally compacting collection");
-    _gc_succeeded = g1h->do_full_collection(false, /* explicit gc */
-                                            true /* clear_all_soft_refs */);
-    guarantee(_gc_succeeded, "Elevated collections during the safepoint must always succeed");
-  } else {
-    _gc_succeeded = true;
   }
 }
 
@@ -132,20 +136,10 @@ void VM_G1CollectForAllocation::doit() {
   // Try a partial collection of some kind.
   _gc_succeeded = g1h->do_collection_pause_at_safepoint(_target_pause_time_ms);
 
-  if (_gc_succeeded) {
-    if (_word_size > 0) {
-      // An allocation had been requested. Do it, eventually trying a stronger
-      // kind of GC.
-      _result = g1h->satisfy_failed_allocation(_word_size, &_gc_succeeded);
-    } else if (g1h->should_upgrade_to_full_gc(_gc_cause)) {
-      // There has been a request to perform a GC to free some space. We have no
-      // information on how much memory has been asked for. In case there are
-      // absolutely no regions left to allocate into, do a maximally compacting full GC.
-      log_info(gc, ergo)("Attempting maximally compacting collection");
-      _gc_succeeded = g1h->do_full_collection(false, /* explicit gc */
-                                              true   /* clear_all_soft_refs */);
-    }
-    guarantee(_gc_succeeded, "Elevated collections during the safepoint must always succeed.");
+  if (_gc_succeeded && (_word_size > 0)) {
+    // An allocation had been requested. Do it, eventually trying a stronger
+    // kind of GC.
+    _result = g1h->satisfy_failed_allocation(_word_size, &_gc_succeeded);
   }
 }
 
@@ -153,7 +147,13 @@ void VM_G1Concurrent::doit() {
   GCIdMark gc_id_mark(_gc_id);
   GCTraceCPUTime tcpu;
   G1CollectedHeap* g1h = G1CollectedHeap::heap();
-  GCTraceTime(Info, gc) t(_message, g1h->concurrent_mark()->gc_timer_cm(), GCCause::_no_gc, true);
+
+  // GCTraceTime(...) only supports sub-phases, so a more verbose version
+  // is needed when we report the top-level pause phase.
+  GCTraceTimeLogger(Info, gc) logger(_message, GCCause::_no_gc, true);
+  GCTraceTimePauseTimer       timer(_message, g1h->concurrent_mark()->gc_timer_cm());
+  GCTraceTimeDriver           t(&logger, &timer);
+
   TraceCollectorStats tcs(g1h->g1mm()->conc_collection_counters());
   SvcGCMarker sgcm(SvcGCMarker::CONCURRENT);
   IsGCActiveMark x;
