@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -33,6 +33,7 @@
 #include "memory/padded.inline.hpp"
 #include "oops/oop.inline.hpp"
 #include "runtime/atomic.hpp"
+#include "runtime/globals_extension.hpp"
 #include "utilities/bitMap.inline.hpp"
 #include "utilities/debug.hpp"
 #include "utilities/formatBuffer.hpp"
@@ -69,8 +70,8 @@ OtherRegionsTable::OtherRegionsTable(Mutex* m) :
   _g1h(G1CollectedHeap::heap()),
   _m(m),
   _num_occupied(0),
-  _coarse_map(G1CollectedHeap::heap()->max_regions(), mtGC),
-  _n_coarse_entries(0),
+  _coarse_map(mtGC),
+  _has_coarse_entries(false),
   _fine_grain_regions(NULL),
   _n_fine_entries(0),
   _first_all_fine_prts(NULL),
@@ -92,14 +93,7 @@ OtherRegionsTable::OtherRegionsTable(Mutex* m) :
     _fine_eviction_stride = _max_fine_entries / _fine_eviction_sample_size;
   }
 
-  _fine_grain_regions = NEW_C_HEAP_ARRAY3(PerRegionTablePtr, _max_fine_entries,
-                        mtGC, CURRENT_PC, AllocFailStrategy::RETURN_NULL);
-
-  if (_fine_grain_regions == NULL) {
-    vm_exit_out_of_memory(sizeof(void*)*_max_fine_entries, OOM_MALLOC_ERROR,
-                          "Failed to allocate _fine_grain_entries.");
-  }
-
+  _fine_grain_regions = NEW_C_HEAP_ARRAY(PerRegionTablePtr, _max_fine_entries, mtGC);
   for (size_t i = 0; i < _max_fine_entries; i++) {
     _fine_grain_regions[i] = NULL;
   }
@@ -109,60 +103,20 @@ void OtherRegionsTable::link_to_all(PerRegionTable* prt) {
   // We always append to the beginning of the list for convenience;
   // the order of entries in this list does not matter.
   if (_first_all_fine_prts != NULL) {
-    assert(_first_all_fine_prts->prev() == NULL, "invariant");
-    _first_all_fine_prts->set_prev(prt);
     prt->set_next(_first_all_fine_prts);
   } else {
     // this is the first element we insert. Adjust the "last" pointer
     _last_all_fine_prts = prt;
     assert(prt->next() == NULL, "just checking");
   }
-  // the new element is always the first element without a predecessor
-  prt->set_prev(NULL);
   _first_all_fine_prts = prt;
 
-  assert(prt->prev() == NULL, "just checking");
   assert(_first_all_fine_prts == prt, "just checking");
   assert((_first_all_fine_prts == NULL && _last_all_fine_prts == NULL) ||
          (_first_all_fine_prts != NULL && _last_all_fine_prts != NULL),
          "just checking");
   assert(_last_all_fine_prts == NULL || _last_all_fine_prts->next() == NULL,
          "just checking");
-  assert(_first_all_fine_prts == NULL || _first_all_fine_prts->prev() == NULL,
-         "just checking");
-}
-
-void OtherRegionsTable::unlink_from_all(PerRegionTable* prt) {
-  if (prt->prev() != NULL) {
-    assert(_first_all_fine_prts != prt, "just checking");
-    prt->prev()->set_next(prt->next());
-    // removing the last element in the list?
-    if (_last_all_fine_prts == prt) {
-      _last_all_fine_prts = prt->prev();
-    }
-  } else {
-    assert(_first_all_fine_prts == prt, "just checking");
-    _first_all_fine_prts = prt->next();
-    // list is empty now?
-    if (_first_all_fine_prts == NULL) {
-      _last_all_fine_prts = NULL;
-    }
-  }
-
-  if (prt->next() != NULL) {
-    prt->next()->set_prev(prt->prev());
-  }
-
-  prt->set_next(NULL);
-  prt->set_prev(NULL);
-
-  assert((_first_all_fine_prts == NULL && _last_all_fine_prts == NULL) ||
-         (_first_all_fine_prts != NULL && _last_all_fine_prts != NULL),
-         "just checking");
-  assert(_last_all_fine_prts == NULL || _last_all_fine_prts->next() == NULL,
-         "just checking");
-  assert(_first_all_fine_prts == NULL || _first_all_fine_prts->prev() == NULL,
-         "just checking");
 }
 
 CardIdx_t OtherRegionsTable::card_within_region(OopOrNarrowOopStar within_region, HeapRegion* hr) {
@@ -179,7 +133,7 @@ void OtherRegionsTable::add_reference(OopOrNarrowOopStar from, uint tid) {
   RegionIdx_t from_hrm_ind = (RegionIdx_t) from_hr->hrm_index();
 
   // If the region is already coarsened, return.
-  if (_coarse_map.at(from_hrm_ind)) {
+  if (is_region_coarsened(from_hrm_ind)) {
     assert(contains_reference(from), "We just found " PTR_FORMAT " in the Coarse table", p2i(from));
     return;
   }
@@ -273,7 +227,6 @@ PerRegionTable* OtherRegionsTable::delete_region_table(size_t& added_by_deleted)
   PerRegionTable* max = NULL;
   jint max_occ = 0;
   PerRegionTable** max_prev = NULL;
-  size_t max_ind;
 
   size_t i = _fine_eviction_start;
   for (size_t k = 0; k < _fine_eviction_sample_size; k++) {
@@ -291,7 +244,6 @@ PerRegionTable* OtherRegionsTable::delete_region_table(size_t& added_by_deleted)
       if (max == NULL || cur_occ > max_occ) {
         max = cur;
         max_prev = prev;
-        max_ind = i;
         max_occ = cur_occ;
       }
       prev = cur->collision_list_next_addr();
@@ -310,11 +262,19 @@ PerRegionTable* OtherRegionsTable::delete_region_table(size_t& added_by_deleted)
   guarantee(max != NULL, "Since _n_fine_entries > 0");
   guarantee(max_prev != NULL, "Since max != NULL.");
 
-  // Set the corresponding coarse bit.
+  // Ensure the corresponding coarse bit is set.
   size_t max_hrm_index = (size_t) max->hr()->hrm_index();
-  if (!_coarse_map.at(max_hrm_index)) {
+  if (Atomic::load(&_has_coarse_entries)) {
     _coarse_map.at_put(max_hrm_index, true);
-    _n_coarse_entries++;
+  } else {
+    // This will lazily initialize an uninitialized bitmap
+    _coarse_map.reinitialize(G1CollectedHeap::heap()->max_reserved_regions());
+    assert(!_coarse_map.at(max_hrm_index), "No coarse entries");
+    _coarse_map.at_put(max_hrm_index, true);
+    // Release store guarantees that the bitmap has initialized before any
+    // concurrent reader will ever see _has_coarse_entries is true
+    // (when read with load_acquire)
+    Atomic::release_store(&_has_coarse_entries, true);
   }
 
   added_by_deleted = HeapRegion::CardsPerRegion - max_occ;
@@ -372,11 +332,11 @@ void OtherRegionsTable::clear() {
 
   _first_all_fine_prts = _last_all_fine_prts = NULL;
   _sparse_table.clear();
-  if (_n_coarse_entries > 0) {
+  if (Atomic::load(&_has_coarse_entries)) {
     _coarse_map.clear();
   }
   _n_fine_entries = 0;
-  _n_coarse_entries = 0;
+  Atomic::store(&_has_coarse_entries, false);
 
   _num_occupied = 0;
 }
@@ -391,19 +351,25 @@ bool OtherRegionsTable::contains_reference_locked(OopOrNarrowOopStar from) const
   HeapRegion* hr = _g1h->heap_region_containing(from);
   RegionIdx_t hr_ind = (RegionIdx_t) hr->hrm_index();
   // Is this region in the coarse map?
-  if (_coarse_map.at(hr_ind)) return true;
+  if (is_region_coarsened(hr_ind)) return true;
 
   PerRegionTable* prt = find_region_table(hr_ind & _mod_max_fine_entries_mask,
                                           hr);
   if (prt != NULL) {
     return prt->contains_reference(from);
-
   } else {
     CardIdx_t card_index = card_within_region(from, hr);
     return _sparse_table.contains_card(hr_ind, card_index);
   }
 }
 
+// A load_acquire on _has_coarse_entries - coupled with the release_store in
+// delete_region_table - guarantees we don't access _coarse_map before
+// it's been properly initialized.
+bool OtherRegionsTable::is_region_coarsened(RegionIdx_t from_hrm_ind) const {
+  return Atomic::load_acquire(&_has_coarse_entries) && _coarse_map.at(from_hrm_ind);
+}
+
 HeapRegionRemSet::HeapRegionRemSet(G1BlockOffsetTable* bot,
                                    HeapRegion* hr)
   : _bot(bot),
