@@ -47,9 +47,6 @@ public:
   template <typename T> void do_oop_nv(T* p)      { _compaction_manager->mark_and_push(p); }
   virtual void do_oop(oop* p)                     { do_oop_nv(p); }
   virtual void do_oop(narrowOop* p)               { do_oop_nv(p); }
-
-  // This closure provides its own oop verification code.
-  debug_only(virtual bool should_verify_oops()    { return false; })
 };
 
 class PCIterateMarkAndPushClosure: public MetadataVisitingOopIterateClosure {
@@ -64,9 +61,6 @@ public:
 
   void do_klass_nv(Klass* k)                      { _compaction_manager->follow_klass(k); }
   void do_cld_nv(ClassLoaderData* cld)            { _compaction_manager->follow_class_loader(cld); }
-
-  // This closure provides its own oop verification code.
-  debug_only(virtual bool should_verify_oops()    { return false; })
 };
 
 inline bool ParCompactionManager::steal(int queue_num, oop& t) {
@@ -133,7 +127,7 @@ inline void follow_array_specialized(objArrayOop obj, int index, ParCompactionMa
 
   const size_t stride = MIN2(len - beg_index, (size_t)ObjArrayMarkingStride);
   const size_t end_index = beg_index + stride;
-  T* const base = (T*)obj->base_raw();
+  T* const base = (T*)obj->base();
   T* const beg = base + beg_index;
   T* const end = base + end_index;
 
