@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -56,15 +56,18 @@ class PSPromotionManager {
   friend class PSRefProcTask;
 
  private:
+  typedef OverflowTaskQueue<ScannerTask, mtGC>           PSScannerTasksQueue;
+  typedef GenericTaskQueueSet<PSScannerTasksQueue, mtGC> PSScannerTasksQueueSet;
+
   static PaddedEnd<PSPromotionManager>* _manager_array;
-  static OopStarTaskQueueSet*           _stack_array_depth;
+  static PSScannerTasksQueueSet*        _stack_array_depth;
   static PreservedMarksSet*             _preserved_marks_set;
   static PSOldGen*                      _old_gen;
   static MutableSpace*                  _young_space;
 
 #if TASKQUEUE_STATS
-  size_t                              _masked_pushes;
-  size_t                              _masked_steals;
+  size_t                              _array_chunk_pushes;
+  size_t                              _array_chunk_steals;
   size_t                              _arrays_chunked;
   size_t                              _array_chunks_processed;
 
@@ -79,7 +82,7 @@ class PSPromotionManager {
   bool                                _young_gen_is_full;
   bool                                _old_gen_is_full;
 
-  OopStarTaskQueue                    _claimed_stack_depth;
+  PSScannerTasksQueue                 _claimed_stack_depth;
   OverflowTaskQueue<oop, mtGC>        _claimed_stack_breadth;
 
   bool                                _totally_drain;
@@ -96,62 +99,19 @@ class PSPromotionManager {
   static MutableSpace* young_space() { return _young_space; }
 
   inline static PSPromotionManager* manager_array(uint index);
-  template <class T> inline void claim_or_forward_internal_depth(T* p);
-
-  // On the task queues we push reference locations as well as
-  // partially-scanned arrays (in the latter case, we push an oop to
-  // the from-space image of the array and the length on the
-  // from-space image indicates how many entries on the array we still
-  // need to scan; this is basically how ParNew does partial array
-  // scanning too). To be able to distinguish between reference
-  // locations and partially-scanned array oops we simply mask the
-  // latter oops with 0x01. The next three methods do the masking,
-  // unmasking, and checking whether the oop is masked or not. Notice
-  // that the signature of the mask and unmask methods looks a bit
-  // strange, as they accept and return different types (oop and
-  // oop*). This is because of the difference in types between what
-  // the task queue holds (oop*) and oops to partially-scanned arrays
-  // (oop). We do all the necessary casting in the mask / unmask
-  // methods to avoid sprinkling the rest of the code with more casts.
-
-  // These are added to the taskqueue so PS_CHUNKED_ARRAY_OOP_MASK (or any
-  // future masks) can't conflict with COMPRESSED_OOP_MASK
-#define PS_CHUNKED_ARRAY_OOP_MASK  0x2
-
-  bool is_oop_masked(StarTask p) {
-    // If something is marked chunked it's always treated like wide oop*
-    return (((intptr_t)(oop*)p) & PS_CHUNKED_ARRAY_OOP_MASK) ==
-                                  PS_CHUNKED_ARRAY_OOP_MASK;
-  }
-
-  oop* mask_chunked_array_oop(oop obj) {
-    assert(!is_oop_masked((oop*) obj), "invariant");
-    oop* ret = (oop*) (cast_from_oop<uintptr_t>(obj) | PS_CHUNKED_ARRAY_OOP_MASK);
-    assert(is_oop_masked(ret), "invariant");
-    return ret;
-  }
-
-  oop unmask_chunked_array_oop(StarTask p) {
-    assert(is_oop_masked(p), "invariant");
-    assert(!p.is_narrow(), "chunked array oops cannot be narrow");
-    oop *chunk = (oop*)p;  // cast p to oop (uses conversion operator)
-    oop ret = oop((oop*)((uintptr_t)chunk & ~PS_CHUNKED_ARRAY_OOP_MASK));
-    assert(!is_oop_masked((oop*) ret), "invariant");
-    return ret;
-  }
 
   template <class T> void  process_array_chunk_work(oop obj,
                                                     int start, int end);
-  void process_array_chunk(oop old);
+  void process_array_chunk(PartialArrayScanTask task);
 
-  template <class T> void push_depth(T* p);
+  void push_depth(ScannerTask task);
 
   inline void promotion_trace_event(oop new_obj, oop old_obj, size_t obj_size,
                                     uint age, bool tenured,
                                     const PSPromotionLAB* lab);
 
- protected:
-  static OopStarTaskQueueSet* stack_array_depth()   { return _stack_array_depth; }
+  static PSScannerTasksQueueSet* stack_array_depth() { return _stack_array_depth; }
+
  public:
   // Static
   static void initialize();
@@ -162,12 +122,12 @@ class PSPromotionManager {
   static PSPromotionManager* gc_thread_promotion_manager(uint index);
   static PSPromotionManager* vm_thread_promotion_manager();
 
-  static bool steal_depth(int queue_num, StarTask& t);
+  static bool steal_depth(int queue_num, ScannerTask& t);
 
   PSPromotionManager();
 
   // Accessors
-  OopStarTaskQueue* claimed_stack_depth() {
+  PSScannerTasksQueue* claimed_stack_depth() {
     return &_claimed_stack_depth;
   }
 
@@ -200,17 +160,17 @@ class PSPromotionManager {
     return claimed_stack_depth()->is_empty();
   }
 
-  inline void process_popped_location_depth(StarTask p);
+  inline void process_popped_location_depth(ScannerTask task);
 
   static bool should_scavenge(oop* p, bool check_to_space = false);
   static bool should_scavenge(narrowOop* p, bool check_to_space = false);
 
-  template <class T, bool promote_immediately>
+  template <bool promote_immediately, class T>
   void copy_and_push_safe_barrier(T* p);
 
   template <class T> inline void claim_or_forward_depth(T* p);
 
-  TASKQUEUE_STATS_ONLY(inline void record_steal(StarTask& p);)
+  TASKQUEUE_STATS_ONLY(inline void record_steal(ScannerTask task);)
 
   void push_contents(oop obj);
 };
