@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -31,14 +31,12 @@
 #include "gc/parallel/psVirtualspace.hpp"
 #include "gc/parallel/spaceCounters.hpp"
 
-class PSMarkSweepDecorator;
-
 class PSYoungGen : public CHeapObj<mtGC> {
   friend class VMStructs;
   friend class ParallelScavengeHeap;
   friend class AdjoiningGenerations;
 
- protected:
+ private:
   MemRegion       _reserved;
   PSVirtualSpace* _virtual_space;
 
@@ -47,22 +45,15 @@ class PSYoungGen : public CHeapObj<mtGC> {
   MutableSpace* _from_space;
   MutableSpace* _to_space;
 
-
-  // MarkSweep Decorators
-  PSMarkSweepDecorator* _eden_mark_sweep;
-  PSMarkSweepDecorator* _from_mark_sweep;
-  PSMarkSweepDecorator* _to_mark_sweep;
-
   // Sizing information, in bytes, set in constructor
-  const size_t _init_gen_size;
   const size_t _min_gen_size;
   const size_t _max_gen_size;
 
   // Performance counters
-  PSGenerationCounters*     _gen_counters;
-  SpaceCounters*            _eden_counters;
-  SpaceCounters*            _from_counters;
-  SpaceCounters*            _to_counters;
+  PSGenerationCounters* _gen_counters;
+  SpaceCounters*        _eden_counters;
+  SpaceCounters*        _from_counters;
+  SpaceCounters*        _to_counters;
 
   // Initialize the space boundaries
   void compute_initial_space_boundaries();
@@ -70,44 +61,41 @@ class PSYoungGen : public CHeapObj<mtGC> {
   // Space boundary helper
   void set_space_boundaries(size_t eden_size, size_t survivor_size);
 
-  virtual bool resize_generation(size_t eden_size, size_t survivor_size);
-  virtual void resize_spaces(size_t eden_size, size_t survivor_size);
+  bool resize_generation(size_t eden_size, size_t survivor_size);
+  void resize_spaces(size_t eden_size, size_t survivor_size);
 
   // Adjust the spaces to be consistent with the virtual space.
   void post_resize();
 
-  // Return number of bytes that the generation can change.
-  // These should not be used by PSYoungGen
-  virtual size_t available_for_expansion();
-  virtual size_t available_for_contraction();
-
   // Given a desired shrinkage in the size of the young generation,
   // return the actual size available for shrinkage.
-  virtual size_t limit_gen_shrink(size_t desired_change);
+  size_t limit_gen_shrink(size_t desired_change);
   // returns the number of bytes available from the current size
   // down to the minimum generation size.
   size_t available_to_min_gen();
   // Return the number of bytes available for shrinkage considering
   // the location the live data in the generation.
-  virtual size_t available_to_live();
+  size_t available_to_live();
+
+  void initialize(ReservedSpace rs, size_t inital_size, size_t alignment);
+  void initialize_work();
+  void initialize_virtual_space(ReservedSpace rs, size_t initial_size, size_t alignment);
 
  public:
   // Initialize the generation.
-  PSYoungGen(size_t        initial_byte_size,
-             size_t        minimum_byte_size,
-             size_t        maximum_byte_size);
-  void initialize_work();
-  virtual void initialize(ReservedSpace rs, size_t alignment);
-  virtual void initialize_virtual_space(ReservedSpace rs, size_t alignment);
+  PSYoungGen(ReservedSpace rs,
+             size_t initial_byte_size,
+             size_t minimum_byte_size,
+             size_t maximum_byte_size);
 
-  MemRegion reserved() const            { return _reserved; }
+  MemRegion reserved() const { return _reserved; }
 
-  bool is_in(const void* p) const   {
-      return _virtual_space->contains((void *)p);
+  bool is_in(const void* p) const {
+    return _virtual_space->contains((void *)p);
   }
 
-  bool is_in_reserved(const void* p) const   {
-      return reserved().contains((void *)p);
+  bool is_in_reserved(const void* p) const {
+    return reserved().contains((void *)p);
   }
 
   MutableSpace*   eden_space() const    { return _eden_space; }
@@ -115,20 +103,6 @@ class PSYoungGen : public CHeapObj<mtGC> {
   MutableSpace*   to_space() const      { return _to_space; }
   PSVirtualSpace* virtual_space() const { return _virtual_space; }
 
-  // For Adaptive size policy
-  size_t min_gen_size() { return _min_gen_size; }
-
-  // MarkSweep support
-  PSMarkSweepDecorator* eden_mark_sweep() const    { return _eden_mark_sweep; }
-  PSMarkSweepDecorator* from_mark_sweep() const    { return _from_mark_sweep; }
-  PSMarkSweepDecorator* to_mark_sweep() const      { return _to_mark_sweep;   }
-
-#if INCLUDE_SERIALGC
-  void precompact();
-  void adjust_pointers();
-  void compact();
-#endif
-
   // Called during/after GC
   void swap_spaces();
 
@@ -147,12 +121,8 @@ class PSYoungGen : public CHeapObj<mtGC> {
   size_t used_in_words() const;
   size_t free_in_words() const;
 
-  // The max this generation can grow to
-  size_t max_size() const { return _reserved.byte_size(); }
-
-  // The max this generation can grow to if the boundary between
-  // the generations are allowed to move.
-  size_t gen_size_limit() const { return _max_gen_size; }
+  size_t min_gen_size() const { return _min_gen_size; }
+  size_t max_gen_size() const { return _max_gen_size; }
 
   bool is_maximal_no_gc() const {
     return true;  // Never expands except at a GC
@@ -171,16 +141,15 @@ class PSYoungGen : public CHeapObj<mtGC> {
   void oop_iterate(OopIterateClosure* cl);
   void object_iterate(ObjectClosure* cl);
 
-  virtual void reset_after_change();
-  virtual void reset_survivors_after_shrink();
+  void reset_survivors_after_shrink();
 
   // Performance Counter support
   void update_counters();
 
   // Debugging - do not use for time critical operations
   void print() const;
-  void print_on(outputStream* st) const;
-  virtual const char* name() const { return "PSYoungGen"; }
+  virtual void print_on(outputStream* st) const;
+  const char* name() const { return "PSYoungGen"; }
 
   void verify();
 
