@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -38,46 +38,6 @@
 #include "runtime/os.hpp"
 #include "utilities/macros.hpp"
 
-class HasAccumulatedModifiedOopsClosure : public CLDClosure {
-  bool _found;
- public:
-  HasAccumulatedModifiedOopsClosure() : _found(false) {}
-  void do_cld(ClassLoaderData* cld) {
-    if (_found) {
-      return;
-    }
-
-    if (cld->has_accumulated_modified_oops()) {
-      _found = true;
-    }
-  }
-  bool found() {
-    return _found;
-  }
-};
-
-bool CLDRemSet::mod_union_is_clear() {
-  HasAccumulatedModifiedOopsClosure closure;
-  ClassLoaderDataGraph::cld_do(&closure);
-
-  return !closure.found();
-}
-
-
-class ClearCLDModUnionClosure : public CLDClosure {
- public:
-  void do_cld(ClassLoaderData* cld) {
-    if (cld->has_accumulated_modified_oops()) {
-      cld->clear_accumulated_modified_oops();
-    }
-  }
-};
-
-void CLDRemSet::clear_mod_union() {
-  ClearCLDModUnionClosure closure;
-  ClassLoaderDataGraph::cld_do(&closure);
-}
-
 CardTable::CardValue CardTableRS::find_unused_youngergenP_card_value() {
   for (CardValue v = youngergenP1_card;
        v < cur_youngergen_and_prev_nonclean_card;
@@ -112,13 +72,10 @@ void CardTableRS::prepare_for_younger_refs_iterate(bool parallel) {
   }
 }
 
-void CardTableRS::younger_refs_iterate(Generation* g,
-                                       OopsInGenClosure* blk,
-                                       uint n_threads) {
+void CardTableRS::at_younger_refs_iterate() {
   // The indexing in this array is slightly odd. We want to access
   // the old generation record here, which is at index 2.
   _last_cur_val_in_gen[2] = cur_youngergen_card_val();
-  g->younger_refs_iterate(blk, n_threads);
 }
 
 inline bool ClearNoncleanCardWrapper::clear_card(CardValue* entry) {
@@ -139,7 +96,7 @@ inline bool ClearNoncleanCardWrapper::clear_card_parallel(CardValue* entry) {
     if (CardTableRS::card_is_dirty_wrt_gen_iter(entry_val)
         || _ct->is_prev_youngergen_card_val(entry_val)) {
       CardValue res =
-        Atomic::cmpxchg(CardTableRS::clean_card_val(), entry, entry_val);
+        Atomic::cmpxchg(entry, entry_val, CardTableRS::clean_card_val());
       if (res == entry_val) {
         break;
       } else {
@@ -244,47 +201,14 @@ void ClearNoncleanCardWrapper::do_MemRegion(MemRegion mr) {
   }
 }
 
-// clean (by dirty->clean before) ==> cur_younger_gen
-// dirty                          ==> cur_youngergen_and_prev_nonclean_card
-// precleaned                     ==> cur_youngergen_and_prev_nonclean_card
-// prev-younger-gen               ==> cur_youngergen_and_prev_nonclean_card
-// cur-younger-gen                ==> cur_younger_gen
-// cur_youngergen_and_prev_nonclean_card ==> no change.
-void CardTableRS::write_ref_field_gc_par(void* field, oop new_val) {
-  volatile CardValue* entry = byte_for(field);
-  do {
-    CardValue entry_val = *entry;
-    // We put this first because it's probably the most common case.
-    if (entry_val == clean_card_val()) {
-      // No threat of contention with cleaning threads.
-      *entry = cur_youngergen_card_val();
-      return;
-    } else if (card_is_dirty_wrt_gen_iter(entry_val)
-               || is_prev_youngergen_card_val(entry_val)) {
-      // Mark it as both cur and prev youngergen; card cleaning thread will
-      // eventually remove the previous stuff.
-      CardValue new_val = cur_youngergen_and_prev_nonclean_card;
-      CardValue res = Atomic::cmpxchg(new_val, entry, entry_val);
-      // Did the CAS succeed?
-      if (res == entry_val) return;
-      // Otherwise, retry, to see the new value.
-      continue;
-    } else {
-      assert(entry_val == cur_youngergen_and_prev_nonclean_card
-             || entry_val == cur_youngergen_card_val(),
-             "should be only possibilities.");
-      return;
-    }
-  } while (true);
-}
-
 void CardTableRS::younger_refs_in_space_iterate(Space* sp,
-                                                OopsInGenClosure* cl,
+                                                HeapWord* gen_boundary,
+                                                OopIterateClosure* cl,
                                                 uint n_threads) {
   verify_used_region_at_save_marks(sp);
 
   const MemRegion urasm = sp->used_region_at_save_marks();
-  non_clean_card_iterate_possibly_parallel(sp, urasm, cl, this, n_threads);
+  non_clean_card_iterate_possibly_parallel(sp, gen_boundary, urasm, cl, this, n_threads);
 }
 
 #ifdef ASSERT
@@ -343,7 +267,7 @@ protected:
            "[_begin, _end) = [" PTR_FORMAT "," PTR_FORMAT ")",
            p2i(jp), p2i(_begin), p2i(_end));
     oop obj = RawAccess<>::oop_load(p);
-    guarantee(obj == NULL || (HeapWord*)obj >= _boundary,
+    guarantee(obj == NULL || cast_from_oop<HeapWord*>(obj) >= _boundary,
               "pointer " PTR_FORMAT " at " PTR_FORMAT " on "
               "clean card crosses boundary" PTR_FORMAT,
               p2i(obj), p2i(jp), p2i(_boundary));
@@ -613,11 +537,7 @@ CardTableRS::CardTableRS(MemRegion whole_heap, bool scanned_concurrently) :
   // max_gens is really GenCollectedHeap::heap()->gen_policy()->number_of_generations()
   // (which is always 2, young & old), but GenCollectedHeap has not been initialized yet.
   uint max_gens = 2;
-  _last_cur_val_in_gen = NEW_C_HEAP_ARRAY3(CardValue, max_gens + 1,
-                         mtGC, CURRENT_PC, AllocFailStrategy::RETURN_NULL);
-  if (_last_cur_val_in_gen == NULL) {
-    vm_exit_during_initialization("Could not create last_cur_val_in_gen array.");
-  }
+  _last_cur_val_in_gen = NEW_C_HEAP_ARRAY(CardValue, max_gens + 1, mtGC);
   for (uint i = 0; i < max_gens + 1; i++) {
     _last_cur_val_in_gen[i] = clean_card_val();
   }
@@ -662,8 +582,9 @@ bool CardTableRS::card_may_have_been_dirty(CardValue cv) {
 
 void CardTableRS::non_clean_card_iterate_possibly_parallel(
   Space* sp,
+  HeapWord* gen_boundary,
   MemRegion mr,
-  OopsInGenClosure* cl,
+  OopIterateClosure* cl,
   CardTableRS* ct,
   uint n_threads)
 {
@@ -676,7 +597,7 @@ void CardTableRS::non_clean_card_iterate_possibly_parallel(
       // This is the single-threaded version used by DefNew.
       const bool parallel = false;
 
-      DirtyCardToOopClosure* dcto_cl = sp->new_dcto_cl(cl, precision(), cl->gen_boundary(), parallel);
+      DirtyCardToOopClosure* dcto_cl = sp->new_dcto_cl(cl, precision(), gen_boundary, parallel);
       ClearNoncleanCardWrapper clear_cl(dcto_cl, ct, parallel);
 
       clear_cl.do_MemRegion(mr);
@@ -685,7 +606,7 @@ void CardTableRS::non_clean_card_iterate_possibly_parallel(
 }
 
 void CardTableRS::non_clean_card_iterate_parallel_work(Space* sp, MemRegion mr,
-                                                       OopsInGenClosure* cl, CardTableRS* ct,
+                                                       OopIterateClosure* cl, CardTableRS* ct,
                                                        uint n_threads) {
   fatal("Parallel gc not supported here.");
 }
