@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -64,9 +64,6 @@ void ReferenceProcessor::init_statics() {
   } else {
     _default_soft_ref_policy = new LRUCurrentHeapPolicy();
   }
-  if (_always_clear_soft_ref_policy == NULL || _default_soft_ref_policy == NULL) {
-    vm_exit_during_initialization("Could not allocate reference policy object");
-  }
   guarantee(RefDiscoveryPolicy == ReferenceBasedDiscovery ||
             RefDiscoveryPolicy == ReferentBasedDiscovery,
             "Unrecognized RefDiscoveryPolicy");
@@ -263,9 +260,7 @@ void DiscoveredListIterator::load_ptrs(DEBUG_ONLY(bool allow_null_referent)) {
   assert(_current_discovered_addr && oopDesc::is_oop_or_null(discovered),
          "Expected an oop or NULL for discovered field at " PTR_FORMAT, p2i(discovered));
   _next_discovered = discovered;
-
-  _referent_addr = java_lang_ref_Reference::referent_addr_raw(_current_discovered);
-  _referent = java_lang_ref_Reference::referent(_current_discovered);
+  _referent = java_lang_ref_Reference::unknown_referent_no_keepalive(_current_discovered);
   assert(Universe::heap()->is_in_or_null(_referent),
          "Wrong oop found in java.lang.Reference object");
   assert(allow_null_referent ?
@@ -298,13 +293,22 @@ void DiscoveredListIterator::remove() {
   _refs_list.dec_length(1);
 }
 
+void DiscoveredListIterator::make_referent_alive() {
+  HeapWord* addr = java_lang_ref_Reference::referent_addr_raw(_current_discovered);
+  if (UseCompressedOops) {
+    _keep_alive->do_oop((narrowOop*)addr);
+  } else {
+    _keep_alive->do_oop((oop*)addr);
+  }
+}
+
 void DiscoveredListIterator::clear_referent() {
-  RawAccess<>::oop_store(_referent_addr, oop(NULL));
+  java_lang_ref_Reference::clear_referent(_current_discovered);
 }
 
 void DiscoveredListIterator::enqueue() {
   HeapAccess<AS_NO_KEEPALIVE>::oop_store_at(_current_discovered,
-                                            java_lang_ref_Reference::discovered_offset,
+                                            java_lang_ref_Reference::discovered_offset(),
                                             _next_discovered);
 }
 
@@ -314,7 +318,7 @@ void DiscoveredListIterator::complete_enqueue() {
     // Swap refs_list into pending list and set obj's
     // discovered to what we read from the pending list.
     oop old = Universe::swap_reference_pending_list(_refs_list.head());
-    HeapAccess<AS_NO_KEEPALIVE>::oop_store_at(_prev_discovered, java_lang_ref_Reference::discovered_offset, old);
+    HeapAccess<AS_NO_KEEPALIVE>::oop_store_at(_prev_discovered, java_lang_ref_Reference::discovered_offset(), old);
   }
 }
 
@@ -785,8 +789,13 @@ void ReferenceProcessor::process_soft_ref_reconsider(BoolObjectClosure* is_alive
 
   phase_times->set_processing_is_mt(_processing_is_mt);
 
-  if (num_soft_refs == 0 || _current_soft_ref_policy == NULL) {
-    log_debug(gc, ref)("Skipped phase1 of Reference Processing due to unavailable references");
+  if (num_soft_refs == 0) {
+    log_debug(gc, ref)("Skipped phase 1 of Reference Processing: no references");
+    return;
+  }
+
+  if (_current_soft_ref_policy == NULL) {
+    log_debug(gc, ref)("Skipped phase 1 of Reference Processing: no policy");
     return;
   }
 
@@ -799,7 +808,7 @@ void ReferenceProcessor::process_soft_ref_reconsider(BoolObjectClosure* is_alive
 
   RefProcPhaseTimeTracker tt(RefPhase1, phase_times);
 
-  log_reflist("Phase1 Soft before", _discoveredSoftRefs, _max_num_queues);
+  log_reflist("Phase 1 Soft before", _discoveredSoftRefs, _max_num_queues);
   if (_processing_is_mt) {
     RefProcPhase1Task phase1(*this, phase_times, _current_soft_ref_policy);
     task_executor->execute(phase1, num_queues());
@@ -814,7 +823,7 @@ void ReferenceProcessor::process_soft_ref_reconsider(BoolObjectClosure* is_alive
 
     phase_times->add_ref_cleared(REF_SOFT, removed);
   }
-  log_reflist("Phase1 Soft after", _discoveredSoftRefs, _max_num_queues);
+  log_reflist("Phase 1 Soft after", _discoveredSoftRefs, _max_num_queues);
 }
 
 void ReferenceProcessor::process_soft_weak_final_refs(BoolObjectClosure* is_alive,
@@ -834,7 +843,7 @@ void ReferenceProcessor::process_soft_weak_final_refs(BoolObjectClosure* is_aliv
   phase_times->set_processing_is_mt(_processing_is_mt);
 
   if (num_total_refs == 0) {
-    log_debug(gc, ref)("Skipped phase2 of Reference Processing due to unavailable references");
+    log_debug(gc, ref)("Skipped phase 2 of Reference Processing: no references");
     return;
   }
 
@@ -849,9 +858,9 @@ void ReferenceProcessor::process_soft_weak_final_refs(BoolObjectClosure* is_aliv
 
   RefProcPhaseTimeTracker tt(RefPhase2, phase_times);
 
-  log_reflist("Phase2 Soft before", _discoveredSoftRefs, _max_num_queues);
-  log_reflist("Phase2 Weak before", _discoveredWeakRefs, _max_num_queues);
-  log_reflist("Phase2 Final before", _discoveredFinalRefs, _max_num_queues);
+  log_reflist("Phase 2 Soft before", _discoveredSoftRefs, _max_num_queues);
+  log_reflist("Phase 2 Weak before", _discoveredWeakRefs, _max_num_queues);
+  log_reflist("Phase 2 Final before", _discoveredFinalRefs, _max_num_queues);
   if (_processing_is_mt) {
     RefProcPhase2Task phase2(*this, phase_times);
     task_executor->execute(phase2, num_queues());
@@ -891,7 +900,7 @@ void ReferenceProcessor::process_soft_weak_final_refs(BoolObjectClosure* is_aliv
   }
   verify_total_count_zero(_discoveredSoftRefs, "SoftReference");
   verify_total_count_zero(_discoveredWeakRefs, "WeakReference");
-  log_reflist("Phase2 Final after", _discoveredFinalRefs, _max_num_queues);
+  log_reflist("Phase 2 Final after", _discoveredFinalRefs, _max_num_queues);
 }
 
 void ReferenceProcessor::process_final_keep_alive(OopClosure* keep_alive,
@@ -905,7 +914,7 @@ void ReferenceProcessor::process_final_keep_alive(OopClosure* keep_alive,
   phase_times->set_processing_is_mt(_processing_is_mt);
 
   if (num_final_refs == 0) {
-    log_debug(gc, ref)("Skipped phase3 of Reference Processing due to unavailable references");
+    log_debug(gc, ref)("Skipped phase 3 of Reference Processing: no references");
     return;
   }
 
@@ -945,7 +954,7 @@ void ReferenceProcessor::process_phantom_refs(BoolObjectClosure* is_alive,
   phase_times->set_processing_is_mt(_processing_is_mt);
 
   if (num_phantom_refs == 0) {
-    log_debug(gc, ref)("Skipped phase4 of Reference Processing due to unavailable references");
+    log_debug(gc, ref)("Skipped phase 4 of Reference Processing: no references");
     return;
   }
 
@@ -959,7 +968,7 @@ void ReferenceProcessor::process_phantom_refs(BoolObjectClosure* is_alive,
   // Phase 4: Walk phantom references appropriately.
   RefProcPhaseTimeTracker tt(RefPhase4, phase_times);
 
-  log_reflist("Phase4 Phantom before", _discoveredPhantomRefs, _max_num_queues);
+  log_reflist("Phase 4 Phantom before", _discoveredPhantomRefs, _max_num_queues);
   if (_processing_is_mt) {
     RefProcPhase4Task phase4(*this, phase_times);
     task_executor->execute(phase4, num_queues());
@@ -1056,7 +1065,7 @@ ReferenceProcessor::add_to_discovered_list_mt(DiscoveredList& refs_list,
 // cleared concurrently by mutators during (or after) discovery.
 void ReferenceProcessor::verify_referent(oop obj) {
   bool da = discovery_is_atomic();
-  oop referent = java_lang_ref_Reference::referent(obj);
+  oop referent = java_lang_ref_Reference::unknown_referent_no_keepalive(obj);
   assert(da ? oopDesc::is_oop(referent) : oopDesc::is_oop_or_null(referent),
          "Bad referent " INTPTR_FORMAT " found in Reference "
          INTPTR_FORMAT " during %satomic discovery ",
@@ -1117,7 +1126,8 @@ bool ReferenceProcessor::discover_reference(oop obj, ReferenceType rt) {
   // known to be strongly reachable.
   if (is_alive_non_header() != NULL) {
     verify_referent(obj);
-    if (is_alive_non_header()->do_object_b(java_lang_ref_Reference::referent(obj))) {
+    oop referent = java_lang_ref_Reference::unknown_referent_no_keepalive(obj);
+    if (is_alive_non_header()->do_object_b(referent)) {
       return false;  // referent is reachable
     }
   }
@@ -1167,7 +1177,7 @@ bool ReferenceProcessor::discover_reference(oop obj, ReferenceType rt) {
     // .. we are an atomic collector and referent is in our span
     if (is_subject_to_discovery(obj) ||
         (discovery_is_atomic() &&
-         is_subject_to_discovery(java_lang_ref_Reference::referent(obj)))) {
+         is_subject_to_discovery(java_lang_ref_Reference::unknown_referent_no_keepalive(obj)))) {
     } else {
       return false;
     }
