@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -39,7 +39,7 @@ class ReferenceProcessorPhaseTimes;
 // List of discovered references.
 class DiscoveredList {
 public:
-  DiscoveredList() : _oop_head(NULL), _compressed_head(0), _len(0) { }
+  DiscoveredList() : _oop_head(NULL), _compressed_head(narrowOop::null), _len(0) { }
   inline oop head() const;
   HeapWord* adr_head() {
     return UseCompressedOops ? (HeapWord*)&_compressed_head :
@@ -71,7 +71,6 @@ private:
   HeapWord*          _current_discovered_addr;
   oop                _next_discovered;
 
-  HeapWord*          _referent_addr;
   oop                _referent;
 
   OopClosure*        _keep_alive;
@@ -120,14 +119,8 @@ public:
   // Remove the current reference from the list
   void remove();
 
-  // Make the referent alive.
-  inline void make_referent_alive() {
-    if (UseCompressedOops) {
-      _keep_alive->do_oop((narrowOop*)_referent_addr);
-    } else {
-      _keep_alive->do_oop((oop*)_referent_addr);
-    }
-  }
+  // Apply the keep_alive function to the referent address.
+  void make_referent_alive();
 
   // Do enqueuing work, i.e. notifying the GC about the changed discovered pointers.
   void enqueue();
