@@ -379,7 +379,7 @@ HeapWord* CompactibleSpace::forward(oop q, size_t size,
   } else {
     // if the object isn't moving we can just set the mark to the default
     // mark and handle it specially later on.
-    q->init_mark_raw();
+    q->init_mark();
     assert(q->forwardee() == NULL, "should be forwarded to NULL");
   }
 
@@ -567,27 +567,6 @@ inline HeapWord* ContiguousSpace::par_allocate_impl(size_t size) {
   } while (true);
 }
 
-HeapWord* ContiguousSpace::allocate_aligned(size_t size) {
-  assert(Heap_lock->owned_by_self() || (SafepointSynchronize::is_at_safepoint() && Thread::current()->is_VM_thread()), "not locked");
-  HeapWord* end_value = end();
-
-  HeapWord* obj = CollectedHeap::align_allocation_or_fail(top(), end_value, SurvivorAlignmentInBytes);
-  if (obj == NULL) {
-    return NULL;
-  }
-
-  if (pointer_delta(end_value, obj) >= size) {
-    HeapWord* new_top = obj + size;
-    set_top(new_top);
-    assert(::is_aligned(obj, SurvivorAlignmentInBytes) && is_aligned(new_top),
-      "checking alignment");
-    return obj;
-  } else {
-    set_top(obj);
-    return NULL;
-  }
-}
-
 // Requires locking.
 HeapWord* ContiguousSpace::allocate(size_t size) {
   return allocate_impl(size);
@@ -617,14 +596,14 @@ void ContiguousSpace::allocate_temporary_filler(int factor) {
     // allocate uninitialized int array
     typeArrayOop t = (typeArrayOop) allocate(size);
     assert(t != NULL, "allocation should succeed");
-    t->set_mark_raw(markWord::prototype());
+    t->set_mark(markWord::prototype());
     t->set_klass(Universe::intArrayKlassObj());
     t->set_length((int)length);
   } else {
     assert(size == CollectedHeap::min_fill_size(),
            "size for smallest fake object doesn't match");
     instanceOop obj = (instanceOop) allocate(size);
-    obj->set_mark_raw(markWord::prototype());
+    obj->set_mark(markWord::prototype());
     obj->set_klass_gap(0);
     obj->set_klass(SystemDictionary::Object_klass());
   }
