@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -54,52 +54,37 @@ inline void GenericTaskQueue<E, F, N>::initialize() {
 
 template<class E, MEMFLAGS F, unsigned int N>
 inline GenericTaskQueue<E, F, N>::~GenericTaskQueue() {
-  ArrayAllocator<E>::free(const_cast<E*>(_elems), N);
-}
-
-template<class E, MEMFLAGS F, unsigned int N>
-bool GenericTaskQueue<E, F, N>::push_slow(E t, uint dirty_n_elems) {
-  if (dirty_n_elems == N - 1) {
-    // Actually means 0, so do the push.
-    uint localBot = _bottom;
-    // g++ complains if the volatile result of the assignment is
-    // unused, so we cast the volatile away.  We cannot cast directly
-    // to void, because gcc treats that as not using the result of the
-    // assignment.  However, casting to E& means that we trigger an
-    // unused-value warning.  So, we cast the E& to void.
-    (void)const_cast<E&>(_elems[localBot] = t);
-    OrderAccess::release_store(&_bottom, increment_index(localBot));
-    TASKQUEUE_STATS_ONLY(stats.record_push());
-    return true;
-  }
-  return false;
+  ArrayAllocator<E>::free(_elems, N);
 }
 
 template<class E, MEMFLAGS F, unsigned int N> inline bool
 GenericTaskQueue<E, F, N>::push(E t) {
-  uint localBot = _bottom;
+  uint localBot = bottom_relaxed();
   assert(localBot < N, "_bottom out of range.");
-  idx_t top = _age.top();
+  idx_t top = age_top_relaxed();
   uint dirty_n_elems = dirty_size(localBot, top);
-  assert(dirty_n_elems < N, "n_elems out of range.");
+  // A dirty_size of N-1 cannot happen in push.  Considering only push:
+  // (1) dirty_n_elems is initially 0.
+  // (2) push adds an element iff dirty_n_elems < max_elems(), which is N - 2.
+  // (3) only push adding an element can increase dirty_n_elems.
+  // => dirty_n_elems <= N - 2, by induction
+  // => dirty_n_elems < N - 1, invariant
+  //
+  // A pop_global that is concurrent with push cannot produce a state where
+  // dirty_size == N-1.  pop_global only removes an element if dirty_elems > 0,
+  // so can't underflow to -1 (== N-1) with push.
+  assert(dirty_n_elems <= max_elems(), "n_elems out of range.");
   if (dirty_n_elems < max_elems()) {
-    // g++ complains if the volatile result of the assignment is
-    // unused, so we cast the volatile away.  We cannot cast directly
-    // to void, because gcc treats that as not using the result of the
-    // assignment.  However, casting to E& means that we trigger an
-    // unused-value warning.  So, we cast the E& to void.
-    (void) const_cast<E&>(_elems[localBot] = t);
-    OrderAccess::release_store(&_bottom, increment_index(localBot));
+    _elems[localBot] = t;
+    release_set_bottom(increment_index(localBot));
     TASKQUEUE_STATS_ONLY(stats.record_push());
     return true;
-  } else {
-    return push_slow(t, dirty_n_elems);
   }
+  return false;                 // Queue is full.
 }
 
 template <class E, MEMFLAGS F, unsigned int N>
-inline bool OverflowTaskQueue<E, F, N>::push(E t)
-{
+inline bool OverflowTaskQueue<E, F, N>::push(E t) {
   if (!taskqueue_t::push(t)) {
     overflow_stack()->push(t);
     TASKQUEUE_STATS_ONLY(stats.record_overflow(overflow_stack()->size()));
@@ -123,73 +108,68 @@ bool GenericTaskQueue<E, F, N>::pop_local_slow(uint localBot, Age oldAge) {
   // This queue was observed to contain exactly one element; either this
   // thread will claim it, or a competing "pop_global".  In either case,
   // the queue will be logically empty afterwards.  Create a new Age value
-  // that represents the empty queue for the given value of "_bottom".  (We
+  // that represents the empty queue for the given value of "bottom".  (We
   // must also increment "tag" because of the case where "bottom == 1",
   // "top == 0".  A pop_global could read the queue element in that case,
   // then have the owner thread do a pop followed by another push.  Without
   // the incrementing of "tag", the pop_global's CAS could succeed,
   // allowing it to believe it has claimed the stale element.)
-  Age newAge((idx_t)localBot, oldAge.tag() + 1);
+  Age newAge((idx_t)localBot, (idx_t)(oldAge.tag() + 1));
   // Perhaps a competing pop_global has already incremented "top", in which
   // case it wins the element.
   if (localBot == oldAge.top()) {
     // No competing pop_global has yet incremented "top"; we'll try to
     // install new_age, thus claiming the element.
-    Age tempAge = _age.cmpxchg(newAge, oldAge);
+    Age tempAge = cmpxchg_age(oldAge, newAge);
     if (tempAge == oldAge) {
       // We win.
-      assert(dirty_size(localBot, _age.top()) != N - 1, "sanity");
+      assert_not_underflow(localBot, age_top_relaxed());
       TASKQUEUE_STATS_ONLY(stats.record_pop_slow());
       return true;
     }
   }
-  // We lose; a completing pop_global gets the element.  But the queue is empty
+  // We lose; a competing pop_global got the element.  But the queue is empty
   // and top is greater than bottom.  Fix this representation of the empty queue
   // to become the canonical one.
-  _age.set(newAge);
-  assert(dirty_size(localBot, _age.top()) != N - 1, "sanity");
+  set_age_relaxed(newAge);
+  assert_not_underflow(localBot, age_top_relaxed());
   return false;
 }
 
 template<class E, MEMFLAGS F, unsigned int N> inline bool
-GenericTaskQueue<E, F, N>::pop_local(volatile E& t, uint threshold) {
-  uint localBot = _bottom;
+GenericTaskQueue<E, F, N>::pop_local(E& t, uint threshold) {
+  uint localBot = bottom_relaxed();
   // This value cannot be N-1.  That can only occur as a result of
   // the assignment to bottom in this method.  If it does, this method
   // resets the size to 0 before the next call (which is sequential,
   // since this is pop_local.)
-  uint dirty_n_elems = dirty_size(localBot, _age.top());
-  assert(dirty_n_elems != N - 1, "Shouldn't be possible...");
+  uint dirty_n_elems = dirty_size(localBot, age_top_relaxed());
+  assert_not_underflow(dirty_n_elems);
   if (dirty_n_elems <= threshold) return false;
   localBot = decrement_index(localBot);
-  _bottom = localBot;
+  set_bottom_relaxed(localBot);
   // This is necessary to prevent any read below from being reordered
   // before the store just above.
   OrderAccess::fence();
-  // g++ complains if the volatile result of the assignment is
-  // unused, so we cast the volatile away.  We cannot cast directly
-  // to void, because gcc treats that as not using the result of the
-  // assignment.  However, casting to E& means that we trigger an
-  // unused-value warning.  So, we cast the E& to void.
-  (void) const_cast<E&>(t = _elems[localBot]);
+  t = _elems[localBot];
   // This is a second read of "age"; the "size()" above is the first.
   // If there's still at least one element in the queue, based on the
   // "_bottom" and "age" we've read, then there can be no interference with
   // a "pop_global" operation, and we're done.
-  idx_t tp = _age.top();    // XXX
-  if (size(localBot, tp) > 0) {
-    assert(dirty_size(localBot, tp) != N - 1, "sanity");
+  idx_t tp = age_top_relaxed();
+  if (clean_size(localBot, tp) > 0) {
+    assert_not_underflow(localBot, tp);
     TASKQUEUE_STATS_ONLY(stats.record_pop());
     return true;
   } else {
     // Otherwise, the queue contained exactly one element; we take the slow
     // path.
 
-    // The barrier is required to prevent reordering the two reads of _age:
-    // one is the _age.get() below, and the other is _age.top() above the if-stmt.
-    // The algorithm may fail if _age.get() reads an older value than _age.top().
+    // The barrier is required to prevent reordering the two reads of age:
+    // one is the age() below, and the other is age_top() above the if-stmt.
+    // The algorithm may fail if age() reads an older value than age_top().
     OrderAccess::loadload();
-    return pop_local_slow(localBot, _age.get());
+    return pop_local_slow(localBot, age_relaxed());
   }
 }
 
@@ -201,34 +181,63 @@ bool OverflowTaskQueue<E, F, N>::pop_overflow(E& t)
   return true;
 }
 
+// A pop_global operation may read an element that is being concurrently
+// written by a push operation.  The pop_global operation will not use
+// such an element, returning failure instead.  But the concurrent read
+// and write places requirements on the element type.
+//
+// Strictly, such concurrent reads and writes are undefined behavior.
+// We ignore that. Instead we require that whatever value tearing may
+// occur as a result is benign. A trivially copyable type (C++14 3.9/9)
+// satisfies the requirement. But we might use classes such as oop that
+// are not trivially copyable (in some build configurations).  Such
+// classes need to be carefully examined with this requirement in mind.
+//
+// The sequence where such a read/write collision can arise is as follows.
+// Assume there is one value in the queue, so bottom == top+1.
+// (1) Thief is doing a pop_global.  It has read age and bottom, and its
+// captured (localBottom - oldAge.top) == 1.
+// (2) Owner does a pop_local and wins the race for that element.  It
+// decrements bottom and increments the age tag.
+// (3) Owner starts a push, writing elems[bottom].  At the same time, Thief
+// reads elems[oldAge.top].  The owner's bottom == the thief's oldAge.top.
+// (4) Thief will discard the read value, because its cmpxchg of age will fail.
 template<class E, MEMFLAGS F, unsigned int N>
-bool GenericTaskQueue<E, F, N>::pop_global(volatile E& t) {
-  Age oldAge = _age.get();
-  // Architectures with weak memory model require a barrier here
-  // to guarantee that bottom is not older than age,
+bool GenericTaskQueue<E, F, N>::pop_global(E& t) {
+  Age oldAge = age_relaxed();
+
+  // Architectures with non-multi-copy-atomic memory model require a
+  // full fence here to guarantee that bottom is not older than age,
   // which is crucial for the correctness of the algorithm.
-#ifndef CPU_MULTI_COPY_ATOMIC
-  OrderAccess::fence();
-#endif
-  uint localBot = OrderAccess::load_acquire(&_bottom);
-  uint n_elems = size(localBot, oldAge.top());
+  //
+  // We need a full fence here for this case:
+  //
+  // Thread1: set bottom (push)
+  // Thread2: read age, read bottom, set age (pop_global)
+  // Thread3: read age, read bottom (pop_global)
+  //
+  // The requirement is that Thread3 must never read an older bottom
+  // value than Thread2 after Thread3 has seen the age value from
+  // Thread2.
+  OrderAccess::loadload_for_IRIW();
+
+  uint localBot = bottom_acquire();
+  uint n_elems = clean_size(localBot, oldAge.top());
   if (n_elems == 0) {
     return false;
   }
 
-  // g++ complains if the volatile result of the assignment is
-  // unused, so we cast the volatile away.  We cannot cast directly
-  // to void, because gcc treats that as not using the result of the
-  // assignment.  However, casting to E& means that we trigger an
-  // unused-value warning.  So, we cast the E& to void.
-  (void) const_cast<E&>(t = _elems[oldAge.top()]);
-  Age newAge(oldAge);
-  newAge.increment();
-  Age resAge = _age.cmpxchg(newAge, oldAge);
+  t = _elems[oldAge.top()];
+  // Increment top; if it wraps, also increment tag, to distinguish it
+  // from any recent _age for the same top() index.
+  idx_t new_top = increment_index(oldAge.top());
+  idx_t new_tag = oldAge.tag() + ((new_top == 0) ? 1 : 0);
+  Age newAge(new_top, new_tag);
+  Age resAge = cmpxchg_age(oldAge, newAge);
 
-  // Note that using "_bottom" here might fail, since a pop_local might
+  // Note that using "bottom" here might fail, since a pop_local might
   // have decremented it.
-  assert(dirty_size(localBot, newAge.top()) != N - 1, "sanity");
+  assert_not_underflow(localBot, newAge.top());
   return resAge == oldAge;
 }
 
@@ -319,19 +328,14 @@ GenericTaskQueueSet<T, F>::steal(uint queue_num, E& t) {
   return false;
 }
 
-template <unsigned int N, MEMFLAGS F>
-inline typename TaskQueueSuper<N, F>::Age TaskQueueSuper<N, F>::Age::cmpxchg(const Age new_age, const Age old_age) volatile {
-  return Atomic::cmpxchg(new_age._data, &_data, old_age._data);
-}
-
 template<class E, MEMFLAGS F, unsigned int N>
 template<class Fn>
 inline void GenericTaskQueue<E, F, N>::iterate(Fn fn) {
   uint iters = size();
-  uint index = _bottom;
+  uint index = bottom_relaxed();
   for (uint i = 0; i < iters; ++i) {
     index = decrement_index(index);
-    fn(const_cast<E&>(_elems[index])); // cast away volatility
+    fn(_elems[index]);
   }
 }
 
