@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,6 +26,7 @@
 #define SHARE_GC_SHARED_WEAKPROCESSOR_HPP
 
 #include "gc/shared/oopStorageParState.hpp"
+#include "gc/shared/oopStorageSetParState.hpp"
 #include "gc/shared/workgroup.hpp"
 #include "memory/allocation.hpp"
 
@@ -46,8 +47,9 @@ public:
   // Visit all oop*s and apply the given closure.
   static void oops_do(OopClosure* closure);
 
-  // Parallel version.  Uses ergo_workers(), active workers, and
-  // phase_time's max_threads to determine the number of threads to use.
+  // Parallel version.  Uses ergo_workers() to determine the number of
+  // threads to use, limited by the total workers and phase_times'
+  // max_threads.
   // IsAlive must be derived from BoolObjectClosure.
   // KeepAlive must be derived from OopClosure.
   template<typename IsAlive, typename KeepAlive>
@@ -56,8 +58,9 @@ public:
                            KeepAlive* keep_alive,
                            WeakProcessorPhaseTimes* phase_times);
 
-  // Convenience parallel version.  Uses ergo_workers() and active workers
-  // to determine the number of threads to run.  Implicitly logs phase times.
+  // Convenience parallel version.  Uses ergo_workers() to determine the
+  // number of threads to use, limited by the total workers.  Implicitly
+  // logs phase times.
   // IsAlive must be derived from BoolObjectClosure.
   // KeepAlive must be derived from OopClosure.
   template<typename IsAlive, typename KeepAlive>
@@ -66,10 +69,16 @@ public:
                            KeepAlive* keep_alive,
                            uint indent_log);
 
+  // Uses the total number of weak references and ReferencesPerThread to
+  // determine the number of threads to use, limited by max_workers.
   static uint ergo_workers(uint max_workers);
+
   class Task;
 
 private:
+  template<typename IsAlive, typename KeepAlive>
+  class CountingClosure;
+
   class GangTask;
 };
 
@@ -78,18 +87,18 @@ class WeakProcessor::Task {
 
   WeakProcessorPhaseTimes* _phase_times;
   uint _nworkers;
-  SubTasksDone _serial_phases_done;
-  StorageState* _storage_states;
+  OopStorageSetWeakParState<false, false> _storage_states;
 
   void initialize();
 
 public:
   Task(uint nworkers);          // No time tracking.
   Task(WeakProcessorPhaseTimes* phase_times, uint nworkers);
-  ~Task();
 
   template<typename IsAlive, typename KeepAlive>
   void work(uint worker_id, IsAlive* is_alive, KeepAlive* keep_alive);
+
+  void report_num_dead();
 };
 
 #endif // SHARE_GC_SHARED_WEAKPROCESSOR_HPP
