@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,7 +28,6 @@
 #include "classfile/stringTable.hpp"
 #include "gc/shared/oopStorage.inline.hpp"
 #include "gc/shared/oopStorageParState.inline.hpp"
-#include "gc/shared/oopStorageSet.hpp"
 #include "gc/shared/weakProcessor.hpp"
 #include "gc/shared/weakProcessorPhases.hpp"
 #include "gc/shared/weakProcessorPhaseTimes.hpp"
@@ -39,52 +38,39 @@
 class BoolObjectClosure;
 class OopClosure;
 
-template<typename IsAlive>
-class CountingIsAliveClosure : public BoolObjectClosure {
-  IsAlive* _inner;
-
-  size_t _num_dead;
-  size_t _num_total;
-
-public:
-  CountingIsAliveClosure(IsAlive* cl) : _inner(cl), _num_dead(0), _num_total(0) { }
-
-  virtual bool do_object_b(oop obj) {
-    bool result = _inner->do_object_b(obj);
-    _num_dead += !result;
-    _num_total++;
-    return result;
-  }
-
-  size_t num_dead() const { return _num_dead; }
-  size_t num_total() const { return _num_total; }
-};
-
 template <typename IsAlive, typename KeepAlive>
-class CountingSkippedIsAliveClosure : public Closure {
-  CountingIsAliveClosure<IsAlive> _counting_is_alive;
+class WeakProcessor::CountingClosure : public Closure {
+  IsAlive* _is_alive;
   KeepAlive* _keep_alive;
-
-  size_t _num_skipped;
+  size_t _old_dead;
+  size_t _new_dead;
+  size_t _live;
 
 public:
-  CountingSkippedIsAliveClosure(IsAlive* is_alive, KeepAlive* keep_alive) :
-    _counting_is_alive(is_alive), _keep_alive(keep_alive), _num_skipped(0) { }
+  CountingClosure(IsAlive* is_alive, KeepAlive* keep_alive) :
+    _is_alive(is_alive),
+    _keep_alive(keep_alive),
+    _old_dead(0),
+    _new_dead(0),
+    _live(0)
+  {}
 
   void do_oop(oop* p) {
     oop obj = *p;
     if (obj == NULL) {
-      _num_skipped++;
-    } else if (_counting_is_alive.do_object_b(obj)) {
+      ++_old_dead;
+    } else if (_is_alive->do_object_b(obj)) {
       _keep_alive->do_oop(p);
+      ++_live;
     } else {
       *p = NULL;
+      ++_new_dead;
     }
   }
 
-  size_t num_dead() const { return _counting_is_alive.num_dead(); }
-  size_t num_skipped() const { return _num_skipped; }
-  size_t num_total() const { return _counting_is_alive.num_total() + num_skipped(); }
+  size_t dead() const { return _old_dead + _new_dead; }
+  size_t new_dead() const { return _new_dead; }
+  size_t total() const { return dead() + _live; }
 };
 
 template<typename IsAlive, typename KeepAlive>
@@ -97,38 +83,17 @@ void WeakProcessor::Task::work(uint worker_id,
 
   typedef WeakProcessorPhases::Iterator Iterator;
 
-  for (Iterator it = WeakProcessorPhases::serial_iterator(); !it.is_end(); ++it) {
-    WeakProcessorPhase phase = *it;
-    CountingIsAliveClosure<IsAlive> cl(is_alive);
-    uint serial_index = WeakProcessorPhases::serial_index(phase);
-    if (_serial_phases_done.try_claim_task(serial_index)) {
-      WeakProcessorPhaseTimeTracker pt(_phase_times, phase);
-      WeakProcessorPhases::processor(phase)(&cl, keep_alive);
-      if (_phase_times != NULL) {
-        _phase_times->record_phase_items(phase, cl.num_dead(), cl.num_total());
-      }
-    }
-  }
-
   for (Iterator it = WeakProcessorPhases::oopstorage_iterator(); !it.is_end(); ++it) {
     WeakProcessorPhase phase = *it;
-    CountingSkippedIsAliveClosure<IsAlive, KeepAlive> cl(is_alive, keep_alive);
+    CountingClosure<IsAlive, KeepAlive> cl(is_alive, keep_alive);
     WeakProcessorPhaseTimeTracker pt(_phase_times, phase, worker_id);
-    uint oopstorage_index = WeakProcessorPhases::oopstorage_index(phase);
-    StorageState& cur_state = _storage_states[oopstorage_index];
-    cur_state.oops_do(&cl);
+    StorageState* cur_state = _storage_states.par_state(phase);
+    cur_state->oops_do(&cl);
+    cur_state->increment_num_dead(cl.dead());
     if (_phase_times != NULL) {
-      _phase_times->record_worker_items(worker_id, phase, cl.num_dead(), cl.num_total());
-    }
-    const OopStorage* cur_storage = cur_state.storage();
-    if (cur_storage == OopStorageSet::string_table_weak()) {
-      StringTable::inc_dead_counter(cl.num_dead() + cl.num_skipped());
-    } else if (cur_storage == OopStorageSet::resolved_method_table_weak()) {
-      ResolvedMethodTable::inc_dead_counter(cl.num_dead() + cl.num_skipped());
+      _phase_times->record_worker_items(worker_id, phase, cl.new_dead(), cl.total());
     }
   }
-
-  _serial_phases_done.all_tasks_completed(_nworkers);
 }
 
 class WeakProcessor::GangTask : public AbstractGangTask {
@@ -159,6 +124,7 @@ public:
   {}
 
   virtual void work(uint worker_id);
+  void report_num_dead() { _task.report_num_dead(); }
 };
 
 template<typename IsAlive, typename KeepAlive>
@@ -168,11 +134,12 @@ void WeakProcessor::weak_oops_do(WorkGang* workers,
                                  WeakProcessorPhaseTimes* phase_times) {
   WeakProcessorTimeTracker tt(phase_times);
 
-  uint nworkers = ergo_workers(MIN2(workers->active_workers(),
+  uint nworkers = ergo_workers(MIN2(workers->total_workers(),
                                     phase_times->max_threads()));
 
   GangTask task("Weak Processor", is_alive, keep_alive, phase_times, nworkers);
   workers->run_task(&task, nworkers);
+  task.report_num_dead();
 }
 
 template<typename IsAlive, typename KeepAlive>
@@ -180,7 +147,7 @@ void WeakProcessor::weak_oops_do(WorkGang* workers,
                                  IsAlive* is_alive,
                                  KeepAlive* keep_alive,
                                  uint indent_log) {
-  uint nworkers = ergo_workers(workers->active_workers());
+  uint nworkers = ergo_workers(workers->total_workers());
   WeakProcessorPhaseTimes pt(nworkers);
   weak_oops_do(workers, is_alive, keep_alive, &pt);
   pt.log_print_phases(indent_log);
