@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,7 +32,7 @@
 
 template<typename T> class WorkerDataArray;
 
-class WeakProcessorPhaseTimes : public CHeapObj<mtGC> {
+class WeakProcessorPhaseTimes {
   enum {
     DeadItems,
     TotalItems
@@ -43,24 +43,15 @@ class WeakProcessorPhaseTimes : public CHeapObj<mtGC> {
   // Total time for weak processor.
   double _total_time_sec;
 
-  // Total time for each serially processed phase.  Entries for phases
-  // processed by multiple threads are unused, as are entries for
-  // unexecuted phases.
-  double _phase_times_sec[WeakProcessorPhases::phase_count];
-  size_t _phase_dead_items[WeakProcessorPhases::phase_count];
-  size_t _phase_total_items[WeakProcessorPhases::phase_count];
-
-  // Per-worker times and linked items, if multiple threads used and the phase was executed.
-  WorkerDataArray<double>* _worker_data[WeakProcessorPhases::oop_storage_phase_count];
-  WorkerDataArray<size_t>* _worker_dead_items[WeakProcessorPhases::oop_storage_phase_count];
-  WorkerDataArray<size_t>* _worker_total_items[WeakProcessorPhases::oop_storage_phase_count];
+  // Per-worker times and linked items.
+  static const uint worker_data_count = WeakProcessorPhases::oopstorage_phase_count;
+  WorkerDataArray<double>* _worker_data[worker_data_count];
 
   WorkerDataArray<double>* worker_data(WeakProcessorPhase phase) const;
 
-  void log_st_phase(WeakProcessorPhase phase, uint indent) const;
-  void log_mt_phase_summary(WeakProcessorPhase phase, uint indent) const;
+  void log_phase_summary(WeakProcessorPhase phase, uint indent) const;
   template <typename T>
-  void log_mt_phase_details(WorkerDataArray<T>* data, uint indent) const;
+  void log_phase_details(WorkerDataArray<T>* data, uint indent) const;
 
 public:
   WeakProcessorPhaseTimes(uint max_threads);
@@ -71,12 +62,9 @@ public:
   void set_active_workers(uint n);
 
   double total_time_sec() const;
-  double phase_time_sec(WeakProcessorPhase phase) const;
   double worker_time_sec(uint worker_id, WeakProcessorPhase phase) const;
 
   void record_total_time_sec(double time_sec);
-  void record_phase_time_sec(WeakProcessorPhase phase, double time_sec);
-  void record_phase_items(WeakProcessorPhase phase, size_t num_dead, size_t num_total);
   void record_worker_time_sec(uint worker_id, WeakProcessorPhase phase, double time_sec);
   void record_worker_items(uint worker_id, WeakProcessorPhase phase, size_t num_dead, size_t num_total);
 
@@ -107,14 +95,8 @@ private:
   Ticks _start_time;
 
 public:
-  // For tracking serial phase times.
-  // Precondition: WeakProcessorPhases::is_serial(phase)
-  WeakProcessorPhaseTimeTracker(WeakProcessorPhaseTimes* times,
-                                WeakProcessorPhase phase);
-
   // For tracking possibly parallel phase times (even if processed by
   // only one thread).
-  // Precondition: WeakProcessorPhases::is_oop_storage(phase)
   // Precondition: worker_id < times->max_threads().
   WeakProcessorPhaseTimeTracker(WeakProcessorPhaseTimes* times,
                                 WeakProcessorPhase phase,
