@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -50,7 +50,9 @@
 
 class AbstractGangWorker;
 class Semaphore;
+class ThreadClosure;
 class WorkGang;
+class GangTaskDispatcher;
 
 // An abstract task to be worked on by a gang.
 // You subclass this to supply your own work() method
@@ -79,28 +81,6 @@ struct WorkData {
   WorkData(AbstractGangTask* task, uint worker_id) : _task(task), _worker_id(worker_id) {}
 };
 
-// Interface to handle the synchronization between the coordinator thread and the worker threads,
-// when a task is dispatched out to the worker threads.
-class GangTaskDispatcher : public CHeapObj<mtGC> {
- public:
-  virtual ~GangTaskDispatcher() {}
-
-  // Coordinator API.
-
-  // Distributes the task out to num_workers workers.
-  // Returns when the task has been completed by all workers.
-  virtual void coordinator_execute_on_workers(AbstractGangTask* task, uint num_workers) = 0;
-
-  // Worker API.
-
-  // Waits for a task to become available to the worker.
-  // Returns when the worker has been assigned a task.
-  virtual WorkData worker_wait_for_task() = 0;
-
-  // Signal to the coordinator that the worker is done with the assigned task.
-  virtual void     worker_done_with_task() = 0;
-};
-
 // The work gang is the collection of workers to execute tasks.
 // The number of workers run for a task is "_active_workers"
 // while "_total_workers" is the number of available of workers.
@@ -187,12 +167,6 @@ class AbstractWorkGang : public CHeapObj<mtInternal> {
   // Debugging.
   const char* name() const { return _name; }
 
-  // Printing
-  void print_worker_threads_on(outputStream *st) const;
-  void print_worker_threads() const {
-    print_worker_threads_on(tty);
-  }
-
  protected:
   virtual AbstractGangWorker* allocate_worker(uint which) = 0;
 };
@@ -220,13 +194,35 @@ public:
   // Run a task with the given number of workers, returns
   // when the task is done. The number of workers must be at most the number of
   // active workers.  Additional workers may be created if an insufficient
-  // number currently exists.
-  void run_task(AbstractGangTask* task, uint num_workers);
+  // number currently exists. If the add_foreground_work flag is true, the current thread
+  // is used to run the task too.
+  void run_task(AbstractGangTask* task, uint num_workers, bool add_foreground_work = false);
 
 protected:
   virtual AbstractGangWorker* allocate_worker(uint which);
 };
 
+// Temporarily try to set the number of active workers.
+// It's not guaranteed that it succeeds, and users need to
+// query the number of active workers.
+class WithUpdatedActiveWorkers : public StackObj {
+private:
+  AbstractWorkGang* const _gang;
+  const uint              _old_active_workers;
+
+public:
+  WithUpdatedActiveWorkers(AbstractWorkGang* gang, uint requested_num_workers) :
+      _gang(gang),
+      _old_active_workers(gang->active_workers()) {
+    uint capped_num_workers = MIN2(requested_num_workers, gang->total_workers());
+    gang->update_active_workers(capped_num_workers);
+  }
+
+  ~WithUpdatedActiveWorkers() {
+    _gang->update_active_workers(_old_active_workers);
+  }
+};
+
 // Several instances of this class run in parallel as workers for a gang.
 class AbstractGangWorker: public WorkerThread {
 public:
@@ -323,6 +319,8 @@ class SubTasksDone: public CHeapObj<mtInternal> {
   // Set all tasks to unclaimed.
   void clear();
 
+  NONCOPYABLE(SubTasksDone);
+
 public:
   // Initializes "this" to a state in which there are "n" tasks to be
   // processed, none of the which are originally claimed.  The number of
