@@ -67,7 +67,9 @@ void ShenandoahAsserts::print_obj(ShenandoahMessageBuffer& msg, oop obj) {
 
   msg.append("  " PTR_FORMAT " - klass " PTR_FORMAT " %s\n", p2i(obj), p2i(obj->klass()), obj->klass()->external_name());
   msg.append("    %3s allocated after mark start\n", ctx->allocated_after_mark_start(obj) ? "" : "not");
-  msg.append("    %3s marked \n",                    ctx->is_marked(obj) ? "" : "not");
+  msg.append("    %3s after update watermark\n",     cast_from_oop<HeapWord*>(obj) >= r->get_update_watermark() ? "" : "not");
+  msg.append("    %3s marked strong\n",              ctx->is_marked_strong(obj) ? "" : "not");
+  msg.append("    %3s marked weak\n",                ctx->is_marked_weak(obj) ? "" : "not");
   msg.append("    %3s in collection set\n",          heap->in_collection_set(obj) ? "" : "not");
   msg.append("  mark:%s\n", mw_ss.as_string());
   msg.append("  region: %s", ss.as_string());
@@ -174,6 +176,16 @@ void ShenandoahAsserts::assert_in_heap(void* interior_loc, oop obj, const char *
   }
 }
 
+void ShenandoahAsserts::assert_in_heap_or_null(void* interior_loc, oop obj, const char *file, int line) {
+  ShenandoahHeap* heap = ShenandoahHeap::heap();
+
+  if (obj != NULL && !heap->is_in(obj)) {
+    print_failure(_safe_unknown, obj, interior_loc, NULL, "Shenandoah assert_in_heap_or_null failed",
+                  "oop must point to a heap address",
+                  file, line);
+  }
+}
+
 void ShenandoahAsserts::assert_correct(void* interior_loc, oop obj, const char* file, int line) {
   ShenandoahHeap* heap = ShenandoahHeap::heap();
 
@@ -342,24 +354,6 @@ void ShenandoahAsserts::print_rp_failure(const char *label, BoolObjectClosure* a
   report_vm_error(file, line, msg.buffer());
 }
 
-void ShenandoahAsserts::assert_rp_isalive_not_installed(const char *file, int line) {
-  ShenandoahHeap* heap = ShenandoahHeap::heap();
-  ReferenceProcessor* rp = heap->ref_processor();
-  if (rp->is_alive_non_header() != NULL) {
-    print_rp_failure("Shenandoah assert_rp_isalive_not_installed failed", rp->is_alive_non_header(),
-                     file, line);
-  }
-}
-
-void ShenandoahAsserts::assert_rp_isalive_installed(const char *file, int line) {
-  ShenandoahHeap* heap = ShenandoahHeap::heap();
-  ReferenceProcessor* rp = heap->ref_processor();
-  if (rp->is_alive_non_header() == NULL) {
-    print_rp_failure("Shenandoah assert_rp_isalive_installed failed", rp->is_alive_non_header(),
-                     file, line);
-  }
-}
-
 void ShenandoahAsserts::assert_locked_or_shenandoah_safepoint(Mutex* lock, const char* file, int line) {
   if (ShenandoahSafepoint::is_at_shenandoah_safepoint()) {
     return;
