@@ -31,8 +31,6 @@
 #include "gc/shared/weakProcessor.inline.hpp"
 #include "gc/shared/gcTimer.hpp"
 #include "gc/shared/gcTrace.hpp"
-#include "gc/shared/referenceProcessor.hpp"
-#include "gc/shared/referenceProcessorPhaseTimes.hpp"
 #include "gc/shared/strongRootsScope.hpp"
 
 #include "gc/shenandoah/shenandoahBarrierSet.inline.hpp"
@@ -40,6 +38,7 @@
 #include "gc/shenandoah/shenandoahConcurrentMark.inline.hpp"
 #include "gc/shenandoah/shenandoahMarkCompact.hpp"
 #include "gc/shenandoah/shenandoahHeap.inline.hpp"
+#include "gc/shenandoah/shenandoahReferenceProcessor.hpp"
 #include "gc/shenandoah/shenandoahRootProcessor.inline.hpp"
 #include "gc/shenandoah/shenandoahOopClosures.inline.hpp"
 #include "gc/shenandoah/shenandoahPhaseTimings.hpp"
@@ -61,7 +60,7 @@ private:
 
   template <class T>
   inline void do_oop_work(T* p) {
-    ShenandoahConcurrentMark::mark_through_ref<T, UPDATE_REFS, NO_DEDUP>(p, _heap, _queue, _mark_context);
+    ShenandoahConcurrentMark::mark_through_ref<T, UPDATE_REFS, NO_DEDUP>(p, _heap, _queue, _mark_context, false);
   }
 
 public:
@@ -74,11 +73,12 @@ public:
   void do_oop(oop* p)       { do_oop_work(p); }
 };
 
-ShenandoahMarkRefsSuperClosure::ShenandoahMarkRefsSuperClosure(ShenandoahObjToScanQueue* q, ReferenceProcessor* rp) :
+ShenandoahMarkRefsSuperClosure::ShenandoahMarkRefsSuperClosure(ShenandoahObjToScanQueue* q, ShenandoahReferenceProcessor* rp) :
   MetadataVisitingOopIterateClosure(rp),
   _queue(q),
   _heap(ShenandoahHeap::heap()),
-  _mark_context(_heap->marking_context())
+  _mark_context(_heap->marking_context()),
+  _weak(false)
 { }
 
 template<UpdateRefsMode UPDATE_REFS>
@@ -87,7 +87,7 @@ private:
   ShenandoahRootScanner* _rp;
 public:
   ShenandoahInitMarkRootsTask(ShenandoahRootScanner* rp) :
-    AbstractGangTask("Shenandoah init mark roots task"),
+    AbstractGangTask("Shenandoah Init Mark Roots"),
     _rp(rp) {
   }
 
@@ -107,20 +107,7 @@ public:
 
 private:
   void do_work(ShenandoahHeap* heap, OopClosure* oops, uint worker_id) {
-    // The rationale for selecting the roots to scan is as follows:
-    //   a. With unload_classes = true, we only want to scan the actual strong roots from the
-    //      code cache. This will allow us to identify the dead classes, unload them, *and*
-    //      invalidate the relevant code cache blobs. This could be only done together with
-    //      class unloading.
-    //   b. With unload_classes = false, we have to nominally retain all the references from code
-    //      cache, because there could be the case of embedded class/oop in the generated code,
-    //      which we will never visit during mark. Without code cache invalidation, as in (a),
-    //      we risk executing that code cache blob, and crashing.
-    if (heap->unload_classes()) {
-      _rp->strong_roots_do(worker_id, oops);
-    } else {
-      _rp->roots_do(worker_id, oops);
-    }
+    _rp->roots_do(worker_id, oops);
   }
 };
 
@@ -130,7 +117,7 @@ private:
   bool                    _check_alive;
 public:
   ShenandoahUpdateRootsTask(ShenandoahRootUpdater* root_updater, bool check_alive) :
-    AbstractGangTask("Shenandoah update roots task"),
+    AbstractGangTask("Shenandoah Update Roots"),
     _root_updater(root_updater),
     _check_alive(check_alive){
   }
@@ -158,7 +145,7 @@ private:
 
 public:
   ShenandoahConcurrentMarkingTask(ShenandoahConcurrentMark* cm, TaskTerminator* terminator) :
-    AbstractGangTask("Root Region Scan"), _cm(cm), _terminator(terminator) {
+    AbstractGangTask("Shenandoah Concurrent Mark"), _cm(cm), _terminator(terminator) {
   }
 
   void work(uint worker_id) {
@@ -166,14 +153,8 @@ public:
     ShenandoahConcurrentWorkerSession worker_session(worker_id);
     ShenandoahSuspendibleThreadSetJoiner stsj(ShenandoahSuspendibleWorkers);
     ShenandoahObjToScanQueue* q = _cm->get_queue(worker_id);
-    ReferenceProcessor* rp;
-    if (heap->process_references()) {
-      rp = heap->ref_processor();
-      shenandoah_assert_rp_isalive_installed();
-    } else {
-      rp = NULL;
-    }
-
+    ShenandoahReferenceProcessor* rp = heap->ref_processor();
+    assert(rp != NULL, "need reference processor");
     _cm->mark_loop(worker_id, _terminator, rp,
                    true, // cancellable
                    ShenandoahStringDedup::is_enabled()); // perform string dedup
@@ -206,8 +187,7 @@ public:
           // * Weakly reachable otherwise
           // Some objects reachable from nmethods, such as the class loader (or klass_holder) of the receiver should be
           // live by the SATB invariant but other oops recorded in nmethods may behave differently.
-          JavaThread* jt = (JavaThread*)thread;
-          jt->nmethods_do(_code_cl);
+          thread->as_Java_thread()->nmethods_do(_code_cl);
         }
       }
     }
@@ -220,7 +200,7 @@ class ShenandoahProcessConcurrentRootsTask : public AbstractGangTask {
 private:
   ShenandoahConcurrentRootScanner<false /* concurrent */> _rs;
   ShenandoahConcurrentMark* const _cm;
-  ReferenceProcessor*             _rp;
+  ShenandoahReferenceProcessor*   _rp;
 public:
 
   ShenandoahProcessConcurrentRootsTask(ShenandoahConcurrentMark* cm,
@@ -233,15 +213,10 @@ template <typename T>
 ShenandoahProcessConcurrentRootsTask<T>::ShenandoahProcessConcurrentRootsTask(ShenandoahConcurrentMark* cm,
                                                                               ShenandoahPhaseTimings::Phase phase,
                                                                               uint nworkers) :
-  AbstractGangTask("Shenandoah STW Concurrent Mark Task"),
+  AbstractGangTask("Shenandoah Process Concurrent Roots"),
   _rs(nworkers, phase),
   _cm(cm),
-  _rp(NULL) {
-  ShenandoahHeap* heap = ShenandoahHeap::heap();
-  if (heap->process_references()) {
-    _rp = heap->ref_processor();
-    shenandoah_assert_rp_isalive_installed();
-  }
+  _rp(ShenandoahHeap::heap()->ref_processor()) {
 }
 
 template <typename T>
@@ -252,56 +227,56 @@ void ShenandoahProcessConcurrentRootsTask<T>::work(uint worker_id) {
   _rs.oops_do(&cl, worker_id);
 }
 
+class ShenandoahClaimThreadClosure : public ThreadClosure {
+private:
+  const uintx _claim_token;
+public:
+  ShenandoahClaimThreadClosure() :
+   _claim_token(Threads::thread_claim_token()) {}
+
+  virtual void do_thread(Thread* thread) {
+    thread->claim_threads_do(false /*is_par*/, _claim_token);
+  }
+};
+
 class ShenandoahFinalMarkingTask : public AbstractGangTask {
 private:
   ShenandoahConcurrentMark* _cm;
   TaskTerminator*           _terminator;
-  bool _dedup_string;
+  bool                      _dedup_string;
 
 public:
   ShenandoahFinalMarkingTask(ShenandoahConcurrentMark* cm, TaskTerminator* terminator, bool dedup_string) :
-    AbstractGangTask("Shenandoah Final Marking"), _cm(cm), _terminator(terminator), _dedup_string(dedup_string) {
+    AbstractGangTask("Shenandoah Final Mark"), _cm(cm), _terminator(terminator), _dedup_string(dedup_string) {
+
+    // Full GC does not need to remark threads and drain SATB buffers, but we need to claim the
+    // threads - it requires a StrongRootsScope around the task.
+    if (ShenandoahHeap::heap()->is_full_gc_in_progress()) {
+      ShenandoahClaimThreadClosure tc;
+      Threads::threads_do(&tc);
+    }
   }
 
   void work(uint worker_id) {
     ShenandoahHeap* heap = ShenandoahHeap::heap();
 
     ShenandoahParallelWorkerSession worker_session(worker_id);
-    ReferenceProcessor* rp;
-    if (heap->process_references()) {
-      rp = heap->ref_processor();
-      shenandoah_assert_rp_isalive_installed();
-    } else {
-      rp = NULL;
-    }
+    ShenandoahReferenceProcessor* rp = heap->ref_processor();
 
-    // First drain remaining SATB buffers.
-    // Notice that this is not strictly necessary for mark-compact. But since
-    // it requires a StrongRootsScope around the task, we need to claim the
-    // threads, and performance-wise it doesn't really matter. Adds about 1ms to
-    // full-gc.
-    {
+    if (!heap->is_full_gc_in_progress()) {
       ShenandoahObjToScanQueue* q = _cm->get_queue(worker_id);
-
       ShenandoahSATBBufferClosure cl(q);
       SATBMarkQueueSet& satb_mq_set = ShenandoahBarrierSet::satb_mark_queue_set();
-      while (satb_mq_set.apply_closure_to_completed_buffer(&cl));
+      while (satb_mq_set.apply_closure_to_completed_buffer(&cl)) {}
+
+      assert(!heap->has_forwarded_objects(), "Not expected");
       bool do_nmethods = heap->unload_classes() && !ShenandoahConcurrentRoots::can_do_concurrent_class_unloading();
-      if (heap->has_forwarded_objects()) {
-        ShenandoahMarkResolveRefsClosure resolve_mark_cl(q, rp);
-        MarkingCodeBlobClosure blobsCl(&resolve_mark_cl, !CodeBlobToOopClosure::FixRelocations);
-        ShenandoahSATBAndRemarkCodeRootsThreadsClosure tc(&cl,
-                                                          ShenandoahStoreValEnqueueBarrier ? &resolve_mark_cl : NULL,
-                                                          do_nmethods ? &blobsCl : NULL);
-        Threads::threads_do(&tc);
-      } else {
-        ShenandoahMarkRefsClosure mark_cl(q, rp);
-        MarkingCodeBlobClosure blobsCl(&mark_cl, !CodeBlobToOopClosure::FixRelocations);
-        ShenandoahSATBAndRemarkCodeRootsThreadsClosure tc(&cl,
-                                                          ShenandoahStoreValEnqueueBarrier ? &mark_cl : NULL,
-                                                          do_nmethods ? &blobsCl : NULL);
-        Threads::threads_do(&tc);
-      }
+      ShenandoahMarkRefsClosure mark_cl(q, rp);
+      MarkingCodeBlobClosure blobsCl(&mark_cl, !CodeBlobToOopClosure::FixRelocations);
+      ShenandoahSATBAndRemarkCodeRootsThreadsClosure tc(&cl,
+                                                        ShenandoahIUBarrier ? &mark_cl : NULL,
+                                                        do_nmethods ? &blobsCl : NULL);
+      Threads::threads_do(&tc);
     }
 
     _cm->mark_loop(worker_id, _terminator, rp,
@@ -317,9 +292,12 @@ void ShenandoahConcurrentMark::mark_roots(ShenandoahPhaseTimings::Phase root_pha
   assert(ShenandoahSafepoint::is_at_shenandoah_safepoint(), "Must be at a safepoint");
 
   ShenandoahHeap* heap = ShenandoahHeap::heap();
-
   ShenandoahGCPhase phase(root_phase);
 
+  ShenandoahReferenceProcessor* ref_processor = heap->ref_processor();
+  ref_processor->reset_thread_locals();
+  ref_processor->set_soft_reference_policy(_heap->soft_ref_policy()->should_clear_all_soft_refs());
+
   WorkGang* workers = heap->workers();
   uint nworkers = workers->active_workers();
 
@@ -421,24 +399,24 @@ void ShenandoahConcurrentMark::initialize(uint workers) {
 // Mark concurrent roots during concurrent phases
 class ShenandoahMarkConcurrentRootsTask : public AbstractGangTask {
 private:
-  SuspendibleThreadSetJoiner         _sts_joiner;
+  SuspendibleThreadSetJoiner          _sts_joiner;
   ShenandoahConcurrentRootScanner<true /* concurrent */> _rs;
-  ShenandoahObjToScanQueueSet* const _queue_set;
-  ReferenceProcessor* const          _rp;
+  ShenandoahObjToScanQueueSet* const  _queue_set;
+  ShenandoahReferenceProcessor* const _rp;
 
 public:
   ShenandoahMarkConcurrentRootsTask(ShenandoahObjToScanQueueSet* qs,
-                                    ReferenceProcessor* rp,
+                                    ShenandoahReferenceProcessor* rp,
                                     ShenandoahPhaseTimings::Phase phase,
                                     uint nworkers);
   void work(uint worker_id);
 };
 
 ShenandoahMarkConcurrentRootsTask::ShenandoahMarkConcurrentRootsTask(ShenandoahObjToScanQueueSet* qs,
-                                                                     ReferenceProcessor* rp,
+                                                                     ShenandoahReferenceProcessor* rp,
                                                                      ShenandoahPhaseTimings::Phase phase,
                                                                      uint nworkers) :
-  AbstractGangTask("Shenandoah Concurrent Mark Task"),
+  AbstractGangTask("Shenandoah Concurrent Mark Roots"),
   _rs(nworkers, phase),
   _queue_set(qs),
   _rp(rp) {
@@ -456,19 +434,7 @@ void ShenandoahConcurrentMark::mark_from_roots() {
   WorkGang* workers = _heap->workers();
   uint nworkers = workers->active_workers();
 
-  ReferenceProcessor* rp = NULL;
-  if (_heap->process_references()) {
-    rp = _heap->ref_processor();
-    rp->set_active_mt_degree(nworkers);
-
-    // enable ("weak") refs discovery
-    rp->enable_discovery(true /*verify_no_refs*/);
-    rp->setup_policy(_heap->soft_ref_policy()->should_clear_all_soft_refs());
-  }
-
-  shenandoah_assert_rp_isalive_not_installed();
-  ShenandoahIsAliveSelector is_alive;
-  ReferenceProcessorIsAliveMutator fix_isalive(_heap->ref_processor(), is_alive.is_alive_closure());
+  ShenandoahReferenceProcessor* rp = _heap->ref_processor();
 
   task_queues()->reserve(nworkers);
 
@@ -494,10 +460,6 @@ void ShenandoahConcurrentMark::finish_mark_from_roots(bool full_gc) {
   uint nworkers = _heap->workers()->active_workers();
 
   {
-    shenandoah_assert_rp_isalive_not_installed();
-    ShenandoahIsAliveSelector is_alive;
-    ReferenceProcessorIsAliveMutator fix_isalive(_heap->ref_processor(), is_alive.is_alive_closure());
-
     // Full GC does not execute concurrent cycle. Degenerated cycle may bypass concurrent cycle.
     // In those cases, concurrent roots might not be scanned, scan them here. Ideally, this
     // should piggyback to ShenandoahFinalMarkingTask, but it makes time tracking very hard.
@@ -538,342 +500,11 @@ void ShenandoahConcurrentMark::finish_mark_from_roots(bool full_gc) {
     assert(task_queues()->is_empty(), "Should be empty");
   }
 
-  // When we're done marking everything, we process weak references.
-  if (_heap->process_references()) {
-    weak_refs_work(full_gc);
-  }
-
   assert(task_queues()->is_empty(), "Should be empty");
   TASKQUEUE_STATS_ONLY(task_queues()->print_taskqueue_stats());
   TASKQUEUE_STATS_ONLY(task_queues()->reset_taskqueue_stats());
 }
 
-// Weak Reference Closures
-class ShenandoahCMDrainMarkingStackClosure: public VoidClosure {
-  uint _worker_id;
-  TaskTerminator* _terminator;
-  bool _reset_terminator;
-
-public:
-  ShenandoahCMDrainMarkingStackClosure(uint worker_id, TaskTerminator* t, bool reset_terminator = false):
-    _worker_id(worker_id),
-    _terminator(t),
-    _reset_terminator(reset_terminator) {
-  }
-
-  void do_void() {
-    assert(ShenandoahSafepoint::is_at_shenandoah_safepoint(), "Must be at a safepoint");
-
-    ShenandoahHeap* sh = ShenandoahHeap::heap();
-    ShenandoahConcurrentMark* scm = sh->concurrent_mark();
-    assert(sh->process_references(), "why else would we be here?");
-    ReferenceProcessor* rp = sh->ref_processor();
-
-    shenandoah_assert_rp_isalive_installed();
-
-    scm->mark_loop(_worker_id, _terminator, rp,
-                   false,   // not cancellable
-                   false);  // do not do strdedup
-
-    if (_reset_terminator) {
-      _terminator->reset_for_reuse();
-    }
-  }
-};
-
-class ShenandoahCMKeepAliveClosure : public OopClosure {
-private:
-  ShenandoahObjToScanQueue* _queue;
-  ShenandoahHeap* _heap;
-  ShenandoahMarkingContext* const _mark_context;
-
-  template <class T>
-  inline void do_oop_work(T* p) {
-    ShenandoahConcurrentMark::mark_through_ref<T, NONE, NO_DEDUP>(p, _heap, _queue, _mark_context);
-  }
-
-public:
-  ShenandoahCMKeepAliveClosure(ShenandoahObjToScanQueue* q) :
-    _queue(q),
-    _heap(ShenandoahHeap::heap()),
-    _mark_context(_heap->marking_context()) {}
-
-  void do_oop(narrowOop* p) { do_oop_work(p); }
-  void do_oop(oop* p)       { do_oop_work(p); }
-};
-
-class ShenandoahCMKeepAliveUpdateClosure : public OopClosure {
-private:
-  ShenandoahObjToScanQueue* _queue;
-  ShenandoahHeap* _heap;
-  ShenandoahMarkingContext* const _mark_context;
-
-  template <class T>
-  inline void do_oop_work(T* p) {
-    ShenandoahConcurrentMark::mark_through_ref<T, SIMPLE, NO_DEDUP>(p, _heap, _queue, _mark_context);
-  }
-
-public:
-  ShenandoahCMKeepAliveUpdateClosure(ShenandoahObjToScanQueue* q) :
-    _queue(q),
-    _heap(ShenandoahHeap::heap()),
-    _mark_context(_heap->marking_context()) {}
-
-  void do_oop(narrowOop* p) { do_oop_work(p); }
-  void do_oop(oop* p)       { do_oop_work(p); }
-};
-
-class ShenandoahWeakUpdateClosure : public OopClosure {
-private:
-  ShenandoahHeap* const _heap;
-
-  template <class T>
-  inline void do_oop_work(T* p) {
-    oop o = _heap->maybe_update_with_forwarded(p);
-    shenandoah_assert_marked_except(p, o, o == NULL);
-  }
-
-public:
-  ShenandoahWeakUpdateClosure() : _heap(ShenandoahHeap::heap()) {}
-
-  void do_oop(narrowOop* p) { do_oop_work(p); }
-  void do_oop(oop* p)       { do_oop_work(p); }
-};
-
-class ShenandoahRefProcTaskProxy : public AbstractGangTask {
-private:
-  AbstractRefProcTaskExecutor::ProcessTask& _proc_task;
-  TaskTerminator* _terminator;
-
-public:
-  ShenandoahRefProcTaskProxy(AbstractRefProcTaskExecutor::ProcessTask& proc_task,
-                             TaskTerminator* t) :
-    AbstractGangTask("Process reference objects in parallel"),
-    _proc_task(proc_task),
-    _terminator(t) {
-  }
-
-  void work(uint worker_id) {
-    ResourceMark rm;
-    HandleMark hm;
-    assert(ShenandoahSafepoint::is_at_shenandoah_safepoint(), "Must be at a safepoint");
-    ShenandoahHeap* heap = ShenandoahHeap::heap();
-    ShenandoahParallelWorkerSession worker_session(worker_id);
-    ShenandoahCMDrainMarkingStackClosure complete_gc(worker_id, _terminator);
-    if (heap->has_forwarded_objects()) {
-      ShenandoahForwardedIsAliveClosure is_alive;
-      ShenandoahCMKeepAliveUpdateClosure keep_alive(heap->concurrent_mark()->get_queue(worker_id));
-      _proc_task.work(worker_id, is_alive, keep_alive, complete_gc);
-    } else {
-      ShenandoahIsAliveClosure is_alive;
-      ShenandoahCMKeepAliveClosure keep_alive(heap->concurrent_mark()->get_queue(worker_id));
-      _proc_task.work(worker_id, is_alive, keep_alive, complete_gc);
-    }
-  }
-};
-
-class ShenandoahRefProcTaskExecutor : public AbstractRefProcTaskExecutor {
-private:
-  WorkGang* _workers;
-
-public:
-  ShenandoahRefProcTaskExecutor(WorkGang* workers) :
-    _workers(workers) {
-  }
-
-  // Executes a task using worker threads.
-  void execute(ProcessTask& task, uint ergo_workers) {
-    assert(ShenandoahSafepoint::is_at_shenandoah_safepoint(), "Must be at a safepoint");
-
-    ShenandoahHeap* heap = ShenandoahHeap::heap();
-    ShenandoahConcurrentMark* cm = heap->concurrent_mark();
-    ShenandoahPushWorkerQueuesScope scope(_workers, cm->task_queues(),
-                                          ergo_workers,
-                                          /* do_check = */ false);
-    uint nworkers = _workers->active_workers();
-    cm->task_queues()->reserve(nworkers);
-    TaskTerminator terminator(nworkers, cm->task_queues());
-    ShenandoahRefProcTaskProxy proc_task_proxy(task, &terminator);
-    _workers->run_task(&proc_task_proxy);
-  }
-};
-
-void ShenandoahConcurrentMark::weak_refs_work(bool full_gc) {
-  assert(_heap->process_references(), "sanity");
-
-  ShenandoahPhaseTimings::Phase phase_root =
-          full_gc ?
-          ShenandoahPhaseTimings::full_gc_weakrefs :
-          ShenandoahPhaseTimings::weakrefs;
-
-  ShenandoahGCPhase phase(phase_root);
-
-  ReferenceProcessor* rp = _heap->ref_processor();
-
-  // NOTE: We cannot shortcut on has_discovered_references() here, because
-  // we will miss marking JNI Weak refs then, see implementation in
-  // ReferenceProcessor::process_discovered_references.
-  weak_refs_work_doit(full_gc);
-
-  rp->verify_no_references_recorded();
-  assert(!rp->discovery_enabled(), "Post condition");
-
-}
-
-void ShenandoahConcurrentMark::weak_refs_work_doit(bool full_gc) {
-  ReferenceProcessor* rp = _heap->ref_processor();
-
-  ShenandoahPhaseTimings::Phase phase_process =
-          full_gc ?
-          ShenandoahPhaseTimings::full_gc_weakrefs_process :
-          ShenandoahPhaseTimings::weakrefs_process;
-
-  shenandoah_assert_rp_isalive_not_installed();
-  ShenandoahIsAliveSelector is_alive;
-  ReferenceProcessorIsAliveMutator fix_isalive(rp, is_alive.is_alive_closure());
-
-  WorkGang* workers = _heap->workers();
-  uint nworkers = workers->active_workers();
-
-  rp->setup_policy(_heap->soft_ref_policy()->should_clear_all_soft_refs());
-  rp->set_active_mt_degree(nworkers);
-
-  assert(task_queues()->is_empty(), "Should be empty");
-
-  // complete_gc and keep_alive closures instantiated here are only needed for
-  // single-threaded path in RP. They share the queue 0 for tracking work, which
-  // simplifies implementation. Since RP may decide to call complete_gc several
-  // times, we need to be able to reuse the terminator.
-  uint serial_worker_id = 0;
-  TaskTerminator terminator(1, task_queues());
-  ShenandoahCMDrainMarkingStackClosure complete_gc(serial_worker_id, &terminator, /* reset_terminator = */ true);
-
-  ShenandoahRefProcTaskExecutor executor(workers);
-
-  ReferenceProcessorPhaseTimes pt(_heap->gc_timer(), rp->num_queues());
-
-  {
-    // Note: Don't emit JFR event for this phase, to avoid overflow nesting phase level.
-    // Reference Processor emits 2 levels JFR event, that can get us over the JFR
-    // event nesting level limits, in case of degenerated GC gets upgraded to
-    // full GC.
-    ShenandoahTimingsTracker phase_timing(phase_process);
-
-    if (_heap->has_forwarded_objects()) {
-      ShenandoahCMKeepAliveUpdateClosure keep_alive(get_queue(serial_worker_id));
-      const ReferenceProcessorStats& stats =
-        rp->process_discovered_references(is_alive.is_alive_closure(), &keep_alive,
-                                          &complete_gc, &executor,
-                                          &pt);
-       _heap->tracer()->report_gc_reference_stats(stats);
-    } else {
-      ShenandoahCMKeepAliveClosure keep_alive(get_queue(serial_worker_id));
-      const ReferenceProcessorStats& stats =
-        rp->process_discovered_references(is_alive.is_alive_closure(), &keep_alive,
-                                          &complete_gc, &executor,
-                                          &pt);
-      _heap->tracer()->report_gc_reference_stats(stats);
-    }
-
-    pt.print_all_references();
-
-    assert(task_queues()->is_empty(), "Should be empty");
-  }
-}
-
-class ShenandoahCancelledGCYieldClosure : public YieldClosure {
-private:
-  ShenandoahHeap* const _heap;
-public:
-  ShenandoahCancelledGCYieldClosure() : _heap(ShenandoahHeap::heap()) {};
-  virtual bool should_return() { return _heap->cancelled_gc(); }
-};
-
-class ShenandoahPrecleanCompleteGCClosure : public VoidClosure {
-public:
-  void do_void() {
-    ShenandoahHeap* sh = ShenandoahHeap::heap();
-    ShenandoahConcurrentMark* scm = sh->concurrent_mark();
-    assert(sh->process_references(), "why else would we be here?");
-    TaskTerminator terminator(1, scm->task_queues());
-
-    ReferenceProcessor* rp = sh->ref_processor();
-    shenandoah_assert_rp_isalive_installed();
-
-    scm->mark_loop(0, &terminator, rp,
-                   false, // not cancellable
-                   false); // do not do strdedup
-  }
-};
-
-class ShenandoahPrecleanTask : public AbstractGangTask {
-private:
-  ReferenceProcessor* _rp;
-
-public:
-  ShenandoahPrecleanTask(ReferenceProcessor* rp) :
-          AbstractGangTask("Precleaning task"),
-          _rp(rp) {}
-
-  void work(uint worker_id) {
-    assert(worker_id == 0, "The code below is single-threaded, only one worker is expected");
-    ShenandoahParallelWorkerSession worker_session(worker_id);
-
-    ShenandoahHeap* sh = ShenandoahHeap::heap();
-    assert(!sh->has_forwarded_objects(), "No forwarded objects expected here");
-
-    ShenandoahObjToScanQueue* q = sh->concurrent_mark()->get_queue(worker_id);
-
-    ShenandoahCancelledGCYieldClosure yield;
-    ShenandoahPrecleanCompleteGCClosure complete_gc;
-
-    ShenandoahIsAliveClosure is_alive;
-    ShenandoahCMKeepAliveClosure keep_alive(q);
-    ResourceMark rm;
-    _rp->preclean_discovered_references(&is_alive, &keep_alive,
-                                        &complete_gc, &yield,
-                                        NULL);
-  }
-};
-
-void ShenandoahConcurrentMark::preclean_weak_refs() {
-  // Pre-cleaning weak references before diving into STW makes sense at the
-  // end of concurrent mark. This will filter out the references which referents
-  // are alive. Note that ReferenceProcessor already filters out these on reference
-  // discovery, and the bulk of work is done here. This phase processes leftovers
-  // that missed the initial filtering, i.e. when referent was marked alive after
-  // reference was discovered by RP.
-
-  assert(_heap->process_references(), "sanity");
-
-  // Shortcut if no references were discovered to avoid winding up threads.
-  ReferenceProcessor* rp = _heap->ref_processor();
-  if (!rp->has_discovered_references()) {
-    return;
-  }
-
-  assert(task_queues()->is_empty(), "Should be empty");
-
-  ReferenceProcessorMTDiscoveryMutator fix_mt_discovery(rp, false);
-
-  shenandoah_assert_rp_isalive_not_installed();
-  ShenandoahIsAliveSelector is_alive;
-  ReferenceProcessorIsAliveMutator fix_isalive(rp, is_alive.is_alive_closure());
-
-  // Execute precleaning in the worker thread: it will give us GCLABs, String dedup
-  // queues and other goodies. When upstream ReferenceProcessor starts supporting
-  // parallel precleans, we can extend this to more threads.
-  WorkGang* workers = _heap->workers();
-  uint nworkers = workers->active_workers();
-  assert(nworkers == 1, "This code uses only a single worker");
-  task_queues()->reserve(nworkers);
-
-  ShenandoahPrecleanTask task(rp);
-  workers->run_task(&task);
-
-  assert(task_queues()->is_empty(), "Should be empty");
-}
-
 void ShenandoahConcurrentMark::cancel() {
   // Clean up marking stacks.
   ShenandoahObjToScanQueueSet* queues = task_queues();
@@ -889,7 +520,7 @@ ShenandoahObjToScanQueue* ShenandoahConcurrentMark::get_queue(uint worker_id) {
 }
 
 template <bool CANCELLABLE>
-void ShenandoahConcurrentMark::mark_loop_prework(uint w, TaskTerminator *t, ReferenceProcessor *rp,
+void ShenandoahConcurrentMark::mark_loop_prework(uint w, TaskTerminator *t, ShenandoahReferenceProcessor* rp,
                                                  bool strdedup) {
   ShenandoahObjToScanQueue* q = get_queue(w);
 
@@ -947,6 +578,8 @@ void ShenandoahConcurrentMark::mark_loop_work(T* cl, ShenandoahLiveData* live_da
   ShenandoahObjToScanQueue* q;
   ShenandoahMarkTask t;
 
+  _heap->ref_processor()->set_mark_closure(worker_id, cl);
+
   /*
    * Process outstanding queues, if any.
    *
