@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2019, 2020, Red Hat, Inc. All rights reserved.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 only, as
@@ -36,12 +37,7 @@ bool ShenandoahConcurrentRoots::should_do_concurrent_roots() {
 }
 
 bool ShenandoahConcurrentRoots::can_do_concurrent_class_unloading() {
-#if defined(X86) && !defined(SOLARIS)
-  return ShenandoahCodeRootsStyle == 2 &&
-         ClassUnloading;
-#else
-  return false;
-#endif
+  return ClassUnloading;
 }
 
 bool ShenandoahConcurrentRoots::should_do_concurrent_class_unloading() {
