@@ -35,11 +35,13 @@
 #include "gc/shenandoah/shenandoahPadding.hpp"
 #include "gc/shenandoah/shenandoahSharedVariables.hpp"
 #include "gc/shenandoah/shenandoahUnload.hpp"
+#include "memory/metaspace.hpp"
 #include "services/memoryManager.hpp"
 #include "utilities/globalDefinitions.hpp"
+#include "utilities/stack.hpp"
 
 class ConcurrentGCTimer;
-class ReferenceProcessor;
+class ObjectIterateScanRootClosure;
 class ShenandoahCollectorPolicy;
 class ShenandoahControlThread;
 class ShenandoahGCSession;
@@ -57,6 +59,7 @@ class ShenandoahFreeSet;
 class ShenandoahConcurrentMark;
 class ShenandoahMarkCompact;
 class ShenandoahMonitoringSupport;
+class ShenandoahReferenceProcessor;
 class ShenandoahPacer;
 class ShenandoahVerifier;
 class ShenandoahWorkGang;
@@ -107,6 +110,7 @@ public:
 
 typedef ShenandoahLock    ShenandoahHeapLock;
 typedef ShenandoahLocker  ShenandoahHeapLocker;
+typedef Stack<oop, mtGC>  ShenandoahScanObjectStack;
 
 // Shenandoah GC is low-pause concurrent GC that uses Brooks forwarding pointers
 // to encode forwarding data. See BrooksPointer for details on forwarding data encoding.
@@ -117,8 +121,8 @@ class ShenandoahHeap : public CollectedHeap {
   friend class VMStructs;
   friend class ShenandoahGCSession;
   friend class ShenandoahGCStateResetter;
+  friend class ShenandoahParallelObjectIterator;
   friend class ShenandoahSafepoint;
-
 // ---------- Locks that guard important data structures in Heap
 //
 private:
@@ -131,9 +135,6 @@ public:
 
 // ---------- Initialization, termination, identification, printing routines
 //
-private:
-  static ShenandoahHeap* _heap;
-
 public:
   static ShenandoahHeap* heap();
 
@@ -203,7 +204,7 @@ public:
   void assert_gc_workers(uint nworker) NOT_DEBUG_RETURN;
 
   WorkGang* workers() const;
-  WorkGang* get_safepoint_workers();
+  WorkGang* safepoint_workers();
 
   void gc_threads_do(ThreadClosure* tcl) const;
 
@@ -389,13 +390,14 @@ public:
   // for concurrent operation.
   void entry_reset();
   void entry_mark();
-  void entry_preclean();
+  void entry_weak_refs();
   void entry_weak_roots();
   void entry_class_unloading();
   void entry_strong_roots();
   void entry_cleanup_early();
   void entry_rendezvous_roots();
   void entry_evac();
+  void entry_update_thread_roots();
   void entry_updaterefs();
   void entry_cleanup_complete();
   void entry_uncommit(double shrink_before, size_t shrink_until);
@@ -413,7 +415,7 @@ private:
 
   void op_reset();
   void op_mark();
-  void op_preclean();
+  void op_weak_refs();
   void op_weak_roots();
   void op_class_unloading();
   void op_strong_roots();
@@ -421,6 +423,7 @@ private:
   void op_rendezvous_roots();
   void op_conc_evac();
   void op_stw_evac();
+  void op_update_thread_roots();
   void op_updaterefs();
   void op_cleanup_complete();
   void op_uncommit(double shrink_before, size_t shrink_until);
@@ -491,20 +494,10 @@ public:
 // ---------- Reference processing
 //
 private:
-  AlwaysTrueClosure    _subject_to_discovery;
-  ReferenceProcessor*  _ref_processor;
-  ShenandoahSharedFlag _process_references;
-  bool                 _ref_proc_mt_discovery;
-  bool                 _ref_proc_mt_processing;
-
-  void ref_processing_init();
+  ShenandoahReferenceProcessor* const _ref_processor;
 
 public:
-  ReferenceProcessor* ref_processor() { return _ref_processor; }
-  bool ref_processor_mt_discovery()   { return _ref_proc_mt_discovery;  }
-  bool ref_processor_mt_processing()  { return _ref_proc_mt_processing; }
-  void set_process_references(bool pr);
-  bool process_references() const;
+  ShenandoahReferenceProcessor* ref_processor() { return _ref_processor; }
 
 // ---------- Class Unloading
 //
@@ -522,12 +515,17 @@ public:
 private:
   void stw_unload_classes(bool full_gc);
   void stw_process_weak_roots(bool full_gc);
+  void stw_weak_refs(bool full_gc);
 
   // Prepare concurrent root processing
   void prepare_concurrent_roots();
   // Prepare and finish concurrent unloading
   void prepare_concurrent_unloading();
   void finish_concurrent_unloading();
+  // Heap iteration support
+  void scan_roots_for_iteration(ShenandoahScanObjectStack* oop_stack, ObjectIterateScanRootClosure* oops);
+  bool prepare_aux_bitmap_for_iteration();
+  void reclaim_aux_bitmap_for_iteration();
 
 // ---------- Generic interface hooks
 // Minor things that super-interface expects us to implement to play nice with
@@ -553,13 +551,12 @@ public:
 
   // Used for native heap walkers: heap dumpers, mostly
   void object_iterate(ObjectClosure* cl);
+  // Parallel heap iteration support
+  virtual ParallelObjectIterator* parallel_object_iterator(uint workers);
 
   // Keep alive an object that was loaded with AS_NO_KEEPALIVE.
   void keep_alive(oop obj);
 
-  // Used by RMI
-  jlong millis_since_last_gc();
-
 // ---------- Safepoint interface hooks
 //
 public:
@@ -603,9 +600,6 @@ public:
 
   void notify_mutator_alloc_words(size_t words, bool waste);
 
-  // Shenandoah supports TLAB allocation
-  bool supports_tlab_allocation() const { return true; }
-
   HeapWord* allocate_new_tlab(size_t min_size, size_t requested_size, size_t* actual_size);
   size_t tlab_capacity(Thread *thr) const;
   size_t unsafe_max_tlab_alloc(Thread *thread) const;
