@@ -49,8 +49,7 @@
 #include "utilities/globalDefinitions.hpp"
 
 inline ShenandoahHeap* ShenandoahHeap::heap() {
-  assert(_heap != NULL, "Heap is not initialized yet");
-  return _heap;
+  return named_heap<ShenandoahHeap>(CollectedHeap::Shenandoah);
 }
 
 inline ShenandoahHeapRegion* ShenandoahRegionIterator::next() {
@@ -67,7 +66,7 @@ inline WorkGang* ShenandoahHeap::workers() const {
   return _workers;
 }
 
-inline WorkGang* ShenandoahHeap::get_safepoint_workers() {
+inline WorkGang* ShenandoahHeap::safepoint_workers() {
   return _safepoint_workers;
 }
 
@@ -150,14 +149,14 @@ inline oop ShenandoahHeap::cas_oop(oop n, oop* addr, oop c) {
 inline oop ShenandoahHeap::cas_oop(oop n, narrowOop* addr, narrowOop c) {
   assert(is_aligned(addr, sizeof(narrowOop)), "Address should be aligned: " PTR_FORMAT, p2i(addr));
   narrowOop val = CompressedOops::encode(n);
-  return CompressedOops::decode((narrowOop) Atomic::cmpxchg(addr, c, val));
+  return CompressedOops::decode(Atomic::cmpxchg(addr, c, val));
 }
 
 inline oop ShenandoahHeap::cas_oop(oop n, narrowOop* addr, oop c) {
   assert(is_aligned(addr, sizeof(narrowOop)), "Address should be aligned: " PTR_FORMAT, p2i(addr));
   narrowOop cmp = CompressedOops::encode(c);
   narrowOop val = CompressedOops::encode(n);
-  return CompressedOops::decode((narrowOop) Atomic::cmpxchg(addr, cmp, val));
+  return CompressedOops::decode(Atomic::cmpxchg(addr, cmp, val));
 }
 
 template <class T>
@@ -327,13 +326,9 @@ inline oop ShenandoahHeap::evacuate_object(oop p, Thread* thread) {
   }
 }
 
-template<bool RESOLVE>
 inline bool ShenandoahHeap::requires_marking(const void* entry) const {
   oop obj = oop(entry);
-  if (RESOLVE) {
-    obj = ShenandoahBarrierSet::resolve_forwarded_not_null(obj);
-  }
-  return !_marking_context->is_marked(obj);
+  return !_marking_context->is_marked_strong(obj);
 }
 
 inline bool ShenandoahHeap::in_collection_set(oop p) const {
@@ -406,7 +401,6 @@ inline void ShenandoahHeap::marked_object_iterate(ShenandoahHeapRegion* region,
   ShenandoahMarkingContext* const ctx = complete_marking_context();
   assert(ctx->is_complete(), "sanity");
 
-  MarkBitMap* mark_bit_map = ctx->mark_bit_map();
   HeapWord* tams = ctx->top_at_mark_start(region);
 
   size_t skip_bitmap_delta = 1;
@@ -418,7 +412,7 @@ inline void ShenandoahHeap::marked_object_iterate(ShenandoahHeapRegion* region,
 
   // Try to scan the initial candidate. If the candidate is above the TAMS, it would
   // fail the subsequent "< limit_bitmap" checks, and fall through to Step 2.
-  HeapWord* cb = mark_bit_map->get_next_marked_addr(start, end);
+  HeapWord* cb = ctx->get_next_marked_addr(start, end);
 
   intx dist = ShenandoahMarkScanPrefetch;
   if (dist > 0) {
@@ -445,7 +439,7 @@ inline void ShenandoahHeap::marked_object_iterate(ShenandoahHeapRegion* region,
         slots[avail++] = cb;
         cb += skip_bitmap_delta;
         if (cb < limit_bitmap) {
-          cb = mark_bit_map->get_next_marked_addr(cb, limit_bitmap);
+          cb = ctx->get_next_marked_addr(cb, limit_bitmap);
         }
       }
 
@@ -468,7 +462,7 @@ inline void ShenandoahHeap::marked_object_iterate(ShenandoahHeapRegion* region,
       cl->do_object(obj);
       cb += skip_bitmap_delta;
       if (cb < limit_bitmap) {
-        cb = mark_bit_map->get_next_marked_addr(cb, limit_bitmap);
+        cb = ctx->get_next_marked_addr(cb, limit_bitmap);
       }
     }
   }
