@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2019, Red Hat, Inc. All rights reserved.
+ * Copyright (c) 2019, 2020, Red Hat, Inc. All rights reserved.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 only, as
@@ -26,24 +27,37 @@
 
 
 #include "classfile/classLoaderDataGraph.hpp"
-#include "classfile/systemDictionary.hpp"
 #include "code/codeCache.hpp"
 #include "gc/shenandoah/shenandoahAsserts.hpp"
-#include "gc/shenandoah/shenandoahHeap.hpp"
+#include "gc/shenandoah/shenandoahHeap.inline.hpp"
 #include "gc/shenandoah/shenandoahPhaseTimings.hpp"
 #include "gc/shenandoah/shenandoahRootVerifier.hpp"
 #include "gc/shenandoah/shenandoahStringDedup.hpp"
 #include "gc/shenandoah/shenandoahUtils.hpp"
-#include "gc/shared/weakProcessor.inline.hpp"
-#include "memory/universe.hpp"
+#include "gc/shared/oopStorage.inline.hpp"
+#include "gc/shared/oopStorageSet.hpp"
 #include "runtime/thread.hpp"
-#include "services/management.hpp"
 #include "utilities/debug.hpp"
 
+ShenandoahGCStateResetter::ShenandoahGCStateResetter() :
+  _heap(ShenandoahHeap::heap()),
+  _gc_state(_heap->gc_state()),
+  _concurrent_weak_root_in_progress(ShenandoahHeap::heap()->is_concurrent_weak_root_in_progress()) {
+  _heap->_gc_state.clear();
+  _heap->set_concurrent_weak_root_in_progress(false);
+}
+
+ShenandoahGCStateResetter::~ShenandoahGCStateResetter() {
+  _heap->_gc_state.set(_gc_state);
+  assert(_heap->gc_state() == _gc_state, "Should be restored");
+  _heap->set_concurrent_weak_root_in_progress(_concurrent_weak_root_in_progress);
+}
+
 // Check for overflow of number of root types.
 STATIC_ASSERT((static_cast<uint>(ShenandoahRootVerifier::AllRoots) + 1) > static_cast<uint>(ShenandoahRootVerifier::AllRoots));
 
-ShenandoahRootVerifier::ShenandoahRootVerifier() : _types(AllRoots) {
+ShenandoahRootVerifier::ShenandoahRootVerifier(RootTypes types) : _types(types) {
+  Threads::change_thread_claim_token();
 }
 
 void ShenandoahRootVerifier::excludes(RootTypes types) {
@@ -51,10 +65,16 @@ void ShenandoahRootVerifier::excludes(RootTypes types) {
 }
 
 bool ShenandoahRootVerifier::verify(RootTypes type) const {
-  return (_types & type) != 0;
+  return (_types & type) == type;
+}
+
+ShenandoahRootVerifier::RootTypes ShenandoahRootVerifier::combine(RootTypes t1, RootTypes t2) {
+  return static_cast<ShenandoahRootVerifier::RootTypes>(static_cast<uint>(t1) | static_cast<uint>(t2));
 }
 
 void ShenandoahRootVerifier::oops_do(OopClosure* oops) {
+  ShenandoahGCStateResetter resetter;
+
   CodeBlobToOopClosure blobs(oops, !CodeBlobToOopClosure::FixRelocations);
   if (verify(CodeRoots)) {
     shenandoah_assert_locked_or_safepoint(CodeCache_lock);
@@ -69,22 +89,17 @@ void ShenandoahRootVerifier::oops_do(OopClosure* oops) {
 
   if (verify(SerialRoots)) {
     shenandoah_assert_safepoint();
-    Universe::oops_do(oops);
-    Management::oops_do(oops);
-    JvmtiExport::oops_do(oops);
-    ObjectSynchronizer::oops_do(oops);
-    SystemDictionary::oops_do(oops);
   }
 
   if (verify(JNIHandleRoots)) {
     shenandoah_assert_safepoint();
     JNIHandles::oops_do(oops);
+    Universe::vm_global()->oops_do(oops);
   }
 
   if (verify(WeakRoots)) {
     shenandoah_assert_safepoint();
-    AlwaysTrueClosure always_true;
-    WeakProcessor::weak_oops_do(&always_true, oops);
+    weak_roots_do(oops);
   }
 
   if (ShenandoahStringDedup::is_enabled() && verify(StringDedupRoots)) {
@@ -102,6 +117,7 @@ void ShenandoahRootVerifier::oops_do(OopClosure* oops) {
 }
 
 void ShenandoahRootVerifier::roots_do(OopClosure* oops) {
+  ShenandoahGCStateResetter resetter;
   shenandoah_assert_safepoint();
 
   CodeBlobToOopClosure blobs(oops, !CodeBlobToOopClosure::FixRelocations);
@@ -110,27 +126,17 @@ void ShenandoahRootVerifier::roots_do(OopClosure* oops) {
   CLDToOopClosure clds(oops, ClassLoaderData::_claim_none);
   ClassLoaderDataGraph::cld_do(&clds);
 
-  Universe::oops_do(oops);
-  Management::oops_do(oops);
-  JvmtiExport::oops_do(oops);
   JNIHandles::oops_do(oops);
-  ObjectSynchronizer::oops_do(oops);
-  SystemDictionary::oops_do(oops);
-
-  AlwaysTrueClosure always_true;
-  WeakProcessor::weak_oops_do(&always_true, oops);
-
-  if (ShenandoahStringDedup::is_enabled()) {
-    ShenandoahStringDedup::oops_do_slow(oops);
-  }
+  Universe::vm_global()->oops_do(oops);
 
   // Do thread roots the last. This allows verification code to find
   // any broken objects from those special roots first, not the accidental
   // dangling reference from the thread root.
-  Threads::possibly_parallel_oops_do(false, oops, &blobs);
+  Threads::possibly_parallel_oops_do(true, oops, &blobs);
 }
 
 void ShenandoahRootVerifier::strong_roots_do(OopClosure* oops) {
+  ShenandoahGCStateResetter resetter;
   shenandoah_assert_safepoint();
 
   CodeBlobToOopClosure blobs(oops, !CodeBlobToOopClosure::FixRelocations);
@@ -138,15 +144,18 @@ void ShenandoahRootVerifier::strong_roots_do(OopClosure* oops) {
   CLDToOopClosure clds(oops, ClassLoaderData::_claim_none);
   ClassLoaderDataGraph::roots_cld_do(&clds, NULL);
 
-  Universe::oops_do(oops);
-  Management::oops_do(oops);
-  JvmtiExport::oops_do(oops);
   JNIHandles::oops_do(oops);
-  ObjectSynchronizer::oops_do(oops);
-  SystemDictionary::oops_do(oops);
+  Universe::vm_global()->oops_do(oops);
 
   // Do thread roots the last. This allows verification code to find
   // any broken objects from those special roots first, not the accidental
   // dangling reference from the thread root.
-  Threads::possibly_parallel_oops_do(false, oops, &blobs);
+  Threads::possibly_parallel_oops_do(true, oops, &blobs);
+}
+
+void ShenandoahRootVerifier::weak_roots_do(OopClosure* cl) {
+  for (OopStorageSet::Iterator it = OopStorageSet::weak_iterator(); !it.is_end(); ++it) {
+    OopStorage* storage = *it;
+    storage->oops_do<OopClosure>(cl);
+  }
 }
