@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2017, 2020, Red Hat, Inc. All rights reserved.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 only, as
@@ -109,7 +110,7 @@ public:
   ShenandoahStringDedupUnlinkOrOopsDoTask(BoolObjectClosure* is_alive,
                                   OopClosure* keep_alive,
                                   bool allow_resize_and_rehash) :
-    AbstractGangTask("StringDedupUnlinkOrOopsDoTask"),
+    AbstractGangTask("Shenandoah String Dedup Unlink/Process"),
     _cl(is_alive, keep_alive) {
       StringDedup::gc_prologue(allow_resize_and_rehash);
   }
