@@ -49,6 +49,7 @@ private:
   uint  _worker_id;
   bool _force_satb_flush;
   int  _disarmed_value;
+  double _paced_time;
 
   ShenandoahThreadLocalData() :
     _gc_state(0),
@@ -59,7 +60,8 @@ private:
     _gclab_size(0),
     _worker_id(INVALID_WORKER_ID),
     _force_satb_flush(false),
-    _disarmed_value(0) {
+    _disarmed_value(0),
+    _paced_time(0) {
 
     // At least on x86_64, nmethod entry barrier encodes _disarmed_value offset
     // in instruction as disp8 immed
@@ -139,6 +141,18 @@ public:
     data(thread)->_gclab_size = v;
   }
 
+  static void add_paced_time(Thread* thread, double v) {
+    data(thread)->_paced_time += v;
+  }
+
+  static double paced_time(Thread* thread) {
+    return data(thread)->_paced_time;
+  }
+
+  static void reset_paced_time(Thread* thread) {
+    data(thread)->_paced_time = 0;
+  }
+
   static void set_disarmed_value(Thread* thread, int value) {
     data(thread)->_disarmed_value = value;
   }
