@@ -28,6 +28,7 @@
 #include "gc/z/zGlobals.hpp"
 #include "utilities/globalDefinitions.hpp"
 #include "utilities/macros.hpp"
+#include "utilities/powerOfTwo.hpp"
 
 inline bool ZAddress::is_null(uintptr_t value) {
   return value == 0;
@@ -70,6 +71,10 @@ inline bool ZAddress::is_marked(uintptr_t value) {
   return value & ZAddressMetadataMarked;
 }
 
+inline bool ZAddress::is_marked_or_null(uintptr_t value) {
+  return is_marked(value) || is_null(value);
+}
+
 inline bool ZAddress::is_finalizable(uintptr_t value) {
   return value & ZAddressMetadataFinalizable;
 }
@@ -92,16 +97,12 @@ inline bool ZAddress::is_in(uintptr_t value) {
   return value & (ZAddressMetadataMask & ~ZAddressMetadataFinalizable);
 }
 
-inline uintptr_t ZAddress::address(uintptr_t value) {
-  return value | ZAddressBase;
-}
-
 inline uintptr_t ZAddress::offset(uintptr_t value) {
   return value & ZAddressOffsetMask;
 }
 
 inline uintptr_t ZAddress::good(uintptr_t value) {
-  return address(offset(value) | ZAddressGoodMask);
+  return offset(value) | ZAddressGoodMask;
 }
 
 inline uintptr_t ZAddress::good_or_null(uintptr_t value) {
@@ -109,23 +110,23 @@ inline uintptr_t ZAddress::good_or_null(uintptr_t value) {
 }
 
 inline uintptr_t ZAddress::finalizable_good(uintptr_t value) {
-  return address(offset(value) | ZAddressMetadataFinalizable | ZAddressGoodMask);
+  return offset(value) | ZAddressMetadataFinalizable | ZAddressGoodMask;
 }
 
 inline uintptr_t ZAddress::marked(uintptr_t value) {
-  return address(offset(value) | ZAddressMetadataMarked);
+  return offset(value) | ZAddressMetadataMarked;
 }
 
 inline uintptr_t ZAddress::marked0(uintptr_t value) {
-  return address(offset(value) | ZAddressMetadataMarked0);
+  return offset(value) | ZAddressMetadataMarked0;
 }
 
 inline uintptr_t ZAddress::marked1(uintptr_t value) {
-  return address(offset(value) | ZAddressMetadataMarked1);
+  return offset(value) | ZAddressMetadataMarked1;
 }
 
 inline uintptr_t ZAddress::remapped(uintptr_t value) {
-  return address(offset(value) | ZAddressMetadataRemapped);
+  return offset(value) | ZAddressMetadataRemapped;
 }
 
 inline uintptr_t ZAddress::remapped_or_null(uintptr_t value) {
