@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,6 +25,8 @@
 #include "gc/shared/gcHeapSummary.hpp"
 #include "gc/shared/suspendibleThreadSet.hpp"
 #include "gc/z/zCollectedHeap.hpp"
+#include "gc/z/zDirector.hpp"
+#include "gc/z/zDriver.hpp"
 #include "gc/z/zGlobals.hpp"
 #include "gc/z/zHeap.inline.hpp"
 #include "gc/z/zNMethod.hpp"
@@ -33,15 +35,13 @@
 #include "gc/z/zServiceability.hpp"
 #include "gc/z/zStat.hpp"
 #include "gc/z/zUtils.inline.hpp"
+#include "memory/classLoaderMetaspace.hpp"
+#include "memory/iterator.hpp"
 #include "memory/universe.hpp"
-#include "runtime/mutexLocker.hpp"
 #include "utilities/align.hpp"
 
 ZCollectedHeap* ZCollectedHeap::heap() {
-  CollectedHeap* heap = Universe::heap();
-  assert(heap != NULL, "Uninitialized access to ZCollectedHeap::heap()");
-  assert(heap->kind() == CollectedHeap::Z, "Invalid name");
-  return (ZCollectedHeap*)heap;
+  return named_heap<ZCollectedHeap>(CollectedHeap::Z);
 }
 
 ZCollectedHeap::ZCollectedHeap() :
@@ -51,7 +51,6 @@ ZCollectedHeap::ZCollectedHeap() :
     _heap(),
     _director(new ZDirector()),
     _driver(new ZDriver()),
-    _uncommitter(new ZUncommitter()),
     _stat(new ZStat()),
     _runtime_workers() {}
 
@@ -77,11 +76,19 @@ void ZCollectedHeap::initialize_serviceability() {
   _heap.serviceability_initialize();
 }
 
+class ZStopConcurrentGCThreadClosure : public ThreadClosure {
+public:
+  virtual void do_thread(Thread* thread) {
+    if (thread->is_ConcurrentGC_thread() &&
+        !thread->is_GC_task_thread()) {
+      static_cast<ConcurrentGCThread*>(thread)->stop();
+    }
+  }
+};
+
 void ZCollectedHeap::stop() {
-  _director->stop();
-  _driver->stop();
-  _uncommitter->stop();
-  _stat->stop();
+  ZStopConcurrentGCThreadClosure cl;
+  gc_threads_do(&cl);
 }
 
 SoftRefPolicy* ZCollectedHeap::soft_ref_policy() {
@@ -198,10 +205,6 @@ void ZCollectedHeap::do_full_collection(bool clear_all_soft_refs) {
   ShouldNotReachHere();
 }
 
-bool ZCollectedHeap::supports_tlab_allocation() const {
-  return true;
-}
-
 size_t ZCollectedHeap::tlab_capacity(Thread* ignored) const {
   return _heap.tlab_capacity();
 }
@@ -218,22 +221,10 @@ size_t ZCollectedHeap::unsafe_max_tlab_alloc(Thread* ignored) const {
   return _heap.unsafe_max_tlab_alloc();
 }
 
-bool ZCollectedHeap::can_elide_tlab_store_barriers() const {
-  return false;
-}
-
-bool ZCollectedHeap::can_elide_initializing_store_barrier(oop new_obj) {
-  // Not supported
-  ShouldNotReachHere();
+bool ZCollectedHeap::uses_stack_watermark_barrier() const {
   return true;
 }
 
-bool ZCollectedHeap::card_mark_must_follow_store() const {
-  // Not supported
-  ShouldNotReachHere();
-  return false;
-}
-
 GrowableArray<GCMemoryManager*> ZCollectedHeap::memory_managers() {
   return GrowableArray<GCMemoryManager*>(1, 1, _heap.serviceability_memory_manager());
 }
@@ -246,8 +237,12 @@ void ZCollectedHeap::object_iterate(ObjectClosure* cl) {
   _heap.object_iterate(cl, true /* visit_weaks */);
 }
 
-void ZCollectedHeap::safe_object_iterate(ObjectClosure* cl) {
-  _heap.object_iterate(cl, true /* visit_weaks */);
+ParallelObjectIterator* ZCollectedHeap::parallel_object_iterator(uint nworkers) {
+  return _heap.parallel_object_iterator(nworkers, true /* visit_weaks */);
+}
+
+void ZCollectedHeap::keep_alive(oop obj) {
+  _heap.keep_alive(obj);
 }
 
 void ZCollectedHeap::register_nmethod(nmethod* nm) {
@@ -266,20 +261,15 @@ void ZCollectedHeap::verify_nmethod(nmethod* nm) {
   // Does nothing
 }
 
-WorkGang* ZCollectedHeap::get_safepoint_workers() {
+WorkGang* ZCollectedHeap::safepoint_workers() {
   return _runtime_workers.workers();
 }
 
-jlong ZCollectedHeap::millis_since_last_gc() {
-  return ZStatCycle::time_since_last() / MILLIUNITS;
-}
-
 void ZCollectedHeap::gc_threads_do(ThreadClosure* tc) const {
   tc->do_thread(_director);
   tc->do_thread(_driver);
-  tc->do_thread(_uncommitter);
   tc->do_thread(_stat);
-  _heap.worker_threads_do(tc);
+  _heap.threads_do(tc);
   _runtime_workers.threads_do(tc);
 }
 
@@ -304,39 +294,30 @@ void ZCollectedHeap::print_on(outputStream* st) const {
 }
 
 void ZCollectedHeap::print_on_error(outputStream* st) const {
+  st->print_cr("ZGC Globals:");
+  st->print_cr(" GlobalPhase:       %u (%s)", ZGlobalPhase, ZGlobalPhaseToString());
+  st->print_cr(" GlobalSeqNum:      %u", ZGlobalSeqNum);
+  st->print_cr(" Offset Max:        " SIZE_FORMAT "%s (" PTR_FORMAT ")",
+               byte_size_in_exact_unit(ZAddressOffsetMax),
+               exact_unit_for_byte_size(ZAddressOffsetMax),
+               ZAddressOffsetMax);
+  st->print_cr(" Page Size Small:   " SIZE_FORMAT "M", ZPageSizeSmall / M);
+  st->print_cr(" Page Size Medium:  " SIZE_FORMAT "M", ZPageSizeMedium / M);
+  st->cr();
+  st->print_cr("ZGC Metadata Bits:");
+  st->print_cr(" Good:              " PTR_FORMAT, ZAddressGoodMask);
+  st->print_cr(" Bad:               " PTR_FORMAT, ZAddressBadMask);
+  st->print_cr(" WeakBad:           " PTR_FORMAT, ZAddressWeakBadMask);
+  st->print_cr(" Marked:            " PTR_FORMAT, ZAddressMetadataMarked);
+  st->print_cr(" Remapped:          " PTR_FORMAT, ZAddressMetadataRemapped);
+  st->cr();
   CollectedHeap::print_on_error(st);
-
-  st->print_cr( "Heap");
-  st->print_cr( "     GlobalPhase:       %u", ZGlobalPhase);
-  st->print_cr( "     GlobalSeqNum:      %u", ZGlobalSeqNum);
-  st->print_cr( "     Offset Max:        " SIZE_FORMAT_W(-15) " (" PTR_FORMAT ")", ZAddressOffsetMax, ZAddressOffsetMax);
-  st->print_cr( "     Page Size Small:   " SIZE_FORMAT_W(-15) " (" PTR_FORMAT ")", ZPageSizeSmall, ZPageSizeSmall);
-  st->print_cr( "     Page Size Medium:  " SIZE_FORMAT_W(-15) " (" PTR_FORMAT ")", ZPageSizeMedium, ZPageSizeMedium);
-  st->print_cr( "Metadata Bits");
-  st->print_cr( "     Good:              " PTR_FORMAT, ZAddressGoodMask);
-  st->print_cr( "     Bad:               " PTR_FORMAT, ZAddressBadMask);
-  st->print_cr( "     WeakBad:           " PTR_FORMAT, ZAddressWeakBadMask);
-  st->print_cr( "     Marked:            " PTR_FORMAT, ZAddressMetadataMarked);
-  st->print_cr( "     Remapped:          " PTR_FORMAT, ZAddressMetadataRemapped);
 }
 
 void ZCollectedHeap::print_extended_on(outputStream* st) const {
   _heap.print_extended_on(st);
 }
 
-void ZCollectedHeap::print_gc_threads_on(outputStream* st) const {
-  _director->print_on(st);
-  st->cr();
-  _driver->print_on(st);
-  st->cr();
-  _uncommitter->print_on(st);
-  st->cr();
-  _stat->print_on(st);
-  st->cr();
-  _heap.print_worker_threads_on(st);
-  _runtime_workers.print_threads_on(st);
-}
-
 void ZCollectedHeap::print_tracing_info() const {
   // Does nothing
 }
@@ -352,3 +333,7 @@ void ZCollectedHeap::verify(VerifyOption option /* ignored */) {
 bool ZCollectedHeap::is_oop(oop object) const {
   return _heap.is_oop(ZOop::to_address(object));
 }
+
+bool ZCollectedHeap::supports_concurrent_gc_breakpoints() const {
+  return true;
+}
