@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,8 +26,11 @@
 
 #include "gc/z/zAttachedArray.hpp"
 #include "gc/z/zForwardingEntry.hpp"
+#include "gc/z/zLock.hpp"
 #include "gc/z/zVirtualMemory.hpp"
 
+class ObjectClosure;
+class ZForwardingAllocator;
 class ZPage;
 
 typedef size_t ZForwardingCursor;
@@ -39,15 +42,13 @@ class ZForwarding {
 private:
   typedef ZAttachedArray<ZForwarding, ZForwardingEntry> AttachedArray;
 
-  const ZVirtualMemory _virtual;
-  const size_t         _object_alignment_shift;
-  const AttachedArray  _entries;
-  ZPage*               _page;
-  volatile uint32_t    _refcount;
-  volatile bool        _pinned;
-
-  bool inc_refcount();
-  bool dec_refcount();
+  const ZVirtualMemory   _virtual;
+  const size_t           _object_alignment_shift;
+  const AttachedArray    _entries;
+  ZPage*                 _page;
+  mutable ZConditionLock _ref_lock;
+  volatile int32_t       _ref_count;
+  bool                   _in_place;
 
   ZForwardingEntry* entries() const;
   ZForwardingEntry at(ZForwardingCursor* cursor) const;
@@ -57,21 +58,24 @@ private:
   ZForwarding(ZPage* page, size_t nentries);
 
 public:
-  static ZForwarding* create(ZPage* page);
-  static void destroy(ZForwarding* forwarding);
+  static uint32_t nentries(const ZPage* page);
+  static ZForwarding* alloc(ZForwardingAllocator* allocator, ZPage* page);
 
+  uint8_t type() const;
   uintptr_t start() const;
   size_t size() const;
   size_t object_alignment_shift() const;
-  ZPage* page() const;
-
-  bool is_pinned() const;
-  void set_pinned();
+  void object_iterate(ObjectClosure *cl);
 
   bool retain_page();
+  ZPage* claim_page();
   void release_page();
+  void wait_page_released() const;
+  ZPage* detach_page();
+
+  void set_in_place();
+  bool in_place() const;
 
-  ZForwardingEntry find(uintptr_t from_index) const;
   ZForwardingEntry find(uintptr_t from_index, ZForwardingCursor* cursor) const;
   uintptr_t insert(uintptr_t from_index, uintptr_t to_offset, ZForwardingCursor* cursor);
 
