@@ -46,7 +46,7 @@ inline T ZFuture<T>::get() {
   // Wait for notification
   Thread* const thread = Thread::current();
   if (thread->is_Java_thread()) {
-    _sema.wait_with_safepoint_check((JavaThread*)thread);
+    _sema.wait_with_safepoint_check(thread->as_Java_thread());
   } else {
     _sema.wait();
   }
