@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -24,30 +24,30 @@
 #include "precompiled.hpp"
 #include "gc/shared/locationPrinter.hpp"
 #include "gc/z/zAddress.inline.hpp"
+#include "gc/z/zArray.inline.hpp"
 #include "gc/z/zGlobals.hpp"
 #include "gc/z/zHeap.inline.hpp"
 #include "gc/z/zHeapIterator.hpp"
+#include "gc/z/zHeuristics.hpp"
 #include "gc/z/zMark.inline.hpp"
 #include "gc/z/zPage.inline.hpp"
 #include "gc/z/zPageTable.inline.hpp"
 #include "gc/z/zRelocationSet.inline.hpp"
-#include "gc/z/zRelocationSetSelector.hpp"
+#include "gc/z/zRelocationSetSelector.inline.hpp"
 #include "gc/z/zResurrection.hpp"
 #include "gc/z/zStat.hpp"
-#include "gc/z/zThread.hpp"
+#include "gc/z/zThread.inline.hpp"
 #include "gc/z/zVerify.hpp"
 #include "gc/z/zWorkers.inline.hpp"
 #include "logging/log.hpp"
 #include "memory/iterator.hpp"
 #include "memory/resourceArea.hpp"
+#include "prims/jvmtiTagMap.hpp"
+#include "runtime/handshake.hpp"
 #include "runtime/safepoint.hpp"
 #include "runtime/thread.hpp"
 #include "utilities/debug.hpp"
 
-static const ZStatSampler ZSamplerHeapUsedBeforeMark("Memory", "Heap Used Before Mark", ZStatUnitBytes);
-static const ZStatSampler ZSamplerHeapUsedAfterMark("Memory", "Heap Used After Mark", ZStatUnitBytes);
-static const ZStatSampler ZSamplerHeapUsedBeforeRelocation("Memory", "Heap Used Before Relocation", ZStatUnitBytes);
-static const ZStatSampler ZSamplerHeapUsedAfterRelocation("Memory", "Heap Used After Relocation", ZStatUnitBytes);
 static const ZStatCounter ZCounterUndoPageAllocation("Memory", "Undo Page Allocation", ZStatUnitOpsPerSecond);
 static const ZStatCounter ZCounterOutOfMemory("Memory", "Out Of Memory", ZStatUnitOpsPerSecond);
 
@@ -56,41 +56,22 @@ ZHeap* ZHeap::_heap = NULL;
 ZHeap::ZHeap() :
     _workers(),
     _object_allocator(),
-    _page_allocator(heap_min_size(), heap_initial_size(), heap_max_size(), heap_max_reserve_size()),
+    _page_allocator(&_workers, MinHeapSize, InitialHeapSize, MaxHeapSize),
     _page_table(),
     _forwarding_table(),
     _mark(&_workers, &_page_table),
     _reference_processor(&_workers),
     _weak_roots_processor(&_workers),
     _relocate(&_workers),
-    _relocation_set(),
+    _relocation_set(&_workers),
     _unload(&_workers),
-    _serviceability(heap_min_size(), heap_max_size()) {
+    _serviceability(min_capacity(), max_capacity()) {
   // Install global heap instance
   assert(_heap == NULL, "Already initialized");
   _heap = this;
 
   // Update statistics
-  ZStatHeap::set_at_initialize(heap_min_size(), heap_max_size(), heap_max_reserve_size());
-}
-
-size_t ZHeap::heap_min_size() const {
-  return MinHeapSize;
-}
-
-size_t ZHeap::heap_initial_size() const {
-  return InitialHeapSize;
-}
-
-size_t ZHeap::heap_max_size() const {
-  return MaxHeapSize;
-}
-
-size_t ZHeap::heap_max_reserve_size() const {
-  // Reserve one small page per worker plus one shared medium page. This is still just
-  // an estimate and doesn't guarantee that we can't run out of memory during relocation.
-  const size_t max_reserve_size = (_workers.nworkers() * ZPageSizeSmall) + ZPageSizeMedium;
-  return MIN2(max_reserve_size, heap_max_size());
+  ZStatHeap::set_at_initialize(_page_allocator.stats());
 }
 
 bool ZHeap::is_initialized() const {
@@ -113,18 +94,6 @@ size_t ZHeap::capacity() const {
   return _page_allocator.capacity();
 }
 
-size_t ZHeap::max_reserve() const {
-  return _page_allocator.max_reserve();
-}
-
-size_t ZHeap::used_high() const {
-  return _page_allocator.used_high();
-}
-
-size_t ZHeap::used_low() const {
-  return _page_allocator.used_low();
-}
-
 size_t ZHeap::used() const {
   return _page_allocator.used();
 }
@@ -133,14 +102,6 @@ size_t ZHeap::unused() const {
   return _page_allocator.unused();
 }
 
-size_t ZHeap::allocated() const {
-  return _page_allocator.allocated();
-}
-
-size_t ZHeap::reclaimed() const {
-  return _page_allocator.reclaimed();
-}
-
 size_t ZHeap::tlab_capacity() const {
   return capacity();
 }
@@ -197,14 +158,11 @@ void ZHeap::set_boost_worker_threads(bool boost) {
   _workers.set_boost(boost);
 }
 
-void ZHeap::worker_threads_do(ThreadClosure* tc) const {
+void ZHeap::threads_do(ThreadClosure* tc) const {
+  _page_allocator.threads_do(tc);
   _workers.threads_do(tc);
 }
 
-void ZHeap::print_worker_threads_on(outputStream* st) const {
-  _workers.print_threads_on(st);
-}
-
 void ZHeap::out_of_memory() {
   ResourceMark rm;
 
@@ -240,46 +198,30 @@ void ZHeap::free_page(ZPage* page, bool reclaimed) {
   _page_allocator.free_page(page, reclaimed);
 }
 
-uint64_t ZHeap::uncommit(uint64_t delay) {
-  return _page_allocator.uncommit(delay);
-}
-
-void ZHeap::before_flip() {
-  if (ZVerifyViews) {
-    // Unmap all pages
-    _page_allocator.debug_unmap_all_pages();
+void ZHeap::free_pages(const ZArray<ZPage*>* pages, bool reclaimed) {
+  // Remove page table entries
+  ZArrayIterator<ZPage*> iter(pages);
+  for (ZPage* page; iter.next(&page);) {
+    _page_table.remove(page);
   }
-}
 
-void ZHeap::after_flip() {
-  if (ZVerifyViews) {
-    // Map all pages
-    ZPageTableIterator iter(&_page_table);
-    for (ZPage* page; iter.next(&page);) {
-      _page_allocator.debug_map_page(page);
-    }
-    _page_allocator.debug_map_cached_pages();
-  }
+  // Free pages
+  _page_allocator.free_pages(pages, reclaimed);
 }
 
 void ZHeap::flip_to_marked() {
-  before_flip();
+  ZVerifyViewsFlip flip(&_page_allocator);
   ZAddress::flip_to_marked();
-  after_flip();
 }
 
 void ZHeap::flip_to_remapped() {
-  before_flip();
+  ZVerifyViewsFlip flip(&_page_allocator);
   ZAddress::flip_to_remapped();
-  after_flip();
 }
 
 void ZHeap::mark_start() {
   assert(SafepointSynchronize::is_at_safepoint(), "Should be at safepoint");
 
-  // Update statistics
-  ZStatSample(ZSamplerHeapUsedBeforeMark, used());
-
   // Flip address view
   flip_to_marked();
 
@@ -299,7 +241,7 @@ void ZHeap::mark_start() {
   _mark.start();
 
   // Update statistics
-  ZStatHeap::set_at_mark_start(soft_max_capacity(), capacity(), used());
+  ZStatHeap::set_at_mark_start(_page_allocator.stats());
 }
 
 void ZHeap::mark(bool initial) {
@@ -326,38 +268,63 @@ bool ZHeap::mark_end() {
   ZVerify::after_mark();
 
   // Update statistics
-  ZStatSample(ZSamplerHeapUsedAfterMark, used());
-  ZStatHeap::set_at_mark_end(capacity(), allocated(), used());
+  ZStatHeap::set_at_mark_end(_page_allocator.stats());
 
   // Block resurrection of weak/phantom references
   ZResurrection::block();
 
-  // Process weak roots
-  _weak_roots_processor.process_weak_roots();
-
-  // Prepare to unload unused classes and code
+  // Prepare to unload stale metadata and nmethods
   _unload.prepare();
 
+  // Notify JVMTI that some tagmap entry objects may have died.
+  JvmtiTagMap::set_needs_cleaning();
+
   return true;
 }
 
+void ZHeap::keep_alive(oop obj) {
+  ZBarrier::keep_alive_barrier_on_oop(obj);
+}
+
 void ZHeap::set_soft_reference_policy(bool clear) {
   _reference_processor.set_soft_reference_policy(clear);
 }
 
+class ZRendezvousClosure : public HandshakeClosure {
+public:
+  ZRendezvousClosure() :
+      HandshakeClosure("ZRendezvous") {}
+
+  void do_thread(Thread* thread) {}
+};
+
 void ZHeap::process_non_strong_references() {
   // Process Soft/Weak/Final/PhantomReferences
   _reference_processor.process_references();
 
-  // Process concurrent weak roots
-  _weak_roots_processor.process_concurrent_weak_roots();
+  // Process weak roots
+  _weak_roots_processor.process_weak_roots();
 
-  // Unload unused classes and code
-  _unload.unload();
+  // Unlink stale metadata and nmethods
+  _unload.unlink();
+
+  // Perform a handshake. This is needed 1) to make sure that stale
+  // metadata and nmethods are no longer observable. And 2), to
+  // prevent the race where a mutator first loads an oop, which is
+  // logically null but not yet cleared. Then this oop gets cleared
+  // by the reference processor and resurrection is unblocked. At
+  // this point the mutator could see the unblocked state and pass
+  // this invalid oop through the normal barrier path, which would
+  // incorrectly try to mark the oop.
+  ZRendezvousClosure cl;
+  Handshake::execute(&cl);
 
   // Unblock resurrection of weak/phantom references
   ZResurrection::unblock();
 
+  // Purge stale metadata and nmethods that were unlinked
+  _unload.purge();
+
   // Enqueue Soft/Weak/Final/PhantomReferences. Note that this
   // must be done after unblocking resurrection. Otherwise the
   // Finalizer thread could call Reference.get() on the Finalizers
@@ -367,6 +334,16 @@ void ZHeap::process_non_strong_references() {
   _reference_processor.enqueue_references();
 }
 
+void ZHeap::free_empty_pages(ZRelocationSetSelector* selector, int bulk) {
+  // Freeing empty pages in bulk is an optimization to avoid grabbing
+  // the page allocator lock, and trying to satisfy stalled allocations
+  // too frequently.
+  if (selector->should_free_empty_pages(bulk)) {
+    free_pages(selector->empty_pages(), true /* reclaimed */);
+    selector->clear_empty_pages();
+  }
+}
+
 void ZHeap::select_relocation_set() {
   // Do not allow pages to be deleted
   _page_allocator.enable_deferred_delete();
@@ -384,19 +361,25 @@ void ZHeap::select_relocation_set() {
       // Register live page
       selector.register_live_page(page);
     } else {
-      // Register garbage page
-      selector.register_garbage_page(page);
+      // Register empty page
+      selector.register_empty_page(page);
 
-      // Reclaim page immediately
-      free_page(page, true /* reclaimed */);
+      // Reclaim empty pages in bulk
+      free_empty_pages(&selector, 64 /* bulk */);
     }
   }
 
+  // Reclaim remaining empty pages
+  free_empty_pages(&selector, 0 /* bulk */);
+
   // Allow pages to be deleted
   _page_allocator.disable_deferred_delete();
 
-  // Select pages to relocate
-  selector.select(&_relocation_set);
+  // Select relocation set
+  selector.select();
+
+  // Install relocation set
+  _relocation_set.install(&selector);
 
   // Setup forwarding table
   ZRelocationSetIterator rs_iter(&_relocation_set);
@@ -405,10 +388,8 @@ void ZHeap::select_relocation_set() {
   }
 
   // Update statistics
-  ZStatRelocation::set_at_select_relocation_set(selector.relocating());
-  ZStatHeap::set_at_select_relocation_set(selector.live(),
-                                          selector.garbage(),
-                                          reclaimed());
+  ZStatRelocation::set_at_select_relocation_set(selector.stats());
+  ZStatHeap::set_at_select_relocation_set(selector.stats());
 }
 
 void ZHeap::reset_relocation_set() {
@@ -425,7 +406,7 @@ void ZHeap::reset_relocation_set() {
 void ZHeap::relocate_start() {
   assert(SafepointSynchronize::is_at_safepoint(), "Should be at safepoint");
 
-  // Finish unloading of classes and code
+  // Finish unloading stale metadata and nmethods
   _unload.finish();
 
   // Flip address view
@@ -435,29 +416,37 @@ void ZHeap::relocate_start() {
   ZGlobalPhase = ZPhaseRelocate;
 
   // Update statistics
-  ZStatSample(ZSamplerHeapUsedBeforeRelocation, used());
-  ZStatHeap::set_at_relocate_start(capacity(), allocated(), used());
+  ZStatHeap::set_at_relocate_start(_page_allocator.stats());
 
-  // Remap/Relocate roots
-  _relocate.start();
+  // Notify JVMTI
+  JvmtiTagMap::set_needs_rehashing();
 }
 
 void ZHeap::relocate() {
   // Relocate relocation set
-  const bool success = _relocate.relocate(&_relocation_set);
+  _relocate.relocate(&_relocation_set);
 
   // Update statistics
-  ZStatSample(ZSamplerHeapUsedAfterRelocation, used());
-  ZStatRelocation::set_at_relocate_end(success);
-  ZStatHeap::set_at_relocate_end(capacity(), allocated(), reclaimed(),
-                                 used(), used_high(), used_low());
+  ZStatHeap::set_at_relocate_end(_page_allocator.stats());
 }
 
 void ZHeap::object_iterate(ObjectClosure* cl, bool visit_weaks) {
   assert(SafepointSynchronize::is_at_safepoint(), "Should be at safepoint");
+  ZHeapIterator iter(1 /* nworkers */, visit_weaks);
+  iter.object_iterate(cl, 0 /* worker_id */);
+}
 
-  ZHeapIterator iter;
-  iter.objects_do(cl, visit_weaks);
+ParallelObjectIterator* ZHeap::parallel_object_iterator(uint nworkers, bool visit_weaks) {
+  assert(SafepointSynchronize::is_at_safepoint(), "Should be at safepoint");
+  return new ZHeapIterator(nworkers, visit_weaks);
+}
+
+void ZHeap::pages_do(ZPageClosure* cl) {
+  ZPageTableIterator iter(&_page_table);
+  for (ZPage* page; iter.next(&page);) {
+    cl->do_page(page);
+  }
+  _page_allocator.pages_do(cl);
 }
 
 void ZHeap::serviceability_initialize() {
@@ -492,6 +481,7 @@ void ZHeap::print_extended_on(outputStream* st) const {
   _page_allocator.enable_deferred_delete();
 
   // Print all pages
+  st->print_cr("ZGC Page Table:");
   ZPageTableIterator iter(&_page_table);
   for (ZPage* page; iter.next(&page);) {
     page->print_on(st);
@@ -499,8 +489,6 @@ void ZHeap::print_extended_on(outputStream* st) const {
 
   // Allow pages to be deleted
   _page_allocator.enable_deferred_delete();
-
-  st->cr();
 }
 
 bool ZHeap::print_location(outputStream* st, uintptr_t addr) const {
