@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,10 +25,10 @@
 #define SHARE_GC_Z_ZHEAP_HPP
 
 #include "gc/z/zAllocationFlags.hpp"
+#include "gc/z/zArray.hpp"
 #include "gc/z/zForwardingTable.hpp"
 #include "gc/z/zMark.hpp"
 #include "gc/z/zObjectAllocator.hpp"
-#include "gc/z/zPage.hpp"
 #include "gc/z/zPageAllocator.hpp"
 #include "gc/z/zPageTable.hpp"
 #include "gc/z/zReferenceProcessor.hpp"
@@ -39,6 +39,10 @@
 #include "gc/z/zUnload.hpp"
 #include "gc/z/zWorkers.hpp"
 
+class ThreadClosure;
+class ZPage;
+class ZRelocationSetSelector;
+
 class ZHeap {
   friend class VMStructs;
 
@@ -58,19 +62,12 @@ private:
   ZUnload             _unload;
   ZServiceability     _serviceability;
 
-  size_t heap_min_size() const;
-  size_t heap_initial_size() const;
-  size_t heap_max_size() const;
-  size_t heap_max_reserve_size() const;
-
-  void before_flip();
-  void after_flip();
-
   void flip_to_marked();
   void flip_to_remapped();
 
+  void free_empty_pages(ZRelocationSetSelector* selector, int bulk);
+
   void out_of_memory();
-  void fixup_partial_loads();
 
 public:
   static ZHeap* heap();
@@ -84,13 +81,8 @@ public:
   size_t max_capacity() const;
   size_t soft_max_capacity() const;
   size_t capacity() const;
-  size_t max_reserve() const;
-  size_t used_high() const;
-  size_t used_low() const;
   size_t used() const;
   size_t unused() const;
-  size_t allocated() const;
-  size_t reclaimed() const;
 
   size_t tlab_capacity() const;
   size_t tlab_used() const;
@@ -100,12 +92,11 @@ public:
   bool is_in(uintptr_t addr) const;
   uint32_t hash_oop(uintptr_t addr) const;
 
-  // Workers
+  // Threads
   uint nconcurrent_worker_threads() const;
   uint nconcurrent_no_boost_worker_threads() const;
   void set_boost_worker_threads(bool boost);
-  void worker_threads_do(ThreadClosure* tc) const;
-  void print_worker_threads_on(outputStream* st) const;
+  void threads_do(ThreadClosure* tc) const;
 
   // Reference processing
   ReferenceDiscoverer* reference_discoverer();
@@ -118,15 +109,13 @@ public:
   ZPage* alloc_page(uint8_t type, size_t size, ZAllocationFlags flags);
   void undo_alloc_page(ZPage* page);
   void free_page(ZPage* page, bool reclaimed);
-
-  // Uncommit memory
-  uint64_t uncommit(uint64_t delay);
+  void free_pages(const ZArray<ZPage*>* pages, bool reclaimed);
 
   // Object allocation
   uintptr_t alloc_tlab(size_t size);
   uintptr_t alloc_object(size_t size);
-  uintptr_t alloc_object_for_relocation(size_t size);
-  void undo_alloc_object_for_relocation(uintptr_t addr, size_t size);
+  uintptr_t alloc_object_non_blocking(size_t size);
+  void undo_alloc_object(uintptr_t addr, size_t size);
   bool is_alloc_stalled() const;
   void check_out_of_memory();
 
@@ -138,6 +127,7 @@ public:
   void mark(bool initial);
   void mark_flush_and_free(Thread* thread);
   bool mark_end();
+  void keep_alive(oop obj);
 
   // Relocation set
   void select_relocation_set();
@@ -151,6 +141,8 @@ public:
 
   // Iteration
   void object_iterate(ObjectClosure* cl, bool visit_weaks);
+  ParallelObjectIterator* parallel_object_iterator(uint nworkers, bool visit_weaks);
+  void pages_do(ZPageClosure* cl);
 
   // Serviceability
   void serviceability_initialize();
