@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -53,7 +53,7 @@ inline void ZReentrantLock::lock() {
 
   if (owner != thread) {
     _lock.lock();
-    Atomic::store(thread, &_owner);
+    Atomic::store(&_owner, thread);
   }
 
   _count++;
@@ -66,7 +66,7 @@ inline void ZReentrantLock::unlock() {
   _count--;
 
   if (_count == 0) {
-    Atomic::store((Thread*)NULL, &_owner);
+    Atomic::store(&_owner, (Thread*)NULL);
     _lock.unlock();
   }
 }
@@ -77,15 +77,43 @@ inline bool ZReentrantLock::is_owned() const {
   return owner == thread;
 }
 
+inline void ZConditionLock::lock() {
+  _lock.lock();
+}
+
+inline bool ZConditionLock::try_lock() {
+  return _lock.try_lock();
+}
+
+inline void ZConditionLock::unlock() {
+  _lock.unlock();
+}
+
+inline bool ZConditionLock::wait(uint64_t millis) {
+  return _lock.wait(millis) == OS_OK;
+}
+
+inline void ZConditionLock::notify() {
+  _lock.notify();
+}
+
+inline void ZConditionLock::notify_all() {
+  _lock.notify_all();
+}
+
 template <typename T>
 inline ZLocker<T>::ZLocker(T* lock) :
     _lock(lock) {
-  _lock->lock();
+  if (_lock != NULL) {
+    _lock->lock();
+  }
 }
 
 template <typename T>
 inline ZLocker<T>::~ZLocker() {
-  _lock->unlock();
+  if (_lock != NULL) {
+    _lock->unlock();
+  }
 }
 
 #endif // SHARE_GC_Z_ZLOCK_INLINE_HPP
