@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,6 +26,7 @@
 
 #include "gc/z/zMarkStack.hpp"
 #include "gc/z/zMarkStackAllocator.hpp"
+#include "gc/z/zMarkStackEntry.hpp"
 #include "gc/z/zMarkTerminate.hpp"
 #include "oops/oopsHierarchy.hpp"
 #include "utilities/globalDefinitions.hpp"
@@ -36,9 +37,7 @@ class ZPageTable;
 class ZWorkers;
 
 class ZMark {
-  friend class ZMarkRootsTask;
   friend class ZMarkTask;
-  friend class ZMarkTryCompleteTask;
 
 private:
   ZWorkers* const     _workers;
@@ -56,7 +55,6 @@ private:
   uint                _nworkers;
 
   size_t calculate_nstripes(uint nworkers) const;
-  void prepare_mark();
 
   bool is_array(uintptr_t addr) const;
   void push_partial_array(uintptr_t addr, size_t size, bool finalizable);
@@ -95,8 +93,8 @@ private:
   void work_with_timeout(ZMarkCache* cache,
                          ZMarkStripe* stripe,
                          ZMarkThreadLocalStacks* stacks,
-                         uint64_t timeout_in_millis);
-  void work(uint64_t timeout_in_millis);
+                         uint64_t timeout_in_micros);
+  void work(uint64_t timeout_in_micros);
 
   void verify_all_stacks_empty() const;
 
@@ -105,7 +103,7 @@ public:
 
   bool is_initialized() const;
 
-  template <bool finalizable, bool publish> void mark_object(uintptr_t addr);
+  template <bool follow, bool finalizable, bool publish> void mark_object(uintptr_t addr);
 
   void start();
   void mark(bool initial);
