@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,19 +22,17 @@
  */
 
 #include "precompiled.hpp"
-#include "gc/z/zCollectedHeap.hpp"
 #include "gc/z/zGlobals.hpp"
 #include "gc/z/zHeap.inline.hpp"
+#include "gc/z/zHeuristics.hpp"
 #include "gc/z/zObjectAllocator.hpp"
 #include "gc/z/zPage.inline.hpp"
 #include "gc/z/zStat.hpp"
-#include "gc/z/zThread.hpp"
-#include "gc/z/zUtils.inline.hpp"
+#include "gc/z/zThread.inline.hpp"
+#include "gc/z/zValue.inline.hpp"
 #include "logging/log.hpp"
 #include "runtime/atomic.hpp"
 #include "runtime/safepoint.hpp"
-#include "runtime/thread.hpp"
-#include "runtime/threadSMR.hpp"
 #include "utilities/align.hpp"
 #include "utilities/debug.hpp"
 
@@ -42,17 +40,25 @@ static const ZStatCounter ZCounterUndoObjectAllocationSucceeded("Memory", "Undo
 static const ZStatCounter ZCounterUndoObjectAllocationFailed("Memory", "Undo Object Allocation Failed", ZStatUnitOpsPerSecond);
 
 ZObjectAllocator::ZObjectAllocator() :
+    _use_per_cpu_shared_small_pages(ZHeuristics::use_per_cpu_shared_small_pages()),
     _used(0),
     _undone(0),
     _shared_medium_page(NULL),
-    _shared_small_page(NULL),
-    _worker_small_page(NULL) {}
+    _shared_small_page(NULL) {}
+
+ZPage** ZObjectAllocator::shared_small_page_addr() {
+  return _use_per_cpu_shared_small_pages ? _shared_small_page.addr() : _shared_small_page.addr(0);
+}
+
+ZPage* const* ZObjectAllocator::shared_small_page_addr() const {
+  return _use_per_cpu_shared_small_pages ? _shared_small_page.addr() : _shared_small_page.addr(0);
+}
 
 ZPage* ZObjectAllocator::alloc_page(uint8_t type, size_t size, ZAllocationFlags flags) {
   ZPage* const page = ZHeap::heap()->alloc_page(type, size, flags);
   if (page != NULL) {
     // Increment used bytes
-    Atomic::add(size, _used.addr());
+    Atomic::add(_used.addr(), size);
   }
 
   return page;
@@ -60,7 +66,7 @@ ZPage* ZObjectAllocator::alloc_page(uint8_t type, size_t size, ZAllocationFlags
 
 void ZObjectAllocator::undo_alloc_page(ZPage* page) {
   // Increment undone bytes
-  Atomic::add(page->size(), _undone.addr());
+  Atomic::add(_undone.addr(), page->size());
 
   ZHeap::heap()->undo_alloc_page(page);
 }
@@ -71,7 +77,7 @@ uintptr_t ZObjectAllocator::alloc_object_in_shared_page(ZPage** shared_page,
                                                         size_t size,
                                                         ZAllocationFlags flags) {
   uintptr_t addr = 0;
-  ZPage* page = *shared_page;
+  ZPage* page = Atomic::load_acquire(shared_page);
 
   if (page != NULL) {
     addr = page->alloc_object_atomic(size);
@@ -86,7 +92,7 @@ uintptr_t ZObjectAllocator::alloc_object_in_shared_page(ZPage** shared_page,
 
     retry:
       // Install new page
-      ZPage* const prev_page = Atomic::cmpxchg(new_page, shared_page, page);
+      ZPage* const prev_page = Atomic::cmpxchg(shared_page, page, new_page);
       if (prev_page != page) {
         if (prev_page == NULL) {
           // Previous page was retired, retry installing the new page
@@ -115,8 +121,6 @@ uintptr_t ZObjectAllocator::alloc_object_in_shared_page(ZPage** shared_page,
 }
 
 uintptr_t ZObjectAllocator::alloc_large_object(size_t size, ZAllocationFlags flags) {
-  assert(ZThread::is_java(), "Should be a Java thread");
-
   uintptr_t addr = 0;
 
   // Allocate new large page
@@ -134,44 +138,8 @@ uintptr_t ZObjectAllocator::alloc_medium_object(size_t size, ZAllocationFlags fl
   return alloc_object_in_shared_page(_shared_medium_page.addr(), ZPageTypeMedium, ZPageSizeMedium, size, flags);
 }
 
-uintptr_t ZObjectAllocator::alloc_small_object_from_nonworker(size_t size, ZAllocationFlags flags) {
-  assert(ZThread::is_java() || ZThread::is_vm() || ZThread::is_runtime_worker(),
-         "Should be a Java, VM or Runtime worker thread");
-
-  // Non-worker small page allocation can never use the reserve
-  flags.set_no_reserve();
-
-  return alloc_object_in_shared_page(_shared_small_page.addr(), ZPageTypeSmall, ZPageSizeSmall, size, flags);
-}
-
-uintptr_t ZObjectAllocator::alloc_small_object_from_worker(size_t size, ZAllocationFlags flags) {
-  assert(ZThread::is_worker(), "Should be a worker thread");
-
-  ZPage* page = _worker_small_page.get();
-  uintptr_t addr = 0;
-
-  if (page != NULL) {
-    addr = page->alloc_object(size);
-  }
-
-  if (addr == 0) {
-    // Allocate new page
-    page = alloc_page(ZPageTypeSmall, ZPageSizeSmall, flags);
-    if (page != NULL) {
-      addr = page->alloc_object(size);
-    }
-    _worker_small_page.set(page);
-  }
-
-  return addr;
-}
-
 uintptr_t ZObjectAllocator::alloc_small_object(size_t size, ZAllocationFlags flags) {
-  if (flags.worker_thread()) {
-    return alloc_small_object_from_worker(size, flags);
-  } else {
-    return alloc_small_object_from_nonworker(size, flags);
-  }
+  return alloc_object_in_shared_page(shared_small_page_addr(), ZPageTypeSmall, ZPageSizeSmall, size, flags);
 }
 
 uintptr_t ZObjectAllocator::alloc_object(size_t size, ZAllocationFlags flags) {
@@ -188,88 +156,28 @@ uintptr_t ZObjectAllocator::alloc_object(size_t size, ZAllocationFlags flags) {
 }
 
 uintptr_t ZObjectAllocator::alloc_object(size_t size) {
-  assert(ZThread::is_java(), "Must be a Java thread");
-
   ZAllocationFlags flags;
-  flags.set_no_reserve();
-
   return alloc_object(size, flags);
 }
 
-uintptr_t ZObjectAllocator::alloc_object_for_relocation(size_t size) {
-  assert(ZThread::is_java() || ZThread::is_vm() || ZThread::is_worker() || ZThread::is_runtime_worker(),
-         "Unknown thread");
-
+uintptr_t ZObjectAllocator::alloc_object_non_blocking(size_t size) {
   ZAllocationFlags flags;
-  flags.set_relocation();
   flags.set_non_blocking();
-
-  if (ZThread::is_worker()) {
-    flags.set_worker_thread();
-  }
-
   return alloc_object(size, flags);
 }
 
-bool ZObjectAllocator::undo_alloc_large_object(ZPage* page) {
-  assert(page->type() == ZPageTypeLarge, "Invalid page type");
-
-  // Undo page allocation
-  undo_alloc_page(page);
-  return true;
-}
-
-bool ZObjectAllocator::undo_alloc_medium_object(ZPage* page, uintptr_t addr, size_t size) {
-  assert(page->type() == ZPageTypeMedium, "Invalid page type");
-
-  // Try atomic undo on shared page
-  return page->undo_alloc_object_atomic(addr, size);
-}
-
-bool ZObjectAllocator::undo_alloc_small_object_from_nonworker(ZPage* page, uintptr_t addr, size_t size) {
-  assert(page->type() == ZPageTypeSmall, "Invalid page type");
-
-  // Try atomic undo on shared page
-  return page->undo_alloc_object_atomic(addr, size);
-}
-
-bool ZObjectAllocator::undo_alloc_small_object_from_worker(ZPage* page, uintptr_t addr, size_t size) {
-  assert(page->type() == ZPageTypeSmall, "Invalid page type");
-  assert(page == _worker_small_page.get(), "Invalid page");
-
-  // Non-atomic undo on worker-local page
-  const bool success = page->undo_alloc_object(addr, size);
-  assert(success, "Should always succeed");
-  return success;
-}
-
-bool ZObjectAllocator::undo_alloc_small_object(ZPage* page, uintptr_t addr, size_t size) {
-  if (ZThread::is_worker()) {
-    return undo_alloc_small_object_from_worker(page, addr, size);
-  } else {
-    return undo_alloc_small_object_from_nonworker(page, addr, size);
-  }
-}
-
-bool ZObjectAllocator::undo_alloc_object(ZPage* page, uintptr_t addr, size_t size) {
+void ZObjectAllocator::undo_alloc_object(ZPage* page, uintptr_t addr, size_t size) {
   const uint8_t type = page->type();
 
-  if (type == ZPageTypeSmall) {
-    return undo_alloc_small_object(page, addr, size);
-  } else if (type == ZPageTypeMedium) {
-    return undo_alloc_medium_object(page, addr, size);
-  } else {
-    return undo_alloc_large_object(page);
-  }
-}
-
-void ZObjectAllocator::undo_alloc_object_for_relocation(ZPage* page, uintptr_t addr, size_t size) {
-  if (undo_alloc_object(page, addr, size)) {
+  if (type == ZPageTypeLarge) {
+    undo_alloc_page(page);
     ZStatInc(ZCounterUndoObjectAllocationSucceeded);
   } else {
-    ZStatInc(ZCounterUndoObjectAllocationFailed);
-    log_trace(gc)("Failed to undo object allocation: " PTR_FORMAT ", Size: " SIZE_FORMAT ", Thread: " PTR_FORMAT " (%s)",
-                  addr, size, ZThread::id(), ZThread::name());
+    if (page->undo_alloc_object_atomic(addr, size)) {
+      ZStatInc(ZCounterUndoObjectAllocationSucceeded);
+    } else {
+      ZStatInc(ZCounterUndoObjectAllocationFailed);
+    }
   }
 }
 
@@ -293,7 +201,7 @@ size_t ZObjectAllocator::used() const {
 size_t ZObjectAllocator::remaining() const {
   assert(ZThread::is_java(), "Should be a Java thread");
 
-  ZPage* page = _shared_small_page.get();
+  const ZPage* const page = Atomic::load_acquire(shared_small_page_addr());
   if (page != NULL) {
     return page->remaining();
   }
@@ -311,5 +219,4 @@ void ZObjectAllocator::retire_pages() {
   // Reset allocation pages
   _shared_medium_page.set(NULL);
   _shared_small_page.set_all(NULL);
-  _worker_small_page.set_all(NULL);
 }
