@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,12 +23,11 @@
 
 #include "precompiled.hpp"
 #include "gc/z/zCollectedHeap.hpp"
-#include "gc/z/zCPU.hpp"
+#include "gc/z/zCPU.inline.hpp"
 #include "gc/z/zGlobals.hpp"
-#include "gc/z/zHeap.inline.hpp"
-#include "gc/z/zLargePages.inline.hpp"
 #include "gc/z/zNMethodTable.hpp"
-#include "gc/z/zNUMA.hpp"
+#include "gc/z/zPageAllocator.inline.hpp"
+#include "gc/z/zRelocationSetSelector.inline.hpp"
 #include "gc/z/zStat.hpp"
 #include "gc/z/zTracer.inline.hpp"
 #include "gc/z/zUtils.hpp"
@@ -37,16 +36,16 @@
 #include "runtime/os.hpp"
 #include "runtime/timer.hpp"
 #include "utilities/align.hpp"
-#include "utilities/compilerWarnings.hpp"
 #include "utilities/debug.hpp"
 #include "utilities/ticks.hpp"
 
-#define ZSIZE_FMT               SIZE_FORMAT "M(%.0f%%)"
-#define ZSIZE_ARGS(size)        ((size) / M), (percent_of(size, ZStatHeap::max_capacity()))
+#define ZSIZE_FMT                       SIZE_FORMAT "M(%.0f%%)"
+#define ZSIZE_ARGS_WITH_MAX(size, max)  ((size) / M), (percent_of(size, max))
+#define ZSIZE_ARGS(size)                ZSIZE_ARGS_WITH_MAX(size, ZStatHeap::max_capacity())
 
-#define ZTABLE_ARGS_NA          "%9s", "-"
-#define ZTABLE_ARGS(size)       SIZE_FORMAT_W(8) "M (%.0f%%)", \
-                                ((size) / M), (percent_of(size, ZStatHeap::max_capacity()))
+#define ZTABLE_ARGS_NA                  "%9s", "-"
+#define ZTABLE_ARGS(size)               SIZE_FORMAT_W(8) "M (%.0f%%)", \
+                                        ((size) / M), (percent_of(size, ZStatHeap::max_capacity()))
 
 //
 // Stat sampler/counter data
@@ -388,24 +387,37 @@ ZStatIterableValue<T>::ZStatIterableValue(const char* group,
 
 template <typename T>
 T* ZStatIterableValue<T>::insert() const {
-  T** current = &_first;
-
-  while (*current != NULL) {
-    // First sort by group, then by name
-    const int group_cmp = strcmp((*current)->group(), group());
-    const int name_cmp = strcmp((*current)->name(), name());
-    if ((group_cmp > 0) || (group_cmp == 0 && name_cmp > 0)) {
-      break;
-    }
-
-    current = &(*current)->_next;
-  }
-
-  T* const next = *current;
-  *current = (T*)this;
+  T* const next = _first;
+  _first = (T*)this;
   return next;
 }
 
+template <typename T>
+void ZStatIterableValue<T>::sort() {
+  T* first_unsorted = _first;
+  _first = NULL;
+
+  while (first_unsorted != NULL) {
+    T* const value = first_unsorted;
+    first_unsorted = value->_next;
+    value->_next = NULL;
+
+    T** current = &_first;
+
+    while (*current != NULL) {
+      // First sort by group, then by name
+      const int group_cmp = strcmp((*current)->group(), value->group());
+      if ((group_cmp > 0) || (group_cmp == 0 && strcmp((*current)->name(), value->name()) > 0)) {
+        break;
+      }
+
+      current = &(*current)->_next;
+    }
+    value->_next = *current;
+    *current = value;
+  }
+}
+
 //
 // Stat sampler
 //
@@ -424,9 +436,9 @@ ZStatSamplerData ZStatSampler::collect_and_reset() const {
   for (uint32_t i = 0; i < ncpus; i++) {
     ZStatSamplerData* const cpu_data = get_cpu_local<ZStatSamplerData>(i);
     if (cpu_data->_nsamples > 0) {
-      const uint64_t nsamples = Atomic::xchg((uint64_t)0, &cpu_data->_nsamples);
-      const uint64_t sum = Atomic::xchg((uint64_t)0, &cpu_data->_sum);
-      const uint64_t max = Atomic::xchg((uint64_t)0, &cpu_data->_max);
+      const uint64_t nsamples = Atomic::xchg(&cpu_data->_nsamples, (uint64_t)0);
+      const uint64_t sum = Atomic::xchg(&cpu_data->_sum, (uint64_t)0);
+      const uint64_t max = Atomic::xchg(&cpu_data->_max, (uint64_t)0);
       all._nsamples += nsamples;
       all._sum += sum;
       if (all._max < max) {
@@ -459,7 +471,7 @@ void ZStatCounter::sample_and_reset() const {
   const uint32_t ncpus = ZCPU::count();
   for (uint32_t i = 0; i < ncpus; i++) {
     ZStatCounterData* const cpu_data = get_cpu_local<ZStatCounterData>(i);
-    counter += Atomic::xchg((uint64_t)0, &cpu_data->_counter);
+    counter += Atomic::xchg(&cpu_data->_counter, (uint64_t)0);
   }
 
   ZStatSample(_sampler, counter);
@@ -481,7 +493,7 @@ ZStatCounterData ZStatUnsampledCounter::collect_and_reset() const {
   const uint32_t ncpus = ZCPU::count();
   for (uint32_t i = 0; i < ncpus; i++) {
     ZStatCounterData* const cpu_data = get_cpu_local<ZStatCounterData>(i);
-    all._counter += Atomic::xchg((uint64_t)0, &cpu_data->_counter);
+    all._counter += Atomic::xchg(&cpu_data->_counter, (uint64_t)0);
   }
 
   return all;
@@ -641,10 +653,10 @@ void ZStatPhaseCycle::register_end(const Ticks& start, const Ticks& end) const {
   ZStatLoad::print();
   ZStatMMU::print();
   ZStatMark::print();
-  ZStatRelocation::print();
   ZStatNMethods::print();
   ZStatMetaspace::print();
   ZStatReferences::print();
+  ZStatRelocation::print();
   ZStatHeap::print();
 
   log_info(gc)("Garbage Collection (%s) " ZSIZE_FMT "->" ZSIZE_FMT,
@@ -716,7 +728,7 @@ void ZStatSubPhase::register_start(const Ticks& start) const {
 }
 
 void ZStatSubPhase::register_end(const Ticks& start, const Ticks& end) const {
-  ZTracer::tracer()->report_thread_phase(*this, start, end);
+  ZTracer::tracer()->report_thread_phase(name(), start, end);
 
   const Tickspan duration = end - start;
   ZStatSample(_sampler, duration.value());
@@ -736,7 +748,7 @@ void ZStatCriticalPhase::register_start(const Ticks& start) const {
 }
 
 void ZStatCriticalPhase::register_end(const Ticks& start, const Ticks& end) const {
-  ZTracer::tracer()->report_thread_phase(*this, start, end);
+  ZTracer::tracer()->report_thread_phase(name(), start, end);
 
   const Tickspan duration = end - start;
   ZStatSample(_sampler, duration.value());
@@ -759,10 +771,10 @@ THREAD_LOCAL uint32_t ZStatTimerDisable::_active = 0;
 //
 // Stat sample/inc
 //
-void ZStatSample(const ZStatSampler& sampler, uint64_t value, bool trace) {
+void ZStatSample(const ZStatSampler& sampler, uint64_t value) {
   ZStatSamplerData* const cpu_data = sampler.get();
-  Atomic::add(1u, &cpu_data->_nsamples);
-  Atomic::add(value, &cpu_data->_sum);
+  Atomic::add(&cpu_data->_nsamples, 1u);
+  Atomic::add(&cpu_data->_sum, value);
 
   uint64_t max = cpu_data->_max;
   for (;;) {
@@ -772,7 +784,7 @@ void ZStatSample(const ZStatSampler& sampler, uint64_t value, bool trace) {
     }
 
     const uint64_t new_max = value;
-    const uint64_t prev_max = Atomic::cmpxchg(new_max, &cpu_data->_max, max);
+    const uint64_t prev_max = Atomic::cmpxchg(&cpu_data->_max, max, new_max);
     if (prev_max == max) {
       // Success
       break;
@@ -782,23 +794,19 @@ void ZStatSample(const ZStatSampler& sampler, uint64_t value, bool trace) {
     max = prev_max;
   }
 
-  if (trace) {
-    ZTracer::tracer()->report_stat_sampler(sampler, value);
-  }
+  ZTracer::tracer()->report_stat_sampler(sampler, value);
 }
 
-void ZStatInc(const ZStatCounter& counter, uint64_t increment, bool trace) {
+void ZStatInc(const ZStatCounter& counter, uint64_t increment) {
   ZStatCounterData* const cpu_data = counter.get();
-  const uint64_t value = Atomic::add(increment, &cpu_data->_counter);
+  const uint64_t value = Atomic::add(&cpu_data->_counter, increment);
 
-  if (trace) {
-    ZTracer::tracer()->report_stat_counter(counter, increment, value);
-  }
+  ZTracer::tracer()->report_stat_counter(counter, increment, value);
 }
 
 void ZStatInc(const ZStatUnsampledCounter& counter, uint64_t increment) {
   ZStatCounterData* const cpu_data = counter.get();
-  Atomic::add(increment, &cpu_data->_counter);
+  Atomic::add(&cpu_data->_counter, increment);
 }
 
 //
@@ -884,6 +892,8 @@ void ZStat::run_service() {
   ZStatSamplerHistory* const history = new ZStatSamplerHistory[ZStatSampler::count()];
   LogTarget(Info, gc, stats) log;
 
+  ZStatSampler::sort();
+
   // Main loop
   while (_metronome.wait_for_tick()) {
     sample_and_collect(history);
@@ -1028,18 +1038,21 @@ public:
 //
 // Stat cycle
 //
-uint64_t  ZStatCycle::_ncycles = 0;
+uint64_t  ZStatCycle::_nwarmup_cycles = 0;
 Ticks     ZStatCycle::_start_of_last;
 Ticks     ZStatCycle::_end_of_last;
-NumberSeq ZStatCycle::_normalized_duration(0.3 /* alpha */);
+NumberSeq ZStatCycle::_normalized_duration(0.7 /* alpha */);
 
 void ZStatCycle::at_start() {
   _start_of_last = Ticks::now();
 }
 
-void ZStatCycle::at_end(double boost_factor) {
+void ZStatCycle::at_end(GCCause::Cause cause, double boost_factor) {
   _end_of_last = Ticks::now();
-  _ncycles++;
+
+  if (cause == GCCause::_z_warmup) {
+    _nwarmup_cycles++;
+  }
 
   // Calculate normalized cycle duration. The measured duration is
   // normalized using the boost factor to avoid artificial deflation
@@ -1049,8 +1062,18 @@ void ZStatCycle::at_end(double boost_factor) {
   _normalized_duration.add(normalized_duration);
 }
 
-uint64_t ZStatCycle::ncycles() {
-  return _ncycles;
+bool ZStatCycle::is_warm() {
+  return _nwarmup_cycles >= 3;
+}
+
+uint64_t ZStatCycle::nwarmup_cycles() {
+  return _nwarmup_cycles;
+}
+
+bool ZStatCycle::is_normalized_duration_trustable() {
+  // The normalized duration is considered trustable if we have
+  // completed at least one warmup cycle
+  return _nwarmup_cycles > 0;
 }
 
 const AbsSeq& ZStatCycle::normalized_duration() {
@@ -1058,8 +1081,8 @@ const AbsSeq& ZStatCycle::normalized_duration() {
 }
 
 double ZStatCycle::time_since_last() {
-  if (_ncycles == 0) {
-    // Return time since VM start-up
+  if (_end_of_last.value() == 0) {
+    // No end recorded yet, return time since VM start
     return os::elapsedTime();
   }
 
@@ -1117,23 +1140,45 @@ void ZStatMark::print() {
 //
 // Stat relocation
 //
-size_t ZStatRelocation::_relocating;
-bool ZStatRelocation::_success;
+ZRelocationSetSelectorStats ZStatRelocation::_selector_stats;
+size_t                      ZStatRelocation::_forwarding_usage;
+size_t                      ZStatRelocation::_small_in_place_count;
+size_t                      ZStatRelocation::_medium_in_place_count;
 
-void ZStatRelocation::set_at_select_relocation_set(size_t relocating) {
-  _relocating = relocating;
+void ZStatRelocation::set_at_select_relocation_set(const ZRelocationSetSelectorStats& selector_stats) {
+  _selector_stats = selector_stats;
 }
 
-void ZStatRelocation::set_at_relocate_end(bool success) {
-  _success = success;
+void ZStatRelocation::set_at_install_relocation_set(size_t forwarding_usage) {
+  _forwarding_usage = forwarding_usage;
+}
+
+void ZStatRelocation::set_at_relocate_end(size_t small_in_place_count, size_t medium_in_place_count) {
+  _small_in_place_count = small_in_place_count;
+  _medium_in_place_count = medium_in_place_count;
+}
+
+void ZStatRelocation::print(const char* name,
+                            const ZRelocationSetSelectorGroupStats& selector_group,
+                            size_t in_place_count) {
+  log_info(gc, reloc)("%s Pages: " SIZE_FORMAT " / " SIZE_FORMAT "M, Empty: " SIZE_FORMAT "M, "
+                      "Relocated: " SIZE_FORMAT "M, In-Place: " SIZE_FORMAT,
+                      name,
+                      selector_group.npages(),
+                      selector_group.total() / M,
+                      selector_group.empty() / M,
+                      selector_group.relocate() / M,
+                      in_place_count);
 }
 
 void ZStatRelocation::print() {
-  if (_success) {
-    log_info(gc, reloc)("Relocation: Successful, " SIZE_FORMAT "M relocated", _relocating / M);
-  } else {
-    log_info(gc, reloc)("Relocation: Incomplete");
+  print("Small", _selector_stats.small(), _small_in_place_count);
+  if (ZPageSizeMedium != 0) {
+    print("Medium", _selector_stats.medium(), _medium_in_place_count);
   }
+  print("Large", _selector_stats.large(), 0 /* in_place_count */);
+
+  log_info(gc, reloc)("Forwarding Usage: " SIZE_FORMAT "M", _forwarding_usage / M);
 }
 
 //
@@ -1150,10 +1195,9 @@ void ZStatNMethods::print() {
 //
 void ZStatMetaspace::print() {
   log_info(gc, metaspace)("Metaspace: "
-                          SIZE_FORMAT "M used, " SIZE_FORMAT "M capacity, "
+                          SIZE_FORMAT "M used, "
                           SIZE_FORMAT "M committed, " SIZE_FORMAT "M reserved",
                           MetaspaceUtils::used_bytes() / M,
-                          MetaspaceUtils::capacity_bytes() / M,
                           MetaspaceUtils::committed_bytes() / M,
                           MetaspaceUtils::reserved_bytes() / M);
 }
@@ -1215,87 +1259,84 @@ ZStatHeap::ZAtMarkEnd ZStatHeap::_at_mark_end;
 ZStatHeap::ZAtRelocateStart ZStatHeap::_at_relocate_start;
 ZStatHeap::ZAtRelocateEnd ZStatHeap::_at_relocate_end;
 
-size_t ZStatHeap::available(size_t used) {
-  return _at_initialize.max_capacity - used;
+size_t ZStatHeap::capacity_high() {
+  return MAX4(_at_mark_start.capacity,
+              _at_mark_end.capacity,
+              _at_relocate_start.capacity,
+              _at_relocate_end.capacity);
 }
 
-size_t ZStatHeap::reserve(size_t used) {
-  return MIN2(_at_initialize.max_reserve, available(used));
+size_t ZStatHeap::capacity_low() {
+  return MIN4(_at_mark_start.capacity,
+              _at_mark_end.capacity,
+              _at_relocate_start.capacity,
+              _at_relocate_end.capacity);
 }
 
 size_t ZStatHeap::free(size_t used) {
-  return available(used) - reserve(used);
+  return _at_initialize.max_capacity - used;
 }
 
-void ZStatHeap::set_at_initialize(size_t min_capacity,
-                                  size_t max_capacity,
-                                  size_t max_reserve) {
-  _at_initialize.min_capacity = min_capacity;
-  _at_initialize.max_capacity = max_capacity;
-  _at_initialize.max_reserve = max_reserve;
+size_t ZStatHeap::allocated(size_t used, size_t reclaimed) {
+  // The amount of allocated memory between point A and B is used(B) - used(A).
+  // However, we might also have reclaimed memory between point A and B. This
+  // means the current amount of used memory must be incremented by the amount
+  // reclaimed, so that used(B) represents the amount of used memory we would
+  // have had if we had not reclaimed anything.
+  return (used + reclaimed) - _at_mark_start.used;
 }
 
-void ZStatHeap::set_at_mark_start(size_t soft_max_capacity,
-                                  size_t capacity,
-                                  size_t used) {
-  _at_mark_start.soft_max_capacity = soft_max_capacity;
-  _at_mark_start.capacity = capacity;
-  _at_mark_start.reserve = reserve(used);
-  _at_mark_start.used = used;
-  _at_mark_start.free = free(used);
+size_t ZStatHeap::garbage(size_t reclaimed) {
+  return _at_mark_end.garbage - reclaimed;
 }
 
-void ZStatHeap::set_at_mark_end(size_t capacity,
-                                size_t allocated,
-                                size_t used) {
-  _at_mark_end.capacity = capacity;
-  _at_mark_end.reserve = reserve(used);
-  _at_mark_end.allocated = allocated;
-  _at_mark_end.used = used;
-  _at_mark_end.free = free(used);
+void ZStatHeap::set_at_initialize(const ZPageAllocatorStats& stats) {
+  _at_initialize.min_capacity = stats.min_capacity();
+  _at_initialize.max_capacity = stats.max_capacity();
 }
 
-void ZStatHeap::set_at_select_relocation_set(size_t live,
-                                             size_t garbage,
-                                             size_t reclaimed) {
+void ZStatHeap::set_at_mark_start(const ZPageAllocatorStats& stats) {
+  _at_mark_start.soft_max_capacity = stats.soft_max_capacity();
+  _at_mark_start.capacity = stats.capacity();
+  _at_mark_start.free = free(stats.used());
+  _at_mark_start.used = stats.used();
+}
+
+void ZStatHeap::set_at_mark_end(const ZPageAllocatorStats& stats) {
+  _at_mark_end.capacity = stats.capacity();
+  _at_mark_end.free = free(stats.used());
+  _at_mark_end.used = stats.used();
+  _at_mark_end.allocated = allocated(stats.used(), 0 /* reclaimed */);
+}
+
+void ZStatHeap::set_at_select_relocation_set(const ZRelocationSetSelectorStats& stats) {
+  const size_t live = stats.small().live() + stats.medium().live() + stats.large().live();
   _at_mark_end.live = live;
-  _at_mark_end.garbage = garbage;
-
-  _at_relocate_start.garbage = garbage - reclaimed;
-  _at_relocate_start.reclaimed = reclaimed;
+  _at_mark_end.garbage = _at_mark_start.used - live;
 }
 
-void ZStatHeap::set_at_relocate_start(size_t capacity,
-                                      size_t allocated,
-                                      size_t used) {
-  _at_relocate_start.capacity = capacity;
-  _at_relocate_start.reserve = reserve(used);
-  _at_relocate_start.allocated = allocated;
-  _at_relocate_start.used = used;
-  _at_relocate_start.free = free(used);
+void ZStatHeap::set_at_relocate_start(const ZPageAllocatorStats& stats) {
+  _at_relocate_start.capacity = stats.capacity();
+  _at_relocate_start.free = free(stats.used());
+  _at_relocate_start.used = stats.used();
+  _at_relocate_start.allocated = allocated(stats.used(), stats.reclaimed());
+  _at_relocate_start.garbage = garbage(stats.reclaimed());
+  _at_relocate_start.reclaimed = stats.reclaimed();
 }
 
-void ZStatHeap::set_at_relocate_end(size_t capacity,
-                                    size_t allocated,
-                                    size_t reclaimed,
-                                    size_t used,
-                                    size_t used_high,
-                                    size_t used_low) {
-  _at_relocate_end.capacity = capacity;
-  _at_relocate_end.capacity_high = capacity;
-  _at_relocate_end.capacity_low = _at_mark_start.capacity;
-  _at_relocate_end.reserve = reserve(used);
-  _at_relocate_end.reserve_high = reserve(used_low);
-  _at_relocate_end.reserve_low = reserve(used_high);
-  _at_relocate_end.garbage = _at_mark_end.garbage - reclaimed;
-  _at_relocate_end.allocated = allocated;
-  _at_relocate_end.reclaimed = reclaimed;
-  _at_relocate_end.used = used;
-  _at_relocate_end.used_high = used_high;
-  _at_relocate_end.used_low = used_low;
-  _at_relocate_end.free = free(used);
-  _at_relocate_end.free_high = free(used_low);
-  _at_relocate_end.free_low = free(used_high);
+void ZStatHeap::set_at_relocate_end(const ZPageAllocatorStats& stats) {
+  _at_relocate_end.capacity = stats.capacity();
+  _at_relocate_end.capacity_high = capacity_high();
+  _at_relocate_end.capacity_low = capacity_low();
+  _at_relocate_end.free = free(stats.used());
+  _at_relocate_end.free_high = free(stats.used_low());
+  _at_relocate_end.free_low = free(stats.used_high());
+  _at_relocate_end.used = stats.used();
+  _at_relocate_end.used_high = stats.used_high();
+  _at_relocate_end.used_low = stats.used_low();
+  _at_relocate_end.allocated = allocated(stats.used(), stats.reclaimed());
+  _at_relocate_end.garbage = garbage(stats.reclaimed());
+  _at_relocate_end.reclaimed = stats.reclaimed();
 }
 
 size_t ZStatHeap::max_capacity() {
@@ -1337,15 +1378,6 @@ void ZStatHeap::print() {
                      .left(ZTABLE_ARGS(_at_relocate_end.capacity_high))
                      .left(ZTABLE_ARGS(_at_relocate_end.capacity_low))
                      .end());
-  log_info(gc, heap)("%s", table()
-                     .right("Reserve:")
-                     .left(ZTABLE_ARGS(_at_mark_start.reserve))
-                     .left(ZTABLE_ARGS(_at_mark_end.reserve))
-                     .left(ZTABLE_ARGS(_at_relocate_start.reserve))
-                     .left(ZTABLE_ARGS(_at_relocate_end.reserve))
-                     .left(ZTABLE_ARGS(_at_relocate_end.reserve_high))
-                     .left(ZTABLE_ARGS(_at_relocate_end.reserve_low))
-                     .end());
   log_info(gc, heap)("%s", table()
                      .right("Free:")
                      .left(ZTABLE_ARGS(_at_mark_start.free))
