@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,7 @@
  */
 
 #include "precompiled.hpp"
-#include "gc/z/zThread.hpp"
+#include "gc/z/zThread.inline.hpp"
 #include "runtime/thread.hpp"
 #include "utilities/debug.hpp"
 
@@ -31,7 +31,6 @@ THREAD_LOCAL uintptr_t ZThread::_id;
 THREAD_LOCAL bool      ZThread::_is_vm;
 THREAD_LOCAL bool      ZThread::_is_java;
 THREAD_LOCAL bool      ZThread::_is_worker;
-THREAD_LOCAL bool      ZThread::_is_runtime_worker;
 THREAD_LOCAL uint      ZThread::_worker_id;
 
 void ZThread::initialize() {
@@ -42,7 +41,6 @@ void ZThread::initialize() {
   _is_vm = thread->is_VM_thread();
   _is_java = thread->is_Java_thread();
   _is_worker = false;
-  _is_runtime_worker = false;
   _worker_id = (uint)-1;
 }
 
@@ -63,11 +61,6 @@ void ZThread::set_worker() {
   _is_worker = true;
 }
 
-void ZThread::set_runtime_worker() {
-  ensure_initialized();
-  _is_runtime_worker = true;
-}
-
 bool ZThread::has_worker_id() {
   return _initialized &&
          _is_worker &&
