@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,7 +27,6 @@
 #include "asm/macroAssembler.inline.hpp"
 #include "compiler/disassembler.hpp"
 #include "interpreter/bytecodeHistogram.hpp"
-#include "interpreter/bytecodeInterpreter.hpp"
 #include "interpreter/bytecodeStream.hpp"
 #include "interpreter/interpreter.hpp"
 #include "interpreter/interpreterRuntime.hpp"
@@ -56,15 +55,10 @@
 // Implementation of platform independent aspects of Interpreter
 
 void AbstractInterpreter::initialize() {
-  if (_code != NULL) return;
-
   // make sure 'imported' classes are initialized
   if (CountBytecodes || TraceBytecodes || StopInterpreterAt) BytecodeCounter::reset();
   if (PrintBytecodeHistogram)                                BytecodeHistogram::reset();
   if (PrintBytecodePairHistogram)                            BytecodePairHistogram::reset();
-
-  InvocationCounter::reinitialize();
-
 }
 
 void AbstractInterpreter::print() {
@@ -120,13 +114,14 @@ AbstractInterpreter::MethodKind AbstractInterpreter::method_kind(const methodHan
   if (m->is_method_handle_intrinsic()) {
     vmIntrinsics::ID id = m->intrinsic_id();
     assert(MethodHandles::is_signature_polymorphic(id), "must match an intrinsic");
-    MethodKind kind = (MethodKind)( method_handle_invoke_FIRST +
-                                    ((int)id - vmIntrinsics::FIRST_MH_SIG_POLY) );
+    MethodKind kind = (MethodKind)(method_handle_invoke_FIRST +
+                                   vmIntrinsics::as_int(id) -
+                                   static_cast<int>(vmIntrinsics::FIRST_MH_SIG_POLY));
     assert(kind <= method_handle_invoke_LAST, "parallel enum ranges");
     return kind;
   }
 
-#ifndef CC_INTERP
+#ifndef ZERO
   switch (m->intrinsic_id()) {
     // Use optimized stub code for CRC32 native methods.
     case vmIntrinsics::_updateCRC32            : return java_util_zip_CRC32_update;
@@ -141,7 +136,7 @@ AbstractInterpreter::MethodKind AbstractInterpreter::method_kind(const methodHan
     case vmIntrinsics::_doubleToRawLongBits: return java_lang_Double_doubleToRawLongBits;
     default:                                 break;
   }
-#endif // CC_INTERP
+#endif // ZERO
 
   // Native method?
   // Note: This test must come _before_ the test for intrinsic
@@ -191,13 +186,14 @@ AbstractInterpreter::MethodKind AbstractInterpreter::method_kind(const methodHan
     default                   : break;
   }
 
-  // Accessor method?
+  // Getter method?
   if (m->is_getter()) {
-    // TODO: We should have used ::is_accessor above, but fast accessors in Zero expect only getters.
-    // See CppInterpreter::accessor_entry in cppInterpreter_zero.cpp. This should be fixed in Zero,
-    // then the call above updated to ::is_accessor
-    assert(m->size_of_parameters() == 1, "fast code for accessors assumes parameter size = 1");
-    return accessor;
+    return getter;
+  }
+
+  // Setter method?
+  if (m->is_setter()) {
+    return setter;
   }
 
   // Note: for now: zero locals for all non-empty methods
@@ -206,22 +202,31 @@ AbstractInterpreter::MethodKind AbstractInterpreter::method_kind(const methodHan
 
 #if INCLUDE_CDS
 
-address AbstractInterpreter::get_trampoline_code_buffer(AbstractInterpreter::MethodKind kind) {
+// For a shared Method m, to improve sharing across processes, we avoid writing to m->_i2i_entry
+// at runtime. Instead, m->_i2i_entry points to a fixed location inside the CDS archive.
+// This location contains a trampoline (generated by generate_entry_for_cds_method)
+// which jumps to _entry_table[kind].
+address AbstractInterpreter::entry_for_cds_method(const methodHandle& m) {
+  MethodKind kind = method_kind(m);
+  assert(0 <= kind && kind < number_of_method_entries, "illegal kind");
+  return entry_for_cds_method(kind);
+}
+
+address AbstractInterpreter::entry_for_cds_method(AbstractInterpreter::MethodKind kind) {
   const size_t trampoline_size = SharedRuntime::trampoline_size();
-  address addr = MetaspaceShared::i2i_entry_code_buffers((size_t)(AbstractInterpreter::number_of_method_entries) * trampoline_size);
+  address addr = MetaspaceShared::i2i_entry_code_buffers();
   addr += (size_t)(kind) * trampoline_size;
 
   return addr;
 }
 
-void AbstractInterpreter::update_cds_entry_table(AbstractInterpreter::MethodKind kind) {
-  if (DumpSharedSpaces || UseSharedSpaces) {
-    address trampoline = get_trampoline_code_buffer(kind);
-    _cds_entry_table[kind] = trampoline;
-
+void AbstractInterpreter::generate_entry_for_cds_method(AbstractInterpreter::MethodKind kind) {
+  if (UseSharedSpaces) {
+    address trampoline = entry_for_cds_method(kind);
     CodeBuffer buffer(trampoline, (int)(SharedRuntime::trampoline_size()));
     MacroAssembler _masm(&buffer);
     SharedRuntime::generate_trampoline(&_masm, _entry_table[kind]);
+    _masm.flush();
 
     if (PrintInterpreter) {
       Disassembler::decode(buffer.insts_begin(), buffer.insts_end());
@@ -237,7 +242,7 @@ void AbstractInterpreter::set_entry_for_kind(AbstractInterpreter::MethodKind kin
   assert(_entry_table[kind] == _entry_table[abstract], "previous value must be AME entry");
   _entry_table[kind] = entry;
 
-  update_cds_entry_table(kind);
+  generate_entry_for_cds_method(kind);
 }
 
 // Return true if the interpreter can prove that the given bytecode has
@@ -267,7 +272,8 @@ bool AbstractInterpreter::is_not_reached(const methodHandle& method, int bci) {
         }
         assert(!invoke_bc.has_index_u4(code), "sanity");
         int method_index = invoke_bc.get_index_u2_cpcache(code);
-        Method* resolved_method = ConstantPool::method_at_if_loaded(cpool, method_index);
+        constantPoolHandle cp(Thread::current(), cpool);
+        Method* resolved_method = ConstantPool::method_at_if_loaded(cp, method_index);
         return (resolved_method == NULL);
       }
       default: ShouldNotReachHere();
@@ -296,7 +302,8 @@ void AbstractInterpreter::print_method_kind(MethodKind kind) {
     case native                 : tty->print("native"                 ); break;
     case native_synchronized    : tty->print("native_synchronized"    ); break;
     case empty                  : tty->print("empty"                  ); break;
-    case accessor               : tty->print("accessor"               ); break;
+    case getter                 : tty->print("getter"                 ); break;
+    case setter                 : tty->print("setter"                 ); break;
     case abstract               : tty->print("abstract"               ); break;
     case java_lang_math_sin     : tty->print("java_lang_math_sin"     ); break;
     case java_lang_math_cos     : tty->print("java_lang_math_cos"     ); break;
@@ -472,6 +479,6 @@ void AbstractInterpreter::initialize_method_handle_entries() {
   for (int i = method_handle_invoke_FIRST; i <= method_handle_invoke_LAST; i++) {
     MethodKind kind = (MethodKind) i;
     _entry_table[kind] = _entry_table[Interpreter::abstract];
-    Interpreter::update_cds_entry_table(kind);
+    Interpreter::generate_entry_for_cds_method(kind);
   }
 }
