@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,87 +25,82 @@
 #ifndef SHARE_JFR_RECORDER_CHECKPOINT_TYPES_TRACEID_JFRTRACEID_INLINE_HPP
 #define SHARE_JFR_RECORDER_CHECKPOINT_TYPES_TRACEID_JFRTRACEID_INLINE_HPP
 
-#include "classfile/classLoaderData.hpp"
-#include "classfile/moduleEntry.hpp"
-#include "classfile/packageEntry.hpp"
 #include "jfr/recorder/checkpoint/types/traceid/jfrTraceId.hpp"
+#include "jfr/recorder/checkpoint/types/traceid/jfrTraceIdLoadBarrier.inline.hpp"
 #include "jfr/recorder/checkpoint/types/traceid/jfrTraceIdBits.inline.hpp"
 #include "jfr/recorder/checkpoint/types/traceid/jfrTraceIdEpoch.hpp"
 #include "jfr/recorder/checkpoint/types/traceid/jfrTraceIdMacros.hpp"
 #include "jfr/support/jfrKlassExtension.hpp"
-#include "oops/arrayKlass.hpp"
 #include "oops/klass.hpp"
-#include "oops/instanceKlass.hpp"
-#include "oops/method.hpp"
 #include "runtime/thread.inline.hpp"
 #include "utilities/debug.hpp"
 
+inline traceid JfrTraceId::load(jclass jc) {
+  return JfrTraceIdLoadBarrier::load(jc);
+}
+
+inline traceid JfrTraceId::load(const Klass* klass) {
+  return JfrTraceIdLoadBarrier::load(klass);
+}
+
+inline traceid JfrTraceId::load(const Method* method) {
+  return JfrTraceIdLoadBarrier::load(method);
+}
+
+inline traceid JfrTraceId::load(const Klass* klass, const Method* method) {
+  return JfrTraceIdLoadBarrier::load(klass, method);
+}
+
+inline traceid JfrTraceId::load(const ModuleEntry* module) {
+  return JfrTraceIdLoadBarrier::load(module);
+}
+
+inline traceid JfrTraceId::load(const PackageEntry* package) {
+  return JfrTraceIdLoadBarrier::load(package);
+}
+
+inline traceid JfrTraceId::load(const ClassLoaderData* cld) {
+  return JfrTraceIdLoadBarrier::load(cld);
+}
+
+inline traceid JfrTraceId::load_leakp(const Klass* klass, const Method* method) {
+  return JfrTraceIdLoadBarrier::load_leakp(klass, method);
+}
+
 template <typename T>
-inline traceid set_used_and_get(const T* type) {
-  assert(type != NULL, "invariant");
-  SET_USED_THIS_EPOCH(type);
-  assert(USED_THIS_EPOCH(type), "invariant");
-  return TRACE_ID(type);
+inline traceid raw_load(const T* t) {
+  assert(t != NULL, "invariant");
+  return TRACE_ID(t);
 }
 
-inline traceid JfrTraceId::get(const Klass* klass) {
-  assert(klass != NULL, "invariant");
-  return TRACE_ID(klass);
+inline traceid JfrTraceId::load_raw(const Klass* klass) {
+  return raw_load(klass);
 }
 
-inline traceid JfrTraceId::get(const Thread* t) {
+inline traceid JfrTraceId::load_raw(const Thread* t) {
   assert(t != NULL, "invariant");
   return TRACE_ID_RAW(t->jfr_thread_local());
 }
 
-inline traceid JfrTraceId::use(const Klass* klass) {
-  assert(klass != NULL, "invariant");
-  return set_used_and_get(klass);
+inline traceid JfrTraceId::load_raw(const Method* method) {
+  return (METHOD_ID(method->method_holder(), method));
 }
 
-inline traceid JfrTraceId::use(const Method* method) {
-  assert(method != NULL, "invariant");
-  return use(method->method_holder(), method);
+inline traceid JfrTraceId::load_raw(const ModuleEntry* module) {
+  return raw_load(module);
 }
 
-inline traceid JfrTraceId::use(const Klass* klass, const Method* method) {
-  assert(klass != NULL, "invariant");
-  assert(method != NULL, "invariant");
-  SET_METHOD_FLAG_USED_THIS_EPOCH(method);
-
-  SET_METHOD_AND_CLASS_USED_THIS_EPOCH(klass);
-  assert(METHOD_AND_CLASS_USED_THIS_EPOCH(klass), "invariant");
-  return (METHOD_ID(klass, method));
+inline traceid JfrTraceId::load_raw(const PackageEntry* package) {
+  return raw_load(package);
 }
 
-inline traceid JfrTraceId::use(const ModuleEntry* module) {
-  assert(module != NULL, "invariant");
-  return set_used_and_get(module);
-}
-
-inline traceid JfrTraceId::use(const PackageEntry* package) {
-  assert(package != NULL, "invariant");
-  return set_used_and_get(package);
-}
-
-inline traceid JfrTraceId::use(const ClassLoaderData* cld) {
-  assert(cld != NULL, "invariant");
-  return cld->is_unsafe_anonymous() ? 0 : set_used_and_get(cld);
-}
-
-inline void JfrTraceId::set_leakp(const Method* method) {
-  assert(method != NULL, "invariant");
-  const Klass* const klass = method->method_holder();
-  assert(klass != NULL, "invariant");
-  assert(METHOD_AND_CLASS_USED_THIS_EPOCH(klass), "invariant");
-  assert(METHOD_FLAG_USED_THIS_EPOCH(method), "invariant");
-  SET_LEAKP(klass);
-  SET_METHOD_LEAKP(method);
+inline traceid JfrTraceId::load_raw(const ClassLoaderData* cld) {
+  return raw_load(cld);
 }
 
 inline bool JfrTraceId::in_visible_set(const Klass* klass) {
   assert(klass != NULL, "invariant");
-  assert(((JavaThread*)Thread::current())->thread_state() == _thread_in_vm, "invariant");
+  assert(JavaThread::current()->thread_state() == _thread_in_vm, "invariant");
   return (IS_JDK_JFR_EVENT_SUBKLASS(klass) && !klass->is_abstract()) || IS_EVENT_HOST_KLASS(klass);
 }
 
