@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -29,13 +29,13 @@
 static const u1* const TOP_CRITICAL_SECTION = NULL;
 
 JfrBuffer::JfrBuffer() : _next(NULL),
-                         _prev(NULL),
                          _identity(NULL),
                          _pos(NULL),
                          _top(NULL),
-                         _flags(0),
+                         _size(0),
                          _header_size(0),
-                         _size(0) {}
+                         _flags(0),
+                         _context(0) {}
 
 bool JfrBuffer::initialize(size_t header_size, size_t size) {
   assert(_next == NULL, "invariant");
@@ -53,8 +53,6 @@ bool JfrBuffer::initialize(size_t header_size, size_t size) {
 
 void JfrBuffer::reinitialize(bool exclusion /* false */) {
   acquire_critical_section_top();
-  assert(!lease(), "invariant");
-  assert(!transient(), "invariant");
   if (exclusion != excluded()) {
     // update
     if (exclusion) {
@@ -124,6 +122,13 @@ bool JfrBuffer::try_acquire(const void* id) {
   return current_id == NULL && Atomic::cmpxchg(&_identity, current_id, id) == current_id;
 }
 
+void JfrBuffer::set_identity(const void* id) {
+  assert(id != NULL, "invariant");
+  assert(_identity == NULL, "invariant");
+  OrderAccess::storestore();
+  _identity = id;
+}
+
 void JfrBuffer::release() {
   assert(identity() != NULL, "invariant");
   Atomic::release_store(&_identity, (const void*)NULL);
@@ -180,25 +185,25 @@ enum FLAG {
   EXCLUDED = 8
 };
 
-inline u2 load(const volatile u2* flags) {
-  assert(flags != NULL, "invariant");
-  return Atomic::load_acquire(flags);
+inline u1 load(const volatile u1* dest) {
+  assert(dest != NULL, "invariant");
+  return Atomic::load_acquire(dest);
 }
 
-inline void set(u2* flags, FLAG flag) {
-  assert(flags != NULL, "invariant");
+inline void set(u1* dest, u1 data) {
+  assert(dest != NULL, "invariant");
   OrderAccess::storestore();
-  *flags |= (u1)flag;
+  *dest |= data;
 }
 
-inline void clear(u2* flags, FLAG flag) {
-  assert(flags != NULL, "invariant");
+inline void clear(u1* dest, u1 data) {
+  assert(dest != NULL, "invariant");
   OrderAccess::storestore();
-  *flags ^= (u1)flag;
+  *dest ^= data;
 }
 
-inline bool test(const u2* flags, FLAG flag) {
-  return (u1)flag == (load(flags) & (u1)flag);
+inline bool test(const u1* dest, u1 data) {
+  return data == (load(dest) & data);
 }
 
 bool JfrBuffer::transient() const {
@@ -260,13 +265,23 @@ bool JfrBuffer::retired() const {
 }
 
 void JfrBuffer::set_retired() {
-  assert(acquired_by_self(), "invariant");
   set(&_flags, RETIRED);
 }
 
 void JfrBuffer::clear_retired() {
   if (retired()) {
-    assert(identity() != NULL, "invariant");
     clear(&_flags, RETIRED);
   }
 }
+
+u1 JfrBuffer::context() const {
+  return load(&_context);
+}
+
+void JfrBuffer::set_context(u1 context) {
+  set(&_context, context);
+}
+
+void JfrBuffer::clear_context() {
+  set(&_context, 0);
+}
