@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -134,29 +134,36 @@ class CodeInstaller : public StackObj {
   friend class JVMCIVMStructs;
 private:
   enum MarkId {
-    VERIFIED_ENTRY                         = 1,
-    UNVERIFIED_ENTRY                       = 2,
-    OSR_ENTRY                              = 3,
-    EXCEPTION_HANDLER_ENTRY                = 4,
-    DEOPT_HANDLER_ENTRY                    = 5,
-    INVOKEINTERFACE                        = 6,
-    INVOKEVIRTUAL                          = 7,
-    INVOKESTATIC                           = 8,
-    INVOKESPECIAL                          = 9,
-    INLINE_INVOKE                          = 10,
-    POLL_NEAR                              = 11,
-    POLL_RETURN_NEAR                       = 12,
-    POLL_FAR                               = 13,
-    POLL_RETURN_FAR                        = 14,
-    CARD_TABLE_ADDRESS                     = 15,
-    CARD_TABLE_SHIFT                       = 16,
-    HEAP_TOP_ADDRESS                       = 17,
-    HEAP_END_ADDRESS                       = 18,
-    NARROW_KLASS_BASE_ADDRESS              = 19,
-    NARROW_OOP_BASE_ADDRESS                = 20,
-    CRC_TABLE_ADDRESS                      = 21,
-    LOG_OF_HEAP_REGION_GRAIN_BYTES         = 22,
-    INLINE_CONTIGUOUS_ALLOCATION_SUPPORTED = 23,
+    INVALID_MARK,
+    VERIFIED_ENTRY,
+    UNVERIFIED_ENTRY,
+    OSR_ENTRY,
+    EXCEPTION_HANDLER_ENTRY,
+    DEOPT_HANDLER_ENTRY,
+    FRAME_COMPLETE,
+    INVOKEINTERFACE,
+    INVOKEVIRTUAL,
+    INVOKESTATIC,
+    INVOKESPECIAL,
+    INLINE_INVOKE,
+    POLL_NEAR,
+    POLL_RETURN_NEAR,
+    POLL_FAR,
+    POLL_RETURN_FAR,
+    CARD_TABLE_ADDRESS,
+    CARD_TABLE_SHIFT,
+    HEAP_TOP_ADDRESS,
+    HEAP_END_ADDRESS,
+    NARROW_KLASS_BASE_ADDRESS,
+    NARROW_OOP_BASE_ADDRESS,
+    CRC_TABLE_ADDRESS,
+    LOG_OF_HEAP_REGION_GRAIN_BYTES,
+    INLINE_CONTIGUOUS_ALLOCATION_SUPPORTED,
+    DEOPT_MH_HANDLER_ENTRY,
+    VERIFY_OOPS,
+    VERIFY_OOP_BITS,
+    VERIFY_OOP_MASK,
+    VERIFY_OOP_COUNT_ADDRESS,
     INVOKE_INVALID                         = -1
   };
 
@@ -193,6 +200,7 @@ private:
   Dependencies*             _dependencies;
   ExceptionHandlerTable     _exception_handler_table;
   ImplicitExceptionTable    _implicit_exception_table;
+  bool                      _has_auto_box;
 
   bool _immutable_pic_compilation;  // Installer is called for Immutable PIC compilation.
 
@@ -202,6 +210,7 @@ private:
   static ConstantIntValue*    _int_1_scope_value;
   static ConstantIntValue*    _int_2_scope_value;
   static LocationValue*       _illegal_value;
+  static MarkerValue*         _virtual_byte_array_marker;
 
   jint pd_next_offset(NativeInstruction* inst, jint pc_offset, JVMCIObject method, JVMCI_TRAPS);
   void pd_patch_OopConstant(int pc_offset, JVMCIObject constant, JVMCI_TRAPS);
@@ -222,7 +231,11 @@ private:
 
 public:
 
-  CodeInstaller(JVMCIEnv* jvmci_env, bool immutable_pic_compilation) : _arena(mtJVMCI), _jvmci_env(jvmci_env), _immutable_pic_compilation(immutable_pic_compilation) {}
+  CodeInstaller(JVMCIEnv* jvmci_env, bool immutable_pic_compilation) :
+    _arena(mtJVMCI),
+    _jvmci_env(jvmci_env),
+    _has_auto_box(false),
+    _immutable_pic_compilation(immutable_pic_compilation) {}
 
 #if INCLUDE_AOT
   JVMCI::CodeInstallResult gather_metadata(JVMCIObject target, JVMCIObject compiled_code, CodeMetadata& metadata, JVMCI_TRAPS);
@@ -294,11 +307,11 @@ protected:
 
   int map_jvmci_bci(int bci);
 
-  void record_scope(jint pc_offset, JVMCIObject debug_info, ScopeMode scope_mode, bool return_oop, JVMCI_TRAPS);
+  void record_scope(jint pc_offset, JVMCIObject debug_info, ScopeMode scope_mode, bool is_mh_invoke, bool return_oop, JVMCI_TRAPS);
   void record_scope(jint pc_offset, JVMCIObject debug_info, ScopeMode scope_mode, JVMCI_TRAPS) {
-    record_scope(pc_offset, debug_info, scope_mode, false /* return_oop */, JVMCIENV);
+    record_scope(pc_offset, debug_info, scope_mode, false /* is_mh_invoke */, false /* return_oop */, JVMCIENV);
   }
-  void record_scope(jint pc_offset, JVMCIObject position, ScopeMode scope_mode, GrowableArray<ScopeValue*>* objects, bool return_oop, JVMCI_TRAPS);
+  void record_scope(jint pc_offset, JVMCIObject position, ScopeMode scope_mode, GrowableArray<ScopeValue*>* objects, bool is_mh_invoke, bool return_oop, JVMCI_TRAPS);
   void record_object_value(ObjectValue* sv, JVMCIObject value, GrowableArray<ScopeValue*>* objects, JVMCI_TRAPS);
 
   GrowableArray<ScopeValue*>* record_virtual_objects(JVMCIObject debug_info, JVMCI_TRAPS);
