@@ -23,12 +23,14 @@
  */
 
 #include "precompiled.hpp"
+#include "gc/shared/gcConfig.hpp"
 #include "jvm.h"
 #include "jvmci/jvmci_globals.hpp"
-#include "gc/shared/gcConfig.hpp"
+#include "runtime/arguments.hpp"
+#include "runtime/flags/jvmFlagAccess.hpp"
+#include "runtime/globals_extension.hpp"
 #include "utilities/defaultStream.hpp"
 #include "utilities/ostream.hpp"
-#include "runtime/globals_extension.hpp"
 
 fileStream* JVMCIGlobals::_jni_config_file = NULL;
 
@@ -37,9 +39,9 @@ bool JVMCIGlobals::check_jvmci_flags_are_consistent() {
 
 #ifndef PRODUCT
 #define APPLY_JVMCI_FLAGS(params3, params4) \
-  JVMCI_FLAGS(params4, params3, params4, params3, params4, params3, params4, params4, IGNORE_RANGE, IGNORE_CONSTRAINT)
-#define JVMCI_DECLARE_CHECK4(type, name, value, doc) bool name##checked = false;
-#define JVMCI_DECLARE_CHECK3(type, name, doc)        bool name##checked = false;
+  JVMCI_FLAGS(params4, params3, params4, params3, params4, IGNORE_RANGE, IGNORE_CONSTRAINT)
+#define JVMCI_DECLARE_CHECK4(type, name, value, ...) bool name##checked = false;
+#define JVMCI_DECLARE_CHECK3(type, name, ...)        bool name##checked = false;
 #define JVMCI_FLAG_CHECKED(name)                          name##checked = true;
   APPLY_JVMCI_FLAGS(JVMCI_DECLARE_CHECK3, JVMCI_DECLARE_CHECK4)
 #else
@@ -108,6 +110,7 @@ bool JVMCIGlobals::check_jvmci_flags_are_consistent() {
   }
   JVMCI_FLAG_CHECKED(EagerJVMCI)
 
+  CHECK_NOT_SET(JVMCIEventLogLevel,           EnableJVMCI)
   CHECK_NOT_SET(JVMCITraceLevel,              EnableJVMCI)
   CHECK_NOT_SET(JVMCICounterSize,             EnableJVMCI)
   CHECK_NOT_SET(JVMCICountersExcludeCompiler, EnableJVMCI)
@@ -129,8 +132,8 @@ bool JVMCIGlobals::check_jvmci_flags_are_consistent() {
 #endif // !COMPILER2
 
 #ifndef PRODUCT
-#define JVMCI_CHECK4(type, name, value, doc) assert(name##checked, #name " flag not checked");
-#define JVMCI_CHECK3(type, name, doc)        assert(name##checked, #name " flag not checked");
+#define JVMCI_CHECK4(type, name, value, ...) assert(name##checked, #name " flag not checked");
+#define JVMCI_CHECK3(type, name, ...)        assert(name##checked, #name " flag not checked");
   // Ensures that all JVMCI flags are checked by this method.
   APPLY_JVMCI_FLAGS(JVMCI_CHECK3, JVMCI_CHECK4)
 #undef APPLY_JVMCI_FLAGS
@@ -155,7 +158,7 @@ bool JVMCIGlobals::check_jvmci_flags_are_consistent() {
 }
 
 // Convert JVMCI flags from experimental to product
-bool JVMCIGlobals::enable_jvmci_product_mode(JVMFlag::Flags origin) {
+bool JVMCIGlobals::enable_jvmci_product_mode(JVMFlagOrigin origin) {
   const char *JVMCIFlags[] = {
     "EnableJVMCI",
     "EnableJVMCIProduct",
@@ -166,6 +169,8 @@ bool JVMCIGlobals::enable_jvmci_product_mode(JVMFlag::Flags origin) {
     "JVMCICounterSize",
     "JVMCICountersExcludeCompiler",
     "JVMCINMethodSizeLimit",
+    "JVMCIEventLogLevel",
+    "JVMCITraceLevel",
     "JVMCILibPath",
     "JVMCILibDumpJNIConfig",
     "UseJVMCINativeLibrary",
@@ -183,7 +188,7 @@ bool JVMCIGlobals::enable_jvmci_product_mode(JVMFlag::Flags origin) {
 
   bool value = true;
   JVMFlag *jvmciEnableFlag = JVMFlag::find_flag("EnableJVMCIProduct");
-  if (JVMFlag::boolAtPut(jvmciEnableFlag, &value, origin) != JVMFlag::SUCCESS) {
+  if (JVMFlagAccess::boolAtPut(jvmciEnableFlag, &value, origin) != JVMFlag::SUCCESS) {
     return false;
   }
 
@@ -195,11 +200,15 @@ bool JVMCIGlobals::enable_jvmci_product_mode(JVMFlag::Flags origin) {
   return true;
 }
 
+bool JVMCIGlobals::gc_supports_jvmci() {
+  return UseSerialGC || UseParallelGC || UseG1GC;
+}
+
 void JVMCIGlobals::check_jvmci_supported_gc() {
   if (EnableJVMCI) {
     // Check if selected GC is supported by JVMCI and Java compiler
-    if (!(UseSerialGC || UseParallelGC || UseG1GC)) {
-      vm_exit_during_initialization("JVMCI Compiler does not support selected GC", GCConfig::hs_err_name());
+    if (!gc_supports_jvmci()) {
+      log_warning(gc, jvmci)("Setting EnableJVMCI to false as selected GC does not support JVMCI: %s", GCConfig::hs_err_name());
       FLAG_SET_DEFAULT(EnableJVMCI, false);
       FLAG_SET_DEFAULT(UseJVMCICompiler, false);
     }
