@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,120 +25,120 @@
 #ifndef SHARE_JVMCI_JVMCI_GLOBALS_HPP
 #define SHARE_JVMCI_JVMCI_GLOBALS_HPP
 
-#include "runtime/flags/jvmFlag.hpp"
+#include "utilities/vmEnums.hpp"
 
 class fileStream;
 
 //
-// Defines all global flags used by the JVMCI compiler. Only flags that need
+// Declare all global flags used by the JVMCI compiler. Only flags that need
 // to be accessible to the JVMCI C++ code should be defined here.
 //
-#define JVMCI_FLAGS(develop, \
-                    develop_pd, \
-                    product, \
-                    product_pd, \
-                    diagnostic, \
-                    diagnostic_pd, \
-                    experimental, \
-                    notproduct, \
-                    range, \
-                    constraint, \
-                    writeable) \
+#define JVMCI_FLAGS(develop,                                                \
+                    develop_pd,                                             \
+                    product,                                                \
+                    product_pd,                                             \
+                    notproduct,                                             \
+                    range,                                                  \
+                    constraint)                                             \
                                                                             \
-  experimental(bool, EnableJVMCI, false,                                    \
+  product(bool, EnableJVMCI, false, EXPERIMENTAL,                           \
           "Enable JVMCI")                                                   \
                                                                             \
-  experimental(bool, EnableJVMCIProduct, false,                             \
+  product(bool, EnableJVMCIProduct, false, EXPERIMENTAL,                    \
           "Allow JVMCI to be used in product mode. This alters a subset of "\
           "JVMCI flags to be non-experimental, defaults UseJVMCICompiler "  \
           "and EnableJVMCI to true and defaults UseJVMCINativeLibrary "     \
           "to true if a JVMCI native library is available.")                \
                                                                             \
-  experimental(bool, UseJVMCICompiler, false,                               \
+  product(bool, UseJVMCICompiler, false, EXPERIMENTAL,                      \
           "Use JVMCI as the default compiler. Defaults to true if "         \
           "EnableJVMCIProduct is true.")                                    \
                                                                             \
-  experimental(bool, JVMCIPrintProperties, false,                           \
+  product(bool, JVMCIPrintProperties, false, EXPERIMENTAL,                  \
           "Prints properties used by the JVMCI compiler and exits")         \
                                                                             \
-  experimental(bool, BootstrapJVMCI, false,                                 \
+  product(bool, BootstrapJVMCI, false, EXPERIMENTAL,                        \
           "Bootstrap JVMCI before running Java main method. This "          \
           "initializes the compile queue with a small set of methods "      \
           "and processes the queue until it is empty. Combining this with " \
           "-XX:-TieredCompilation makes JVMCI compile more of itself.")     \
                                                                             \
-  experimental(bool, EagerJVMCI, false,                                     \
+  product(bool, EagerJVMCI, false, EXPERIMENTAL,                            \
           "Force eager JVMCI initialization")                               \
                                                                             \
-  experimental(bool, PrintBootstrap, true,                                  \
+  product(bool, PrintBootstrap, true, EXPERIMENTAL,                         \
           "Print JVMCI bootstrap progress and summary")                     \
                                                                             \
-  experimental(intx, JVMCIThreads, 1,                                       \
+  product(intx, JVMCIThreads, 1, EXPERIMENTAL,                              \
           "Force number of JVMCI compiler threads to use. Ignored if "      \
           "UseJVMCICompiler is false.")                                     \
           range(1, max_jint)                                                \
                                                                             \
-  experimental(intx, JVMCIHostThreads, 1,                                   \
+  product(intx, JVMCIHostThreads, 1, EXPERIMENTAL,                          \
           "Force number of C1 compiler threads. Ignored if "                \
           "UseJVMCICompiler is false.")                                     \
           range(1, max_jint)                                                \
                                                                             \
   NOT_COMPILER2(product(intx, MaxVectorSize, 64,                            \
           "Max vector size in bytes, "                                      \
-          "actual size could be less depending on elements type"))          \
+          "actual size could be less depending on elements type")           \
+          range(0, max_jint))                                               \
                                                                             \
   NOT_COMPILER2(product(bool, ReduceInitialCardMarks, true,                 \
           "Defer write barriers of young objects"))                         \
                                                                             \
-  experimental(intx, JVMCITraceLevel, 0,                                    \
-          "Trace level for JVMCI: "                                         \
-          "1 means emit a message for each CompilerToVM call,"              \
-          "levels greater than 1 provide progressively greater detail")     \
+  product(intx, JVMCIEventLogLevel, 1, EXPERIMENTAL,                        \
+          "Event log level for JVMCI")                                      \
+          range(0, 4)                                                       \
                                                                             \
-  experimental(intx, JVMCICounterSize, 0,                                   \
+  product(intx, JVMCITraceLevel, 0, EXPERIMENTAL,                           \
+          "Trace level for JVMCI")                                          \
+          range(0, 4)                                                       \
+                                                                            \
+  product(intx, JVMCICounterSize, 0, EXPERIMENTAL,                          \
           "Reserved size for benchmark counters")                           \
-          range(0, max_jint)                                                \
+          range(0, 1000000)                                                 \
                                                                             \
-  experimental(bool, JVMCICountersExcludeCompiler, true,                    \
+  product(bool, JVMCICountersExcludeCompiler, true, EXPERIMENTAL,           \
           "Exclude JVMCI compiler threads from benchmark counters")         \
                                                                             \
   develop(bool, JVMCIUseFastLocking, true,                                  \
           "Use fast inlined locking code")                                  \
                                                                             \
-  experimental(intx, JVMCINMethodSizeLimit, (80*K)*wordSize,                \
+  product(intx, JVMCINMethodSizeLimit, (80*K)*wordSize, EXPERIMENTAL,       \
           "Maximum size of a compiled method.")                             \
+          range(0, max_jint)                                                \
                                                                             \
-  experimental(intx, MethodProfileWidth, 0,                                 \
-          "Number of methods to record in call profile")                    \
-                                                                            \
-  experimental(ccstr, JVMCILibPath, NULL,                                   \
+  product(ccstr, JVMCILibPath, NULL, EXPERIMENTAL,                          \
           "LD path for loading the JVMCI shared library")                   \
                                                                             \
-  experimental(ccstr, JVMCILibDumpJNIConfig, NULL,                          \
+  product(ccstr, JVMCILibDumpJNIConfig, NULL, EXPERIMENTAL,                 \
           "Dumps to the given file a description of the classes, fields "   \
           "and methods the JVMCI shared library must provide")              \
                                                                             \
-  experimental(bool, UseJVMCINativeLibrary, false,                          \
+  product(bool, UseJVMCINativeLibrary, false, EXPERIMENTAL,                 \
           "Execute JVMCI Java code from a shared library "                  \
           "instead of loading it from class files and executing it "        \
           "on the HotSpot heap. Defaults to true if EnableJVMCIProduct is " \
           "true and a JVMCI native library is available.")                  \
                                                                             \
-  NOT_COMPILER2(diagnostic(bool, UseMultiplyToLenIntrinsic, false,          \
+  NOT_COMPILER2(product(bool, UseMultiplyToLenIntrinsic, false, DIAGNOSTIC, \
           "Enables intrinsification of BigInteger.multiplyToLen()"))        \
                                                                             \
-  NOT_COMPILER2(diagnostic(bool, UseSquareToLenIntrinsic, false,            \
+  NOT_COMPILER2(product(bool, UseSquareToLenIntrinsic, false, DIAGNOSTIC,   \
           "Enables intrinsification of BigInteger.squareToLen()"))          \
                                                                             \
-  NOT_COMPILER2(diagnostic(bool, UseMulAddIntrinsic, false,                 \
+  NOT_COMPILER2(product(bool, UseMulAddIntrinsic, false, DIAGNOSTIC,        \
           "Enables intrinsification of BigInteger.mulAdd()"))               \
                                                                             \
-  NOT_COMPILER2(diagnostic(bool, UseMontgomeryMultiplyIntrinsic, false,     \
+  NOT_COMPILER2(product(bool, UseMontgomeryMultiplyIntrinsic, false, DIAGNOSTIC, \
           "Enables intrinsification of BigInteger.montgomeryMultiply()"))   \
                                                                             \
-  NOT_COMPILER2(diagnostic(bool, UseMontgomerySquareIntrinsic, false,       \
+  NOT_COMPILER2(product(bool, UseMontgomerySquareIntrinsic, false, DIAGNOSTIC, \
           "Enables intrinsification of BigInteger.montgomerySquare()"))
 
+// end of JVMCI_FLAGS
+
 // The base name for the shared library containing the JVMCI based compiler
 #define JVMCI_SHARED_LIBRARY_NAME "jvmcicompiler"
 
@@ -153,9 +153,12 @@ class JVMCIGlobals {
   static bool check_jvmci_flags_are_consistent();
 
   // Convert JVMCI experimental flags to product
-  static bool enable_jvmci_product_mode(JVMFlag::Flags);
+  static bool enable_jvmci_product_mode(JVMFlagOrigin);
 
-  // Check and exit VM with error if selected GC is not supported by JVMCI.
+  // Returns true iff the GC fully supports JVMCI.
+  static bool gc_supports_jvmci();
+
+  // Check and turn off EnableJVMCI if selected GC does not support JVMCI.
   static void check_jvmci_supported_gc();
 
   static fileStream* get_jni_config_file() { return _jni_config_file; }
