@@ -28,11 +28,7 @@
 #include "runtime/os.hpp"
 #include "services/memTracker.hpp"
 #include "utilities/align.hpp"
-
-size_t CodeHeap::header_size() {
-  return sizeof(HeapBlock);
-}
-
+#include "utilities/powerOfTwo.hpp"
 
 // Implementation of Heap
 
@@ -204,6 +200,7 @@ bool CodeHeap::reserve(ReservedSpace rs, size_t committed_size, size_t segment_s
   assert(rs.size() >= committed_size, "reserved < committed");
   assert(segment_size >= sizeof(FreeBlock), "segment size is too small");
   assert(is_power_of_2(segment_size), "segment_size must be a power of 2");
+  assert_locked_or_safepoint(CodeCache_lock);
 
   _segment_size      = segment_size;
   _log2_segment_size = exact_log2(segment_size);
@@ -252,6 +249,8 @@ bool CodeHeap::reserve(ReservedSpace rs, size_t committed_size, size_t segment_s
 
 
 bool CodeHeap::expand_by(size_t size) {
+  assert_locked_or_safepoint(CodeCache_lock);
+
   // expand _memory space
   size_t dm = align_to_page_size(_memory.committed_size() + size) - _memory.committed_size();
   if (dm > 0) {
@@ -282,6 +281,7 @@ bool CodeHeap::expand_by(size_t size) {
 void* CodeHeap::allocate(size_t instance_size) {
   size_t number_of_segments = size_to_segments(instance_size + header_size());
   assert(segments_to_size(number_of_segments) >= sizeof(FreeBlock), "not enough room for FreeList");
+  assert_locked_or_safepoint(CodeCache_lock);
 
   // First check if we can satisfy request from freelist
   NOT_PRODUCT(verify());
@@ -346,6 +346,8 @@ HeapBlock* CodeHeap::split_block(HeapBlock *b, size_t split_at) {
 
 void CodeHeap::deallocate_tail(void* p, size_t used_size) {
   assert(p == find_start(p), "illegal deallocation");
+  assert_locked_or_safepoint(CodeCache_lock);
+
   // Find start of HeapBlock
   HeapBlock* b = (((HeapBlock *)p) - 1);
   assert(b->allocated_space() == p, "sanity check");
@@ -362,6 +364,8 @@ void CodeHeap::deallocate_tail(void* p, size_t used_size) {
 
 void CodeHeap::deallocate(void* p) {
   assert(p == find_start(p), "illegal deallocation");
+  assert_locked_or_safepoint(CodeCache_lock);
+
   // Find start of HeapBlock
   HeapBlock* b = (((HeapBlock *)p) - 1);
   assert(b->allocated_space() == p, "sanity check");
@@ -766,7 +770,8 @@ int CodeHeap::segmap_hops(size_t beg, size_t end) {
   if (beg < end) {
     // setup _segmap pointers for faster indexing
     address p = (address)_segmap.low() + beg;
-    int hops_expected = (int)(((end-beg-1)+(free_sentinel-2))/(free_sentinel-1));
+    int hops_expected
+      = checked_cast<int>(((end-beg-1)+(free_sentinel-2))/(free_sentinel-1));
     int nhops = 0;
     size_t ix = end-beg-1;
     while (p[ix] > 0) {
@@ -789,6 +794,7 @@ void CodeHeap::print() {
 
 void CodeHeap::verify() {
   if (VerifyCodeCache) {
+    assert_locked_or_safepoint(CodeCache_lock);
     size_t len = 0;
     int count = 0;
     for(FreeBlock* b = _freelist; b != NULL; b = b->link()) {
