@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,18 +26,22 @@
 #define SHARE_MEMORY_HEAPSHARED_HPP
 
 #include "classfile/compactHashtable.hpp"
+#include "classfile/javaClasses.hpp"
 #include "classfile/systemDictionary.hpp"
 #include "memory/allocation.hpp"
 #include "memory/metaspaceShared.hpp"
 #include "oops/compressedOops.hpp"
 #include "oops/objArrayKlass.hpp"
 #include "oops/oop.hpp"
+#include "oops/oopHandle.hpp"
 #include "oops/typeArrayKlass.hpp"
 #include "utilities/bitMap.hpp"
 #include "utilities/growableArray.hpp"
 #include "utilities/resourceHash.hpp"
 
 #if INCLUDE_CDS_JAVA_HEAP
+class DumpedInternedStrings;
+
 struct ArchivableStaticFieldInfo {
   const char* klass_name;
   const char* field_name;
@@ -61,12 +65,24 @@ class KlassSubGraphInfo: public CHeapObj<mtClass> {
   // A list of _k's static fields as the entry points of archived sub-graphs.
   // For each entry field, it is a tuple of field_offset, field_value and
   // is_closed_archive flag.
-  GrowableArray<juint>*  _subgraph_entry_fields;
+  GrowableArray<int>* _subgraph_entry_fields;
+
+  // Does this KlassSubGraphInfo belong to the archived full module graph
+  bool _is_full_module_graph;
+
+  // Does this KlassSubGraphInfo references any classes that were loaded while
+  // JvmtiExport::is_early_phase()!=true. If so, this KlassSubGraphInfo cannot be
+  // used at runtime if JVMTI ClassFileLoadHook is enabled.
+  bool _has_non_early_klasses;
+  static bool is_non_early_klass(Klass* k);
 
  public:
-  KlassSubGraphInfo(Klass* k) :
+  KlassSubGraphInfo(Klass* k, bool is_full_module_graph) :
     _k(k),  _subgraph_object_klasses(NULL),
-    _subgraph_entry_fields(NULL) {}
+    _subgraph_entry_fields(NULL),
+    _is_full_module_graph(is_full_module_graph),
+    _has_non_early_klasses(false) {}
+
   ~KlassSubGraphInfo() {
     if (_subgraph_object_klasses != NULL) {
       delete _subgraph_object_klasses;
@@ -80,7 +96,7 @@ class KlassSubGraphInfo: public CHeapObj<mtClass> {
   GrowableArray<Klass*>* subgraph_object_klasses() {
     return _subgraph_object_klasses;
   }
-  GrowableArray<juint>*  subgraph_entry_fields() {
+  GrowableArray<int>* subgraph_entry_fields() {
     return _subgraph_entry_fields;
   }
   void add_subgraph_entry_field(int static_field_offset, oop v,
@@ -90,6 +106,8 @@ class KlassSubGraphInfo: public CHeapObj<mtClass> {
     return _subgraph_object_klasses == NULL ? 0 :
            _subgraph_object_klasses->length();
   }
+  bool is_full_module_graph() const { return _is_full_module_graph; }
+  bool has_non_early_klasses() const { return _has_non_early_klasses; }
 };
 
 // An archived record of object sub-graphs reachable from static
@@ -98,9 +116,11 @@ class KlassSubGraphInfo: public CHeapObj<mtClass> {
 class ArchivedKlassSubGraphInfoRecord {
  private:
   Klass* _k;
+  bool _is_full_module_graph;
+  bool _has_non_early_klasses;
 
   // contains pairs of field offset and value for each subgraph entry field
-  Array<juint>* _entry_field_records;
+  Array<int>* _entry_field_records;
 
   // klasses of objects in archived sub-graphs referenced from the entry points
   // (static fields) in the containing class
@@ -110,8 +130,10 @@ class ArchivedKlassSubGraphInfoRecord {
     _k(NULL), _entry_field_records(NULL), _subgraph_object_klasses(NULL) {}
   void init(KlassSubGraphInfo* info);
   Klass* klass() const { return _k; }
-  Array<juint>*  entry_field_records() const { return _entry_field_records; }
+  Array<int>* entry_field_records() const { return _entry_field_records; }
   Array<Klass*>* subgraph_object_klasses() const { return _subgraph_object_klasses; }
+  bool is_full_module_graph() const { return _is_full_module_graph; }
+  bool has_non_early_klasses() const { return _has_non_early_klasses; }
 };
 #endif // INCLUDE_CDS_JAVA_HEAP
 
@@ -123,12 +145,18 @@ class HeapShared: AllStatic {
   static bool _closed_archive_heap_region_mapped;
   static bool _open_archive_heap_region_mapped;
   static bool _archive_heap_region_fixed;
+  static DumpedInternedStrings *_dumped_interned_strings;
 
+public:
   static bool oop_equals(oop const& p1, oop const& p2) {
     return p1 == p2;
   }
   static unsigned oop_hash(oop const& p);
+  static unsigned string_oop_hash(oop const& string) {
+    return java_lang_String::hash_code(string);
+  }
 
+private:
   typedef ResourceHashtable<oop, oop,
       HeapShared::oop_hash,
       HeapShared::oop_equals,
@@ -141,7 +169,8 @@ class HeapShared: AllStatic {
   }
 
   static unsigned klass_hash(Klass* const& klass) {
-    return primitive_hash<address>((address)klass);
+    // Generate deterministic hashcode even if SharedBaseAddress is changed due to ASLR.
+    return primitive_hash<address>(address(klass) - SharedBaseAddress);
   }
 
   class DumpTimeKlassSubGraphInfoTable
@@ -176,6 +205,7 @@ private:
   static void archive_object_subgraphs(ArchivableStaticFieldInfo fields[],
                                        int num,
                                        bool is_closed_archive,
+                                       bool is_full_module_graph,
                                        Thread* THREAD);
 
   // Archive object sub-graph starting from the given static field
@@ -190,6 +220,7 @@ private:
   static void verify_reachable_objects_from(oop obj, bool is_archived) PRODUCT_RETURN;
   static void verify_subgraph_from(oop orig_obj) PRODUCT_RETURN;
 
+  static KlassSubGraphInfo* init_subgraph_info(Klass *k, bool is_full_module_graph);
   static KlassSubGraphInfo* get_subgraph_info(Klass *k);
 
   static void init_subgraph_entry_fields(ArchivableStaticFieldInfo fields[],
@@ -207,6 +238,10 @@ private:
 
   static SeenObjectsTable *_seen_objects_table;
 
+  static GrowableArrayCHeap<oop, mtClassShared>* _pending_roots;
+  static narrowOop _roots_narrow;
+  static OopHandle _roots;
+
   static void init_seen_objects_table() {
     assert(_seen_objects_table == NULL, "must be");
     _seen_objects_table = new (ResourceObj::C_HEAP, mtClass)SeenObjectsTable();
@@ -229,13 +264,26 @@ private:
   static int _num_total_recorded_klasses;
   static int _num_total_verifications;
 
-  static void start_recording_subgraph(InstanceKlass *k, const char* klass_name);
+  static void start_recording_subgraph(InstanceKlass *k, const char* klass_name,
+                                       bool is_full_module_graph);
   static void done_recording_subgraph(InstanceKlass *k, const char* klass_name);
 
   static bool has_been_seen_during_subgraph_recording(oop obj);
   static void set_has_been_seen_during_subgraph_recording(oop obj);
 
+  static void check_module_oop(oop orig_module_obj);
+  static void copy_roots();
+
+  static void resolve_classes_for_subgraphs(ArchivableStaticFieldInfo fields[],
+                                            int num, TRAPS);
+  static void resolve_classes_for_subgraph_of(Klass* k, TRAPS);
+  static void clear_archived_roots_of(Klass* k);
+  static const ArchivedKlassSubGraphInfoRecord*
+               resolve_or_init_classes_for_subgraph_of(Klass* k, bool do_init, TRAPS);
+  static void resolve_or_init(Klass* k, bool do_init, TRAPS);
+  static void init_archived_fields_for(Klass* k, const ArchivedKlassSubGraphInfoRecord* record, TRAPS);
  public:
+  static void reset_archived_object_states(TRAPS);
   static void create_archived_object_cache() {
     _archived_object_cache =
       new (ResourceObj::C_HEAP, mtClass)ArchivedObjectCache();
@@ -250,7 +298,6 @@ private:
 
   static oop find_archived_heap_object(oop obj);
   static oop archive_heap_object(oop obj, Thread* THREAD);
-  static oop materialize_archived_object(narrowOop v);
 
   static void archive_klass_objects(Thread* THREAD);
 
@@ -273,9 +320,38 @@ private:
                                             TRAPS);
 
   static ResourceBitMap calculate_oopmap(MemRegion region);
+  static void add_to_dumped_interned_strings(oop string);
+
+  // We use the HeapShared::roots() array to make sure that objects stored in the
+  // archived heap regions are not prematurely collected. These roots include:
+  //
+  //    - mirrors of classes that have not yet been loaded.
+  //    - ConstantPool::resolved_references() of classes that have not yet been loaded.
+  //    - ArchivedKlassSubGraphInfoRecords that have not been initialized
+  //    - java.lang.Module objects that have not yet been added to the module graph
+  //
+  // When a mirror M becomes referenced by a newly loaded class K, M will be removed
+  // from HeapShared::roots() via clear_root(), and K will be responsible for
+  // keeping M alive.
+  //
+  // Other types of roots are also cleared similarly when they become referenced.
+
+  // Dump-time only. Returns the index of the root, which can be used at run time to read
+  // the root using get_root(index, ...).
+  static int append_root(oop obj);
+
+  // Dump-time and runtime
+  static objArrayOop roots();
+  static oop get_root(int index, bool clear=false);
+
+  // Run-time only
+  static void set_roots(narrowOop roots);
+  static void clear_root(int index);
 #endif // INCLUDE_CDS_JAVA_HEAP
 
  public:
+  static void run_full_gc_in_vm_thread() NOT_CDS_JAVA_HEAP_RETURN;
+
   static bool is_heap_object_archiving_allowed() {
     CDS_JAVA_HEAP_ONLY(return (UseG1GC && UseCompressedOops && UseCompressedClassPointers);)
     NOT_CDS_JAVA_HEAP(return false;)
@@ -283,32 +359,36 @@ private:
 
   static bool is_heap_region(int idx) {
     CDS_JAVA_HEAP_ONLY(return (idx >= MetaspaceShared::first_closed_archive_heap_region &&
-                               idx <= MetaspaceShared::last_open_archive_heap_region));
+                               idx <= MetaspaceShared::last_open_archive_heap_region);)
     NOT_CDS_JAVA_HEAP_RETURN_(false);
   }
 
   static void set_closed_archive_heap_region_mapped() {
-    CDS_JAVA_HEAP_ONLY(_closed_archive_heap_region_mapped = true);
+    CDS_JAVA_HEAP_ONLY(_closed_archive_heap_region_mapped = true;)
     NOT_CDS_JAVA_HEAP_RETURN;
   }
   static bool closed_archive_heap_region_mapped() {
-    CDS_JAVA_HEAP_ONLY(return _closed_archive_heap_region_mapped);
+    CDS_JAVA_HEAP_ONLY(return _closed_archive_heap_region_mapped;)
     NOT_CDS_JAVA_HEAP_RETURN_(false);
   }
   static void set_open_archive_heap_region_mapped() {
-    CDS_JAVA_HEAP_ONLY(_open_archive_heap_region_mapped = true);
+    CDS_JAVA_HEAP_ONLY(_open_archive_heap_region_mapped = true;)
     NOT_CDS_JAVA_HEAP_RETURN;
   }
   static bool open_archive_heap_region_mapped() {
-    CDS_JAVA_HEAP_ONLY(return _open_archive_heap_region_mapped);
+    CDS_JAVA_HEAP_ONLY(return _open_archive_heap_region_mapped;)
     NOT_CDS_JAVA_HEAP_RETURN_(false);
   }
+  static bool is_mapped() {
+    return closed_archive_heap_region_mapped() && open_archive_heap_region_mapped();
+  }
 
   static void fixup_mapped_heap_regions() NOT_CDS_JAVA_HEAP_RETURN;
 
   inline static bool is_archived_object(oop p) NOT_CDS_JAVA_HEAP_RETURN_(false);
 
-  static void initialize_from_archived_subgraph(Klass* k) NOT_CDS_JAVA_HEAP_RETURN;
+  static void resolve_classes(TRAPS) NOT_CDS_JAVA_HEAP_RETURN;
+  static void initialize_from_archived_subgraph(Klass* k, TRAPS) NOT_CDS_JAVA_HEAP_RETURN;
 
   // NarrowOops stored in the CDS archive may use a different encoding scheme
   // than CompressedOops::{base,shift} -- see FileMapInfo::map_heap_regions_impl.
@@ -321,8 +401,20 @@ private:
   static void patch_archived_heap_embedded_pointers(MemRegion mem, address  oopmap,
                                                     size_t oopmap_in_bits) NOT_CDS_JAVA_HEAP_RETURN;
 
+  static void init_for_dumping(Thread* THREAD) NOT_CDS_JAVA_HEAP_RETURN;
   static void init_subgraph_entry_fields(Thread* THREAD) NOT_CDS_JAVA_HEAP_RETURN;
   static void write_subgraph_info_table() NOT_CDS_JAVA_HEAP_RETURN;
   static void serialize_subgraph_info_table_header(SerializeClosure* soc) NOT_CDS_JAVA_HEAP_RETURN;
 };
+
+#if INCLUDE_CDS_JAVA_HEAP
+class DumpedInternedStrings :
+  public ResourceHashtable<oop, bool,
+                           HeapShared::string_oop_hash,
+                           HeapShared::oop_equals,
+                           15889, // prime number
+                           ResourceObj::C_HEAP>
+{};
+#endif
+
 #endif // SHARE_MEMORY_HEAPSHARED_HPP
