@@ -26,10 +26,10 @@
 #define SHARE_OOPS_METHODDATA_INLINE_HPP
 
 #include "oops/methodData.hpp"
-#include "runtime/orderAccess.hpp"
+#include "runtime/atomic.hpp"
 
 inline void DataLayout::release_set_cell_at(int index, intptr_t value) {
-  OrderAccess::release_store(&_cells[index], value);
+  Atomic::release_store(&_cells[index], value);
 }
 
 inline void ProfileData::release_set_intptr_at(int index, intptr_t value) {
