@@ -294,17 +294,17 @@ void ObjArrayKlass::copy_array(arrayOop s, int src_pos, arrayOop d,
     size_t src_offset = (size_t) objArrayOopDesc::obj_at_offset<narrowOop>(src_pos);
     size_t dst_offset = (size_t) objArrayOopDesc::obj_at_offset<narrowOop>(dst_pos);
     assert(arrayOopDesc::obj_offset_to_raw<narrowOop>(s, src_offset, NULL) ==
-           objArrayOop(s)->obj_at_addr_raw<narrowOop>(src_pos), "sanity");
+           objArrayOop(s)->obj_at_addr<narrowOop>(src_pos), "sanity");
     assert(arrayOopDesc::obj_offset_to_raw<narrowOop>(d, dst_offset, NULL) ==
-           objArrayOop(d)->obj_at_addr_raw<narrowOop>(dst_pos), "sanity");
+           objArrayOop(d)->obj_at_addr<narrowOop>(dst_pos), "sanity");
     do_copy(s, src_offset, d, dst_offset, length, CHECK);
   } else {
     size_t src_offset = (size_t) objArrayOopDesc::obj_at_offset<oop>(src_pos);
     size_t dst_offset = (size_t) objArrayOopDesc::obj_at_offset<oop>(dst_pos);
     assert(arrayOopDesc::obj_offset_to_raw<oop>(s, src_offset, NULL) ==
-           objArrayOop(s)->obj_at_addr_raw<oop>(src_pos), "sanity");
+           objArrayOop(s)->obj_at_addr<oop>(src_pos), "sanity");
     assert(arrayOopDesc::obj_offset_to_raw<oop>(d, dst_offset, NULL) ==
-           objArrayOop(d)->obj_at_addr_raw<oop>(dst_pos), "sanity");
+           objArrayOop(d)->obj_at_addr<oop>(dst_pos), "sanity");
     do_copy(s, src_offset, d, dst_offset, length, CHECK);
   }
 }
@@ -321,7 +321,7 @@ Klass* ObjArrayKlass::array_klass_impl(bool or_null, int n, TRAPS) {
     if (or_null) return NULL;
 
     ResourceMark rm;
-    JavaThread *jt = (JavaThread *)THREAD;
+    JavaThread *jt = THREAD->as_Java_thread();
     {
       // Ensure atomic creation of higher dimensions
       MutexLocker mu(THREAD, MultiArray_lock);
