@@ -32,24 +32,20 @@
 #include "runtime/globals.hpp"
 
 inline HeapWord* objArrayOopDesc::base() const { return (HeapWord*) arrayOopDesc::base(T_OBJECT); }
-inline HeapWord* objArrayOopDesc::base_raw() const { return (HeapWord*) arrayOopDesc::base_raw(T_OBJECT); }
 
 template <class T> T* objArrayOopDesc::obj_at_addr(int index) const {
-  assert(is_within_bounds(index), "index out of bounds");
+  assert(is_within_bounds(index), "index %d out of bounds %d", index, length());
   return &((T*)base())[index];
 }
 
-template <class T> T* objArrayOopDesc::obj_at_addr_raw(int index) const {
-  assert(is_within_bounds(index), "index out of bounds");
-  return &((T*)base_raw())[index];
-}
-
 inline oop objArrayOopDesc::obj_at(int index) const {
+  assert(is_within_bounds(index), "index %d out of bounds %d", index, length());
   ptrdiff_t offset = UseCompressedOops ? obj_at_offset<narrowOop>(index) : obj_at_offset<oop>(index);
   return HeapAccess<IS_ARRAY>::oop_load_at(as_oop(), offset);
 }
 
 inline void objArrayOopDesc::obj_at_put(int index, oop value) {
+  assert(is_within_bounds(index), "index %d out of bounds %d", index, length());
   ptrdiff_t offset = UseCompressedOops ? obj_at_offset<narrowOop>(index) : obj_at_offset<oop>(index);
   HeapAccess<IS_ARRAY>::oop_store_at(as_oop(), offset, value);
 }
