@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,6 +27,7 @@
 
 #include "oops/access.inline.hpp"
 #include "oops/oopHandle.hpp"
+#include "gc/shared/oopStorage.inline.hpp"
 
 inline oop OopHandle::resolve() const {
   return (_obj == NULL) ? (oop)NULL : NativeAccess<>::oop_load(_obj);
@@ -36,4 +37,31 @@ inline oop OopHandle::peek() const {
   return (_obj == NULL) ? (oop)NULL : NativeAccess<AS_NO_KEEPALIVE>::oop_load(_obj);
 }
 
+inline OopHandle::OopHandle(OopStorage* storage, oop obj) :
+    _obj(storage->allocate()) {
+  if (_obj == NULL) {
+    vm_exit_out_of_memory(sizeof(oop), OOM_MALLOC_ERROR,
+                          "Cannot create oop handle");
+  }
+  NativeAccess<>::oop_store(_obj, obj);
+}
+
+inline void OopHandle::release(OopStorage* storage) {
+  if (peek() != NULL) {
+    // Clear the OopHandle first
+    NativeAccess<>::oop_store(_obj, (oop)NULL);
+    storage->release(_obj);
+  }
+}
+
+inline void OopHandle::replace(oop obj) {
+  oop* ptr = ptr_raw();
+  assert(ptr != NULL, "should not use replace");
+  NativeAccess<>::oop_store(ptr, obj);
+}
+
+inline oop OopHandle::xchg(oop new_value) {
+  return NativeAccess<MO_SEQ_CST>::oop_atomic_xchg(_obj, new_value);
+}
+
 #endif // SHARE_OOPS_OOPHANDLE_INLINE_HPP
