@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,6 +27,7 @@
 #include "classfile/systemDictionary.hpp"
 #include "classfile/vmSymbols.hpp"
 #include "compiler/compileBroker.hpp"
+#include "compiler/compilerEvent.hpp"
 #include "compiler/compileLog.hpp"
 #include "interpreter/linkResolver.hpp"
 #include "jfr/jfrEvents.hpp"
@@ -41,13 +42,12 @@
 InlineTree::InlineTree(Compile* c,
                        const InlineTree *caller_tree, ciMethod* callee,
                        JVMState* caller_jvms, int caller_bci,
-                       float site_invoke_ratio, int max_inline_level) :
+                       int max_inline_level) :
   C(c),
   _caller_jvms(caller_jvms),
   _method(callee),
   _caller_tree((InlineTree*) caller_tree),
   _count_inline_bcs(method()->code_size_for_inlining()),
-  _site_invoke_ratio(site_invoke_ratio),
   _max_inline_level(max_inline_level),
   _subtrees(c->comp_arena(), 2, 0, NULL),
   _msg(NULL)
@@ -531,25 +531,6 @@ const char* InlineTree::check_can_parse(ciMethod* callee) {
   return NULL;
 }
 
-static void post_inlining_event(int compile_id,const char* msg, bool success, int bci, ciMethod* caller, ciMethod* callee) {
-  assert(caller != NULL, "invariant");
-  assert(callee != NULL, "invariant");
-  EventCompilerInlining event;
-  if (event.should_commit()) {
-    JfrStructCalleeMethod callee_struct;
-    callee_struct.set_type(callee->holder()->name()->as_utf8());
-    callee_struct.set_name(callee->name()->as_utf8());
-    callee_struct.set_descriptor(callee->signature()->as_symbol()->as_utf8());
-    event.set_compileId(compile_id);
-    event.set_message(msg);
-    event.set_succeeded(success);
-    event.set_bci(bci);
-    event.set_caller(caller->get_Method());
-    event.set_callee(callee_struct);
-    event.commit();
-  }
-}
-
 //------------------------------print_inlining---------------------------------
 void InlineTree::print_inlining(ciMethod* callee_method, int caller_bci,
                                 ciMethod* caller_method, bool success) const {
@@ -566,14 +547,17 @@ void InlineTree::print_inlining(ciMethod* callee_method, int caller_bci,
                                                caller_bci, inline_msg);
   if (C->print_inlining()) {
     C->print_inlining(callee_method, inline_level(), caller_bci, inline_msg);
-    guarantee(callee_method != NULL, "would crash in post_inlining_event");
+    guarantee(callee_method != NULL, "would crash in CompilerEvent::InlineEvent::post");
     if (Verbose) {
       const InlineTree *top = this;
       while (top->caller_tree() != NULL) { top = top->caller_tree(); }
       //tty->print("  bcs: %d+%d  invoked: %d", top->count_inline_bcs(), callee_method->code_size(), callee_method->interpreter_invocation_count());
     }
   }
-  post_inlining_event(C->compile_id(), inline_msg, success, caller_bci, caller_method, callee_method);
+  EventCompilerInlining event;
+  if (event.should_commit()) {
+    CompilerEvent::InlineEvent::post(event, C->compile_id(), caller_method->get_Method(), callee_method, success, inline_msg, caller_bci);
+  }
 }
 
 //------------------------------ok_to_inline-----------------------------------
@@ -661,21 +645,8 @@ WarmCallInfo* InlineTree::ok_to_inline(ciMethod* callee_method, JVMState* jvms,
   return NULL;
 }
 
-//------------------------------compute_callee_frequency-----------------------
-float InlineTree::compute_callee_frequency( int caller_bci ) const {
-  int count  = method()->interpreter_call_site_count(caller_bci);
-  int invcnt = method()->interpreter_invocation_count();
-  float freq = (float)count/(float)invcnt;
-  // Call-site count / interpreter invocation count, scaled recursively.
-  // Always between 0.0 and 1.0.  Represents the percentage of the method's
-  // total execution time used at this call site.
-
-  return freq;
-}
-
 //------------------------------build_inline_tree_for_callee-------------------
 InlineTree *InlineTree::build_inline_tree_for_callee( ciMethod* callee_method, JVMState* caller_jvms, int caller_bci) {
-  float recur_frequency = _site_invoke_ratio * compute_callee_frequency(caller_bci);
   // Attempt inlining.
   InlineTree* old_ilt = callee_at(caller_bci, callee_method);
   if (old_ilt != NULL) {
@@ -700,7 +671,7 @@ InlineTree *InlineTree::build_inline_tree_for_callee( ciMethod* callee_method, J
     }
   }
   // Allocate in the comp_arena to make sure the InlineTree is live when dumping a replay compilation file
-  InlineTree* ilt = new (C->comp_arena()) InlineTree(C, this, callee_method, caller_jvms, caller_bci, recur_frequency, _max_inline_level + max_inline_level_adjust);
+  InlineTree* ilt = new (C->comp_arena()) InlineTree(C, this, callee_method, caller_jvms, caller_bci, _max_inline_level + max_inline_level_adjust);
   _subtrees.append(ilt);
 
   NOT_PRODUCT( _count_inlines += 1; )
@@ -726,7 +697,7 @@ InlineTree *InlineTree::build_inline_tree_root() {
   Compile* C = Compile::current();
 
   // Root of inline tree
-  InlineTree* ilt = new InlineTree(C, NULL, C->method(), NULL, -1, 1.0F, MaxInlineLevel);
+  InlineTree* ilt = new InlineTree(C, NULL, C->method(), NULL, -1, MaxInlineLevel);
 
   return ilt;
 }
