@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -59,8 +59,10 @@ macro(CallJava)
 macro(CallLeaf)
 macro(CallLeafNoFP)
 macro(CallRuntime)
+macro(CallNative)
 macro(CallStaticJava)
 macro(CastII)
+macro(CastLL)
 macro(CastX2P)
 macro(CastP2X)
 macro(CastPP)
@@ -142,6 +144,8 @@ macro(CountedLoop)
 macro(CountedLoopEnd)
 macro(OuterStripMinedLoop)
 macro(OuterStripMinedLoopEnd)
+macro(LongCountedLoop)
+macro(LongCountedLoopEnd)
 macro(CountLeadingZerosI)
 macro(CountLeadingZerosL)
 macro(CountTrailingZerosI)
@@ -198,9 +202,10 @@ macro(LoopLimit)
 macro(Mach)
 macro(MachProj)
 macro(MulAddS2I)
+macro(MaxI)
+macro(MaxL)
 macro(MaxD)
 macro(MaxF)
-macro(MaxI)
 macro(MemBarAcquire)
 macro(LoadFence)
 macro(SetVectMaskI)
@@ -212,9 +217,10 @@ macro(MemBarReleaseLock)
 macro(MemBarVolatile)
 macro(MemBarStoreStore)
 macro(MergeMem)
-macro(MinD)
-macro(MinF)
 macro(MinI)
+macro(MinL)
+macro(MinF)
+macro(MinD)
 macro(ModD)
 macro(ModF)
 macro(ModI)
@@ -229,6 +235,8 @@ macro(MulHiL)
 macro(MulI)
 macro(MulL)
 macro(Multi)
+macro(NegI)
+macro(NegL)
 macro(NegD)
 macro(NegF)
 macro(NeverBranch)
@@ -268,6 +276,10 @@ macro(RoundDouble)
 macro(RoundDoubleMode)
 macro(RoundDoubleModeV)
 macro(RoundFloat)
+macro(RotateLeft)
+macro(RotateLeftV)
+macro(RotateRight)
+macro(RotateRightV)
 macro(SafePoint)
 macro(SafePointScalarObject)
 #if INCLUDE_SHENANDOAHGC
@@ -281,9 +293,13 @@ shmacro(ShenandoahCompareAndSwapN)
 shmacro(ShenandoahCompareAndSwapP)
 shmacro(ShenandoahWeakCompareAndSwapN)
 shmacro(ShenandoahWeakCompareAndSwapP)
-shmacro(ShenandoahEnqueueBarrier)
+shmacro(ShenandoahIUBarrier)
 shmacro(ShenandoahLoadReferenceBarrier)
 macro(SCMemProj)
+macro(CopySignD)
+macro(CopySignF)
+macro(SignumD)
+macro(SignumF)
 macro(SqrtD)
 macro(SqrtF)
 macro(Start)
@@ -316,6 +332,8 @@ macro(TailJump)
 macro(MacroLogicV)
 macro(ThreadLocal)
 macro(Unlock)
+macro(URShiftB)
+macro(URShiftS)
 macro(URShiftI)
 macro(URShiftL)
 macro(XorI)
@@ -358,6 +376,7 @@ macro(AbsVI)
 macro(AbsVL)
 macro(AbsVF)
 macro(AbsVD)
+macro(NegVI)
 macro(NegVF)
 macro(NegVD)
 macro(SqrtVD)
@@ -387,7 +406,12 @@ macro(MaxV)
 macro(MinReductionV)
 macro(MaxReductionV)
 macro(LoadVector)
+macro(LoadVectorGather)
 macro(StoreVector)
+macro(StoreVectorScatter)
+macro(LoadVectorMasked)
+macro(StoreVectorMasked)
+macro(VectorMaskGen)
 macro(Pack)
 macro(PackB)
 macro(PackS)
@@ -416,3 +440,24 @@ macro(Digit)
 macro(LowerCase)
 macro(UpperCase)
 macro(Whitespace)
+macro(VectorBox)
+macro(VectorBoxAllocate)
+macro(VectorUnbox)
+macro(VectorMaskWrapper)
+macro(VectorMaskCmp)
+macro(VectorTest)
+macro(VectorBlend)
+macro(VectorRearrange)
+macro(VectorLoadMask)
+macro(VectorLoadShuffle)
+macro(VectorLoadConst)
+macro(VectorStoreMask)
+macro(VectorReinterpret)
+macro(VectorCast)
+macro(VectorCastB2X)
+macro(VectorCastS2X)
+macro(VectorCastI2X)
+macro(VectorCastL2X)
+macro(VectorCastF2X)
+macro(VectorCastD2X)
+macro(VectorInsert)
