@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -81,8 +81,8 @@ class IdealGraphPrinter : public CHeapObj<mtCompiler> {
   static const char *METHOD_SHORT_NAME_PROPERTY;
   static const char *ASSEMBLY_ELEMENT;
 
-    static int _file_count;
-  networkStream *_stream;
+  static int _file_count;
+  networkStream *_network_stream;
   xmlStream *_xml;
   outputStream *_output;
   ciMethod *_current_method;
@@ -108,11 +108,14 @@ class IdealGraphPrinter : public CHeapObj<mtCompiler> {
   void tail(const char *name);
   void head(const char *name);
   void text(const char *s);
+  void init(const char* file_name, bool use_multiple_files, bool append);
+  void init_file_stream(const char* file_name, bool use_multiple_files, bool append);
+  void init_network_stream();
   IdealGraphPrinter();
   ~IdealGraphPrinter();
 
  public:
-
+  IdealGraphPrinter(Compile* compile, const char* file_name = NULL, bool append = false);
   static void clean_up();
   static IdealGraphPrinter *printer();
 
@@ -123,8 +126,8 @@ class IdealGraphPrinter : public CHeapObj<mtCompiler> {
   void end_method();
   void print_method(const char *name, int level = 0);
   void print(const char *name, Node *root);
-  bool should_print(int level);
   void set_compile(Compile* compile) {C = compile; }
+  void update_compiled_method(ciMethod* current_method);
 };
 
 #endif
