@@ -39,7 +39,7 @@ const uint IdealKit::first_var = TypeFunc::Parms + 1;
 
 //----------------------------IdealKit-----------------------------------------
 IdealKit::IdealKit(GraphKit* gkit, bool delay_all_transforms, bool has_declarations) :
-  _gvn(gkit->gvn()), C(gkit->C) {
+  C(gkit->C), _gvn(gkit->gvn()) {
   _initial_ctrl = gkit->control();
   _initial_memory = gkit->merged_memory();
   _initial_i_o = gkit->i_o();
@@ -168,7 +168,7 @@ void IdealKit::loop(GraphKit* gkit, int nargs, IdealVariable& iv, Node* init, Bo
     // Sync IdealKit and graphKit.
     gkit->sync_kit(*this);
     // Add loop predicate.
-    gkit->add_predicate(nargs);
+    gkit->add_empty_predicates(nargs);
     // Update IdealKit memory.
     sync_kit(gkit);
   }
