@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -31,6 +31,7 @@
 #include "opto/mulnode.hpp"
 #include "opto/phaseX.hpp"
 #include "opto/subnode.hpp"
+#include "utilities/powerOfTwo.hpp"
 
 // Portions of code courtesy of Clifford Click
 
@@ -86,12 +87,13 @@ Node *MulNode::Ideal(PhaseGVN *phase, bool can_reshape) {
     Node *mul1 = in(1);
 #ifdef ASSERT
     // Check for dead loop
-    int   op1 = mul1->Opcode();
-    if( phase->eqv( mul1, this ) || phase->eqv( in(2), this ) ||
-        ( ( op1 == mul_opcode() || op1 == add_opcode() ) &&
-          ( phase->eqv( mul1->in(1), this ) || phase->eqv( mul1->in(2), this ) ||
-            phase->eqv( mul1->in(1), mul1 ) || phase->eqv( mul1->in(2), mul1 ) ) ) )
+    int op1 = mul1->Opcode();
+    if ((mul1 == this) || (in(2) == this) ||
+        ((op1 == mul_opcode() || op1 == add_opcode()) &&
+         ((mul1->in(1) == this) || (mul1->in(2) == this) ||
+          (mul1->in(1) == mul1) || (mul1->in(2) == mul1)))) {
       assert(false, "dead loop in MulNode::Ideal");
+    }
 #endif
 
     if( mul1->Opcode() == mul_opcode() ) {  // Left input is a multiply?
@@ -306,7 +308,7 @@ Node *MulLNode::Ideal(PhaseGVN *phase, bool can_reshape) {
       Node *n2 = phase->transform(new LShiftLNode(in(1), phase->intcon(log2_long(bit2))));
       res = new AddLNode(n2, n1);
 
-    } else if (is_power_of_2_long(abs_con+1)) {
+    } else if (is_power_of_2(abs_con+1)) {
       // Sleezy: power-of-2 -1.  Next time be generic.
       julong temp = abs_con + 1;
       Node *n1 = phase->transform( new LShiftLNode(in(1), phase->intcon(log2_long(temp))));
@@ -435,7 +437,9 @@ const Type *AndINode::mul_ring( const Type *t0, const Type *t1 ) const {
 Node* AndINode::Identity(PhaseGVN* phase) {
 
   // x & x => x
-  if (phase->eqv(in(1), in(2))) return in(1);
+  if (in(1) == in(2)) {
+    return in(1);
+  }
 
   Node* in1 = in(1);
   uint op = in1->Opcode();
@@ -557,7 +561,9 @@ const Type *AndLNode::mul_ring( const Type *t0, const Type *t1 ) const {
 Node* AndLNode::Identity(PhaseGVN* phase) {
 
   // x & x => x
-  if (phase->eqv(in(1), in(2))) return in(1);
+  if (in(1) == in(2)) {
+    return in(1);
+  }
 
   Node *usr = in(1);
   const TypeLong *t2 = phase->type( in(2) )->isa_long();
@@ -631,32 +637,44 @@ Node *AndLNode::Ideal(PhaseGVN *phase, bool can_reshape) {
 
 //=============================================================================
 
-static int getShiftCon(PhaseGVN *phase, Node *shiftNode, int retVal) {
-  const Type *t = phase->type(shiftNode->in(2));
-  if (t == Type::TOP) return retVal;       // Right input is dead.
-  const TypeInt *t2 = t->isa_int();
-  if (!t2 || !t2->is_con()) return retVal; // Right input is a constant.
-
-  return t2->get_con();
+static bool const_shift_count(PhaseGVN* phase, Node* shiftNode, int* count) {
+  const TypeInt* tcount = phase->type(shiftNode->in(2))->isa_int();
+  if (tcount != NULL && tcount->is_con()) {
+    *count = tcount->get_con();
+    return true;
+  }
+  return false;
 }
 
-static int maskShiftAmount(PhaseGVN *phase, Node *shiftNode, int nBits) {
-  int       shift = getShiftCon(phase, shiftNode, 0);
-  int maskedShift = shift & (nBits - 1);
+static int maskShiftAmount(PhaseGVN* phase, Node* shiftNode, int nBits) {
+  int count = 0;
+  if (const_shift_count(phase, shiftNode, &count)) {
+    int maskedShift = count & (nBits - 1);
+    if (maskedShift == 0) {
+      // Let Identity() handle 0 shift count.
+      return 0;
+    }
 
-  if (maskedShift == 0) return 0;         // Let Identity() handle 0 shift count.
-
-  if (shift != maskedShift) {
-    shiftNode->set_req(2, phase->intcon(maskedShift)); // Replace shift count with masked value.
-    phase->igvn_rehash_node_delayed(shiftNode);
+    if (count != maskedShift) {
+      shiftNode->set_req(2, phase->intcon(maskedShift)); // Replace shift count with masked value.
+      PhaseIterGVN* igvn = phase->is_IterGVN();
+      if (igvn) {
+        igvn->rehash_node_delayed(shiftNode);
+      }
+    }
+    return maskedShift;
   }
-
-  return maskedShift;
+  return 0;
 }
 
 //------------------------------Identity---------------------------------------
 Node* LShiftINode::Identity(PhaseGVN* phase) {
-  return ((getShiftCon(phase, this, -1) & (BitsPerJavaInteger - 1)) == 0) ? in(1) : this;
+  int count = 0;
+  if (const_shift_count(phase, this, &count) && (count & (BitsPerJavaInteger - 1)) == 0) {
+    // Shift by a multiple of 32 does nothing
+    return in(1);
+  }
+  return this;
 }
 
 //------------------------------Ideal------------------------------------------
@@ -764,7 +782,12 @@ const Type* LShiftINode::Value(PhaseGVN* phase) const {
 //=============================================================================
 //------------------------------Identity---------------------------------------
 Node* LShiftLNode::Identity(PhaseGVN* phase) {
-  return ((getShiftCon(phase, this, -1) & (BitsPerJavaLong - 1)) == 0) ? in(1) : this;
+  int count = 0;
+  if (const_shift_count(phase, this, &count) && (count & (BitsPerJavaLong - 1)) == 0) {
+    // Shift by a multiple of 64 does nothing
+    return in(1);
+  }
+  return this;
 }
 
 //------------------------------Ideal------------------------------------------
@@ -869,26 +892,30 @@ const Type* LShiftLNode::Value(PhaseGVN* phase) const {
 //=============================================================================
 //------------------------------Identity---------------------------------------
 Node* RShiftINode::Identity(PhaseGVN* phase) {
-  int shift = getShiftCon(phase, this, -1);
-  if (shift == -1) return this;
-  if ((shift & (BitsPerJavaInteger - 1)) == 0) return in(1);
-
-  // Check for useless sign-masking
-  if (in(1)->Opcode() == Op_LShiftI &&
-      in(1)->req() == 3 &&
-      in(1)->in(2) == in(2)) {
-    shift &= BitsPerJavaInteger-1; // semantics of Java shifts
-    // Compute masks for which this shifting doesn't change
-    int lo = (-1 << (BitsPerJavaInteger - ((uint)shift)-1)); // FFFF8000
-    int hi = ~lo;               // 00007FFF
-    const TypeInt *t11 = phase->type(in(1)->in(1))->isa_int();
-    if (!t11) return this;
-    // Does actual value fit inside of mask?
-    if (lo <= t11->_lo && t11->_hi <= hi) {
-      return in(1)->in(1);      // Then shifting is a nop
+  int count = 0;
+  if (const_shift_count(phase, this, &count)) {
+    if ((count & (BitsPerJavaInteger - 1)) == 0) {
+      // Shift by a multiple of 32 does nothing
+      return in(1);
+    }
+    // Check for useless sign-masking
+    if (in(1)->Opcode() == Op_LShiftI &&
+        in(1)->req() == 3 &&
+        in(1)->in(2) == in(2)) {
+      count &= BitsPerJavaInteger-1; // semantics of Java shifts
+      // Compute masks for which this shifting doesn't change
+      int lo = (-1 << (BitsPerJavaInteger - ((uint)count)-1)); // FFFF8000
+      int hi = ~lo;               // 00007FFF
+      const TypeInt* t11 = phase->type(in(1)->in(1))->isa_int();
+      if (t11 == NULL) {
+        return this;
+      }
+      // Does actual value fit inside of mask?
+      if (lo <= t11->_lo && t11->_hi <= hi) {
+        return in(1)->in(1);      // Then shifting is a nop
+      }
     }
   }
-
   return this;
 }
 
@@ -1073,8 +1100,11 @@ const Type* RShiftLNode::Value(PhaseGVN* phase) const {
 //=============================================================================
 //------------------------------Identity---------------------------------------
 Node* URShiftINode::Identity(PhaseGVN* phase) {
-  int shift = getShiftCon(phase, this, -1);
-  if ((shift & (BitsPerJavaInteger - 1)) == 0) return in(1);
+  int count = 0;
+  if (const_shift_count(phase, this, &count) && (count & (BitsPerJavaInteger - 1)) == 0) {
+    // Shift by a multiple of 32 does nothing
+    return in(1);
+  }
 
   // Check for "((x << LogBytesPerWord) + (wordSize-1)) >> LogBytesPerWord" which is just "x".
   // Happens during new-array length computation.
@@ -1165,6 +1195,18 @@ Node *URShiftINode::Ideal(PhaseGVN *phase, bool can_reshape) {
       phase->type(shl->in(2)) == t2 )
     return new AndINode( shl->in(1), phase->intcon(mask) );
 
+  // Check for (x >> n) >>> 31. Replace with (x >>> 31)
+  Node *shr = in(1);
+  if ( in1_op == Op_RShiftI ) {
+    Node *in11 = shr->in(1);
+    Node *in12 = shr->in(2);
+    const TypeInt *t11 = phase->type(in11)->isa_int();
+    const TypeInt *t12 = phase->type(in12)->isa_int();
+    if ( t11 && t2 && t2->is_con(31) && t12 && t12->is_con() ) {
+      return new URShiftINode(in11, phase->intcon(31));
+    }
+  }
+
   return NULL;
 }
 
@@ -1245,7 +1287,12 @@ const Type* URShiftINode::Value(PhaseGVN* phase) const {
 //=============================================================================
 //------------------------------Identity---------------------------------------
 Node* URShiftLNode::Identity(PhaseGVN* phase) {
-  return ((getShiftCon(phase, this, -1) & (BitsPerJavaLong - 1)) == 0) ? in(1) : this;
+  int count = 0;
+  if (const_shift_count(phase, this, &count) && (count & (BitsPerJavaLong - 1)) == 0) {
+    // Shift by a multiple of 64 does nothing
+    return in(1);
+  }
+  return this;
 }
 
 //------------------------------Ideal------------------------------------------
@@ -1294,6 +1341,17 @@ Node *URShiftLNode::Ideal(PhaseGVN *phase, bool can_reshape) {
       phase->type(shl->in(2)) == t2 )
     return new AndLNode( shl->in(1), phase->longcon(mask) );
 
+  // Check for (x >> n) >>> 63. Replace with (x >>> 63)
+  Node *shr = in(1);
+  if ( shr->Opcode() == Op_RShiftL ) {
+    Node *in11 = shr->in(1);
+    Node *in12 = shr->in(2);
+    const TypeLong *t11 = phase->type(in11)->isa_long();
+    const TypeInt *t12 = phase->type(in12)->isa_int();
+    if ( t11 && t2 && t2->is_con(63) && t12 && t12->is_con() ) {
+      return new URShiftLNode(in11, phase->intcon(63));
+    }
+  }
   return NULL;
 }
 
@@ -1410,3 +1468,145 @@ uint MulAddS2INode::hash() const {
   return (uintptr_t)in(1) + (uintptr_t)in(2) + (uintptr_t)in(3) + (uintptr_t)in(4) + Opcode();
 }
 
+//------------------------------Rotate Operations ------------------------------
+
+Node* RotateLeftNode::Identity(PhaseGVN* phase) {
+  const Type* t1 = phase->type(in(1));
+  if (t1 == Type::TOP) {
+    return this;
+  }
+  int count = 0;
+  assert(t1->isa_int() || t1->isa_long(), "Unexpected type");
+  int mask = (t1->isa_int() ? BitsPerJavaInteger : BitsPerJavaLong) - 1;
+  if (const_shift_count(phase, this, &count) && (count & mask) == 0) {
+    // Rotate by a multiple of 32/64 does nothing
+    return in(1);
+  }
+  return this;
+}
+
+const Type* RotateLeftNode::Value(PhaseGVN* phase) const {
+  const Type* t1 = phase->type(in(1));
+  const Type* t2 = phase->type(in(2));
+  // Either input is TOP ==> the result is TOP
+  if (t1 == Type::TOP || t2 == Type::TOP) {
+    return Type::TOP;
+  }
+
+  if (t1->isa_int()) {
+    const TypeInt* r1 = t1->is_int();
+    const TypeInt* r2 = t2->is_int();
+
+    // Left input is ZERO ==> the result is ZERO.
+    if (r1 == TypeInt::ZERO) {
+      return TypeInt::ZERO;
+    }
+    // Rotate by zero does nothing
+    if (r2 == TypeInt::ZERO) {
+      return r1;
+    }
+    if (r1->is_con() && r2->is_con()) {
+      juint r1_con = (juint)r1->get_con();
+      juint shift = (juint)(r2->get_con()) & (juint)(BitsPerJavaInteger - 1); // semantics of Java shifts
+      return TypeInt::make((r1_con << shift) | (r1_con >> (32 - shift)));
+    }
+    return TypeInt::INT;
+  } else {
+    assert(t1->isa_long(), "Type must be a long");
+    const TypeLong* r1 = t1->is_long();
+    const TypeInt*  r2 = t2->is_int();
+
+    // Left input is ZERO ==> the result is ZERO.
+    if (r1 == TypeLong::ZERO) {
+      return TypeLong::ZERO;
+    }
+    // Rotate by zero does nothing
+    if (r2 == TypeInt::ZERO) {
+      return r1;
+    }
+    if (r1->is_con() && r2->is_con()) {
+      julong r1_con = (julong)r1->get_con();
+      julong shift = (julong)(r2->get_con()) & (julong)(BitsPerJavaLong - 1); // semantics of Java shifts
+      return TypeLong::make((r1_con << shift) | (r1_con >> (64 - shift)));
+    }
+    return TypeLong::LONG;
+  }
+}
+
+Node* RotateLeftNode::Ideal(PhaseGVN *phase, bool can_reshape) {
+  const Type* t1 = phase->type(in(1));
+  const Type* t2 = phase->type(in(2));
+  if (t2->isa_int() && t2->is_int()->is_con()) {
+    if (t1->isa_int()) {
+      int lshift = t2->is_int()->get_con() & 31;
+      return new RotateRightNode(in(1), phase->intcon(32 - (lshift & 31)), TypeInt::INT);
+    } else if (t1 != Type::TOP) {
+      assert(t1->isa_long(), "Type must be a long");
+      int lshift = t2->is_int()->get_con() & 63;
+      return new RotateRightNode(in(1), phase->intcon(64 - (lshift & 63)), TypeLong::LONG);
+    }
+  }
+  return NULL;
+}
+
+Node* RotateRightNode::Identity(PhaseGVN* phase) {
+  const Type* t1 = phase->type(in(1));
+  if (t1 == Type::TOP) {
+    return this;
+  }
+  int count = 0;
+  assert(t1->isa_int() || t1->isa_long(), "Unexpected type");
+  int mask = (t1->isa_int() ? BitsPerJavaInteger : BitsPerJavaLong) - 1;
+  if (const_shift_count(phase, this, &count) && (count & mask) == 0) {
+    // Rotate by a multiple of 32/64 does nothing
+    return in(1);
+  }
+  return this;
+}
+
+const Type* RotateRightNode::Value(PhaseGVN* phase) const {
+  const Type* t1 = phase->type(in(1));
+  const Type* t2 = phase->type(in(2));
+  // Either input is TOP ==> the result is TOP
+  if (t1 == Type::TOP || t2 == Type::TOP) {
+    return Type::TOP;
+  }
+
+  if (t1->isa_int()) {
+    const TypeInt* r1 = t1->is_int();
+    const TypeInt* r2 = t2->is_int();
+
+    // Left input is ZERO ==> the result is ZERO.
+    if (r1 == TypeInt::ZERO) {
+      return TypeInt::ZERO;
+    }
+    // Rotate by zero does nothing
+    if (r2 == TypeInt::ZERO) {
+      return r1;
+    }
+    if (r1->is_con() && r2->is_con()) {
+      juint r1_con = (juint)r1->get_con();
+      juint shift = (juint)(r2->get_con()) & (juint)(BitsPerJavaInteger - 1); // semantics of Java shifts
+      return TypeInt::make((r1_con >> shift) | (r1_con << (32 - shift)));
+    }
+    return TypeInt::INT;
+  } else {
+    assert(t1->isa_long(), "Type must be a long");
+    const TypeLong* r1 = t1->is_long();
+    const TypeInt*  r2 = t2->is_int();
+    // Left input is ZERO ==> the result is ZERO.
+    if (r1 == TypeLong::ZERO) {
+      return TypeLong::ZERO;
+    }
+    // Rotate by zero does nothing
+    if (r2 == TypeInt::ZERO) {
+      return r1;
+    }
+    if (r1->is_con() && r2->is_con()) {
+      julong r1_con = (julong)r1->get_con();
+      julong shift = (julong)(r2->get_con()) & (julong)(BitsPerJavaLong - 1); // semantics of Java shifts
+      return TypeLong::make((r1_con >> shift) | (r1_con << (64 - shift)));
+    }
+    return TypeLong::LONG;
+  }
+}
