@@ -59,6 +59,8 @@ public:
     Ideal_Loop,                       // Find idealized trip-counted loops
     Macro_Expand,                     // Expand macro nodes
     Peephole,                         // Apply peephole optimizations
+    Vector,
+    Output,
     last_phase
   };
 
@@ -74,6 +76,10 @@ public:
         _t_incrInline_igvn,
         _t_incrInline_pru,
         _t_incrInline_inline,
+      _t_vector,
+        _t_vector_elimination,
+          _t_vector_igvn,
+          _t_vector_pru,
       _t_renumberLive,
       _t_idealLoop,
       _t_idealLoopVerify,
@@ -83,6 +89,7 @@ public:
       _t_barrierExpand,
       _t_graphReshaping,
     _t_matcher,
+      _t_postselect_cleanup,
     _t_scheduler,
     _t_registerAllocation,
       _t_ctorChaitin,
@@ -131,7 +138,7 @@ protected:
   // Object; if you wish to check an Object you need to load the Object's
   // class prior to coming here.
   // Used in GraphKit and PhaseMacroExpand
-  static Node* gen_subtype_check(Node* subklass, Node* superklass, Node** ctrl, MergeMemNode* mem, PhaseGVN* gvn);
+  static Node* gen_subtype_check(Node* subklass, Node* superklass, Node** ctrl, Node* mem, PhaseGVN& gvn);
 
 public:
   Compile * C;
