@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -291,11 +291,11 @@ class SuperWord : public ResourceObj {
   void unrolling_analysis(int &local_loop_unroll_factor);
 
   // Accessors for SWPointer
-  PhaseIdealLoop* phase()          { return _phase; }
-  IdealLoopTree* lpt()             { return _lpt; }
-  PhiNode* iv()                    { return _iv; }
+  PhaseIdealLoop* phase() const    { return _phase; }
+  IdealLoopTree* lpt() const       { return _lpt; }
+  PhiNode* iv() const              { return _iv; }
 
-  bool early_return()              { return _early_return; }
+  bool early_return() const        { return _early_return; }
 
 #ifndef PRODUCT
   bool     is_debug()              { return _vector_loop_debug > 0; }
@@ -310,7 +310,8 @@ class SuperWord : public ResourceObj {
   bool     do_reserve_copy()       { return _do_reserve_copy; }
  private:
   IdealLoopTree* _lpt;             // Current loop tree node
-  LoopNode*      _lp;              // Current LoopNode
+  CountedLoopNode* _lp;            // Current CountedLoopNode
+  CountedLoopEndNode* _pre_loop_end; // Current CountedLoopEndNode of pre loop
   Node*          _bb;              // Current basic block
   PhiNode*       _iv;              // Induction var
   bool           _race_possible;   // In cases where SDMU is true
@@ -330,14 +331,33 @@ class SuperWord : public ResourceObj {
   Arena* arena()                   { return _arena; }
 
   Node* bb()                       { return _bb; }
-  void  set_bb(Node* bb)           { _bb = bb; }
-
+  void set_bb(Node* bb)            { _bb = bb; }
   void set_lpt(IdealLoopTree* lpt) { _lpt = lpt; }
+  CountedLoopNode* lp() const      { return _lp; }
+  void set_lp(CountedLoopNode* lp) {
+    _lp = lp;
+    _iv = lp->as_CountedLoop()->phi()->as_Phi();
+  }
+  int iv_stride() const            { return lp()->stride_con(); }
 
-  LoopNode* lp()                   { return _lp; }
-  void      set_lp(LoopNode* lp)   { _lp = lp;
-                                     _iv = lp->as_CountedLoop()->phi()->as_Phi(); }
-  int      iv_stride()             { return lp()->as_CountedLoop()->stride_con(); }
+  CountedLoopNode* pre_loop_head() const {
+    assert(_pre_loop_end != NULL && _pre_loop_end->loopnode() != NULL, "should find head from pre loop end");
+    return _pre_loop_end->loopnode();
+  }
+  void set_pre_loop_end(CountedLoopEndNode* pre_loop_end) {
+    assert(pre_loop_end, "must be valid");
+    _pre_loop_end = pre_loop_end;
+  }
+  CountedLoopEndNode* pre_loop_end() const {
+#ifdef ASSERT
+    assert(_lp != NULL, "sanity");
+    assert(_pre_loop_end != NULL, "should be set when fetched");
+    Node* found_pre_end = find_pre_loop_end(_lp);
+    assert(_pre_loop_end == found_pre_end && _pre_loop_end == pre_loop_head()->loopexit(),
+           "should find the pre loop end and must be the same result");
+#endif
+    return _pre_loop_end;
+  }
 
   int vector_width(Node* n) {
     BasicType bt = velt_basic_type(n);
@@ -370,6 +390,9 @@ class SuperWord : public ResourceObj {
   // Ensure node_info contains element "i"
   void grow_node_info(int i) { if (i >= _node_info.length()) _node_info.at_put_grow(i, SWNodeInfo::initial); }
 
+  // should we align vector memory references on this platform?
+  bool vectors_should_be_aligned() { return !Matcher::misaligned_vectors_ok() || AlignVector; }
+
   // memory alignment for a node
   int alignment(Node* n)                     { return _node_info.adr_at(bb_idx(n))->_alignment; }
   void set_alignment(Node* n, int a)         { int i = bb_idx(n); grow_node_info(i); _node_info.adr_at(i)->_alignment = a; }
@@ -528,7 +551,7 @@ class SuperWord : public ResourceObj {
   // to align_to_ref will be a position zero in the vector.
   void align_initial_loop_index(MemNode* align_to_ref);
   // Find pre loop end from main loop.  Returns null if none.
-  CountedLoopEndNode* get_pre_loop_end(CountedLoopNode *cl);
+  CountedLoopEndNode* find_pre_loop_end(CountedLoopNode *cl) const;
   // Is the use of d1 in u1 at the same operand position as d2 in u2?
   bool opnd_positions_match(Node* d1, Node* u1, Node* d2, Node* u2);
   void init();
@@ -556,19 +579,23 @@ class SWPointer {
 
   Node* _base;               // NULL if unsafe nonheap reference
   Node* _adr;                // address pointer
-  jint  _scale;              // multiplier for iv (in bytes), 0 if no loop iv
-  jint  _offset;             // constant offset (in bytes)
+  int   _scale;              // multiplier for iv (in bytes), 0 if no loop iv
+  int   _offset;             // constant offset (in bytes)
+
   Node* _invar;              // invariant offset (in bytes), NULL if none
   bool  _negate_invar;       // if true then use: (0 - _invar)
+  Node* _invar_scale;        // multiplier for invariant
+
   Node_Stack* _nstack;       // stack used to record a swpointer trace of variants
   bool        _analyze_only; // Used in loop unrolling only for swpointer trace
   uint        _stack_idx;    // Used in loop unrolling only for swpointer trace
 
-  PhaseIdealLoop* phase() { return _slp->phase(); }
-  IdealLoopTree*  lpt()   { return _slp->lpt(); }
-  PhiNode*        iv()    { return _slp->iv();  } // Induction var
+  PhaseIdealLoop* phase() const { return _slp->phase(); }
+  IdealLoopTree*  lpt() const   { return _slp->lpt(); }
+  PhiNode*        iv() const    { return _slp->iv();  } // Induction var
 
-  bool invariant(Node* n);
+  bool is_main_loop_member(Node* n) const;
+  bool invariant(Node* n) const;
 
   // Match: k*iv + offset
   bool scaled_iv_plus_offset(Node* n);
@@ -600,17 +627,22 @@ class SWPointer {
   int   scale_in_bytes()   { return _scale; }
   Node* invar()            { return _invar; }
   bool  negate_invar()     { return _negate_invar; }
+  Node* invar_scale()      { return _invar_scale; }
   int   offset_in_bytes()  { return _offset; }
   int   memory_size()      { return _mem->memory_size(); }
   Node_Stack* node_stack() { return _nstack; }
 
   // Comparable?
+  bool invar_equals(SWPointer& q) {
+      return (_invar        == q._invar   &&
+              _invar_scale  == q._invar_scale &&
+              _negate_invar == q._negate_invar);
+  }
+
   int cmp(SWPointer& q) {
     if (valid() && q.valid() &&
         (_adr == q._adr || (_base == _adr && q._base == q._adr)) &&
-        _scale == q._scale   &&
-        _invar == q._invar   &&
-        _negate_invar == q._negate_invar) {
+        _scale == q._scale   && invar_equals(q)) {
       bool overlap = q._offset <   _offset +   memory_size() &&
                        _offset < q._offset + q.memory_size();
       return overlap ? Equal : (_offset < q._offset ? Less : Greater);
@@ -635,7 +667,7 @@ class SWPointer {
     SuperWord*   _slp;
     static int   _depth;
     int _depth_save;
-    void print_depth();
+    void print_depth() const;
     int  depth() const    { return _depth; }
     void set_depth(int d) { _depth = d; }
     void inc_depth()      { _depth++;}
@@ -661,7 +693,7 @@ class SWPointer {
     void ctor_5(Node* adr, Node* base,  int i);
     void ctor_6(Node* mem);
 
-    void invariant_1(Node *n, Node *n_c);
+    void invariant_1(Node *n, Node *n_c) const;
 
     void scaled_iv_plus_offset_1(Node* n);
     void scaled_iv_plus_offset_2(Node* n);
@@ -680,7 +712,7 @@ class SWPointer {
     void scaled_iv_6(Node* n, int scale);
     void scaled_iv_7(Node* n);
     void scaled_iv_8(Node* n, SWPointer* tmp);
-    void scaled_iv_9(Node* n, int _scale, int _offset, int mult);
+    void scaled_iv_9(Node* n, int _scale, int _offset, Node* _invar, bool _negate_invar);
     void scaled_iv_10(Node* n);
 
     void offset_plus_k_1(Node* n);
