@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -29,6 +29,7 @@
 #include "memory/allocation.hpp"
 #include "memory/iterator.hpp"
 #include "oops/oop.hpp"
+#include "oops/oopHandle.hpp"
 #include "oops/oopsHierarchy.hpp"
 #include "runtime/frame.hpp"
 #include "runtime/handles.hpp"
@@ -46,6 +47,8 @@ class JvmtiManageCapabilities;
 class JvmtiEnv;
 class JvmtiThreadState;
 
+class OopStorage;
+
 #define JVMTI_SUPPORT_FLAG(key)                                           \
   private:                                                                \
   static bool  _##key;                                                    \
@@ -161,6 +164,10 @@ class JvmtiExport : public AllStatic {
   static void post_dynamic_code_generated_internal(const char *name, const void *code_begin, const void *code_end) NOT_JVMTI_RETURN;
 
   static void post_class_unload_internal(const char *name) NOT_JVMTI_RETURN;
+
+  static void initialize_oop_storage() NOT_JVMTI_RETURN;
+  static OopStorage* jvmti_oop_storage();
+  static OopStorage* weak_tag_storage();
  private:
 
   // GenerateEvents support to allow posting of CompiledMethodLoad and
@@ -187,6 +194,13 @@ class JvmtiExport : public AllStatic {
   // dependency information is complete or not.
   static bool _all_dependencies_are_recorded;
 
+  static void post_method_exit_inner(JavaThread* thread,
+                                     methodHandle& mh,
+                                     JvmtiThreadState *state,
+                                     bool exception_exit,
+                                     frame current_frame,
+                                     jvalue& value);
+
  public:
   inline static bool has_redefined_a_class() {
     JVMTI_ONLY(return _redefinition_count != 0);
@@ -394,9 +408,6 @@ class JvmtiExport : public AllStatic {
   static void cleanup_thread             (JavaThread* thread) NOT_JVMTI_RETURN;
   static void clear_detected_exception   (JavaThread* thread) NOT_JVMTI_RETURN;
 
-  static void oops_do(OopClosure* f) NOT_JVMTI_RETURN;
-  static void weak_oops_do(BoolObjectClosure* b, OopClosure* f) NOT_JVMTI_RETURN;
-
   static void transition_pending_onload_raw_monitors() NOT_JVMTI_RETURN;
 
 #if INCLUDE_SERVICES
@@ -491,24 +502,18 @@ class JvmtiDynamicCodeEventCollector : public JvmtiEventCollector {
 //
 class JvmtiObjectAllocEventCollector : public JvmtiEventCollector {
  protected:
-  GrowableArray<oop>* _allocated;      // field to record collected allocated object oop.
+  GrowableArray<OopHandle>* _allocated;      // field to record collected allocated object oop.
   bool _enable;                   // This flag is enabled in constructor if set up in the thread state
                                   // and disabled in destructor before posting event. To avoid
                                   // collection of objects allocated while running java code inside
                                   // agent post_X_object_alloc() event handler.
   void (*_post_callback)(JavaThread*, oop); // what callback to use when destroying the collector.
 
-  //GC support
-  void oops_do(OopClosure* f);
-
   friend class JvmtiExport;
 
   // Record allocated object oop.
   inline void record_allocation(oop obj);
 
-  //GC support
-  static void oops_do_for_all_threads(OopClosure* f);
-
  public:
   JvmtiObjectAllocEventCollector()  NOT_JVMTI_RETURN;
 
