@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,6 +28,7 @@
 #include "jvmtifiles/jvmti.h"
 #include "memory/allocation.hpp"
 #include "prims/jvmtiEventController.hpp"
+#include "prims/jvmtiExport.hpp"
 #include "runtime/thread.hpp"
 #include "utilities/growableArray.hpp"
 
@@ -39,6 +40,9 @@ class JvmtiEnvBase;
 class JvmtiEnvThreadState;
 class JvmtiDynamicCodeEventCollector;
 
+class JvmtiDeferredEvent;
+class JvmtiDeferredEventQueue;
+
 enum JvmtiClassLoadKind {
   jvmti_class_load_kind_load = 100,
   jvmti_class_load_kind_retransform,
@@ -75,6 +79,8 @@ class JvmtiThreadState : public CHeapObj<mtInternal> {
  private:
   friend class JvmtiEnv;
   JavaThread        *_thread;
+  // Jvmti Events that cannot be posted in their current context.
+  JvmtiDeferredEventQueue* _jvmti_event_queue;
   bool              _hide_single_stepping;
   bool              _pending_step_for_popframe;
   bool              _pending_step_for_earlyret;
@@ -94,6 +100,7 @@ class JvmtiThreadState : public CHeapObj<mtInternal> {
   // info to the class file load hook event handler.
   Klass*                _class_being_redefined;
   JvmtiClassLoadKind    _class_load_kind;
+  GrowableArray<Klass*>* _classes_being_redefined;
 
   // This is only valid when is_interp_only_mode() returns true
   int               _cur_stack_depth;
@@ -239,22 +246,20 @@ class JvmtiThreadState : public CHeapObj<mtInternal> {
     return _class_load_kind;
   }
 
+  // Get the classes that are currently being redefined by this thread.
+  inline GrowableArray<Klass*>* get_classes_being_redefined() {
+    return _classes_being_redefined;
+  }
+
+  inline void set_classes_being_redefined(GrowableArray<Klass*>* redef_classes) {
+    _classes_being_redefined = redef_classes;
+  }
+
   // RedefineClasses support
   // The bug 6214132 caused the verification to fail.
   //
-  // Below is the detailed description of the fix approach taken:
-  // 1. What's done in RedefineClasses() before verification:
-  //  a) A reference to the class being redefined (_the_class) and a
-  //     reference to new version of the class (_scratch_class) are
-  //     saved here for use during the bytecode verification phase of
-  //     RedefineClasses. See RedefineVerifyMark for how these fields
-  //     are managed.
-  //   b) The _java_mirror field from _the_class is copied to the
-  //     _java_mirror field in _scratch_class. This means that a jclass
-  //     returned for _the_class or _scratch_class will refer to the
-  //     same Java mirror. The verifier will see the "one true mirror"
-  //     for the class being verified.
-  // 2. What is done at verification:
+  // What is done at verification:
+  //   (This seems to only apply to the old verifier.)
   //   When the verifier makes calls into the VM to ask questions about
   //   the class being verified, it will pass the jclass to JVM_* functions.
   //   The jclass is always pointing to the mirror of _the_class.
@@ -384,33 +389,16 @@ class JvmtiThreadState : public CHeapObj<mtInternal> {
   static ByteSize earlyret_oop_offset()   { return byte_offset_of(JvmtiThreadState, _earlyret_oop); }
   static ByteSize earlyret_value_offset() { return byte_offset_of(JvmtiThreadState, _earlyret_value); }
 
-  void oops_do(OopClosure* f) NOT_JVMTI_RETURN; // GC support
+  void oops_do(OopClosure* f, CodeBlobClosure* cf) NOT_JVMTI_RETURN; // GC support
+  void nmethods_do(CodeBlobClosure* cf) NOT_JVMTI_RETURN;
 
 public:
   void set_should_post_on_exceptions(bool val) { _thread->set_should_post_on_exceptions_flag(val ? JNI_TRUE : JNI_FALSE); }
-};
 
-class RedefineVerifyMark : public StackObj {
- private:
-  JvmtiThreadState* _state;
-  Klass*            _scratch_class;
-  OopHandle         _scratch_mirror;
-
- public:
-  RedefineVerifyMark(Klass* the_class, Klass* scratch_class,
-                     JvmtiThreadState *state) : _state(state), _scratch_class(scratch_class)
-  {
-    _state->set_class_versions_map(the_class, scratch_class);
-    _scratch_mirror = _scratch_class->java_mirror_handle();
-    _scratch_class->set_java_mirror_handle(the_class->java_mirror_handle());
-  }
-
-  ~RedefineVerifyMark() {
-    // Restore the scratch class's mirror, so when scratch_class is removed
-    // the correct mirror pointing to it can be cleared.
-    _scratch_class->set_java_mirror_handle(_scratch_mirror);
-    _state->clear_class_versions_map();
-  }
+  // Thread local event queue, which doesn't require taking the Service_lock.
+  void enqueue_event(JvmtiDeferredEvent* event) NOT_JVMTI_RETURN;
+  void post_events(JvmtiEnv* env);
+  void run_nmethod_entry_barriers();
 };
 
 #endif // SHARE_PRIMS_JVMTITHREADSTATE_HPP
