@@ -509,6 +509,7 @@ protected:
   JavaThread* _biased_locker;
   BiasedLocking::Condition _status_code;
   traceid _biased_locker_id;
+  bool _executed;
 
 public:
   RevokeOneBias(Handle obj, JavaThread* requesting_thread, JavaThread* biased_locker)
@@ -517,10 +518,14 @@ public:
     , _requesting_thread(requesting_thread)
     , _biased_locker(biased_locker)
     , _status_code(BiasedLocking::NOT_BIASED)
-    , _biased_locker_id(0) {}
+    , _biased_locker_id(0)
+    , _executed(false) {}
+
+  bool executed() { return _executed; }
 
   void do_thread(Thread* target) {
     assert(target == _biased_locker, "Wrong thread");
+    _executed = true;
 
     oop o = _obj();
     markWord mark = o->mark();
@@ -622,11 +627,11 @@ BiasedLocking::Condition BiasedLocking::single_revoke_with_handshake(Handle obj,
                                      p2i(biaser), p2i(obj()));
 
   RevokeOneBias revoke(obj, requester, biaser);
-  bool executed = Handshake::execute_direct(&revoke, biaser);
+  Handshake::execute(&revoke, biaser);
   if (revoke.status_code() == NOT_REVOKED) {
     return NOT_REVOKED;
   }
-  if (executed) {
+  if (revoke.executed()) {
     log_info(biasedlocking, handshake)("Handshake revocation for object " INTPTR_FORMAT " succeeded. Bias was %srevoked",
                                        p2i(obj()), (revoke.status_code() == BIAS_REVOKED ? "" : "already "));
     if (event.should_commit() && revoke.status_code() == BIAS_REVOKED) {
@@ -668,7 +673,7 @@ BiasedLocking::Condition BiasedLocking::single_revoke_with_handshake(Handle obj,
 void BiasedLocking::walk_stack_and_revoke(oop obj, JavaThread* biased_locker) {
   Thread* cur = Thread::current();
   assert(!SafepointSynchronize::is_at_safepoint(), "this should always be executed outside safepoints");
-  assert(cur == biased_locker || cur == biased_locker->active_handshaker(), "wrong thread");
+  assert(biased_locker->is_handshake_safe_for(cur), "wrong thread");
 
   markWord mark = obj->mark();
   assert(mark.biased_locker() == biased_locker &&
@@ -725,8 +730,7 @@ void BiasedLocking::walk_stack_and_revoke(oop obj, JavaThread* biased_locker) {
 }
 
 void BiasedLocking::revoke_own_lock(Handle obj, TRAPS) {
-  assert(THREAD->is_Java_thread(), "must be called by a JavaThread");
-  JavaThread* thread = (JavaThread*)THREAD;
+  JavaThread* thread = THREAD->as_Java_thread();
 
   markWord mark = obj->mark();
 
@@ -740,7 +744,7 @@ void BiasedLocking::revoke_own_lock(Handle obj, TRAPS) {
   ResourceMark rm;
   log_info(biasedlocking)("Revoking bias by walking my own stack:");
   EventBiasedLockSelfRevocation event;
-  BiasedLocking::walk_stack_and_revoke(obj(), (JavaThread*) thread);
+  BiasedLocking::walk_stack_and_revoke(obj(), thread);
   thread->set_cached_monitor_info(NULL);
   assert(!obj->mark().has_bias_pattern(), "invariant");
   if (event.should_commit()) {
@@ -830,7 +834,7 @@ void BiasedLocking::revoke(Handle obj, TRAPS) {
         }
         return;
       } else {
-        BiasedLocking::Condition cond = single_revoke_with_handshake(obj, (JavaThread*)THREAD, blt);
+        BiasedLocking::Condition cond = single_revoke_with_handshake(obj, THREAD->as_Java_thread(), blt);
         if (cond != NOT_REVOKED) {
           return;
         }
@@ -839,7 +843,7 @@ void BiasedLocking::revoke(Handle obj, TRAPS) {
       assert((heuristics == HR_BULK_REVOKE) ||
          (heuristics == HR_BULK_REBIAS), "?");
       EventBiasedLockClassRevocation event;
-      VM_BulkRevokeBias bulk_revoke(&obj, (JavaThread*)THREAD,
+      VM_BulkRevokeBias bulk_revoke(&obj, THREAD->as_Java_thread(),
                                     (heuristics == HR_BULK_REBIAS));
       VMThread::execute(&bulk_revoke);
       if (event.should_commit()) {
@@ -903,11 +907,12 @@ void BiasedLocking::preserve_marks() {
   // monitors in a prepass and, if they are biased, preserve their
   // mark words here. This should be a relatively small set of objects
   // especially compared to the number of objects in the heap.
-  _preserved_mark_stack = new (ResourceObj::C_HEAP, mtInternal) GrowableArray<markWord>(10, true);
-  _preserved_oop_stack = new (ResourceObj::C_HEAP, mtInternal) GrowableArray<Handle>(10, true);
+  _preserved_mark_stack = new (ResourceObj::C_HEAP, mtGC) GrowableArray<markWord>(10, mtGC);
+  _preserved_oop_stack = new (ResourceObj::C_HEAP, mtGC) GrowableArray<Handle>(10, mtGC);
 
-  ResourceMark rm;
   Thread* cur = Thread::current();
+  ResourceMark rm(cur);
+
   for (JavaThreadIteratorWithHandle jtiwh; JavaThread *thread = jtiwh.next(); ) {
     if (thread->has_last_Java_frame()) {
       RegisterMap rm(thread);
