@@ -30,37 +30,54 @@
 #include "runtime/handshake.hpp"
 #include "runtime/interfaceSupport.inline.hpp"
 #include "runtime/osThread.hpp"
-#include "runtime/semaphore.inline.hpp"
+#include "runtime/stackWatermarkSet.hpp"
 #include "runtime/task.hpp"
 #include "runtime/thread.hpp"
 #include "runtime/vmThread.hpp"
 #include "utilities/formatBuffer.hpp"
+#include "utilities/filterQueue.inline.hpp"
 #include "utilities/preserveException.hpp"
 
+class HandshakeOperation : public CHeapObj<mtThread> {
+  friend class HandshakeState;
+ protected:
+  HandshakeClosure*   _handshake_cl;
+  // Keeps track of emitted and completed handshake operations.
+  // Once it reaches zero all handshake operations have been performed.
+  int32_t             _pending_threads;
+  JavaThread*         _target;
 
-class HandshakeOperation: public StackObj {
-  HandshakeClosure* _handshake_cl;
-  int32_t _pending_threads;
-  bool _executed;
-  bool _is_direct;
-public:
-  HandshakeOperation(HandshakeClosure* cl, bool is_direct = false) :
+  // Must use AsyncHandshakeOperation when using AsyncHandshakeClosure.
+  HandshakeOperation(AsyncHandshakeClosure* cl, JavaThread* target) :
     _handshake_cl(cl),
     _pending_threads(1),
-    _executed(false),
-    _is_direct(is_direct) {}
+    _target(target) {}
 
+ public:
+  HandshakeOperation(HandshakeClosure* cl, JavaThread* target) :
+    _handshake_cl(cl),
+    _pending_threads(1),
+    _target(target) {}
+  virtual ~HandshakeOperation() {}
   void do_handshake(JavaThread* thread);
   bool is_completed() {
     int32_t val = Atomic::load(&_pending_threads);
     assert(val >= 0, "_pending_threads=%d cannot be negative", val);
     return val == 0;
   }
-  void add_target_count(int count) { Atomic::add(&_pending_threads, count, memory_order_relaxed); }
-  bool executed() const { return _executed; }
-  const char* name() { return _handshake_cl->name(); }
+  void add_target_count(int count) { Atomic::add(&_pending_threads, count); }
+  const char* name()               { return _handshake_cl->name(); }
+  bool is_async()                  { return _handshake_cl->is_async(); }
+};
 
-  bool is_direct() { return _is_direct; }
+class AsyncHandshakeOperation : public HandshakeOperation {
+ private:
+  jlong _start_time_ns;
+ public:
+  AsyncHandshakeOperation(AsyncHandshakeClosure* cl, JavaThread* target, jlong start_ns)
+    : HandshakeOperation(cl, target), _start_time_ns(start_ns) {}
+  virtual ~AsyncHandshakeOperation() { delete _handshake_cl; }
+  jlong start_time() const           { return _start_time_ns; }
 };
 
 // Performing handshakes requires a custom yielding strategy because without it
@@ -79,7 +96,6 @@ class HandshakeSpinYield : public StackObj {
   int _result_count[2][HandshakeState::_number_states];
   int _prev_result_pos;
 
-  int prev_result_pos() { return _prev_result_pos & 0x1; }
   int current_result_pos() { return (_prev_result_pos + 1) & 0x1; }
 
   void wait_raw(jlong now) {
@@ -141,7 +157,7 @@ class HandshakeSpinYield : public StackObj {
       // On UP this is always true.
       Thread* self = Thread::current();
       if (self->is_Java_thread()) {
-        wait_blocked((JavaThread*)self, now);
+        wait_blocked(self->as_Java_thread(), now);
       } else {
         wait_raw(now);
       }
@@ -176,8 +192,8 @@ bool VM_Handshake::handshake_has_timed_out(jlong start_time) {
 
 void VM_Handshake::handle_timeout() {
   LogStreamHandle(Warning, handshake) log_stream;
-  for (JavaThreadIteratorWithHandle jtiwh; JavaThread *thr = jtiwh.next(); ) {
-    if (thr->has_handshake()) {
+  for (JavaThreadIteratorWithHandle jtiwh; JavaThread* thr = jtiwh.next(); ) {
+    if (thr->handshake_state()->has_operation()) {
       log_stream.print("Thread " PTR_FORMAT " has not cleared its handshake op", p2i(thr));
       thr->print_thread_state_on(&log_stream);
     }
@@ -186,78 +202,34 @@ void VM_Handshake::handle_timeout() {
   fatal("Handshake operation timed out");
 }
 
-static void log_handshake_info(jlong start_time_ns, const char* name, int targets, int vmt_executed, const char* extra = NULL) {
-  if (start_time_ns != 0) {
+static void log_handshake_info(jlong start_time_ns, const char* name, int targets, int emitted_handshakes_executed, const char* extra = NULL) {
+  if (log_is_enabled(Info, handshake)) {
     jlong completion_time = os::javaTimeNanos() - start_time_ns;
-    log_info(handshake)("Handshake \"%s\", Targeted threads: %d, Executed by targeted threads: %d, Total completion time: " JLONG_FORMAT " ns%s%s",
+    log_info(handshake)("Handshake \"%s\", Targeted threads: %d, Executed by requesting thread: %d, Total completion time: " JLONG_FORMAT " ns%s%s",
                         name, targets,
-                        targets - vmt_executed,
+                        emitted_handshakes_executed,
                         completion_time,
                         extra != NULL ? ", " : "",
                         extra != NULL ? extra : "");
   }
 }
 
-class VM_HandshakeOneThread: public VM_Handshake {
-  JavaThread* _target;
- public:
-  VM_HandshakeOneThread(HandshakeOperation* op, JavaThread* target) :
-    VM_Handshake(op), _target(target) {}
-
-  void doit() {
-    jlong start_time_ns = os::javaTimeNanos();
-
-    ThreadsListHandle tlh;
-    if (tlh.includes(_target)) {
-      _target->set_handshake_operation(_op);
-    } else {
-      log_handshake_info(start_time_ns, _op->name(), 0, 0, "(thread dead)");
-      return;
-    }
-
-    log_trace(handshake)("JavaThread " INTPTR_FORMAT " signaled, begin attempt to process by VMThtread", p2i(_target));
-    HandshakeState::ProcessResult pr = HandshakeState::_no_operation;
-    HandshakeSpinYield hsy(start_time_ns);
-    do {
-      if (handshake_has_timed_out(start_time_ns)) {
-        handle_timeout();
-      }
-      pr = _target->handshake_try_process(_op);
-      hsy.add_result(pr);
-      hsy.process();
-    } while (!_op->is_completed());
-
-    // This pairs up with the release store in do_handshake(). It prevents future
-    // loads from floating above the load of _pending_threads in is_completed()
-    // and thus prevents reading stale data modified in the handshake closure
-    // by the Handshakee.
-    OrderAccess::acquire();
-
-    log_handshake_info(start_time_ns, _op->name(), 1, (pr == HandshakeState::_success) ? 1 : 0);
-  }
-
-  VMOp_Type type() const { return VMOp_HandshakeOneThread; }
-
-  bool executed() const { return _op->executed(); }
-};
-
 class VM_HandshakeAllThreads: public VM_Handshake {
  public:
   VM_HandshakeAllThreads(HandshakeOperation* op) : VM_Handshake(op) {}
 
   void doit() {
     jlong start_time_ns = os::javaTimeNanos();
-    int handshake_executed_by_vm_thread = 0;
 
     JavaThreadIteratorWithHandle jtiwh;
     int number_of_threads_issued = 0;
-    for (JavaThread *thr = jtiwh.next(); thr != NULL; thr = jtiwh.next()) {
-      thr->set_handshake_operation(_op);
+    for (JavaThread* thr = jtiwh.next(); thr != NULL; thr = jtiwh.next()) {
+      thr->handshake_state()->add_operation(_op);
       number_of_threads_issued++;
     }
 
     if (number_of_threads_issued < 1) {
-      log_handshake_info(start_time_ns, _op->name(), 0, 0);
+      log_handshake_info(start_time_ns, _op->name(), 0, 0, "no threads alive");
       return;
     }
     // _op was created with a count == 1 so don't double count.
@@ -265,6 +237,9 @@ class VM_HandshakeAllThreads: public VM_Handshake {
 
     log_trace(handshake)("Threads signaled, begin processing blocked threads by VMThread");
     HandshakeSpinYield hsy(start_time_ns);
+    // Keeps count on how many of own emitted handshakes
+    // this thread execute.
+    int emitted_handshakes_executed = 0;
     do {
       // Check if handshake operation has timed out
       if (handshake_has_timed_out(start_time_ns)) {
@@ -273,16 +248,16 @@ class VM_HandshakeAllThreads: public VM_Handshake {
 
       // Have VM thread perform the handshake operation for blocked threads.
       // Observing a blocked state may of course be transient but the processing is guarded
-      // by semaphores and we optimistically begin by working on the blocked threads
+      // by mutexes and we optimistically begin by working on the blocked threads
       jtiwh.rewind();
-      for (JavaThread *thr = jtiwh.next(); thr != NULL; thr = jtiwh.next()) {
+      for (JavaThread* thr = jtiwh.next(); thr != NULL; thr = jtiwh.next()) {
         // A new thread on the ThreadsList will not have an operation,
         // hence it is skipped in handshake_try_process.
-        HandshakeState::ProcessResult pr = thr->handshake_try_process(_op);
-        if (pr == HandshakeState::_success) {
-          handshake_executed_by_vm_thread++;
-        }
+        HandshakeState::ProcessResult pr = thr->handshake_state()->try_process(_op);
         hsy.add_result(pr);
+        if (pr == HandshakeState::_succeeded) {
+          emitted_handshakes_executed++;
+        }
       }
       hsy.process();
     } while (!_op->is_completed());
@@ -293,7 +268,7 @@ class VM_HandshakeAllThreads: public VM_Handshake {
     // by the Handshakee.
     OrderAccess::acquire();
 
-    log_handshake_info(start_time_ns, _op->name(), number_of_threads_issued, handshake_executed_by_vm_thread);
+    log_handshake_info(start_time_ns, _op->name(), number_of_threads_issued, emitted_handshakes_executed);
   }
 
   VMOp_Type type() const { return VMOp_HandshakeAllThreads; }
@@ -307,8 +282,8 @@ void HandshakeOperation::do_handshake(JavaThread* thread) {
 
   // Only actually execute the operation for non terminated threads.
   if (!thread->is_terminated()) {
+    NoSafepointVerifier nsv;
     _handshake_cl->do_thread(thread);
-    _executed = true;
   }
 
   if (start_time_ns != 0) {
@@ -327,41 +302,44 @@ void HandshakeOperation::do_handshake(JavaThread* thread) {
   // It is no longer safe to refer to 'this' as the VMThread/Handshaker may have destroyed this operation
 }
 
-void Handshake::execute(HandshakeClosure* thread_cl) {
-  HandshakeOperation cto(thread_cl);
+void Handshake::execute(HandshakeClosure* hs_cl) {
+  HandshakeOperation cto(hs_cl, NULL);
   VM_HandshakeAllThreads handshake(&cto);
   VMThread::execute(&handshake);
 }
 
-bool Handshake::execute(HandshakeClosure* thread_cl, JavaThread* target) {
-  HandshakeOperation cto(thread_cl);
-  VM_HandshakeOneThread handshake(&cto, target);
-  VMThread::execute(&handshake);
-  return handshake.executed();
-}
-
-bool Handshake::execute_direct(HandshakeClosure* thread_cl, JavaThread* target) {
+void Handshake::execute(HandshakeClosure* hs_cl, JavaThread* target) {
   JavaThread* self = JavaThread::current();
-  HandshakeOperation op(thread_cl, /*is_direct*/ true);
+  HandshakeOperation op(hs_cl, target);
 
   jlong start_time_ns = os::javaTimeNanos();
 
   ThreadsListHandle tlh;
   if (tlh.includes(target)) {
-    target->set_handshake_operation(&op);
+    target->handshake_state()->add_operation(&op);
   } else {
-    log_handshake_info(start_time_ns, op.name(), 0, 0, "(thread dead)");
-    return false;
+    char buf[128];
+    jio_snprintf(buf, sizeof(buf),  "(thread= " INTPTR_FORMAT " dead)", p2i(target));
+    log_handshake_info(start_time_ns, op.name(), 0, 0, buf);
+    return;
   }
 
-  HandshakeState::ProcessResult pr =  HandshakeState::_no_operation;
+  // Keeps count on how many of own emitted handshakes
+  // this thread execute.
+  int emitted_handshakes_executed = 0;
   HandshakeSpinYield hsy(start_time_ns);
   while (!op.is_completed()) {
-    HandshakeState::ProcessResult pr = target->handshake_try_process(&op);
+    HandshakeState::ProcessResult pr = target->handshake_state()->try_process(&op);
+    if (pr == HandshakeState::_succeeded) {
+      emitted_handshakes_executed++;
+    }
+    if (op.is_completed()) {
+      break;
+    }
     hsy.add_result(pr);
     // Check for pending handshakes to avoid possible deadlocks where our
     // target is trying to handshake us.
-    if (SafepointMechanism::should_block(self)) {
+    if (SafepointMechanism::should_process(self)) {
       ThreadBlockInVM tbivm(self);
     }
     hsy.process();
@@ -373,73 +351,90 @@ bool Handshake::execute_direct(HandshakeClosure* thread_cl, JavaThread* target)
   // by the Handshakee.
   OrderAccess::acquire();
 
-  log_handshake_info(start_time_ns, op.name(), 1,  (pr == HandshakeState::_success) ? 1 : 0);
-
-  return op.executed();
+  log_handshake_info(start_time_ns, op.name(), 1, emitted_handshakes_executed);
 }
 
-HandshakeState::HandshakeState() :
-  _operation(NULL),
-  _operation_direct(NULL),
-  _handshake_turn_sem(1),
-  _processing_sem(1),
-  _thread_in_process_handshake(false)
-{
-  DEBUG_ONLY(_active_handshaker = NULL;)
-}
+void Handshake::execute(AsyncHandshakeClosure* hs_cl, JavaThread* target) {
+  jlong start_time_ns = os::javaTimeNanos();
+  AsyncHandshakeOperation* op = new AsyncHandshakeOperation(hs_cl, target, start_time_ns);
 
-void HandshakeState::set_operation(HandshakeOperation* op) {
-  if (!op->is_direct()) {
-    assert(Thread::current()->is_VM_thread(), "should be the VMThread");
-    _operation = op;
+  ThreadsListHandle tlh;
+  if (tlh.includes(target)) {
+    target->handshake_state()->add_operation(op);
   } else {
-    assert(Thread::current()->is_Java_thread(), "should be a JavaThread");
-    // Serialize direct handshakes so that only one proceeds at a time for a given target
-    _handshake_turn_sem.wait_with_safepoint_check(JavaThread::current());
-    _operation_direct = op;
+    log_handshake_info(start_time_ns, op->name(), 0, 0, "(thread dead)");
+    delete op;
   }
+}
+
+HandshakeState::HandshakeState(JavaThread* target) :
+  _handshakee(target),
+  _queue(),
+  _lock(Monitor::leaf, "HandshakeState", Mutex::_allow_vm_block_flag, Monitor::_safepoint_check_never),
+  _active_handshaker()
+{
+}
+
+void HandshakeState::add_operation(HandshakeOperation* op) {
+  // Adds are done lock free and so is arming.
+  // Calling this method with lock held is considered an error.
+  assert(!_lock.owned_by_self(), "Lock should not be held");
+  _queue.push(op);
   SafepointMechanism::arm_local_poll_release(_handshakee);
 }
 
-void HandshakeState::clear_handshake(bool is_direct) {
-  if (!is_direct) {
-    _operation = NULL;
-  } else {
-    _operation_direct = NULL;
-    _handshake_turn_sem.signal();
-  }
+HandshakeOperation* HandshakeState::pop_for_self() {
+  assert(_handshakee == Thread::current(), "Must be called by self");
+  assert(_lock.owned_by_self(), "Lock must be held");
+  return _queue.pop();
+};
+
+static bool non_self_queue_filter(HandshakeOperation* op) {
+  return !op->is_async();
 }
 
-void HandshakeState::process_self_inner() {
+bool HandshakeState::have_non_self_executable_operation() {
+  assert(_handshakee != Thread::current(), "Must not be called by self");
+  assert(_lock.owned_by_self(), "Lock must be held");
+  return _queue.contains(non_self_queue_filter);
+}
+
+HandshakeOperation* HandshakeState::pop() {
+  assert(_handshakee != Thread::current(), "Must not be called by self");
+  assert(_lock.owned_by_self(), "Lock must be held");
+  return _queue.pop(non_self_queue_filter);
+};
+
+void HandshakeState::process_by_self() {
   assert(Thread::current() == _handshakee, "should call from _handshakee");
   assert(!_handshakee->is_terminated(), "should not be a terminated thread");
   assert(_handshakee->thread_state() != _thread_blocked, "should not be in a blocked state");
   assert(_handshakee->thread_state() != _thread_in_native, "should not be in native");
-  JavaThread* self = _handshakee;
+  ThreadInVMForHandshake tivm(_handshakee);
+  {
+    NoSafepointVerifier nsv;
+    process_self_inner();
+  }
+}
 
-  do {
-    ThreadInVMForHandshake tivm(self);
-    if (!_processing_sem.trywait()) {
-      _processing_sem.wait_with_safepoint_check(self);
-    }
-    if (has_operation()) {
-      HandleMark hm(self);
-      CautiouslyPreserveExceptionMark pem(self);
-      HandshakeOperation * op = _operation;
-      if (op != NULL) {
-        // Disarm before executing the operation
-        clear_handshake(/*is_direct*/ false);
-        op->do_handshake(self);
-      }
-      op = _operation_direct;
-      if (op != NULL) {
-        // Disarm before executing the operation
-        clear_handshake(/*is_direct*/ true);
-        op->do_handshake(self);
+void HandshakeState::process_self_inner() {
+  while (should_process()) {
+    HandleMark hm(_handshakee);
+    CautiouslyPreserveExceptionMark pem(_handshakee);
+    MutexLocker ml(&_lock, Mutex::_no_safepoint_check_flag);
+    HandshakeOperation* op = pop_for_self();
+    if (op != NULL) {
+      assert(op->_target == NULL || op->_target == Thread::current(), "Wrong thread");
+      bool async = op->is_async();
+      log_trace(handshake)("Proc handshake %s " INTPTR_FORMAT " on " INTPTR_FORMAT " by self",
+                           async ? "asynchronous" : "synchronous", p2i(op), p2i(_handshakee));
+      op->do_handshake(_handshakee);
+      if (async) {
+        log_handshake_info(((AsyncHandshakeOperation*)op)->start_time(), op->name(), 1, 0, "asynchronous");
+        delete op;
       }
     }
-    _processing_sem.signal();
-  } while (has_operation());
+  }
 }
 
 bool HandshakeState::can_process_handshake() {
@@ -450,9 +445,6 @@ bool HandshakeState::can_process_handshake() {
 
 bool HandshakeState::possibly_can_process_handshake() {
   // Note that this method is allowed to produce false positives.
-  if (_handshakee->is_ext_suspended()) {
-    return true;
-  }
   if (_handshakee->is_terminated()) {
     return true;
   }
@@ -469,60 +461,82 @@ bool HandshakeState::possibly_can_process_handshake() {
   }
 }
 
-bool HandshakeState::claim_handshake(bool is_direct) {
-  if (!_processing_sem.trywait()) {
+bool HandshakeState::claim_handshake() {
+  if (!_lock.try_lock()) {
     return false;
   }
-  if (has_specific_operation(is_direct)){
-    return true;
+  // Operations are added lock free and then the poll is armed.
+  // If all handshake operations for the handshakee are finished and someone
+  // just adds an operation we may see it here. But if the handshakee is not
+  // armed yet it is not safe to proceed.
+  if (have_non_self_executable_operation()) {
+    if (SafepointMechanism::local_poll_armed(_handshakee)) {
+      return true;
+    }
   }
-  _processing_sem.signal();
+  _lock.unlock();
   return false;
 }
 
-HandshakeState::ProcessResult HandshakeState::try_process(HandshakeOperation* op) {
-  bool is_direct = op->is_direct();
-
-  if (!has_specific_operation(is_direct)){
+HandshakeState::ProcessResult HandshakeState::try_process(HandshakeOperation* match_op) {
+  if (!has_operation()) {
     // JT has already cleared its handshake
-    return _no_operation;
+    return HandshakeState::_no_operation;
   }
 
   if (!possibly_can_process_handshake()) {
     // JT is observed in an unsafe state, it must notice the handshake itself
-    return _not_safe;
+    return HandshakeState::_not_safe;
   }
 
-  // Claim the semaphore if there still an operation to be executed.
-  if (!claim_handshake(is_direct)) {
-    return _state_busy;
+  // Claim the mutex if there still an operation to be executed.
+  if (!claim_handshake()) {
+    return HandshakeState::_claim_failed;
   }
 
-  // Check if the handshake operation is the same as the one we meant to execute. The
-  // handshake could have been already processed by the handshakee and a new handshake
-  // by another JavaThread might be in progress.
-  if (is_direct && op != _operation_direct) {
-    _processing_sem.signal();
-    return _no_operation;
-  }
-
-  // If we own the semaphore at this point and while owning the semaphore
+  // If we own the mutex at this point and while owning the mutex we
   // can observe a safe state the thread cannot possibly continue without
-  // getting caught by the semaphore.
-  ProcessResult pr = _not_safe;
-  if (can_process_handshake()) {
-    guarantee(!_processing_sem.trywait(), "we should already own the semaphore");
-    log_trace(handshake)("Processing handshake by %s", Thread::current()->is_VM_thread() ? "VMThread" : "Handshaker");
-    DEBUG_ONLY(_active_handshaker = Thread::current();)
-    op->do_handshake(_handshakee);
-    DEBUG_ONLY(_active_handshaker = NULL;)
-    // Disarm after we have executed the operation.
-    clear_handshake(is_direct);
-    pr = _success;
+  // getting caught by the mutex.
+  if (!can_process_handshake()) {
+    _lock.unlock();
+    return HandshakeState::_not_safe;
   }
 
-  // Release the thread
-  _processing_sem.signal();
+  Thread* current_thread = Thread::current();
 
-  return pr;
+  HandshakeState::ProcessResult pr_ret = HandshakeState::_processed;
+  int executed = 0;
+
+  do {
+    HandshakeOperation* op = pop();
+    if (op != NULL) {
+      assert(SafepointMechanism::local_poll_armed(_handshakee), "Must be");
+      assert(op->_target == NULL || _handshakee == op->_target, "Wrong thread");
+      log_trace(handshake)("Processing handshake " INTPTR_FORMAT " by %s(%s)", p2i(op),
+                           op == match_op ? "handshaker" : "cooperative",
+                           current_thread->is_VM_thread() ? "VM Thread" : "JavaThread");
+
+      if (op == match_op) {
+        pr_ret = HandshakeState::_succeeded;
+      }
+
+      if (!_handshakee->is_terminated()) {
+        StackWatermarkSet::start_processing(_handshakee, StackWatermarkKind::gc);
+      }
+
+      _active_handshaker = current_thread;
+      op->do_handshake(_handshakee);
+      _active_handshaker = NULL;
+
+      executed++;
+    }
+  } while (have_non_self_executable_operation());
+
+  _lock.unlock();
+
+  log_trace(handshake)("%s(" INTPTR_FORMAT ") executed %d ops for JavaThread: " INTPTR_FORMAT " %s target op: " INTPTR_FORMAT,
+                       current_thread->is_VM_thread() ? "VM Thread" : "JavaThread",
+                       p2i(current_thread), executed, p2i(_handshakee),
+                       pr_ret == HandshakeState::_succeeded ? "including" : "excluding", p2i(match_op));
+  return pr_ret;
 }
