@@ -49,14 +49,6 @@ inline int PerfDataManager::count() {
   return _all->length();
 }
 
-inline int PerfDataManager::sampled_count() {
-  return _sampled->length();
-}
-
-inline int PerfDataManager::constants_count() {
-  return _constants->length();
-}
-
 inline bool PerfDataManager::exists(const char* name) {
   if (_all != NULL) {
     return _all->contains(name);
