@@ -69,7 +69,7 @@ bool KlassStream::eos() {
 
 
 GrowableArray<FilteredField*> *FilteredFieldsMap::_filtered_fields =
-  new (ResourceObj::C_HEAP, mtInternal) GrowableArray<FilteredField*>(3,true);
+  new (ResourceObj::C_HEAP, mtServiceability) GrowableArray<FilteredField*>(3, mtServiceability);
 
 
 void FilteredFieldsMap::initialize() {
