@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -70,7 +70,7 @@ class SafepointSynchronize : AllStatic {
 
   // The enums are listed in the order of the tasks when done serially.
   enum SafepointCleanupTasks {
-    SAFEPOINT_CLEANUP_DEFLATE_MONITORS,
+    SAFEPOINT_CLEANUP_LAZY_ROOT_PROCESSING,
     SAFEPOINT_CLEANUP_UPDATE_INLINE_CACHES,
     SAFEPOINT_CLEANUP_COMPILATION_POLICY,
     SAFEPOINT_CLEANUP_SYMBOL_TABLE_REHASH,
@@ -108,7 +108,6 @@ class SafepointSynchronize : AllStatic {
   // JavaThreads that need to block for the safepoint will stop on the
   // _wait_barrier, where they can quickly be started again.
   static WaitBarrier* _wait_barrier;
-  static long         _end_of_last_safepoint;     // Time of last safepoint in milliseconds
   static julong       _coalesced_vmop_count;     // coalesced vmop count
 
   // For debug long safepoint
@@ -163,7 +162,6 @@ public:
   static void handle_polling_page_exception(JavaThread *thread);
 
   static bool is_cleanup_needed();
-  static bool is_forced_cleanup_needed();
   static void do_cleanup_tasks();
 
   static void set_is_at_safepoint()             { _state = _synchronized; }
@@ -253,8 +251,7 @@ private:
   static jlong _last_safepoint_sync_time_ns;
   static jlong _last_safepoint_cleanup_time_ns;
   static jlong _last_safepoint_end_time_ns;
-  // amount of ms since epoch
-  static jlong _last_safepoint_end_time_epoch_ms;
+
   // Relative
   static jlong _last_app_time_ns;
 
@@ -280,11 +277,11 @@ public:
   static void statistics_exit_log();
 
   static jlong time_since_last_safepoint_ms() {
-    return (os::javaTimeNanos() - _last_safepoint_end_time_ns) / (NANOUNITS / MILLIUNITS);
+    return nanos_to_millis(os::javaTimeNanos() - _last_safepoint_end_time_ns);
   }
 
-  static jlong end_of_last_safepoint_epoch_ms() {
-    return _last_safepoint_end_time_epoch_ms;
+  static jlong end_of_last_safepoint_ms() {
+    return nanos_to_millis(_last_safepoint_end_time_ns);
   }
 
   static jlong start_of_safepoint() {
