@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,6 +23,8 @@
  */
 
 #include "precompiled.hpp"
+#include "classfile/classLoaderDataGraph.inline.hpp"
+#include "classfile/javaClasses.hpp"
 #include "classfile/protectionDomainCache.hpp"
 #include "classfile/stringTable.hpp"
 #include "classfile/symbolTable.hpp"
@@ -30,6 +32,7 @@
 #include "gc/shared/oopStorage.hpp"
 #include "gc/shared/oopStorageSet.hpp"
 #include "memory/universe.hpp"
+#include "oops/oopHandle.inline.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/interfaceSupport.inline.hpp"
 #include "runtime/javaCalls.hpp"
@@ -38,6 +41,7 @@
 #include "runtime/mutexLocker.hpp"
 #include "runtime/os.hpp"
 #include "prims/jvmtiImpl.hpp"
+#include "prims/jvmtiTagMap.hpp"
 #include "prims/resolvedMethodTable.hpp"
 #include "services/diagnosticArgument.hpp"
 #include "services/diagnosticFramework.hpp"
@@ -52,6 +56,36 @@ JvmtiDeferredEvent* ServiceThread::_jvmti_event = NULL;
 // to add this field to the per-JavaThread event queue.  TODO: fix this sometime later
 JvmtiDeferredEventQueue ServiceThread::_jvmti_service_queue;
 
+// Defer releasing JavaThread OopHandle to the ServiceThread
+class OopHandleList : public CHeapObj<mtInternal> {
+  OopHandle      _handle;
+  OopHandleList* _next;
+ public:
+   OopHandleList(OopHandle h, OopHandleList* next) : _handle(h), _next(next) {}
+   ~OopHandleList() {
+     _handle.release(JavaThread::thread_oop_storage());
+   }
+   OopHandleList* next() const { return _next; }
+};
+
+static OopHandleList* _oop_handle_list = NULL;
+
+static void release_oop_handles() {
+  OopHandleList* list;
+  {
+    MutexLocker ml(Service_lock, Mutex::_no_safepoint_check_flag);
+    list = _oop_handle_list;
+    _oop_handle_list = NULL;
+  }
+  assert(!SafepointSynchronize::is_at_safepoint(), "cannot be called at a safepoint");
+
+  while (list != NULL) {
+    OopHandleList* l = list;
+    list = l->next();
+    delete l;
+  }
+}
+
 void ServiceThread::initialize() {
   EXCEPTION_MARK;
 
@@ -68,7 +102,7 @@ void ServiceThread::initialize() {
                           CHECK);
 
   {
-    MutexLocker mu(Threads_lock);
+    MutexLocker mu(THREAD, Threads_lock);
     ServiceThread* thread =  new ServiceThread(&service_thread_entry);
 
     // At this point it may be possible that no osthread was created for the
@@ -111,6 +145,9 @@ void ServiceThread::service_thread_entry(JavaThread* jt, TRAPS) {
     bool protection_domain_table_work = false;
     bool oopstorage_work = false;
     JvmtiDeferredEvent jvmti_event;
+    bool oop_handles_to_release = false;
+    bool cldg_cleanup_work = false;
+    bool jvmti_tagmap_work = false;
     {
       // Need state transition ThreadBlockInVM so that this thread
       // will be handled by safepoint correctly when this thread is
@@ -136,7 +173,10 @@ void ServiceThread::service_thread_entry(JavaThread* jt, TRAPS) {
               (resolved_method_table_work = ResolvedMethodTable::has_work()) |
               (thread_id_table_work = ThreadIdTable::has_work()) |
               (protection_domain_table_work = SystemDictionary::pd_cache_table()->has_work()) |
-              (oopstorage_work = OopStorage::has_cleanup_work_and_reset())
+              (oopstorage_work = OopStorage::has_cleanup_work_and_reset()) |
+              (oop_handles_to_release = (_oop_handle_list != NULL)) |
+              (cldg_cleanup_work = ClassLoaderDataGraph::should_clean_metaspaces_and_reset()) |
+              (jvmti_tagmap_work = JvmtiTagMap::has_object_free_events_and_reset())
              ) == 0) {
         // Wait until notified that there is some work to do.
         ml.wait();
@@ -191,6 +231,18 @@ void ServiceThread::service_thread_entry(JavaThread* jt, TRAPS) {
     if (oopstorage_work) {
       cleanup_oopstorages();
     }
+
+    if (oop_handles_to_release) {
+      release_oop_handles();
+    }
+
+    if (cldg_cleanup_work) {
+      ClassLoaderDataGraph::safepoint_and_clean_metaspaces();
+    }
+
+    if (jvmti_tagmap_work) {
+      JvmtiTagMap::flush_all_object_free_events();
+    }
   }
 }
 
@@ -204,18 +256,16 @@ void ServiceThread::enqueue_deferred_event(JvmtiDeferredEvent* event) {
   Service_lock->notify_all();
  }
 
-void ServiceThread::oops_do(OopClosure* f, CodeBlobClosure* cf) {
-  JavaThread::oops_do(f, cf);
+void ServiceThread::oops_do_no_frames(OopClosure* f, CodeBlobClosure* cf) {
+  JavaThread::oops_do_no_frames(f, cf);
   // The ServiceThread "owns" the JVMTI Deferred events, scan them here
   // to keep them alive until they are processed.
-  if (cf != NULL) {
-    if (_jvmti_event != NULL) {
-      _jvmti_event->oops_do(f, cf);
-    }
-    // Requires a lock, because threads can be adding to this queue.
-    MutexLocker ml(Service_lock, Mutex::_no_safepoint_check_flag);
-    _jvmti_service_queue.oops_do(f, cf);
+  if (_jvmti_event != NULL) {
+    _jvmti_event->oops_do(f, cf);
   }
+  // Requires a lock, because threads can be adding to this queue.
+  MutexLocker ml(Service_lock, Mutex::_no_safepoint_check_flag);
+  _jvmti_service_queue.oops_do(f, cf);
 }
 
 void ServiceThread::nmethods_do(CodeBlobClosure* cf) {
@@ -229,3 +279,10 @@ void ServiceThread::nmethods_do(CodeBlobClosure* cf) {
     _jvmti_service_queue.nmethods_do(cf);
   }
 }
+
+void ServiceThread::add_oop_handle_release(OopHandle handle) {
+  MutexLocker ml(Service_lock, Mutex::_no_safepoint_check_flag);
+  OopHandleList* new_head = new OopHandleList(handle, _oop_handle_list);
+  _oop_handle_list = new_head;
+  Service_lock->notify_all();
+}
