@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -33,7 +33,7 @@
 #include "gc/z/zBarrier.inline.hpp"
 #endif
 #if INCLUDE_SHENANDOAHGC
-#include "gc/shenandoah/shenandoahBarrierSet.hpp"
+#include "gc/shenandoah/shenandoahBarrierSet.inline.hpp"
 #endif
 
 StackValue* StackValue::create_stack_value(const frame* fr, const RegisterMap* reg_map, ScopeValue* sv) {
@@ -41,12 +41,6 @@ StackValue* StackValue::create_stack_value(const frame* fr, const RegisterMap* r
     // Stack or register value
     Location loc = ((LocationValue *)sv)->location();
 
-#ifdef SPARC
-    // %%%%% Callee-save floats will NOT be working on a Sparc until we
-    // handle the case of a 2 floats in a single double register.
-    assert( !(loc.is_register() && loc.type() == Location::float_in_dbl), "Sparc does not handle callee-save floats yet" );
-#endif // SPARC
-
     // First find address of value
 
     address value_addr = loc.is_register()
@@ -104,8 +98,12 @@ StackValue* StackValue::create_stack_value(const frame* fr, const RegisterMap* r
         // The callee has no clue whether the register holds an int,
         // long or is unused.  He always saves a long.  Here we know
         // a long was saved, but we only want an int back.  Narrow the
-        // saved long to the int that the JVM wants.
-        value.noop =  (narrowOop) *(julong*) value_addr;
+        // saved long to the int that the JVM wants.  We can't just
+        // use narrow_oop_cast directly, because we don't know what
+        // the high bits of the value might be.
+        static_assert(sizeof(narrowOop) == sizeof(juint), "size mismatch");
+        juint narrow_value = (juint) *(julong*)value_addr;
+        value.noop = CompressedOops::narrow_oop_cast(narrow_value);
       } else {
         value.noop = *(narrowOop*) value_addr;
       }
@@ -152,8 +150,12 @@ StackValue* StackValue::create_stack_value(const frame* fr, const RegisterMap* r
       value.ji = *(jint*)value_addr;
       return new StackValue(value.p);
     }
-    case Location::invalid:
+    case Location::invalid: {
       return new StackValue();
+    }
+    case Location::vector: {
+      ShouldNotReachHere(); // should be handled by Deoptimization::realloc_objects()
+    }
     default:
       ShouldNotReachHere();
     }
@@ -184,8 +186,10 @@ StackValue* StackValue::create_stack_value(const frame* fr, const RegisterMap* r
   } else if (sv->is_object()) { // Scalar replaced object in compiled frame
     Handle ov = ((ObjectValue *)sv)->value();
     return new StackValue(ov, (ov.is_null()) ? 1 : 0);
+  } else if (sv->is_marker()) {
+    // Should never need to directly construct a marker.
+    ShouldNotReachHere();
   }
-
   // Unknown ScopeValue type
   ShouldNotReachHere();
   return new StackValue((intptr_t) 0);   // dummy
@@ -221,8 +225,8 @@ void StackValue::print_on(outputStream* st) const {
       } else {
         st->print("NULL");
       }
-      st->print(" <" INTPTR_FORMAT ">", p2i((address)_handle_value()));
-     break;
+      st->print(" <" INTPTR_FORMAT ">", p2i(_handle_value()));
+      break;
 
     case T_CONFLICT:
      st->print("conflict");
