@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,6 +32,7 @@
 #include "oops/methodData.hpp"
 #include "oops/oop.inline.hpp"
 #include "prims/jvmtiThreadState.hpp"
+#include "prims/methodHandles.hpp"
 #include "runtime/frame.inline.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/monitorChunk.hpp"
@@ -71,8 +72,10 @@ void vframeArrayElement::fill_in(compiledVFrame* vf, bool realloc_failures) {
   int index;
 
   {
-    ResourceMark rm;
-    HandleMark hm;
+    Thread* current_thread = Thread::current();
+    ResourceMark rm(current_thread);
+    HandleMark hm(current_thread);
+
     // Get the monitors off-stack
 
     GrowableArray<MonitorInfo*>* list = vf->monitors();
@@ -174,7 +177,7 @@ void vframeArrayElement::unpack_on_stack(int caller_actual_parameters,
                                          bool is_top_frame,
                                          bool is_bottom_frame,
                                          int exec_mode) {
-  JavaThread* thread = (JavaThread*) Thread::current();
+  JavaThread* thread = JavaThread::current();
 
   bool realloc_failure_exception = thread->frames_to_pop_failed_realloc() > 0;
 
@@ -229,14 +232,7 @@ void vframeArrayElement::unpack_on_stack(int caller_actual_parameters,
         (thread->has_pending_popframe() || thread->popframe_forcing_deopt_reexecution())) {
       if (thread->has_pending_popframe()) {
         // Pop top frame after deoptimization
-#ifndef CC_INTERP
         pc = Interpreter::remove_activation_preserving_args_entry();
-#else
-        // Do an uncommon trap type entry. c++ interpreter will know
-        // to pop frame and preserve the args
-        pc = Interpreter::deopt_entry(vtos, 0);
-        use_next_mdp = false;
-#endif
       } else {
         // Reexecute invoke in top frame
         pc = Interpreter::deopt_entry(vtos, 0);
@@ -246,11 +242,10 @@ void vframeArrayElement::unpack_on_stack(int caller_actual_parameters,
         // Deoptimization::fetch_unroll_info_helper
         popframe_preserved_args_size_in_words = in_words(thread->popframe_preserved_args_size_in_words());
       }
-    } else if (!realloc_failure_exception && JvmtiExport::can_force_early_return() && state != NULL && state->is_earlyret_pending()) {
+    } else if (!realloc_failure_exception && JvmtiExport::can_force_early_return() && state != NULL &&
+               state->is_earlyret_pending()) {
       // Force early return from top frame after deoptimization
-#ifndef CC_INTERP
       pc = Interpreter::remove_activation_early_entry(state->earlyret_tos());
-#endif
     } else {
       if (realloc_failure_exception && JvmtiExport::can_force_early_return() && state != NULL && state->is_earlyret_pending()) {
         state->clr_earlyret_pending();
