@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -33,6 +33,7 @@
 #include "interpreter/oopMapCache.hpp"
 #include "oops/instanceKlass.hpp"
 #include "oops/oop.inline.hpp"
+#include "prims/jvmtiDeferredUpdates.hpp"
 #include "runtime/basicLock.hpp"
 #include "runtime/frame.inline.hpp"
 #include "runtime/handles.inline.hpp"
@@ -64,7 +65,7 @@ StackValueCollection* compiledVFrame::locals() const {
 
   // Replace the original values with any stores that have been
   // performed through compiledVFrame::update_locals.
-  GrowableArray<jvmtiDeferredLocalVariableSet*>* list = thread()->deferred_locals();
+  GrowableArray<jvmtiDeferredLocalVariableSet*>* list = JvmtiDeferredUpdates::deferred_locals(thread());
   if (list != NULL ) {
     // In real life this never happens or is typically a single element search
     for (int i = 0; i < list->length(); i++) {
@@ -97,13 +98,13 @@ void compiledVFrame::update_stack(BasicType type, int index, jvalue value) {
 void compiledVFrame::update_monitor(int index, MonitorInfo* val) {
   assert(index >= 0, "out of bounds");
   jvalue value;
-  value.l = (jobject) val->owner();
+  value.l = cast_from_oop<jobject>(val->owner());
   update_deferred_value(T_OBJECT, index + method()->max_locals() + method()->max_stack(), value);
 }
 
 void compiledVFrame::update_deferred_value(BasicType type, int index, jvalue value) {
   assert(fr().is_deoptimized_frame(), "frame must be scheduled for deoptimization");
-  GrowableArray<jvmtiDeferredLocalVariableSet*>* deferred = thread()->deferred_locals();
+  GrowableArray<jvmtiDeferredLocalVariableSet*>* deferred = JvmtiDeferredUpdates::deferred_locals(thread());
   jvmtiDeferredLocalVariableSet* locals = NULL;
   if (deferred != NULL ) {
     // See if this vframe has already had locals with deferred writes
@@ -117,8 +118,8 @@ void compiledVFrame::update_deferred_value(BasicType type, int index, jvalue val
   } else {
     // No deferred updates pending for this thread.
     // allocate in C heap
-    deferred =  new(ResourceObj::C_HEAP, mtCompiler) GrowableArray<jvmtiDeferredLocalVariableSet*> (1, true);
-    thread()->set_deferred_locals(deferred);
+    JvmtiDeferredUpdates::create_for(thread());
+    deferred = JvmtiDeferredUpdates::deferred_locals(thread());
   }
   if (locals == NULL) {
     locals = new jvmtiDeferredLocalVariableSet(method(), bci(), fr().id(), vframe_id());
@@ -128,6 +129,56 @@ void compiledVFrame::update_deferred_value(BasicType type, int index, jvalue val
   locals->set_value_at(index, type, value);
 }
 
+// After object deoptimization, that is object reallocation and relocking, we
+// create deferred updates for all objects in scope. No new update will be
+// created if a deferred update already exists. It is not easy to see how this
+// is achieved: the deoptimized objects are in the arrays returned by locals(),
+// expressions(), and monitors(). For each object in these arrays we create a
+// deferred updated. If an update already exists, then it will override the
+// corresponding deoptimized object returned in one of the arrays. So the
+// original update is kept.
+void compiledVFrame::create_deferred_updates_after_object_deoptimization() {
+  // locals
+  GrowableArray<ScopeValue*>* scopeLocals = scope()->locals();
+  StackValueCollection* lcls = locals();
+  if (lcls != NULL) {
+    for (int i2 = 0; i2 < lcls->size(); i2++) {
+      StackValue* var = lcls->at(i2);
+      if (var->type() == T_OBJECT && scopeLocals->at(i2)->is_object()) {
+        jvalue val;
+        val.l = cast_from_oop<jobject>(lcls->at(i2)->get_obj()());
+        update_local(T_OBJECT, i2, val);
+      }
+    }
+  }
+
+  // expressions
+  GrowableArray<ScopeValue*>* scopeExpressions = scope()->expressions();
+  StackValueCollection* exprs = expressions();
+  if (exprs != NULL) {
+    for (int i2 = 0; i2 < exprs->size(); i2++) {
+      StackValue* var = exprs->at(i2);
+      if (var->type() == T_OBJECT && scopeExpressions->at(i2)->is_object()) {
+        jvalue val;
+        val.l = cast_from_oop<jobject>(exprs->at(i2)->get_obj()());
+        update_stack(T_OBJECT, i2, val);
+      }
+    }
+  }
+
+  // monitors
+  GrowableArray<MonitorInfo*>* mtrs = monitors();
+  if (mtrs != NULL) {
+    for (int i2 = 0; i2 < mtrs->length(); i2++) {
+      if (mtrs->at(i2)->eliminated()) {
+        assert(!mtrs->at(i2)->owner_is_scalar_replaced(),
+               "reallocation failure, should not update");
+        update_monitor(i2, mtrs->at(i2));
+      }
+    }
+  }
+}
+
 StackValueCollection* compiledVFrame::expressions() const {
   // Natives has no scope
   if (scope() == NULL) return new StackValueCollection(0);
@@ -144,7 +195,7 @@ StackValueCollection* compiledVFrame::expressions() const {
 
   // Replace the original values with any stores that have been
   // performed through compiledVFrame::update_stack.
-  GrowableArray<jvmtiDeferredLocalVariableSet*>* list = thread()->deferred_locals();
+  GrowableArray<jvmtiDeferredLocalVariableSet*>* list = JvmtiDeferredUpdates::deferred_locals(thread());
   if (list != NULL ) {
     // In real life this never happens or is typically a single element search
     for (int i = 0; i < list->length(); i++) {
@@ -218,7 +269,7 @@ GrowableArray<MonitorInfo*>* compiledVFrame::monitors() const {
 
   // Replace the original values with any stores that have been
   // performed through compiledVFrame::update_monitors.
-  GrowableArray<jvmtiDeferredLocalVariableSet*>* list = thread()->deferred_locals();
+  GrowableArray<jvmtiDeferredLocalVariableSet*>* list = JvmtiDeferredUpdates::deferred_locals(thread());
   if (list != NULL ) {
     // In real life this never happens or is typically a single element search
     for (int i = 0; i < list->length(); i++) {
@@ -309,6 +360,24 @@ bool compiledVFrame::should_reexecute() const {
   return scope()->should_reexecute();
 }
 
+bool compiledVFrame::has_ea_local_in_scope() const {
+  if (scope() == NULL) {
+    // native nmethod, all objs escape
+    assert(code()->as_nmethod()->is_native_method(), "must be native");
+    return false;
+  }
+  return (scope()->objects() != NULL) || scope()->has_ea_local_in_scope();
+}
+
+bool compiledVFrame::arg_escape() const {
+  if (scope() == NULL) {
+    // native nmethod, all objs escape
+    assert(code()->as_nmethod()->is_native_method(), "must be native");
+    return false;
+  }
+  return scope()->arg_escape();
+}
+
 vframe* compiledVFrame::sender() const {
   const frame f = fr();
   if (scope() == NULL) {
@@ -329,7 +398,8 @@ jvmtiDeferredLocalVariableSet::jvmtiDeferredLocalVariableSet(Method* method, int
   _id = id;
   _vframe_id = vframe_id;
   // Alway will need at least one, must be on C heap
-  _locals = new(ResourceObj::C_HEAP, mtCompiler) GrowableArray<jvmtiDeferredLocalVariable*> (1, true);
+  _locals = new(ResourceObj::C_HEAP, mtCompiler) GrowableArray<jvmtiDeferredLocalVariable*> (1, mtCompiler);
+  _objects_are_deoptimized = false;
 }
 
 jvmtiDeferredLocalVariableSet::~jvmtiDeferredLocalVariableSet() {
@@ -424,7 +494,11 @@ void jvmtiDeferredLocalVariableSet::update_monitors(GrowableArray<MonitorInfo*>*
     if (val->index() >= method()->max_locals() + method()->max_stack()) {
       int lock_index = val->index() - (method()->max_locals() + method()->max_stack());
       MonitorInfo* info = monitors->at(lock_index);
-      MonitorInfo* new_info = new MonitorInfo((oopDesc*)val->value().l, info->lock(), info->eliminated(), info->owner_is_scalar_replaced());
+      // Originally the owner may have been scalar replaced but as an update
+      // exists it must have been deoptimized, i.e. reallocated to the heap, and
+      // now it is considered not to be scalar replaced.
+      MonitorInfo* new_info = new MonitorInfo((oopDesc*)val->value().l, info->lock(),
+                                              info->eliminated(), false);
       monitors->at_put(lock_index, new_info);
     }
   }
