@@ -55,7 +55,7 @@ class MemoryCounter {
   inline void allocate(size_t sz) {
     Atomic::inc(&_count);
     if (sz > 0) {
-      Atomic::add(sz, &_size);
+      Atomic::add(&_size, sz);
       DEBUG_ONLY(_peak_size = MAX2(_peak_size, _size));
     }
     DEBUG_ONLY(_peak_count = MAX2(_peak_count, _count);)
@@ -66,14 +66,14 @@ class MemoryCounter {
     assert(_size >= sz, "deallocation > allocated");
     Atomic::dec(&_count);
     if (sz > 0) {
-      Atomic::sub(sz, &_size);
+      Atomic::sub(&_size, sz);
     }
   }
 
   inline void resize(ssize_t sz) {
     if (sz != 0) {
       assert(sz >= 0 || _size >= size_t(-sz), "Must be");
-      Atomic::add(size_t(sz), &_size);
+      Atomic::add(&_size, size_t(sz));
       DEBUG_ONLY(_peak_size = MAX2(_size, _peak_size);)
     }
   }
@@ -145,12 +145,6 @@ class MallocMemorySnapshot : public ResourceObj {
     return &_malloc[index];
   }
 
-  inline MallocMemory* by_index(int index) {
-    assert(index >= 0, "Index out of bound");
-    assert(index < mt_number_of_types, "Index out of bound");
-    return &_malloc[index];
-  }
-
   inline MemoryCounter* malloc_overhead() {
     return &_tracking_header;
   }
@@ -269,7 +263,7 @@ class MallocHeader {
       return;
     }
 
-    _flags = flags;
+    _flags = NMTUtil::flag_to_index(flags);
     set_size(size);
     if (level == NMT_detail) {
       size_t bucket_idx;
