@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,8 +22,8 @@
  *
  */
 
-#ifndef SHARE_VM_SERVICES_MEM_REPORTER_HPP
-#define SHARE_VM_SERVICES_MEM_REPORTER_HPP
+#ifndef SHARE_SERVICES_MEMREPORTER_HPP
+#define SHARE_SERVICES_MEMREPORTER_HPP
 
 #if INCLUDE_NMT
 
@@ -230,10 +230,9 @@ class MemDetailDiffReporter : public MemSummaryDiffReporter {
   void diff_malloc_site(const NativeCallStack* stack, size_t current_size,
     size_t currrent_count, size_t early_size, size_t early_count, MEMFLAGS flags) const;
   void diff_virtual_memory_site(const NativeCallStack* stack, size_t current_reserved,
-    size_t current_committed, size_t early_reserved, size_t early_committed) const;
+    size_t current_committed, size_t early_reserved, size_t early_committed, MEMFLAGS flag) const;
 };
 
 #endif // INCLUDE_NMT
 
-#endif
-
+#endif // SHARE_SERVICES_MEMREPORTER_HPP
