@@ -633,18 +633,6 @@ SetExecname(char **argv)
     return exec_path;
 }
 
-/*
- * BSD's implementation of CounterGet()
- */
-int64_t
-CounterGet()
-{
-    struct timeval tv;
-    gettimeofday(&tv, NULL);
-    return (tv.tv_sec * 1000000) + tv.tv_usec;
-}
-
-
 /* --- Splash Screen shared library support --- */
 
 static JavaVM* SetJavaVMValue()
@@ -725,13 +713,6 @@ void* SplashProcAddress(const char* name) {
     }
 }
 
-void SplashFreeLibrary() {
-    if (hSplashLib) {
-        dlclose(hSplashLib);
-        hSplashLib = NULL;
-    }
-}
-
 /*
  * Signature adapter for pthread_create().
  */
@@ -773,10 +754,6 @@ CallJavaMainInNewThread(jlong stack_size, void* args) {
     return rslt;
 }
 
-void SetJavaLauncherPlatformProps() {
-   /* Linux only */
-}
-
 static JavaVM* jvmInstance = NULL;
 static jboolean sameThread = JNI_FALSE; /* start VM in current thread */
 
