@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -40,7 +40,7 @@ import java.nio.charset.MalformedInputException;
 import java.nio.charset.UnmappableCharacterException;
 import java.nio.charset.UnsupportedCharsetException;
 import java.util.Arrays;
-import jdk.internal.HotSpotIntrinsicCandidate;
+import jdk.internal.vm.annotation.IntrinsicCandidate;
 import sun.nio.cs.HistoricallyNamedCharset;
 import sun.nio.cs.ArrayDecoder;
 import sun.nio.cs.ArrayEncoder;
@@ -139,7 +139,7 @@ class StringCoding {
         }
     }
 
-    @HotSpotIntrinsicCandidate
+    @IntrinsicCandidate
     public static boolean hasNegatives(byte[] ba, int off, int len) {
         for (int i = off; i < off + len; i++) {
             if (ba[i] < 0) {
@@ -523,25 +523,16 @@ class StringCoding {
     private static native void err(String msg);
 
      /* The cached Result for each thread */
-    private static final ThreadLocal<SoftReference<Result>>
+    private static final ThreadLocal<StringCoding.Result>
         resultCached = new ThreadLocal<>() {
-            protected SoftReference<Result> initialValue() {
-                return new SoftReference<>(new Result());
+            protected StringCoding.Result initialValue() {
+                return new StringCoding.Result();
             }};
-    private static Result resultCached() {
-        SoftReference<Result> sr = resultCached.get();
-        Result r;
-        if (sr == null || (r = sr.get()) == null) {
-            r = new Result();
-            resultCached.set(new SoftReference<>(r));
-        }
-        return r;
-    }
 
     ////////////////////////// ascii //////////////////////////////
 
     private static Result decodeASCII(byte[] ba, int off, int len) {
-        Result result = resultCached();
+        Result result = resultCached.get();
         if (COMPACT_STRINGS && !hasNegatives(ba, off, len)) {
             return result.with(Arrays.copyOfRange(ba, off, off + len),
                                LATIN1);
@@ -591,7 +582,7 @@ class StringCoding {
     ////////////////////////// latin1/8859_1 ///////////////////////////
 
     private static Result decodeLatin1(byte[] ba, int off, int len) {
-       Result result = resultCached();
+       Result result = resultCached.get();
        if (COMPACT_STRINGS) {
            return result.with(Arrays.copyOfRange(ba, off, off + len), LATIN1);
        } else {
@@ -599,7 +590,7 @@ class StringCoding {
        }
     }
 
-    @HotSpotIntrinsicCandidate
+    @IntrinsicCandidate
     private static int implEncodeISOArray(byte[] sa, int sp,
                                           byte[] da, int dp, int len) {
         int i = 0;
@@ -729,13 +720,13 @@ class StringCoding {
     private static Result decodeUTF8(byte[] src, int sp, int len, boolean doReplace) {
         // ascii-bais, which has a relative impact to the non-ascii-only bytes
         if (COMPACT_STRINGS && !hasNegatives(src, sp, len))
-            return resultCached().with(Arrays.copyOfRange(src, sp, sp + len),
+            return resultCached.get().with(Arrays.copyOfRange(src, sp, sp + len),
                                            LATIN1);
         return decodeUTF8_0(src, sp, len, doReplace);
     }
 
     private static Result decodeUTF8_0(byte[] src, int sp, int len, boolean doReplace) {
-        Result ret = resultCached();
+        Result ret = resultCached.get();
 
         int sl = sp + len;
         int dp = 0;
@@ -930,17 +921,14 @@ class StringCoding {
         int sp = 0;
         int sl = val.length >> 1;
         byte[] dst = new byte[sl * 3];
-        while (sp < sl) {
+        char c;
+        while (sp < sl && (c = StringUTF16.getChar(val, sp)) < '\u0080') {
             // ascii fast loop;
-            char c = StringUTF16.getChar(val, sp);
-            if (c >= '\u0080') {
-                break;
-            }
             dst[dp++] = (byte)c;
             sp++;
         }
         while (sp < sl) {
-            char c = StringUTF16.getChar(val, sp++);
+            c = StringUTF16.getChar(val, sp++);
             if (c < 0x80) {
                 dst[dp++] = (byte)c;
             } else if (c < 0x800) {
@@ -1069,7 +1057,7 @@ class StringCoding {
         } catch (CharacterCodingException x) {
             throw new IllegalArgumentException(x);  // todo
         }
-        Result ret = resultCached().with(ca, 0, cb.position());
+        Result ret = resultCached.get().with(ca, 0, cb.position());
         return new String(ret.value, ret.coder);
     }
 
