@@ -26,19 +26,21 @@
 package sun.net.www.protocol.http;
 
 import java.io.*;
-import java.net.URL;
-import java.net.ProtocolException;
 import java.net.PasswordAuthentication;
-import java.util.Arrays;
-import java.util.Random;
-
-import sun.net.www.HeaderParser;
-import sun.net.NetProperties;
+import java.net.ProtocolException;
+import java.net.URL;
+import java.security.AccessController;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.PrivilegedAction;
-import java.security.AccessController;
+import java.util.Arrays;
 import java.util.Objects;
+import java.util.Random;
+
+import sun.net.NetProperties;
+import sun.net.www.HeaderParser;
+import sun.nio.cs.ISO_8859_1;
+
 import static sun.net.www.protocol.http.HttpURLConnection.HTTP_CONNECT;
 
 /**
@@ -76,6 +78,9 @@ class DigestAuthentication extends AuthenticationInfo {
     // One instance of these may be shared among several DigestAuthentication
     // instances as a result of a single authorization (for multiple domains)
 
+    // There don't appear to be any blocking IO calls performed from
+    // within the synchronized code blocks in the Parameters class, so there don't
+    // seem to be any need to migrate it to using java.util.concurrent.locks
     static class Parameters implements java.io.Serializable {
         private static final long serialVersionUID = -3584543755194526252L;
 
@@ -296,6 +301,10 @@ class DigestAuthentication extends AuthenticationInfo {
      */
     @Override
     public boolean setHeaders(HttpURLConnection conn, HeaderParser p, String raw) {
+        // no need to synchronize here:
+        //   already locked by s.n.w.p.h.HttpURLConnection
+        assert conn.isLockHeldByCurrentThread();
+
         params.setNonce (p.findValue("nonce"));
         params.setOpaque (p.findValue("opaque"));
         params.setQop (p.findValue("qop"));
@@ -521,11 +530,7 @@ class DigestAuthentication extends AuthenticationInfo {
     };
 
     private String encode(String src, char[] passwd, MessageDigest md) {
-        try {
-            md.update(src.getBytes("ISO-8859-1"));
-        } catch (java.io.UnsupportedEncodingException uee) {
-            assert false;
-        }
+        md.update(src.getBytes(ISO_8859_1.INSTANCE));
         if (passwd != null) {
             byte[] passwdBytes = new byte[passwd.length];
             for (int i=0; i<passwd.length; i++)
