@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,7 +28,6 @@ package sun.security.ssl;
 import java.security.*;
 import java.util.*;
 import static sun.security.util.SecurityConstants.PROVIDER_VER;
-import static sun.security.provider.SunEntries.createAliases;
 
 /**
  * The JSSE provider.
@@ -54,6 +53,7 @@ import static sun.security.provider.SunEntries.createAliases;
  */
 public class SunJSSE extends java.security.Provider {
 
+    @java.io.Serial
     private static final long serialVersionUID = 3231825739635378733L;
 
     private static String info = "Sun JSSE provider" +
@@ -73,8 +73,8 @@ public class SunJSSE extends java.security.Provider {
     }
 
     private void ps(String type, String algo, String cn,
-            List<String> aliases, HashMap<String, String> attrs) {
-        putService(new Provider.Service(this, type, algo, cn, aliases, attrs));
+            List<String> a, HashMap<String, String> attrs) {
+        putService(new Provider.Service(this, type, algo, cn, a, attrs));
     }
 
     private void doRegister() {
@@ -85,18 +85,18 @@ public class SunJSSE extends java.security.Provider {
             "sun.security.ssl.KeyManagerFactoryImpl$SunX509", null, null);
         ps("KeyManagerFactory", "NewSunX509",
             "sun.security.ssl.KeyManagerFactoryImpl$X509",
-            createAliases("PKIX"), null);
+            List.of("PKIX"), null);
 
         ps("TrustManagerFactory", "SunX509",
             "sun.security.ssl.TrustManagerFactoryImpl$SimpleFactory",
             null, null);
         ps("TrustManagerFactory", "PKIX",
             "sun.security.ssl.TrustManagerFactoryImpl$PKIXFactory",
-            createAliases("SunPKIX", "X509", "X.509"), null);
+            List.of("SunPKIX", "X509", "X.509"), null);
 
         ps("SSLContext", "TLSv1",
             "sun.security.ssl.SSLContextImpl$TLS10Context",
-            createAliases("SSLv3"), null);
+            List.of("SSLv3"), null);
         ps("SSLContext", "TLSv1.1",
             "sun.security.ssl.SSLContextImpl$TLS11Context", null, null);
         ps("SSLContext", "TLSv1.2",
@@ -105,7 +105,7 @@ public class SunJSSE extends java.security.Provider {
             "sun.security.ssl.SSLContextImpl$TLS13Context", null, null);
         ps("SSLContext", "TLS",
             "sun.security.ssl.SSLContextImpl$TLSContext",
-            createAliases("SSL"), null);
+            List.of("SSL"), null);
 
         ps("SSLContext", "DTLSv1.0",
             "sun.security.ssl.SSLContextImpl$DTLS10Context", null, null);
