@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2004, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -43,6 +43,7 @@ import java.util.concurrent.atomic.AtomicLong;
 import javax.net.ssl.*;
 import sun.security.provider.certpath.AlgorithmChecker;
 import sun.security.validator.Validator;
+import sun.security.util.KnownOIDs;
 
 /**
  * The new X509 key manager implementation. The main differences to the
@@ -90,6 +91,7 @@ final class X509KeyManagerImpl extends X509ExtendedKeyManager
     // LinkedHashMap with a max size of 10
     // see LinkedHashMap JavaDocs
     private static class SizedMap<K,V> extends LinkedHashMap<K,V> {
+        @java.io.Serial
         private static final long serialVersionUID = -8211222668790986062L;
 
         @Override protected boolean removeEldestEntry(Map.Entry<K,V> eldest) {
@@ -521,14 +523,19 @@ final class X509KeyManagerImpl extends X509ExtendedKeyManager
 
         // enum constant for "tls client" check
         // valid EKU for TLS client: any, tls_client
-        CLIENT(new HashSet<String>(Arrays.asList(new String[] {
-            "2.5.29.37.0", "1.3.6.1.5.5.7.3.2" }))),
+        CLIENT(new HashSet<String>(List.of(
+            KnownOIDs.anyExtendedKeyUsage.value(),
+            KnownOIDs.clientAuth.value()
+        ))),
 
         // enum constant for "tls server" check
         // valid EKU for TLS server: any, tls_server, ns_sgc, ms_sgc
-        SERVER(new HashSet<String>(Arrays.asList(new String[] {
-            "2.5.29.37.0", "1.3.6.1.5.5.7.3.1", "2.16.840.1.113730.4.1",
-            "1.3.6.1.4.1.311.10.3.3" })));
+        SERVER(new HashSet<String>(List.of(
+            KnownOIDs.anyExtendedKeyUsage.value(),
+            KnownOIDs.serverAuth.value(),
+            KnownOIDs.NETSCAPE_ExportApproved.value(),
+            KnownOIDs.MICROSOFT_ExportApproved.value()
+        )));
 
         // set of valid EKU values for this type
         final Set<String> validEku;
