@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -163,6 +163,7 @@ public class ZoneInfo extends TimeZone {
      */
     private transient boolean dirty = false;
 
+    @java.io.Serial
     private static final long serialVersionUID = 2653134537216586139L;
 
     /**
@@ -284,7 +285,7 @@ public class ZoneInfo extends TimeZone {
             int dstoffset = tz.getOffset(msec) - rawOffset;
 
             // Check if it's in a standard-to-daylight transition.
-            if (dstoffset > 0 && tz.getOffset(msec - dstoffset) == rawoffset && type == WALL_TIME) {
+            if (dstoffset > 0 && tz.getOffset(msec - dstoffset) == rawoffset) {
                 dstoffset = 0;
             }
 
@@ -341,7 +342,7 @@ public class ZoneInfo extends TimeZone {
         return low - 1;
     }
 
-   /**
+    /**
      * Returns the difference in milliseconds between local time and
      * UTC, taking into account both the raw offset and the effect of
      * daylight savings, for the specified date and time.  This method
@@ -726,6 +727,7 @@ public class ZoneInfo extends TimeZone {
          return ZoneInfoFile.getAliasMap();
     }
 
+    @java.io.Serial
     private void readObject(ObjectInputStream stream)
             throws IOException, ClassNotFoundException {
         stream.defaultReadObject();
