@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -934,20 +934,6 @@ JNU_ReleaseStringPlatformChars(JNIEnv *env, jstring jstr, const char *str)
     free((void *)str);
 }
 
-/*
- * Export the platform dependent path canonicalization so that
- * VM can find it when loading system classes.
- * This function is also used by the instrumentation agent.
- */
-extern int canonicalize(char *path, const char *out, int len);
-
-JNIEXPORT int
-Canonicalize(JNIEnv *unused, char *orig, char *out, int len)
-{
-    /* canonicalize an already natived path */
-    return canonicalize(orig, out, len);
-}
-
 JNIEXPORT jclass JNICALL
 JNU_ClassString(JNIEnv *env)
 {
