@@ -205,7 +205,6 @@ JNIEXPORT jobject JNICALL Java_java_net_NetworkInterface_getByName0
     jboolean isCopy;
     const char* name_utf;
     char *colonP;
-    char searchName[IFNAMESIZE];
     jobject obj = NULL;
 
     if (name != NULL) {
@@ -229,15 +228,11 @@ JNIEXPORT jobject JNICALL Java_java_net_NetworkInterface_getByName0
 
     // search the list of interfaces based on name,
     // if it is virtual sub interface search with parent first.
-    strncpy(searchName, name_utf, IFNAMESIZE);
-    searchName[IFNAMESIZE - 1] = '\0';
-    colonP = strchr(searchName, ':');
-    if (colonP != NULL) {
-        *colonP = '\0';
-    }
+    colonP = strchr(name_utf, ':');
+    size_t limit = colonP != NULL ? (size_t)(colonP - name_utf) : strlen(name_utf);
     curr = ifs;
     while (curr != NULL) {
-        if (strcmp(searchName, curr->name) == 0) {
+        if (strlen(curr->name) == limit && memcmp(name_utf, curr->name, limit) == 0) {
             break;
         }
         curr = curr->next;
@@ -1296,7 +1291,8 @@ static netif *enumIPv6Interfaces(JNIEnv *env, int sock, netif *ifs) {
 static int getIndex(int sock, const char *name) {
     struct ifreq if2;
     memset((char *)&if2, 0, sizeof(if2));
-    strncpy(if2.ifr_name, name, sizeof(if2.ifr_name) - 1);
+    strncpy(if2.ifr_name, name, sizeof(if2.ifr_name));
+    if2.ifr_name[sizeof(if2.ifr_name) - 1] = 0;
 
     if (ioctl(sock, SIOCGIFINDEX, (char *)&if2) < 0) {
         return -1;
@@ -1359,7 +1355,8 @@ static int getMTU(JNIEnv *env, int sock, const char *ifname) {
 static int getFlags(int sock, const char *ifname, int *flags) {
     struct ifreq if2;
     memset((char *)&if2, 0, sizeof(if2));
-    strncpy(if2.ifr_name, ifname, sizeof(if2.ifr_name) - 1);
+    strncpy(if2.ifr_name, ifname, sizeof(if2.ifr_name));
+    if2.ifr_name[sizeof(if2.ifr_name) - 1] = 0;
 
     if (ioctl(sock, SIOCGIFFLAGS, (char *)&if2) < 0) {
         return -1;
