@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,6 +28,7 @@ package sun.nio.fs;
 import java.nio.file.*;
 import java.nio.file.attribute.*;
 import java.nio.channels.*;
+import java.nio.charset.StandardCharsets;
 import java.net.URI;
 import java.util.concurrent.ExecutorService;
 import java.io.*;
@@ -45,7 +46,7 @@ import static sun.nio.fs.WindowsConstants.*;
 class WindowsFileSystemProvider
     extends AbstractFileSystemProvider
 {
-    private static final Unsafe unsafe = Unsafe.getUnsafe();
+    private static final byte[] EMPTY_PATH = new byte[0];
 
     private final WindowsFileSystem theFileSystem;
 
@@ -333,6 +334,13 @@ class WindowsFileSystemProvider
                                 0L);
             fc.close();
         } catch (WindowsException exc) {
+            try {
+                if (exc.lastError() == ERROR_CANT_ACCESS_FILE && isUnixDomainSocket(file)) {
+                    // socket file is accessible
+                    return;
+                }
+            } catch (WindowsException ignore) {}
+
             // Windows errors are very inconsistent when the file is a directory
             // (ERROR_PATH_NOT_FOUND returned for root directories for example)
             // so we retry by attempting to open it as a directory.
@@ -345,6 +353,11 @@ class WindowsFileSystemProvider
         }
     }
 
+    private static boolean isUnixDomainSocket(WindowsPath path) throws WindowsException {
+        WindowsFileAttributes attrs = WindowsFileAttributes.get(path, false);
+        return attrs.isUnixDomainSocket();
+    }
+
     @Override
     public void checkAccess(Path obj, AccessMode... modes) throws IOException {
         WindowsPath file = WindowsPath.toWindowsPath(obj);
@@ -622,4 +635,12 @@ class WindowsFileSystemProvider
         String target = WindowsLinkSupport.readLink(link);
         return WindowsPath.createFromNormalizedPath(fs, target);
     }
+
+    @Override
+    public byte[] getSunPathForSocketFile(Path obj) {
+        WindowsPath file = WindowsPath.toWindowsPath(obj);
+        String s = file.toString();
+        return s.isEmpty() ? EMPTY_PATH : s.getBytes(StandardCharsets.UTF_8);
+    }
+
 }
