@@ -569,6 +569,8 @@ GetJavaProperties(JNIEnv* env)
         sprops.os_arch = "amd64";
 #elif defined(_X86_)
         sprops.os_arch = "x86";
+#elif defined(_M_ARM64)
+        sprops.os_arch = "aarch64";
 #else
         sprops.os_arch = "unknown";
 #endif
