@@ -28,6 +28,7 @@
 #import "java_awt_dnd_DnDConstants.h"
 
 #import <Cocoa/Cocoa.h>
+#import <JavaNativeFoundation/JavaNativeFoundation.h>
 
 #import "AWTEvent.h"
 #import "AWTView.h"
@@ -37,7 +38,6 @@
 #import "DnDUtilities.h"
 #import "ThreadUtilities.h"
 #import "LWCToolkit.h"
-#import "JNIUtilities.h"
 
 
 // When sIsJavaDragging is true Java drag gesture has been recognized and a drag is/has been initialized.
@@ -69,17 +69,9 @@ static BOOL sIsJavaDragging;
 }
 @end
 
-static jclass DataTransfererClass = NULL;
-static jclass CDragSourceContextPeerClass =  NULL;
-
-#define GET_DT_CLASS() \
-    GET_CLASS(DataTransfererClass, "sun/awt/datatransfer/DataTransferer");
-
-#define GET_DT_CLASS_RETURN(ret) \
-    GET_CLASS_RETURN(DataTransfererClass, "sun/awt/datatransfer/DataTransferer", ret);
-
-#define GET_DSCP_CLASS() \
-    GET_CLASS(CDragSourceContextPeerClass, "sun/lwawt/macosx/CDragSourceContextPeer");
+JNF_CLASS_CACHE(DataTransfererClass, "sun/awt/datatransfer/DataTransferer");
+JNF_CLASS_CACHE(CDragSourceContextPeerClass, "sun/lwawt/macosx/CDragSourceContextPeer");
+JNF_CLASS_CACHE(CImageClass, "sun/lwawt/macosx/CImage");
 
 static NSDragOperation    sDragOperation;
 static NSPoint            sDraggingLocation;
@@ -168,32 +160,32 @@ static BOOL                sNeedsEnter;
 
     // Clean up JNI refs
     if (fComponent != NULL) {
-        (*env)->DeleteGlobalRef(env, fComponent);
+        JNFDeleteGlobalRef(env, fComponent);
         fComponent = NULL;
     }
 
     if (fDragSourceContextPeer != NULL) {
-        (*env)->DeleteGlobalRef(env, fDragSourceContextPeer);
+        JNFDeleteGlobalRef(env, fDragSourceContextPeer);
         fDragSourceContextPeer = NULL;
     }
 
     if (fTransferable != NULL) {
-        (*env)->DeleteGlobalRef(env, fTransferable);
+        JNFDeleteGlobalRef(env, fTransferable);
         fTransferable = NULL;
     }
 
     if (fTriggerEvent != NULL) {
-        (*env)->DeleteGlobalRef(env, fTriggerEvent);
+        JNFDeleteGlobalRef(env, fTriggerEvent);
         fTriggerEvent = NULL;
     }
 
     if (fFormats != NULL) {
-        (*env)->DeleteGlobalRef(env, fFormats);
+        JNFDeleteGlobalRef(env, fFormats);
         fFormats = NULL;
     }
 
     if (fFormatMap != NULL) {
-        (*env)->DeleteGlobalRef(env, fFormatMap);
+        JNFDeleteGlobalRef(env, fFormatMap);
         fFormatMap = NULL;
     }
 
@@ -220,11 +212,8 @@ static BOOL                sNeedsEnter;
 //
 - (jobject)dataTransferer:(JNIEnv*)env
 {
-    GET_DT_CLASS_RETURN(NULL);
-    DECLARE_STATIC_METHOD_RETURN(getInstanceMethod, DataTransfererClass, "getInstance", "()Lsun/awt/datatransfer/DataTransferer;", NULL);
-    jobject o = (*env)->CallStaticObjectMethod(env, DataTransfererClass, getInstanceMethod);
-    CHECK_EXCEPTION();
-    return o;
+    JNF_STATIC_MEMBER_CACHE(getInstanceMethod, DataTransfererClass, "getInstance", "()Lsun/awt/datatransfer/DataTransferer;");
+    return JNFCallStaticObjectMethod(env, getInstanceMethod);
 }
 
 // Appropriated from Windows' awt_DataTransferer.cpp:
@@ -238,11 +227,9 @@ static BOOL                sNeedsEnter;
     jbyteArray data = nil;
 
     if (transferer != NULL) {
-        GET_DT_CLASS_RETURN(NULL);
-        DECLARE_METHOD_RETURN(convertDataMethod, DataTransfererClass, "convertData", "(Ljava/lang/Object;Ljava/awt/datatransfer/Transferable;JLjava/util/Map;Z)[B", NULL);
-        data = (*env)->CallObjectMethod(env, transferer, convertDataMethod, fComponent, fTransferable, format, fFormatMap, (jboolean) TRUE);
+        JNF_MEMBER_CACHE(convertDataMethod, DataTransfererClass, "convertData", "(Ljava/lang/Object;Ljava/awt/datatransfer/Transferable;JLjava/util/Map;Z)[B");
+        data = JNFCallObjectMethod(env, transferer, convertDataMethod, fComponent, fTransferable, format, fFormatMap, (jboolean) TRUE);
     }
-    CHECK_EXCEPTION();
 
     return data;
 }
@@ -568,14 +555,11 @@ static BOOL                sNeedsEnter;
         }
 
         // DragSourceContextPeer.dragDropFinished() should be called even if there was an error:
-        GET_DSCP_CLASS();
-        DECLARE_METHOD(dragDropFinishedMethod, CDragSourceContextPeerClass, "dragDropFinished", "(ZIII)V");
+        JNF_MEMBER_CACHE(dragDropFinishedMethod, CDragSourceContextPeerClass, "dragDropFinished", "(ZIII)V");
         DLog3(@"  -> posting dragDropFinished, point %f, %f", point.x, point.y);
-        (*env)->CallVoidMethod(env, fDragSourceContextPeer, dragDropFinishedMethod, success, dragOp, (jint) point.x, (jint) point.y);
-        CHECK_EXCEPTION();
-        DECLARE_METHOD(resetHoveringMethod, CDragSourceContextPeerClass, "resetHovering", "()V");
-        (*env)->CallVoidMethod(env, fDragSourceContextPeer, resetHoveringMethod); // Hust reset static variable
-        CHECK_EXCEPTION();
+        JNFCallVoidMethod(env, fDragSourceContextPeer, dragDropFinishedMethod, success, dragOp, (jint) point.x, (jint) point.y); // AWT_THREADING Safe (event)
+                JNF_MEMBER_CACHE(resetHoveringMethod, CDragSourceContextPeerClass, "resetHovering", "()V");
+        JNFCallVoidMethod(env, fDragSourceContextPeer, resetHoveringMethod); // Hust reset static variable
     } @finally {
         sNeedsEnter = NO;
         AWTToolkit.inDoDragDropLoop = NO;
@@ -593,7 +577,7 @@ static BOOL                sNeedsEnter;
 {
     AWT_ASSERT_NOT_APPKIT_THREAD;
 
-    [self performSelectorOnMainThread:@selector(doDrag) withObject:nil waitUntilDone:YES];
+    [self performSelectorOnMainThread:@selector(doDrag) withObject:nil waitUntilDone:YES]; // AWT_THREADING Safe (called from unique asynchronous thread)
 }
 
 /********************************  BEGIN NSDraggingSource Interface  ********************************/
@@ -610,10 +594,8 @@ static BOOL                sNeedsEnter;
     DLog3(@"  -> posting operationChanged, point %f, %f", point.x, point.y);
     jint modifiedModifiers = fDragKeyModifiers | fDragMouseModifiers | [DnDUtilities javaKeyModifiersForNSDragOperation:dragOp];
 
-    GET_DSCP_CLASS();
-    DECLARE_METHOD(operationChangedMethod, CDragSourceContextPeerClass, "operationChanged", "(IIII)V");
-    (*env)->CallVoidMethod(env, fDragSourceContextPeer, operationChangedMethod, targetActions, modifiedModifiers, (jint) point.x, (jint) point.y);
-    CHECK_EXCEPTION();
+    JNF_MEMBER_CACHE(operationChangedMethod, CDragSourceContextPeerClass, "operationChanged", "(IIII)V");
+    JNFCallVoidMethod(env, fDragSourceContextPeer, operationChangedMethod, targetActions, modifiedModifiers, (jint) point.x, (jint) point.y); // AWT_THREADING Safe (event)
 }
 
 - (NSDragOperation)draggingSourceOperationMaskForLocal:(BOOL)localDrag {
@@ -644,7 +626,7 @@ static BOOL                sNeedsEnter;
     //DLog4(@"[CDragSource draggedImage moved]: (%d, %d) %@\n", (int) screenPoint.x, (int) screenPoint.y, self);
     JNIEnv* env = [ThreadUtilities getJNIEnv];
 
-JNI_COCOA_ENTER(env);
+JNF_COCOA_ENTER(env);
     [AWTToolkit eventCountPlusPlus];
     // There are two things we would be interested in:
     // a) mouse pointer has moved
@@ -685,16 +667,14 @@ JNI_COCOA_ENTER(env);
         DLog4(@"[CDragSource draggedImage moved]: (%f, %f) %@\n", screenPoint.x, screenPoint.y, self);
 
         DLog3(@"  -> posting dragMotion, point %f, %f", point.x, point.y);
-        GET_DSCP_CLASS();
-        DECLARE_METHOD(dragMotionMethod, CDragSourceContextPeerClass, "dragMotion", "(IIII)V");
-        (*env)->CallVoidMethod(env, fDragSourceContextPeer, dragMotionMethod, targetActions, (jint) fModifiers, (jint) point.x, (jint) point.y);
-        CHECK_EXCEPTION();
+        JNF_MEMBER_CACHE(dragMotionMethod, CDragSourceContextPeerClass, "dragMotion", "(IIII)V");
+        JNFCallVoidMethod(env, fDragSourceContextPeer, dragMotionMethod, targetActions, (jint) fModifiers, (jint) point.x, (jint) point.y); // AWT_THREADING Safe (event)
+
         DLog3(@"  -> posting dragMouseMoved, point %f, %f", point.x, point.y);
-        DECLARE_METHOD(dragMouseMovedMethod, CDragSourceContextPeerClass, "dragMouseMoved", "(IIII)V");
-        (*env)->CallVoidMethod(env, fDragSourceContextPeer, dragMouseMovedMethod, targetActions, (jint) fModifiers, (jint) point.x, (jint) point.y);
-        CHECK_EXCEPTION();
+        JNF_MEMBER_CACHE(dragMouseMovedMethod, CDragSourceContextPeerClass, "dragMouseMoved", "(IIII)V");
+        JNFCallVoidMethod(env, fDragSourceContextPeer, dragMouseMovedMethod, targetActions, (jint) fModifiers, (jint) point.x, (jint) point.y); // AWT_THREADING Safe (event)
     }
-JNI_COCOA_EXIT(env);
+JNF_COCOA_EXIT(env);
 }
 
 - (BOOL)ignoreModifierKeysWhileDragging {
@@ -716,10 +696,8 @@ JNI_COCOA_EXIT(env);
 
     NSPoint point = [self mapNSScreenPointToJavaWithOffset:sDraggingLocation];
     DLog3(@"  -> posting dragEnter, point %f, %f", point.x, point.y);
-    GET_DSCP_CLASS();
-    DECLARE_METHOD(dragEnterMethod, CDragSourceContextPeerClass, "dragEnter", "(IIII)V");
-    (*env)->CallVoidMethod(env, fDragSourceContextPeer, dragEnterMethod, targetActions, (jint) fModifiers, (jint) point.x, (jint) point.y);
-     CHECK_EXCEPTION();
+    JNF_MEMBER_CACHE(dragEnterMethod, CDragSourceContextPeerClass, "dragEnter", "(IIII)V");
+    JNFCallVoidMethod(env, fDragSourceContextPeer, dragEnterMethod, targetActions, (jint) fModifiers, (jint) point.x, (jint) point.y); // AWT_THREADING Safe (event)
 }
 
 - (void) postDragExit {
@@ -728,11 +706,8 @@ JNI_COCOA_EXIT(env);
 
     NSPoint point = [self mapNSScreenPointToJavaWithOffset:sDraggingLocation];
     DLog3(@"  -> posting dragExit, point %f, %f", point.x, point.y);
-    GET_DSCP_CLASS();
-    DECLARE_METHOD(dragExitMethod, CDragSourceContextPeerClass, "dragExit", "(II)V");
-    (*env)->CallVoidMethod(env, fDragSourceContextPeer, dragExitMethod, (jint) point.x, (jint) point.y);
-    CHECK_EXCEPTION();
-
+    JNF_MEMBER_CACHE(dragExitMethod, CDragSourceContextPeerClass, "dragExit", "(II)V");
+    JNFCallVoidMethod(env, fDragSourceContextPeer, dragExitMethod, (jint) point.x, (jint) point.y); // AWT_THREADING Safe (event)
 }
 
 
