@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -70,7 +70,7 @@
     AWT_ASSERT_APPKIT_THREAD;
     JNIEnv *env = [ThreadUtilities getJNIEnv];
     JNF_COCOA_ENTER(env);
-    
+
     // If we are called as a result of user pressing a shortcut, do nothing,
     // because AVTView has already sent corresponding key event to the Java
     // layer from performKeyEquivalent.
@@ -81,45 +81,26 @@
     // from this "frameless" menu, because there are no active windows. This
     // means we have to handle it here.
     NSEvent *currEvent = [[NSApplication sharedApplication] currentEvent];
+
+    if ([currEvent type] == NSKeyDown) {
+        // The action event can be ignored only if the key window is an AWT window.
+        // Otherwise, the action event is the only notification and must be processed.
+        NSWindow *keyWindow = [NSApp keyWindow];
+        if (keyWindow != nil && [AWTWindow isAWTWindow: keyWindow]) {
+            return;
+        }
+    }
+
     if (fIsCheckbox) {
         static JNF_CLASS_CACHE(jc_CCheckboxMenuItem, "sun/lwawt/macosx/CCheckboxMenuItem");
         static JNF_MEMBER_CACHE(jm_ckHandleAction, jc_CCheckboxMenuItem, "handleAction", "(Z)V");
-        
+
         // Send the opposite of what's currently checked -- the action
         // indicates what state we're going to.
         NSInteger state = [sender state];
         jboolean newState = (state == NSOnState ? JNI_FALSE : JNI_TRUE);
         JNFCallVoidMethod(env, fPeer, jm_ckHandleAction, newState);
-    }
-    else {
-        if ([currEvent type] == NSKeyDown) {
-            // Event available through sender variable hence NSApplication
-            // not needed for checking the keyboard input sans the modifier keys
-            // Also, the method used to fetch eventKey earlier would be locale dependent
-            // With earlier implementation, if MenuKey: e EventKey: ा ; if input method
-            // is not U.S. (Devanagari in this case)
-            // With current implementation, EventKey = MenuKey = e irrespective of
-            // input method
-            NSString *eventKey = [sender keyEquivalent];
-            // Apple uses characters from private Unicode range for some of the
-            // keys, so we need to do the same translation here that we do
-            // for the regular key down events
-            if ([eventKey length] == 1) {
-                unichar origChar = [eventKey characterAtIndex:0];
-                unichar newChar =  NsCharToJavaChar(origChar, 0);
-                if (newChar == java_awt_event_KeyEvent_CHAR_UNDEFINED) {
-                    newChar = origChar;
-                }
-                eventKey = [NSString stringWithCharacters: &newChar length: 1];
-            }
-            // The action event can be ignored only if the key window is an AWT window.
-            // Otherwise, the action event is the only notification and must be processed.
-            NSWindow *keyWindow = [NSApp keyWindow];
-            if (keyWindow != nil && [AWTWindow isAWTWindow: keyWindow]) {
-                return;
-            }
-        }
-        
+    } else {
         static JNF_CLASS_CACHE(jc_CMenuItem, "sun/lwawt/macosx/CMenuItem");
         static JNF_MEMBER_CACHE(jm_handleAction, jc_CMenuItem, "handleAction", "(JI)V"); // AWT_THREADING Safe (event)
 
@@ -129,20 +110,19 @@
         JNFCallVoidMethod(env, fPeer, jm_handleAction, UTC(currEvent), javaModifiers); // AWT_THREADING Safe (event)
     }
     JNF_COCOA_EXIT(env);
-    
 }
 
 - (void) setJavaLabel:(NSString *)theLabel shortcut:(NSString *)theKeyEquivalent modifierMask:(jint)modifiers {
-    
+
     NSUInteger modifierMask = 0;
-    
+
     if (![theKeyEquivalent isEqualToString:@""]) {
         // Force the key equivalent to lower case if not using the shift key.
         // Otherwise AppKit will draw a Shift glyph in the menu.
         if ((modifiers & java_awt_event_KeyEvent_SHIFT_MASK) == 0) {
             theKeyEquivalent = [theKeyEquivalent lowercaseString];
         }
-        
+
         // Hack for the question mark -- SHIFT and / means use the question mark.
         if ((modifiers & java_awt_event_KeyEvent_SHIFT_MASK) != 0 &&
             [theKeyEquivalent isEqualToString:@"/"])
@@ -150,10 +130,10 @@
             theKeyEquivalent = @"?";
             modifiers &= ~java_awt_event_KeyEvent_SHIFT_MASK;
         }
-        
+
         modifierMask = JavaModifiersToNsKeyModifiers(modifiers, NO);
     }
-    
+
     [ThreadUtilities performOnMainThreadWaiting:YES block:^(){
         [fMenuItem setKeyEquivalent:theKeyEquivalent];
         [fMenuItem setKeyEquivalentModifierMask:modifierMask];
@@ -162,14 +142,14 @@
 }
 
 - (void) setJavaImage:(NSImage *)theImage {
-    
+
     [ThreadUtilities performOnMainThreadWaiting:NO block:^(){
         [fMenuItem setImage:theImage];
     }];
 }
 
 - (void) setJavaToolTipText:(NSString *)theText {
-    
+
     [ThreadUtilities performOnMainThreadWaiting:NO block:^(){
         [fMenuItem setToolTip:theText];
     }];
@@ -177,11 +157,11 @@
 
 
 - (void)setJavaEnabled:(BOOL) enabled {
-    
+
     [ThreadUtilities performOnMainThreadWaiting:NO block:^(){
         @synchronized(self) {
             fIsEnabled = enabled;
-            
+
             // Warning:  This won't work if the parent menu is disabled.
             // See [CMenu syncFromJava]. We still need to call it here so
             // the NSMenuItem itself gets properly updated.
@@ -191,7 +171,7 @@
 }
 
 - (BOOL)isEnabled {
-    
+
     BOOL enabled = NO;
     @synchronized(self) {
         enabled = fIsEnabled;
@@ -201,7 +181,7 @@
 
 
 - (void)setJavaState:(BOOL)newState {
-    
+
     [ThreadUtilities performOnMainThreadWaiting:NO block:^(){
         [fMenuItem setState:(newState ? NSOnState : NSOffState)];
     }];
@@ -212,7 +192,7 @@
     [fMenuItem setTarget:nil];
     [fMenuItem release];
     fMenuItem = nil;
-    
+
     [super dealloc];
 }
 
@@ -237,7 +217,7 @@
 /** Convert a Java keycode for SetMenuItemCmd */
 static unichar AWTKeyToMacShortcut(jint awtKey, BOOL doShift) {
     unichar macKey = 0;
-    
+
     if ((awtKey >= java_awt_event_KeyEvent_VK_0 && awtKey <= java_awt_event_KeyEvent_VK_9) ||
         (awtKey >= java_awt_event_KeyEvent_VK_A && awtKey <= java_awt_event_KeyEvent_VK_Z))
     {
@@ -254,43 +234,43 @@ static unichar AWTKeyToMacShortcut(jint awtKey, BOOL doShift) {
         switch (awtKey) {
             case java_awt_event_KeyEvent_VK_BACK_QUOTE      : macKey = '`'; break;
             case java_awt_event_KeyEvent_VK_QUOTE           : macKey = '\''; break;
-                
+
             case java_awt_event_KeyEvent_VK_ESCAPE          : macKey = 0x1B; break;
             case java_awt_event_KeyEvent_VK_SPACE           : macKey = ' '; break;
             case java_awt_event_KeyEvent_VK_PAGE_UP         : macKey = NSPageUpFunctionKey; break;
             case java_awt_event_KeyEvent_VK_PAGE_DOWN       : macKey = NSPageDownFunctionKey; break;
             case java_awt_event_KeyEvent_VK_END             : macKey = NSEndFunctionKey; break;
             case java_awt_event_KeyEvent_VK_HOME            : macKey = NSHomeFunctionKey; break;
-                
+
             case java_awt_event_KeyEvent_VK_LEFT            : macKey = NSLeftArrowFunctionKey; break;
             case java_awt_event_KeyEvent_VK_UP              : macKey = NSUpArrowFunctionKey; break;
             case java_awt_event_KeyEvent_VK_RIGHT           : macKey = NSRightArrowFunctionKey; break;
             case java_awt_event_KeyEvent_VK_DOWN            : macKey = NSDownArrowFunctionKey; break;
-                
+
             case java_awt_event_KeyEvent_VK_COMMA           : macKey = ','; break;
-                
+
                 // Mac OS doesn't distinguish between the two '-' keys...
             case java_awt_event_KeyEvent_VK_MINUS           :
             case java_awt_event_KeyEvent_VK_SUBTRACT        : macKey = '-'; break;
-                
+
                 // or the two '.' keys...
             case java_awt_event_KeyEvent_VK_DECIMAL         :
             case java_awt_event_KeyEvent_VK_PERIOD          : macKey = '.'; break;
-                
+
                 // or the two '/' keys.
             case java_awt_event_KeyEvent_VK_DIVIDE          :
             case java_awt_event_KeyEvent_VK_SLASH           : macKey = '/'; break;
-                
+
             case java_awt_event_KeyEvent_VK_SEMICOLON       : macKey = ';'; break;
             case java_awt_event_KeyEvent_VK_EQUALS          : macKey = '='; break;
-                
+
             case java_awt_event_KeyEvent_VK_OPEN_BRACKET    : macKey = '['; break;
             case java_awt_event_KeyEvent_VK_BACK_SLASH      : macKey = '\\'; break;
             case java_awt_event_KeyEvent_VK_CLOSE_BRACKET   : macKey = ']'; break;
-                
+
             case java_awt_event_KeyEvent_VK_MULTIPLY        : macKey = '*'; break;
             case java_awt_event_KeyEvent_VK_ADD             : macKey = '+'; break;
-                
+
             case java_awt_event_KeyEvent_VK_HELP            : macKey = NSHelpFunctionKey; break;
             case java_awt_event_KeyEvent_VK_TAB             : macKey = NSTabCharacter; break;
             case java_awt_event_KeyEvent_VK_ENTER           : macKey = NSNewlineCharacter; break;
@@ -334,18 +314,18 @@ Java_sun_lwawt_macosx_CMenuItem_nativeSetLabel
     NSString *theLabel = JNFJavaToNSString(env, label);
     NSString *theKeyEquivalent = nil;
     unichar macKey = shortcutKey;
-    
+
     if (macKey == 0) {
         macKey = AWTKeyToMacShortcut(shortcutKeyCode, (mods & java_awt_event_KeyEvent_SHIFT_MASK) != 0);
     }
-    
+
     if (macKey != 0) {
         unichar equivalent[1] = {macKey};
         theKeyEquivalent = [NSString stringWithCharacters:equivalent length:1];
     } else {
         theKeyEquivalent = @"";
     }
-    
+
     [((CMenuItem *)jlong_to_ptr(menuItemObj)) setJavaLabel:theLabel shortcut:theKeyEquivalent modifierMask:mods];
     JNF_COCOA_EXIT(env);
 }
@@ -388,12 +368,12 @@ JNIEXPORT jlong JNICALL
 Java_sun_lwawt_macosx_CMenuItem_nativeCreate
 (JNIEnv *env, jobject peer, jlong parentCMenuObj, jboolean isSeparator)
 {
-    
+
     __block CMenuItem *aCMenuItem = nil;
     BOOL asSeparator = (isSeparator == JNI_TRUE) ? YES: NO;
     CMenu *parentCMenu = (CMenu *)jlong_to_ptr(parentCMenuObj);
     JNF_COCOA_ENTER(env);
-    
+
     jobject cPeerObjGlobal = (*env)->NewGlobalRef(env, peer);
 
     [ThreadUtilities performOnMainThreadWaiting:YES block:^(){
@@ -401,16 +381,16 @@ Java_sun_lwawt_macosx_CMenuItem_nativeCreate
                                          asSeparator: asSeparator];
         // the CMenuItem is released in CMenuComponent.dispose()
     }];
-    
+
     if (aCMenuItem == nil) {
         return 0L;
     }
-    
+
     // and add it to the parent item.
     [parentCMenu addJavaMenuItem: aCMenuItem];
-    
+
     // setLabel will be called after creation completes.
-    
+
     JNF_COCOA_EXIT(env);
     return ptr_to_jlong(aCMenuItem);
 }
