@@ -23,13 +23,17 @@
  * questions.
  */
 
+/*
+ * Must include this before JavaNativeFoundation.h to get jni.h from build
+ */
 #include "jni.h"
 #include "jni_util.h"
 
 #import <Cocoa/Cocoa.h>
+#import <JavaNativeFoundation/JavaNativeFoundation.h>
 
 
-@interface PropertiesUtilities : NSObject
+JNIEXPORT @interface PropertiesUtilities : NSObject
 
 + (NSString *) javaSystemPropertyForKey:(NSString *)key withEnv:(JNIEnv *)env;
 
