@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,12 +27,11 @@ package java.awt.event;
 
 import java.awt.Component;
 import java.awt.GraphicsEnvironment;
+import java.awt.IllegalComponentStateException;
 import java.awt.Point;
 import java.awt.Toolkit;
 import java.io.IOException;
 import java.io.ObjectInputStream;
-import java.awt.IllegalComponentStateException;
-import java.awt.MouseInfo;
 
 import sun.awt.AWTAccessor;
 import sun.awt.SunToolkit;
@@ -1188,6 +1187,11 @@ public class MouseEvent extends InputEvent {
 
     /**
      * Sets new modifiers by the old ones.
+     *
+     * @param  s the {@code ObjectInputStream} to read
+     * @throws ClassNotFoundException if the class of a serialized object could
+     *         not be found
+     * @throws IOException if an I/O error occurs
      * @serial
      */
     @SuppressWarnings("deprecation")
