@@ -400,14 +400,6 @@ typedef struct _NativePrimitive {
     jint                dstflags;
 } NativePrimitive;
 
-/*
- * This function should be defined to return a pointer to
- * an accelerated version of a primitive function 'func_c'
- * if it exists and to return a copy of the input parameter
- * otherwise.
- */
-extern AnyFunc* MapAccelFunction(AnyFunc *func_c);
-
 /*
  * The global collection of all primitive types.  Specific NativePrimitive
  * structures can be statically initialized by pointing to these structures.
@@ -490,6 +482,8 @@ extern struct _CompositeTypes {
 #define PtrCoord(p, x, xinc, y, yinc)   PtrAddBytes(p, \
                                                     ((ptrdiff_t)(y))*(yinc) + \
                                                     ((ptrdiff_t)(x))*(xinc))
+#define PtrPixelsRow(p, y, scanStride)    PtrAddBytes(p, \
+    ((intptr_t) (y)) * (scanStride))
 
 /*
  * The function to call with an array of NativePrimitive structures
