@@ -29,6 +29,11 @@
 #include "hb.hh"
 #include "hb-machinery.hh"
 
+#include <locale.h>
+
+#ifdef HB_NO_SETLOCALE
+#define setlocale(Category, Locale) "C"
+#endif
 
 /**
  * SECTION:hb-common
@@ -73,7 +78,7 @@ _hb_options_init ()
   }
 
   /* This is idempotent and threadsafe. */
-  _hb_options = u.i;
+  _hb_options.set_relaxed (u.i);
 }
 
 
@@ -81,15 +86,12 @@ _hb_options_init ()
 
 /**
  * hb_tag_from_string:
- * @str: (array length=len) (element-type uint8_t): String to convert
- * @len: Length of @str, or -1 if it is `NULL`-terminated
+ * @str: (array length=len) (element-type uint8_t):
+ * @len:
  *
- * Converts a string into an #hb_tag_t. Valid tags
- * are four characters. Shorter input strings will be
- * padded with spaces. Longer input strings will be
- * truncated.
  *
- * Return value: The #hb_tag_t corresponding to @str
+ *
+ * Return value:
  *
  * Since: 0.9.2
  **/
@@ -114,11 +116,10 @@ hb_tag_from_string (const char *str, int len)
 
 /**
  * hb_tag_to_string:
- * @tag: #hb_tag_t to convert
- * @buf: (out caller-allocates) (array fixed-size=4) (element-type uint8_t): Converted string
+ * @tag:
+ * @buf: (out caller-allocates) (array fixed-size=4) (element-type uint8_t):
+ *
  *
- * Converts an #hb_tag_t to a string and returns it in @buf.
- * Strings will be four characters long.
  *
  * Since: 0.9.5
  **/
@@ -134,7 +135,7 @@ hb_tag_to_string (hb_tag_t tag, char *buf)
 
 /* hb_direction_t */
 
-static const char direction_strings[][4] = {
+const char direction_strings[][4] = {
   "ltr",
   "rtl",
   "ttb",
@@ -143,17 +144,12 @@ static const char direction_strings[][4] = {
 
 /**
  * hb_direction_from_string:
- * @str: (array length=len) (element-type uint8_t): String to convert
- * @len: Length of @str, or -1 if it is `NULL`-terminated
+ * @str: (array length=len) (element-type uint8_t):
+ * @len:
  *
- * Converts a string to an #hb_direction_t.
  *
- * Matching is loose and applies only to the first letter. For
- * examples, "LTR" and "left-to-right" will both return #HB_DIRECTION_LTR.
  *
- * Unmatched strings will return #HB_DIRECTION_INVALID.
- *
- * Return value: The #hb_direction_t matching @str
+ * Return value:
  *
  * Since: 0.9.2
  **/
@@ -176,11 +172,11 @@ hb_direction_from_string (const char *str, int len)
 
 /**
  * hb_direction_to_string:
- * @direction: The #hb_direction_t to convert
+ * @direction:
  *
- * Converts an #hb_direction_t to a string.
  *
- * Return value: (transfer none): The string corresponding to @direction
+ *
+ * Return value: (transfer none):
  *
  * Since: 0.9.2
  **/
@@ -252,14 +248,16 @@ struct hb_language_item_t {
   bool operator == (const char *s) const
   { return lang_equal (lang, s); }
 
-  hb_language_item_t & operator = (const char *s)
-  {
-    /* We can't call strdup(), because we allow custom allocators. */
+  hb_language_item_t & operator = (const char *s) {
+    /* If a custom allocated is used calling strdup() pairs
+    badly with a call to the custom free() in fini() below.
+    Therefore don't call strdup(), implement its behavior.
+    */
     size_t len = strlen(s) + 1;
-    lang = (hb_language_t) hb_malloc(len);
+    lang = (hb_language_t) malloc(len);
     if (likely (lang))
     {
-      hb_memcpy((unsigned char *) lang, s, len);
+      memcpy((unsigned char *) lang, s, len);
       for (unsigned char *p = (unsigned char *) lang; *p; p++)
         *p = canon_map[*p];
     }
@@ -267,15 +265,16 @@ struct hb_language_item_t {
     return *this;
   }
 
-  void fini () { hb_free ((void *) lang); }
+  void fini () { free ((void *) lang); }
 };
 
 
-/* Thread-safe lockfree language list */
+/* Thread-safe lock-free language list */
 
 static hb_atomic_ptr_t <hb_language_item_t> langs;
 
-static inline void
+#if HB_USE_ATEXIT
+static void
 free_langs ()
 {
 retry:
@@ -286,10 +285,11 @@ retry:
   while (first_lang) {
     hb_language_item_t *next = first_lang->next;
     first_lang->fini ();
-    hb_free (first_lang);
+    free (first_lang);
     first_lang = next;
   }
 }
+#endif
 
 static hb_language_item_t *
 lang_find_or_insert (const char *key)
@@ -302,26 +302,28 @@ retry:
       return lang;
 
   /* Not found; allocate one. */
-  hb_language_item_t *lang = (hb_language_item_t *) hb_calloc (1, sizeof (hb_language_item_t));
+  hb_language_item_t *lang = (hb_language_item_t *) calloc (1, sizeof (hb_language_item_t));
   if (unlikely (!lang))
     return nullptr;
   lang->next = first_lang;
   *lang = key;
   if (unlikely (!lang->lang))
   {
-    hb_free (lang);
+    free (lang);
     return nullptr;
   }
 
   if (unlikely (!langs.cmpexch (first_lang, lang)))
   {
     lang->fini ();
-    hb_free (lang);
+    free (lang);
     goto retry;
   }
 
+#if HB_USE_ATEXIT
   if (!first_lang)
-    hb_atexit (free_langs); /* First person registers atexit() callback. */
+    atexit (free_langs); /* First person registers atexit() callback. */
+#endif
 
   return lang;
 }
@@ -331,7 +333,7 @@ retry:
  * hb_language_from_string:
  * @str: (array length=len) (element-type uint8_t): a string representing
  *       a BCP 47 language tag
- * @len: length of the @str, or -1 if it is `NULL`-terminated.
+ * @len: length of the @str, or -1 if it is %NULL-terminated.
  *
  * Converts @str representing a BCP 47 language tag to the corresponding
  * #hb_language_t.
@@ -353,7 +355,7 @@ hb_language_from_string (const char *str, int len)
     /* NUL-terminate it. */
     char strbuf[64];
     len = hb_min (len, (int) sizeof (strbuf) - 1);
-    hb_memcpy (strbuf, str, len);
+    memcpy (strbuf, str, len);
     strbuf[len] = '\0';
     item = lang_find_or_insert (strbuf);
   }
@@ -365,12 +367,12 @@ hb_language_from_string (const char *str, int len)
 
 /**
  * hb_language_to_string:
- * @language: The #hb_language_t to convert
+ * @language: an #hb_language_t to convert.
  *
- * Converts an #hb_language_t to a string.
+ * See hb_language_from_string().
  *
  * Return value: (transfer none):
- * A `NULL`-terminated string representing the @language. Must not be freed by
+ * A %NULL-terminated string representing the @language. Must not be freed by
  * the caller.
  *
  * Since: 0.9.2
@@ -386,17 +388,16 @@ hb_language_to_string (hb_language_t language)
 /**
  * hb_language_get_default:
  *
- * Fetch the default language from current locale.
+ * Get default language from current locale.
  *
- * <note>Note that the first time this function is called, it calls
+ * Note that the first time this function is called, it calls
  * "setlocale (LC_CTYPE, nullptr)" to fetch current locale.  The underlying
  * setlocale function is, in many implementations, NOT threadsafe.  To avoid
  * problems, call this function once before multiple threads can call it.
  * This function is only used from hb_buffer_guess_segment_properties() by
- * HarfBuzz itself.</note>
+ * HarfBuzz itself.
  *
- * Return value: (transfer none): The default language of the locale as
- * an #hb_language_t
+ * Return value: (transfer none):
  *
  * Since: 0.9.2
  **/
@@ -408,45 +409,13 @@ hb_language_get_default ()
   hb_language_t language = default_language;
   if (unlikely (language == HB_LANGUAGE_INVALID))
   {
-    language = hb_language_from_string (hb_setlocale (LC_CTYPE, nullptr), -1);
+    language = hb_language_from_string (setlocale (LC_CTYPE, nullptr), -1);
     (void) default_language.cmpexch (HB_LANGUAGE_INVALID, language);
   }
 
   return language;
 }
 
-/**
- * hb_language_matches:
- * @language: The #hb_language_t to work on
- * @specific: Another #hb_language_t
- *
- * Check whether a second language tag is the same or a more
- * specific version of the provided language tag.  For example,
- * "fa_IR.utf8" is a more specific tag for "fa" or for "fa_IR".
- *
- * Return value: `true` if languages match, `false` otherwise.
- *
- * Since: 5.0.0
- **/
-hb_bool_t
-hb_language_matches (hb_language_t language,
-                     hb_language_t specific)
-{
-  if (language == specific) return true;
-  if (!language || !specific) return false;
-
-  const char *l = language->s;
-  const char *s = specific->s;
-  unsigned ll = strlen (l);
-  unsigned sl = strlen (s);
-
-  if (ll > sl)
-    return false;
-
-  return strncmp (l, s, ll) == 0 &&
-         (s[ll] == '\0' || s[ll] == '-');
-}
-
 
 /* hb_script_t */
 
@@ -479,12 +448,7 @@ hb_script_from_iso15924_tag (hb_tag_t tag)
     case HB_TAG('Q','a','a','c'): return HB_SCRIPT_COPTIC;
 
     /* Script variants from https://unicode.org/iso15924/ */
-    case HB_TAG('A','r','a','n'): return HB_SCRIPT_ARABIC;
     case HB_TAG('C','y','r','s'): return HB_SCRIPT_CYRILLIC;
-    case HB_TAG('G','e','o','k'): return HB_SCRIPT_GEORGIAN;
-    case HB_TAG('H','a','n','s'): return HB_SCRIPT_HAN;
-    case HB_TAG('H','a','n','t'): return HB_SCRIPT_HAN;
-    case HB_TAG('J','a','m','o'): return HB_SCRIPT_HANGUL;
     case HB_TAG('L','a','t','f'): return HB_SCRIPT_LATIN;
     case HB_TAG('L','a','t','g'): return HB_SCRIPT_LATIN;
     case HB_TAG('S','y','r','e'): return HB_SCRIPT_SYRIAC;
@@ -504,7 +468,7 @@ hb_script_from_iso15924_tag (hb_tag_t tag)
  * hb_script_from_string:
  * @str: (array length=len) (element-type uint8_t): a string representing an
  *       ISO 15924 tag.
- * @len: length of the @str, or -1 if it is `NULL`-terminated.
+ * @len: length of the @str, or -1 if it is %NULL-terminated.
  *
  * Converts a string @str representing an ISO 15924 script tag to a
  * corresponding #hb_script_t. Shorthand for hb_tag_from_string() then
@@ -525,7 +489,7 @@ hb_script_from_string (const char *str, int len)
  * hb_script_to_iso15924_tag:
  * @script: an #hb_script_t to convert.
  *
- * Converts an #hb_script_t to a corresponding ISO 15924 script tag.
+ * See hb_script_from_iso15924_tag().
  *
  * Return value:
  * An #hb_tag_t representing an ISO 15924 script tag.
@@ -540,16 +504,11 @@ hb_script_to_iso15924_tag (hb_script_t script)
 
 /**
  * hb_script_get_horizontal_direction:
- * @script: The #hb_script_t to query
+ * @script:
  *
- * Fetches the #hb_direction_t of a script when it is
- * set horizontally. All right-to-left scripts will return
- * #HB_DIRECTION_RTL. All left-to-right scripts will return
- * #HB_DIRECTION_LTR.  Scripts that can be written either
- * horizontally or vertically will return #HB_DIRECTION_INVALID.
- * Unknown scripts will return #HB_DIRECTION_LTR.
  *
- * Return value: The horizontal #hb_direction_t of @script
+ *
+ * Return value:
  *
  * Since: 0.9.2
  **/
@@ -622,9 +581,6 @@ hb_script_get_horizontal_direction (hb_script_t script)
     case HB_SCRIPT_CHORASMIAN:
     case HB_SCRIPT_YEZIDI:
 
-    /* Unicode-14.0 additions */
-    case HB_SCRIPT_OLD_UYGHUR:
-
       return HB_DIRECTION_RTL;
 
 
@@ -632,7 +588,6 @@ hb_script_get_horizontal_direction (hb_script_t script)
     case HB_SCRIPT_OLD_HUNGARIAN:
     case HB_SCRIPT_OLD_ITALIC:
     case HB_SCRIPT_RUNIC:
-    case HB_SCRIPT_TIFINAGH:
 
       return HB_DIRECTION_INVALID;
   }
@@ -658,9 +613,9 @@ hb_script_get_horizontal_direction (hb_script_t script)
 
 /**
  * hb_version:
- * @major: (out): Library major version component
- * @minor: (out): Library minor version component
- * @micro: (out): Library micro version component
+ * @major: (out): Library major version component.
+ * @minor: (out): Library minor version component.
+ * @micro: (out): Library micro version component.
  *
  * Returns library version as three integer components.
  *
@@ -681,7 +636,7 @@ hb_version (unsigned int *major,
  *
  * Returns library version as a string with three components.
  *
- * Return value: Library version string
+ * Return value: library version string.
  *
  * Since: 0.9.2
  **/
@@ -693,15 +648,13 @@ hb_version_string ()
 
 /**
  * hb_version_atleast:
- * @major: Library major version component
- * @minor: Library minor version component
- * @micro: Library micro version component
+ * @major:
+ * @minor:
+ * @micro:
  *
- * Tests the library version against a minimum value,
- * as three integer components.
  *
- * Return value: `true` if the library is equal to or greater than
- * the test value, `false` otherwise
+ *
+ * Return value:
  *
  * Since: 0.9.30
  **/
@@ -888,7 +841,7 @@ parse_one_feature (const char **pp, const char *end, hb_feature_t *feature)
 /**
  * hb_feature_from_string:
  * @str: (array length=len) (element-type uint8_t): a string to parse
- * @len: length of @str, or -1 if string is `NULL` terminated
+ * @len: length of @str, or -1 if string is %NULL terminated
  * @feature: (out): the #hb_feature_t to initialize with the parsed values
  *
  * Parses a string into a #hb_feature_t.
@@ -930,7 +883,7 @@ parse_one_feature (const char **pp, const char *end, hb_feature_t *feature)
  * </informaltable>
  *
  * Return value:
- * `true` if @str is successfully parsed, `false` otherwise
+ * %true if @str is successfully parsed, %false otherwise.
  *
  * Since: 0.9.5
  **/
@@ -951,7 +904,7 @@ hb_feature_from_string (const char *str, int len,
   }
 
   if (feature)
-    hb_memset (feature, 0, sizeof (*feature));
+    memset (feature, 0, sizeof (*feature));
   return false;
 }
 
@@ -961,7 +914,7 @@ hb_feature_from_string (const char *str, int len,
  * @buf: (array length=size) (out): output string
  * @size: the allocated size of @buf
  *
- * Converts a #hb_feature_t into a `NULL`-terminated string in the format
+ * Converts a #hb_feature_t into a %NULL-terminated string in the format
  * understood by hb_feature_from_string(). The client in responsible for
  * allocating big enough size for @buf, 128 bytes is more than enough.
  *
@@ -1000,7 +953,7 @@ hb_feature_to_string (hb_feature_t *feature,
   }
   assert (len < ARRAY_LENGTH (s));
   len = hb_min (len, size - 1);
-  hb_memcpy (buf, s, len);
+  memcpy (buf, s, len);
   buf[len] = '\0';
 }
 
@@ -1028,21 +981,6 @@ parse_one_variation (const char **pp, const char *end, hb_variation_t *variation
 
 /**
  * hb_variation_from_string:
- * @str: (array length=len) (element-type uint8_t): a string to parse
- * @len: length of @str, or -1 if string is `NULL` terminated
- * @variation: (out): the #hb_variation_t to initialize with the parsed values
- *
- * Parses a string into a #hb_variation_t.
- *
- * The format for specifying variation settings follows. All valid CSS
- * font-variation-settings values other than 'normal' and 'inherited' are also
- * accepted, though, not documented below.
- *
- * The format is a tag, optionally followed by an equals sign, followed by a
- * number. For example `wght=500`, or `slnt=-7.5`.
- *
- * Return value:
- * `true` if @str is successfully parsed, `false` otherwise
  *
  * Since: 1.4.2
  */
@@ -1063,60 +1001,12 @@ hb_variation_from_string (const char *str, int len,
   }
 
   if (variation)
-    hb_memset (variation, 0, sizeof (*variation));
+    memset (variation, 0, sizeof (*variation));
   return false;
 }
 
-#ifndef HB_NO_SETLOCALE
-
-static inline void free_static_C_locale ();
-
-static struct hb_C_locale_lazy_loader_t : hb_lazy_loader_t<hb_remove_pointer<hb_locale_t>,
-                                                           hb_C_locale_lazy_loader_t>
-{
-  static hb_locale_t create ()
-  {
-    hb_locale_t l = newlocale (LC_ALL_MASK, "C", NULL);
-    if (!l)
-      return l;
-
-    hb_atexit (free_static_C_locale);
-
-    return l;
-  }
-  static void destroy (hb_locale_t l)
-  {
-    freelocale (l);
-  }
-  static hb_locale_t get_null ()
-  {
-    return (hb_locale_t) 0;
-  }
-} static_C_locale;
-
-static inline
-void free_static_C_locale ()
-{
-  static_C_locale.free_instance ();
-}
-
-static hb_locale_t
-get_C_locale ()
-{
-  return static_C_locale.get_unconst ();
-}
-
-#endif
-
 /**
  * hb_variation_to_string:
- * @variation: an #hb_variation_t to convert
- * @buf: (array length=size) (out caller-allocates): output string
- * @size: the allocated size of @buf
- *
- * Converts an #hb_variation_t into a `NULL`-terminated string in the format
- * understood by hb_variation_from_string(). The client in responsible for
- * allocating big enough size for @buf, 128 bytes is more than enough.
  *
  * Since: 1.4.2
  */
@@ -1133,25 +1023,19 @@ hb_variation_to_string (hb_variation_t *variation,
   while (len && s[len - 1] == ' ')
     len--;
   s[len++] = '=';
-
-  hb_locale_t oldlocale HB_UNUSED;
-  oldlocale = hb_uselocale (get_C_locale ());
   len += hb_max (0, snprintf (s + len, ARRAY_LENGTH (s) - len, "%g", (double) variation->value));
-  (void) hb_uselocale (oldlocale);
 
   assert (len < ARRAY_LENGTH (s));
   len = hb_min (len, size - 1);
-  hb_memcpy (buf, s, len);
+  memcpy (buf, s, len);
   buf[len] = '\0';
 }
 
 /**
  * hb_color_get_alpha:
- * @color: an #hb_color_t we are interested in its channels.
+ * color: a #hb_color_t we are interested in its channels.
  *
- * Fetches the alpha channel of the given @color.
- *
- * Return value: Alpha channel value
+ * Return value: Alpha channel value of the given color
  *
  * Since: 2.1.0
  */
@@ -1163,11 +1047,9 @@ uint8_t
 
 /**
  * hb_color_get_red:
- * @color: an #hb_color_t we are interested in its channels.
+ * color: a #hb_color_t we are interested in its channels.
  *
- * Fetches the red channel of the given @color.
- *
- * Return value: Red channel value
+ * Return value: Red channel value of the given color
  *
  * Since: 2.1.0
  */
@@ -1179,11 +1061,9 @@ uint8_t
 
 /**
  * hb_color_get_green:
- * @color: an #hb_color_t we are interested in its channels.
+ * color: a #hb_color_t we are interested in its channels.
  *
- * Fetches the green channel of the given @color.
- *
- * Return value: Green channel value
+ * Return value: Green channel value of the given color
  *
  * Since: 2.1.0
  */
@@ -1195,11 +1075,9 @@ uint8_t
 
 /**
  * hb_color_get_blue:
- * @color: an #hb_color_t we are interested in its channels.
+ * color: a #hb_color_t we are interested in its channels.
  *
- * Fetches the blue channel of the given @color.
- *
- * Return value: Blue channel value
+ * Return value: Blue channel value of the given color
  *
  * Since: 2.1.0
  */
