@@ -34,6 +34,7 @@
 
 #include "hb-coretext.h"
 #include "hb-aat-layout.hh"
+#include <math.h>
 
 
 /**
@@ -189,10 +190,7 @@ create_ct_font (CGFontRef cg_font, CGFloat font_size)
    * reconfiguring the cascade list causes CoreText crashes. For details, see
    * crbug.com/549610 */
   // 0x00070000 stands for "kCTVersionNumber10_10", see CoreText.h
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
   if (&CTGetCoreTextVersion != nullptr && CTGetCoreTextVersion() < 0x00070000) {
-#pragma GCC diagnostic pop
     CFStringRef fontName = CTFontCopyPostScriptName (ct_font);
     bool isEmojiFont = CFStringCompare (fontName, CFSTR("AppleColorEmoji"), 0) == kCFCompareEqualTo;
     CFRelease (fontName);
@@ -348,7 +346,7 @@ retry:
   const hb_coretext_font_data_t *data = font->data.coretext;
   if (unlikely (!data)) return nullptr;
 
-  if (fabs (CTFontGetSize ((CTFontRef) data) - (CGFloat) font->ptem) > (CGFloat) .5)
+  if (fabs (CTFontGetSize ((CTFontRef) data) - (CGFloat) font->ptem) > .5)
   {
     /* XXX-MT-bug
      * Note that evaluating condition above can be dangerous if another thread
@@ -404,7 +402,7 @@ hb_coretext_font_create (CTFontRef ct_font)
 }
 
 /**
- * hb_coretext_font_get_ct_font:
+ * hb_coretext_face_get_ct_font:
  * @font: #hb_font_t to work upon
  *
  * Fetches the CTFontRef associated with the specified
@@ -860,7 +858,7 @@ resize_and_retry:
 
     buffer->len = 0;
     uint32_t status_and = ~0, status_or = 0;
-    CGFloat advances_so_far = 0;
+    double advances_so_far = 0;
     /* For right-to-left runs, CoreText returns the glyphs positioned such that
      * any trailing whitespace is to the left of (0,0).  Adjust coordinate system
      * to fix for that.  Test with any RTL string with trailing spaces.
@@ -882,10 +880,10 @@ resize_and_retry:
       status_or  |= run_status;
       status_and &= run_status;
       DEBUG_MSG (CORETEXT, run, "CTRunStatus: %x", run_status);
-      CGFloat run_advance = CTRunGetTypographicBounds (run, range_all, nullptr, nullptr, nullptr);
+      double run_advance = CTRunGetTypographicBounds (run, range_all, nullptr, nullptr, nullptr);
       if (HB_DIRECTION_IS_VERTICAL (buffer->props.direction))
           run_advance = -run_advance;
-      DEBUG_MSG (CORETEXT, run, "Run advance: %g", (double) run_advance);
+      DEBUG_MSG (CORETEXT, run, "Run advance: %g", run_advance);
 
       /* CoreText does automatic font fallback (AKA "cascading") for  characters
        * not supported by the requested font, and provides no way to turn it off,
@@ -1064,7 +1062,7 @@ resize_and_retry:
           hb_position_t x_offset = (positions[0].x - advances_so_far) * x_mult;
           for (unsigned int j = 0; j < num_glyphs; j++)
           {
-            CGFloat advance;
+            double advance;
             if (likely (j + 1 < num_glyphs))
               advance = positions[j + 1].x - positions[j].x;
             else /* last glyph */
@@ -1080,7 +1078,7 @@ resize_and_retry:
           hb_position_t y_offset = (positions[0].y - advances_so_far) * y_mult;
           for (unsigned int j = 0; j < num_glyphs; j++)
           {
-            CGFloat advance;
+            double advance;
             if (likely (j + 1 < num_glyphs))
               advance = positions[j + 1].y - positions[j].y;
             else /* last glyph */
