@@ -37,7 +37,7 @@
 #include "hb-shaper-impl.hh"
 
 #include "hb-ot-shape.hh"
-#include "hb-ot-shaper.hh"
+#include "hb-ot-shape-complex.hh"
 #include "hb-ot-shape-fallback.hh"
 #include "hb-ot-shape-normalize.hh"
 
@@ -46,21 +46,15 @@
 #include "hb-set.hh"
 
 #include "hb-aat-layout.hh"
-#include "hb-ot-layout-gdef-table.hh"
-#include "hb-ot-stat-table.hh"
 
 
-static inline bool
-_hb_codepoint_is_regional_indicator (hb_codepoint_t u)
-{ return hb_in_range<hb_codepoint_t> (u, 0x1F1E6u, 0x1F1FFu); }
-
 #ifndef HB_NO_AAT_SHAPE
 static inline bool
-_hb_apply_morx (hb_face_t *face, const hb_segment_properties_t &props)
+_hb_apply_morx (hb_face_t *face, const hb_segment_properties_t *props)
 {
   /* https://github.com/harfbuzz/harfbuzz/issues/2124 */
   return hb_aat_layout_has_substitution (face) &&
-         (HB_DIRECTION_IS_HORIZONTAL (props.direction) || !hb_ot_layout_has_substitution (face));
+         (HB_DIRECTION_IS_HORIZONTAL (props->direction) || !hb_ot_layout_has_substitution (face));
 }
 #endif
 
@@ -80,25 +74,23 @@ hb_ot_shape_collect_features (hb_ot_shape_planner_t          *planner,
                               unsigned int                    num_user_features);
 
 hb_ot_shape_planner_t::hb_ot_shape_planner_t (hb_face_t                     *face,
-                                              const hb_segment_properties_t &props) :
+                                              const hb_segment_properties_t *props) :
                                                 face (face),
-                                                props (props),
-                                                map (face, props)
+                                                props (*props),
+                                                map (face, props),
+                                                aat_map (face, props)
 #ifndef HB_NO_AAT_SHAPE
-                                                , aat_map (face, props)
                                                 , apply_morx (_hb_apply_morx (face, props))
 #endif
 {
-  shaper = hb_ot_shaper_categorize (props.script, props.direction, map.chosen_script[0]);
+  shaper = hb_ot_shape_complex_categorize (this);
 
   script_zero_marks = shaper->zero_width_marks != HB_OT_SHAPE_ZERO_WIDTH_MARKS_NONE;
   script_fallback_mark_positioning = shaper->fallback_position;
 
-#ifndef HB_NO_AAT_SHAPE
   /* https://github.com/harfbuzz/harfbuzz/issues/1528 */
-  if (apply_morx && shaper != &_hb_ot_shaper_default)
-    shaper = &_hb_ot_shaper_dumber;
-#endif
+  if (apply_morx && shaper != &_hb_ot_complex_shaper_default)
+    shaper = &_hb_ot_complex_shaper_dumber;
 }
 
 void
@@ -129,6 +121,10 @@ hb_ot_shape_planner_t::compile (hb_ot_shape_plan_t           &plan,
   plan.kern_mask = plan.map.get_mask (kern_tag);
   plan.requested_kerning = !!plan.kern_mask;
 #endif
+#ifndef HB_NO_AAT_SHAPE
+  plan.trak_mask = plan.map.get_mask (HB_TAG ('t','r','a','k'));
+  plan.requested_tracking = !!plan.trak_mask;
+#endif
 
   bool has_gpos_kern = plan.map.get_feature_index (1, kern_tag) != HB_OT_LAYOUT_NO_FEATURE_INDEX;
   bool disable_gpos = plan.shaper->gpos_tag &&
@@ -153,38 +149,29 @@ hb_ot_shape_planner_t::compile (hb_ot_shape_plan_t           &plan,
    * Decide who does positioning. GPOS, kerx, kern, or fallback.
    */
 
+  if (0)
+    ;
 #ifndef HB_NO_AAT_SHAPE
-  bool has_kerx = hb_aat_layout_has_positioning (face);
-  bool has_gsub = !apply_morx && hb_ot_layout_has_substitution (face);
-#endif
-  bool has_gpos = !disable_gpos && hb_ot_layout_has_positioning (face);
-  if (false)
-    {}
-#ifndef HB_NO_AAT_SHAPE
-  /* Prefer GPOS over kerx if GSUB is present;
-   * https://github.com/harfbuzz/harfbuzz/issues/3008 */
-  else if (has_kerx && !(has_gsub && has_gpos))
+  else if (hb_aat_layout_has_positioning (face))
     plan.apply_kerx = true;
 #endif
-  else if (has_gpos)
+  else if (!apply_morx && !disable_gpos && hb_ot_layout_has_positioning (face))
     plan.apply_gpos = true;
 
   if (!plan.apply_kerx && (!has_gpos_kern || !plan.apply_gpos))
   {
-    if (false) {}
+    /* Apparently Apple applies kerx if GPOS kern was not applied. */
 #ifndef HB_NO_AAT_SHAPE
-    else if (has_kerx)
+    if (hb_aat_layout_has_positioning (face))
       plan.apply_kerx = true;
+    else
 #endif
 #ifndef HB_NO_OT_KERN
-    else if (hb_ot_layout_has_kerning (face))
+    if (hb_ot_layout_has_kerning (face))
       plan.apply_kern = true;
 #endif
-    else {}
   }
 
-  plan.apply_fallback_kern = !(plan.apply_gpos || plan.apply_kerx || plan.apply_kern);
-
   plan.zero_marks = script_zero_marks &&
                     !plan.apply_kerx &&
                     (!plan.apply_kern
@@ -206,19 +193,8 @@ hb_ot_shape_planner_t::compile (hb_ot_shape_plan_t           &plan,
                                    script_fallback_mark_positioning;
 
 #ifndef HB_NO_AAT_SHAPE
-  /* If we're using morx shaping, we cancel mark position adjustment because
-     Apple Color Emoji assumes this will NOT be done when forming emoji sequences;
-     https://github.com/harfbuzz/harfbuzz/issues/2967. */
-  if (plan.apply_morx)
-    plan.adjust_mark_positioning_when_zeroing = false;
-
-  /* According to Ned, trak is applied by default for "modern fonts", as detected by presence of STAT table. */
-#ifndef HB_NO_STYLE
-  plan.apply_trak = hb_aat_layout_has_tracking (face) && face->table.STAT->has_data ();
-#else
-  plan.apply_trak = false;
-#endif
-
+  /* Currently we always apply trak. */
+  plan.apply_trak = plan.requested_tracking && hb_aat_layout_has_tracking (face);
 #endif
 }
 
@@ -227,9 +203,12 @@ hb_ot_shape_plan_t::init0 (hb_face_t                     *face,
                            const hb_shape_plan_key_t     *key)
 {
   map.init ();
+#ifndef HB_NO_AAT_SHAPE
+  aat_map.init ();
+#endif
 
   hb_ot_shape_planner_t planner (face,
-                                 key->props);
+                                 &key->props);
 
   hb_ot_shape_collect_features (&planner,
                                 key->user_features,
@@ -243,6 +222,9 @@ hb_ot_shape_plan_t::init0 (hb_face_t                     *face,
     if (unlikely (!data))
     {
       map.fini ();
+#ifndef HB_NO_AAT_SHAPE
+      aat_map.fini ();
+#endif
       return false;
     }
   }
@@ -257,13 +239,21 @@ hb_ot_shape_plan_t::fini ()
     shaper->data_destroy (const_cast<void *> (data));
 
   map.fini ();
+#ifndef HB_NO_AAT_SHAPE
+  aat_map.fini ();
+#endif
 }
 
 void
 hb_ot_shape_plan_t::substitute (hb_font_t   *font,
                                 hb_buffer_t *buffer) const
 {
-  map.substitute (this, font, buffer);
+#ifndef HB_NO_AAT_SHAPE
+  if (unlikely (apply_morx))
+    hb_aat_layout_substitute (this, font, buffer);
+  else
+#endif
+    map.substitute (this, font, buffer);
 }
 
 void
@@ -276,12 +266,11 @@ hb_ot_shape_plan_t::position (hb_font_t   *font,
   else if (this->apply_kerx)
     hb_aat_layout_position (this, font, buffer);
 #endif
-
 #ifndef HB_NO_OT_KERN
-  if (this->apply_kern)
+  else if (this->apply_kern)
     hb_ot_layout_kern (this, font, buffer);
 #endif
-  else if (this->apply_fallback_kern)
+  else
     _hb_ot_shape_fallback_kern (this, font, buffer);
 
 #ifndef HB_NO_AAT_SHAPE
@@ -317,19 +306,16 @@ horizontal_features[] =
 };
 
 static void
-hb_ot_shape_collect_features (hb_ot_shape_planner_t *planner,
-                              const hb_feature_t    *user_features,
-                              unsigned int           num_user_features)
+hb_ot_shape_collect_features (hb_ot_shape_planner_t          *planner,
+                              const hb_feature_t             *user_features,
+                              unsigned int                    num_user_features)
 {
   hb_ot_map_builder_t *map = &planner->map;
 
-  map->is_simple = true;
-
   map->enable_feature (HB_TAG('r','v','r','n'));
   map->add_gsub_pause (nullptr);
 
-  switch (planner->props.direction)
-  {
+  switch (planner->props.direction) {
     case HB_DIRECTION_LTR:
       map->enable_feature (HB_TAG ('l','t','r','a'));
       map->enable_feature (HB_TAG ('l','t','r','m'));
@@ -355,17 +341,19 @@ hb_ot_shape_collect_features (hb_ot_shape_planner_t *planner,
   /* Random! */
   map->enable_feature (HB_TAG ('r','a','n','d'), F_RANDOM, HB_OT_MAP_MAX_VALUE);
 
-  map->enable_feature (HB_TAG ('H','a','r','f')); /* Considered required. */
-  map->enable_feature (HB_TAG ('H','A','R','F')); /* Considered discretionary. */
+#ifndef HB_NO_AAT_SHAPE
+  /* Tracking.  We enable dummy feature here just to allow disabling
+   * AAT 'trak' table using features.
+   * https://github.com/harfbuzz/harfbuzz/issues/1303 */
+  map->enable_feature (HB_TAG ('t','r','a','k'), F_HAS_FALLBACK);
+#endif
+
+  map->enable_feature (HB_TAG ('H','A','R','F'));
 
   if (planner->shaper->collect_features)
-  {
-    map->is_simple = false;
     planner->shaper->collect_features (planner);
-  }
 
-  map->enable_feature (HB_TAG ('B','u','z','z')); /* Considered required. */
-  map->enable_feature (HB_TAG ('B','U','Z','Z')); /* Considered discretionary. */
+  map->enable_feature (HB_TAG ('B','U','Z','Z'));
 
   for (unsigned int i = 0; i < ARRAY_LENGTH (common_features); i++)
     map->add_feature (common_features[i]);
@@ -375,10 +363,6 @@ hb_ot_shape_collect_features (hb_ot_shape_planner_t *planner,
       map->add_feature (horizontal_features[i]);
   else
   {
-    /* We only apply `vert` feature. See:
-     * https://github.com/harfbuzz/harfbuzz/commit/d71c0df2d17f4590d5611239577a6cb532c26528
-     * https://lists.freedesktop.org/archives/harfbuzz/2013-August/003490.html */
-
     /* We really want to find a 'vert' feature if there's any in the font, no
      * matter which script/langsys it is listed (or not) under.
      * See various bugs referenced from:
@@ -386,8 +370,6 @@ hb_ot_shape_collect_features (hb_ot_shape_planner_t *planner,
     map->enable_feature (HB_TAG ('v','e','r','t'), F_GLOBAL_SEARCH);
   }
 
-  if (num_user_features)
-    map->is_simple = false;
   for (unsigned int i = 0; i < num_user_features; i++)
   {
     const hb_feature_t *feature = &user_features[i];
@@ -397,6 +379,18 @@ hb_ot_shape_collect_features (hb_ot_shape_planner_t *planner,
                       feature->value);
   }
 
+#ifndef HB_NO_AAT_SHAPE
+  if (planner->apply_morx)
+  {
+    hb_aat_map_builder_t *aat_map = &planner->aat_map;
+    for (unsigned int i = 0; i < num_user_features; i++)
+    {
+      const hb_feature_t *feature = &user_features[i];
+      aat_map->add_feature (feature->tag, feature->value);
+    }
+  }
+#endif
+
   if (planner->shaper->override_features)
     planner->shaper->override_features (planner);
 }
@@ -424,26 +418,17 @@ _hb_ot_shaper_face_data_destroy (hb_ot_face_data_t *data)
  * shaper font data
  */
 
-struct hb_ot_font_data_t {
-  OT::ItemVariationStore::cache_t unused; // Just for alignment
-};
+struct hb_ot_font_data_t {};
 
 hb_ot_font_data_t *
-_hb_ot_shaper_font_data_create (hb_font_t *font)
+_hb_ot_shaper_font_data_create (hb_font_t *font HB_UNUSED)
 {
-  if (!font->num_coords)
-    return (hb_ot_font_data_t *) HB_SHAPER_DATA_SUCCEEDED;
-
-  const OT::ItemVariationStore &var_store = font->face->table.GDEF->table->get_var_store ();
-  auto *cache = (hb_ot_font_data_t *) var_store.create_cache ();
-  return cache ? cache : (hb_ot_font_data_t *) HB_SHAPER_DATA_SUCCEEDED;
+  return (hb_ot_font_data_t *) HB_SHAPER_DATA_SUCCEEDED;
 }
 
 void
-_hb_ot_shaper_font_data_destroy (hb_ot_font_data_t *data)
+_hb_ot_shaper_font_data_destroy (hb_ot_font_data_t *data HB_UNUSED)
 {
-  if (data == HB_SHAPER_DATA_SUCCEEDED) return;
-  OT::ItemVariationStore::destroy_cache ((OT::ItemVariationStore::cache_t *) data);
 }
 
 
@@ -488,28 +473,11 @@ hb_set_unicode_props (hb_buffer_t *buffer)
   {
     _hb_glyph_info_set_unicode_props (&info[i], buffer);
 
-    unsigned gen_cat = _hb_glyph_info_get_general_category (&info[i]);
-    if (FLAG_UNSAFE (gen_cat) &
-        (FLAG (HB_UNICODE_GENERAL_CATEGORY_LOWERCASE_LETTER) |
-         FLAG (HB_UNICODE_GENERAL_CATEGORY_UPPERCASE_LETTER) |
-         FLAG (HB_UNICODE_GENERAL_CATEGORY_TITLECASE_LETTER) |
-         FLAG (HB_UNICODE_GENERAL_CATEGORY_OTHER_LETTER) |
-         FLAG (HB_UNICODE_GENERAL_CATEGORY_SPACE_SEPARATOR)))
-      continue;
-
     /* Marks are already set as continuation by the above line.
      * Handle Emoji_Modifier and ZWJ-continuation. */
-    if (unlikely (gen_cat == HB_UNICODE_GENERAL_CATEGORY_MODIFIER_SYMBOL &&
+    if (unlikely (_hb_glyph_info_get_general_category (&info[i]) == HB_UNICODE_GENERAL_CATEGORY_MODIFIER_SYMBOL &&
                   hb_in_range<hb_codepoint_t> (info[i].codepoint, 0x1F3FBu, 0x1F3FFu)))
     {
-      _hb_glyph_info_set_continuation (&info[i]);
-    }
-    /* Regional_Indicators are hairy as hell...
-     * https://github.com/harfbuzz/harfbuzz/issues/2265 */
-    else if (unlikely (i && _hb_codepoint_is_regional_indicator (info[i].codepoint)))
-    {
-      if (_hb_codepoint_is_regional_indicator (info[i - 1].codepoint) &&
-          !_hb_glyph_info_is_continuation (&info[i - 1]))
         _hb_glyph_info_set_continuation (&info[i]);
     }
 #ifndef HB_NO_EMOJI_SEQUENCES
@@ -526,20 +494,18 @@ hb_set_unicode_props (hb_buffer_t *buffer)
     }
 #endif
     /* Or part of the Other_Grapheme_Extend that is not marks.
-     * As of Unicode 15 that is just:
+     * As of Unicode 11 that is just:
      *
      * 200C          ; Other_Grapheme_Extend # Cf       ZERO WIDTH NON-JOINER
      * FF9E..FF9F    ; Other_Grapheme_Extend # Lm   [2] HALFWIDTH KATAKANA VOICED SOUND MARK..HALFWIDTH KATAKANA SEMI-VOICED SOUND MARK
      * E0020..E007F  ; Other_Grapheme_Extend # Cf  [96] TAG SPACE..CANCEL TAG
      *
      * ZWNJ is special, we don't want to merge it as there's no need, and keeping
-     * it separate results in more granular clusters.
+     * it separate results in more granular clusters.  Ignore Katakana for now.
      * Tags are used for Emoji sub-region flag sequences:
      * https://github.com/harfbuzz/harfbuzz/issues/1556
-     * Katakana ones were requested:
-     * https://github.com/harfbuzz/harfbuzz/issues/3844
      */
-    else if (unlikely (hb_in_ranges<hb_codepoint_t> (info[i].codepoint, 0xFF9Eu, 0xFF9Fu, 0xE0020u, 0xE007Fu)))
+    else if (unlikely (hb_in_range<hb_codepoint_t> (info[i].codepoint, 0xE0020u, 0xE007Fu)))
       _hb_glyph_info_set_continuation (&info[i]);
   }
 }
@@ -568,9 +534,10 @@ hb_insert_dotted_circle (hb_buffer_t *buffer, hb_font_t *font)
   hb_glyph_info_t info = dottedcircle;
   info.cluster = buffer->cur().cluster;
   info.mask = buffer->cur().mask;
-  (void) buffer->output_info (info);
-
-  buffer->sync ();
+  buffer->output_info (info);
+  while (buffer->idx < buffer->len && buffer->successful)
+    buffer->next_glyph ();
+  buffer->swap_buffers ();
 }
 
 static void
@@ -579,7 +546,7 @@ hb_form_clusters (hb_buffer_t *buffer)
   if (!(buffer->scratch_flags & HB_BUFFER_SCRATCH_FLAG_HAS_NON_ASCII))
     return;
 
-  if (HB_BUFFER_CLUSTER_LEVEL_IS_GRAPHEMES (buffer->cluster_level))
+  if (buffer->cluster_level == HB_BUFFER_CLUSTER_LEVEL_MONOTONE_GRAPHEMES)
     foreach_grapheme (buffer, start, end)
       buffer->merge_clusters (start, end);
   else
@@ -593,55 +560,29 @@ hb_ensure_native_direction (hb_buffer_t *buffer)
   hb_direction_t direction = buffer->props.direction;
   hb_direction_t horiz_dir = hb_script_get_horizontal_direction (buffer->props.script);
 
-  /* Numeric runs in natively-RTL scripts are actually native-LTR, so we reset
-   * the horiz_dir if the run contains at least one decimal-number char, and no
-   * letter chars (ideally we should be checking for chars with strong
-   * directionality but hb-unicode currently lacks bidi categories).
-   *
-   * This allows digit sequences in Arabic etc to be shaped in "native"
-   * direction, so that features like ligatures will work as intended.
-   *
-   * https://github.com/harfbuzz/harfbuzz/issues/501
-   *
-   * Similar thing about Regional_Indicators; They are bidi=L, but Script=Common.
-   * If they are present in a run of natively-RTL text, they get assigned a script
-   * with natively RTL direction, which would result in wrong shaping if we
-   * assign such native RTL direction to them then. Detect that as well.
-   *
-   * https://github.com/harfbuzz/harfbuzz/issues/3314
-   */
-  if (unlikely (horiz_dir == HB_DIRECTION_RTL && direction == HB_DIRECTION_LTR))
-  {
-    bool found_number = false, found_letter = false, found_ri = false;
-    const auto* info = buffer->info;
-    const auto count = buffer->len;
-    for (unsigned i = 0; i < count; i++)
-    {
-      auto gc = _hb_glyph_info_get_general_category (&info[i]);
-      if (gc == HB_UNICODE_GENERAL_CATEGORY_DECIMAL_NUMBER)
-        found_number = true;
-      else if (HB_UNICODE_GENERAL_CATEGORY_IS_LETTER (gc))
-      {
-        found_letter = true;
-        break;
-      }
-      else if (_hb_codepoint_is_regional_indicator (info[i].codepoint))
-        found_ri = true;
-    }
-    if ((found_number || found_ri) && !found_letter)
-      horiz_dir = HB_DIRECTION_LTR;
-  }
-
   /* TODO vertical:
    * The only BTT vertical script is Ogham, but it's not clear to me whether OpenType
    * Ogham fonts are supposed to be implemented BTT or not.  Need to research that
    * first. */
   if ((HB_DIRECTION_IS_HORIZONTAL (direction) &&
-       direction != horiz_dir && HB_DIRECTION_IS_VALID (horiz_dir)) ||
+       direction != horiz_dir && horiz_dir != HB_DIRECTION_INVALID) ||
       (HB_DIRECTION_IS_VERTICAL   (direction) &&
        direction != HB_DIRECTION_TTB))
   {
-    _hb_ot_layout_reverse_graphemes (buffer);
+
+    if (buffer->cluster_level == HB_BUFFER_CLUSTER_LEVEL_MONOTONE_CHARACTERS)
+      foreach_grapheme (buffer, start, end)
+      {
+        buffer->merge_clusters (start, end);
+        buffer->reverse_range (start, end);
+      }
+    else
+      foreach_grapheme (buffer, start, end)
+        /* form_clusters() merged clusters already, we don't merge. */
+        buffer->reverse_range (start, end);
+
+    buffer->reverse ();
+
     buffer->props.direction = HB_DIRECTION_REVERSE (buffer->props.direction);
   }
 }
@@ -651,7 +592,6 @@ hb_ensure_native_direction (hb_buffer_t *buffer)
  * Substitute
  */
 
-#ifndef HB_NO_VERTICAL
 static hb_codepoint_t
 hb_vert_char_for (hb_codepoint_t u)
 {
@@ -702,7 +642,6 @@ hb_vert_char_for (hb_codepoint_t u)
 
   return u;
 }
-#endif
 
 static inline void
 hb_ot_rotate_chars (const hb_ot_shape_context_t *c)
@@ -725,7 +664,6 @@ hb_ot_rotate_chars (const hb_ot_shape_context_t *c)
     }
   }
 
-#ifndef HB_NO_VERTICAL
   if (HB_DIRECTION_IS_VERTICAL (c->target_direction) && !c->plan->has_vert)
   {
     for (unsigned int i = 0; i < count; i++) {
@@ -734,7 +672,6 @@ hb_ot_rotate_chars (const hb_ot_shape_context_t *c)
         info[i].codepoint = codepoint;
     }
   }
-#endif
 }
 
 static inline void
@@ -777,14 +714,6 @@ hb_ot_shape_setup_masks_fraction (const hb_ot_shape_context_t *c)
              _hb_glyph_info_get_general_category (&info[end]) ==
              HB_UNICODE_GENERAL_CATEGORY_DECIMAL_NUMBER)
         end++;
-      if (start == i || end == i + 1)
-      {
-        if (start == i)
-          buffer->unsafe_to_concat (start, start + 1);
-        if (end == i + 1)
-          buffer->unsafe_to_concat (end - 1, end);
-        continue;
-      }
 
       buffer->unsafe_to_break (start, end);
 
@@ -848,28 +777,6 @@ hb_ot_zero_width_default_ignorables (const hb_buffer_t *buffer)
       pos[i].x_advance = pos[i].y_advance = pos[i].x_offset = pos[i].y_offset = 0;
 }
 
-static void
-hb_ot_deal_with_variation_selectors (hb_buffer_t *buffer)
-{
-  if (!(buffer->scratch_flags & HB_BUFFER_SCRATCH_FLAG_HAS_VARIATION_SELECTOR_FALLBACK) ||
-        buffer->not_found_variation_selector == HB_CODEPOINT_INVALID)
-    return;
-
-  unsigned int count = buffer->len;
-  hb_glyph_info_t *info = buffer->info;
-  hb_glyph_position_t *pos = buffer->pos;
-
-  for (unsigned int i = 0; i < count; i++)
-  {
-    if (_hb_glyph_info_is_variation_selector (&info[i]))
-    {
-      info[i].codepoint = buffer->not_found_variation_selector;
-      pos[i].x_advance = pos[i].y_advance = pos[i].x_offset = pos[i].y_offset = 0;
-      _hb_glyph_info_set_variation_selector (&info[i], false);
-    }
-  }
-}
-
 static void
 hb_ot_hide_default_ignorables (hb_buffer_t *buffer,
                                hb_font_t   *font)
@@ -893,7 +800,7 @@ hb_ot_hide_default_ignorables (hb_buffer_t *buffer,
     }
   }
   else
-    buffer->delete_glyphs_inplace (_hb_glyph_info_is_default_ignorable);
+    hb_ot_layout_delete_glyphs_inplace (buffer, _hb_glyph_info_is_default_ignorable);
 }
 
 
@@ -958,7 +865,7 @@ hb_ot_substitute_default (const hb_ot_shape_context_t *c)
 }
 
 static inline void
-hb_ot_substitute_plan (const hb_ot_shape_context_t *c)
+hb_ot_substitute_complex (const hb_ot_shape_context_t *c)
 {
   hb_buffer_t *buffer = c->buffer;
 
@@ -967,13 +874,7 @@ hb_ot_substitute_plan (const hb_ot_shape_context_t *c)
   if (c->plan->fallback_glyph_classes)
     hb_synthesize_glyph_classes (c->buffer);
 
-#ifndef HB_NO_AAT_SHAPE
-  if (unlikely (c->plan->apply_morx))
-    hb_aat_layout_substitute (c->plan, c->font, c->buffer,
-                              c->user_features, c->num_user_features);
-  else
-#endif
-    c->plan->substitute (c->font, buffer);
+  c->plan->substitute (c->font, buffer);
 }
 
 static inline void
@@ -983,30 +884,20 @@ hb_ot_substitute_pre (const hb_ot_shape_context_t *c)
 
   _hb_buffer_allocate_gsubgpos_vars (c->buffer);
 
-  hb_ot_substitute_plan (c);
-
-#ifndef HB_NO_AAT_SHAPE
-  if (c->plan->apply_morx && c->plan->apply_gpos)
-    hb_aat_layout_remove_deleted_glyphs (c->buffer);
-#endif
+  hb_ot_substitute_complex (c);
 }
 
 static inline void
 hb_ot_substitute_post (const hb_ot_shape_context_t *c)
 {
+  hb_ot_hide_default_ignorables (c->buffer, c->font);
 #ifndef HB_NO_AAT_SHAPE
-  if (c->plan->apply_morx && !c->plan->apply_gpos)
+  if (c->plan->apply_morx)
     hb_aat_layout_remove_deleted_glyphs (c->buffer);
 #endif
 
-  hb_ot_deal_with_variation_selectors (c->buffer);
-  hb_ot_hide_default_ignorables (c->buffer, c->font);
-
-  if (c->plan->shaper->postprocess_glyphs &&
-    c->buffer->message(c->font, "start postprocess-glyphs")) {
+  if (c->plan->shaper->postprocess_glyphs)
     c->plan->shaper->postprocess_glyphs (c->plan, c->buffer, c->font);
-    (void) c->buffer->message(c->font, "end postprocess-glyphs");
-  }
 }
 
 
@@ -1077,7 +968,7 @@ hb_ot_position_default (const hb_ot_shape_context_t *c)
 }
 
 static inline void
-hb_ot_position_plan (const hb_ot_shape_context_t *c)
+hb_ot_position_complex (const hb_ot_shape_context_t *c)
 {
   unsigned int count = c->buffer->len;
   hb_glyph_info_t *info = c->buffer->info;
@@ -1089,8 +980,8 @@ hb_ot_position_plan (const hb_ot_shape_context_t *c)
    * direction is backward we don't shift and it will end up
    * hanging over the next glyph after the final reordering.
    *
-   * Note: If fallback positioning happens, we don't care about
-   * this as it will be overridden.
+   * Note: If fallback positinoing happens, we don't care about
+   * this as it will be overriden.
    */
   bool adjust_offsets_when_zeroing = c->plan->adjust_mark_positioning_when_zeroing &&
                                      HB_DIRECTION_IS_FORWARD (c->buffer->props.direction);
@@ -1137,6 +1028,10 @@ hb_ot_position_plan (const hb_ot_shape_context_t *c)
   /* Finish off.  Has to follow a certain order. */
   hb_ot_layout_position_finish_advances (c->font, c->buffer);
   hb_ot_zero_width_default_ignorables (c->buffer);
+#ifndef HB_NO_AAT_SHAPE
+  if (c->plan->apply_morx)
+    hb_aat_layout_zero_width_deleted_glyphs (c->buffer);
+#endif
   hb_ot_layout_position_finish_offsets (c->font, c->buffer);
 
   /* The nil glyph_h_origin() func returns 0, so no need to apply it. */
@@ -1158,7 +1053,7 @@ hb_ot_position (const hb_ot_shape_context_t *c)
 
   hb_ot_position_default (c);
 
-  hb_ot_position_plan (c);
+  hb_ot_position_complex (c);
 
   if (HB_DIRECTION_IS_BACKWARD (c->buffer->props.direction))
     hb_buffer_reverse (c->buffer);
@@ -1172,42 +1067,23 @@ hb_propagate_flags (hb_buffer_t *buffer)
   /* Propagate cluster-level glyph flags to be the same on all cluster glyphs.
    * Simplifies using them. */
 
-  if (!(buffer->scratch_flags & HB_BUFFER_SCRATCH_FLAG_HAS_GLYPH_FLAGS))
+  if (!(buffer->scratch_flags & HB_BUFFER_SCRATCH_FLAG_HAS_UNSAFE_TO_BREAK))
     return;
 
-  /* If we are producing SAFE_TO_INSERT_TATWEEL, then do two things:
-   *
-   * - If the places that the Arabic shaper marked as SAFE_TO_INSERT_TATWEEL,
-   *   are UNSAFE_TO_BREAK, then clear the SAFE_TO_INSERT_TATWEEL,
-   * - Any place that is SAFE_TO_INSERT_TATWEEL, is also now UNSAFE_TO_BREAK.
-   *
-   * We couldn't make this interaction earlier. It has to be done here.
-   */
-  bool flip_tatweel = buffer->flags & HB_BUFFER_FLAG_PRODUCE_SAFE_TO_INSERT_TATWEEL;
-
-  bool clear_concat = (buffer->flags & HB_BUFFER_FLAG_PRODUCE_UNSAFE_TO_CONCAT) == 0;
-
   hb_glyph_info_t *info = buffer->info;
 
   foreach_cluster (buffer, start, end)
   {
     unsigned int mask = 0;
     for (unsigned int i = start; i < end; i++)
-      mask |= info[i].mask & HB_GLYPH_FLAG_DEFINED;
-
-    if (flip_tatweel)
-    {
-      if (mask & HB_GLYPH_FLAG_UNSAFE_TO_BREAK)
-        mask &= ~HB_GLYPH_FLAG_SAFE_TO_INSERT_TATWEEL;
-      if (mask & HB_GLYPH_FLAG_SAFE_TO_INSERT_TATWEEL)
-        mask |= HB_GLYPH_FLAG_UNSAFE_TO_BREAK | HB_GLYPH_FLAG_UNSAFE_TO_CONCAT;
-    }
-
-    if (clear_concat)
-        mask &= ~HB_GLYPH_FLAG_UNSAFE_TO_CONCAT;
-
-    for (unsigned int i = start; i < end; i++)
-      info[i].mask = mask;
+      if (info[i].mask & HB_GLYPH_FLAG_UNSAFE_TO_BREAK)
+      {
+         mask = HB_GLYPH_FLAG_UNSAFE_TO_BREAK;
+         break;
+      }
+    if (mask)
+      for (unsigned int i = start; i < end; i++)
+        info[i].mask |= mask;
   }
 }
 
@@ -1216,11 +1092,26 @@ hb_propagate_flags (hb_buffer_t *buffer)
 static void
 hb_ot_shape_internal (hb_ot_shape_context_t *c)
 {
+  c->buffer->deallocate_var_all ();
+  c->buffer->scratch_flags = HB_BUFFER_SCRATCH_FLAG_DEFAULT;
+  if (likely (!hb_unsigned_mul_overflows (c->buffer->len, HB_BUFFER_MAX_LEN_FACTOR)))
+  {
+    c->buffer->max_len = hb_max (c->buffer->len * HB_BUFFER_MAX_LEN_FACTOR,
+                                 (unsigned) HB_BUFFER_MAX_LEN_MIN);
+  }
+  if (likely (!hb_unsigned_mul_overflows (c->buffer->len, HB_BUFFER_MAX_OPS_FACTOR)))
+  {
+    c->buffer->max_ops = hb_max (c->buffer->len * HB_BUFFER_MAX_OPS_FACTOR,
+                                 (unsigned) HB_BUFFER_MAX_OPS_MIN);
+  }
+
   /* Save the original direction, we use it later. */
   c->target_direction = c->buffer->props.direction;
 
   _hb_buffer_allocate_unicode_vars (c->buffer);
 
+  c->buffer->clear_output ();
+
   hb_ot_shape_initialize_masks (c);
   hb_set_unicode_props (c->buffer);
   hb_insert_dotted_circle (c->buffer, c->font);
@@ -1229,12 +1120,8 @@ hb_ot_shape_internal (hb_ot_shape_context_t *c)
 
   hb_ensure_native_direction (c->buffer);
 
-  if (c->plan->shaper->preprocess_text &&
-      c->buffer->message(c->font, "start preprocess-text"))
-  {
+  if (c->plan->shaper->preprocess_text)
     c->plan->shaper->preprocess_text (c->plan, c->buffer, c->font);
-    (void) c->buffer->message(c->font, "end preprocess-text");
-  }
 
   hb_ot_substitute_pre (c);
   hb_ot_position (c);
@@ -1246,7 +1133,9 @@ hb_ot_shape_internal (hb_ot_shape_context_t *c)
 
   c->buffer->props.direction = c->target_direction;
 
-  c->buffer->leave ();
+  c->buffer->max_len = HB_BUFFER_MAX_LEN_DEFAULT;
+  c->buffer->max_ops = HB_BUFFER_MAX_OPS_DEFAULT;
+  c->buffer->deallocate_var_all ();
 }
 
 
@@ -1266,12 +1155,6 @@ _hb_ot_shape (hb_shape_plan_t    *shape_plan,
 
 /**
  * hb_ot_shape_plan_collect_lookups:
- * @shape_plan: #hb_shape_plan_t to query
- * @table_tag: GSUB or GPOS
- * @lookup_indexes: (out): The #hb_set_t set of lookups returned
- *
- * Computes the complete set of GSUB or GPOS lookups that are applicable
- * under a given @shape_plan.
  *
  * Since: 0.9.7
  **/
@@ -1284,36 +1167,6 @@ hb_ot_shape_plan_collect_lookups (hb_shape_plan_t *shape_plan,
 }
 
 
-/**
- * hb_ot_shape_plan_get_feature_tags:
- * @shape_plan: A shaping plan
- * @start_offset: The index of first feature to retrieve
- * @tag_count: (inout): Input = the maximum number of features to return;
- *                      Output = the actual number of features returned (may be zero)
- * @tags: (out) (array length=tag_count): The array of enabled feature
- *
- * Fetches the list of OpenType feature tags enabled for a shaping plan, if possible.
- *
- * Return value: Total number of feature tagss.
- *
- * Since: 10.3.0
- */
-unsigned int
-hb_ot_shape_plan_get_feature_tags (hb_shape_plan_t *shape_plan,
-                                   unsigned int     start_offset,
-                                   unsigned int    *tag_count, /* IN/OUT */
-                                   hb_tag_t        *tags /* OUT */)
-{
-#ifndef HB_NO_OT_SHAPE
-  return shape_plan->ot.map.get_feature_tags (start_offset, tag_count, tags);
-#else
-  if (tag_count)
-        *tag_count = 0;
-  return 0;
-#endif
-}
-
-
 /* TODO Move this to hb-ot-shape-normalize, make it do decompose, and make it public. */
 static void
 add_char (hb_font_t          *font,
@@ -1336,15 +1189,6 @@ add_char (hb_font_t          *font,
 
 /**
  * hb_ot_shape_glyphs_closure:
- * @font: #hb_font_t to work upon
- * @buffer: The input buffer to compute from
- * @features: (array length=num_features): The features enabled on the buffer
- * @num_features: The number of features enabled on the buffer
- * @glyphs: (out): The #hb_set_t set of glyphs comprising the transitive closure of the query
- *
- * Computes the transitive closure of glyphs needed for a specified
- * input buffer under the given font and feature list. The closure is
- * computed as a set, not as a list.
  *
  * Since: 0.9.2
  **/
