@@ -32,59 +32,36 @@
 #include "hb-ot-cff1-table.hh"
 #include "hb-set.h"
 #include "hb-bimap.hh"
+#include "hb-subset-cff1.hh"
 #include "hb-subset-plan.hh"
 #include "hb-subset-cff-common.hh"
 #include "hb-cff1-interp-cs.hh"
 
 using namespace CFF;
 
-struct remap_sid_t
+struct remap_sid_t : hb_inc_bimap_t
 {
-  unsigned get_population () const { return vector.length; }
-
-  void alloc (unsigned size)
-  {
-    map.alloc (size);
-    vector.alloc_exact (size);
-  }
-
-  bool in_error () const
-  { return map.in_error () || vector.in_error (); }
-
   unsigned int add (unsigned int sid)
   {
-    if (is_std_str (sid) || (sid == CFF_UNDEF_SID))
-      return sid;
-
-    sid = unoffset_sid (sid);
-    unsigned v = next;
-    if (map.set (sid, v, false))
-    {
-      vector.push (sid);
-      next++;
-    }
+    if ((sid != CFF_UNDEF_SID) && !is_std_std (sid))
+      return offset_sid (hb_inc_bimap_t::add (unoffset_sid (sid)));
     else
-      v = map.get (sid); // already exists
-    return offset_sid (v);
+      return sid;
   }
 
   unsigned int operator[] (unsigned int sid) const
   {
-    if (is_std_str (sid) || (sid == CFF_UNDEF_SID))
+    if (is_std_std (sid) || (sid == CFF_UNDEF_SID))
       return sid;
-
-    return offset_sid (map.get (unoffset_sid (sid)));
+    else
+      return offset_sid (get (unoffset_sid (sid)));
   }
 
   static const unsigned int num_std_strings = 391;
 
-  static bool is_std_str (unsigned int sid) { return sid < num_std_strings; }
+  static bool is_std_std (unsigned int sid) { return sid < num_std_strings; }
   static unsigned int offset_sid (unsigned int sid) { return sid + num_std_strings; }
   static unsigned int unoffset_sid (unsigned int sid) { return sid - num_std_strings; }
-  unsigned next = 0;
-
-  hb_map_t map;
-  hb_vector_t<unsigned> vector;
 };
 
 struct cff1_sub_table_info_t : cff_sub_table_info_t
@@ -190,10 +167,9 @@ struct cff1_top_dict_op_serializer_t : cff_top_dict_op_serializer_t<cff1_top_dic
            * for supplement, the original byte string is copied along with the op code */
           op_str_t supp_op;
           supp_op.op = op;
-          if ( unlikely (!(opstr.length >= opstr.last_arg_offset + 3)))
+          if ( unlikely (!(opstr.str.length >= opstr.last_arg_offset + 3)))
             return_trace (false);
-          supp_op.ptr = opstr.ptr + opstr.last_arg_offset;
-          supp_op.length = opstr.length - opstr.last_arg_offset;
+          supp_op.str = byte_str_t (&opstr.str + opstr.last_arg_offset, opstr.str.length - opstr.last_arg_offset);
           return_trace (UnsizedByteStr::serialize_int2 (c, mod.nameSIDs[name_dict_values_t::registry]) &&
                         UnsizedByteStr::serialize_int2 (c, mod.nameSIDs[name_dict_values_t::ordering]) &&
                         copy_opstr (c, supp_op));
@@ -257,7 +233,7 @@ struct cff1_cs_opset_flatten_t : cff1_cs_opset_t<cff1_cs_opset_flatten_t, flatte
   {
     str_encoder_t  encoder (param.flatStr);
     for (unsigned int i = env.arg_start; i < env.argStack.get_count (); i++)
-      encoder.encode_num_cs (env.eval_arg (i));
+      encoder.encode_num (env.eval_arg (i));
     SUPER::flush_args (env, param);
   }
 
@@ -271,7 +247,7 @@ struct cff1_cs_opset_flatten_t : cff1_cs_opset_t<cff1_cs_opset_flatten_t, flatte
   {
     assert (env.has_width);
     str_encoder_t  encoder (param.flatStr);
-    encoder.encode_num_cs (env.width);
+    encoder.encode_num (env.width);
   }
 
   static void flush_hintmask (op_code_t op, cff1_cs_interp_env_t &env, flatten_param_t& param)
@@ -294,17 +270,16 @@ struct range_list_t : hb_vector_t<code_pair_t>
   /* replace the first glyph ID in the "glyph" field each range with a nLeft value */
   bool complete (unsigned int last_glyph)
   {
-    hb_codepoint_t all_glyphs = 0;
-    unsigned count = this->length;
-    for (unsigned int i = count; i; i--)
+    bool  two_byte = false;
+    for (unsigned int i = (*this).length; i > 0; i--)
     {
-      code_pair_t &pair = arrayZ[i - 1];
-      unsigned int nLeft = last_glyph - pair.glyph - 1;
-      all_glyphs |= nLeft;
+      code_pair_t &pair = (*this)[i - 1];
+      unsigned int  nLeft = last_glyph - pair.glyph - 1;
+      if (nLeft >= 0x100)
+        two_byte = true;
       last_glyph = pair.glyph;
       pair.glyph = nLeft;
     }
-    bool two_byte = all_glyphs >= 0x100;
     return two_byte;
   }
 };
@@ -359,36 +334,6 @@ struct cff1_cs_opset_subr_subset_t : cff1_cs_opset_t<cff1_cs_opset_subr_subset_t
   typedef cff1_cs_opset_t<cff1_cs_opset_subr_subset_t, subr_subset_param_t> SUPER;
 };
 
-struct cff1_private_dict_op_serializer_t : op_serializer_t
-{
-  cff1_private_dict_op_serializer_t (bool desubroutinize_, bool drop_hints_)
-    : desubroutinize (desubroutinize_), drop_hints (drop_hints_) {}
-
-  bool serialize (hb_serialize_context_t *c,
-                  const op_str_t &opstr,
-                  objidx_t subrs_link) const
-  {
-    TRACE_SERIALIZE (this);
-
-    if (drop_hints && dict_opset_t::is_hint_op (opstr.op))
-      return_trace (true);
-
-    if (opstr.op == OpCode_Subrs)
-    {
-      if (desubroutinize || !subrs_link)
-        return_trace (true);
-      else
-        return_trace (FontDict::serialize_link2_op (c, opstr.op, subrs_link));
-    }
-
-    return_trace (copy_opstr (c, opstr));
-  }
-
-  protected:
-  const bool desubroutinize;
-  const bool drop_hints;
-};
-
 struct cff1_subr_subsetter_t : subr_subsetter_t<cff1_subr_subsetter_t, CFF1Subrs, const OT::cff1::accelerator_subset_t, cff1_cs_interp_env_t, cff1_cs_opset_subr_subset_t, OpCode_endchar>
 {
   cff1_subr_subsetter_t (const OT::cff1::accelerator_subset_t &acc_, const hb_subset_plan_t *plan_)
@@ -415,20 +360,50 @@ struct cff1_subr_subsetter_t : subr_subsetter_t<cff1_subr_subsetter_t, CFF1Subrs
   }
 };
 
-namespace OT {
-struct cff1_subset_plan
-{
-  cff1_subset_plan ()
+struct cff_subset_plan {
+  cff_subset_plan ()
+    : info (),
+      orig_fdcount (0),
+      subset_fdcount (1),
+      subset_fdselect_format (0),
+      drop_hints (false),
+      desubroutinize(false)
   {
+    topdict_mod.init ();
+    subset_fdselect_ranges.init ();
+    fdmap.init ();
+    subset_charstrings.init ();
+    subset_globalsubrs.init ();
+    subset_localsubrs.init ();
+    fontdicts_mod.init ();
+    subset_enc_code_ranges.init ();
+    subset_enc_supp_codes.init ();
+    subset_charset_ranges.init ();
+    sidmap.init ();
     for (unsigned int i = 0; i < name_dict_values_t::ValCount; i++)
       topDictModSIDs[i] = CFF_UNDEF_SID;
   }
 
+  ~cff_subset_plan ()
+  {
+    topdict_mod.fini ();
+    subset_fdselect_ranges.fini ();
+    fdmap.fini ();
+    subset_charstrings.fini_deep ();
+    subset_globalsubrs.fini_deep ();
+    subset_localsubrs.fini_deep ();
+    fontdicts_mod.fini ();
+    subset_enc_code_ranges.fini ();
+    subset_enc_supp_codes.fini ();
+    subset_charset_ranges.fini ();
+    sidmap.fini ();
+  }
+
   void plan_subset_encoding (const OT::cff1::accelerator_subset_t &acc, hb_subset_plan_t *plan)
   {
     const Encoding *encoding = acc.encoding;
-    unsigned int  size0, size1;
-    unsigned code, last_code = CFF_UNDEF_CODE - 1;
+    unsigned int  size0, size1, supp_size;
+    hb_codepoint_t  code, last_code = CFF_UNDEF_CODE;
     hb_vector_t<hb_codepoint_t> supp_codes;
 
     if (unlikely (!subset_enc_code_ranges.resize (0)))
@@ -437,44 +412,43 @@ struct cff1_subset_plan
       return;
     }
 
+    supp_size = 0;
     supp_codes.init ();
 
-    code_pair_t glyph_to_sid_cache {0, HB_CODEPOINT_INVALID};
     subset_enc_num_codes = plan->num_output_glyphs () - 1;
     unsigned int glyph;
-    auto it = hb_iter (plan->new_to_old_gid_list);
-    if (it->first == 0) it++;
-    auto _ = *it;
-    for (glyph = 1; glyph < num_glyphs; glyph++)
+    for (glyph = 1; glyph < plan->num_output_glyphs (); glyph++)
     {
-      hb_codepoint_t old_glyph;
-      if (glyph == _.first)
+      hb_codepoint_t  old_glyph;
+      if (!plan->old_gid_for_new_gid (glyph, &old_glyph))
       {
-        old_glyph = _.second;
-        _ = *++it;
-      }
-      else
-      {
-        /* Retain the SID for the old missing glyph ID */
+        /* Retain the code for the old missing glyph ID */
         old_glyph = glyph;
       }
-      code = acc.glyph_to_code (old_glyph, &glyph_to_sid_cache);
+      code = acc.glyph_to_code (old_glyph);
       if (code == CFF_UNDEF_CODE)
       {
         subset_enc_num_codes = glyph - 1;
         break;
       }
 
-      if (code != last_code + 1)
-        subset_enc_code_ranges.push (code_pair_t {code, glyph});
+      if ((last_code == CFF_UNDEF_CODE) || (code != last_code + 1))
+      {
+        code_pair_t pair = { code, glyph };
+        subset_enc_code_ranges.push (pair);
+      }
       last_code = code;
 
       if (encoding != &Null (Encoding))
       {
-        hb_codepoint_t  sid = acc.glyph_to_sid (old_glyph, &glyph_to_sid_cache);
+        hb_codepoint_t  sid = acc.glyph_to_sid (old_glyph);
         encoding->get_supplement_codes (sid, supp_codes);
         for (unsigned int i = 0; i < supp_codes.length; i++)
-          subset_enc_supp_codes.push (code_pair_t {supp_codes[i], sid});
+        {
+          code_pair_t pair = { supp_codes[i], sid };
+          subset_enc_supp_codes.push (pair);
+        }
+        supp_size += SuppEncoding::static_size * supp_codes.length;
       }
     }
     supp_codes.fini ();
@@ -491,93 +465,46 @@ struct cff1_subset_plan
       subset_enc_format = 1;
   }
 
-  bool plan_subset_charset (const OT::cff1::accelerator_subset_t &acc, hb_subset_plan_t *plan)
+  void plan_subset_charset (const OT::cff1::accelerator_subset_t &acc, hb_subset_plan_t *plan)
   {
     unsigned int  size0, size_ranges;
-    unsigned last_sid = CFF_UNDEF_CODE - 1;
+    hb_codepoint_t  sid, last_sid = CFF_UNDEF_CODE;
 
     if (unlikely (!subset_charset_ranges.resize (0)))
     {
       plan->check_success (false);
-      return false;
+      return;
     }
 
-    code_pair_t glyph_to_sid_cache {0, HB_CODEPOINT_INVALID};
-
-    unsigned num_glyphs = plan->num_output_glyphs ();
-
-    if (unlikely (!subset_charset_ranges.alloc (hb_min (num_glyphs,
-                                                        acc.num_charset_entries))))
+    unsigned int glyph;
+    for (glyph = 1; glyph < plan->num_output_glyphs (); glyph++)
     {
-      plan->check_success (false);
-      return false;
-    }
-
-    glyph_to_sid_map_t *glyph_to_sid_map = acc.cff_accelerator ?
-                                           acc.cff_accelerator->glyph_to_sid_map.get_acquire () :
-                                           nullptr;
-    bool created_map = false;
-    if (!glyph_to_sid_map && acc.cff_accelerator)
-    {
-      created_map = true;
-      glyph_to_sid_map = acc.create_glyph_to_sid_map ();
-    }
-
-    auto it = hb_iter (plan->new_to_old_gid_list);
-    if (it->first == 0) it++;
-    auto _ = *it;
-    bool not_is_cid = !acc.is_CID ();
-    bool skip = !not_is_cid && glyph_to_sid_map;
-    if (not_is_cid)
-      sidmap.alloc (num_glyphs);
-    for (hb_codepoint_t glyph = 1; glyph < num_glyphs; glyph++)
-    {
-      hb_codepoint_t old_glyph;
-      if (glyph == _.first)
-      {
-        old_glyph = _.second;
-        _ = *++it;
-      }
-      else
+      hb_codepoint_t  old_glyph;
+      if (!plan->old_gid_for_new_gid (glyph, &old_glyph))
       {
         /* Retain the SID for the old missing glyph ID */
         old_glyph = glyph;
       }
-      unsigned sid = glyph_to_sid_map ?
-                     glyph_to_sid_map->arrayZ[old_glyph].code :
-                     acc.glyph_to_sid (old_glyph, &glyph_to_sid_cache);
+      sid = acc.glyph_to_sid (old_glyph);
 
-      if (not_is_cid)
+      if (!acc.is_CID ())
         sid = sidmap.add (sid);
 
-      if (sid != last_sid + 1)
-        subset_charset_ranges.push (code_pair_t {sid, glyph});
-
-      if (glyph == old_glyph && skip)
+      if ((last_sid == CFF_UNDEF_CODE) || (sid != last_sid + 1))
       {
-        glyph = hb_min (_.first - 1, glyph_to_sid_map->arrayZ[old_glyph].glyph);
-        sid += glyph - old_glyph;
+        code_pair_t pair = { sid, glyph };
+        subset_charset_ranges.push (pair);
       }
       last_sid = sid;
     }
 
-    if (created_map)
-    {
-      if ((!plan->accelerator && acc.cff_accelerator) ||
-          !acc.cff_accelerator->glyph_to_sid_map.cmpexch (nullptr, glyph_to_sid_map))
-      {
-        glyph_to_sid_map->~glyph_to_sid_map_t ();
-        hb_free (glyph_to_sid_map);
-      }
-    }
+    bool two_byte = subset_charset_ranges.complete (glyph);
 
-    bool two_byte = subset_charset_ranges.complete (num_glyphs);
-
-    size0 = Charset0::get_size (plan->num_output_glyphs ());
+    size0 = Charset0::min_size + HBUINT16::static_size * (plan->num_output_glyphs () - 1);
     if (!two_byte)
-      size_ranges = Charset1::get_size_for_ranges (subset_charset_ranges.length);
+      size_ranges = Charset1::min_size + Charset1_Range::static_size * subset_charset_ranges.length;
     else
-      size_ranges = Charset2::get_size_for_ranges (subset_charset_ranges.length);
+      size_ranges = Charset2::min_size + Charset2_Range::static_size * subset_charset_ranges.length;
 
     if (size0 < size_ranges)
       subset_charset_format = 0;
@@ -585,18 +512,19 @@ struct cff1_subset_plan
       subset_charset_format = 1;
     else
       subset_charset_format = 2;
-
-    return true;
   }
 
   bool collect_sids_in_dicts (const OT::cff1::accelerator_subset_t &acc)
   {
+    sidmap.reset ();
+
     for (unsigned int i = 0; i < name_dict_values_t::ValCount; i++)
     {
       unsigned int sid = acc.topDict.nameSIDs[i];
       if (sid != CFF_UNDEF_SID)
       {
-        topDictModSIDs[i] = sidmap.add (sid);
+        (void)sidmap.add (sid);
+        topDictModSIDs[i] = sidmap[sid];
       }
     }
 
@@ -617,27 +545,22 @@ struct cff1_subset_plan
 
     num_glyphs = plan->num_output_glyphs ();
     orig_fdcount = acc.fdCount;
-    drop_hints = plan->flags & HB_SUBSET_FLAGS_NO_HINTING;
-    desubroutinize = plan->flags & HB_SUBSET_FLAGS_DESUBROUTINIZE;
+    drop_hints = plan->drop_hints;
+    desubroutinize = plan->desubroutinize;
 
- #ifdef HB_EXPERIMENTAL_API
-    min_charstrings_off_size = (plan->flags & HB_SUBSET_FLAGS_IFTB_REQUIREMENTS) ? 4 : 0;
- #else
-    min_charstrings_off_size = 0;
- #endif
-
-    subset_charset = !acc.is_predef_charset ();
-    if (!subset_charset)
-      /* check whether the subset renumbers any glyph IDs */
-      for (const auto &_ : plan->new_to_old_gid_list)
-      {
-        if (_.first != _.second)
-        {
-          subset_charset = true;
-          break;
-        }
+    /* check whether the subset renumbers any glyph IDs */
+    gid_renum = false;
+    for (hb_codepoint_t new_glyph = 0; new_glyph < plan->num_output_glyphs (); new_glyph++)
+    {
+      if (!plan->old_gid_for_new_gid(new_glyph, &old_glyph))
+        continue;
+      if (new_glyph != old_glyph) {
+        gid_renum = true;
+        break;
       }
+    }
 
+    subset_charset = gid_renum || !acc.is_predef_charset ();
     subset_encoding = !acc.is_CID() && !acc.is_predef_encoding ();
 
     /* top dict INDEX */
@@ -679,8 +602,7 @@ struct cff1_subset_plan
       if (unlikely (sidmap.get_population () > 0x8000)) /* assumption: a dict won't reference that many strings */
         return false;
 
-      if (subset_charset && !plan_subset_charset (acc, plan))
-        return false;
+      if (subset_charset) plan_subset_charset (acc, plan);
 
       topdict_mod.reassignSIDs (sidmap);
     }
@@ -744,18 +666,17 @@ struct cff1_subset_plan
       ;
     }
 
-    return !plan->in_error () &&
-           (subset_charstrings.length == plan->num_output_glyphs ()) &&
-           (fontdicts_mod.length == subset_fdcount);
+    return ((subset_charstrings.length == plan->num_output_glyphs ())
+           && (fontdicts_mod.length == subset_fdcount));
   }
 
   cff1_top_dict_values_mod_t    topdict_mod;
   cff1_sub_table_info_t         info;
 
   unsigned int    num_glyphs;
-  unsigned int    orig_fdcount = 0;
-  unsigned int    subset_fdcount = 1;
-  unsigned int    subset_fdselect_format = 0;
+  unsigned int    orig_fdcount;
+  unsigned int    subset_fdcount;
+  unsigned int    subset_fdselect_format;
   hb_vector_t<code_pair_t>   subset_fdselect_ranges;
 
   /* font dict index remap table from fullset FDArray to subset FDArray.
@@ -767,7 +688,7 @@ struct cff1_subset_plan
   hb_vector_t<str_buff_vec_t>   subset_localsubrs;
   hb_vector_t<cff1_font_dict_values_mod_t>  fontdicts_mod;
 
-  bool          drop_hints = false;
+  bool          drop_hints;
 
   bool          gid_renum;
   bool          subset_encoding;
@@ -783,54 +704,26 @@ struct cff1_subset_plan
   remap_sid_t   sidmap;
   unsigned int  topDictModSIDs[name_dict_values_t::ValCount];
 
-  bool          desubroutinize = false;
-
-  unsigned      min_charstrings_off_size = 0;
+  bool          desubroutinize;
 };
-} // namespace OT
 
-static bool _serialize_cff1_charstrings (hb_serialize_context_t *c,
-                                         struct OT::cff1_subset_plan &plan,
-                                         const OT::cff1::accelerator_subset_t  &acc)
+static bool _serialize_cff1 (hb_serialize_context_t *c,
+                             cff_subset_plan &plan,
+                             const OT::cff1::accelerator_subset_t  &acc,
+                             unsigned int num_glyphs)
 {
-  c->push<CFF1CharStrings> ();
-
-  unsigned data_size = 0;
-  unsigned total_size = CFF1CharStrings::total_size (plan.subset_charstrings, &data_size, plan.min_charstrings_off_size);
-  if (unlikely (!c->start_zerocopy (total_size)))
-    return false;
-
-  auto *cs = c->start_embed<CFF1CharStrings> ();
-  if (unlikely (!cs->serialize (c, plan.subset_charstrings, &data_size, plan.min_charstrings_off_size))) {
-    c->pop_discard ();
-    return false;
-  }
-
-  plan.info.char_strings_link = c->pop_pack (false);
-  return true;
-}
-
-bool
-OT::cff1::accelerator_subset_t::serialize (hb_serialize_context_t *c,
-                                           struct OT::cff1_subset_plan &plan) const
-{
-  /* push charstrings onto the object stack first which will ensure it packs as the last
-     object in the table. Keeping the chastrings last satisfies the requirements for patching
-     via IFTB. If this ordering needs to be changed in the future, charstrings should be left
-     at the end whenever HB_SUBSET_FLAGS_ITFB_REQUIREMENTS is enabled. */
-  if (!_serialize_cff1_charstrings(c, plan, *this))
-    return false;
-
   /* private dicts & local subrs */
-  for (int i = (int) privateDicts.length; --i >= 0 ;)
+  for (int i = (int)acc.privateDicts.length; --i >= 0 ;)
   {
     if (plan.fdmap.has (i))
     {
       objidx_t  subrs_link = 0;
       if (plan.subset_localsubrs[i].length > 0)
       {
-        auto *dest = c->push <CFF1Subrs> ();
-        if (likely (dest->serialize (c, plan.subset_localsubrs[i])))
+        CFF1Subrs *dest = c->start_embed <CFF1Subrs> ();
+        if (unlikely (!dest)) return false;
+        c->push ();
+        if (likely (dest && dest->serialize (c, plan.subset_localsubrs[i])))
           subrs_link = c->pop_pack ();
         else
         {
@@ -839,10 +732,12 @@ OT::cff1::accelerator_subset_t::serialize (hb_serialize_context_t *c,
         }
       }
 
-      auto *pd = c->push<PrivateDict> ();
-      cff1_private_dict_op_serializer_t privSzr (plan.desubroutinize, plan.drop_hints);
+      PrivateDict *pd = c->start_embed<PrivateDict> ();
+      if (unlikely (!pd)) return false;
+      c->push ();
+      cff_private_dict_op_serializer_t privSzr (plan.desubroutinize, plan.drop_hints);
       /* N.B. local subrs immediately follows its corresponding private dict. i.e., subr offset == private dict size */
-      if (likely (pd->serialize (c, privateDicts[i], privSzr, subrs_link)))
+      if (likely (pd->serialize (c, acc.privateDicts[i], privSzr, subrs_link)))
       {
         unsigned fd = plan.fdmap[i];
         plan.fontdicts_mod[fd].privateDictInfo.size = c->length ();
@@ -856,13 +751,29 @@ OT::cff1::accelerator_subset_t::serialize (hb_serialize_context_t *c,
     }
   }
 
-  if (!is_CID ())
+  if (!acc.is_CID ())
     plan.info.privateDictInfo = plan.fontdicts_mod[0].privateDictInfo;
 
-  /* FDArray (FD Index) */
-  if (fdArray != &Null (CFF1FDArray))
+  /* CharStrings */
   {
-    auto *fda = c->push<CFF1FDArray> ();
+    CFF1CharStrings  *cs = c->start_embed<CFF1CharStrings> ();
+    if (unlikely (!cs)) return false;
+    c->push ();
+    if (likely (cs->serialize (c, plan.subset_charstrings)))
+      plan.info.char_strings_link = c->pop_pack ();
+    else
+    {
+      c->pop_discard ();
+      return false;
+    }
+  }
+
+  /* FDArray (FD Index) */
+  if (acc.fdArray != &Null (CFF1FDArray))
+  {
+    CFF1FDArray *fda = c->start_embed<CFF1FDArray> ();
+    if (unlikely (!fda)) return false;
+    c->push ();
     cff1_font_dict_op_serializer_t  fontSzr;
     auto it = + hb_zip (+ hb_iter (plan.fontdicts_mod), + hb_iter (plan.fontdicts_mod));
     if (likely (fda->serialize (c, it, fontSzr)))
@@ -875,10 +786,10 @@ OT::cff1::accelerator_subset_t::serialize (hb_serialize_context_t *c,
   }
 
   /* FDSelect */
-  if (fdSelect != &Null (CFF1FDSelect))
+  if (acc.fdSelect != &Null (CFF1FDSelect))
   {
     c->push ();
-    if (likely (hb_serialize_cff_fdselect (c, plan.num_glyphs, *fdSelect, fdCount,
+    if (likely (hb_serialize_cff_fdselect (c, num_glyphs, *acc.fdSelect, acc.fdCount,
                                            plan.subset_fdselect_format, plan.info.fd_select.size,
                                            plan.subset_fdselect_ranges)))
       plan.info.fd_select.link = c->pop_pack ();
@@ -892,7 +803,9 @@ OT::cff1::accelerator_subset_t::serialize (hb_serialize_context_t *c,
   /* Charset */
   if (plan.subset_charset)
   {
-    auto *dest = c->push<Charset> ();
+    Charset *dest = c->start_embed<Charset> ();
+    if (unlikely (!dest)) return false;
+    c->push ();
     if (likely (dest->serialize (c,
                                  plan.subset_charset_format,
                                  plan.num_glyphs,
@@ -908,7 +821,9 @@ OT::cff1::accelerator_subset_t::serialize (hb_serialize_context_t *c,
   /* Encoding */
   if (plan.subset_encoding)
   {
-    auto *dest = c->push<Encoding> ();
+    Encoding *dest = c->start_embed<Encoding> ();
+    if (unlikely (!dest)) return false;
+    c->push ();
     if (likely (dest->serialize (c,
                                  plan.subset_enc_format,
                                  plan.subset_enc_num_codes,
@@ -924,9 +839,11 @@ OT::cff1::accelerator_subset_t::serialize (hb_serialize_context_t *c,
 
   /* global subrs */
   {
-    auto *dest = c->push <CFF1Subrs> ();
+    c->push ();
+    CFF1Subrs *dest = c->start_embed <CFF1Subrs> ();
+    if (unlikely (!dest)) return false;
     if (likely (dest->serialize (c, plan.subset_globalsubrs)))
-      c->pop_pack (false);
+      c->pop_pack ();
     else
     {
       c->pop_discard ();
@@ -936,9 +853,10 @@ OT::cff1::accelerator_subset_t::serialize (hb_serialize_context_t *c,
 
   /* String INDEX */
   {
-    auto *dest = c->push<CFF1StringIndex> ();
-    if (likely (!plan.sidmap.in_error () &&
-                dest->serialize (c, *stringIndex, plan.sidmap.vector)))
+    CFF1StringIndex *dest = c->start_embed<CFF1StringIndex> ();
+    if (unlikely (!dest)) return false;
+    c->push ();
+    if (likely (dest->serialize (c, *acc.stringIndex, plan.sidmap)))
       c->pop_pack ();
     else
     {
@@ -958,12 +876,14 @@ OT::cff1::accelerator_subset_t::serialize (hb_serialize_context_t *c,
   cff->offSize = 4; /* unused? */
 
   /* name INDEX */
-  if (unlikely (!c->embed (*nameIndex))) return false;
+  if (unlikely (!(*acc.nameIndex).copy (c))) return false;
 
   /* top dict INDEX */
   {
     /* serialize singleton TopDict */
-    auto *top = c->push<TopDict> ();
+    TopDict *top = c->start_embed<TopDict> ();
+    if (!top) return false;
+    c->push ();
     cff1_top_dict_op_serializer_t topSzr;
     unsigned top_size = 0;
     top_dict_modifiers_t  modifier (plan.info, plan.topDictModSIDs);
@@ -978,23 +898,42 @@ OT::cff1::accelerator_subset_t::serialize (hb_serialize_context_t *c,
       return false;
     }
     /* serialize INDEX header for above */
-    auto *dest = c->start_embed<CFF1Index> ();
-    return dest->serialize_header (c, hb_iter (&top_size, 1), top_size);
+    CFF1Index *dest = c->start_embed<CFF1Index> ();
+    if (!dest) return false;
+    return dest->serialize_header (c, hb_iter (hb_array_t<unsigned> (&top_size, 1)));
   }
 }
 
-bool
-OT::cff1::accelerator_subset_t::subset (hb_subset_context_t *c) const
+static bool
+_hb_subset_cff1 (const OT::cff1::accelerator_subset_t  &acc,
+                hb_subset_context_t     *c)
 {
-  cff1_subset_plan cff_plan;
+  cff_subset_plan cff_plan;
 
-  if (unlikely (!cff_plan.create (*this, c->plan)))
+  if (unlikely (!cff_plan.create (acc, c->plan)))
   {
     DEBUG_MSG(SUBSET, nullptr, "Failed to generate a cff subsetting plan.");
     return false;
   }
 
-  return serialize (c->serializer, cff_plan);
+  return _serialize_cff1 (c->serializer, cff_plan, acc, c->plan->num_output_glyphs ());
+}
+
+/**
+ * hb_subset_cff1:
+ * Subsets the CFF table according to a provided plan.
+ *
+ * Return value: subsetted cff table.
+ **/
+bool
+hb_subset_cff1 (hb_subset_context_t *c)
+{
+  OT::cff1::accelerator_subset_t acc;
+  acc.init (c->plan->source);
+  bool result = likely (acc.is_valid ()) && _hb_subset_cff1 (acc, c);
+  acc.fini ();
+
+  return result;
 }
 
 
