@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -31,9 +31,43 @@
 #include "Trace.h"
 #include "Disposer.h"
 #include <lcms2.h>
-#include <lcms2_plugin.h>
 #include "jlong.h"
 
+
+#define ALIGNLONG(x) (((x)+3) & ~(3))         // Aligns to DWORD boundary
+
+#ifdef USE_BIG_ENDIAN
+#define AdjustEndianess32(a)
+#else
+
+static
+void AdjustEndianess32(cmsUInt8Number *pByte)
+{
+    cmsUInt8Number temp1;
+    cmsUInt8Number temp2;
+
+    temp1 = *pByte++;
+    temp2 = *pByte++;
+    *(pByte-1) = *pByte;
+    *pByte++ = temp2;
+    *(pByte-3) = *pByte;
+    *pByte = temp1;
+}
+
+#endif
+
+// Transports to properly encoded values - note that icc profiles does use
+// big endian notation.
+
+static
+cmsInt32Number TransportValue32(cmsInt32Number Value)
+{
+    cmsInt32Number Temp = Value;
+
+    AdjustEndianess32((cmsUInt8Number*) &Temp);
+    return Temp;
+}
+
 #define SigMake(a,b,c,d) \
                     ( ( ((int) ((unsigned char) (a))) << 24) | \
                       ( ((int) ((unsigned char) (b))) << 16) | \
@@ -615,22 +649,9 @@ JNIEXPORT jobject JNICALL Java_sun_java2d_cmm_lcms_LCMS_getProfileID
     if (pf == NULL) {
         return NULL;
     }
-
-    jclass pcls = (*env)->GetObjectClass(env, pf);
-    if (pcls == NULL) {
-        return NULL;
-    }
-    jmethodID mid = (*env)->GetMethodID(env, pcls, "activate", "()V");
-    if (mid == NULL) {
-        return NULL;
-    }
-    (*env)->CallVoidMethod(env, pf, mid);
-    if ((*env)->ExceptionOccurred(env)) {
-        return NULL;
-    }
-
-    fid = (*env)->GetFieldID(env, pcls, "cmmProfile",
-                             "Lsun/java2d/cmm/Profile;");
+    fid = (*env)->GetFieldID (env,
+        (*env)->GetObjectClass(env, pf),
+        "cmmProfile", "Lsun/java2d/cmm/Profile;");
     if (fid == NULL) {
         return NULL;
     }
@@ -750,18 +771,16 @@ static cmsBool _setHeaderInfo(cmsHPROFILE pf, jbyte* pBuffer, jint bufferSize)
   memcpy(&pfHeader, pBuffer, sizeof(cmsICCHeader));
 
   // now set header fields, which we can access using the lcms2 public API
-  cmsSetHeaderFlags(pf, _cmsAdjustEndianess32(pfHeader.flags));
-  cmsSetHeaderManufacturer(pf, _cmsAdjustEndianess32(pfHeader.manufacturer));
-  cmsSetHeaderModel(pf, _cmsAdjustEndianess32(pfHeader.model));
-  cmsUInt64Number attributes;
-  _cmsAdjustEndianess64(&attributes, &pfHeader.attributes);
-  cmsSetHeaderAttributes(pf, attributes);
+  cmsSetHeaderFlags(pf, pfHeader.flags);
+  cmsSetHeaderManufacturer(pf, pfHeader.manufacturer);
+  cmsSetHeaderModel(pf, pfHeader.model);
+  cmsSetHeaderAttributes(pf, pfHeader.attributes);
   cmsSetHeaderProfileID(pf, (cmsUInt8Number*)&(pfHeader.profileID));
-  cmsSetHeaderRenderingIntent(pf, _cmsAdjustEndianess32(pfHeader.renderingIntent));
-  cmsSetPCS(pf, _cmsAdjustEndianess32(pfHeader.pcs));
-  cmsSetColorSpace(pf, _cmsAdjustEndianess32(pfHeader.colorSpace));
-  cmsSetDeviceClass(pf, _cmsAdjustEndianess32(pfHeader.deviceClass));
-  cmsSetEncodedICCversion(pf, _cmsAdjustEndianess32(pfHeader.version));
+  cmsSetHeaderRenderingIntent(pf, pfHeader.renderingIntent);
+  cmsSetPCS(pf, pfHeader.pcs);
+  cmsSetColorSpace(pf, pfHeader.colorSpace);
+  cmsSetDeviceClass(pf, pfHeader.deviceClass);
+  cmsSetEncodedICCversion(pf, pfHeader.version);
 
   return TRUE;
 }
