@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,6 +23,10 @@
  * questions.
  */
 
+#ifdef HEADLESS
+    #error This file should not be included in headless library
+#endif
+
 #include "jni_util.h"
 #include "awt_p.h"
 #include "awt.h"
@@ -31,14 +35,12 @@
 #include <sun_awt_X11GraphicsEnvironment.h>
 #include <sun_awt_X11GraphicsDevice.h>
 #include <sun_awt_X11GraphicsConfig.h>
-#ifndef HEADLESS
 #include <X11/extensions/Xdbe.h>
 #include <X11/XKBlib.h>
 #ifndef NO_XRANDR
 #include <X11/extensions/Xrandr.h>
 #endif
 #include "GLXGraphicsConfig.h"
-#endif /* !HEADLESS */
 
 #include <jni.h>
 #include <jni_util.h>
@@ -54,8 +56,6 @@
 #include <dlfcn.h>
 #include "Trace.h"
 
-#ifndef HEADLESS
-
 int awt_numScreens;     /* Xinerama-aware number of screens */
 
 AwtScreenDataPtr x11Screens;
@@ -66,12 +66,6 @@ AwtScreenDataPtr x11Screens;
  */
 static jboolean glxRequested = JNI_FALSE;
 
-#endif /* !HEADLESS */
-
-#ifdef HEADLESS
-#define Display void
-#endif /* HEADLESS */
-
 Display *awt_display;
 
 jclass tkClass = NULL;
@@ -92,9 +86,7 @@ jboolean awtLockInited = JNI_FALSE;
 
 struct X11GraphicsConfigIDs x11GraphicsConfigIDs;
 
-#ifndef HEADLESS
 int awtCreateX11Colormap(AwtGraphicsConfigDataPtr adata);
-#endif /* HEADLESS */
 
 static char *x11GraphicsConfigClassName = "sun/awt/X11GraphicsConfig";
 
@@ -134,8 +126,6 @@ Java_sun_awt_X11GraphicsConfig_initIDs (JNIEnv *env, jclass cls)
     CHECK_NULL(x11GraphicsConfigIDs.bitsPerPixel);
 }
 
-#ifndef HEADLESS
-
 /*
  * XIOErrorHandler
  */
@@ -405,23 +395,11 @@ getAllConfigs (JNIEnv *env, int screen, AwtScreenDataPtr screenDataPtr) {
         DTRACE_PRINTLN("RENDER extension available");
         xrenderLibHandle = dlopen("libXrender.so.1", RTLD_LAZY | RTLD_GLOBAL);
 
-#ifdef MACOSX
-#define XRENDER_LIB "/usr/X11/lib/libXrender.dylib"
-#else
-#define XRENDER_LIB "libXrender.so"
-#endif
-
         if (xrenderLibHandle == NULL) {
-            xrenderLibHandle = dlopen(XRENDER_LIB,
-                                      RTLD_LAZY | RTLD_GLOBAL);
+            xrenderLibHandle = dlopen("libXrender.so", RTLD_LAZY | RTLD_GLOBAL);
         }
 
-#if defined(__solaris__)
-        if (xrenderLibHandle == NULL) {
-            xrenderLibHandle = dlopen("/usr/lib/libXrender.so.1",
-                                      RTLD_LAZY | RTLD_GLOBAL);
-        }
-#elif defined(_AIX)
+#if defined(_AIX)
         if (xrenderLibHandle == NULL) {
             xrenderLibHandle = dlopen("libXrender.a(libXrender.so.0)",
                                       RTLD_MEMBER | RTLD_LAZY | RTLD_GLOBAL);
@@ -605,8 +583,6 @@ cleanup:
     AWT_UNLOCK ();
 }
 
-#ifndef HEADLESS
-
 /*
  * Checks if Xinerama is running and perform Xinerama-related initialization.
  */
@@ -668,7 +644,6 @@ static void xineramaInit(void) {
         DTRACE_PRINTLN1("\ncouldn't open shared library: %s\n", dlerror());
     }
 }
-#endif /* HEADLESS */
 
 Display *
 awt_init_Display(JNIEnv *env, jobject this)
@@ -745,7 +720,6 @@ awt_init_Display(JNIEnv *env, jobject this)
 
     return dpy;
 }
-#endif /* !HEADLESS */
 
 /*
  * Class:     sun_awt_X11GraphicsEnvironment
@@ -756,14 +730,9 @@ JNIEXPORT jint JNICALL
 Java_sun_awt_X11GraphicsEnvironment_getDefaultScreenNum(
 JNIEnv *env, jobject this)
 {
-#ifdef HEADLESS
-    return (jint)0;
-#else
     return DefaultScreen(awt_display);
-#endif /* !HEADLESS */
 }
 
-#ifndef HEADLESS
 static void ensureConfigsInited(JNIEnv* env, int screen) {
    if (x11Screens[screen].numConfigs == 0) {
        if (env == NULL) {
@@ -772,25 +741,13 @@ static void ensureConfigsInited(JNIEnv* env, int screen) {
        getAllConfigs (env, screen, &(x11Screens[screen]));
     }
 }
-#endif
 
-#ifdef HEADLESS
-void* getDefaultConfig(int screen) {
-    return NULL;
-}
-#else
 AwtGraphicsConfigDataPtr
 getDefaultConfig(int screen) {
     ensureConfigsInited(NULL, screen);
     return x11Screens[screen].defaultConfig;
 }
 
-AwtScreenDataPtr
-getScreenData(int screen) {
-    return &(x11Screens[screen]);
-}
-#endif /* !HEADLESS */
-
 /*
  * Class:     sun_awt_X11GraphicsEnvironment
  * Method:    initDisplay
@@ -800,10 +757,8 @@ JNIEXPORT void JNICALL
 Java_sun_awt_X11GraphicsEnvironment_initDisplay(JNIEnv *env, jobject this,
                                                 jboolean glxReq)
 {
-#ifndef HEADLESS
     glxRequested = glxReq;
     (void) awt_init_Display(env, this);
-#endif /* !HEADLESS */
 }
 
 /*
@@ -814,7 +769,6 @@ Java_sun_awt_X11GraphicsEnvironment_initDisplay(JNIEnv *env, jobject this,
 JNIEXPORT jboolean JNICALL
 Java_sun_awt_X11GraphicsEnvironment_initGLX(JNIEnv *env, jclass x11ge)
 {
-#ifndef HEADLESS
     jboolean glxAvailable;
 
     AWT_LOCK();
@@ -822,9 +776,6 @@ Java_sun_awt_X11GraphicsEnvironment_initGLX(JNIEnv *env, jclass x11ge)
     AWT_UNLOCK();
 
     return glxAvailable;
-#else
-    return JNI_FALSE;
-#endif /* !HEADLESS */
 }
 
 /*
@@ -835,11 +786,7 @@ Java_sun_awt_X11GraphicsEnvironment_initGLX(JNIEnv *env, jclass x11ge)
 JNIEXPORT jint JNICALL
 Java_sun_awt_X11GraphicsEnvironment_getNumScreens(JNIEnv *env, jobject this)
 {
-#ifdef HEADLESS
-    return (jint)0;
-#else
     return awt_numScreens;
-#endif /* !HEADLESS */
 }
 
 /*
@@ -850,11 +797,7 @@ Java_sun_awt_X11GraphicsEnvironment_getNumScreens(JNIEnv *env, jobject this)
 JNIEXPORT jlong JNICALL
 Java_sun_awt_X11GraphicsDevice_getDisplay(JNIEnv *env, jobject this)
 {
-#ifdef HEADLESS
-    return NULL;
-#else
     return ptr_to_jlong(awt_display);
-#endif /* !HEADLESS */
 }
 
 #ifdef MITSHM
@@ -981,11 +924,7 @@ JNIEXPORT jstring JNICALL
 Java_sun_awt_X11GraphicsEnvironment_getDisplayString
   (JNIEnv *env, jobject this)
 {
-#ifdef HEADLESS
-    return (jstring)NULL;
-#else
     return (*env)->NewStringUTF(env, DisplayString(awt_display));
-#endif /* HEADLESS */
 }
 
 
@@ -998,12 +937,8 @@ JNIEXPORT jint JNICALL
 Java_sun_awt_X11GraphicsDevice_getNumConfigs(
 JNIEnv *env, jobject this, jint screen)
 {
-#ifdef HEADLESS
-    return (jint)0;
-#else
     ensureConfigsInited(env, screen);
     return x11Screens[screen].numConfigs;
-#endif /* !HEADLESS */
 }
 
 /*
@@ -1015,9 +950,6 @@ JNIEXPORT jint JNICALL
 Java_sun_awt_X11GraphicsDevice_getConfigVisualId(
 JNIEnv *env, jobject this, jint index, jint screen)
 {
-#ifdef HEADLESS
-    return (jint)0;
-#else
     int visNum;
 
     ensureConfigsInited(env, screen);
@@ -1026,7 +958,6 @@ JNIEnv *env, jobject this, jint index, jint screen)
     } else {
         return ((jint)x11Screens[screen].configs[index]->awt_visInfo.visualid);
     }
-#endif /* !HEADLESS */
 }
 
 /*
@@ -1038,9 +969,6 @@ JNIEXPORT jint JNICALL
 Java_sun_awt_X11GraphicsDevice_getConfigDepth(
 JNIEnv *env, jobject this, jint index, jint screen)
 {
-#ifdef HEADLESS
-    return (jint)0;
-#else
     int visNum;
 
     ensureConfigsInited(env, screen);
@@ -1049,7 +977,6 @@ JNIEnv *env, jobject this, jint index, jint screen)
     } else {
         return ((jint)x11Screens[screen].configs[index]->awt_visInfo.depth);
     }
-#endif /* !HEADLESS */
 }
 
 /*
@@ -1061,9 +988,6 @@ JNIEXPORT jint JNICALL
 Java_sun_awt_X11GraphicsDevice_getConfigColormap(
 JNIEnv *env, jobject this, jint index, jint screen)
 {
-#ifdef HEADLESS
-    return (jint)0;
-#else
     int visNum;
 
     ensureConfigsInited(env, screen);
@@ -1072,7 +996,6 @@ JNIEnv *env, jobject this, jint index, jint screen)
     } else {
         return ((jint)x11Screens[screen].configs[index]->awt_cmap);
     }
-#endif /* !HEADLESS */
 }
 
 /*
@@ -1084,7 +1007,6 @@ JNIEXPORT void JNICALL
 Java_sun_awt_X11GraphicsDevice_resetNativeData
     (JNIEnv *env, jclass x11gd, jint screen)
 {
-#ifndef HEADLESS
     /*
      * Reset references to the various configs; the actual native config data
      * will be free'd later by the Disposer mechanism when the Java-level
@@ -1098,7 +1020,6 @@ Java_sun_awt_X11GraphicsDevice_resetNativeData
     }
     x11Screens[screen].defaultConfig = NULL;
     x11Screens[screen].numConfigs = 0;
-#endif /* !HEADLESS */
 }
 
 /*
@@ -1110,7 +1031,6 @@ JNIEXPORT void JNICALL
 Java_sun_awt_X11GraphicsConfig_dispose
     (JNIEnv *env, jclass x11gc, jlong configData)
 {
-#ifndef HEADLESS
     AwtGraphicsConfigDataPtr aData = (AwtGraphicsConfigDataPtr)
         jlong_to_ptr(configData);
 
@@ -1152,7 +1072,6 @@ Java_sun_awt_X11GraphicsConfig_dispose
     }
 
     free(aData);
-#endif /* !HEADLESS */
 }
 
 /*
@@ -1164,12 +1083,8 @@ JNIEXPORT jdouble JNICALL
 Java_sun_awt_X11GraphicsConfig_getXResolution(
 JNIEnv *env, jobject this, jint screen)
 {
-#ifdef HEADLESS
-    return (jdouble)0;
-#else
     return ((DisplayWidth(awt_display, screen) * 25.4) /
             DisplayWidthMM(awt_display, screen));
-#endif /* !HEADLESS */
 }
 
 /*
@@ -1181,12 +1096,8 @@ JNIEXPORT jdouble JNICALL
 Java_sun_awt_X11GraphicsConfig_getYResolution(
 JNIEnv *env, jobject this, jint screen)
 {
-#ifdef HEADLESS
-    return (jdouble)0;
-#else
     return ((DisplayHeight(awt_display, screen) * 25.4) /
             DisplayHeightMM(awt_display, screen));
-#endif /* !HEADLESS */
 }
 
 
@@ -1199,16 +1110,12 @@ JNIEXPORT jint JNICALL
 Java_sun_awt_X11GraphicsConfig_getNumColors(
 JNIEnv *env, jobject this)
 {
-#ifdef HEADLESS
-    return (jint)0;
-#else
     AwtGraphicsConfigData *adata;
 
     adata = (AwtGraphicsConfigData *) JNU_GetLongFieldAsPtr(env, this,
                                               x11GraphicsConfigIDs.aData);
 
     return adata->awt_num_colors;
-#endif /* !HEADLESS */
 }
 
 /*
@@ -1220,7 +1127,6 @@ JNIEXPORT void JNICALL
 Java_sun_awt_X11GraphicsConfig_init(
 JNIEnv *env, jobject this, jint visualNum, jint screen)
 {
-#ifndef HEADLESS
     AwtGraphicsConfigData *adata = NULL;
     AwtScreenData asd = x11Screens[screen];
     int i, n;
@@ -1261,11 +1167,8 @@ JNIEnv *env, jobject this, jint visualNum, jint screen)
     (*env)->SetIntField(env, this, x11GraphicsConfigIDs.bitsPerPixel,
                         (jint)tempImage->bits_per_pixel);
     XDestroyImage(tempImage);
-#endif /* !HEADLESS */
 }
 
-
-
 /*
  * Class:     sun_awt_X11GraphicsConfig
  * Method:    makeColorModel
@@ -1275,9 +1178,6 @@ JNIEXPORT jobject JNICALL
 Java_sun_awt_X11GraphicsConfig_makeColorModel(
 JNIEnv *env, jobject this)
 {
-#ifdef HEADLESS
-    return NULL;
-#else
     AwtGraphicsConfigData *adata;
     jobject colorModel;
 
@@ -1306,7 +1206,6 @@ JNIEnv *env, jobject this)
     AWT_UNLOCK ();
 
     return colorModel;
-#endif /* !HEADLESS */
 }
 
 
@@ -1318,9 +1217,6 @@ JNIEnv *env, jobject this)
 JNIEXPORT jobject JNICALL
 Java_sun_awt_X11GraphicsConfig_pGetBounds(JNIEnv *env, jobject this, jint screen)
 {
-#ifdef HEADLESS
-    return NULL;
-#else
     jclass clazz;
     jmethodID mid;
     jobject bounds = NULL;
@@ -1379,7 +1275,6 @@ Java_sun_awt_X11GraphicsConfig_pGetBounds(JNIEnv *env, jobject this, jint screen
         }
     }
     return bounds;
-#endif /* !HEADLESS */
 }
 
 /*
@@ -1455,15 +1350,11 @@ JNIEXPORT jboolean JNICALL
 Java_sun_awt_X11GraphicsConfig_isTranslucencyCapable
     (JNIEnv *env, jobject this, jlong configData)
 {
-#ifdef HEADLESS
-    return JNI_FALSE;
-#else
     AwtGraphicsConfigDataPtr aData = (AwtGraphicsConfigDataPtr)jlong_to_ptr(configData);
     if (aData == NULL) {
         return JNI_FALSE;
     }
     return aData->isTranslucencySupported ? JNI_TRUE : JNI_FALSE;
-#endif
 }
 
 /*
@@ -1474,9 +1365,6 @@ Java_sun_awt_X11GraphicsConfig_isTranslucencyCapable
 JNIEXPORT jboolean JNICALL
 Java_sun_awt_X11GraphicsDevice_isDBESupported(JNIEnv *env, jobject this)
 {
-#ifdef HEADLESS
-    return JNI_FALSE;
-#else
     int opcode = 0, firstEvent = 0, firstError = 0;
     jboolean ret;
 
@@ -1485,7 +1373,6 @@ Java_sun_awt_X11GraphicsDevice_isDBESupported(JNIEnv *env, jobject this)
                                     &opcode, &firstEvent, &firstError);
     AWT_FLUSH_UNLOCK();
     return ret;
-#endif /* !HEADLESS */
 }
 
 /*
@@ -1497,7 +1384,6 @@ JNIEXPORT void JNICALL
 Java_sun_awt_X11GraphicsDevice_getDoubleBufferVisuals(JNIEnv *env,
     jobject this, jint screen)
 {
-#ifndef HEADLESS
     jclass clazz;
     jmethodID midAddVisual;
     Window rootWindow;
@@ -1532,7 +1418,6 @@ Java_sun_awt_X11GraphicsDevice_getDoubleBufferVisuals(JNIEnv *env,
             break;
         }
     }
-#endif /* !HEADLESS */
 }
 
 /*
@@ -1544,19 +1429,13 @@ JNIEXPORT jboolean JNICALL
 Java_sun_awt_X11GraphicsEnvironment_pRunningXinerama(JNIEnv *env,
     jobject this)
 {
-#ifdef HEADLESS
-    return JNI_FALSE;
-#else
     return usingXinerama ? JNI_TRUE : JNI_FALSE;
-#endif /* HEADLESS */
 }
 
 /**
  * Begin DisplayMode/FullScreen support
  */
 
-#ifndef HEADLESS
-
 #ifndef NO_XRANDR
 
 #define BIT_DEPTH_MULTI java_awt_DisplayMode_BIT_DEPTH_MULTI
@@ -1794,7 +1673,6 @@ X11GD_SetFullscreenMode(Window win, jboolean enabled)
                &event);
     XSync(awt_display, False);
 }
-#endif /* !HEADLESS */
 
 /*
  * Class:     sun_awt_X11GraphicsDevice
@@ -1805,7 +1683,7 @@ JNIEXPORT jboolean JNICALL
 Java_sun_awt_X11GraphicsDevice_initXrandrExtension
     (JNIEnv *env, jclass x11gd)
 {
-#if defined(HEADLESS) || defined(NO_XRANDR)
+#if defined(NO_XRANDR)
     return JNI_FALSE;
 #else
     int opcode = 0, firstEvent = 0, firstError = 0;
@@ -1820,7 +1698,7 @@ Java_sun_awt_X11GraphicsDevice_initXrandrExtension
     AWT_FLUSH_UNLOCK();
 
     return ret;
-#endif /* HEADLESS */
+#endif /* NO_XRANDR */
 }
 
 /*
@@ -1832,7 +1710,7 @@ JNIEXPORT jobject JNICALL
 Java_sun_awt_X11GraphicsDevice_getCurrentDisplayMode
     (JNIEnv* env, jclass x11gd, jint screen)
 {
-#if defined(HEADLESS) || defined(NO_XRANDR)
+#if defined(NO_XRANDR)
     return NULL;
 #else
     XRRScreenConfiguration *config;
@@ -1915,7 +1793,7 @@ Java_sun_awt_X11GraphicsDevice_getCurrentDisplayMode
     AWT_FLUSH_UNLOCK();
 
     return displayMode;
-#endif /* HEADLESS */
+#endif /* NO_XRANDR */
 }
 
 /*
@@ -1928,7 +1806,7 @@ Java_sun_awt_X11GraphicsDevice_enumDisplayModes
     (JNIEnv* env, jclass x11gd,
      jint screen, jobject arrayList)
 {
-#if !defined(HEADLESS) && !defined(NO_XRANDR)
+#if !defined(NO_XRANDR)
 
     AWT_LOCK();
 
@@ -2003,7 +1881,7 @@ ret1:
     }
 
     AWT_FLUSH_UNLOCK();
-#endif /* !HEADLESS */
+#endif /* !NO_XRANDR */
 }
 
 /*
@@ -2016,7 +1894,7 @@ Java_sun_awt_X11GraphicsDevice_configDisplayMode
     (JNIEnv* env, jclass x11gd,
      jint screen, jint width, jint height, jint refreshRate)
 {
-#if !defined(HEADLESS) && !defined(NO_XRANDR)
+#if !defined(NO_XRANDR)
     jboolean success = JNI_FALSE;
     XRRScreenConfiguration *config;
     Drawable root;
@@ -2086,7 +1964,7 @@ Java_sun_awt_X11GraphicsDevice_configDisplayMode
     if (!success && !(*env)->ExceptionCheck(env)) {
         JNU_ThrowInternalError(env, "Could not set display mode");
     }
-#endif /* !HEADLESS */
+#endif /* !NO_XRANDR */
 }
 
 /*
@@ -2099,14 +1977,12 @@ Java_sun_awt_X11GraphicsDevice_enterFullScreenExclusive
     (JNIEnv* env, jclass x11gd,
      jlong window)
 {
-#ifndef HEADLESS
     Window win = (Window)window;
 
     AWT_LOCK();
     XSync(awt_display, False); /* ensures window is visible first */
     X11GD_SetFullscreenMode(win, JNI_TRUE);
     AWT_UNLOCK();
-#endif /* !HEADLESS */
 }
 
 /*
@@ -2119,13 +1995,11 @@ Java_sun_awt_X11GraphicsDevice_exitFullScreenExclusive
     (JNIEnv* env, jclass x11gd,
      jlong window)
 {
-#ifndef HEADLESS
     Window win = (Window)window;
 
     AWT_LOCK();
     X11GD_SetFullscreenMode(win, JNI_FALSE);
     AWT_UNLOCK();
-#endif /* !HEADLESS */
 }
 
 /**
