@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -59,6 +59,7 @@ import sun.rmi.registry.RegistryImpl;
  * @since       1.2
  * @see         java.rmi.activation.ActivationGroup
  */
+@SuppressWarnings("removal")
 public class ActivationGroupImpl extends ActivationGroup {
 
     // use serialVersionUID from JDK 1.2.2 for interoperability
@@ -69,6 +70,7 @@ public class ActivationGroupImpl extends ActivationGroup {
         new Hashtable<>();
     private boolean groupInactive = false;
     private final ActivationGroupID groupID;
+    @SuppressWarnings("serial")  // Conditionally serializable
     private final List<ActivationID> lockedIDs = new ArrayList<>();
 
     /**
