@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
  */
 /*
  * Licensed to the Apache Software Foundation (ASF) under one or more
@@ -30,7 +30,7 @@ import org.xml.sax.SAXException;
 
 import com.sun.org.apache.xml.internal.serializer.utils.MsgKey;
 import com.sun.org.apache.xml.internal.serializer.utils.Utils;
-import jdk.xml.internal.JdkXmlUtils;
+import javax.xml.transform.ErrorListener;
 
 /**
  * This serializer takes a series of SAX or
@@ -41,7 +41,7 @@ import jdk.xml.internal.JdkXmlUtils;
  * because it is used from another package.
  *
  * @xsl.usage internal
- * @LastModified: July 2021
+ * @LastModified: Aug 2019
  */
 public final class ToHTMLStream extends ToStream
 {
@@ -639,12 +639,15 @@ public final class ToHTMLStream extends ToStream
      */
     public ToHTMLStream()
     {
+        this(null);
+    }
 
-        super();
+    public ToHTMLStream(ErrorListener l)
+    {
+        super(l);
         m_charInfo = m_htmlcharInfo;
         // initialize namespaces
         m_prefixMap = new NamespaceMappings();
-
     }
 
     /** The name of the current element. */
@@ -676,10 +679,28 @@ public final class ToHTMLStream extends ToStream
                 final java.io.Writer writer = m_writer;
                 try
                 {
-                    writer.write("<!DOCTYPE html");
-                    writer.write(JdkXmlUtils.getDTDExternalDecl(doctypePublic, doctypeSystem));
-                    writer.write('>');
-                    outputLineSep();
+                writer.write("<!DOCTYPE html");
+
+                if (null != doctypePublic)
+                {
+                    writer.write(" PUBLIC \"");
+                    writer.write(doctypePublic);
+                    writer.write('"');
+                }
+
+                if (null != doctypeSystem)
+                {
+                    if (null == doctypePublic)
+                        writer.write(" SYSTEM \"");
+                    else
+                        writer.write(" \"");
+
+                    writer.write(doctypeSystem);
+                    writer.write('"');
+                }
+
+                writer.write('>');
+                outputLineSep();
                 }
                 catch(IOException e)
                 {
@@ -702,7 +723,7 @@ public final class ToHTMLStream extends ToStream
     public final void endDocument() throws org.xml.sax.SAXException
     {
         if (m_doIndent) {
-            flushCharactersBuffer();
+            flushCharactersBuffer(false);
         }
         flushPending();
         if (m_doIndent && !m_isprevtext)
@@ -765,7 +786,7 @@ public final class ToHTMLStream extends ToStream
         if (m_doIndent) {
             // will add extra one if having namespace but no matter
             m_childNodeNum++;
-            flushCharactersBuffer();
+            flushCharactersBuffer(false);
         }
         ElemContext elemContext = m_elemContext;
 
@@ -906,7 +927,7 @@ public final class ToHTMLStream extends ToStream
         throws org.xml.sax.SAXException
     {
         if (m_doIndent) {
-            flushCharactersBuffer();
+            flushCharactersBuffer(false);
         }
         // deal with any pending issues
         if (m_cdataTagOpen)
@@ -1420,23 +1441,32 @@ public final class ToHTMLStream extends ToStream
                             }
                         }
                     }
+
+                    // The next is kind of a hack to keep from escaping in the case
+                    // of Shift_JIS and the like.
+
+                    /*
+                    else if ((ch < m_maxCharacter) && (m_maxCharacter == 0xFFFF)
+                    && (ch != 160))
+                    {
+                    writer.write(ch);  // no escaping in this case
+                    }
+                    else
+                    */
+                    String outputStringForChar = m_charInfo.getOutputStringForChar(ch);
+                    if (null != outputStringForChar)
+                    {
+                        writer.write(outputStringForChar);
+                    }
+                    else if (escapingNotNeeded(ch))
+                    {
+                        writer.write(ch); // no escaping in this case
+                    }
                     else
                     {
-                        String outputStringForChar = m_charInfo.getOutputStringForChar(ch);
-                        if (null != outputStringForChar)
-                        {
-                            writer.write(outputStringForChar);
-                        }
-                        else if (escapingNotNeeded(ch))
-                        {
-                            writer.write(ch); // no escaping in this case
-                        }
-                        else
-                        {
-                            writer.write("&#");
-                            writer.write(Integer.toString(ch));
-                            writer.write(';');
-                        }
+                        writer.write("&#");
+                        writer.write(Integer.toString(ch));
+                        writer.write(';');
                     }
                 }
                 cleanStart = i + 1;
@@ -1619,7 +1649,7 @@ public final class ToHTMLStream extends ToStream
     {
         if (m_doIndent) {
             m_childNodeNum++;
-            flushCharactersBuffer();
+            flushCharactersBuffer(false);
         }
         // Process any pending starDocument and startElement first.
         flushPending();
