@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -45,6 +45,11 @@ import sun.awt.AWTPermissions;
 
 public class AWTEventMonitor {
 
+    /**
+     * Constructs an {@code AWTEventMonitor}.
+     */
+    public AWTEventMonitor() {}
+
     /**
      * The current component with keyboard focus.
      *
