@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2017, 2019, Red Hat, Inc. All rights reserved.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 only, as
@@ -32,16 +33,17 @@ import sun.jvm.hotspot.oops.Oop;
 import sun.jvm.hotspot.oops.UnknownOopException;
 import sun.jvm.hotspot.types.*;
 import sun.jvm.hotspot.runtime.VM;
+import sun.jvm.hotspot.runtime.VMObject;
 import sun.jvm.hotspot.debugger.Address;
 import sun.jvm.hotspot.utilities.AddressOps;
 
 import java.util.ArrayList;
 import java.util.List;
-import java.util.Observable;
-import java.util.Observer;
+import sun.jvm.hotspot.utilities.Observable;
+import sun.jvm.hotspot.utilities.Observer;
 
 
-public class ShenandoahHeapRegion extends ContiguousSpace implements LiveRegionsProvider {
+public class ShenandoahHeapRegion extends VMObject implements LiveRegionsProvider {
     private static int EmptyUncommitted;
     private static int EmptyCommitted;
     private static int Regular;
@@ -55,9 +57,13 @@ public class ShenandoahHeapRegion extends ContiguousSpace implements LiveRegions
 
     private static CIntegerField RegionSizeBytesField;
     private static Field         RegionStateField;
-    private static CIntegerField RegionNumberField;
+    private static CIntegerField RegionIndexField;
     private static CIntegerField RegionSizeBytesShiftField;
 
+    private static AddressField BottomField;
+    private static AddressField TopField;
+    private static AddressField EndField;
+
     private ShenandoahHeap heap;
 
     static {
@@ -72,7 +78,10 @@ public class ShenandoahHeapRegion extends ContiguousSpace implements LiveRegions
         Type type = db.lookupType("ShenandoahHeapRegion");
         RegionSizeBytesField = type.getCIntegerField("RegionSizeBytes");
         RegionStateField = type.getField("_state");
-        RegionNumberField = type.getCIntegerField("_region_number");
+        RegionIndexField = type.getCIntegerField("_index");
+        BottomField = type.getAddressField("_bottom");
+        TopField = type.getAddressField("_top");
+        EndField = type.getAddressField("_end");
 
         RegionSizeBytesShiftField = type.getCIntegerField("RegionSizeBytesShift");
 
@@ -104,16 +113,28 @@ public class ShenandoahHeapRegion extends ContiguousSpace implements LiveRegions
         this.heap = heap;
     }
 
+    public Address bottom() {
+        return BottomField.getValue(addr);
+    }
+
+    public Address top() {
+        return TopField.getValue(addr);
+    }
+
+    public Address end() {
+        return EndField.getValue(addr);
+    }
+
     @Override
     public int hashCode() {
-        return (int)regionNumber();
+        return (int)index();
     }
 
     @Override
     public boolean equals(Object other) {
         if (other instanceof ShenandoahHeapRegion) {
             ShenandoahHeapRegion otherRegion = (ShenandoahHeapRegion)other;
-            return otherRegion.regionNumber() == regionNumber();
+            return otherRegion.index() == index();
         }
         return false;
     }
@@ -149,7 +170,7 @@ public class ShenandoahHeapRegion extends ContiguousSpace implements LiveRegions
     }
 
     private void handleHumongousRegion(List<MemRegion> res) {
-        long index = regionNumber();
+        long index = index();
         Address topAddr = top();
         ShenandoahHeapRegion region = heap.getRegion(++ index);
         while (region.regionState() == HumongousCont) {
@@ -184,10 +205,10 @@ public class ShenandoahHeapRegion extends ContiguousSpace implements LiveRegions
                 if (regionStart == null) {
                     regionStart = start;
                 } else {
-                    regionEnd = start.addOffsetTo(heap.oopOffset() + size);
+                    regionEnd = start.addOffsetTo(size);
                 }
             }
-            start = start.addOffsetTo(heap.oopOffset() + size);
+            start = start.addOffsetTo(size);
         }
 
         if (regionStart != null) {
@@ -196,20 +217,18 @@ public class ShenandoahHeapRegion extends ContiguousSpace implements LiveRegions
         }
     }
 
-    public long regionNumber() {
-        return RegionNumberField.getValue(addr);
+    public long index() {
+        return RegionIndexField.getValue(addr);
     }
 
     private boolean hasForwardee(Address rawPtr) {
-        // Use Mark as a helper to read forward pointer value.
+        // Forwarding pointer is stored in mark word when it is flagged "marked"
         Mark mark = new Mark(rawPtr);
-        Address forwardee = mark.valueAsAddress();
-        return (forwardee != rawPtr.addOffsetTo(heap.oopOffset()));
+        return mark.isMarked();
     }
 
     private long getObjectSize(Address rawPtr) {
-        // Dealing with a raw pointer, offsets forward pointer to find real Oop.
-        OopHandle handle = rawPtr.addOffsetToAsOopHandle(heap.oopOffset());
+        OopHandle handle = rawPtr.addOffsetToAsOopHandle(0);
         Oop obj = null;
 
         try {
