@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -56,7 +56,7 @@ final class CompilerToVM {
     private static native void registerNatives();
 
     /**
-     * These values mirror the equivalent values from {@link Unsafe} but are approriate for the JVM
+     * These values mirror the equivalent values from {@code Unsafe} but are appropriate for the JVM
      * being compiled against.
      */
     // Checkstyle: stop
@@ -514,10 +514,10 @@ final class CompilerToVM {
 
     /**
      * Reads an object pointer within a VM data structure. That is, any {@link VMField} whose
-     * {@link VMField#type type} is {@code "oop"} (e.g.,
-     * {@code Klass::_java_mirror}, {@code JavaThread::_threadObj}).
+     * {@link VMField#type type} is {@code "oop"} (e.g., {@code Klass::_java_mirror},
+     * {@code JavaThread::_threadObj}).
      *
-     * Note that {@link Unsafe#getObject(Object, long)} cannot be used for this since it does a
+     * Note that {@code Unsafe.getObject(Object, long)} cannot be used for this since it does a
      * {@code narrowOop} read if the VM is using compressed oops whereas oops within VM data
      * structures are (currently) always uncompressed.
      *
@@ -557,10 +557,10 @@ final class CompilerToVM {
     native int getCountersSize();
 
     /**
-     * Change the size of the counters allocated for JVMCI. This requires a safepoint to
+     * Attempt to change the size of the counters allocated for JVMCI. This requires a safepoint to
      * safely reallocate the storage but it's advisable to increase the size in reasonable chunks.
      */
-    native void setCountersSize(int newSize);
+    native boolean setCountersSize(int newSize);
 
     /**
      * Determines if {@code metaspaceMethodData} is mature.
@@ -772,6 +772,11 @@ final class CompilerToVM {
      */
     native void ensureInitialized(HotSpotResolvedObjectTypeImpl type);
 
+    /**
+     * Forces linking of {@code type}.
+     */
+    native void ensureLinked(HotSpotResolvedObjectTypeImpl type);
+
     /**
      * Checks if {@code object} is a String and is an interned string value.
      */
@@ -967,9 +972,15 @@ final class CompilerToVM {
     native boolean isCurrentThreadAttached();
 
     /**
+     * @see HotSpotJVMCIRuntime#getCurrentJavaThread()
+     */
+    native long getCurrentJavaThread();
+
+    /**
+     * @param name name of current thread if in a native image otherwise {@code null}
      * @see HotSpotJVMCIRuntime#attachCurrentThread
      */
-    native boolean attachCurrentThread(boolean asDaemon);
+    native boolean attachCurrentThread(byte[] name, boolean asDaemon);
 
     /**
      * @see HotSpotJVMCIRuntime#detachCurrentThread()
@@ -980,4 +991,27 @@ final class CompilerToVM {
      * @see HotSpotJVMCIRuntime#exitHotSpot(int)
      */
     native void callSystemExit(int status);
+
+    /**
+     * @see JFR.Ticks#now
+     */
+    native long ticksNow();
+
+    /**
+     * Adds phases in HotSpot JFR.
+     *
+     * @see JFR.CompilerPhaseEvent#write
+     */
+    native int registerCompilerPhase(String phaseName);
+
+    /**
+     * @see JFR.CompilerPhaseEvent#write
+     */
+    native void notifyCompilerPhaseEvent(long startTime, int phase, int compileId, int level);
+
+    /**
+     * @see JFR.CompilerInliningEvent#write
+     */
+    native void notifyCompilerInliningEvent(int compileId, HotSpotResolvedJavaMethodImpl caller, HotSpotResolvedJavaMethodImpl callee, boolean succeeded, String message, int bci);
+
 }
