@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,11 +32,11 @@ import static jdk.vm.ci.amd64.AMD64.rbp;
 import static jdk.vm.ci.amd64.AMD64.rsp;
 import static org.graalvm.compiler.asm.amd64.AMD64BaseAssembler.EVEXPrefixConfig.B0;
 import static org.graalvm.compiler.asm.amd64.AMD64BaseAssembler.EVEXPrefixConfig.B1;
-import static org.graalvm.compiler.asm.amd64.AMD64BaseAssembler.EVEXPrefixConfig.L512;
 import static org.graalvm.compiler.asm.amd64.AMD64BaseAssembler.EVEXPrefixConfig.Z0;
 import static org.graalvm.compiler.asm.amd64.AMD64BaseAssembler.EVEXPrefixConfig.Z1;
 import static org.graalvm.compiler.asm.amd64.AMD64BaseAssembler.VEXPrefixConfig.L128;
 import static org.graalvm.compiler.asm.amd64.AMD64BaseAssembler.VEXPrefixConfig.L256;
+import static org.graalvm.compiler.asm.amd64.AMD64BaseAssembler.VEXPrefixConfig.L512;
 import static org.graalvm.compiler.asm.amd64.AMD64BaseAssembler.VEXPrefixConfig.LZ;
 import static org.graalvm.compiler.asm.amd64.AMD64BaseAssembler.VEXPrefixConfig.M_0F;
 import static org.graalvm.compiler.asm.amd64.AMD64BaseAssembler.VEXPrefixConfig.M_0F38;
@@ -291,11 +291,16 @@ public abstract class AMD64BaseAssembler extends Assembler {
         emitByte(Prefix.REXW);
     }
 
+    private static boolean isInvalidEncoding(Register reg) {
+        return Register.None.equals(reg) || AMD64.rip.equals(reg);
+    }
+
     protected final void prefix(Register reg) {
         prefix(reg, false);
     }
 
     protected final void prefix(Register reg, boolean byteinst) {
+        assert !isInvalidEncoding(reg);
         int regEnc = reg.encoding;
         if (regEnc >= 8) {
             emitByte(Prefix.REXB);
@@ -305,6 +310,7 @@ public abstract class AMD64BaseAssembler extends Assembler {
     }
 
     protected final void prefixq(Register reg) {
+        assert !isInvalidEncoding(reg);
         if (reg.encoding < 8) {
             emitByte(Prefix.REXW);
         } else {
@@ -317,6 +323,7 @@ public abstract class AMD64BaseAssembler extends Assembler {
     }
 
     protected final void prefix(Register dst, boolean dstIsByte, Register src, boolean srcIsByte) {
+        assert !isInvalidEncoding(dst) && !isInvalidEncoding(src);
         int dstEnc = dst.encoding;
         int srcEnc = src.encoding;
         if (dstEnc < 8) {
@@ -339,6 +346,7 @@ public abstract class AMD64BaseAssembler extends Assembler {
      * in the prefix.
      */
     protected final void prefixq(Register reg, Register rm) {
+        assert !isInvalidEncoding(reg) && !isInvalidEncoding(rm);
         int regEnc = reg.encoding;
         int rmEnc = rm.encoding;
         if (regEnc < 8) {
@@ -356,10 +364,15 @@ public abstract class AMD64BaseAssembler extends Assembler {
         }
     }
 
-    private static boolean needsRex(Register reg) {
+    protected static boolean needsRex(Register reg) {
+        // rip is excluded implicitly.
         return reg.encoding >= MinEncodingNeedsRex;
     }
 
+    protected static boolean needsRex(Register src, boolean srcIsByte) {
+        return srcIsByte ? src.encoding >= 4 : needsRex(src);
+    }
+
     protected final void prefix(AMD64Address adr) {
         if (needsRex(adr.getBase())) {
             if (needsRex(adr.getIndex())) {
@@ -399,6 +412,7 @@ public abstract class AMD64BaseAssembler extends Assembler {
     }
 
     protected void prefix(AMD64Address adr, Register reg, boolean byteinst) {
+        assert !isInvalidEncoding(reg);
         if (reg.encoding < 8) {
             if (needsRex(adr.getBase())) {
                 if (needsRex(adr.getIndex())) {
@@ -431,6 +445,7 @@ public abstract class AMD64BaseAssembler extends Assembler {
     }
 
     protected void prefixq(AMD64Address adr, Register src) {
+        assert !isInvalidEncoding(src);
         if (src.encoding < 8) {
             if (needsRex(adr.getBase())) {
                 if (needsRex(adr.getIndex())) {
@@ -468,6 +483,7 @@ public abstract class AMD64BaseAssembler extends Assembler {
      * field. The X bit must be 0.
      */
     protected static int getRXB(Register reg, Register rm) {
+        assert !isInvalidEncoding(rm) && !isInvalidEncoding(reg);
         int rxb = (reg == null ? 0 : reg.encoding & 0x08) >> 1;
         rxb |= (rm == null ? 0 : rm.encoding & 0x08) >> 3;
         return rxb;
@@ -481,11 +497,12 @@ public abstract class AMD64BaseAssembler extends Assembler {
      * There is an SIB byte: In that case, X extends SIB.index and B extends SIB.base.
      */
     protected static int getRXB(Register reg, AMD64Address rm) {
+        assert !isInvalidEncoding(reg);
         int rxb = (reg == null ? 0 : reg.encoding & 0x08) >> 1;
-        if (!rm.getIndex().equals(Register.None)) {
+        if (!isInvalidEncoding(rm.getIndex())) {
             rxb |= (rm.getIndex().encoding & 0x08) >> 2;
         }
-        if (!rm.getBase().equals(Register.None)) {
+        if (!isInvalidEncoding(rm.getBase())) {
             rxb |= (rm.getBase().encoding & 0x08) >> 3;
         }
         return rxb;
@@ -498,6 +515,7 @@ public abstract class AMD64BaseAssembler extends Assembler {
      */
     protected final void emitModRM(int reg, Register rm) {
         assert (reg & 0x07) == reg;
+        assert !isInvalidEncoding(rm);
         emitByte(0xC0 | (reg << 3) | (rm.encoding & 0x07));
     }
 
@@ -507,30 +525,33 @@ public abstract class AMD64BaseAssembler extends Assembler {
      * Format: [ 11 reg r/m ]
      */
     protected final void emitModRM(Register reg, Register rm) {
+        assert !isInvalidEncoding(reg);
         emitModRM(reg.encoding & 0x07, rm);
     }
 
+    public static final int DEFAULT_DISP8_SCALE = 1;
+
     /**
      * Emits the ModR/M byte and optionally the SIB byte for one register and one memory operand.
      *
      * @param force4Byte use 4 byte encoding for displacements that would normally fit in a byte
      */
     protected final void emitOperandHelper(Register reg, AMD64Address addr, boolean force4Byte, int additionalInstructionSize) {
-        assert !reg.equals(Register.None);
-        emitOperandHelper(encode(reg), addr, force4Byte, additionalInstructionSize, 1);
+        assert !isInvalidEncoding(reg);
+        emitOperandHelper(encode(reg), addr, force4Byte, additionalInstructionSize, DEFAULT_DISP8_SCALE);
     }
 
     protected final void emitOperandHelper(int reg, AMD64Address addr, int additionalInstructionSize) {
-        emitOperandHelper(reg, addr, false, additionalInstructionSize, 1);
+        emitOperandHelper(reg, addr, false, additionalInstructionSize, DEFAULT_DISP8_SCALE);
     }
 
     protected final void emitOperandHelper(Register reg, AMD64Address addr, int additionalInstructionSize) {
-        assert !reg.equals(Register.None);
-        emitOperandHelper(encode(reg), addr, false, additionalInstructionSize, 1);
+        assert !isInvalidEncoding(reg);
+        emitOperandHelper(encode(reg), addr, false, additionalInstructionSize, DEFAULT_DISP8_SCALE);
     }
 
-    protected final void emitEVEXOperandHelper(Register reg, AMD64Address addr, int additionalInstructionSize, int evexDisp8Scale) {
-        assert !reg.equals(Register.None);
+    protected final void emitOperandHelper(Register reg, AMD64Address addr, int additionalInstructionSize, int evexDisp8Scale) {
+        assert !isInvalidEncoding(reg);
         emitOperandHelper(encode(reg), addr, false, additionalInstructionSize, evexDisp8Scale);
     }
 
@@ -557,7 +578,7 @@ public abstract class AMD64BaseAssembler extends Assembler {
         int disp = addr.getDisplacement();
 
         if (base.equals(AMD64.rip)) { // also matches addresses returned by getPlaceholder()
-            // [00 000 101] disp32
+            // [00 reg 101] disp32
             assert index.equals(Register.None) : "cannot use RIP relative addressing with index register";
             emitByte(0x05 | regenc);
             if (codePatchingAnnotationConsumer != null && addr.instructionStartPosition >= 0) {
@@ -739,6 +760,7 @@ public abstract class AMD64BaseAssembler extends Assembler {
     public static final class VEXPrefixConfig {
         public static final int L128 = 0;
         public static final int L256 = 1;
+        public static final int L512 = 2;
         public static final int LZ = 0;
 
         public static final int W0 = 0;
@@ -936,18 +958,29 @@ public abstract class AMD64BaseAssembler extends Assembler {
         }
     }
 
-    public final void vexPrefix(Register dst, Register nds, Register src, AVXSize size, int pp, int mmmmm, int w, boolean checkAVX) {
-        emitVEX(getLFlag(size), pp, mmmmm, w, getRXB(dst, src), nds.isValid() ? nds.encoding() : 0, checkAVX);
+    public static boolean isAVX512Register(Register reg) {
+        return reg != null && reg.isValid() && AMD64.XMM.equals(reg.getRegisterCategory()) && reg.encoding > 15;
     }
 
-    public final void vexPrefix(Register dst, Register nds, AMD64Address src, AVXSize size, int pp, int mmmmm, int w, boolean checkAVX) {
+    public final boolean vexPrefix(Register dst, Register nds, Register src, AVXSize size, int pp, int mmmmm, int w, int wEvex, boolean checkAVX) {
+        if (isAVX512Register(dst) || isAVX512Register(nds) || isAVX512Register(src) || size == AVXSize.ZMM) {
+            evexPrefix(dst, Register.None, nds, src, size, pp, mmmmm, wEvex, Z0, B0);
+            return true;
+        }
         emitVEX(getLFlag(size), pp, mmmmm, w, getRXB(dst, src), nds.isValid() ? nds.encoding() : 0, checkAVX);
+        return false;
+    }
+
+    public final boolean vexPrefix(Register dst, Register nds, AMD64Address src, AVXSize size, int pp, int mmmmm, int w, int wEvex, boolean checkAVX) {
+        if (isAVX512Register(dst) || isAVX512Register(nds) || size == AVXSize.ZMM) {
+            evexPrefix(dst, Register.None, nds, src, size, pp, mmmmm, wEvex, Z0, B0);
+            return true;
+        }
+        emitVEX(getLFlag(size), pp, mmmmm, w, getRXB(dst, src), nds.isValid() ? nds.encoding() : 0, checkAVX);
+        return false;
     }
 
     protected static final class EVEXPrefixConfig {
-        public static final int L512 = 2;
-        public static final int LIG = 0;
-
         public static final int Z0 = 0x0;
         public static final int Z1 = 0x1;
 
@@ -967,6 +1000,7 @@ public abstract class AMD64BaseAssembler extends Assembler {
      * Reference: Intel Software Developer's Manual Volume 2, Section 2.6.5
      */
     protected enum EVEXTuple {
+        INVALID(NOT_SUPPORTED_VECTOR_LENGTH, NOT_SUPPORTED_VECTOR_LENGTH, NOT_SUPPORTED_VECTOR_LENGTH),
         FV_NO_BROADCAST_32BIT(16, 32, 64),
         FV_BROADCAST_32BIT(4, 4, 4),
         FV_NO_BROADCAST_64BIT(16, 32, 64),
@@ -1022,6 +1056,17 @@ public abstract class AMD64BaseAssembler extends Assembler {
         }
     }
 
+    public static final class EVEXComparisonPredicate {
+        public static final int EQ = 0;
+        public static final int LT = 1;
+        public static final int LE = 2;
+        public static final int FALSE = 3;
+        public static final int NEQ = 4;
+        public static final int NLT = 5;
+        public static final int NLE = 6;
+        public static final int TRUE = 7;
+    }
+
  // @formatter:off
  //
  // Instruction Format and EVEX illustrated below (optional []):
@@ -1154,7 +1199,7 @@ public abstract class AMD64BaseAssembler extends Assembler {
      * Helper method for emitting EVEX prefix in the form of RRRM. Because the memory addressing in
      * EVEX-encoded instructions employ a compressed displacement scheme when using disp8 form, the
      * user of this API should make sure to encode the operands using
-     * {@link #emitEVEXOperandHelper(Register, AMD64Address, int, int)}.
+     * {@link #emitOperandHelper(Register, AMD64Address, int, int)}.
      */
     protected final void evexPrefix(Register dst, Register mask, Register nds, AMD64Address src, AVXSize size, int pp, int mm, int w, int z, int b) {
         assert !mask.isValid() || inRC(MASK, mask);
