@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -343,11 +343,16 @@ public class MatchProcessor extends AbstractProcessor {
         final boolean commutative;
 
         /**
-         * Can multiple users of this node subsume it. Constants can be swallowed into a match even
-         * if there are multiple users.
+         * Can multiple users of this node subsume it.
          */
         final boolean shareable;
 
+        /**
+         * Can this node be swallowed into a match. Constants can be consumed by a match even if it
+         * has multiple users.
+         */
+        final boolean consumable;
+
         /**
          * Can this node be subsumed into a match even if there are side effecting nodes between
          * this node and the match.
@@ -356,7 +361,8 @@ public class MatchProcessor extends AbstractProcessor {
 
         final Set<Element> originatingElements = new HashSet<>();
 
-        TypeDescriptor(TypeMirror mirror, String shortName, String nodeClass, String nodePackage, List<String> inputs, boolean commutative, boolean shareable, boolean ignoresSideEffects) {
+        TypeDescriptor(TypeMirror mirror, String shortName, String nodeClass, String nodePackage, List<String> inputs,
+                        boolean commutative, boolean shareable, boolean consumable, boolean ignoresSideEffects) {
             this.mirror = mirror;
             this.shortName = shortName;
             this.nodeClass = nodeClass;
@@ -364,6 +370,7 @@ public class MatchProcessor extends AbstractProcessor {
             this.inputs = inputs;
             this.commutative = commutative;
             this.shareable = shareable;
+            this.consumable = consumable;
             this.ignoresSideEffects = ignoresSideEffects;
             assert !commutative || inputs.size() == 2;
         }
@@ -376,9 +383,9 @@ public class MatchProcessor extends AbstractProcessor {
 
     private TypeDescriptor valueType;
 
-    private void declareType(TypeMirror mirror, String shortName, String nodeClass, String nodePackage, List<String> inputs, boolean commutative, boolean shareable, boolean ignoresSideEffects,
-                    Element element) {
-        TypeDescriptor descriptor = new TypeDescriptor(mirror, shortName, nodeClass, nodePackage, inputs, commutative, shareable, ignoresSideEffects);
+    private void declareType(TypeMirror mirror, String shortName, String nodeClass, String nodePackage, List<String> inputs,
+                    boolean commutative, boolean shareable, boolean consumable, boolean ignoresSideEffects, Element element) {
+        TypeDescriptor descriptor = new TypeDescriptor(mirror, shortName, nodeClass, nodePackage, inputs, commutative, shareable, consumable, ignoresSideEffects);
         descriptor.originatingElements.add(element);
         knownTypes.put(shortName, descriptor);
     }
@@ -461,7 +468,7 @@ public class MatchProcessor extends AbstractProcessor {
 
         private String formatPrefix() {
             if (nodeType == valueType) {
-                return String.format("new MatchPattern(%s, false, false", name != null ? ("\"" + name + "\"") : "null");
+                return String.format("new MatchPattern(%s, false, false, false", name != null ? ("\"" + name + "\"") : "null");
             } else {
                 return String.format("new MatchPattern(%s.class, %s", nodeType.nodeClass, name != null ? ("\"" + name + "\"") : "null");
             }
@@ -470,13 +477,13 @@ public class MatchProcessor extends AbstractProcessor {
         private String formatSuffix() {
             if (nodeType != null) {
                 if (inputs.length != nodeType.inputs.size()) {
-                    return ", true, " + nodeType.ignoresSideEffects + ")";
+                    return ", true, " + nodeType.consumable + ", " + nodeType.ignoresSideEffects + ")";
                 } else {
                     if (nodeType.inputs.size() > 0) {
-                        return ", " + nodeType.nodeClass + "_positions, " + !nodeType.shareable + ", " + nodeType.ignoresSideEffects + ")";
+                        return ", " + nodeType.nodeClass + "_positions, " + !nodeType.shareable + ", " + nodeType.consumable + ", " + nodeType.ignoresSideEffects + ")";
                     }
                     if (nodeType.shareable) {
-                        return ", false, " + nodeType.ignoresSideEffects + ")";
+                        return ", false, " + nodeType.consumable + ", " + nodeType.ignoresSideEffects + ")";
                     }
                 }
             }
@@ -519,6 +526,9 @@ public class MatchProcessor extends AbstractProcessor {
             out.println("import org.graalvm.compiler.core.gen.NodeMatchRules;");
             out.println("import org.graalvm.compiler.graph.Position;");
             for (String p : info.requiredPackages) {
+                if (p.equals(pkg)) {
+                    continue;
+                }
                 out.println("import " + p + ".*;");
             }
             out.println("");
@@ -545,7 +555,7 @@ public class MatchProcessor extends AbstractProcessor {
                 }
                 out.printf("    private static final String[] %s = new String[] {%s};\n", invoker.argumentsListName(), args);
                 out.printf("    private static final class %s implements MatchGenerator {\n", invoker.wrapperClass());
-                out.printf("        static MatchGenerator instance = new %s();\n", invoker.wrapperClass());
+                out.printf("        static final MatchGenerator instance = new %s();\n", invoker.wrapperClass());
                 out.printf("        @Override\n");
                 out.printf("        public ComplexMatchResult match(NodeMatchRules nodeMatchRules, Object...args) {\n");
                 out.printf("            return ((%s) nodeMatchRules).%s(%s);\n", invoker.nodeLIRBuilderClass, invoker.methodName, types);
@@ -729,7 +739,7 @@ public class MatchProcessor extends AbstractProcessor {
             // Define a TypeDescriptor for the generic node but don't enter it into the nodeTypes
             // table since it shouldn't be mentioned in match rules.
             TypeMirror valueTypeMirror = getTypeElement(VALUE_NODE_CLASS_NAME).asType();
-            valueType = new TypeDescriptor(valueTypeMirror, "Value", "ValueNode", "org.graalvm.compiler.nodes", Collections.emptyList(), false, false, false);
+            valueType = new TypeDescriptor(valueTypeMirror, "Value", "ValueNode", "org.graalvm.compiler.nodes", Collections.emptyList(), false, false, false, false);
 
             Map<TypeElement, MatchRuleDescriptor> map = new HashMap<>();
 
@@ -774,7 +784,7 @@ public class MatchProcessor extends AbstractProcessor {
                 if (mirror != null) {
                     matchableNodeAnnotations = getAnnotationValueList(mirror, "value", AnnotationMirror.class);
                 } else {
-                    mirror = getAnnotation(element, getType(MATCHABLE_NODES_CLASS_NAME));
+                    mirror = getAnnotation(element, getType(MATCHABLE_NODE_CLASS_NAME));
                     if (mirror != null) {
                         matchableNodeAnnotations = Collections.singletonList(mirror);
                     } else {
@@ -839,8 +849,9 @@ public class MatchProcessor extends AbstractProcessor {
 
         boolean commutative = getAnnotationValue(matchable, "commutative", Boolean.class);
         boolean shareable = getAnnotationValue(matchable, "shareable", Boolean.class);
+        boolean consumable = getAnnotationValue(matchable, "consumable", Boolean.class);
         boolean ignoresSideEffects = getAnnotationValue(matchable, "ignoresSideEffects", Boolean.class);
-        declareType(nodeClassMirror, shortName, nodeClass, nodePackage, inputs, commutative, shareable, ignoresSideEffects, element);
+        declareType(nodeClassMirror, shortName, nodeClass, nodePackage, inputs, commutative, shareable, consumable, ignoresSideEffects, element);
     }
 
     private void processMatchRules(Map<TypeElement, MatchRuleDescriptor> map, Element element, List<AnnotationMirror> matchRules) {
