@@ -32,8 +32,6 @@ import org.graalvm.compiler.nodes.StructuredGraph;
 import org.graalvm.compiler.nodes.java.InstanceOfNode;
 import org.graalvm.compiler.phases.OptimisticOptimizations;
 import org.graalvm.compiler.phases.tiers.HighTierContext;
-import org.graalvm.compiler.phases.tiers.PhaseContext;
-import org.graalvm.compiler.phases.common.CanonicalizerPhase;
 import org.graalvm.compiler.serviceprovider.GraalServices;
 import org.junit.Test;
 
@@ -76,7 +74,7 @@ public class SingleImplementorInterfaceTest extends GraalCompilerTest {
         ResolvedJavaType expectedReferencedType = getMetaAccess().lookupJavaType(Interface2.class);
         singleImplementorType.initialize();
         StructuredGraph graph = parseEager("singleImplementorInterfaceSnippet1", StructuredGraph.AllowAssumptions.YES);
-        new CanonicalizerPhase().apply(graph, new PhaseContext(getProviders()));
+        createCanonicalizerPhase().apply(graph, getProviders());
         // Devirtualization shouldn't work in this case. The invoke should remain intact.
         InvokeNode invoke = graph.getNodes().filter(InvokeNode.class).first();
         assertTrue(invoke != null, "Should have an invoke");
@@ -92,7 +90,7 @@ public class SingleImplementorInterfaceTest extends GraalCompilerTest {
         ResolvedJavaType singleImplementorType = getMetaAccess().lookupJavaType(SingleImplementor1.class);
         singleImplementorType.initialize();
         StructuredGraph graph = parseEager("singleImplementorInterfaceSnippet2", StructuredGraph.AllowAssumptions.YES);
-        new CanonicalizerPhase().apply(graph, new PhaseContext(getProviders()));
+        createCanonicalizerPhase().apply(graph, getProviders());
         InvokeNode invoke = graph.getNodes().filter(InvokeNode.class).first();
         assertTrue(invoke != null, "Should have an invoke");
         if (GraalServices.hasLookupReferencedType()) {
