@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -154,14 +154,18 @@ public abstract class Node implements Cloneable, Formattable, NodeInterface {
     /**
      * Annotates a method that can be replaced by a compiler intrinsic. A (resolved) call to the
      * annotated method will be processed by a generated {@code InvocationPlugin} that calls either
-     * a factory method or a constructor corresponding with the annotated method.
+     * a factory method or a constructor corresponding with the annotated method. By default the
+     * intrinsics are implemented by invoking the constructor but a factory method may be used
+     * instead. To use a factory method the class implementing the intrinsic must be annotated with
+     * {@link NodeIntrinsicFactory}. To ease error checking of NodeIntrinsics all intrinsics are
+     * expected to be implemented in the same way, so it's not possible to mix constructor and
+     * factory intrinsification in the same class.
      * <p>
      * A factory method corresponding to an annotated method is a static method named
      * {@code intrinsify} defined in the class denoted by {@link #value()}. In order, its signature
      * is as follows:
      * <ol>
      * <li>A {@code GraphBuilderContext} parameter.</li>
-     * <li>A {@code ResolvedJavaMethod} parameter.</li>
      * <li>A sequence of zero or more {@linkplain InjectedNodeParameter injected} parameters.</li>
      * <li>Remaining parameters that match the declared parameters of the annotated method.</li>
      * </ol>
@@ -199,6 +203,15 @@ public abstract class Node implements Cloneable, Formattable, NodeInterface {
         boolean hasSideEffect() default false;
     }
 
+    /**
+     * Marker annotation indicating that the class uses factory methods instead of constructors for
+     * intrinsification.
+     */
+    @java.lang.annotation.Retention(RetentionPolicy.RUNTIME)
+    @java.lang.annotation.Target(ElementType.TYPE)
+    public @interface NodeIntrinsicFactory {
+    }
+
     /**
      * Marker for a node that can be replaced by another node via global value numbering. A
      * {@linkplain NodeClass#isLeafNode() leaf} node can be replaced by another node of the same
@@ -427,12 +440,30 @@ public abstract class Node implements Cloneable, Formattable, NodeInterface {
     }
 
     /**
-     * Checks whether this node has exactly one usgae.
+     * Checks whether this node has exactly one usage.
      */
     public final boolean hasExactlyOneUsage() {
         return hasUsages() && !hasMoreThanOneUsage();
     }
 
+    /**
+     * Checks whether this node has only usages of that type.
+     *
+     * @param type the type of usages to look for
+     */
+    public final boolean hasOnlyUsagesOfType(InputType type) {
+        for (Node usage : usages()) {
+            for (Position pos : usage.inputPositions()) {
+                if (pos.get(usage) == this) {
+                    if (pos.getInputType() != type) {
+                        return false;
+                    }
+                }
+            }
+        }
+        return true;
+    }
+
     /**
      * Adds a given node to this node's {@linkplain #usages() usages}.
      *
@@ -613,6 +644,7 @@ public abstract class Node implements Cloneable, Formattable, NodeInterface {
                 assert assertTrue(newSuccessor.predecessor == null, "unexpected non-null predecessor in new successor (%s): %s, this=%s", newSuccessor, newSuccessor.predecessor, this);
                 newSuccessor.predecessor = this;
             }
+            maybeNotifyInputChanged(this);
         }
     }
 
@@ -817,22 +849,24 @@ public abstract class Node implements Cloneable, Formattable, NodeInterface {
 
     private void replaceAtMatchingUsages(Node other, Predicate<Node> filter, Node toBeDeleted) {
         if (filter == null) {
-            fail("filter cannot be null");
+            throw fail("filter cannot be null");
         }
         checkReplaceWith(other);
         int i = 0;
-        while (i < this.getUsageCount()) {
+        int usageCount = this.getUsageCount();
+        while (i < usageCount) {
             Node usage = this.getUsageAt(i);
             if (filter.test(usage)) {
                 replaceAtUsage(other, toBeDeleted, usage);
                 this.movUsageFromEndTo(i);
+                usageCount--;
             } else {
                 ++i;
             }
         }
     }
 
-    public Node getUsageAt(int index) {
+    private Node getUsageAt(int index) {
         if (index == 0) {
             return this.usage0;
         } else if (index == 1) {
@@ -847,14 +881,61 @@ public abstract class Node implements Cloneable, Formattable, NodeInterface {
         replaceAtMatchingUsages(other, usagePredicate, null);
     }
 
-    public void replaceAtUsages(InputType type, Node other) {
+    private void replaceAtUsagePos(Node other, Node usage, Position pos) {
+        pos.initialize(usage, other);
+        maybeNotifyInputChanged(usage);
+        if (other != null) {
+            other.addUsage(usage);
+        }
+    }
+
+    public void replaceAtUsages(Node other, InputType type) {
         checkReplaceWith(other);
-        for (Node usage : usages().snapshot()) {
+        int i = 0;
+        int usageCount = this.getUsageCount();
+        if (usageCount == 0) {
+            return;
+        }
+        usages: while (i < usageCount) {
+            Node usage = this.getUsageAt(i);
             for (Position pos : usage.inputPositions()) {
                 if (pos.getInputType() == type && pos.get(usage) == this) {
-                    pos.set(usage, other);
+                    replaceAtUsagePos(other, usage, pos);
+                    this.movUsageFromEndTo(i);
+                    usageCount--;
+                    continue usages;
                 }
             }
+            i++;
+        }
+        if (hasNoUsages()) {
+            maybeNotifyZeroUsages(this);
+        }
+    }
+
+    public void replaceAtUsages(Node other, InputType... inputTypes) {
+        checkReplaceWith(other);
+        int i = 0;
+        int usageCount = this.getUsageCount();
+        if (usageCount == 0) {
+            return;
+        }
+        usages: while (i < usageCount) {
+            Node usage = this.getUsageAt(i);
+            for (Position pos : usage.inputPositions()) {
+                for (InputType type : inputTypes) {
+                    if (pos.getInputType() == type && pos.get(usage) == this) {
+                        replaceAtUsagePos(other, usage, pos);
+                        this.movUsageFromEndTo(i);
+                        usageCount--;
+                        continue usages;
+                    }
+                }
+            }
+            i++;
+        }
+        if (hasNoUsages()) {
+            maybeNotifyZeroUsages(this);
         }
     }
 
@@ -912,6 +993,20 @@ public abstract class Node implements Cloneable, Formattable, NodeInterface {
         }
     }
 
+    public void replaceAllInputs(Node oldInput, Node newInput) {
+        while (nodeClass.replaceFirstInput(this, oldInput, newInput)) {
+            updateUsages(oldInput, newInput);
+        }
+    }
+
+    public void replaceFirstInput(Node oldInput, Node newInput, InputType type) {
+        for (Position pos : inputPositions()) {
+            if (pos.getInputType() == type && pos.get(this) == oldInput) {
+                pos.set(this, newInput);
+            }
+        }
+    }
+
     public void clearInputs() {
         assert assertFalse(isDeleted(), "cannot clear inputs of deleted node");
         getNodeClass().unregisterAtInputsAsUsage(this);
@@ -1057,7 +1152,10 @@ public abstract class Node implements Cloneable, Formattable, NodeInterface {
             } else {
                 assertFalse(input.isDeleted(), "input was deleted %s", input);
                 assertTrue(input.isAlive(), "input is not alive yet, i.e., it was not yet added to the graph");
-                assertTrue(pos.getInputType() == InputType.Unchecked || input.isAllowedUsageType(pos.getInputType()), "invalid usage type %s %s", input, pos.getInputType());
+                assertTrue(pos.getInputType() == InputType.Unchecked || input.isAllowedUsageType(pos.getInputType()), "invalid usage type input:%s inputType:%s inputField:%s", input,
+                                pos.getInputType(), pos.getName());
+                Class<?> expectedType = pos.getType();
+                assertTrue(expectedType.isAssignableFrom(input.getClass()), "Invalid input type for %s: expected a %s but was a %s", pos, expectedType, input.getClass());
             }
         }
         return true;
