@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2018, Red Hat Inc. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -63,6 +63,7 @@ import org.graalvm.compiler.hotspot.HotSpotForeignCallLinkage;
 import org.graalvm.compiler.hotspot.HotSpotLIRGenerationResult;
 import org.graalvm.compiler.hotspot.HotSpotLIRGenerator;
 import org.graalvm.compiler.hotspot.HotSpotLockStack;
+import org.graalvm.compiler.hotspot.HotSpotMarkId;
 import org.graalvm.compiler.hotspot.meta.HotSpotConstantLoadAction;
 import org.graalvm.compiler.hotspot.meta.HotSpotProviders;
 import org.graalvm.compiler.hotspot.meta.HotSpotRegistersProvider;
@@ -84,7 +85,6 @@ import org.graalvm.compiler.lir.aarch64.AArch64Move.StoreOp;
 import org.graalvm.compiler.lir.aarch64.AArch64PrefetchOp;
 import org.graalvm.compiler.lir.aarch64.AArch64RestoreRegistersOp;
 import org.graalvm.compiler.lir.aarch64.AArch64SaveRegistersOp;
-import org.graalvm.compiler.lir.aarch64.AArch64ZeroMemoryOp;
 import org.graalvm.compiler.lir.gen.LIRGenerationResult;
 import org.graalvm.compiler.options.OptionValues;
 
@@ -271,7 +271,7 @@ public class AArch64HotSpotLIRGenerator extends AArch64LIRGenerator implements H
             if (encoding.hasBase() || GeneratePIC.getValue(options)) {
                 if (GeneratePIC.getValue(options)) {
                     Variable baseAddress = newVariable(lirKindTool.getWordKind());
-                    AArch64HotSpotMove.BaseMove move = new AArch64HotSpotMove.BaseMove(baseAddress, config);
+                    AArch64HotSpotMove.BaseMove move = new AArch64HotSpotMove.BaseMove(baseAddress);
                     append(move);
                     base = baseAddress;
                 } else {
@@ -300,7 +300,7 @@ public class AArch64HotSpotLIRGenerator extends AArch64LIRGenerator implements H
             if (encoding.hasBase() || GeneratePIC.getValue(options)) {
                 if (GeneratePIC.getValue(options)) {
                     Variable baseAddress = newVariable(LIRKind.value(AArch64Kind.QWORD));
-                    AArch64HotSpotMove.BaseMove move = new AArch64HotSpotMove.BaseMove(baseAddress, config);
+                    AArch64HotSpotMove.BaseMove move = new AArch64HotSpotMove.BaseMove(baseAddress);
                     append(move);
                     base = baseAddress;
                 } else {
@@ -400,6 +400,12 @@ public class AArch64HotSpotLIRGenerator extends AArch64LIRGenerator implements H
         append(new AArch64HotSpotDeoptimizeCallerOp(config));
     }
 
+    @Override
+    public void emitDeoptimizeWithExceptionInCaller(Value exception) {
+        Register thread = getProviders().getRegisters().getThreadRegister();
+        append(new AArch64HotSpotDeoptimizeWithExceptionCallerOp(config, exception, thread));
+    }
+
     @Override
     public void emitDeoptimize(Value actionAndReason, Value failedSpeculation, LIRFrameState state) {
         moveDeoptValuesToThread(actionAndReason, failedSpeculation);
@@ -467,7 +473,7 @@ public class AArch64HotSpotLIRGenerator extends AArch64LIRGenerator implements H
     }
 
     @Override
-    public Value emitLoadConfigValue(int markId, LIRKind kind) {
+    public Value emitLoadConfigValue(HotSpotMarkId markId, LIRKind kind) {
         Variable result = newVariable(kind);
         append(new AArch64HotSpotLoadConfigValueOp(markId, result));
         return result;
@@ -544,15 +550,23 @@ public class AArch64HotSpotLIRGenerator extends AArch64LIRGenerator implements H
     }
 
     @Override
-    public void emitZeroMemory(Value address, Value length) {
-        int dczidValue = config.psrInfoDczidValue;
-        EnumSet<AArch64.Flag> flags = ((AArch64) target().arch).getFlags();
+    public void emitZeroMemory(Value address, Value length, boolean isAligned) {
+        final EnumSet<AArch64.Flag> flags = ((AArch64) target().arch).getFlags();
 
-        // ARMv8-A architecture reference manual D12.2.35 Data Cache Zero ID register says:
-        // * BS, bits [3:0] indicate log2 of the DC ZVA block size in (4-byte) words.
-        // * DZP, bit [4] of indicates whether use of DC ZVA instruction is prohibited.
-        int zvaLength = 4 << (dczidValue & 0xF);
-        boolean isDcZvaProhibited = ((dczidValue & 0x10) != 0);
+        boolean isDcZvaProhibited = true;
+        int zvaLength = 0;
+        if (GraalHotSpotVMConfig.JDK >= 16) {
+            zvaLength = config.zvaLength;
+            isDcZvaProhibited = 0 == config.zvaLength;
+        } else {
+            int dczidValue = config.psrInfoDczidValue;
+
+            // ARMv8-A architecture reference manual D12.2.35 Data Cache Zero ID register says:
+            // * BS, bits [3:0] indicate log2 of the DC ZVA block size in (4-byte) words.
+            // * DZP, bit [4] of indicates whether use of DC ZVA instruction is prohibited.
+            zvaLength = 4 << (dczidValue & 0xF);
+            isDcZvaProhibited = ((dczidValue & 0x10) != 0);
+        }
 
         // Use DC ZVA if it's not prohibited and AArch64 HotSpot flag UseBlockZeroing is on.
         boolean useDcZva = !isDcZvaProhibited && flags.contains(AArch64.Flag.UseBlockZeroing);
@@ -563,7 +577,6 @@ public class AArch64HotSpotLIRGenerator extends AArch64LIRGenerator implements H
             useDcZva = false;
         }
 
-        // Value address is 8-byte aligned; Value length is multiple of 8.
-        append(new AArch64ZeroMemoryOp(asAllocatable(address), asAllocatable(length), useDcZva, zvaLength));
+        emitZeroMemory(address, length, isAligned, useDcZva, zvaLength);
     }
 }
