@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -24,18 +24,22 @@
 
 package org.graalvm.compiler.hotspot;
 
-import static jdk.vm.ci.common.InitTimer.timer;
-import static jdk.vm.ci.hotspot.HotSpotJVMCIRuntime.runtime;
-import static org.graalvm.compiler.core.common.GraalOptions.GeneratePIC;
-import static org.graalvm.compiler.core.common.GraalOptions.HotSpotPrintInlining;
-
-import java.io.PrintStream;
-import java.util.ArrayList;
-import java.util.EnumMap;
-import java.util.List;
-import java.util.Map;
-import java.util.concurrent.atomic.AtomicReference;
-
+import jdk.vm.ci.code.Architecture;
+import jdk.vm.ci.code.stack.StackIntrospection;
+import jdk.vm.ci.common.InitTimer;
+import jdk.vm.ci.hotspot.HotSpotCompilationRequest;
+import jdk.vm.ci.hotspot.HotSpotJVMCIRuntime;
+import jdk.vm.ci.hotspot.HotSpotResolvedJavaMethod;
+import jdk.vm.ci.hotspot.HotSpotResolvedJavaType;
+import jdk.vm.ci.hotspot.HotSpotResolvedObjectType;
+import jdk.vm.ci.hotspot.HotSpotVMConfigStore;
+import jdk.vm.ci.meta.JavaKind;
+import jdk.vm.ci.meta.MetaAccessProvider;
+import jdk.vm.ci.meta.ResolvedJavaMethod;
+import jdk.vm.ci.meta.ResolvedJavaType;
+import jdk.vm.ci.runtime.JVMCI;
+import jdk.vm.ci.runtime.JVMCIBackend;
+import jdk.vm.ci.services.Services;
 import jdk.internal.vm.compiler.collections.EconomicMap;
 import jdk.internal.vm.compiler.collections.EconomicSet;
 import jdk.internal.vm.compiler.collections.Equivalence;
@@ -44,10 +48,14 @@ import org.graalvm.compiler.api.replacements.SnippetReflectionProvider;
 import org.graalvm.compiler.api.runtime.GraalRuntime;
 import org.graalvm.compiler.core.CompilationWrapper.ExceptionAction;
 import org.graalvm.compiler.core.common.CompilationIdentifier;
+import org.graalvm.compiler.core.common.CompilationListenerProfiler;
+import org.graalvm.compiler.core.common.CompilerProfiler;
 import org.graalvm.compiler.core.common.GraalOptions;
 import org.graalvm.compiler.core.common.spi.ForeignCallsProvider;
 import org.graalvm.compiler.core.target.Backend;
+import org.graalvm.compiler.debug.Assertions;
 import org.graalvm.compiler.debug.DebugContext;
+import org.graalvm.compiler.debug.DebugContext.Builder;
 import org.graalvm.compiler.debug.DebugContext.Description;
 import org.graalvm.compiler.debug.DebugHandlersFactory;
 import org.graalvm.compiler.debug.DebugOptions;
@@ -72,22 +80,19 @@ import org.graalvm.compiler.replacements.SnippetCounter.Group;
 import org.graalvm.compiler.runtime.RuntimeProvider;
 import org.graalvm.compiler.serviceprovider.GraalServices;
 
-import jdk.vm.ci.code.Architecture;
-import jdk.vm.ci.code.stack.StackIntrospection;
-import jdk.vm.ci.common.InitTimer;
-import jdk.vm.ci.hotspot.HotSpotCompilationRequest;
-import jdk.vm.ci.hotspot.HotSpotJVMCIRuntime;
-import jdk.vm.ci.hotspot.HotSpotResolvedJavaMethod;
-import jdk.vm.ci.hotspot.HotSpotResolvedJavaType;
-import jdk.vm.ci.hotspot.HotSpotResolvedObjectType;
-import jdk.vm.ci.hotspot.HotSpotVMConfigStore;
-import jdk.vm.ci.meta.JavaKind;
-import jdk.vm.ci.meta.MetaAccessProvider;
-import jdk.vm.ci.meta.ResolvedJavaMethod;
-import jdk.vm.ci.meta.ResolvedJavaType;
-import jdk.vm.ci.runtime.JVMCI;
-import jdk.vm.ci.runtime.JVMCIBackend;
-import jdk.vm.ci.services.Services;
+import java.io.PrintStream;
+import java.util.ArrayList;
+import java.util.EnumMap;
+import java.util.List;
+import java.util.Map;
+import java.util.concurrent.atomic.AtomicReference;
+import java.util.function.Supplier;
+
+import static jdk.vm.ci.common.InitTimer.timer;
+import static jdk.vm.ci.hotspot.HotSpotJVMCIRuntime.runtime;
+import static org.graalvm.compiler.core.common.GraalOptions.GeneratePIC;
+import static org.graalvm.compiler.core.common.GraalOptions.HotSpotPrintInlining;
+import static org.graalvm.compiler.hotspot.GraalHotSpotVMConfigAccess.JDK;
 
 //JaCoCo Exclude
 
@@ -98,6 +103,12 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
 
     private static final boolean IS_AOT = Boolean.parseBoolean(Services.getSavedProperties().get("com.oracle.graalvm.isaot"));
 
+    /**
+     * A factory for a {@link HotSpotGraalManagementRegistration} injected by
+     * {@code Target_org_graalvm_compiler_hotspot_HotSpotGraalRuntime}.
+     */
+    private static final Supplier<HotSpotGraalManagementRegistration> AOT_INJECTED_MANAGEMENT = null;
+
     private static boolean checkArrayIndexScaleInvariants(MetaAccessProvider metaAccess) {
         assert metaAccess.getArrayIndexScale(JavaKind.Byte) == 1;
         assert metaAccess.getArrayIndexScale(JavaKind.Boolean) == 1;
@@ -126,6 +137,8 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
 
     private final GraalHotSpotVMConfig config;
 
+    private final Instrumentation instrumentation;
+
     /**
      * The options can be {@linkplain #setOptionValues(String[], String[]) updated} by external
      * interfaces such as JMX. This comes with the risk that inconsistencies can arise as an
@@ -137,6 +150,8 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
     private final DiagnosticsOutputDirectory outputDirectory;
     private final Map<ExceptionAction, Integer> compilationProblemsPerAction;
 
+    private final CompilerProfiler compilerProfiler;
+
     /**
      * @param nameQualifier a qualifier to be added to this runtime's {@linkplain #getName() name}
      * @param compilerConfigurationFactory factory for the compiler configuration
@@ -164,12 +179,21 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         CompilerConfiguration compilerConfiguration = compilerConfigurationFactory.createCompilerConfiguration();
         compilerConfigurationName = compilerConfigurationFactory.getName();
 
+        this.instrumentation = compilerConfigurationFactory.createInstrumentation(options);
+
         if (IS_AOT) {
-            management = null;
+            management = AOT_INJECTED_MANAGEMENT == null ? null : AOT_INJECTED_MANAGEMENT.get();
         } else {
             management = GraalServices.loadSingle(HotSpotGraalManagementRegistration.class, false);
-            if (management != null) {
-                management.initialize(this);
+        }
+        if (management != null) {
+            try {
+                management.initialize(this, config);
+            } catch (ThreadDeath td) {
+                throw td;
+            } catch (Throwable error) {
+                TTY.println("Cannot install GraalVM MBean due to " + error.getMessage());
+                management = null;
             }
         }
 
@@ -218,54 +242,114 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
 
         runtimeStartTime = System.nanoTime();
         bootstrapJVMCI = config.getFlag("BootstrapJVMCI", Boolean.class);
+
+        this.compilerProfiler = GraalServices.loadSingle(CompilerProfiler.class, false);
     }
 
     /**
-     * Constants denoting the GC algorithms available in HotSpot.
+     * Constants denoting the GC algorithms available in HotSpot. The names of the constants match
+     * the constants in the {@code CollectedHeap::Name} C++ enum.
      */
     public enum HotSpotGC {
         // Supported GCs
-        Serial(true, "UseSerialGC"),
-        Parallel(true, "UseParallelGC", "UseParallelOldGC", "UseParNewGC"),
-        CMS(true, "UseConcMarkSweepGC"),
-        G1(true, "UseG1GC"),
+        Serial(true, JDK >= 11, "UseSerialGC", true),
+        Parallel(true, JDK >= 11, "UseParallelGC", true, "UseParallelOldGC", JDK < 15, "UseParNewGC", JDK < 10),
+        CMS(true, JDK >= 11 && JDK <= 14, "UseConcMarkSweepGC", JDK < 14),
+        G1(true, JDK >= 11, "UseG1GC", true),
 
         // Unsupported GCs
-        Epsilon(false, "UseEpsilonGC"),
-        Z(false, "UseZGC");
+        Epsilon(false, JDK >= 11, "UseEpsilonGC", JDK >= 11),
+        Z(false, JDK >= 11, "UseZGC", JDK >= 11),
+        Shenandoah(false, JDK >= 12, "UseShenandoahGC", JDK >= 12);
 
-        HotSpotGC(boolean supported, String... flags) {
+        HotSpotGC(boolean supported, boolean expectNamePresent,
+                        String flag1, boolean expectFlagPresent1,
+                        String flag2, boolean expectFlagPresent2,
+                        String flag3, boolean expectFlagPresent3) {
             this.supported = supported;
-            this.flags = flags;
+            this.expectNamePresent = expectNamePresent;
+            this.expectFlagsPresent = new boolean[]{expectFlagPresent1, expectFlagPresent2, expectFlagPresent3};
+            this.flags = new String[]{flag1, flag2, flag3};
         }
 
+        HotSpotGC(boolean supported, boolean expectNamePresent, String flag, boolean expectFlagPresent) {
+            this.supported = supported;
+            this.expectNamePresent = expectNamePresent;
+            this.expectFlagsPresent = new boolean[]{expectFlagPresent};
+            this.flags = new String[]{flag};
+        }
+
+        /**
+         * Specifies if this GC supported by Graal.
+         */
         final boolean supported;
+
+        /**
+         * Specifies if {@link #name()} is expected to be present in the {@code CollectedHeap::Name}
+         * C++ enum.
+         */
+        final boolean expectNamePresent;
+
+        /**
+         * The VM flags that will select this GC.
+         */
         private final String[] flags;
 
+        /**
+         * Specifies which {@link #flags} are expected to be present in the VM.
+         */
+        final boolean[] expectFlagsPresent;
+
         public boolean isSelected(GraalHotSpotVMConfig config) {
-            for (String flag : flags) {
+            boolean selected = false;
+            for (int i = 0; i < flags.length; i++) {
                 final boolean notPresent = false;
-                if (config.getFlag(flag, Boolean.class, notPresent)) {
-                    return true;
+                if (config.getFlag(flags[i], Boolean.class, notPresent, expectFlagsPresent[i])) {
+                    selected = true;
+                    if (!Assertions.assertionsEnabled()) {
+                        // When asserting, check that isSelected works for all flag names
+                        break;
+                    }
                 }
             }
-            return false;
+            return selected;
         }
 
+        /**
+         * Gets the GC matching {@code name}.
+         *
+         * @param name the ordinal of a {@code CollectedHeap::Name} value
+         */
+        static HotSpotGC forName(int name, GraalHotSpotVMConfig config) {
+            for (HotSpotGC gc : HotSpotGC.values()) {
+                if (config.getConstant("CollectedHeap::" + gc.name(), Integer.class, -1, gc.expectNamePresent) == name) {
+                    return gc;
+                }
+            }
+            return null;
+        }
     }
 
     private HotSpotGC getSelectedGC() throws GraalError {
+        HotSpotGC selected = null;
         for (HotSpotGC gc : HotSpotGC.values()) {
             if (gc.isSelected(config)) {
                 if (!gc.supported) {
                     throw new GraalError(gc.name() + " garbage collector is not supported by Graal");
                 }
-                return gc;
+                selected = gc;
+                if (!Assertions.assertionsEnabled()) {
+                    // When asserting, check that isSelected works for all HotSpotGC values
+                    break;
+                }
             }
         }
-        // As of JDK 9, exactly one GC flag is guaranteed to be selected.
-        // On JDK 8, the default GC is Serial when no GC flag is true.
-        return HotSpotGC.Serial;
+        if (selected == null) {
+            // As of JDK 9, exactly one GC flag is guaranteed to be selected.
+            // On JDK 8, the default GC is Serial when no GC flag is true.
+            selected = HotSpotGC.Serial;
+        }
+        return selected;
     }
 
     private HotSpotBackend registerBackend(HotSpotBackend backend) {
@@ -292,19 +376,31 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
                 HotSpotResolvedObjectType type = ((HotSpotResolvedJavaMethod) compilable).getDeclaringClass();
                 if (type instanceof HotSpotResolvedJavaType) {
                     Class<?> clazz = runtime().getMirror(type);
-                    try {
-                        ClassLoader cl = clazz.getClassLoader();
-                        if (cl != null) {
-                            loaders.add(cl);
+                    if (clazz != null) {
+                        try {
+                            ClassLoader cl = clazz.getClassLoader();
+                            if (cl != null) {
+                                loaders.add(cl);
+                            }
+                        } catch (SecurityException e) {
+                            // This loader can obviously not be used for resolving class names
                         }
-                    } catch (SecurityException e) {
-                        // This loader can obviously not be used for resolving class names
                     }
                 }
             }
         }
+
         Description description = new Description(compilable, compilationId.toString(CompilationIdentifier.Verbosity.ID));
-        return DebugContext.create(compilationOptions, description, metricValues, logStream, factories);
+        Builder builder = new Builder(compilationOptions, factories).//
+                        globalMetrics(metricValues).//
+                        description(description).//
+                        logStream(logStream);
+        if (compilerProfiler != null) {
+            int compileId = ((HotSpotCompilationIdentifier) compilationId).getRequest().getId();
+            builder.compilationListener(new CompilationListenerProfiler(compilerProfiler, compileId));
+        }
+        return builder.build();
+
     }
 
     @Override
@@ -348,6 +444,7 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         return null;
     }
 
+    @Override
     public HotSpotGC getGarbageCollector() {
         return garbageCollector;
     }
@@ -368,6 +465,11 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         return compilerConfigurationName;
     }
 
+    @Override
+    public Instrumentation getInstrumentation() {
+        return instrumentation;
+    }
+
     private long runtimeStartTime;
 
     /**
@@ -375,6 +477,11 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
      */
     private volatile boolean shutdown;
 
+    /**
+     * Shutdown hooks that should be run on the same thread doing the shutdown.
+     */
+    private List<Runnable> shutdownHooks = new ArrayList<>();
+
     /**
      * Take action related to entering a new execution phase.
      *
@@ -386,8 +493,29 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         }
     }
 
-    void shutdown() {
+    /**
+     * Adds a {@link Runnable} that will be run when this runtime is {@link #shutdown()}. The
+     * runnable will be run on the same thread doing the shutdown. All the advice for regular
+     * {@linkplain Runtime#addShutdownHook(Thread) shutdown hooks} also applies here but even more
+     * so since the hook runs on the shutdown thread.
+     */
+    public synchronized void addShutdownHook(Runnable hook) {
+        if (!shutdown) {
+            shutdownHooks.add(hook);
+        }
+    }
+
+    synchronized void shutdown() {
         shutdown = true;
+
+        for (Runnable r : shutdownHooks) {
+            try {
+                r.run();
+            } catch (Throwable e) {
+                e.printStackTrace(TTY.out);
+            }
+        }
+
         metricValues.print(optionsRef.get());
 
         phaseTransition("final");
@@ -407,7 +535,7 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
     /**
      * Substituted by
      * {@code com.oracle.svm.graal.hotspot.libgraal.Target_org_graalvm_compiler_hotspot_HotSpotGraalRuntime}
-     * to call {@code org.graalvm.nativeimage.VMRuntime.shutdown()}.
+     * to call {@code jdk.internal.vm.compiler.nativeimage.VMRuntime.shutdown()}.
      */
     private static void shutdownLibGraal() {
     }
@@ -445,7 +573,7 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
 
     // ------- Management interface ---------
 
-    private final HotSpotGraalManagementRegistration management;
+    private HotSpotGraalManagementRegistration management;
 
     /**
      * @returns the management object for this runtime or {@code null}
