@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2016, Intel Corporation. All rights reserved.
  * Intel Math Library (LIBM) Source Code
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
@@ -46,6 +46,7 @@ import static org.graalvm.compiler.lir.amd64.AMD64HotSpotHelper.recordExternalAd
 import org.graalvm.compiler.asm.Label;
 import org.graalvm.compiler.asm.amd64.AMD64Address;
 import org.graalvm.compiler.asm.amd64.AMD64Assembler;
+import org.graalvm.compiler.asm.amd64.AMD64Assembler.ConditionFlag;
 import org.graalvm.compiler.asm.amd64.AMD64MacroAssembler;
 import org.graalvm.compiler.lir.LIRInstructionClass;
 import org.graalvm.compiler.lir.asm.ArrayDataPointerConstant;
@@ -256,8 +257,7 @@ public final class AMD64MathLogOp extends AMD64MathIntrinsicUnaryOp {
         masm.pshufd(xmm6, xmm5, 228);
         masm.psrlq(xmm1, 12);
         masm.subl(rax, 16);
-        masm.cmpl(rax, 32736);
-        masm.jcc(AMD64Assembler.ConditionFlag.AboveEqual, block0);
+        masm.cmplAndJcc(rax, 32736, ConditionFlag.AboveEqual, block0, false);
 
         masm.bind(block1);
         masm.paddd(xmm0, xmm4);
@@ -324,10 +324,8 @@ public final class AMD64MathLogOp extends AMD64MathIntrinsicUnaryOp {
         masm.movq(xmm0, new AMD64Address(rsp, 0));
         masm.movq(xmm1, new AMD64Address(rsp, 0));
         masm.addl(rax, 16);
-        masm.cmpl(rax, 32768);
-        masm.jcc(AMD64Assembler.ConditionFlag.AboveEqual, block2);
-        masm.cmpl(rax, 16);
-        masm.jcc(AMD64Assembler.ConditionFlag.Below, block3);
+        masm.cmplAndJcc(rax, 32768, ConditionFlag.AboveEqual, block2, false);
+        masm.cmplAndJcc(rax, 16, ConditionFlag.Below, block3, false);
 
         masm.bind(block4);
         masm.addsd(xmm0, xmm0);
@@ -335,8 +333,7 @@ public final class AMD64MathLogOp extends AMD64MathIntrinsicUnaryOp {
 
         masm.bind(block5);
         masm.jcc(AMD64Assembler.ConditionFlag.Above, block4);
-        masm.cmpl(rdx, 0);
-        masm.jcc(AMD64Assembler.ConditionFlag.Above, block4);
+        masm.cmplAndJcc(rdx, 0, ConditionFlag.Above, block4, false);
         masm.jmp(block6);
 
         masm.bind(block3);
@@ -346,8 +343,7 @@ public final class AMD64MathLogOp extends AMD64MathIntrinsicUnaryOp {
         masm.psrlq(xmm1, 32);
         masm.movdl(rcx, xmm1);
         masm.orl(rdx, rcx);
-        masm.cmpl(rdx, 0);
-        masm.jcc(AMD64Assembler.ConditionFlag.Equal, block7);
+        masm.cmplAndJcc(rdx, 0, ConditionFlag.Equal, block7, false);
         masm.xorpd(xmm1, xmm1);
         masm.movl(rax, 18416);
         masm.pinsrw(xmm1, rax, 3);
@@ -369,11 +365,9 @@ public final class AMD64MathLogOp extends AMD64MathIntrinsicUnaryOp {
         masm.psrlq(xmm1, 32);
         masm.movdl(rcx, xmm1);
         masm.addl(rcx, rcx);
-        masm.cmpl(rcx, -2097152);
-        masm.jcc(AMD64Assembler.ConditionFlag.AboveEqual, block5);
+        masm.cmplAndJcc(rcx, -2097152, ConditionFlag.AboveEqual, block5, false);
         masm.orl(rdx, rcx);
-        masm.cmpl(rdx, 0);
-        masm.jcc(AMD64Assembler.ConditionFlag.Equal, block7);
+        masm.cmplAndJcc(rdx, 0, ConditionFlag.Equal, block7, false);
 
         masm.bind(block6);
         masm.xorpd(xmm1, xmm1);
