@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -69,12 +69,19 @@ public final class IntegerSwitchNode extends SwitchNode implements LIRLowerable,
     public static final NodeClass<IntegerSwitchNode> TYPE = NodeClass.create(IntegerSwitchNode.class);
 
     protected final int[] keys;
+    /**
+     * True if keys are contiguous ([n, n+1, n+2, n+3, ..., n+k]).
+     *
+     * When this is the case, we can lookup successor index in O(1).
+     */
+    protected final boolean areKeysContiguous;
 
     public IntegerSwitchNode(ValueNode value, AbstractBeginNode[] successors, int[] keys, double[] keyProbabilities, int[] keySuccessors) {
         super(TYPE, value, successors, keySuccessors, keyProbabilities);
         assert keySuccessors.length == keys.length + 1;
         assert keySuccessors.length == keyProbabilities.length;
         this.keys = keys;
+        areKeysContiguous = keys.length < 2 || keys[keys.length - 1] - keys[0] + 1 == keys.length;
         assert value.stamp(NodeView.DEFAULT) instanceof PrimitiveStamp && value.stamp(NodeView.DEFAULT).getStackKind().isNumericInteger();
         assert assertSorted();
         assert assertNoUntargettedSuccessor();
@@ -151,10 +158,12 @@ public final class IntegerSwitchNode extends SwitchNode implements LIRLowerable,
     }
 
     public int successorIndexAtKey(int key) {
-        for (int i = 0; i < keyCount(); i++) {
-            if (keys[i] == key) {
-                return keySuccessorIndex(i);
-            }
+        if (areKeysContiguous && keys[0] <= key && key <= keys[keys.length - 1]) {
+            return keySuccessorIndex(key - keys[0]);
+        }
+        int index = Arrays.binarySearch(keys, key);
+        if (index >= 0) {
+            return keySuccessorIndex(index);
         }
         return keySuccessorIndex(keyCount());
     }
@@ -429,11 +438,14 @@ public final class IntegerSwitchNode extends SwitchNode implements LIRLowerable,
         }
 
         /*
-         * Collect dead successors. Successors have to be cleaned before adding the new node to the
-         * graph.
+         * Surviving successors have to be cleaned before adding the new node to the graph. Keep the
+         * dead ones attached to the old node for later cleanup.
          */
-        List<AbstractBeginNode> deadSuccessors = successors.filter(s -> !newSuccessors.contains(s)).snapshot();
-        successors.clear();
+        for (int i = 0; i < successors.size(); i++) {
+            if (newSuccessors.contains(successors.get(i))) {
+                successors.set(i, null);
+            }
+        }
 
         /*
          * Create the new switch node. This is done before removing dead successors as `killCFG`
@@ -443,14 +455,11 @@ public final class IntegerSwitchNode extends SwitchNode implements LIRLowerable,
         AbstractBeginNode[] successorsArray = newSuccessors.toArray(new AbstractBeginNode[newSuccessors.size()]);
         SwitchNode newSwitch = graph().add(new IntegerSwitchNode(newValue, successorsArray, newKeys, newKeyProbabilities, newKeySuccessors));
 
-        /* Remove dead successors. */
-        for (AbstractBeginNode successor : deadSuccessors) {
-            GraphUtil.killCFG(successor);
-        }
-
         /* Replace ourselves with the new switch */
         ((FixedWithNextNode) predecessor()).setNext(newSwitch);
-        GraphUtil.killWithUnusedFloatingInputs(this);
+
+        // Remove the old switch and the dead successors.
+        GraphUtil.killCFG(this);
     }
 
     @Override
