@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -33,6 +33,7 @@ import java.lang.reflect.Method;
 import org.graalvm.compiler.api.replacements.Fold;
 import org.graalvm.compiler.debug.GraalError;
 import org.graalvm.compiler.graph.Node.NodeIntrinsic;
+import org.graalvm.compiler.nodes.PluginReplacementNode;
 import org.graalvm.compiler.nodes.ValueNode;
 
 import jdk.vm.ci.meta.MetaAccessProvider;
@@ -45,6 +46,8 @@ import jdk.vm.ci.meta.ResolvedJavaType;
  */
 public abstract class GeneratedInvocationPlugin implements InvocationPlugin {
 
+    private ResolvedJavaMethod executeMethod;
+
     /**
      * Gets the class of the annotation for which this plugin was generated.
      */
@@ -69,21 +72,43 @@ public abstract class GeneratedInvocationPlugin implements InvocationPlugin {
             return true;
         }
 
-        if (IS_IN_NATIVE_IMAGE || IS_BUILDING_NATIVE_IMAGE) {
+        if (IS_IN_NATIVE_IMAGE) {
             // The reflection here is problematic for SVM.
             return true;
         }
 
-        MetaAccessProvider metaAccess = b.getMetaAccess();
-        ResolvedJavaMethod executeMethod = metaAccess.lookupJavaMethod(getExecuteMethod());
-        ResolvedJavaType thisClass = metaAccess.lookupJavaType(getClass());
-        ResolvedJavaMethod thisExecuteMethod = thisClass.resolveConcreteMethod(executeMethod, thisClass);
+        if (b.getMethod().equals(foldAnnotatedMethod)) {
+            return false;
+        }
+
+        if (IS_BUILDING_NATIVE_IMAGE) {
+            // The use of this plugin in the plugin itself shouldn't be folded since that defeats
+            // the purpose of the fold.
+            ResolvedJavaType foldNodeClass = b.getMetaAccess().lookupJavaType(PluginReplacementNode.ReplacementFunction.class);
+            if (foldNodeClass.isAssignableFrom(b.getMethod().getDeclaringClass())) {
+                return false;
+            }
+            ResolvedJavaType foldPluginClass = b.getMetaAccess().lookupJavaType(GeneratedFoldInvocationPlugin.class);
+            if (foldPluginClass.isAssignableFrom(b.getMethod().getDeclaringClass())) {
+                return false;
+            }
+        }
+
+        ResolvedJavaMethod thisExecuteMethod = getExecutedMethod(b);
         if (b.getMethod().equals(thisExecuteMethod)) {
-            // The "execute" method of this plugin is itself being compiled. In (only) this context,
-            // the injected argument of the call to the @Fold annotated method will be non-null.
             return true;
         }
-        throw new AssertionError("must pass null to injected argument of " + foldAnnotatedMethod.format("%H.%n(%p)") + ", not " + arg);
+        throw new AssertionError("must pass null to injected argument of " + foldAnnotatedMethod.format("%H.%n(%p)") + ", not " + arg + " in " + b.getMethod().format("%H.%n(%p)"));
+    }
+
+    private ResolvedJavaMethod getExecutedMethod(GraphBuilderContext b) {
+        if (executeMethod == null) {
+            MetaAccessProvider metaAccess = b.getMetaAccess();
+            ResolvedJavaMethod baseMethod = metaAccess.lookupJavaMethod(getExecuteMethod());
+            ResolvedJavaType thisClass = metaAccess.lookupJavaType(getClass());
+            executeMethod = thisClass.resolveConcreteMethod(baseMethod, thisClass);
+        }
+        return executeMethod;
     }
 
     private static Method getExecuteMethod() {
@@ -93,4 +118,8 @@ public abstract class GeneratedInvocationPlugin implements InvocationPlugin {
             throw new GraalError(e);
         }
     }
+
+    public final boolean isGeneratedFromFoldOrNodeIntrinsic() {
+        return getSource().equals(Fold.class) || getSource().equals(NodeIntrinsic.class);
+    }
 }
