@@ -203,6 +203,7 @@ public final class GraphBuilderConfiguration {
     private final boolean trackNodeSourcePosition;
     private final boolean retainLocalVariables;
     private final Plugins plugins;
+    private final boolean replaceLocalsWithConstants;
 
     public enum BytecodeExceptionMode {
         /**
@@ -231,6 +232,7 @@ public final class GraphBuilderConfiguration {
                     boolean insertFullInfopoints,
                     boolean trackNodeSourcePosition,
                     boolean retainLocalVariables,
+                    boolean replaceLocalsWithConstants,
                     List<ResolvedJavaType> skippedExceptionTypes,
                     Plugins plugins) {
         this.eagerResolving = eagerResolving;
@@ -240,6 +242,7 @@ public final class GraphBuilderConfiguration {
         this.insertFullInfopoints = insertFullInfopoints;
         this.trackNodeSourcePosition = trackNodeSourcePosition;
         this.retainLocalVariables = retainLocalVariables;
+        this.replaceLocalsWithConstants = replaceLocalsWithConstants;
         this.skippedExceptionTypes = skippedExceptionTypes;
         this.plugins = plugins;
     }
@@ -259,6 +262,7 @@ public final class GraphBuilderConfiguration {
                         insertFullInfopoints,
                         trackNodeSourcePosition,
                         retainLocalVariables,
+                        replaceLocalsWithConstants,
                         skippedExceptionTypes,
                         newPlugins);
         return result;
@@ -279,6 +283,7 @@ public final class GraphBuilderConfiguration {
                         insertFullInfopoints,
                         trackNodeSourcePosition,
                         retainLocalVariables,
+                        replaceLocalsWithConstants,
                         skippedExceptionTypes,
                         plugins);
     }
@@ -292,6 +297,7 @@ public final class GraphBuilderConfiguration {
                         insertFullInfopoints,
                         trackNodeSourcePosition,
                         retainLocalVariables,
+                        replaceLocalsWithConstants,
                         skippedExceptionTypes,
                         plugins);
     }
@@ -305,6 +311,7 @@ public final class GraphBuilderConfiguration {
                         insertFullInfopoints,
                         trackNodeSourcePosition,
                         retainLocalVariables,
+                        replaceLocalsWithConstants,
                         Collections.unmodifiableList(Arrays.asList(newSkippedExceptionTypes)),
                         plugins);
     }
@@ -317,6 +324,7 @@ public final class GraphBuilderConfiguration {
                         insertFullInfopoints,
                         trackNodeSourcePosition,
                         retainLocalVariables,
+                        replaceLocalsWithConstants,
                         skippedExceptionTypes,
                         plugins);
     }
@@ -330,6 +338,7 @@ public final class GraphBuilderConfiguration {
                         insertFullInfopoints,
                         trackNodeSourcePosition,
                         retainLocalVariables,
+                        replaceLocalsWithConstants,
                         skippedExceptionTypes,
                         plugins);
     }
@@ -343,6 +352,7 @@ public final class GraphBuilderConfiguration {
                         newInsertFullInfopoints,
                         trackNodeSourcePosition,
                         retainLocalVariables,
+                        replaceLocalsWithConstants,
                         skippedExceptionTypes,
                         plugins);
     }
@@ -356,6 +366,7 @@ public final class GraphBuilderConfiguration {
                         insertFullInfopoints,
                         newTrackNodeSourcePosition,
                         retainLocalVariables,
+                        replaceLocalsWithConstants,
                         skippedExceptionTypes,
                         plugins);
     }
@@ -369,6 +380,21 @@ public final class GraphBuilderConfiguration {
                         insertFullInfopoints,
                         trackNodeSourcePosition,
                         newRetainLocalVariables,
+                        replaceLocalsWithConstants,
+                        skippedExceptionTypes,
+                        plugins);
+    }
+
+    public GraphBuilderConfiguration withReplaceLocalsWithConstants(boolean newReplaceLocalsWithConstants) {
+        return new GraphBuilderConfiguration(
+                        eagerResolving,
+                        unresolvedIsError,
+                        bytecodeExceptionMode,
+                        omitAssertions,
+                        insertFullInfopoints,
+                        trackNodeSourcePosition,
+                        retainLocalVariables,
+                        newReplaceLocalsWithConstants,
                         skippedExceptionTypes,
                         plugins);
     }
@@ -401,6 +427,10 @@ public final class GraphBuilderConfiguration {
         return insertFullInfopoints;
     }
 
+    public boolean replaceLocalsWithConstants() {
+        return this.replaceLocalsWithConstants;
+    }
+
     public static GraphBuilderConfiguration getDefault(Plugins plugins) {
         return new GraphBuilderConfiguration(
                         /* eagerResolving: */ false,
@@ -410,6 +440,7 @@ public final class GraphBuilderConfiguration {
                         /* insertFullInfopoints: */ false,
                         /* trackNodeSourcePosition: */ false,
                         /* retainLocalVariables */ false,
+                        /* replaceLocalsWithConstants */ false,
                         Collections.emptyList(),
                         plugins);
     }
@@ -423,6 +454,7 @@ public final class GraphBuilderConfiguration {
                         /* insertFullInfopoints: */ false,
                         /* trackNodeSourcePosition: */ false,
                         /* retainLocalVariables */ false,
+                        /* replaceLocalsWithConstants */ false,
                         Collections.emptyList(),
                         plugins);
     }
