@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,7 +25,11 @@
 package org.graalvm.compiler.replacements;
 
 import static org.graalvm.compiler.debug.GraalError.unimplemented;
+import static org.graalvm.compiler.nodeinfo.InputType.Anchor;
+import static org.graalvm.compiler.nodeinfo.InputType.Guard;
+import static org.graalvm.compiler.nodeinfo.NodeCycles.CYCLES_0;
 import static org.graalvm.compiler.nodeinfo.NodeCycles.CYCLES_IGNORED;
+import static org.graalvm.compiler.nodeinfo.NodeSize.SIZE_0;
 import static org.graalvm.compiler.nodeinfo.NodeSize.SIZE_IGNORED;
 
 import java.net.URI;
@@ -41,7 +45,6 @@ import jdk.internal.vm.compiler.collections.Equivalence;
 import org.graalvm.compiler.api.replacements.Fold;
 import org.graalvm.compiler.bytecode.Bytecode;
 import org.graalvm.compiler.bytecode.BytecodeProvider;
-import org.graalvm.compiler.bytecode.ResolvedJavaMethodBytecode;
 import org.graalvm.compiler.core.common.PermanentBailoutException;
 import org.graalvm.compiler.core.common.cfg.CFGVerifier;
 import org.graalvm.compiler.core.common.spi.ConstantFieldProvider;
@@ -51,6 +54,7 @@ import org.graalvm.compiler.core.common.type.StampPair;
 import org.graalvm.compiler.debug.DebugCloseable;
 import org.graalvm.compiler.debug.DebugContext;
 import org.graalvm.compiler.debug.GraalError;
+import org.graalvm.compiler.graph.IterableNodeType;
 import org.graalvm.compiler.graph.Node;
 import org.graalvm.compiler.graph.Node.NodeIntrinsic;
 import org.graalvm.compiler.graph.NodeClass;
@@ -59,12 +63,14 @@ import org.graalvm.compiler.graph.SourceLanguagePosition;
 import org.graalvm.compiler.graph.SourceLanguagePositionProvider;
 import org.graalvm.compiler.graph.spi.Canonicalizable;
 import org.graalvm.compiler.java.GraphBuilderPhase;
+import org.graalvm.compiler.nodeinfo.InputType;
 import org.graalvm.compiler.nodeinfo.NodeInfo;
 import org.graalvm.compiler.nodes.AbstractBeginNode;
 import org.graalvm.compiler.nodes.AbstractMergeNode;
 import org.graalvm.compiler.nodes.CallTargetNode;
 import org.graalvm.compiler.nodes.CallTargetNode.InvokeKind;
 import org.graalvm.compiler.nodes.ControlSinkNode;
+import org.graalvm.compiler.nodes.DeoptBciSupplier;
 import org.graalvm.compiler.nodes.DeoptimizeNode;
 import org.graalvm.compiler.nodes.EncodedGraph;
 import org.graalvm.compiler.nodes.FixedNode;
@@ -76,6 +82,7 @@ import org.graalvm.compiler.nodes.InvokeWithExceptionNode;
 import org.graalvm.compiler.nodes.MergeNode;
 import org.graalvm.compiler.nodes.NodeView;
 import org.graalvm.compiler.nodes.ParameterNode;
+import org.graalvm.compiler.nodes.PluginReplacementNode;
 import org.graalvm.compiler.nodes.ReturnNode;
 import org.graalvm.compiler.nodes.SimplifyingGraphDecoder;
 import org.graalvm.compiler.nodes.StateSplit;
@@ -83,7 +90,8 @@ import org.graalvm.compiler.nodes.StructuredGraph;
 import org.graalvm.compiler.nodes.UnwindNode;
 import org.graalvm.compiler.nodes.ValueNode;
 import org.graalvm.compiler.nodes.cfg.ControlFlowGraph;
-import org.graalvm.compiler.nodes.extended.ForeignCallNode;
+import org.graalvm.compiler.nodes.extended.AnchoringNode;
+import org.graalvm.compiler.nodes.extended.GuardingNode;
 import org.graalvm.compiler.nodes.extended.IntegerSwitchNode;
 import org.graalvm.compiler.nodes.graphbuilderconf.GeneratedInvocationPlugin;
 import org.graalvm.compiler.nodes.graphbuilderconf.GraphBuilderContext;
@@ -95,6 +103,7 @@ import org.graalvm.compiler.nodes.graphbuilderconf.InvocationPlugins;
 import org.graalvm.compiler.nodes.graphbuilderconf.InvocationPlugins.InvocationPluginReceiver;
 import org.graalvm.compiler.nodes.graphbuilderconf.LoopExplosionPlugin;
 import org.graalvm.compiler.nodes.graphbuilderconf.LoopExplosionPlugin.LoopExplosionKind;
+import org.graalvm.compiler.nodes.graphbuilderconf.MethodSubstitutionPlugin;
 import org.graalvm.compiler.nodes.graphbuilderconf.NodePlugin;
 import org.graalvm.compiler.nodes.graphbuilderconf.ParameterPlugin;
 import org.graalvm.compiler.nodes.java.LoadFieldNode;
@@ -106,6 +115,8 @@ import org.graalvm.compiler.nodes.java.NewInstanceNode;
 import org.graalvm.compiler.nodes.java.NewMultiArrayNode;
 import org.graalvm.compiler.nodes.java.StoreFieldNode;
 import org.graalvm.compiler.nodes.java.StoreIndexedNode;
+import org.graalvm.compiler.nodes.spi.CoreProviders;
+import org.graalvm.compiler.nodes.spi.Replacements;
 import org.graalvm.compiler.nodes.spi.StampProvider;
 import org.graalvm.compiler.nodes.type.StampTool;
 import org.graalvm.compiler.nodes.util.GraphUtil;
@@ -188,6 +199,23 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
             return caller != null;
         }
 
+        /**
+         * Gets the call stack representing this method scope and its callers.
+         */
+        public StackTraceElement[] getCallStack() {
+            StackTraceElement[] stack = new StackTraceElement[inliningDepth + 1];
+            PEMethodScope frame = this;
+            int index = 0;
+            int bci = -1;
+            while (frame != null) {
+                stack[index++] = frame.method.asStackTraceElement(bci);
+                bci = frame.invokeData == null ? 0 : frame.invokeData.invoke.bci();
+                frame = frame.caller;
+            }
+            assert index == stack.length : index + " != " + stack.length;
+            return stack;
+        }
+
         @Override
         public NodeSourcePosition getCallerBytecodePosition(NodeSourcePosition position) {
             if (caller == null) {
@@ -223,6 +251,11 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
             }
             return res;
         }
+
+        @Override
+        public String toString() {
+            return getClass().getSimpleName() + '[' + method.format("%H.%n(%p)") + ']';
+        }
     }
 
     private static final class UnresolvedSourceLanguagePosition implements SourceLanguagePosition {
@@ -271,7 +304,7 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
                     int count = 0;
                     PEGraphDecoder.PEMethodScope scope = methodScope;
                     while (scope != null) {
-                        if (scope.method.equals(callInlinedMethod)) {
+                        if (scope.method.equals(peRootForInlining)) {
                             count++;
                         }
                         scope = scope.caller;
@@ -298,33 +331,38 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
          */
         @Override
         public boolean canDeferPlugin(GeneratedInvocationPlugin plugin) {
-            return plugin.getSource().equals(Fold.class) || plugin.getSource().equals(Node.NodeIntrinsic.class);
+            return plugin.isGeneratedFromFoldOrNodeIntrinsic();
         }
 
         @Override
         public BailoutException bailout(String string) {
             BailoutException bailout = new PermanentBailoutException(string);
-            throw GraphUtil.createBailoutException(string, bailout, GraphUtil.approxSourceStackTraceElement(methodScope.getCallerBytecodePosition()));
+            throw GraphUtil.createBailoutException(string, bailout, methodScope.getCallStack());
         }
 
         @Override
         public StampProvider getStampProvider() {
-            return stampProvider;
+            return providers.getStampProvider();
         }
 
         @Override
         public MetaAccessProvider getMetaAccess() {
-            return metaAccess;
+            return providers.getMetaAccess();
         }
 
         @Override
         public ConstantReflectionProvider getConstantReflection() {
-            return constantReflection;
+            return providers.getConstantReflection();
         }
 
         @Override
         public ConstantFieldProvider getConstantFieldProvider() {
-            return constantFieldProvider;
+            return providers.getConstantFieldProvider();
+        }
+
+        @Override
+        public Replacements getReplacements() {
+            return providers.getReplacements();
         }
 
         @Override
@@ -339,7 +377,7 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
 
         @Override
         public IntrinsicContext getIntrinsic() {
-            return null;
+            return PEGraphDecoder.this.getIntrinsic();
         }
 
         @Override
@@ -367,6 +405,11 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
             return false;
         }
 
+        @Override
+        public boolean intrinsify(ResolvedJavaMethod targetMethod, StructuredGraph substituteGraph, InvocationPlugin.Receiver receiver, ValueNode[] argsIncludingReceiver) {
+            return false;
+        }
+
         @Override
         public void setStateAfter(StateSplit stateSplit) {
             throw unimplemented();
@@ -384,12 +427,13 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
 
         @Override
         public ResolvedJavaMethod getMethod() {
-            throw unimplemented();
+            return methodScope.method;
         }
 
         @Override
         public int bci() {
-            return invoke.bci();
+            // There is no BCI available when decoding an encoded method
+            return -1;
         }
 
         @Override
@@ -405,16 +449,18 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
         @Override
         public String toString() {
             Formatter fmt = new Formatter();
-            PEMethodScope scope = this.methodScope;
-            fmt.format("%s", new ResolvedJavaMethodBytecode(scope.method).asStackTraceElement(invoke.bci()));
-            NodeSourcePosition callers = scope.getCallerBytecodePosition();
-            if (callers != null) {
-                fmt.format("%n%s", callers);
+            fmt.format("Decoding %s", methodScope.method.format("%H.%n(%p)"));
+            for (StackTraceElement e : methodScope.getCallStack()) {
+                fmt.format("%n\tat %s", e);
             }
             return fmt.toString();
         }
     }
 
+    protected IntrinsicContext getIntrinsic() {
+        return null;
+    }
+
     protected class PEAppendGraphBuilderContext extends PENonAppendGraphBuilderContext {
         protected FixedWithNextNode lastInstr;
         protected ValueNode pushedNode;
@@ -516,9 +562,74 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
             appendInvoke(methodScope.caller, methodScope.callerLoopScope, methodScope.invokeData, callTarget);
             updateLastInstruction(invoke.asNode());
         }
+
+        @Override
+        public GraphBuilderContext getNonIntrinsicAncestor() {
+            return null;
+        }
     }
 
-    @NodeInfo(cycles = CYCLES_IGNORED, size = SIZE_IGNORED)
+    protected class PEPluginGraphBuilderContext extends PENonAppendGraphBuilderContext {
+        protected FixedWithNextNode insertBefore;
+        protected ValueNode pushedNode;
+
+        public PEPluginGraphBuilderContext(PEMethodScope inlineScope, FixedWithNextNode insertBefore) {
+            super(inlineScope, inlineScope.invokeData != null ? inlineScope.invokeData.invoke : null);
+            this.insertBefore = insertBefore;
+        }
+
+        @Override
+        public void push(JavaKind kind, ValueNode value) {
+            if (pushedNode != null) {
+                throw unimplemented("Only one push is supported");
+            }
+            pushedNode = value;
+        }
+
+        @Override
+        public void setStateAfter(StateSplit sideEffect) {
+            assert sideEffect.hasSideEffect();
+            FrameState stateAfter = getGraph().add(new FrameState(BytecodeFrame.BEFORE_BCI));
+            sideEffect.setStateAfter(stateAfter);
+        }
+
+        @SuppressWarnings("try")
+        @Override
+        public <T extends ValueNode> T append(T v) {
+            if (v.graph() != null) {
+                return v;
+            }
+            try (DebugCloseable position = withNodeSoucePosition()) {
+                T added = getGraph().addOrUniqueWithInputs(v);
+                if (added == v) {
+                    updateLastInstruction(v);
+                }
+                return added;
+            }
+        }
+
+        private DebugCloseable withNodeSoucePosition() {
+            if (getGraph().trackNodeSourcePosition()) {
+                NodeSourcePosition callerBytecodePosition = methodScope.getCallerBytecodePosition();
+                if (callerBytecodePosition != null) {
+                    return getGraph().withNodeSourcePosition(callerBytecodePosition);
+                }
+            }
+            return null;
+        }
+
+        private <T extends ValueNode> void updateLastInstruction(T value) {
+            if (value instanceof FixedWithNextNode) {
+                FixedWithNextNode fixed = (FixedWithNextNode) value;
+                graph.addBeforeFixed(insertBefore, fixed);
+            } else if (value instanceof FixedNode) {
+                // Block terminating fixed nodes shouldn't be inserted
+                throw GraalError.shouldNotReachHere();
+            }
+        }
+    }
+
+    @NodeInfo(cycles = CYCLES_IGNORED, size = SIZE_IGNORED, allowedUsageTypes = {InputType.Value, InputType.Guard, InputType.Anchor})
     static class ExceptionPlaceholderNode extends ValueNode {
         public static final NodeClass<ExceptionPlaceholderNode> TYPE = NodeClass.create(ExceptionPlaceholderNode.class);
 
@@ -527,6 +638,18 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
         }
     }
 
+    @NodeInfo(allowedUsageTypes = {Anchor, Guard, InputType.Value}, cycles = CYCLES_0, size = SIZE_0)
+    static final class FixedAnchorNode extends FixedWithNextNode implements AnchoringNode, GuardingNode, IterableNodeType {
+
+        public static final NodeClass<FixedAnchorNode> TYPE = NodeClass.create(FixedAnchorNode.class);
+        @Input ValueNode value;
+
+        protected FixedAnchorNode(ValueNode value) {
+            super(TYPE, value.stamp(NodeView.DEFAULT));
+            this.value = value;
+        }
+    }
+
     protected static class SpecialCallTargetCacheKey {
         private final InvokeKind invokeKind;
         private final ResolvedJavaMethod targetMethod;
@@ -562,14 +685,14 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
     private final NodePlugin[] nodePlugins;
     private final EconomicMap<SpecialCallTargetCacheKey, Object> specialCallTargetCache;
     private final EconomicMap<ResolvedJavaMethod, Object> invocationPluginCache;
-    private final ResolvedJavaMethod callInlinedMethod;
+    private final ResolvedJavaMethod peRootForInlining;
     protected final SourceLanguagePositionProvider sourceLanguagePositionProvider;
 
-    public PEGraphDecoder(Architecture architecture, StructuredGraph graph, MetaAccessProvider metaAccess, ConstantReflectionProvider constantReflection, ConstantFieldProvider constantFieldProvider,
-                    StampProvider stampProvider, LoopExplosionPlugin loopExplosionPlugin, InvocationPlugins invocationPlugins, InlineInvokePlugin[] inlineInvokePlugins,
+    public PEGraphDecoder(Architecture architecture, StructuredGraph graph, CoreProviders providers, LoopExplosionPlugin loopExplosionPlugin, InvocationPlugins invocationPlugins,
+                    InlineInvokePlugin[] inlineInvokePlugins,
                     ParameterPlugin parameterPlugin,
-                    NodePlugin[] nodePlugins, ResolvedJavaMethod callInlinedMethod, SourceLanguagePositionProvider sourceLanguagePositionProvider) {
-        super(architecture, graph, metaAccess, constantReflection, constantFieldProvider, stampProvider, true);
+                    NodePlugin[] nodePlugins, ResolvedJavaMethod peRootForInlining, SourceLanguagePositionProvider sourceLanguagePositionProvider) {
+        super(architecture, graph, providers, true);
         this.loopExplosionPlugin = loopExplosionPlugin;
         this.invocationPlugins = invocationPlugins;
         this.inlineInvokePlugins = inlineInvokePlugins;
@@ -577,7 +700,7 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
         this.nodePlugins = nodePlugins;
         this.specialCallTargetCache = EconomicMap.create(Equivalence.DEFAULT);
         this.invocationPluginCache = EconomicMap.create(Equivalence.DEFAULT);
-        this.callInlinedMethod = callInlinedMethod;
+        this.peRootForInlining = peRootForInlining;
         this.sourceLanguagePositionProvider = sourceLanguagePositionProvider;
     }
 
@@ -593,6 +716,7 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
     public void decode(ResolvedJavaMethod method, boolean isSubstitution, boolean trackNodeSourcePosition) {
         try (DebugContext.Scope scope = debug.scope("PEGraphDecode", graph)) {
             EncodedGraph encodedGraph = lookupEncodedGraph(method, null, null, isSubstitution, trackNodeSourcePosition);
+            recordGraphElements(encodedGraph);
             PEMethodScope methodScope = new PEMethodScope(graph, null, null, encodedGraph, method, null, 0, loopExplosionPlugin, null);
             decode(createInitialLoopScope(methodScope, null));
             cleanupGraph(methodScope);
@@ -607,7 +731,33 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
             /* Check that the control flow graph can be computed, to catch problems early. */
             assert CFGVerifier.verify(ControlFlowGraph.compute(graph, true, true, true, true));
         } catch (Throwable ex) {
-            throw GraalError.shouldNotReachHere("Control flow graph not valid after partial evaluation");
+            throw GraalError.shouldNotReachHere(ex, "Control flow graph not valid after partial evaluation");
+        }
+    }
+
+    private void recordGraphElements(EncodedGraph encodedGraph) {
+        List<ResolvedJavaMethod> inlinedMethods = encodedGraph.getInlinedMethods();
+        if (inlinedMethods != null) {
+            for (ResolvedJavaMethod other : inlinedMethods) {
+                graph.recordMethod(other);
+            }
+        }
+        Assumptions assumptions = graph.getAssumptions();
+        Assumptions inlinedAssumptions = encodedGraph.getAssumptions();
+        if (assumptions != null) {
+            if (inlinedAssumptions != null) {
+                assumptions.record(inlinedAssumptions);
+            }
+        } else {
+            assert inlinedAssumptions == null : String.format("cannot inline graph (%s) which makes assumptions into a graph (%s) that doesn't", encodedGraph, graph);
+        }
+        if (encodedGraph.getFields() != null) {
+            for (ResolvedJavaField field : encodedGraph.getFields()) {
+                graph.recordField(field);
+            }
+        }
+        if (encodedGraph.hasUnsafeAccess()) {
+            graph.markUnsafeAccess();
         }
     }
 
@@ -633,12 +783,37 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
                 assert frameState.isDeleted();
             }
         }
+        /*
+         * Cleanup anchor nodes introduced for exception object anchors during inlining. When we
+         * inline through an invoke with exception and the caller has an exception handler attached
+         * this handler can use the exception object node as anchor or guard. During inlining the
+         * exception object node is replaced with its actual value produced in the callee, however
+         * this value is not necessarily a guarding or anchoring node. Thus, we introduce artificial
+         * fixed anchor nodes in the exceptional path in the callee the caller can use as anchor,
+         * guard or value input. After we are done we remove all anchors that are not needed any
+         * more, i.e., they do not have any guard or anchor usages. If they have guard or anchor
+         * usages we rewrite the anchor for exactly those nodes to a real value anchor node that can
+         * be optimized later.
+         *
+         * Optimizing the anchor early is not possible during partial evaluation, since we do not
+         * know if a node not yet decoded in the caller will reference the exception object
+         * replacement node in the callee as an anchor or guard.
+         */
+        for (FixedAnchorNode anchor : graph.getNodes(FixedAnchorNode.TYPE).snapshot()) {
+            AbstractBeginNode newAnchor = AbstractBeginNode.prevBegin(anchor);
+            assert newAnchor != null : "Must find prev begin node";
+            anchor.replaceAtUsages(newAnchor, InputType.Guard, InputType.Anchor);
+            // all other usages can really consume the value
+            anchor.replaceAtUsages(anchor.value);
+            assert anchor.hasNoUsages();
+            GraphUtil.unlinkFixedNode(anchor);
+            anchor.safeDelete();
+        }
     }
 
     @Override
     protected void checkLoopExplosionIteration(MethodScope s, LoopScope loopScope) {
         PEMethodScope methodScope = (PEMethodScope) s;
-
         if (loopScope.loopIteration > Options.MaximumLoopExplosionCount.getValue(options)) {
             throw tooManyLoopExplosionIterations(methodScope, options);
         }
@@ -647,13 +822,12 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
     private static RuntimeException tooManyLoopExplosionIterations(PEMethodScope methodScope, OptionValues options) {
         String message = "too many loop explosion iterations - does the explosion not terminate for method " + methodScope.method + "?";
         RuntimeException bailout = Options.FailedLoopExplosionIsFatal.getValue(options) ? new RuntimeException(message) : new PermanentBailoutException(message);
-        throw GraphUtil.createBailoutException(message, bailout, GraphUtil.approxSourceStackTraceElement(methodScope.getCallerBytecodePosition()));
+        throw GraphUtil.createBailoutException(message, bailout, methodScope.getCallStack());
     }
 
     @Override
     protected LoopScope handleInvoke(MethodScope s, LoopScope loopScope, InvokeData invokeData) {
         PEMethodScope methodScope = (PEMethodScope) s;
-
         /*
          * Decode the call target, but do not add it to the graph yet. This avoids adding usages for
          * all the arguments, which are expensive to remove again when we can inline the method.
@@ -664,13 +838,8 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
             MethodCallTargetNode methodCall = (MethodCallTargetNode) callTarget;
             if (methodCall.invokeKind().hasReceiver()) {
                 invokeData.constantReceiver = methodCall.arguments().get(0).asJavaConstant();
-                NodeSourcePosition invokePosition = invokeData.invoke.asNode().getNodeSourcePosition();
-                if (invokeData.constantReceiver != null && invokePosition != null) {
-                    // new NodeSourcePosition(invokeData.constantReceiver,
-                    // invokePosition.getCaller(), invokePosition.getMethod(),
-                    // invokePosition.getBCI());
-                }
             }
+            callTarget = trySimplifyCallTarget(methodScope, invokeData, (MethodCallTargetNode) callTarget);
             LoopScope inlineLoopScope = trySimplifyInvoke(methodScope, loopScope, invokeData, (MethodCallTargetNode) callTarget);
             if (inlineLoopScope != null) {
                 return inlineLoopScope;
@@ -683,14 +852,23 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
         return super.handleInvoke(methodScope, loopScope, invokeData);
     }
 
-    protected LoopScope trySimplifyInvoke(PEMethodScope methodScope, LoopScope loopScope, InvokeData invokeData, MethodCallTargetNode callTarget) {
+    protected MethodCallTargetNode trySimplifyCallTarget(PEMethodScope methodScope, InvokeData invokeData, MethodCallTargetNode callTarget) {
         // attempt to devirtualize the call
         ResolvedJavaMethod specialCallTarget = getSpecialCallTarget(invokeData, callTarget);
         if (specialCallTarget != null) {
             callTarget.setTargetMethod(specialCallTarget);
             callTarget.setInvokeKind(InvokeKind.Special);
+            return callTarget;
         }
+        if (callTarget.invokeKind().isInterface()) {
+            Invoke invoke = invokeData.invoke;
+            ResolvedJavaType contextType = methodScope.method.getDeclaringClass();
+            return MethodCallTargetNode.tryDevirtualizeInterfaceCall(callTarget.receiver(), callTarget.targetMethod(), null, graph.getAssumptions(), contextType, callTarget, invoke.asNode());
+        }
+        return callTarget;
+    }
 
+    protected LoopScope trySimplifyInvoke(PEMethodScope methodScope, LoopScope loopScope, InvokeData invokeData, MethodCallTargetNode callTarget) {
         if (tryInvocationPlugin(methodScope, loopScope, invokeData, callTarget)) {
             /*
              * The invocation plugin handled the call, so decoding continues in the calling method.
@@ -775,6 +953,10 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
             if (graphBuilderContext.invokeConsumed) {
                 /* Nothing to do. */
             } else if (graphBuilderContext.lastInstr != null) {
+                if (graphBuilderContext.lastInstr instanceof DeoptBciSupplier && !BytecodeFrame.isPlaceholderBci(invokeData.invoke.bci()) &&
+                                BytecodeFrame.isPlaceholderBci(((DeoptBciSupplier) graphBuilderContext.lastInstr).bci())) {
+                    ((DeoptBciSupplier) graphBuilderContext.lastInstr).setBci(invokeData.invoke.bci());
+                }
                 registerNode(loopScope, invokeData.invokeOrderId, graphBuilderContext.pushedNode, true, true);
                 invoke.asNode().replaceAtUsages(graphBuilderContext.pushedNode);
                 graphBuilderContext.lastInstr.setNext(nodeAfterInvoke(methodScope, loopScope, invokeData, AbstractBeginNode.prevBegin(graphBuilderContext.lastInstr)));
@@ -822,10 +1004,10 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
         for (InlineInvokePlugin plugin : inlineInvokePlugins) {
             InlineInfo inlineInfo = plugin.shouldInlineInvoke(graphBuilderContext, targetMethod, arguments);
             if (inlineInfo != null) {
-                if (inlineInfo.getMethodToInline() == null) {
-                    return null;
-                } else {
+                if (inlineInfo.allowsInlining()) {
                     return doInline(methodScope, loopScope, invokeData, inlineInfo, arguments);
+                } else {
+                    return null;
                 }
             }
         }
@@ -837,9 +1019,7 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
             return null;
         }
         ResolvedJavaMethod inlineMethod = inlineInfo.getMethodToInline();
-        ResolvedJavaMethod originalMethod = inlineInfo.getOriginalMethod();
-        boolean isSubstitution = originalMethod != null && !originalMethod.equals(inlineMethod);
-        EncodedGraph graphToInline = lookupEncodedGraph(inlineMethod, originalMethod, inlineInfo.getIntrinsicBytecodeProvider(), isSubstitution, graph.trackNodeSourcePosition());
+        EncodedGraph graphToInline = lookupEncodedGraph(inlineMethod, inlineInfo.getPlugin(), inlineInfo.getIntrinsicBytecodeProvider(), inlineInfo.isSubstitution(), graph.trackNodeSourcePosition());
         if (graphToInline == null) {
             return null;
         }
@@ -893,33 +1073,8 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
             inlineLoopScope.createdNodes[firstArgumentNodeId + i] = arguments[i];
         }
 
-        // Copy assumptions from inlinee to caller
-        Assumptions assumptions = graph.getAssumptions();
-        Assumptions inlinedAssumptions = graphToInline.getAssumptions();
-        if (assumptions != null) {
-            if (inlinedAssumptions != null) {
-                assumptions.record(inlinedAssumptions);
-            }
-        } else {
-            assert inlinedAssumptions == null : String.format("cannot inline graph (%s) which makes assumptions into a graph (%s) that doesn't", inlineMethod, graph);
-        }
-
         // Copy inlined methods from inlinee to caller
-        List<ResolvedJavaMethod> inlinedMethods = graphToInline.getInlinedMethods();
-        if (inlinedMethods != null) {
-            for (ResolvedJavaMethod other : inlinedMethods) {
-                graph.recordMethod(other);
-            }
-        }
-
-        if (graphToInline.getFields() != null) {
-            for (ResolvedJavaField field : graphToInline.getFields()) {
-                graph.recordField(field);
-            }
-        }
-        if (graphToInline.hasUnsafeAccess()) {
-            graph.markUnsafeAccess();
-        }
+        recordGraphElements(graphToInline);
 
         /*
          * Do the actual inlining by returning the initial loop scope for the inlined method scope.
@@ -978,10 +1133,25 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
                 exceptionValue = InliningUtil.mergeValueProducers(unwindMergeNode, getMatchingNodes(returnAndUnwindNodes, returnNodeCount > 0, UnwindNode.class, unwindNodeCount),
                                 null, unwindNode -> unwindNode.exception());
                 unwindMergeNode.setNext(unwindReplacement);
-
                 ensureExceptionStateDecoded(inlineScope);
                 unwindMergeNode.setStateAfter(inlineScope.exceptionState.duplicateModified(JavaKind.Object, JavaKind.Object, exceptionValue));
             }
+            if (invoke instanceof InvokeWithExceptionNode) {
+                /*
+                 * Exceptionobject nodes are begin nodes, i.e., they can be used as guards/anchors
+                 * thus we need to ensure nodes decoded later have a correct guarding/anchoring node
+                 * in place, i.e., the exception value must be a node that can be used like the
+                 * original node as value, guard and anchor.
+                 *
+                 * The node unwindReplacement is a stub, its not yet processed thus we insert our
+                 * artificial anchor before it.
+                 */
+                assert unwindReplacement != exceptionValue : "Unschedulable unwind replacement";
+                FixedAnchorNode anchor = graph.add(new FixedAnchorNode(exceptionValue));
+                graph.addBeforeFixed(unwindReplacement, anchor);
+                exceptionValue = anchor;
+                assert anchor.predecessor() != null;
+            }
         }
 
         assert invoke.next() == null;
@@ -1017,6 +1187,8 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
         }
         deleteInvoke(invoke);
 
+        assert exceptionValue == null || exceptionValue instanceof FixedAnchorNode && exceptionValue.predecessor() != null;
+
         for (InlineInvokePlugin plugin : inlineInvokePlugins) {
             plugin.notifyAfterInline(inlineMethod);
         }
@@ -1108,23 +1280,9 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
         }
     }
 
-    protected abstract EncodedGraph lookupEncodedGraph(ResolvedJavaMethod method, ResolvedJavaMethod originalMethod, BytecodeProvider intrinsicBytecodeProvider, boolean isSubstitution,
+    protected abstract EncodedGraph lookupEncodedGraph(ResolvedJavaMethod method, MethodSubstitutionPlugin plugin, BytecodeProvider intrinsicBytecodeProvider, boolean isSubstitution,
                     boolean trackNodeSourcePosition);
 
-    @Override
-    protected void handleFixedNode(MethodScope s, LoopScope loopScope, int nodeOrderId, FixedNode node) {
-        PEMethodScope methodScope = (PEMethodScope) s;
-
-        if (node instanceof ForeignCallNode) {
-            ForeignCallNode foreignCall = (ForeignCallNode) node;
-            if (foreignCall.getBci() == BytecodeFrame.UNKNOWN_BCI && methodScope.invokeData != null) {
-                foreignCall.setBci(methodScope.invokeData.invoke.bci());
-            }
-        }
-
-        super.handleFixedNode(methodScope, loopScope, nodeOrderId, node);
-    }
-
     @SuppressWarnings("try")
     @Override
     protected Node canonicalizeFixedNode(MethodScope s, Node node) {
@@ -1231,10 +1389,25 @@ public abstract class PEGraphDecoder extends SimplifyingGraphDecoder {
                 }
             }
         }
+        if (node instanceof PluginReplacementNode) {
+            PluginReplacementNode pluginReplacementNode = (PluginReplacementNode) node;
+            PEPluginGraphBuilderContext graphBuilderContext = new PEPluginGraphBuilderContext(methodScope,
+                            pluginReplacementNode);
+            boolean success = pluginReplacementNode.replace(graphBuilderContext, providers.getReplacements());
+            if (success) {
+                replacedNode = graphBuilderContext.pushedNode;
+            } else if (pluginReplacementMustSucceed()) {
+                throw new GraalError("Plugin failed:" + node);
+            }
+        }
 
         return super.canonicalizeFixedNode(methodScope, replacedNode);
     }
 
+    protected boolean pluginReplacementMustSucceed() {
+        return false;
+    }
+
     @Override
     protected Node handleFloatingNodeBeforeAdd(MethodScope s, LoopScope loopScope, Node n) {
         PEMethodScope methodScope = (PEMethodScope) s;
