@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,20 +25,22 @@
 
 package jdk.javadoc.internal.doclets.formats.html;
 
-import java.util.*;
+import java.util.Set;
 
 import javax.lang.model.element.TypeElement;
 
+import jdk.javadoc.internal.doclets.formats.html.markup.ContentBuilder;
+import jdk.javadoc.internal.doclets.formats.html.markup.Entity;
 import jdk.javadoc.internal.doclets.formats.html.markup.HtmlStyle;
-import jdk.javadoc.internal.doclets.formats.html.markup.HtmlTag;
+import jdk.javadoc.internal.doclets.formats.html.markup.TagName;
 import jdk.javadoc.internal.doclets.formats.html.markup.HtmlTree;
-import jdk.javadoc.internal.doclets.formats.html.markup.Navigation;
-import jdk.javadoc.internal.doclets.formats.html.markup.Navigation.PageMode;
+import jdk.javadoc.internal.doclets.formats.html.Navigation.PageMode;
 import jdk.javadoc.internal.doclets.formats.html.markup.StringContent;
 import jdk.javadoc.internal.doclets.toolkit.Content;
 import jdk.javadoc.internal.doclets.toolkit.SerializedFormWriter;
 import jdk.javadoc.internal.doclets.toolkit.util.DocFileIOException;
 import jdk.javadoc.internal.doclets.toolkit.util.DocPaths;
+import jdk.javadoc.internal.doclets.toolkit.util.IndexItem;
 
 /**
  *  Generates the Serialized Form Information Page, <i>serialized-form.html</i>.
@@ -47,28 +49,19 @@ import jdk.javadoc.internal.doclets.toolkit.util.DocPaths;
  *  If you write code that depends on this, you do so at your own risk.
  *  This code and its internal interfaces are subject to change or
  *  deletion without notice.</b>
- *
- * @author Atul M Dambalkar
  */
 public class SerializedFormWriterImpl extends SubWriterHolderWriter
     implements SerializedFormWriter {
 
     Set<TypeElement> visibleClasses;
 
-    /**
-     * HTML tree for main tag.
-     */
-    private HtmlTree mainTree = HtmlTree.MAIN();
-
-    private final Navigation navBar;
-
     /**
      * @param configuration the configuration data for the doclet
      */
     public SerializedFormWriterImpl(HtmlConfiguration configuration) {
         super(configuration, DocPaths.SERIALIZED_FORM);
         visibleClasses = configuration.getIncludedTypeElements();
-        this.navBar = new Navigation(null, configuration, fixedNavDiv, PageMode.SERIALIZEDFORM, path);
+        configuration.conditionalPages.add(HtmlConfiguration.ConditionalPage.SERIALIZED_FORM);
     }
 
     /**
@@ -77,18 +70,15 @@ public class SerializedFormWriterImpl extends SubWriterHolderWriter
      * @param header the header to write
      * @return the body content tree
      */
+    @Override
     public Content getHeader(String header) {
-        HtmlTree bodyTree = getBody(true, getWindowTitle(header));
-        HtmlTree htmlTree = HtmlTree.HEADER();
-        addTop(htmlTree);
-        navBar.setUserHeader(getUserHeaderFooter(true));
-        htmlTree.add(navBar.getContent(true));
-        bodyTree.add(htmlTree);
+        HtmlTree bodyTree = getBody(getWindowTitle(header));
         Content h1Content = new StringContent(header);
-        Content heading = HtmlTree.HEADING(Headings.PAGE_TITLE_HEADING, true,
+        Content heading = HtmlTree.HEADING_TITLE(Headings.PAGE_TITLE_HEADING,
                 HtmlStyle.title, h1Content);
         Content div = HtmlTree.DIV(HtmlStyle.header, heading);
-        mainTree.add(div);
+        bodyContents.setHeader(getHeader(PageMode.SERIALIZED_FORM))
+                .addMainContent(div);
         return bodyTree;
     }
 
@@ -97,8 +87,9 @@ public class SerializedFormWriterImpl extends SubWriterHolderWriter
      *
      * @return the serialized form summary header tree
      */
+    @Override
     public Content getSerializedSummariesHeader() {
-        HtmlTree ul = new HtmlTree(HtmlTag.UL);
+        HtmlTree ul = new HtmlTree(TagName.UL);
         ul.setStyle(HtmlStyle.blockList);
         return ul;
     }
@@ -108,8 +99,9 @@ public class SerializedFormWriterImpl extends SubWriterHolderWriter
      *
      * @return the package serialized form header tree
      */
+    @Override
     public Content getPackageSerializedHeader() {
-        return HtmlTree.SECTION();
+        return HtmlTree.SECTION(HtmlStyle.serializedPackageContainer);
     }
 
     /**
@@ -118,10 +110,11 @@ public class SerializedFormWriterImpl extends SubWriterHolderWriter
      * @param packageName the package header to write
      * @return a content tree for the package header
      */
+    @Override
     public Content getPackageHeader(String packageName) {
-        Content heading = HtmlTree.HEADING(Headings.SerializedForm.PACKAGE_HEADING, true,
+        Content heading = HtmlTree.HEADING_TITLE(Headings.SerializedForm.PACKAGE_HEADING,
                 contents.packageLabel);
-        heading.add(Contents.SPACE);
+        heading.add(Entity.NO_BREAK_SPACE);
         heading.add(packageName);
         return heading;
     }
@@ -131,8 +124,9 @@ public class SerializedFormWriterImpl extends SubWriterHolderWriter
      *
      * @return a content tree for the serialized class header
      */
+    @Override
     public Content getClassSerializedHeader() {
-        HtmlTree ul = new HtmlTree(HtmlTag.UL);
+        HtmlTree ul = new HtmlTree(TagName.UL);
         ul.setStyle(HtmlStyle.blockList);
         return ul;
     }
@@ -153,13 +147,14 @@ public class SerializedFormWriterImpl extends SubWriterHolderWriter
      * @param typeElement the class being processed
      * @return a content tree for the class header
      */
+    @Override
     public Content getClassHeader(TypeElement typeElement) {
         Content classLink = (isVisibleClass(typeElement))
                 ? getLink(new LinkInfoImpl(configuration, LinkInfoImpl.Kind.DEFAULT, typeElement)
                         .label(configuration.getClassName(typeElement)))
                 : new StringContent(utils.getFullyQualifiedName(typeElement));
-        Content li = HtmlTree.LI(HtmlStyle.blockList, links.createAnchor(
-                utils.getFullyQualifiedName(typeElement)));
+        Content section = HtmlTree.SECTION(HtmlStyle.serializedClassDetails)
+                .setId(utils.getFullyQualifiedName(typeElement));
         Content superClassLink = typeElement.getSuperclass() != null
                 ? getLink(new LinkInfoImpl(configuration, LinkInfoImpl.Kind.SERIALIZED_FORM,
                         typeElement.getSuperclass()))
@@ -172,8 +167,8 @@ public class SerializedFormWriterImpl extends SubWriterHolderWriter
             contents.getContent(
             "doclet.Class_0_extends_implements_serializable", classLink,
             superClassLink);
-        li.add(HtmlTree.HEADING(Headings.SerializedForm.CLASS_HEADING, className));
-        return li;
+        section.add(HtmlTree.HEADING(Headings.SerializedForm.CLASS_HEADING, className));
+        return section;
     }
 
     /**
@@ -181,10 +176,9 @@ public class SerializedFormWriterImpl extends SubWriterHolderWriter
      *
      * @return a content tree for the serial uid info header
      */
+    @Override
     public Content getSerialUIDInfoHeader() {
-        HtmlTree dl = new HtmlTree(HtmlTag.DL);
-        dl.setStyle(HtmlStyle.nameValue);
-        return dl;
+        return HtmlTree.DL(HtmlStyle.nameValue);
     }
 
     /**
@@ -195,8 +189,11 @@ public class SerializedFormWriterImpl extends SubWriterHolderWriter
      * @param serialUidTree the serial UID content tree to which the serial UID
      *                      content will be added
      */
-    public void addSerialUIDInfo(String header, String serialUID,
-            Content serialUidTree) {
+    @Override
+    public void addSerialUIDInfo(String header,
+                                 String serialUID,
+                                 Content serialUidTree)
+    {
         Content headerContent = new StringContent(header);
         serialUidTree.add(HtmlTree.DT(headerContent));
         Content serialContent = new StringContent(serialUID);
@@ -208,52 +205,47 @@ public class SerializedFormWriterImpl extends SubWriterHolderWriter
      *
      * @return a content tree for the class serialize content header
      */
+    @Override
     public Content getClassContentHeader() {
-        HtmlTree ul = new HtmlTree(HtmlTag.UL);
+        HtmlTree ul = new HtmlTree(TagName.UL);
         ul.setStyle(HtmlStyle.blockList);
         return ul;
     }
 
     /**
-     * Get the serialized content tree section.
+     * Add the serialized content tree section.
      *
      * @param serializedTreeContent the serialized content tree to be added
-     * @return a div content tree
      */
-    public Content getSerializedContent(Content serializedTreeContent) {
-        HtmlTree divContent = HtmlTree.DIV(HtmlStyle.serializedFormContainer,
-                serializedTreeContent);
-        mainTree.add(divContent);
-        return mainTree;
+    @Override
+    public void addSerializedContent(Content serializedTreeContent) {
+        bodyContents.addMainContent(serializedTreeContent);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    @Override
     public void addPackageSerializedTree(Content serializedSummariesTree,
-            Content packageSerializedTree) {
-        serializedSummariesTree.add(HtmlTree.LI(HtmlStyle.blockList, packageSerializedTree));
+                                         Content packageSerializedTree)
+    {
+        serializedSummariesTree.add(HtmlTree.LI(packageSerializedTree));
     }
 
     /**
      * Add the footer.
-     *
-     * @param serializedTree the serialized tree to be added
-     */
-    public void addFooter(Content serializedTree) {
-        Content htmlTree = HtmlTree.FOOTER();
-        navBar.setUserFooter(getUserHeaderFooter(false));
-        htmlTree.add(navBar.getContent(false));
-        addBottom(htmlTree);
-        serializedTree.add(htmlTree);
-    }
-
-    /**
-     * {@inheritDoc}
      */
+    @Override
+    public void addFooter() {
+        bodyContents.setFooter(getFooter());
+    }
+
     @Override
     public void printDocument(Content serializedTree) throws DocFileIOException {
+        serializedTree.add(bodyContents);
         printHtmlDocument(null, "serialized forms", serializedTree);
+
+        if (configuration.mainIndex != null) {
+            configuration.mainIndex.add(IndexItem.of(IndexItem.Category.TAGS,
+                    resources.getText("doclet.Serialized_Form"), path));
+        }
     }
 
     /**
@@ -261,6 +253,7 @@ public class SerializedFormWriterImpl extends SubWriterHolderWriter
      *
      * @return an instance of a SerialFieldWriter.
      */
+    @Override
     public SerialFieldWriter getSerialFieldWriter(TypeElement typeElement) {
         return new HtmlSerialFieldWriter(this, typeElement);
     }
@@ -270,6 +263,7 @@ public class SerializedFormWriterImpl extends SubWriterHolderWriter
      *
      * @return an instance of a SerialMethodWriter.
      */
+    @Override
     public SerialMethodWriter getSerialMethodWriter(TypeElement typeElement) {
         return new HtmlSerialMethodWriter(this, typeElement);
     }
