@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -31,7 +31,7 @@ import javax.lang.model.element.Element;
 import javax.lang.model.element.TypeElement;
 import javax.lang.model.element.VariableElement;
 
-import jdk.javadoc.internal.doclets.toolkit.BaseConfiguration;
+import jdk.javadoc.internal.doclets.toolkit.BaseOptions;
 import jdk.javadoc.internal.doclets.toolkit.Content;
 import jdk.javadoc.internal.doclets.toolkit.DocletException;
 import jdk.javadoc.internal.doclets.toolkit.EnumConstantWriter;
@@ -45,9 +45,6 @@ import static jdk.javadoc.internal.doclets.toolkit.util.VisibleMemberTable.Kind.
  *  If you write code that depends on this, you do so at your own risk.
  *  This code and its internal interfaces are subject to change or
  *  deletion without notice.</b>
- *
- * @author Jamie Ho
- * @author Bhavesh Patel (Modified)
  */
 public class EnumConstantBuilder extends AbstractMemberBuilder {
 
@@ -77,7 +74,7 @@ public class EnumConstantBuilder extends AbstractMemberBuilder {
     private EnumConstantBuilder(Context context,
             TypeElement typeElement, EnumConstantWriter writer) {
         super(context, typeElement);
-        this.writer = writer;
+        this.writer = Objects.requireNonNull(writer);
         enumConstants = getVisibleMembers(ENUM_CONSTANTS);
     }
 
@@ -85,7 +82,7 @@ public class EnumConstantBuilder extends AbstractMemberBuilder {
      * Construct a new EnumConstantsBuilder.
      *
      * @param context  the build context.
-     * @param typeElement the class whoses members are being documented.
+     * @param typeElement the class whose members are being documented.
      * @param writer the doclet specific writer.
      * @return the new EnumConstantsBuilder
      */
@@ -104,9 +101,6 @@ public class EnumConstantBuilder extends AbstractMemberBuilder {
         return !enumConstants.isEmpty();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     @Override
     public void build(Content contentTree) throws DocletException {
         buildEnumConstant(contentTree);
@@ -115,32 +109,30 @@ public class EnumConstantBuilder extends AbstractMemberBuilder {
     /**
      * Build the enum constant documentation.
      *
-     * @param memberDetailsTree the content tree to which the documentation will be added
+     * @param detailsList the content tree to which the documentation will be added
      * @throws DocletException is there is a problem while building the documentation
      */
-    protected void buildEnumConstant(Content memberDetailsTree) throws DocletException {
-        if (writer == null) {
-            return;
-        }
+    protected void buildEnumConstant(Content detailsList) throws DocletException {
         if (hasMembersToDocument()) {
             Content enumConstantsDetailsTreeHeader = writer.getEnumConstantsDetailsTreeHeader(typeElement,
-                    memberDetailsTree);
-            Content enumConstantsDetailsTree = writer.getMemberTreeHeader();
+                    detailsList);
+            Content memberList = writer.getMemberList();
 
             for (Element enumConstant : enumConstants) {
                 currentElement = (VariableElement)enumConstant;
                 Content enumConstantsTree = writer.getEnumConstantsTreeHeader(currentElement,
-                        enumConstantsDetailsTree);
+                        memberList);
 
                 buildSignature(enumConstantsTree);
                 buildDeprecationInfo(enumConstantsTree);
                 buildEnumConstantComments(enumConstantsTree);
                 buildTagInfo(enumConstantsTree);
 
-                enumConstantsDetailsTree.add(writer.getEnumConstants(enumConstantsTree));
+                memberList.add(writer.getMemberListItem(enumConstantsTree));
             }
-            memberDetailsTree.add(
-                    writer.getEnumConstantsDetails(enumConstantsDetailsTreeHeader, enumConstantsDetailsTree));
+            Content enumConstantDetails = writer.getEnumConstantsDetails(
+                    enumConstantsDetailsTreeHeader, memberList);
+            detailsList.add(enumConstantDetails);
         }
     }
 
@@ -164,12 +156,12 @@ public class EnumConstantBuilder extends AbstractMemberBuilder {
 
     /**
      * Build the comments for the enum constant.  Do nothing if
-     * {@link BaseConfiguration#nocomment} is set to true.
+     * {@link BaseOptions#noComment()} is set to true.
      *
      * @param enumConstantsTree the content tree to which the documentation will be added
      */
     protected void buildEnumConstantComments(Content enumConstantsTree) {
-        if (!configuration.nocomment) {
+        if (!options.noComment()) {
             writer.addComments(currentElement, enumConstantsTree);
         }
     }
