@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,6 +28,7 @@
 #include "eventHandler.h"
 #include "threadControl.h"
 #include "invoker.h"
+#include "signature.h"
 
 
 #define COMMAND_LOOP_THREAD_NAME "JDWP Event Helper Thread"
@@ -479,7 +480,7 @@ handleFrameEventCommandSingle(JNIEnv* env, PacketOutputStream *out,
     writeCodeLocation(out, command->clazz, command->method, command->location);
     if (command->typeKey) {
         (void)outStream_writeValue(env, out, command->typeKey, command->returnValue);
-        if (isObjectTag(command->typeKey) &&
+        if (isReferenceTag(command->typeKey) &&
             command->returnValue.l != NULL) {
             tossGlobalRef(env, &(command->returnValue.l));
         }
@@ -851,7 +852,7 @@ saveEventInfoRefs(JNIEnv *env, EventInfo *evinfo)
                 saveGlobalRef(env, clazz, pclazz);
             }
             sig = evinfo->u.field_modification.signature_type;
-            if ((sig == JDWP_TAG(ARRAY)) || (sig == JDWP_TAG(OBJECT))) {
+            if (isReferenceTag(sig)) {
                 if ( evinfo->u.field_modification.new_value.l != NULL ) {
                     pobject = &(evinfo->u.field_modification.new_value.l);
                     object = *pobject;
@@ -904,7 +905,7 @@ tossEventInfoRefs(JNIEnv *env, EventInfo *evinfo)
                 tossGlobalRef(env, &(evinfo->u.field_modification.field_clazz));
             }
             sig = evinfo->u.field_modification.signature_type;
-            if ((sig == JDWP_TAG(ARRAY)) || (sig == JDWP_TAG(OBJECT))) {
+            if (isReferenceTag(sig)) {
                 if ( evinfo->u.field_modification.new_value.l != NULL ) {
                     tossGlobalRef(env, &(evinfo->u.field_modification.new_value.l));
                 }
@@ -1108,7 +1109,7 @@ eventHelper_recordFrameEvent(jint id, jbyte suspendPolicy, EventIndex ei,
         /*
          * V or B C D F I J S Z L <classname> ;    [ ComponentType
          */
-        if (isObjectTag(frameCommand->typeKey) &&
+        if (isReferenceTag(frameCommand->typeKey) &&
             returnValue.l != NULL) {
             saveGlobalRef(env, returnValue.l, &(frameCommand->returnValue.l));
         } else {
