@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,14 +25,11 @@
 
 package com.sun.management.internal;
 
-import java.util.concurrent.TimeUnit;
-import java.util.function.DoubleSupplier;
-import java.util.function.LongSupplier;
-import java.util.function.ToDoubleFunction;
-
 import jdk.internal.platform.Metrics;
 import sun.management.BaseOperatingSystemImpl;
 import sun.management.VMManagement;
+
+import java.util.concurrent.TimeUnit;
 /**
  * Implementation class for the operating system.
  * Standard and committed hotspot-specific metrics if any.
@@ -45,138 +42,6 @@ class OperatingSystemImpl extends BaseOperatingSystemImpl
 
     private static final int MAX_ATTEMPTS_NUMBER = 10;
     private final Metrics containerMetrics;
-    private ContainerCpuTicks systemLoadTicks = new SystemCpuTicks();
-    private ContainerCpuTicks processLoadTicks = new ProcessCpuTicks();
-
-    private abstract class ContainerCpuTicks {
-        private long usageTicks = 0;
-        private long totalTicks = 0;
-
-        private double getUsageDividesTotal(long usageTicks, long totalTicks) {
-            // If cpu quota or cpu shares are in effect. Calculate the cpu load
-            // based on the following formula (similar to how
-            // getCpuLoad0() is being calculated):
-            //
-            //   | usageTicks - usageTicks' |
-            //  ------------------------------
-            //   | totalTicks - totalTicks' |
-            //
-            // where usageTicks' and totalTicks' are historical values
-            // retrieved via an earlier call of this method.
-            if (usageTicks < 0 || totalTicks <= 0) {
-                return -1;
-            }
-            long distance = usageTicks - this.usageTicks;
-            this.usageTicks = usageTicks;
-            long totalDistance = totalTicks - this.totalTicks;
-            this.totalTicks = totalTicks;
-            double systemLoad = 0.0;
-            if (distance > 0 && totalDistance > 0) {
-                systemLoad = ((double)distance) / totalDistance;
-            }
-            // Ensure the return value is in the range 0.0 -> 1.0
-            systemLoad = Math.max(0.0, systemLoad);
-            systemLoad = Math.min(1.0, systemLoad);
-            return systemLoad;
-        }
-
-        public double getContainerCpuLoad() {
-            assert(containerMetrics != null);
-            long quota = containerMetrics.getCpuQuota();
-            long share = containerMetrics.getCpuShares();
-            if (quota > 0) {
-                long numPeriods = containerMetrics.getCpuNumPeriods();
-                long quotaNanos = TimeUnit.MICROSECONDS.toNanos(quota * numPeriods);
-                return getUsageDividesTotal(cpuUsageSupplier().getAsLong(), quotaNanos);
-            } else if (share > 0) {
-                long hostTicks = getHostTotalCpuTicks0();
-                int totalCPUs = getHostOnlineCpuCount0();
-                int containerCPUs = getAvailableProcessors();
-                // scale the total host load to the actual container cpus
-                double scaleFactor = ((double) containerCPUs) / totalCPUs;
-                hostTicks = (long) (hostTicks * scaleFactor);
-                return getUsageDividesTotal(cpuUsageSupplier().getAsLong(), hostTicks);
-            } else {
-                // If CPU quotas and shares are not active then find the average load for
-                // all online CPUs that are allowed to run this container.
-
-                // If the cpuset is the same as the host's one there is no need to iterate over each CPU
-                if (isCpuSetSameAsHostCpuSet()) {
-                    return defaultCpuLoadSupplier().getAsDouble();
-                } else {
-                    int[] cpuSet = containerMetrics.getEffectiveCpuSetCpus();
-                    // in case the effectiveCPUSetCpus are not available, attempt to use just cpusets.cpus
-                    if (cpuSet == null || cpuSet.length <= 0) {
-                        cpuSet = containerMetrics.getCpuSetCpus();
-                    }
-                    if (cpuSet == null) {
-                        // cgroups is mounted, but CPU resource is not limited.
-                        // We can assume the VM is run on the host CPUs.
-                        return defaultCpuLoadSupplier().getAsDouble();
-                    } else if (cpuSet.length > 0) {
-                        return cpuSetCalc().applyAsDouble(cpuSet);
-                    }
-                    return -1;
-                }
-            }
-        }
-
-        protected abstract DoubleSupplier defaultCpuLoadSupplier();
-        protected abstract ToDoubleFunction<int[]> cpuSetCalc();
-        protected abstract LongSupplier cpuUsageSupplier();
-    }
-
-    private class ProcessCpuTicks extends ContainerCpuTicks {
-
-        @Override
-        protected DoubleSupplier defaultCpuLoadSupplier() {
-            return () -> getProcessCpuLoad0();
-        }
-
-        @Override
-        protected ToDoubleFunction<int[]> cpuSetCalc() {
-            return (int[] cpuSet) -> {
-                int totalCPUs = getHostOnlineCpuCount0();
-                int containerCPUs = getAvailableProcessors();
-                return Math.min(1.0, getProcessCpuLoad0() * totalCPUs / containerCPUs);
-            };
-        }
-
-        @Override
-        protected LongSupplier cpuUsageSupplier() {
-            return () ->  getProcessCpuTime();
-        }
-
-    }
-
-    private class SystemCpuTicks extends ContainerCpuTicks {
-
-        @Override
-        protected DoubleSupplier defaultCpuLoadSupplier() {
-            return () -> getSystemCpuLoad0();
-        }
-
-        @Override
-        protected ToDoubleFunction<int[]> cpuSetCalc() {
-            return (int[] cpuSet) -> {
-                double systemLoad = 0.0;
-                for (int cpu : cpuSet) {
-                    double cpuLoad = getSingleCpuLoad0(cpu);
-                    if (cpuLoad < 0) {
-                        return -1;
-                    }
-                    systemLoad += cpuLoad;
-                }
-                return systemLoad / cpuSet.length;
-            };
-        }
-
-        @Override
-        protected LongSupplier cpuUsageSupplier() {
-            return () -> containerMetrics.getCpuUsage();
-        }
-
-    }
 
     OperatingSystemImpl(VMManagement vm) {
         super(vm);
@@ -196,8 +61,7 @@ class OperatingSystemImpl extends BaseOperatingSystemImpl
             // it can use as much memory as the host's OS allows.
             long memLimit = containerMetrics.getMemoryLimit();
             if (limit >= 0 && memLimit >= 0) {
-                // we see a limit == 0 on some machines where "kernel does not support swap limit capabilities"
-                return (limit < memLimit) ? 0 : limit - memLimit;
+                return limit - memLimit; // might potentially be 0 for limit == memLimit
             }
         }
         return getTotalSwapSpaceSize0();
@@ -239,7 +103,7 @@ class OperatingSystemImpl extends BaseOperatingSystemImpl
         return getProcessCpuTime0();
     }
 
-    public long getFreePhysicalMemorySize() {
+    public long getFreeMemorySize() {
         if (containerMetrics != null) {
             long usage = containerMetrics.getMemoryUsage();
             long limit = containerMetrics.getMemoryLimit();
@@ -247,17 +111,17 @@ class OperatingSystemImpl extends BaseOperatingSystemImpl
                 return limit - usage;
             }
         }
-        return getFreePhysicalMemorySize0();
+        return getFreeMemorySize0();
     }
 
-    public long getTotalPhysicalMemorySize() {
+    public long getTotalMemorySize() {
         if (containerMetrics != null) {
             long limit = containerMetrics.getMemoryLimit();
             if (limit >= 0) {
                 return limit;
             }
         }
-        return getTotalPhysicalMemorySize0();
+        return getTotalMemorySize0();
     }
 
     public long getOpenFileDescriptorCount() {
@@ -268,17 +132,54 @@ class OperatingSystemImpl extends BaseOperatingSystemImpl
         return getMaxFileDescriptorCount0();
     }
 
-    public double getSystemCpuLoad() {
+    public double getCpuLoad() {
         if (containerMetrics != null) {
-            return systemLoadTicks.getContainerCpuLoad();
+            long quota = containerMetrics.getCpuQuota();
+            if (quota > 0) {
+                long periodLength = containerMetrics.getCpuPeriod();
+                long numPeriods = containerMetrics.getCpuNumPeriods();
+                long usageNanos = containerMetrics.getCpuUsage();
+                if (periodLength > 0 && numPeriods > 0 && usageNanos > 0) {
+                    long elapsedNanos = TimeUnit.MICROSECONDS.toNanos(periodLength * numPeriods);
+                    double systemLoad = (double) usageNanos / elapsedNanos;
+                    // Ensure the return value is in the range 0.0 -> 1.0
+                    systemLoad = Math.max(0.0, systemLoad);
+                    systemLoad = Math.min(1.0, systemLoad);
+                    return systemLoad;
+                }
+                return -1;
+            } else {
+                // If CPU quotas are not active then find the average system load for
+                // all online CPUs that are allowed to run this container.
+
+                // If the cpuset is the same as the host's one there is no need to iterate over each CPU
+                if (isCpuSetSameAsHostCpuSet()) {
+                    return getCpuLoad0();
+                } else {
+                    int[] cpuSet = containerMetrics.getEffectiveCpuSetCpus();
+                    // in case the effectiveCPUSetCpus are not available, attempt to use just cpusets.cpus
+                    if (cpuSet == null || cpuSet.length <= 0) {
+                        cpuSet = containerMetrics.getCpuSetCpus();
+                    }
+                    if (cpuSet != null && cpuSet.length > 0) {
+                        double systemLoad = 0.0;
+                        for (int cpu : cpuSet) {
+                            double cpuLoad = getSingleCpuLoad0(cpu);
+                            if (cpuLoad < 0) {
+                                return -1;
+                            }
+                            systemLoad += cpuLoad;
+                        }
+                        return systemLoad / cpuSet.length;
+                    }
+                    return -1;
+                }
+            }
         }
-        return getSystemCpuLoad0();
+        return getCpuLoad0();
     }
 
     public double getProcessCpuLoad() {
-        if (containerMetrics != null) {
-            return processLoadTicks.getContainerCpuLoad();
-        }
         return getProcessCpuLoad0();
     }
 
@@ -291,20 +192,18 @@ class OperatingSystemImpl extends BaseOperatingSystemImpl
 
     /* native methods */
     private native long getCommittedVirtualMemorySize0();
-    private native long getFreePhysicalMemorySize0();
+    private native long getFreeMemorySize0();
     private native long getFreeSwapSpaceSize0();
     private native long getMaxFileDescriptorCount0();
     private native long getOpenFileDescriptorCount0();
     private native long getProcessCpuTime0();
     private native double getProcessCpuLoad0();
-    private native double getSystemCpuLoad0();
-    private native long getTotalPhysicalMemorySize0();
+    private native double getCpuLoad0();
+    private native long getTotalMemorySize0();
     private native long getTotalSwapSpaceSize0();
     private native double getSingleCpuLoad0(int cpuNum);
     private native int getHostConfiguredCpuCount0();
     private native int getHostOnlineCpuCount0();
-    // CPU ticks since boot in nanoseconds
-    private native long getHostTotalCpuTicks0();
 
     static {
         initialize0();
