@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -57,8 +57,8 @@ TEST_VM(os_windows, reserve_memory_special) {
   }
 
   // set globals to make sure we hit the correct code path
-  FLAG_GUARD(UseLargePagesIndividualAllocation);
-  FLAG_GUARD(UseNUMAInterleaving);
+  AutoSaveRestore<bool> FLAG_GUARD(UseLargePagesIndividualAllocation);
+  AutoSaveRestore<bool> FLAG_GUARD(UseNUMAInterleaving);
   FLAG_SET_CMDLINE(UseLargePagesIndividualAllocation, false);
   FLAG_SET_CMDLINE(UseNUMAInterleaving, false);
 
@@ -135,8 +135,20 @@ static void create_rel_directory_w(const wchar_t* path) {
 static void delete_empty_rel_directory_w(const wchar_t* path) {
   WITH_ABS_PATH(path);
   EXPECT_TRUE(file_exists_w(abs_path)) << "Can't delete directory: \"" << path << "\" does not exists";
-  BOOL result = RemoveDirectoryW(abs_path);
-  EXPECT_TRUE(result) << "Failed to delete directory \"" << path << "\": " << GetLastError();
+  const int retry_count = 20;
+
+  // If the directory cannot be deleted directly, a file in it might be kept
+  // open by a virus scanner. Try a few times, since this should be temporary.
+  for (int i = 0; i <= retry_count; ++i) {
+    BOOL result = RemoveDirectoryW(abs_path);
+
+    if (!result && (i < retry_count)) {
+      Sleep(1);
+    } else {
+      EXPECT_TRUE(result) << "Failed to delete directory \"" << path << "\": " << GetLastError();
+      return;
+    }
+  }
 }
 
 static void create_rel_file_w(const wchar_t* path) {
@@ -316,6 +328,20 @@ static void check_dir(wchar_t* path, bool should_be_empty) {
 static void check_file(wchar_t* path) {
   check_file_impl(path);
 
+  // Check os::same_files at least somewhat.
+  char buf[JVM_MAXPATHLEN];
+
+  if (convert_to_cstring(buf, JVM_MAXPATHLEN, path)) {
+    wchar_t mod[JVM_MAXPATHLEN];
+
+    if (unnormalize_path(mod, JVM_MAXPATHLEN, false, path)) {
+      char mod_c[JVM_MAXPATHLEN];
+      if (convert_to_cstring(mod_c, JVM_MAXPATHLEN, mod)) {
+        EXPECT_EQ(os::same_files(buf, mod_c), true) << "os::same files failed for \\" << path << "\" and \"" << mod_c << "\"";
+      }
+    }
+  }
+
   for (int i = 0; mods_filter != Allow_None && i < mods_per_path; ++i) {
     wchar_t tmp[JVM_MAXPATHLEN];
     if (unnormalize_path(tmp, JVM_MAXPATHLEN, false, path)) {
@@ -453,7 +479,7 @@ static void print_attr_result(wchar_t* format, ...) {
 #define NAME_PART_50 L"01234567890123456789012345678901234567890123456789"
 #define NAME_PART_250 NAME_PART_50 NAME_PART_50 NAME_PART_50 NAME_PART_50 NAME_PART_50
 
-// Test which tries to find out if the os::stat, os::open and os::dir_is_empty methods
+// Test which tries to find out if the os::stat, os::open, os::same_files and os::dir_is_empty methods
 // can handle long path names correctly.
 TEST_VM(os_windows, handle_long_paths) {
   static wchar_t cwd[JVM_MAXPATHLEN];
