@@ -23,8 +23,10 @@
 
 #include "precompiled.hpp"
 #include "runtime/os.hpp"
-#include "unittest.hpp"
+#include "utilities/align.hpp"
 #include "utilities/globalDefinitions.hpp"
+#include <type_traits>
+#include "unittest.hpp"
 
 static ::testing::AssertionResult testPageAddress(
   const char* expected_addr_expr,
@@ -187,3 +189,65 @@ TEST(globalDefinitions, byte_size_in_exact_unit) {
   EXPECT_EQ(M + K + 1, byte_size_in_exact_unit(G + M + K));
 #endif
 }
+
+#define EXPECT_EQ_LOG2(fn, type)                                \
+{                                                               \
+  int limit = sizeof (type) * BitsPerByte;                      \
+  if (std::is_signed<type>::value) {                            \
+    EXPECT_EQ(limit - 1, fn(std::numeric_limits<type>::min())); \
+    EXPECT_EQ(limit - 1, fn((type)-1));                         \
+    limit--;                                                    \
+  }                                                             \
+  {                                                             \
+    /* Test the all-1s bit patterns */                          \
+    type var = 1;                                               \
+    for (int i = 0; i < limit; i++, var = (var << 1) | 1) {     \
+      EXPECT_EQ(i, fn(var));                                    \
+    }                                                           \
+  }                                                             \
+  {                                                             \
+    /* Test the powers of 2 and powers + 1*/                    \
+    type var = 1;                                               \
+    for (int i = 0; i < limit; i++, var <<= 1) {                \
+      EXPECT_EQ(i, fn(var));                                    \
+      EXPECT_EQ(i, fn(var | 1));                                \
+    }                                                           \
+  }                                                             \
+}
+
+TEST(globalDefinitions, log2) {
+  EXPECT_EQ_LOG2(log2_intptr, uintptr_t);
+  EXPECT_EQ_LOG2(log2_intptr, intptr_t);
+  EXPECT_EQ_LOG2(log2_long, julong);
+  EXPECT_EQ_LOG2(log2_int, int);
+  EXPECT_EQ_LOG2(log2_jint, jint);
+  EXPECT_EQ_LOG2(log2_uint, uint);
+  EXPECT_EQ_LOG2(log2_jlong, jlong);
+}
+
+TEST(globalDefinitions, array_size) {
+  const size_t test_size = 10;
+
+  {
+    int test_array[test_size] = {};
+    static_assert(test_size == ARRAY_SIZE(test_array), "must be");
+  }
+
+  {
+    double test_array[test_size] = {};
+    static_assert(test_size == ARRAY_SIZE(test_array), "must be");
+  }
+
+  struct ArrayElt { int x; };
+
+  {
+    ArrayElt test_array[test_size] = {};
+    static_assert(test_size == ARRAY_SIZE(test_array), "must be");
+  }
+
+  {
+    const ArrayElt test_array[] = { {0}, {1}, {2}, {3}, {4}, {5} };
+    static_assert(6 == ARRAY_SIZE(test_array), "must be");
+  }
+
+}
