@@ -30,47 +30,47 @@ do
    flags="-XX:+UnlockExperimentalVMOptions -XX:+EnableJVMCI"
    requireInfo=""
    vmargs=""
-  
+
    if [ "$#" -ge 2 ] ; then
      fName="$1Test.java"; shift
      pName=$1; shift
-     
+
      echo "Generate $fName for $pName ..."
-     
+
      while [ $# -gt 0 ]; do
         case $1 in
            --requires)
              shift
              requireInfo="@requires $1"
              shift
-           ;;           
-           
+           ;;
+
            --timeout)
              shift
              action="$action/timeout=$1"
              shift
            ;;
-           
+
            --vmargs)
              shift
              vmargs="-vmargs $1"
              shift
            ;;
-           
-           * )
+
+           *)
              echo "WARN: unknown option $1 in line \"$TEST_ARGS\" will be ignored"
              shift
-           ;;  
+           ;;
         esac
      done
   else
      echo "ERROR in line \"$TEST_ARGS\""
   fi
-      
-  
+
+
   cat << EOF | sed 's/[[:blank:]]*$//' > $fName
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -94,20 +94,15 @@ do
 
 /*
  * @test
- * @summary
  * @requires vm.jvmci
  * $requireInfo
  * @modules jdk.internal.vm.compiler
  *
  * @library /test/lib /compiler/graalunit /
  *
- * @build compiler.graalunit.common.GraalUnitTestLauncher
- *
- * @run driver jdk.test.lib.FileInstaller ../../ProblemList-graal.txt ExcludeList.txt
- *
- * @run $action $flags compiler.graalunit.common.GraalUnitTestLauncher -prefix $pName -exclude ExcludeList.txt $vmargs
+ * @run $action $flags compiler.graalunit.common.GraalUnitTestLauncher -prefix $pName -exclude \${test.root}/ProblemList-graal.txt $vmargs
  */
- 
+
 /* DO NOT MODIFY THIS FILE. GENERATED BY generateTests.sh */
 EOF
 
