@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -24,6 +24,7 @@
 
 /*
  * @test
+ * @key cgroups
  * @summary Ensure that certain JFR events return correct results for resource values
  *          when run inside Docker container, such as available CPU and memory.
  *          Also make sure that PIDs are based on value provided by container,
@@ -36,6 +37,7 @@
  * @build JfrReporter
  * @run driver TestJFREvents
  */
+import java.util.List;
 import jdk.test.lib.containers.docker.Common;
 import jdk.test.lib.containers.docker.DockerRunOptions;
 import jdk.test.lib.containers.docker.DockerTestUtils;
@@ -46,6 +48,8 @@ import jdk.test.lib.Utils;
 
 public class TestJFREvents {
     private static final String imageName = Common.imageName("jfr-events");
+    private static final String TEST_ENV_VARIABLE = "UNIQUE_VARIABLE_ABC592903XYZ";
+    private static final String TEST_ENV_VALUE = "unique_value_abc592903xyz";
     private static final int availableCPUs = Runtime.getRuntime().availableProcessors();
 
     public static void main(String[] args) throws Exception {
@@ -57,11 +61,6 @@ public class TestJFREvents {
         DockerTestUtils.buildJdkDockerImage(imageName, "Dockerfile-BasicTest", "jdk-docker");
 
         try {
-            // leave one CPU for system and tools, otherwise this test may be unstable
-            int maxNrOfAvailableCpus =  availableCPUs - 1;
-            for (int i=1; i < maxNrOfAvailableCpus; i = i * 2) {
-                testCPUInfo(i, i);
-            }
 
             long MB = 1024*1024;
             testMemory("200m", "" + 200*MB);
@@ -76,18 +75,26 @@ public class TestJFREvents {
         }
     }
 
+    // This test case is currently not in use.
+    // Once new Container events are available, this test case can be used to test
+    // processor-related configuration such as active processor count (see JDK-8203359).
+    private static void cpuTestCase() throws Exception {
+            // leave one CPU for system and tools, otherwise this test may be unstable
+            int maxNrOfAvailableCpus =  availableCPUs - 1;
+            for (int i=1; i < maxNrOfAvailableCpus; i = i * 2) {
+                testCPUInfo("jdk.ContainerConfiguration", i, i);
+            }
+    }
 
-    private static void testCPUInfo(int valueToSet, int expectedValue) throws Exception {
+    private static void testCPUInfo(String eventName, int valueToSet, int expectedValue) throws Exception {
         Common.logNewTestCase("CPUInfo: --cpus = " + valueToSet);
+        String fieldName = "activeProcessorCount";
         DockerTestUtils.dockerRunJava(
                                       commonDockerOpts()
                                       .addDockerOpts("--cpus=" + valueToSet)
-                                      .addClassOptions("jdk.CPUInformation"))
-            .shouldHaveExitValue(0);
-        // The following assertion is currently disabled due to JFR reporting incorrect values.
-        // JFR reports values for the host system as opposed to values for the container.
-        // @ignore 8219999
-        // .shouldContain("cores = " + expectedValue");
+                                      .addClassOptions(eventName))
+            .shouldHaveExitValue(0)
+            .shouldContain(fieldName + " = " + expectedValue);
     }
 
 
@@ -119,26 +126,29 @@ public class TestJFREvents {
     }
 
 
-    // JTReg always defines the environment variable JAVA_MAIN_CLASS_<SOME_NUMBER>.
-    // This variable fits well for use in this test, since it is rather unique.
-    private static String getTestEnvironmentVariable() throws Exception {
-        for (String key : System.getenv().keySet()) {
-            if (key.startsWith("JAVA_MAIN_CLASS")) {
-                return key;
-            }
-        }
-        throw new RuntimeException("JAVA_MAIN_CLASS_* is not defined");
-    }
-
-
     private static void testEnvironmentVariables() throws Exception {
         Common.logNewTestCase("EnvironmentVariables");
 
-        DockerTestUtils.dockerRunJava(
+        List<String> cmd = DockerTestUtils.buildJavaCommand(
                                       commonDockerOpts()
-                                      .addClassOptions("jdk.InitialEnvironmentVariable"))
-            .shouldHaveExitValue(0)
+                                      .addClassOptions("jdk.InitialEnvironmentVariable"));
+
+        ProcessBuilder pb = new ProcessBuilder(cmd);
+        // Container has JAVA_HOME defined via the Dockerfile; make sure
+        // it is reported by JFR event.
+        // Environment variable set in host system should not be visible inside a container,
+        // and should not be reported by JFR.
+        pb.environment().put(TEST_ENV_VARIABLE, TEST_ENV_VALUE);
+
+        System.out.println("[COMMAND]\n" + Utils.getCommandLine(pb));
+        OutputAnalyzer out = new OutputAnalyzer(pb.start());
+        System.out.println("[STDERR]\n" + out.getStderr());
+        System.out.println("[STDOUT]\n" + out.getStdout());
+
+        out.shouldHaveExitValue(0)
             .shouldContain("key = JAVA_HOME")
-            .shouldNotContain(getTestEnvironmentVariable());
+            .shouldContain("value = /jdk")
+            .shouldNotContain(TEST_ENV_VARIABLE)
+            .shouldNotContain(TEST_ENV_VALUE);
     }
 }
