@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,7 +25,6 @@ package gc.arguments;
 
 /*
  * @test TestParallelGCThreads
- * @key gc
  * @bug 8059527 8081382
  * @summary Tests argument processing for ParallelGCThreads
  * @library /test/lib
@@ -33,7 +32,7 @@ package gc.arguments;
  * @modules java.base/jdk.internal.misc
  *          java.management
  * @build sun.hotspot.WhiteBox
- * @run driver ClassFileInstaller sun.hotspot.WhiteBox sun.hotspot.WhiteBox$WhiteBoxPermission
+ * @run driver ClassFileInstaller sun.hotspot.WhiteBox
  * @run main/othervm -Xbootclasspath/a:. -XX:+UnlockDiagnosticVMOptions -XX:+WhiteBoxAPI gc.arguments.TestParallelGCThreads
  */
 
@@ -80,7 +79,7 @@ public class TestParallelGCThreads {
   }
 
   public static void testFlags() throws Exception {
-    // For each parallel collector (G1, Parallel, ParNew/CMS)
+    // For each parallel collector (G1, Parallel)
     List<String> supportedGC = new ArrayList<String>();
 
     if (GC.G1.isSupported()) {
@@ -89,26 +88,28 @@ public class TestParallelGCThreads {
     if (GC.Parallel.isSupported()) {
       supportedGC.add("Parallel");
     }
-    if (GC.ConcMarkSweep.isSupported()) {
-      supportedGC.add("ConcMarkSweep");
-    }
 
     if (supportedGC.isEmpty()) {
-      throw new SkippedException("Skipping test because none of G1/Parallel/ConcMarkSweep is supported.");
+      throw new SkippedException("Skipping test because none of G1/Parallel is supported.");
     }
 
     for (String gc : supportedGC) {
-
       // Make sure the VM does not allow ParallelGCThreads set to 0
-      String[] flags = new String[] {"-XX:+Use" + gc + "GC", "-XX:ParallelGCThreads=0", "-XX:+PrintFlagsFinal", "-version"};
-      ProcessBuilder pb = GCArguments.createJavaProcessBuilder(flags);
+      ProcessBuilder pb = GCArguments.createJavaProcessBuilder(
+          "-XX:+Use" + gc + "GC",
+          "-XX:ParallelGCThreads=0",
+          "-XX:+PrintFlagsFinal",
+          "-version");
       OutputAnalyzer output = new OutputAnalyzer(pb.start());
       output.shouldHaveExitValue(1);
 
       // Do some basic testing to ensure the flag updates the count
       for (long i = 1; i <= 3; i++) {
-        flags = new String[] {"-XX:+Use" + gc + "GC", "-XX:ParallelGCThreads=" + i, "-XX:+PrintFlagsFinal", "-version"};
-        long count = getParallelGCThreadCount(flags);
+        long count = getParallelGCThreadCount(
+            "-XX:+Use" + gc + "GC",
+            "-XX:ParallelGCThreads=" + i,
+            "-XX:+PrintFlagsFinal",
+            "-version");
         Asserts.assertEQ(count, i, "Specifying ParallelGCThreads=" + i + " for " + gc + "GC does not set the thread count properly!");
       }
     }
@@ -117,13 +118,16 @@ public class TestParallelGCThreads {
     // So setting ParallelGCThreads=4294967295 should give back 4294967295
     // and setting ParallelGCThreads=4294967296 should give back 0. (SerialGC is ok with ParallelGCThreads=0)
     for (long i = 4294967295L; i <= 4294967296L; i++) {
-      String[] flags = new String[] {"-XX:+UseSerialGC", "-XX:ParallelGCThreads=" + i, "-XX:+PrintFlagsFinal", "-version"};
-      long count = getParallelGCThreadCount(flags);
+      long count = getParallelGCThreadCount(
+          "-XX:+UseSerialGC",
+          "-XX:ParallelGCThreads=" + i,
+          "-XX:+PrintFlagsFinal",
+          "-version");
       Asserts.assertEQ(count, i % 4294967296L, "Specifying ParallelGCThreads=" + i + " does not set the thread count properly!");
     }
   }
 
-  public static long getParallelGCThreadCount(String flags[]) throws Exception {
+  public static long getParallelGCThreadCount(String... flags) throws Exception {
     ProcessBuilder pb = GCArguments.createJavaProcessBuilder(flags);
     OutputAnalyzer output = new OutputAnalyzer(pb.start());
     output.shouldHaveExitValue(0);
