@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2018, Red Hat, Inc. All rights reserved.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 only, as
@@ -21,12 +22,14 @@
  *
  */
 
-import java.util.concurrent.*;
+import java.util.Random;
+import jdk.test.lib.Utils;
 
 /*
  * @test TestArrayCopyStress
- * @key gc
+ * @key randomness
  * @requires vm.gc.Shenandoah
+ * @library /test/lib
  *
  * @run main/othervm -XX:+UnlockExperimentalVMOptions -XX:+UseShenandoahGC -XX:TieredStopAtLevel=0 -Xmx16m TestArrayCopyStress
  */
@@ -56,10 +59,10 @@ public class TestArrayCopyStress {
         for (int i = 0; i < ARRAY_SIZE; i++) {
             array[i] = new Foo(i);
         }
-
-        int src_idx = ThreadLocalRandom.current().nextInt(0, ARRAY_SIZE);
-        int dst_idx = ThreadLocalRandom.current().nextInt(0, ARRAY_SIZE);
-        int len = ThreadLocalRandom.current().nextInt(0, Math.min(ARRAY_SIZE - src_idx, ARRAY_SIZE - dst_idx));
+        Random rng = Utils.getRandomInstance();
+        int src_idx = rng.nextInt(ARRAY_SIZE);
+        int dst_idx = rng.nextInt(ARRAY_SIZE);
+        int len = rng.nextInt(Math.min(ARRAY_SIZE - src_idx, ARRAY_SIZE - dst_idx));
         System.arraycopy(array, src_idx, array, dst_idx, len);
 
         for (int i = 0; i < ARRAY_SIZE; i++) {
