@@ -40,9 +40,10 @@ import jdk.test.lib.process.OutputAnalyzer;
 public class TestLoopMiningArguments {
 
     public static void testWith(String msg, boolean cls, int iters, String... args) throws Exception {
-        String[] cmds = Arrays.copyOf(args, args.length + 2);
-        cmds[args.length] = "-XX:+PrintFlagsFinal";
-        cmds[args.length + 1] = "-version";
+        String[] cmds = Arrays.copyOf(args, args.length + 3);
+        cmds[args.length] = "-Xmx128m";
+        cmds[args.length + 1] = "-XX:+PrintFlagsFinal";
+        cmds[args.length + 2] = "-version";
         ProcessBuilder pb = ProcessTools.createJavaProcessBuilder(cmds);
         OutputAnalyzer output = new OutputAnalyzer(pb.start());
         output.shouldHaveExitValue(0);
