@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -37,6 +37,7 @@ import java.io.File;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
@@ -45,7 +46,7 @@ public class GTestWrapper {
     public static void main(String[] args) throws Throwable {
         // gtestLauncher is located in <test_image>/hotspot/gtest/<vm_variant>/
         // nativePath points either to <test_image>/hotspot/jtreg/native or to <test_image>/hotspot/gtest
-        Path nativePath = Paths.get(System.getProperty("test.nativepath"));
+        Path nativePath = Paths.get(Utils.TEST_NATIVE_PATH);
         String jvmVariantDir = getJVMVariantSubDir();
         // let's assume it's <test_image>/hotspot/gtest
         Path path = nativePath.resolve(jvmVariantDir);
@@ -57,7 +58,7 @@ public class GTestWrapper {
                              .resolve(jvmVariantDir);
         }
         if (!path.toFile().exists()) {
-            throw new Error("TESTBUG: the library has not been found in " + nativePath);
+            throw new Error("TESTBUG: the library has not been found in " + nativePath + ". Did you forget to use --with-gtest to configure?");
         }
 
         Path execPath = path.resolve("gtestLauncher" + (Platform.isWindows() ? ".exe" : ""));
@@ -75,9 +76,16 @@ public class GTestWrapper {
         }
 
         Path resultFile = Paths.get("test_result.xml");
-        pb.command(execPath.toAbsolutePath().toString(),
-                "-jdk", Utils.TEST_JDK,
-                "--gtest_output=xml:" + resultFile);
+
+        ArrayList<String> command = new ArrayList<>();
+        command.add(execPath.toAbsolutePath().toString());
+        command.add("-jdk");
+        command.add(Utils.TEST_JDK);
+        command.add("--gtest_output=xml:" + resultFile);
+        for (String a : args) {
+            command.add(a);
+        }
+        pb.command(command);
         int exitCode = ProcessTools.executeCommand(pb).getExitValue();
         if (exitCode != 0) {
             List<String> failedTests = failedTests(resultFile);
@@ -110,8 +118,10 @@ public class GTestWrapper {
             return "client";
         } else if (Platform.isMinimal()) {
             return "minimal";
+        } else if (Platform.isZero()) {
+            return "zero";
         } else {
-            throw new Error("TESTBUG: unsuppported vm variant");
+            throw new Error("TESTBUG: unsupported vm variant");
         }
     }
 }
