@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -68,7 +68,7 @@ import cdsutils.DynamicDumpHelper;
  *    prefix + opts + suffix
  */
 public class TestCommon extends CDSTestUtils {
-    private static final String JSA_FILE_PREFIX = System.getProperty("user.dir") +
+    private static final String JSA_FILE_PREFIX = CDSTestUtils.getOutputDir() +
         File.separator;
 
     private static final SimpleDateFormat timeStampFormat =
@@ -115,7 +115,7 @@ public class TestCommon extends CDSTestUtils {
     // to the file; in such cases the File.delete() operation will silently fail, w/o
     // throwing an exception, thus allowing testing to continue.
     public static void deletePriorArchives() {
-        File dir = new File(System.getProperty("user.dir"));
+        File dir = CDSTestUtils.getOutputDirAsFile();
         String files[] = dir.list();
         for (String name : files) {
             if (name.startsWith("appcds-") && name.endsWith(".jsa")) {
@@ -137,6 +137,22 @@ public class TestCommon extends CDSTestUtils {
         return createArchive(appJarDir, appJar, classList, suffix);
     }
 
+    /**
+     * Dump the base archive. The JDK's default class list is used (unless otherwise specified
+     * in cmdLineSuffix).
+     */
+    public static OutputAnalyzer dumpBaseArchive(String baseArchiveName, String ... cmdLineSuffix)
+        throws Exception
+    {
+        CDSOptions opts = new CDSOptions();
+        opts.setArchiveName(baseArchiveName);
+        opts.addSuffix(cmdLineSuffix);
+        opts.addSuffix("-Djava.class.path=");
+        OutputAnalyzer out = CDSTestUtils.createArchive(opts);
+        CDSTestUtils.checkBaseDump(out);
+        return out;
+    }
+
     // Create AppCDS archive using most common args - convenience method
     public static OutputAnalyzer createArchive(String appJar, String classList[],
                                                String... suffix) throws Exception {
@@ -158,6 +174,9 @@ public class TestCommon extends CDSTestUtils {
     // Simulate -Xshare:dump with -XX:ArchiveClassesAtExit. See comments around patchJarForDynamicDump()
     private static final Class tmp = DynamicDumpHelper.class;
 
+    // name of the base archive to be used for dynamic dump
+    private static String tempBaseArchive = null;
+
     // Create AppCDS archive using appcds options
     public static OutputAnalyzer createArchive(AppCDSOptions opts)
         throws Exception {
@@ -182,9 +201,14 @@ public class TestCommon extends CDSTestUtils {
         }
 
         if (DYNAMIC_DUMP) {
+            File baseArchive = null;
+            if (tempBaseArchive == null || !(new File(tempBaseArchive)).isFile()) {
+                tempBaseArchive = getNewArchiveName("tempBaseArchive");
+                dumpBaseArchive(tempBaseArchive);
+            }
             cmd.add("-Xshare:on");
+            cmd.add("-XX:SharedArchiveFile=" + tempBaseArchive);
             cmd.add("-XX:ArchiveClassesAtExit=" + opts.archiveName);
-
             cmd.add("-Xlog:cds");
             cmd.add("-Xlog:cds+dynamic");
             boolean mainModuleSpecified = false;
@@ -226,6 +250,7 @@ public class TestCommon extends CDSTestUtils {
         } else {
             // static dump
             cmd.add("-Xshare:dump");
+            cmd.add("-Xlog:cds");
             cmd.add("-XX:SharedArchiveFile=" + opts.archiveName);
 
             if (opts.classList != null) {
@@ -237,8 +262,7 @@ public class TestCommon extends CDSTestUtils {
             }
         }
 
-        String[] cmdLine = cmd.toArray(new String[cmd.size()]);
-        ProcessBuilder pb = ProcessTools.createJavaProcessBuilder(true, cmdLine);
+        ProcessBuilder pb = ProcessTools.createTestJvm(cmd);
         if (opts.appJarDir != null) {
             pb.directory(new File(opts.appJarDir));
         }
@@ -297,12 +321,11 @@ public class TestCommon extends CDSTestUtils {
             firstJar = firstJar.substring(0, n);
         }
         String classDir = System.getProperty("test.classes");
-        String expected1 = classDir + File.separator;
-        String expected2 = System.getProperty("user.dir") + File.separator;
+        String expected = getOutputDir() + File.separator;
 
-        if (!firstJar.startsWith(expected1) && !firstJar.startsWith(expected2)) {
+        if (!firstJar.startsWith(expected)) {
             throw new RuntimeException("FIXME: jar file not at a supported location ('"
-                                       + expected1 + "', or '" + expected2 + "'): " + firstJar);
+                                       + expected + "'): " + firstJar);
         }
 
         String replaceJar = firstJar + ".tmp";
@@ -384,8 +407,7 @@ public class TestCommon extends CDSTestUtils {
             }
         }
 
-        String[] cmdLine = cmd.toArray(new String[cmd.size()]);
-        ProcessBuilder pb = ProcessTools.createJavaProcessBuilder(true, cmdLine);
+        ProcessBuilder pb = ProcessTools.createTestJvm(cmd);
         if (opts.appJarDir != null) {
             pb.directory(new File(opts.appJarDir));
         }
