@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,6 +25,7 @@
  * @test
  * @library /test/lib
  * @summary Test that redefinition reuses metaspace blocks that are freed
+ * @requires vm.jvmti
  * @modules java.base/jdk.internal.misc
  * @modules java.instrument
  *          jdk.jartool/sun.tools.jar
@@ -101,12 +102,14 @@ public class RedefineLeak {
         }
         if (argv.length == 1 && argv[0].equals("runtest")) {
             // run outside of jtreg to not OOM on jtreg classes that are loaded after metaspace is full
-            String[] javaArgs1 = { "-XX:MetaspaceSize=12m", "-XX:MaxMetaspaceSize=12m",
-                                   "-javaagent:redefineagent.jar", "RedefineLeak"};
-            ProcessBuilder pb = ProcessTools.createJavaProcessBuilder(javaArgs1);
-
+            ProcessBuilder pb = ProcessTools.createJavaProcessBuilder(
+                    "-XX:MetaspaceSize=12m",
+                    "-XX:MaxMetaspaceSize=12m",
+                    "-javaagent:redefineagent.jar",
+                    "RedefineLeak");
             OutputAnalyzer output = new OutputAnalyzer(pb.start());
             output.shouldContain("transformCount:10000");
+            output.shouldHaveExitValue(0);
         }
     }
 }
