@@ -62,15 +62,17 @@
  */
 
 import sun.hotspot.code.Compiler;
+import sun.hotspot.WhiteBox;
+import sun.hotspot.gc.GC;
 
 public class TestDriver {
     public static void main(String[] args) throws Exception {
-        sun.hotspot.WhiteBox wb = sun.hotspot.WhiteBox.getWhiteBox();
+        WhiteBox wb = WhiteBox.getWhiteBox();
         Boolean isExplicitGCInvokesConcurrentOn = wb.getBooleanVMFlag("ExplicitGCInvokesConcurrent");
-        Boolean isUseG1GCon = wb.getBooleanVMFlag("UseG1GC");
-        Boolean isUseZGCon = wb.getBooleanVMFlag("UseZGC");
-        Boolean isShenandoahGCon = wb.getBooleanVMFlag("UseShenandoahGC");
-        Boolean isUseEpsilonGCon = wb.getBooleanVMFlag("UseEpsilonGC");
+        boolean isUseG1GCon = GC.G1.isSelected();
+        boolean isUseZGCon = GC.Z.isSelected();
+        boolean isShenandoahGCon = GC.Shenandoah.isSelected();
+        boolean isUseEpsilonGCon = GC.Epsilon.isSelected();
 
         if (Compiler.isGraalEnabled() &&
             (isUseZGCon || isUseEpsilonGCon || isShenandoahGCon)) {
@@ -87,8 +89,8 @@ public class TestDriver {
         nsk.jvmti.scenarios.general_functions.GF08.gf08t.main(new String[] {
                 "gf08t001",
                 nsk.jvmti.scenarios.general_functions.GF08.gf08t001.class.getName(),
-                keyPhrase,
-                "gc"});
+                "gc",
+                keyPhrase});
     }
 }
 
