@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -49,8 +49,6 @@ import com.sun.net.httpserver.HttpsConfigurator;
 import com.sun.net.httpserver.HttpsServer;
 import jdk.test.lib.net.SimpleSSLContext;
 import org.testng.ITestContext;
-import org.testng.ITestResult;
-import org.testng.SkipException;
 import org.testng.annotations.AfterClass;
 import org.testng.annotations.AfterTest;
 import org.testng.annotations.BeforeMethod;
@@ -74,7 +72,6 @@ import java.net.http.HttpRequest;
 import java.net.http.HttpResponse;
 import java.net.http.HttpResponse.BodyHandlers;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.List;
 import java.util.Optional;
 import java.util.concurrent.ConcurrentHashMap;
@@ -83,7 +80,6 @@ import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Executor;
 import java.util.concurrent.Executors;
 import java.util.concurrent.atomic.AtomicLong;
-import java.util.concurrent.atomic.AtomicReference;
 
 import static java.lang.System.err;
 import static java.lang.System.out;
@@ -152,37 +148,17 @@ public class ForbiddenHeadTest implements HttpServerAdapters {
         return Boolean.getBoolean("jdk.internal.httpclient.debug");
     }
 
-    final AtomicReference<SkipException> skiptests = new AtomicReference<>();
-    void checkSkip() {
-        var skip = skiptests.get();
-        if (skip != null) throw skip;
-    }
-    static String name(ITestResult result) {
-        var params = result.getParameters();
-        return result.getName()
-                + (params == null ? "()" : Arrays.toString(result.getParameters()));
-    }
-
     @BeforeMethod
     void beforeMethod(ITestContext context) {
         if (stopAfterFirstFailure() && context.getFailedTests().size() > 0) {
-            if (skiptests.get() == null) {
-                SkipException skip = new SkipException("some tests failed");
-                skip.setStackTrace(new StackTraceElement[0]);
-                skiptests.compareAndSet(null, skip);
-            }
+            throw new RuntimeException("some tests failed");
         }
     }
 
     @AfterClass
-    static final void printFailedTests(ITestContext context) {
+    static final void printFailedTests() {
         out.println("\n=========================");
         try {
-            // Exceptions should already have been added to FAILURES
-            // var failed = context.getFailedTests().getAllResults().stream()
-            //        .collect(Collectors.toMap(r -> name(r), ITestResult::getThrowable));
-            // FAILURES.putAll(failed);
-
             out.printf("%n%sCreated %d servers and %d clients%n",
                     now(), serverCount.get(), clientCount.get());
             if (FAILURES.isEmpty()) return;
@@ -210,18 +186,18 @@ public class ForbiddenHeadTest implements HttpServerAdapters {
     @DataProvider(name = "all")
     public Object[][] allcases() {
         List<Object[]> result = new ArrayList<>();
-        for (boolean useAuth : List.of(true, false)) {
+        for (var client : List.of(authClient, noAuthClient)) {
             for (boolean async : List.of(true, false)) {
                 for (int code : List.of(UNAUTHORIZED, PROXY_UNAUTHORIZED)) {
                     var srv = code == PROXY_UNAUTHORIZED ? "/proxy" : "/server";
                     for (var auth : List.of("/auth", "/noauth")) {
                         var pcode = code;
                         if (auth.equals("/noauth")) {
-                            if (useAuth) continue;
+                            if (client == authClient) continue;
                             pcode = FORBIDDEN;
                         }
                         for (var uri : List.of(httpURI, httpsURI, http2URI, https2URI)) {
-                            result.add(new Object[]{uri + srv + auth, pcode, async, useAuth});
+                            result.add(new Object[]{uri + srv + auth, pcode, async, client});
                         }
                     }
                 }
@@ -242,13 +218,11 @@ public class ForbiddenHeadTest implements HttpServerAdapters {
     static final AtomicLong sleepCount = new AtomicLong();
 
     @Test(dataProvider = "all")
-    void test(String uriString, int code, boolean async, boolean useAuth) throws Throwable {
-        checkSkip();
-        HttpClient client = useAuth ? authClient : noAuthClient;
+    void test(String uriString, int code, boolean async, HttpClient client) throws Throwable {
         var name = String.format("test(%s, %d, %s, %s)", uriString, code, async ? "async" : "sync",
                 client.authenticator().isPresent() ? "authClient" : "noAuthClient");
         out.printf("%n---- starting %s ----%n", name);
-        assert client.authenticator().isPresent() == useAuth;
+        assert client.authenticator().isPresent() ? client == authClient : client == noAuthClient;
         uriString = uriString + "/ForbiddenTest";
         for (int i=0; i<ITERATIONS; i++) {
             if (ITERATIONS > 1) out.printf("---- ITERATION %d%n",i);
@@ -266,11 +240,11 @@ public class ForbiddenHeadTest implements HttpServerAdapters {
     }
 
     static String authHeaderName(int code) {
-        switch (code) {
-            case UNAUTHORIZED: return "WWW-Authenticate";
-            case PROXY_UNAUTHORIZED: return "Proxy-Authenticate";
-            default: return null;
-        }
+        return switch (code) {
+            case UNAUTHORIZED -> "WWW-Authenticate";
+            case PROXY_UNAUTHORIZED -> "Proxy-Authenticate";
+            default -> null;
+        };
     }
 
     private void doTest(String uriString, int code, boolean async, HttpClient client) throws Throwable {
@@ -390,17 +364,14 @@ public class ForbiddenHeadTest implements HttpServerAdapters {
     public void teardown() throws Exception {
         authClient = noAuthClient = null;
         Thread.sleep(100);
-        AssertionError fail = TRACKER.check(1500);
-        try {
-            proxy.stop();
-            authproxy.stop();
-            httpTestServer.stop();
-            httpsTestServer.stop();
-            http2TestServer.stop();
-            https2TestServer.stop();
-        } finally {
-            if (fail != null) throw fail;
-        }
+        AssertionError fail = TRACKER.check(500);
+
+        proxy.stop();
+        authproxy.stop();
+        httpTestServer.stop();
+        httpsTestServer.stop();
+        http2TestServer.stop();
+        https2TestServer.stop();
     }
 
     static class TestProxySelector extends ProxySelector {
