@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -50,8 +50,6 @@ import com.sun.net.httpserver.HttpsConfigurator;
 import com.sun.net.httpserver.HttpsServer;
 import jdk.test.lib.net.SimpleSSLContext;
 import org.testng.ITestContext;
-import org.testng.ITestResult;
-import org.testng.SkipException;
 import org.testng.annotations.AfterClass;
 import org.testng.annotations.AfterTest;
 import org.testng.annotations.BeforeMethod;
@@ -72,7 +70,6 @@ import java.net.http.HttpClient;
 import java.net.http.HttpRequest;
 import java.net.http.HttpResponse;
 import java.net.http.HttpResponse.BodyHandlers;
-import java.util.Arrays;
 import java.util.List;
 import java.util.Optional;
 import java.util.concurrent.ConcurrentHashMap;
@@ -81,8 +78,6 @@ import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Executor;
 import java.util.concurrent.Executors;
 import java.util.concurrent.atomic.AtomicLong;
-import java.util.concurrent.atomic.AtomicReference;
-import java.util.stream.Collectors;
 
 import static java.lang.System.err;
 import static java.lang.System.out;
@@ -153,25 +148,10 @@ public class ProxySelectorTest implements HttpServerAdapters {
         return Boolean.getBoolean("jdk.internal.httpclient.debug");
     }
 
-    final AtomicReference<SkipException> skiptests = new AtomicReference<>();
-    void checkSkip() {
-        var skip = skiptests.get();
-        if (skip != null) throw skip;
-    }
-    static String name(ITestResult result) {
-        var params = result.getParameters();
-        return result.getName()
-                + (params == null ? "()" : Arrays.toString(result.getParameters()));
-    }
-
     @BeforeMethod
     void beforeMethod(ITestContext context) {
         if (stopAfterFirstFailure() && context.getFailedTests().size() > 0) {
-            if (skiptests.get() == null) {
-                SkipException skip = new SkipException("some tests failed");
-                skip.setStackTrace(new StackTraceElement[0]);
-                skiptests.compareAndSet(null, skip);
-            }
+            throw new RuntimeException("some tests failed");
         }
     }
 
@@ -179,11 +159,6 @@ public class ProxySelectorTest implements HttpServerAdapters {
     static final void printFailedTests() {
         out.println("\n=========================");
         try {
-            // Exceptions should already have been added to FAILURES
-            // var failed = context.getFailedTests().getAllResults().stream()
-            //        .collect(Collectors.toMap(r -> name(r), ITestResult::getThrowable));
-            // FAILURES.putAll(failed);
-
             out.printf("%n%sCreated %d servers and %d clients%n",
                     now(), serverCount.get(), clientCount.get());
             if (FAILURES.isEmpty()) return;
@@ -235,7 +210,6 @@ public class ProxySelectorTest implements HttpServerAdapters {
     void test(Schemes scheme, HttpClient.Version version, String uri, boolean async)
             throws Throwable
     {
-        checkSkip();
         var name = String.format("test(%s, %s, %s)", scheme, version, async);
         out.printf("%n---- starting %s ----%n", name);
 
@@ -386,18 +360,15 @@ public class ProxySelectorTest implements HttpServerAdapters {
         client = null;
         Thread.sleep(100);
         AssertionError fail = TRACKER.check(500);
-        try {
-            proxy.stop();
-            authproxy.stop();
-            httpTestServer.stop();
-            proxyHttpTestServer.stop();
-            authProxyHttpTestServer.stop();
-            httpsTestServer.stop();
-            http2TestServer.stop();
-            https2TestServer.stop();
-        } finally {
-            if (fail != null) throw fail;
-        }
+
+        proxy.stop();
+        authproxy.stop();
+        httpTestServer.stop();
+        proxyHttpTestServer.stop();
+        authProxyHttpTestServer.stop();
+        httpsTestServer.stop();
+        http2TestServer.stop();
+        https2TestServer.stop();
     }
 
     class TestProxySelector extends ProxySelector {
