@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,6 +28,7 @@ package jdk.jfr.event.runtime;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
+import java.lang.reflect.Method;
 import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
 import java.util.List;
@@ -39,11 +40,16 @@ import jdk.test.lib.Asserts;
 import jdk.test.lib.jfr.EventNames;
 import jdk.test.lib.jfr.Events;
 
+import jdk.test.lib.compiler.InMemoryJavaCompiler;
+
 /**
  * @test
  * @key jfr
  * @requires vm.hasJFR
  * @library /test/lib /test/jdk
+ * @modules java.base/jdk.internal.misc
+ *          jdk.compiler
+ *          jdk.jfr
  * @build jdk.jfr.event.runtime.TestClasses
  * @run main/othervm jdk.jfr.event.runtime.TestClassLoaderStatsEvent
  */
@@ -73,12 +79,13 @@ public class TestClassLoaderStatsEvent {
             if (CLASSLOADER_TYPE_NAME.equals(recordedClassLoader.getType().getName())) {
                 Asserts.assertEquals(CLASS_LOADER_NAME, recordedClassLoader.getName(),
                     "Expected class loader name " + CLASS_LOADER_NAME + ", got name " + recordedClassLoader.getName());
-                Events.assertField(event, "classCount").equal(1L);
+                Events.assertField(event, "classCount").equal(2L);
                 Events.assertField(event, "chunkSize").above(1L);
                 Events.assertField(event, "blockSize").above(1L);
-                Events.assertField(event, "unsafeAnonymousClassCount").equal(1L);
-                Events.assertField(event, "unsafeAnonymousChunkSize").above(1L);
-                Events.assertField(event, "unsafeAnonymousBlockSize").above(1L);
+                // Hidden classes stats include both hidden and unsafe anonymous classes.
+                Events.assertField(event, "hiddenClassCount").equal(4L);
+                Events.assertField(event, "hiddenChunkSize").above(0L);
+                Events.assertField(event, "hiddenBlockSize").above(0L);
                 isAnyFound = true;
             }
         }
@@ -91,6 +98,18 @@ public class TestClassLoaderStatsEvent {
         if (c.getClassLoader() != dummyloader) {
             throw new RuntimeException("TestClass defined by wrong classloader: " + c.getClassLoader());
         }
+
+        // Compile a class for method createNonFindableClasses() to use to create both a
+        // weak hidden class and an anonymous class.
+        byte klassbuf[] = InMemoryJavaCompiler.compile("jdk.jfr.event.runtime.TestClass",
+            "package jdk.jfr.event.runtime; " +
+            "public class TestClass { " +
+            "    public static void concat(String one, String two) throws Throwable { " +
+            " } } ");
+
+        Method m = c.getDeclaredMethod("createNonFindableClasses", byte[].class);
+        m.setAccessible(true);
+        m.invoke(null, klassbuf);
     }
 
     public static class DummyClassLoader extends ClassLoader {
