@@ -23,7 +23,7 @@
 
 /*
  * @test
- * @bug 8076190 8242151
+ * @bug 8076190 8242151 8153005
  * @library /test/lib
  * @modules java.base/sun.security.pkcs
  *          java.base/sun.security.util
@@ -41,6 +41,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.UncheckedIOException;
+import java.nio.file.DirectoryStream;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.security.KeyStore;
@@ -48,8 +49,7 @@ import java.util.Base64;
 import java.util.Objects;
 
 import static jdk.test.lib.security.DerUtils.*;
-import sun.security.util.ObjectIdentifier;
-import sun.security.util.KnownOIDs;
+import static sun.security.util.KnownOIDs.*;
 import static sun.security.pkcs.ContentInfo.*;
 
 public class ParamsTest  {
@@ -57,15 +57,18 @@ public class ParamsTest  {
     public static void main(String[] args) throws Throwable {
 
         // De-BASE64 textual files in ./params to `pwd`
-        Files.newDirectoryStream(Path.of(System.getProperty("test.src"), "params"))
-                .forEach(p -> {
-                    try (InputStream is = Files.newInputStream(p);
-                         OutputStream os = Files.newOutputStream(p.getFileName())){
-                        Base64.getMimeDecoder().wrap(is).transferTo(os);
-                    } catch (IOException e) {
-                        throw new UncheckedIOException(e);
-                    }
-                });
+        try (DirectoryStream<Path> stream = Files.newDirectoryStream(
+                Path.of(System.getProperty("test.src"), "params"),
+                p -> !p.getFileName().toString().equals("README"))) {
+            stream.forEach(p -> {
+                try (InputStream is = Files.newInputStream(p);
+                     OutputStream os = Files.newOutputStream(p.getFileName())) {
+                    Base64.getMimeDecoder().wrap(is).transferTo(os);
+                } catch (IOException e) {
+                    throw new UncheckedIOException(e);
+                }
+            });
+        }
 
         byte[] data;
 
@@ -100,34 +103,44 @@ public class ParamsTest  {
         // Current default pkcs12 setting
         keytool("-importkeystore -srckeystore ks -srcstorepass changeit "
                 + "-destkeystore ksnormal -deststorepass changeit");
-        data = Files.readAllBytes(Path.of("ksnormal"));
-        checkInt(data, "22", 100000); // Mac ic
-        checkAlg(data, "2000", oid(KnownOIDs.SHA_1)); // Mac alg
-        checkAlg(data, "110c010c01000", oid(KnownOIDs.PBEWithSHA1AndDESede)); // key alg
-        checkInt(data, "110c010c010011", 50000); // key ic
-        checkAlg(data, "110c10", ENCRYPTED_DATA_OID);
-        checkAlg(data, "110c110110", oid(KnownOIDs.PBEWithSHA1AndRC2_40)); // cert alg
-        checkInt(data, "110c1101111", 50000); // cert ic
 
+        data = Files.readAllBytes(Path.of("ksnormal"));
+        checkInt(data, "22", 10000); // Mac ic
+        checkAlg(data, "2000", SHA_256); // Mac alg
+        checkAlg(data, "110c010c01000", PBES2); // key alg
+        checkInt(data, "110c010c01001011", 10000); // key ic
+        checkAlg(data, "110c10", ENCRYPTED_DATA_OID);
+        checkAlg(data, "110c110110", PBES2); // cert alg
         check("ksnormal", "a", "changeit", "changeit", true, true, true);
         check("ksnormal", "a", null, "changeit", true, false, true);
         check("ksnormal", "a", "wrongpass", "-", IOException.class, "-", "-");
 
+        // Import it into a new keystore with legacy algorithms
+        keytool("-importkeystore -srckeystore ksnormal -srcstorepass changeit "
+                + "-destkeystore kslegacyimp -deststorepass changeit "
+                + "-J-Dkeystore.pkcs12.legacy");
+        data = Files.readAllBytes(Path.of("kslegacyimp"));
+        checkInt(data, "22", 100000); // Mac ic
+        checkAlg(data, "2000", SHA_1); // Mac alg
+        checkAlg(data, "110c010c01000", PBEWithSHA1AndDESede); // key alg
+        checkInt(data, "110c010c010011", 50000); // key ic
+        checkAlg(data, "110c110110", PBEWithSHA1AndRC2_40); // cert alg
+        checkInt(data, "110c1101111", 50000); // cert ic
+
         // Add a new entry with password-less settings, still has a storepass
         keytool("-keystore ksnormal -genkeypair -keyalg DSA "
                 + "-storepass changeit -alias b -dname CN=b "
                 + "-J-Dkeystore.pkcs12.certProtectionAlgorithm=NONE "
                 + "-J-Dkeystore.pkcs12.macAlgorithm=NONE");
         data = Files.readAllBytes(Path.of("ksnormal"));
-        checkInt(data, "22", 100000); // Mac ic
-        checkAlg(data, "2000", oid(KnownOIDs.SHA_1)); // Mac alg
-        checkAlg(data, "110c010c01000", oid(KnownOIDs.PBEWithSHA1AndDESede)); // key alg
-        checkInt(data, "110c010c010011", 50000); // key ic
-        checkAlg(data, "110c010c11000", oid(KnownOIDs.PBEWithSHA1AndDESede)); // new key alg
-        checkInt(data, "110c010c110011", 50000); // new key ic
+        checkInt(data, "22", 10000); // Mac ic
+        checkAlg(data, "2000", SHA_256); // Mac alg
+        checkAlg(data, "110c010c01000", PBES2); // key alg
+        checkInt(data, "110c010c01001011", 10000); // key ic
+        checkAlg(data, "110c010c11000", PBES2); // new key alg
+        checkInt(data, "110c010c11001011", 10000); // new key ic
         checkAlg(data, "110c10", ENCRYPTED_DATA_OID);
-        checkAlg(data, "110c110110", oid(KnownOIDs.PBEWithSHA1AndRC2_40)); // cert alg
-        checkInt(data, "110c1101111", 50000); // cert ic
+        checkAlg(data, "110c110110", PBES2); // cert alg
         check("ksnormal", "b", null, "changeit", true, false, true);
         check("ksnormal", "b", "changeit", "changeit", true, true, true);
 
@@ -139,8 +152,8 @@ public class ParamsTest  {
                 + "-J-Dkeystore.pkcs12.macAlgorithm=NONE");
         data = Files.readAllBytes(Path.of("ksnopass"));
         shouldNotExist(data, "2"); // no Mac
-        checkAlg(data, "110c010c01000", oid(KnownOIDs.PBEWithSHA1AndRC4_128));
-        checkInt(data, "110c010c010011", 50000);
+        checkAlg(data, "110c010c01000", PBEWithSHA1AndRC4_128);
+        checkInt(data, "110c010c010011", 10000);
         checkAlg(data, "110c10", DATA_OID);
         check("ksnopass", "a", null, "changeit", true, true, true);
         check("ksnopass", "a", "changeit", "changeit", true, true, true);
@@ -151,10 +164,10 @@ public class ParamsTest  {
                 + "-storepass changeit -alias b -dname CN=B");
         data = Files.readAllBytes(Path.of("ksnopass"));
         shouldNotExist(data, "2"); // no Mac
-        checkAlg(data, "110c010c01000", oid(KnownOIDs.PBEWithSHA1AndRC4_128));
-        checkInt(data, "110c010c010011", 50000);
-        checkAlg(data, "110c010c11000", oid(KnownOIDs.PBEWithSHA1AndDESede));
-        checkInt(data, "110c010c110011", 50000);
+        checkAlg(data, "110c010c01000", PBEWithSHA1AndRC4_128);
+        checkInt(data, "110c010c010011", 10000);
+        checkAlg(data, "110c010c11000", PBES2);
+        checkInt(data, "110c010c11001011", 10000);
         checkAlg(data, "110c10", DATA_OID);
         check("ksnopass", "a", null, "changeit", true, true, true);
         check("ksnopass", "b", null, "changeit", true, true, true);
@@ -166,11 +179,11 @@ public class ParamsTest  {
                 + "-J-Dkeystore.pkcs12.keyPbeIterationCount=7777");
         data = Files.readAllBytes(Path.of("ksnewic"));
         checkInt(data, "22", 5555); // Mac ic
-        checkAlg(data, "2000", oid(KnownOIDs.SHA_1)); // Mac alg
-        checkAlg(data, "110c010c01000", oid(KnownOIDs.PBEWithSHA1AndDESede)); // key alg
-        checkInt(data, "110c010c010011", 7777); // key ic
-        checkAlg(data, "110c110110", oid(KnownOIDs.PBEWithSHA1AndRC2_40)); // cert alg
-        checkInt(data, "110c1101111", 6666); // cert ic
+        checkAlg(data, "2000", SHA_256); // Mac alg
+        checkAlg(data, "110c010c01000", PBES2); // key alg
+        checkInt(data, "110c010c01001011", 7777); // key ic
+        checkAlg(data, "110c110110", PBES2); // cert alg
+        checkInt(data, "110c110111011", 6666); // cert ic
 
         // keypbe alg cannot be NONE
         keytool("-keystore ksnewic -genkeypair -keyalg DSA "
@@ -185,13 +198,13 @@ public class ParamsTest  {
                 + "-J-Dkeystore.pkcs12.keyProtectionAlgorithm=PBEWithSHA1AndRC4_128");
         data = Files.readAllBytes(Path.of("ksnewic"));
         checkInt(data, "22", 5555); // Mac ic
-        checkAlg(data, "2000", oid(KnownOIDs.SHA_1)); // Mac alg
-        checkAlg(data, "110c010c01000", oid(KnownOIDs.PBEWithSHA1AndDESede)); // key alg
-        checkInt(data, "110c010c010011", 7777); // key ic
-        checkAlg(data, "110c010c11000", oid(KnownOIDs.PBEWithSHA1AndRC4_128)); // new key alg
-        checkInt(data, "110c010c110011", 50000); // new key ic
-        checkAlg(data, "110c110110", oid(KnownOIDs.PBEWithSHA1AndRC2_40)); // cert alg
-        checkInt(data, "110c1101111", 6666); // cert ic
+        checkAlg(data, "2000", SHA_256); // Mac alg
+        checkAlg(data, "110c010c01000", PBES2); // key alg
+        checkInt(data, "110c010c01001011", 7777); // key ic
+        checkAlg(data, "110c010c11000", PBEWithSHA1AndRC4_128); // new key alg
+        checkInt(data, "110c010c110011", 10000); // new key ic
+        checkAlg(data, "110c110110", PBES2); // cert alg
+        checkInt(data, "110c110111011", 6666); // cert ic
 
         // Check KeyStore loading multiple keystores
         KeyStore ks = KeyStore.getInstance("pkcs12");
@@ -201,15 +214,15 @@ public class ParamsTest  {
             ks.store(fos, "changeit".toCharArray());
         }
         data = Files.readAllBytes(Path.of("ksnormaldup"));
-        checkInt(data, "22", 100000); // Mac ic
-        checkAlg(data, "2000", oid(KnownOIDs.SHA_1)); // Mac alg
-        checkAlg(data, "110c010c01000", oid(KnownOIDs.PBEWithSHA1AndDESede)); // key alg
-        checkInt(data, "110c010c010011", 50000); // key ic
-        checkAlg(data, "110c010c11000", oid(KnownOIDs.PBEWithSHA1AndDESede)); // new key alg
-        checkInt(data, "110c010c110011", 50000); // new key ic
+        checkInt(data, "22", 10000); // Mac ic
+        checkAlg(data, "2000", SHA_256); // Mac alg
+        checkAlg(data, "110c010c01000", PBES2); // key alg
+        checkInt(data, "110c010c01001011", 10000); // key ic
+        checkAlg(data, "110c010c11000", PBES2); // new key alg
+        checkInt(data, "110c010c11001011", 10000); // new key ic
         checkAlg(data, "110c10", ENCRYPTED_DATA_OID);
-        checkAlg(data, "110c110110", oid(KnownOIDs.PBEWithSHA1AndRC2_40)); // cert alg
-        checkInt(data, "110c1101111", 50000); // cert ic
+        checkAlg(data, "110c110110", PBES2); // cert alg
+        checkInt(data, "110c110111011", 10000); // cert ic
 
         try (FileInputStream fis = new FileInputStream("ksnopass");
              FileOutputStream fos = new FileOutputStream("ksnopassdup")) {
@@ -218,10 +231,10 @@ public class ParamsTest  {
         }
         data = Files.readAllBytes(Path.of("ksnopassdup"));
         shouldNotExist(data, "2"); // no Mac
-        checkAlg(data, "110c010c01000", oid(KnownOIDs.PBEWithSHA1AndRC4_128));
-        checkInt(data, "110c010c010011", 50000);
-        checkAlg(data, "110c010c11000", oid(KnownOIDs.PBEWithSHA1AndDESede));
-        checkInt(data, "110c010c110011", 50000);
+        checkAlg(data, "110c010c01000", PBEWithSHA1AndRC4_128);
+        checkInt(data, "110c010c010011", 10000);
+        checkAlg(data, "110c010c11000", PBES2);
+        checkInt(data, "110c010c11001011", 10000);
         checkAlg(data, "110c10", DATA_OID);
 
         try (FileInputStream fis = new FileInputStream("ksnewic");
@@ -231,13 +244,13 @@ public class ParamsTest  {
         }
         data = Files.readAllBytes(Path.of("ksnewicdup"));
         checkInt(data, "22", 5555); // Mac ic
-        checkAlg(data, "2000", oid(KnownOIDs.SHA_1)); // Mac alg
-        checkAlg(data, "110c010c01000", oid(KnownOIDs.PBEWithSHA1AndDESede)); // key alg
-        checkInt(data, "110c010c010011", 7777); // key ic
-        checkAlg(data, "110c010c11000", oid(KnownOIDs.PBEWithSHA1AndRC4_128)); // new key alg
-        checkInt(data, "110c010c110011", 50000); // new key ic
-        checkAlg(data, "110c110110", oid(KnownOIDs.PBEWithSHA1AndRC2_40)); // cert alg
-        checkInt(data, "110c1101111", 6666); // cert ic
+        checkAlg(data, "2000", SHA_256); // Mac alg
+        checkAlg(data, "110c010c01000", PBES2); // key alg
+        checkInt(data, "110c010c01001011", 7777); // key ic
+        checkAlg(data, "110c010c11000", PBEWithSHA1AndRC4_128); // new key alg
+        checkInt(data, "110c010c110011", 10000); // new key ic
+        checkAlg(data, "110c110110", PBES2); // cert alg
+        checkInt(data, "110c110111011", 6666); // cert ic
 
         // Check keytool behavior
 
@@ -434,10 +447,6 @@ public class ParamsTest  {
         Asserts.assertEQ(expectedKey, actualKey, label + "-key");
     }
 
-    private static ObjectIdentifier oid(KnownOIDs o) {
-        return ObjectIdentifier.of(o);
-    }
-
     static OutputAnalyzer keytool(String s) throws Throwable {
         return SecurityTools.keytool(s);
     }
