@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -119,22 +119,6 @@ public abstract class RSAKeyPairGenerator extends KeyPairGeneratorSpi {
 
         // Pre-calculated p and q for e == RSAKeyGenParameterSpec.F4
         switch (keySize) {
-            case 512:
-                p = new BigInteger("8020197120481219323433199611941970" +
-                        "7343202627066030554371666273840736868057027");
-
-                q = new BigInteger("1034972075609195648369724560234440" +
-                        "70295399787770567842730763283716564384306019");
-                break;
-            case 768:
-                p = new BigInteger("3014340057405340213534944319905083" +
-                        "307463736669377067548946180566257416015519309" +
-                        "4284652308891881268862122066025991763");
-
-                q = new BigInteger("2296482383752927761663381890127318" +
-                        "246354728737890494883686642246681037847109357" +
-                        "2243186228256258231783150577032535521");
-                break;
             case 1024:
                 p = new BigInteger("1220491537800192366196661816910427"
                         + "2375185130493819649338056226264568132442590"
@@ -246,90 +230,6 @@ public abstract class RSAKeyPairGenerator extends KeyPairGeneratorSpi {
                         + "5341584429803583589276956979963609078497238"
                         + "760757619468018224491053");
                 break;
-            case 5120:
-                p = new BigInteger("3832424741858359087085019371467455" +
-                        "477820253497664907405863412466091484202728975" +
-                        "073716151647542449749677812957312366114168136" +
-                        "601126462176029012452318767677007998900979141" +
-                        "998941420505000455608776648534383479255437038" +
-                        "823585008944551642245975099397794466252853815" +
-                        "203014018988068346182132859181656248769392421" +
-                        "674329339264774434516306264105498304974256178" +
-                        "026250495864309250770172644263271598862641423" +
-                        "631834585580910907527816840629665632700476957" +
-                        "043059773597106256672930878739201142231424811" +
-                        "069114153595397879660681041405350927706894635" +
-                        "213653200929550087449740488813888141436925212" +
-                        "442118365403614623169477625363286007075561882" +
-                        "667104018888315477880055907473210438614579462" +
-                        "496680112778003198202297772073923181825706950" +
-                        "337756420826065243712623977225389265073815099" +
-                        "11493486387569031");
-
-                q = new BigInteger("4146920568848191633539637174274014" +
-                        "273474016268898436739861681496795368277264211" +
-                        "347843247126933726314115942365962498640261418" +
-                        "205807625467516350689984100345264883329988411" +
-                        "779552170850267866351632396757500932855514738" +
-                        "069918613980688598735820287717773558232108195" +
-                        "582003010318600589422737510968348435291394630" +
-                        "302856598706075108993749783272560317909096664" +
-                        "669346475063020551195971583247945204761628694" +
-                        "928169578936651221139603528255637489949563762" +
-                        "679333707194876390341639312682559633604823162" +
-                        "011775857712214779083777623361101169846144970" +
-                        "895460110829130520744350827300192454548398367" +
-                        "308247776459836067815323028595970305077016546" +
-                        "572729562552659889374473339319508744706500295" +
-                        "871005194700994677334996554312946263536447080" +
-                        "506480369599180714418422948431890690584559846" +
-                        "74443760221378817");
-                break;
-            case 6144:
-                p = new BigInteger("4116778174071734089881578561570208" +
-                        "284278483208207611885263908178511953898663059" +
-                        "734920127853932340647715543357847246656399270" +
-                        "584421304381161243387321196180504266791989629" +
-                        "037762533723859313828066684108658077147663308" +
-                        "349610753424620992228874944364889251603796733" +
-                        "651690820777521473807836887101380451450340893" +
-                        "342182241046213770274098088066973392388380321" +
-                        "597140066726105421813589290486721393630060991" +
-                        "762786515266517210263824648882183969111257670" +
-                        "482546599227987866663009035297583354975557124" +
-                        "885067375690161316321537733738836231124022045" +
-                        "342294989201664821765828714777945420537586846" +
-                        "048441021936371397157175119512813296245432265" +
-                        "178387966306100222317125784086707249132094899" +
-                        "996583841504000177878867233453992573758531314" +
-                        "930995997084778704191508142386227640558575022" +
-                        "133615865313158902645671060856578407704407774" +
-                        "541333134066077091323113676552873589627627108" +
-                        "360212137049807676300972916221155242551418611" +
-                        "845736012720292530472299883528545969");
-
-                q = new BigInteger("4309839393870191337347865939376524" +
-                        "576203152274153819273872971875048034824772989" +
-                        "396672731779116057590220960038178464518939532" +
-                        "420543642549157490693480032819877624270041000" +
-                        "978557189754354533164374250973267430850549095" +
-                        "826504764380972668230808804187197733712467810" +
-                        "336183971273199424298548393174475795085431856" +
-                        "013835093176229700961388036392126193800974224" +
-                        "471901942503594710577458613014825799700253747" +
-                        "894059557606488589338390076087557449706202211" +
-                        "490152794976124198853288767811370937925352946" +
-                        "840084185059748423462857292189804953877120675" +
-                        "812342626896129968757865091819031550949977046" +
-                        "088481307609680346652447103811683263607048407" +
-                        "341877388569162602224005230757828237769174382" +
-                        "379831366827054223659952996908127997873168014" +
-                        "060581963520521443425267958786876837371303659" +
-                        "489061589736155453144095545148190985660891326" +
-                        "576261326549991792303912873247304434509759694" +
-                        "274088584815015779154382635174538158301861164" +
-                        "439921681110768794043391613853281233");
-                break;
             case 7680:
                 p = new BigInteger("7034022146817764608206409206476311"
                         + "1371065855827199565170055133179419153145313"
