@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,7 +25,6 @@
  * @test
  * @bug      4905786 6259611 8162363 8196202
  * @summary  Make sure that headings use the TH tag instead of the TD tag.
- * @author   jamieh
  * @library ../../lib
  * @modules jdk.javadoc/jdk.javadoc.internal.tool
  * @build    javadoc.tester.*
@@ -46,6 +45,7 @@ public class TestHeadings extends JavadocTester {
     public void test() {
         javadoc("-d", "out",
                 "-sourcepath", testSrc,
+                "--no-platform-links",
                 "-use",
                 "-header", "Test Files",
                 "pkg1", "pkg2");
@@ -53,43 +53,48 @@ public class TestHeadings extends JavadocTester {
 
         //Package summary
         checkOutput("pkg1/package-summary.html", true,
-                "<th class=\"colFirst\" scope=\"col\">"
-                + "Class</th>\n"
-                + "<th class=\"colLast\" scope=\"col\""
-                + ">Description</th>");
+                """
+                    <div class="table-header col-first">Class</div>
+                    <div class="table-header col-last">Description</div>""");
 
         // Class documentation
         checkOutput("pkg1/C1.html", true,
-                "<th class=\"colFirst\" scope=\"col\">Modifier and Type</th>\n"
-                + "<th class=\"colSecond\" scope=\"col\">Field</th>\n"
-                + "<th class=\"colLast\" scope=\"col\">Description</th>",
-                "<h3>Methods inherited from class&nbsp;java.lang.Object</h3>");
+                """
+                    <div class="table-header col-first">Modifier and Type</div>
+                    <div class="table-header col-second">Field</div>
+                    <div class="table-header col-last">Description</div>""",
+                """
+                    <h3 id="methods.inherited.from.class.java.lang.Object">Methods inherited from class&nbsp;java.lang.Object</h3>""");
 
         // Class use documentation
         checkOutput("pkg1/class-use/C1.html", true,
-                "<th class=\"colFirst\" scope=\"col\">Package</th>\n"
-                + "<th class=\"colLast\" scope=\"col\">Description</th>",
-                "<th class=\"colFirst\" scope=\"col\">Modifier and Type</th>\n"
-                + "<th class=\"colSecond\" scope=\"col\">Field</th>\n"
-                + "<th class=\"colLast\" scope=\"col\">Description</th>");
+                """
+                    <div class="table-header col-first">Package</div>
+                    <div class="table-header col-last">Description</div>""",
+                """
+                    <div class="table-header col-first">Modifier and Type</div>
+                    <div class="table-header col-second">Field</div>
+                    <div class="table-header col-last">Description</div>""");
 
         // Deprecated
         checkOutput("deprecated-list.html", true,
-                "<th class=\"colFirst\" scope=\"col\">Method</th>\n"
-                + "<th class=\"colLast\" scope=\"col\">Description</th>");
+                """
+                    <div class="table-header col-first">Method</div>
+                    <div class="table-header col-last">Description</div>""");
 
         // Constant values
         checkOutput("constant-values.html", true,
-                "<th class=\"colFirst\" scope=\"col\">"
-                + "Modifier and Type</th>\n"
-                + "<th class=\"colSecond\" scope=\"col\">Constant Field</th>\n"
-                + "<th class=\"colLast\" scope=\"col\">Value</th>");
+                """
+                    <div class="table-header col-first">Modifier and Type</div>
+                    <div class="table-header col-second">Constant Field</div>
+                    <div class="table-header col-last">Value</div>""");
 
         // Serialized Form
         checkOutput("serialized-form.html", true,
-                "<h2 title=\"Package\">Package&nbsp;pkg1</h2>",
-                "<h3>Class <a href=\"pkg1/C1.html\" title=\"class in pkg1\">"
-                + "pkg1.C1</a> extends java.lang.Object implements Serializable</h3>",
+                """
+                    <h2 title="Package">Package&nbsp;pkg1</h2>""",
+                """
+                    <h3>Class <a href="pkg1/C1.html" title="class in pkg1">pkg1.C1</a> extends java.lang.Object implements Serializable</h3>""",
                 "<h4>Serialized Fields</h4>");
 
         // Overview Summary
