@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -44,17 +44,16 @@ public class TestOptions extends JavadocTester {
     }
 
     @Test
-    public void testHeaderFooter() {
+    public void testHeader() {
         javadoc("-d", "out-1",
                 "-header", "Test header",
-                "-footer", "Test footer",
                 "-sourcepath", testSrc,
                 "pkg");
         checkExit(Exit.OK);
 
         checkOutput("pkg/package-summary.html", true,
-                "<div class=\"aboutLanguage\">Test header</div>",
-                "<div class=\"aboutLanguage\">Test footer</div>");
+                """
+                    <div class="about-language">Test header</div>""");
     }
 
     @Test
@@ -103,7 +102,8 @@ public class TestOptions extends JavadocTester {
 
         checkFiles(false, "overview-tree.html");
         checkFiles(false, "pkg/package-tree.html");
-        checkOutput("pkg/Foo.html", false, "<li><a href=\"package-tree.html\">Tree</a></li>");
+        checkOutput("pkg/Foo.html", false, """
+            <li><a href="package-tree.html">Tree</a></li>""");
     }
 
     @Test
@@ -115,8 +115,8 @@ public class TestOptions extends JavadocTester {
         checkExit(Exit.OK);
 
         checkOutput("custom-stylesheet.css", true, "Custom javadoc style sheet");
-        checkOutput("pkg/Foo.html", true, "<link rel=\"stylesheet\" type=\"text/css\" "
-                + "href=\"../custom-stylesheet.css\" title=\"Style\">");
+        checkOutput("pkg/Foo.html", true, """
+            <link rel="stylesheet" type="text/css" href="../custom-stylesheet.css" title="Style">""");
     }
 
     @Test
@@ -128,8 +128,8 @@ public class TestOptions extends JavadocTester {
         checkExit(Exit.OK);
 
         checkOutput("custom-stylesheet.css", true, "Custom javadoc style sheet");
-        checkOutput("pkg/Foo.html", true, "<link rel=\"stylesheet\" type=\"text/css\" "
-                + "href=\"../custom-stylesheet.css\" title=\"Style\">");
+        checkOutput("pkg/Foo.html", true, """
+            <link rel="stylesheet" type="text/css" href="../custom-stylesheet.css" title="Style">""");
     }
 
     @Test
@@ -146,9 +146,10 @@ public class TestOptions extends JavadocTester {
         checkOutput("additional-stylesheet-2.css", true, "Additional javadoc style sheet 2");
         checkOutput("additional-stylesheet-3.css", true, "Additional javadoc style sheet 3");
         checkOutput("pkg/Foo.html", true,
-                "<link rel=\"stylesheet\" type=\"text/css\" href=\"../additional-stylesheet-1.css\" title=\"Style\">\n"
-                + "<link rel=\"stylesheet\" type=\"text/css\" href=\"../additional-stylesheet-2.css\" title=\"Style\">\n"
-                + "<link rel=\"stylesheet\" type=\"text/css\" href=\"../additional-stylesheet-3.css\" title=\"Style\">");
+                """
+                    <link rel="stylesheet" type="text/css" href="../additional-stylesheet-1.css" title="Style">
+                    <link rel="stylesheet" type="text/css" href="../additional-stylesheet-2.css" title="Style">
+                    <link rel="stylesheet" type="text/css" href="../additional-stylesheet-3.css" title="Style">""");
     }
 
     @Test
@@ -181,6 +182,7 @@ public class TestOptions extends JavadocTester {
     public void testLinkSource() {
         javadoc("-d", "out-9",
                 "-linksource",
+                "--no-platform-links",
                 "-javafx",
                 "--disable-javafx-strict-checks",
                 "-sourcepath", testSrc,
@@ -189,72 +191,91 @@ public class TestOptions extends JavadocTester {
         checkExit(Exit.OK);
         checkLinks();
         checkOutput("linksource/AnnotationTypeField.html", true,
-                "<pre>@Documented\npublic @interface <a href="
-                + "\"../src-html/linksource/AnnotationTypeField.html#line.31\">"
-                + "AnnotationTypeField</a></pre>",
-                "<h3><a id=\"DEFAULT_NAME\">DEFAULT_NAME</a></h3>\n"
-                + "<div class=\"memberSignature\"><span class=\"modifiers\">static final</span>&nbsp;"
-                + "<span class=\"returnType\">java.lang.String</span>&nbsp;<span class=\"memberName\">"
-                + "<a href=\"../src-html/linksource/AnnotationTypeField.html#line.32\">DEFAULT_NAME</a></span></div>",
-                "<h3><a id=\"name()\">name</a></h3>\n"
-                + "<div class=\"memberSignature\"><span class=\"returnType\">java.lang.String</span>&nbsp;"
-                + "<span class=\"memberName\"><a href=\"../src-html/linksource/AnnotationTypeField.html#line.34\">"
-                + "name</a></span></div>");
+                """
+                    <div class="type-signature"><span class="annotations">@Documented
+                    </span><span class="modifiers">public @interface </span><span class="element-name"><a hr\
+                    ef="../src-html/linksource/AnnotationTypeField.html#line.31">AnnotationTypeField\
+                    </a></span></div>""",
+                """
+                    <section class="detail" id="DEFAULT_NAME">
+                    <h3>DEFAULT_NAME</h3>
+                    <div class="member-signature"><span class="modifiers">static final</span>&nbsp;<\
+                    span class="return-type">java.lang.String</span>&nbsp;<span class="element-name"><a href\
+                    ="../src-html/linksource/AnnotationTypeField.html#line.32">DEFAULT_NAME</a></spa\
+                    n></div>""",
+                """
+                    <section class="detail" id="name()">
+                    <h3>name</h3>
+                    <div class="member-signature"><span class="return-type">java.lang.String</span>&\
+                    nbsp;<span class="element-name"><a href="../src-html/linksource/AnnotationTypeField.html\
+                    #line.34">name</a></span></div>""");
 
         checkOutput("src-html/linksource/AnnotationTypeField.html", true,
                 "<title>Source code</title>",
-                "<span class=\"sourceLineNo\">031</span><a id=\"line.31\">"
-                + "@Documented public @interface AnnotationTypeField {</a>");
+                """
+                    <span class="source-line-no">031</span><span id="line.31">@Documented public @interface AnnotationTypeField {</span>""");
 
         checkOutput("linksource/Properties.html", true,
-                "<pre>public class <a href=\"../src-html/linksource/Properties.html#line.29\">"
-                + "Properties</a>",
-                "<div class=\"memberSignature\"><span class=\"modifiers\">public</span>&nbsp;"
-                + "<span class=\"returnType\">java.lang.Object</span>&nbsp;<span class=\"memberName\">"
-                + "<a href=\"../src-html/linksource/Properties.html#line.31\">someProperty</a></span></div>");
+                """
+                    <div class="type-signature"><span class="modifiers">public class </span><span cl\
+                    ass="element-name"><a href="../src-html/linksource/Properties.html#line.29">Properties</a>""",
+                """
+                    <div class="member-signature"><span class="modifiers">public</span>&nbsp;<span c\
+                    lass="return-type">java.lang.Object</span>&nbsp;<span class="element-name"><a href="../s\
+                    rc-html/linksource/Properties.html#line.31">someProperty</a></span></div>""");
 
         checkOutput("src-html/linksource/Properties.html", true,
                 "<title>Source code</title>",
-                "<span class=\"sourceLineNo\">031</span><a id=\"line.31\">    "
-                + "public Object someProperty() {</a>");
+                """
+                    <span class="source-line-no">031</span><span id="line.31">    public Object someProperty() {</span>""");
 
         checkOutput("linksource/SomeClass.html", true,
-                "<pre>public class <a href=\"../src-html/linksource/SomeClass.html#line.29\">"
-                + "SomeClass</a>\nextends java.lang.Object</pre>",
-                "<div class=\"memberSignature\"><span class=\"modifiers\">public</span>&nbsp;"
-                + "<span class=\"returnType\">int</span>&nbsp;<span class=\"memberName\">"
-                + "<a href=\"../src-html/linksource/SomeClass.html#line.31\">field</a></span></div>",
-                "<div class=\"memberSignature\"><span class=\"modifiers\">public</span>&nbsp;"
-                + "<span class=\"memberName\"><a href=\"../src-html/linksource/SomeClass.html#line.33\">"
-                + "SomeClass</a></span>()</div>",
-                "<div class=\"memberSignature\"><span class=\"modifiers\">public</span>&nbsp;"
-                + "<span class=\"returnType\">int</span>&nbsp;<span class=\"memberName\">"
-                + "<a href=\"../src-html/linksource/SomeClass.html#line.36\">method</a></span>()</div>");
+                """
+                    <div class="type-signature"><span class="modifiers">public class </span><span cl\
+                    ass="element-name"><a href="../src-html/linksource/SomeClass.html#line.29">SomeC\
+                    lass</a></span>
+                    <span class="extends-implements">extends java.lang.Object</span></div>""",
+                """
+                    <div class="member-signature"><span class="modifiers">public</span>&nbsp;<span c\
+                    lass="return-type">int</span>&nbsp;<span class="element-name"><a href="../src-html/links\
+                    ource/SomeClass.html#line.31">field</a></span></div>""",
+                """
+                    <div class="member-signature"><span class="modifiers">public</span>&nbsp;<span c\
+                    lass="element-name"><a href="../src-html/linksource/SomeClass.html#line.33">Some\
+                    Class</a></span>()</div>""",
+                """
+                    <div class="member-signature"><span class="modifiers">public</span>&nbsp;<span c\
+                    lass="return-type">int</span>&nbsp;<span class="element-name"><a href="../src-html/links\
+                    ource/SomeClass.html#line.36">method</a></span>()</div>""");
 
         checkOutput("src-html/linksource/SomeClass.html", true,
                 "<title>Source code</title>",
-                "<span class=\"sourceLineNo\">029</span><a id=\"line.29\">"
-                + "public class SomeClass {</a>",
-                "<span class=\"sourceLineNo\">031</span><a id=\"line.31\">    "
-                + "public int field;</a>",
-                "<span class=\"sourceLineNo\">033</span><a id=\"line.33\">    "
-                + "public SomeClass() {</a>",
-                "<span class=\"sourceLineNo\">036</span><a id=\"line.36\">    "
-                + "public int method() {</a>");
+                """
+                    <span class="source-line-no">029</span><span id="line.29">public class SomeClass {</span>""",
+                """
+                    <span class="source-line-no">031</span><span id="line.31">    public int field;</span>""",
+                """
+                    <span class="source-line-no">033</span><span id="line.33">    public SomeClass() {</span>""",
+                """
+                    <span class="source-line-no">036</span><span id="line.36">    public int method() {</span>""");
 
         checkOutput("linksource/SomeEnum.html", true,
-                "<div class=\"memberSignature\"><span class=\"modifiers\">public static final</span>&nbsp;"
-                + "<span class=\"returnType\"><a href=\"SomeEnum.html\" title=\"enum in linksource\">"
-                + "SomeEnum</a></span>&nbsp;<span class=\"memberName\">"
-                + "<a href=\"../src-html/linksource/SomeEnum.html#line.29\">VALUE1</a></span></div>",
-                "<div class=\"memberSignature\"><span class=\"modifiers\">public static final</span>&nbsp;"
-                + "<span class=\"returnType\"><a href=\"SomeEnum.html\" title=\"enum in linksource\">"
-                + "SomeEnum</a></span>&nbsp;<span class=\"memberName\">"
-                + "<a href=\"../src-html/linksource/SomeEnum.html#line.30\">VALUE2</a></span></div>");
+                """
+                    <div class="member-signature"><span class="modifiers">public static final</span>\
+                    &nbsp;<span class="return-type"><a href="SomeEnum.html" title="enum class in linksourc\
+                    e">SomeEnum</a></span>&nbsp;<span class="element-name"><a href="../src-html/linksource/S\
+                    omeEnum.html#line.29">VALUE1</a></span></div>""",
+                """
+                    <div class="member-signature"><span class="modifiers">public static final</span>\
+                    &nbsp;<span class="return-type"><a href="SomeEnum.html" title="enum class in linksourc\
+                    e">SomeEnum</a></span>&nbsp;<span class="element-name"><a href="../src-html/linksource/S\
+                    omeEnum.html#line.30">VALUE2</a></span></div>""");
 
         checkOutput("src-html/linksource/SomeEnum.html", true,
-                "<span class=\"sourceLineNo\">029</span><a id=\"line.29\">    VALUE1,</a>",
-                "<span class=\"sourceLineNo\">030</span><a id=\"line.30\">    VALUE2</a>");
+                """
+                    <span class="source-line-no">029</span><span id="line.29">    VALUE1,</span>""",
+                """
+                    <span class="source-line-no">030</span><span id="line.30">    VALUE2</span>""");
     }
 
     @Test
@@ -266,9 +287,11 @@ public class TestOptions extends JavadocTester {
         checkExit(Exit.OK);
 
         checkOutput("pkg/Foo.html", true,
-                "<div class=\"inheritance\">Foo</div>");
+                """
+                    <div class="inheritance">Foo</div>""");
         checkOutput("deprecated/Foo.html", true,
-                "<div class=\"inheritance\">deprecated.Foo</div>");
+                """
+                    <div class="inheritance">deprecated.Foo</div>""");
 
         javadoc("-d", "out-10a",
                 "-noqualifier", "all",
@@ -277,8 +300,10 @@ public class TestOptions extends JavadocTester {
         checkExit(Exit.OK);
 
         checkOutput("pkg/Foo.html", true,
-                "<div class=\"inheritance\">Foo</div>");
+                """
+                    <div class="inheritance">Foo</div>""");
         checkOutput("deprecated/Foo.html", true,
-                "<div class=\"inheritance\">Foo</div>");
+                """
+                    <div class="inheritance">Foo</div>""");
     }
 }
