@@ -23,13 +23,12 @@
 
 /*
  * @test
- * @bug      8225055 8239804
+ * @bug      8225055 8239804 8246774 8258338
  * @summary  Record types
  * @library  /tools/lib ../../lib
  * @modules jdk.javadoc/jdk.javadoc.internal.tool
  * @build    toolbox.ToolBox javadoc.tester.*
- * @compile --enable-preview --source ${jdk.version} TestRecordTypes.java
- * @run main/othervm --enable-preview TestRecordTypes
+ * @run main TestRecordTypes
  */
 
 
@@ -69,15 +68,14 @@ public class TestRecordTypes extends JavadocTester {
         javadoc("-d", base.resolve("out").toString(),
                 "-quiet", "-noindex",
                 "-sourcepath", src.toString(),
-                "--enable-preview", "--source", thisRelease,
                 src.resolve("R.java").toString());
         checkExit(Exit.OK);
 
         checkOutput("R.html", true,
                 """
-                    <h1 title="Record R" class="title">Record R</h1>""",
+                    <h1 title="Record Class R" class="title">Record Class R</h1>""",
                 """
-                    public record <span class="type-name-label">R</span>""",
+                    <span class="modifiers">public record </span><span class="element-name type-name-label">R</span>""",
                 """
                     <code><span class="member-name-link"><a href="#%3Cinit%3E(int)">R</a></span>&#8203;(int&nbsp;r1)</code>""");
     }
@@ -91,15 +89,14 @@ public class TestRecordTypes extends JavadocTester {
         javadoc("-d", base.resolve("out").toString(),
                 "-quiet", "-noindex",
                 "-sourcepath", src.toString(),
-                "--enable-preview", "--source", thisRelease,
                 "p");
         checkExit(Exit.OK);
 
         checkOutput("p/R.html", true,
                 """
-                    <h1 title="Record R" class="title">Record R</h1>""",
+                    <h1 title="Record Class R" class="title">Record Class R</h1>""",
                 """
-                    public record <span class="type-name-label">R</span>""",
+                    <span class="modifiers">public record </span><span class="element-name type-name-label">R</span>""",
                 """
                     <code><span class="member-name-link"><a href="#%3Cinit%3E(int)">R</a></span>&#8203;(int&nbsp;r1)</code>""");
     }
@@ -113,15 +110,14 @@ public class TestRecordTypes extends JavadocTester {
         javadoc("-d", base.resolve("out").toString(),
                 "-quiet", "-noindex",
                 "-sourcepath", src.toString(),
-                "--enable-preview", "--source", thisRelease,
                 "p");
         checkExit(Exit.OK);
 
         checkOutput("p/R.html", true,
                 """
-                    <h1 title="Record R" class="title">Record R</h1>""",
+                    <h1 title="Record Class R" class="title">Record Class R</h1>""",
                 """
-                    public record <span class="type-name-label">R</span>""",
+                    <span class="modifiers">public record </span><span class="element-name type-name-label">R</span>""",
                 """
                     <code><span class="member-name-link"><a href="#%3Cinit%3E()">R</a></span>()</code>""");
     }
@@ -139,15 +135,14 @@ public class TestRecordTypes extends JavadocTester {
         javadoc("-d", base.resolve("out").toString(),
                 "-quiet", "-noindex",
                 "-sourcepath", src.toString(),
-                "--enable-preview", "--source", thisRelease,
                 "p");
         checkExit(Exit.OK);
 
         checkOutput("p/R.html", true,
                 """
-                    <h1 title="Record R" class="title">Record R</h1>""",
+                    <h1 title="Record Class R" class="title">Record Class R</h1>""",
                 """
-                    public record <span class="type-name-label">R</span>""",
+                    <span class="modifiers">public record </span><span class="element-name type-name-label">R</span>""",
                 """
                     <dl class="notes">
                     <dt>Record Components:</dt>
@@ -171,15 +166,14 @@ public class TestRecordTypes extends JavadocTester {
         javadoc("-d", base.resolve("out").toString(),
                 "-quiet", "-noindex",
                 "-sourcepath", src.toString(),
-                "--enable-preview", "--source", thisRelease,
                 "p");
         checkExit(Exit.OK);
 
         checkOutput("p/R.html", true,
                 """
-                    <h1 title="Record R" class="title">Record R&lt;T&gt;</h1>""",
+                    <h1 title="Record Class R" class="title">Record Class R&lt;T&gt;</h1>""",
                 """
-                    public record <span class="type-name-label">R&lt;T&gt;</span>""",
+                    <span class="modifiers">public record </span><span class="element-name type-name-label">R&lt;T&gt;</span>""",
                 """
                     <dl class="notes">
                     <dt>Type Parameters:</dt>
@@ -204,7 +198,6 @@ public class TestRecordTypes extends JavadocTester {
         javadoc("-d", base.resolve("out").toString(),
                 "-quiet", "-noindex",
                 "-sourcepath", src.toString(),
-                "--enable-preview", "--source", thisRelease,
                 "p");
         checkExit(Exit.OK);
 
@@ -215,7 +208,7 @@ public class TestRecordTypes extends JavadocTester {
                 """
                     <section class="constructor-summary" id="constructor.summary">""",
                 "<a href=\"#%3Cinit%3E(int)\">R</a>",
-                "Creates an instance of a <code>R</code> record.",
+                "Creates an instance of a <code>R</code> record class.",
                 """
                     <section class="method-summary" id="method.summary">""",
                 """
@@ -230,25 +223,25 @@ public class TestRecordTypes extends JavadocTester {
                     Returns the value of the <a href="#param-r1"><code>r1</code></a> record component.""",
                 """
                     <a href="#toString()">toString</a>""",
-                "Returns a string representation of this record.",
+                "Returns a string representation of this record class.",
                 "Method Details",
                 """
-                    <span class="member-name">toString</span>""",
-                "Returns a string representation of this record. The representation "
-                + "contains the name of the type, followed by the name and value of "
+                    <span class="element-name">toString</span>""",
+                "Returns a string representation of this record class. The representation "
+                + "contains the name of the class, followed by the name and value of "
                 + "each of the record components.",
                 """
-                    <span class="member-name">hashCode</span>""",
+                    <span class="element-name">hashCode</span>""",
                 "Returns a hash code value for this object. The value is derived "
                 + "from the hash code of each of the record components.",
                 """
-                    <span class="member-name">equals</span>""",
+                    <span class="element-name">equals</span>""",
                 """
                     Indicates whether some other object is "equal to" this one. The objects are equa\
                     l if the other object is of the same class and if all the record components are \
-                    equal. All components in this record are compared with '=='.""",
+                    equal. All components in this record class are compared with '=='.""",
                 """
-                    <span class="member-name">r1</span>""",
+                    <span class="element-name">r1</span>""",
                 """
                     Returns the value of the <a href="#param-r1"><code>r1</code></a> record component."""
         );
@@ -268,7 +261,6 @@ public class TestRecordTypes extends JavadocTester {
                 "-quiet", "-noindex",
                 "-sourcepath", src.toString(),
                 "-linkoffline", externalDocs, localDocs,
-                "--enable-preview", "--source", thisRelease,
                 "p");
         checkExit(Exit.OK);
 
@@ -279,7 +271,7 @@ public class TestRecordTypes extends JavadocTester {
                 """
                     <section class="constructor-summary" id="constructor.summary">""",
                 "<a href=\"#%3Cinit%3E(int)\">R</a>",
-                "Creates an instance of a <code>R</code> record.",
+                "Creates an instance of a <code>R</code> record class.",
                 """
                     <section class="method-summary" id="method.summary">""",
                 """
@@ -294,25 +286,25 @@ public class TestRecordTypes extends JavadocTester {
                     Returns the value of the <a href="#param-r1"><code>r1</code></a> record component.""",
                 """
                     <a href="#toString()">toString</a>""",
-                "Returns a string representation of this record.",
+                "Returns a string representation of this record class.",
                 "Method Details",
                 """
-                    <span class="member-name">toString</span>""",
-                "Returns a string representation of this record. The representation "
-                + "contains the name of the type, followed by the name and value of "
+                    <span class="element-name">toString</span>""",
+                "Returns a string representation of this record class. The representation "
+                + "contains the name of the class, followed by the name and value of "
                 + "each of the record components.",
                 """
-                    <span class="member-name">hashCode</span>""",
+                    <span class="element-name">hashCode</span>""",
                 "Returns a hash code value for this object. The value is derived "
                 + "from the hash code of each of the record components.",
                 """
-                    <span class="member-name">equals</span>""",
+                    <span class="element-name">equals</span>""",
                 """
                     Indicates whether some other object is "equal to" this one. The objects are equa\
                     l if the other object is of the same class and if all the record components are \
-                    equal. All components in this record are compared with '=='.""",
+                    equal. All components in this record class are compared with '=='.""",
                 """
-                    <span class="member-name">r1</span>""",
+                    <span class="element-name">r1</span>""",
                 """
                     Returns the value of the <a href="#param-r1"><code>r1</code></a> record component."""
         );
@@ -321,13 +313,13 @@ public class TestRecordTypes extends JavadocTester {
     @Test
     public void testGeneratedEqualsPrimitive(Path base) throws IOException {
         testGeneratedEquals(base, "int a, int b",
-             "All components in this record are compared with '=='.");
+             "All components in this record class are compared with '=='.");
     }
 
     @Test
     public void testGeneratedEqualsReference(Path base) throws IOException {
         testGeneratedEquals(base, "Object a, Object b",
-             "All components in this record are compared with <code>Objects::equals(Object,Object)</code>");
+             "All components in this record class are compared with <code>Objects::equals(Object,Object)</code>");
     }
 
     @Test
@@ -347,8 +339,8 @@ public class TestRecordTypes extends JavadocTester {
 
         javadoc("-d", base.resolve("out").toString(),
                 "-quiet", "-noindex",
+                "--no-platform-links",
                 "-sourcepath", src.toString(),
-                "--enable-preview", "--source", thisRelease,
                 "p");
         checkExit(Exit.OK);
 
@@ -374,7 +366,6 @@ public class TestRecordTypes extends JavadocTester {
         javadoc("-d", base.resolve("out").toString(),
                 "-quiet", "-noindex",
                 "-sourcepath", src.toString(),
-                "--enable-preview", "--source", thisRelease,
                 "p");
         checkExit(Exit.OK);
 
@@ -405,7 +396,6 @@ public class TestRecordTypes extends JavadocTester {
                 "-quiet", "-noindex",
                 "-sourcepath", testSrc.toString(),
                 "-linksource",
-                "--enable-preview", "--source", thisRelease,
                 "examples");
 
         checkExit(Exit.OK);
@@ -414,13 +404,11 @@ public class TestRecordTypes extends JavadocTester {
                 "-sourcepath", testSrc.toString(),
                 "-linksource",
                 "-linkoffline", externalDocs, localDocs,
-                "--enable-preview", "--source", thisRelease,
                 "examples");
         checkExit(Exit.OK);
     }
 
     @Test
-    @SuppressWarnings("preview")
     public void testAnnotations(Path base) throws IOException {
         ElementType[] types = {
                 ElementType.FIELD,
@@ -464,9 +452,9 @@ public class TestRecordTypes extends JavadocTester {
 
         javadoc("-d", dir.resolve("out").toString(),
                 "-quiet", "-noindex",
+                "--no-platform-links",
                 "-sourcepath", src.toString(),
                 "-private",
-                "--enable-preview", "--source", thisRelease,
                 "p");
         checkExit(Exit.OK);
 
@@ -478,31 +466,98 @@ public class TestRecordTypes extends JavadocTester {
             <a href="Anno.html" title="annotation in p">@Anno</a>""";
         String rcAnno = t.contains(ElementType.RECORD_COMPONENT) ? anno + " " : "";
         String fAnno = t.contains(ElementType.FIELD) ? "<span class=\"annotations\">" + anno + "\n</span>" : "";
-        String pAnno = t.contains(ElementType.PARAMETER) ? anno + "\n" : "";
+        String pAnno = t.contains(ElementType.PARAMETER) ? anno + "\n " : "";
         String mAnno= t.contains(ElementType.METHOD) ? "<span class=\"annotations\">" + anno + "\n</span>" : "";
 
         checkOutput("p/R.html", true,
                 """
-                    <pre>public record <span class="type-name-label">R</span>("""
+                    <div class="type-signature"><span class="modifiers">public record </span><span c\
+                    lass="element-name type-name-label">R</span>("""
                         + rcAnno
                         + """
                             int&nbsp;i)
-                            extends java.lang.Record</pre>""",
+                            <span class="extends-implements">extends java.lang.Record</span></div>""",
                 "<div class=\"member-signature\">"
                         + fAnno
                         + """
                             <span class="modifiers">private final</span>&nbsp;<span class="return-type">int<\
-                            /span>&nbsp;<span class="member-name">i</span></div>""",
+                            /span>&nbsp;<span class="element-name">i</span></div>""",
                 """
                     <div class="member-signature"><span class="modifiers">public</span>&nbsp;<span c\
-                    lass="member-name">R</span>&#8203;(<span class="parameters">"""
+                    lass="element-name">R</span>&#8203;<span class="parameters">("""
                         + pAnno
                         + "int&nbsp;i)</span></div>",
                 "<div class=\"member-signature\">"
                         + mAnno
                         + """
                             <span class="modifiers">public</span>&nbsp;<span class="return-type">int</span>&\
-                            nbsp;<span class="member-name">i</span>()</div>""");
+                            nbsp;<span class="element-name">i</span>()</div>""");
 
     }
+
+    @Test
+    public void testDeprecatedRecord(Path base) throws IOException {
+        Path src = base.resolve("src");
+        tb.writeJavaFiles(src,
+                """
+                    package p; /** This is record R.
+                     * @deprecated Do not use.
+                     */
+                    @Deprecated
+                    public record R(int r1) { }""");
+
+        javadoc("-d", base.resolve("out").toString(),
+                "-quiet", "-noindex",
+                "-sourcepath", src.toString(),
+                "p");
+        checkExit(Exit.OK);
+
+        checkOutput("deprecated-list.html", true,
+                """
+                    <h2 title="Contents">Contents</h2>
+                    <ul>
+                    <li><a href="#record.class">Record Classes</a></li>
+                    </ul>""",
+                """
+                    <div id="record.class">
+                    <div class="caption"><span>Record Classes</span></div>
+                    <div class="summary-table two-column-summary">
+                    <div class="table-header col-first">Record Class</div>
+                    <div class="table-header col-last">Description</div>
+                    <div class="col-deprecated-item-name even-row-color"><a href="p/R.html" title="class in p">p.R</a></div>
+                    <div class="col-last even-row-color">
+                    <div class="deprecation-comment">Do not use.</div>
+                    </div>""");
+    }
+
+    @Test
+    public void testDeprecatedRecordComponent(Path base) throws IOException {
+        Path src = base.resolve("src");
+        tb.writeJavaFiles(src,
+                """
+                    package p; /** This is record R. */
+                    public record R(@Deprecated int r1) { }""");
+
+        javadoc("-d", base.resolve("out").toString(),
+                "-quiet", "-noindex",
+                "-sourcepath", src.toString(),
+                "p");
+        checkExit(Exit.OK);
+
+        checkOutput("deprecated-list.html", true,
+                """
+                    <h2 title="Contents">Contents</h2>
+                    <ul>
+                    <li><a href="#method">Methods</a></li>
+                    </ul>""",
+                """
+                    <div id="method">
+                    <div class="caption"><span>Methods</span></div>
+                    <div class="summary-table two-column-summary">
+                    <div class="table-header col-first">Method</div>
+                    <div class="table-header col-last">Description</div>
+                    <div class="col-deprecated-item-name even-row-color"><a href="p/R.html#r1()">p.R.r1()</a></div>
+                    <div class="col-last even-row-color"></div>
+                    </div>""");
+    }
 }
