@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,7 +25,6 @@
  * @test
  * @bug     4496223 4496270 4618686 4720974 4812240 6253614 6253604
  * @summary <DESC>
- * @author  jamieh
  * @library ../../lib
  * @modules jdk.javadoc/jdk.javadoc.internal.tool
  * @build javadoc.tester.*
@@ -63,10 +62,12 @@ public class TestTagInheritence extends JavadocTester {
 
         //First sentence test (6253614)
         checkOutput("firstSentence/B.html", true,
-                "<div class=\"block\">First sentence.</div>");
+                """
+                    <div class="block">First sentence.</div>""");
 
         //Another first sentence test (6253604)
         checkOutput("firstSentence2/C.html", true,
-                "<div class=\"block\">First sentence.</div>");
+                """
+                    <div class="block">First sentence.</div>""");
     }
 }
