@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -61,6 +61,7 @@ public class EventNames {
     public final static String JavaMonitorEnter = PREFIX + "JavaMonitorEnter";
     public final static String JavaMonitorWait = PREFIX + "JavaMonitorWait";
     public final static String JavaMonitorInflate = PREFIX + "JavaMonitorInflate";
+    public final static String SyncOnValueBasedClass = PREFIX + "SyncOnValueBasedClass";
     public final static String ClassLoad = PREFIX + "ClassLoad";
     public final static String ClassDefine = PREFIX + "ClassDefine";
     public final static String ClassUnload = PREFIX + "ClassUnload";
@@ -87,6 +88,10 @@ public class EventNames {
     public final static String PlaceholderTableStatistics = PREFIX + "PlaceholderTableStatistics";
     public final static String LoaderConstraintsTableStatistics = PREFIX + "LoaderConstraintsTableStatistics";
     public final static String ProtectionDomainCacheTableStatistics = PREFIX + "ProtectionDomainCacheTableStatistics";
+    public static final String RedefineClasses = PREFIX + "RedefineClasses";
+    public static final String RetransformClasses = PREFIX + "RetransformClasses";
+    public static final String ClassRedefinition = PREFIX + "ClassRedefinition";
+
     // This event is hard to test
     public final static String ReservedStackActivation = PREFIX + "ReservedStackActivation";
 
@@ -137,6 +142,13 @@ public class EventNames {
     public final static String AllocationRequiringGC = PREFIX + "AllocationRequiringGC";
     public final static String GCPhaseParallel = PREFIX + "GCPhaseParallel";
     public final static String GCPhaseConcurrent = PREFIX + "GCPhaseConcurrent";
+    public final static String GCPhaseConcurrentLevel1 = PREFIX + "GCPhaseConcurrentLevel1";
+    public final static String ZAllocationStall = PREFIX + "ZAllocationStall";
+    public final static String ZPageAllocation = PREFIX + "ZPageAllocation";
+    public final static String ZRelocationSet = PREFIX + "ZRelocationSet";
+    public final static String ZRelocationSetGroup = PREFIX + "ZRelocationSetGroup";
+    public final static String ZUncommit = PREFIX + "ZUncommit";
+    public final static String ZUnmap = PREFIX + "ZUnmap";
 
     // Compiler
     public final static String Compilation = PREFIX + "Compilation";
@@ -153,6 +165,7 @@ public class EventNames {
     public final static String CodeCacheFull = PREFIX + "CodeCacheFull";
     public final static String ObjectAllocationInNewTLAB = PREFIX + "ObjectAllocationInNewTLAB";
     public final static String ObjectAllocationOutsideTLAB = PREFIX + "ObjectAllocationOutsideTLAB";
+    public final static String ObjectAllocationSample = PREFIX + "ObjectAllocationSample";
     public final static String Deoptimization = PREFIX + "Deoptimization";
 
     // OS
@@ -167,6 +180,7 @@ public class EventNames {
     public final static String NativeLibrary = PREFIX + "NativeLibrary";
     public final static String PhysicalMemory = PREFIX + "PhysicalMemory";
     public final static String NetworkUtilization = PREFIX + "NetworkUtilization";
+    public static final String ProcessStart = PREFIX + "ProcessStart";
 
     // JDK
     public static final String FileForce  = PREFIX + "FileForce";
@@ -183,6 +197,7 @@ public class EventNames {
     public final static String X509Certificate = PREFIX + "X509Certificate";
     public final static String X509Validation = PREFIX + "X509Validation";
     public final static String SecurityProperty = PREFIX + "SecurityPropertyModification";
+    public final static String DirectBufferStatistics = PREFIX + "DirectBufferStatistics";
 
     // Flight Recorder
     public final static String DumpReason = PREFIX + "DumpReason";
@@ -191,11 +206,9 @@ public class EventNames {
     public final static String ActiveRecording = PREFIX + "ActiveRecording";
     public final static String ActiveSetting = PREFIX + "ActiveSetting";
     public static final String Flush = PREFIX + "Flush";
-    public static final String FlushStringPool = PREFIX + "FlushStringPool";
-    public static final String FlushStacktrace = PREFIX + "FlushStacktrace";
-    public static final String FlushStorage = PREFIX + "FlushStorage";
-    public static final String FlushMetadata = PREFIX + "FlushMetadata";
-    public static final String FlushTypeSet = PREFIX + "FlushTypeSet";
+
+    // Diagnostics
+    public static final String HeapDump = PREFIX + "HeapDump";
 
     public static boolean isGcEvent(EventType et) {
         return et.getCategoryNames().contains(GC_CATEGORY);
